/* This is a generated file! */
/* Generated by `utils/build_parsers.py` via `utils/rustify.py` */
/* This process can be run via tox: `tox -e generate-rs` */
#![cfg_attr(rustfmt, rustfmt_skip)]

use sqlfluffrs_types::{
    GrammarInst, GrammarVariant, GrammarFlags,
    ParseMode, GrammarId, GrammarTables,
    SimpleHintData, RootGrammar
};
// Flattening grammar tree...
// Validating tables...
// Validation passed!

// Table Statistics:
//   Instructions:      3671 × 20 bytes =    73420 bytes
//   Child IDs:         2458 ×  4 bytes =     9832 bytes
//   Terminators:         52 ×  4 bytes =      208 bytes
//   Strings:           2029 strings      =    27122 bytes
//   Aux Data:         11976 ×  4 bytes =    47904 bytes
//   Aux Offsets:       3671 ×  4 bytes =    14684 bytes
//   Regex Patterns:       5 patterns     =      199 bytes
//   Simple Hints:      1233 hints        =    19728 bytes
//   Hint Indices:      1831 ×  4 bytes =     7324 bytes
//   TOTAL:                                    ≈   200421 bytes

pub static INSTRUCTIONS: &[GrammarInst] = &[
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 0, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [0] StringParser("ABORT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 0, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [1] StringParser("ABS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 0, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [2] StringParser("ABSOLUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 0, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [3] StringParser("ACCESS")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 0, child_count: 3, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [4] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [5] Ref(AccountKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [6] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5, child_count: 17, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [7] OneOf(17 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 22, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [8] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [9] Ref(ResourceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [10] Ref(MonitorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [11] Ref(WarehouseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [12] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [13] Ref(DomainKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [14] Ref(IntegrationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [15] Ref(LanguageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [16] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [17] Ref(CatalogKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [18] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [19] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [20] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [21] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 26, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [22] Ref(ForeignKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 26, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [23] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 28, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [24] Ref(ServerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 28, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [25] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 30, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [26] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 30, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [27] Ref(WrapperKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 30, child_count: 4, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [28] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 34, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [29] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 34, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [30] Ref(SchemasKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 34, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [31] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 34, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [32] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 34, child_count: 4, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [33] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 38, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [34] Ref(FutureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 38, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [35] Ref(SchemasKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 38, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [36] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 38, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [37] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 38, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [38] Ref(AccessSchemaObjectSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 38, child_count: 4, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [39] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 42, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [40] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 42, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [41] Ref(AccessSchemaPluralObjectSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 42, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [42] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 42, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [43] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 42, child_count: 4, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [44] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 46, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [45] Ref(FutureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 46, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [46] Ref(AccessSchemaPluralObjectSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 46, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [47] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 46, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [48] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 48, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [49] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 48, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [50] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 48, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 2, _padding: 0 }, // [51] Delimited(3 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 50, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [52] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 53, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [53] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 53, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [54] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 55, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [55] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 55, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [56] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 55, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [57] Ref(WildcardIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 55, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [58] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 55, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [59] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 55, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [60] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 55, child_count: 3, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [61] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 58, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [62] Ref(LargeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 58, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [63] Ref(ObjectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 58, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [64] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 58, child_count: 6, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [65] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 64, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [66] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 66, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [67] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 66, child_count: 5, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [68] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 71, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [69] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 71, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [70] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 71, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [71] Ref(WarehouseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 71, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [72] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 71, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [73] Ref(IntegrationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 71, child_count: 3, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [74] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 74, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [75] Ref(ApplyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 74, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [76] Ref(MaskingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 74, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [77] Ref(PolicyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 74, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [78] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 76, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [79] Ref(ExecuteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 76, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [80] Ref(TaskKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 76, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [81] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 78, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [82] Ref(ManageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 78, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [83] Ref(GrantsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 78, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [84] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 80, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [85] Ref(MonitorKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 80, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [86] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 82, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [87] Ref(ExecutionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 82, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [88] Ref(UsageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 82, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [89] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 84, child_count: 26, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [90] OneOf(26 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 110, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [91] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 112, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [92] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 112, child_count: 4, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [93] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 116, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [94] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 116, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [95] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 118, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [96] Ref(MaskingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 118, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [97] Ref(PolicyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 118, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [98] Ref(PipeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 118, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [99] Ref(AccessSchemaObjectSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 118, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [100] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 120, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [101] Ref(UseKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 120, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [102] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 122, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [103] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 122, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [104] Ref(CatalogKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 122, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [105] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [106] Ref(ImportedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [107] Ref(PrivilegesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [108] Ref(ApplyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [109] Ref(ConnectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [110] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [111] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [112] Ref(ExecuteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [113] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [114] Ref(ModifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [115] Ref(MonitorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [116] Ref(OperateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [117] Ref(OwnershipKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [118] Ref(ReadKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [119] Ref(Reference_usageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [120] Ref(ReferencesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [121] Ref(SelectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [122] Ref(TempKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [123] Ref(TemporaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [124] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [125] Ref(TruncateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [126] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [127] Ref(UsageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [128] Ref(Use_any_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [129] Ref(WriteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [130] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 126, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [131] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 126, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [132] Ref(PrivilegesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 126, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [133] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 126, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 1, _padding: 0 }, // [134] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 128, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [135] Ref(AccessPermissionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 128, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [136] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 128, child_count: 14, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [137] OneOf(14 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [138] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [139] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [140] Ref(StageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [141] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [142] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [143] Ref(RoutineKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [144] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [145] Ref(StreamKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [146] Ref(TaskKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [147] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [148] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 144, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [149] Ref(MaskingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 144, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [150] Ref(PolicyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 144, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [151] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 146, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [152] Ref(MaterializedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 146, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [153] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 146, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [154] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 148, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [155] Ref(ExternalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 148, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [156] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 148, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [157] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 150, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [158] Ref(FileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 150, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [159] Ref(FormatKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 150, child_count: 14, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [160] OneOf(14 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [161] Ref(TablesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [162] Ref(ViewsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [163] Ref(StagesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [164] Ref(FunctionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [165] Ref(ProceduresKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [166] Ref(RoutinesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [167] Ref(SequencesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [168] Ref(StreamsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [169] Ref(TasksKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [170] Ref(SchemasKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [171] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 166, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [172] Ref(MaskingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 166, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [173] Ref(PoliciesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 166, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [174] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 168, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [175] Ref(MaterializedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 168, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [176] Ref(ViewsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 168, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [177] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 170, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [178] Ref(ExternalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 170, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [179] Ref(TablesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 170, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [180] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 172, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [181] Ref(FileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 172, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [182] Ref(FormatsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 172, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [183] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 174, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [184] Ref(GrantStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 174, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [185] Ref(RevokeStatementSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 174, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [186] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 176, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [187] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 178, child_count: 5, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [188] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 183, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [189] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 185, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [190] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 185, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [191] Ref(UserReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 185, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [192] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 187, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [193] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 187, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [194] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 187, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [195] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 189, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [196] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 189, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [197] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 189, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [198] Ref(PublicKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 189, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [199] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 189, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [200] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 191, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [201] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 191, child_count: 1, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [202] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 192, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [203] Ref(ArrayAccessorSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 192, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [204] StringParser("ACCOUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 192, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [205] StringParser("ACCOUNTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 192, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [206] StringParser("ACTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 192, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [207] StringParser("ACTIVATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 192, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [208] StringParser("ADA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 192, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [209] StringParser("ADD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 192, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [210] StringParser("ADMIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 192, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [211] StringParser("AFTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 192, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [212] StringParser("AGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 192, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [213] StringParser("AGGREGATE")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 192, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [214] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 192, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [215] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 196, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [216] Meta("indent")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 196, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [217] Ref(AsAliasOperatorSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 196, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [218] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 198, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [219] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 200, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [220] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 200, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [221] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 203, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [222] Ref(SingleIdentifierListSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 203, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [223] StringParser("(")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 203, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [224] StringParser(")")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 203, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [225] Ref(SingleQuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 203, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [226] Meta("dedent")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 203, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [227] StringParser("ALIAS")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 203, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [228] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 205, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [229] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 205, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [230] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 205, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [231] StringParser("ALL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 205, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [232] StringParser("ALLOCATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 205, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [233] StringParser("ALLOW")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 205, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [234] StringParser("ALSO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 205, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [235] StringParser("ALTER")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 205, child_count: 6, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [236] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 211, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [237] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 214, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [238] Ref(IncrementKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 214, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [239] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 214, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [240] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 214, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [241] Ref(SequenceMinValueGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 214, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [242] Ref(SequenceMaxValueGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 214, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [243] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 216, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [244] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 218, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [245] Ref(CacheKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 218, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [246] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 218, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [247] Ref(NocacheKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 218, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [248] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 220, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [249] Ref(CycleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 220, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [250] Ref(NocycleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 220, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [251] Ref(OrderNoOrderGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 220, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [252] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 224, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [253] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 224, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [254] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 224, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [255] Ref(SequenceReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 224, child_count: 1, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [256] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 225, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [257] Ref(AlterSequenceOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 225, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [258] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 229, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [259] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 229, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [260] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 229, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [261] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 229, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [262] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 229, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [263] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 233, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [264] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 236, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [265] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 236, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [266] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 236, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [267] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 238, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [268] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 238, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [269] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 238, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [270] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 242, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [271] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 244, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [272] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 244, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [273] Ref(ModifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 244, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [274] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 244, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [275] Ref(ColumnDefinitionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 244, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [276] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 246, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [277] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 248, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [278] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 250, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [279] Ref(FirstKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 250, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [280] Ref(AfterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 250, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [281] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 250, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [282] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 250, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [283] Ref(AlterTableDropColumnGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 250, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [284] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 253, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [285] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 253, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [286] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 255, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [287] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 255, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [288] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 255, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [289] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 255, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [290] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 259, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [291] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 259, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [292] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 259, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [293] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 259, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [294] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 261, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [295] Ref(AlterTableOptionsGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 261, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [296] StringParser("ALWAYS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 261, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [297] StringParser("&")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 261, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [298] StringParser("ANALYSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 261, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [299] StringParser("ANALYZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 261, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [300] StringParser("AND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 261, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [301] StringParser("AND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 261, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [302] StringParser("ANY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 261, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [303] StringParser("APPLY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 261, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [304] StringParser("ARE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 261, child_count: 10, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [305] OneOf(10 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 271, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [306] Ref(PlusSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 271, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [307] Ref(MinusSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 271, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [308] Ref(DivideSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 271, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [309] Ref(MultiplySegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 271, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [310] Ref(ModuloSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 271, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [311] Ref(BitwiseAndSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 271, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [312] Ref(BitwiseOrSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 271, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [313] Ref(BitwiseXorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 271, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [314] Ref(BitwiseLShiftSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 271, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [315] Ref(BitwiseRShiftSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 271, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [316] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 274, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [317] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 276, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [318] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 278, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [319] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 278, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [320] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 278, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [321] Ref(SliceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 278, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [322] StringParser("[")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 278, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [323] StringParser("]")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 278, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [324] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 278, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [325] StringParser("ARRAY")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 278, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [326] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 281, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [327] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 283, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [328] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 283, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [329] Nothing
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 283, child_count: 1, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [330] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 284, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [331] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 284, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [332] StringParser("AS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 284, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [333] StringParser("ASC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 284, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [334] StringParser("ASENSITIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 284, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [335] StringParser("ASSERTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 284, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [336] StringParser("ASSIGNMENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 284, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [337] StringParser("ASSOCIATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 284, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [338] StringParser("ASUTIME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 284, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [339] StringParser("ASYMMETRIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 284, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [340] StringParser("ASYNCHRONY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 284, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [341] StringParser("AT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 284, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [342] StringParser("ATOMIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 284, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [343] StringParser("ATTRIBUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 284, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [344] StringParser("ATTRIBUTES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 284, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [345] StringParser("AUDIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 284, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [346] StringParser("AUTHORIZATION")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 284, child_count: 1, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [347] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 285, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [348] Ref(Auto_incrementKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 285, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [349] StringParser("AUTO_INCREMENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 285, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [350] StringParser("AUX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 285, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [351] StringParser("AUXILIARY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 285, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [352] StringParser("AVG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 285, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [353] StringParser("AVG_ROW_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 285, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [354] StringParser("BACKUP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 285, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [355] StringParser("BACKWARD")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 285, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [356] Ref(SpecialRegisterGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 285, child_count: 7, min_times: 0, first_terminator_idx: 3, terminator_count: 2, _padding: 0 }, // [357] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 292, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [358] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 292, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [359] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 292, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [360] Ref(IntervalExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 292, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [361] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 292, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [362] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 292, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [363] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 292, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [364] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 294, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [365] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 294, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [366] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 294, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [367] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 294, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [368] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 294, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [369] Token("file")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 294, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [370] Token("base")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 294, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [371] StringParser("BEFORE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 294, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [372] StringParser("BEGIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 294, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [373] StringParser("BERNOULLI")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 294, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [374] StringParser("BETWEEN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 294, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [375] StringParser("BIGINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 294, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [376] StringParser("BINARY")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 294, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [377] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 298, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [378] Ref(ArithmeticBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 298, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [379] Ref(StringBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 298, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [380] Ref(BooleanBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 298, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [381] Ref(ComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 298, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [382] Token("binary_operator")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 298, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [383] StringParser("BINDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 298, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [384] StringParser("BIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 298, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [385] StringParser("BIT_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 298, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [386] StringParser("BITVAR")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 298, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [387] Ref(AmpersandSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 298, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [388] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 300, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [389] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 300, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [390] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 300, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [391] Ref(PipeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 300, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [392] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 302, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [393] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 302, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [394] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 302, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [395] StringParser("^")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 302, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [396] StringParser("BLOB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 302, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [397] StringParser("BOOL")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 302, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [398] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 304, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [399] Ref(AndOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 304, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [400] Ref(OrOperatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 304, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [401] StringParser("BOOLEAN")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 304, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [402] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 306, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [403] Ref(TrueSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 306, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [404] Ref(FalseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 306, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [405] StringParser("BOTH")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 306, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [406] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 309, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [407] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 311, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [408] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 311, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [409] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 314, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [410] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 316, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [411] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 316, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [412] Token("bracketed")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 316, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [413] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 319, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [414] Ref(SetExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 319, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [415] StringParser("BREADTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 319, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [416] StringParser("BREAK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 319, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [417] StringParser("BROWSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 319, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [418] StringParser("BUFFERPOOL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 319, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [419] StringParser("BULK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 319, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [420] StringParser("BUSINESS_TIME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 319, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [421] StringParser("BY")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 319, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [422] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 322, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [423] Ref(SingleIdentifierListSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 322, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [424] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 326, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [425] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 326, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [426] Ref(CTEColumnList)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 326, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [427] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 326, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [428] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 329, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [429] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 329, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [430] StringParser("CACHE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 329, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [431] StringParser("CALL")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 329, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [432] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 331, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [433] Ref(CallKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 331, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [434] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 333, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [435] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(6), first_child_idx: 333, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [436] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 333, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [437] StringParser("CALLED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 333, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [438] StringParser("CAPTURE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 333, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [439] StringParser("CARDINALITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 333, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [440] StringParser("CASCADE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 333, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [441] StringParser("CASCADED")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 333, child_count: 2, min_times: 0, first_terminator_idx: 11, terminator_count: 3, _padding: 0 }, // [442] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 335, child_count: 6, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [443] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 341, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [444] Ref(CaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 341, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [445] Meta("implicit_indent")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(7), first_child_idx: 341, child_count: 1, min_times: 0, first_terminator_idx: 5, terminator_count: 2, _padding: 0 }, // [446] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 342, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [447] Ref(WhenClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 342, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [448] Ref(ElseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 342, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [449] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(7), first_child_idx: 342, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 1, _padding: 0 }, // [450] Ref(ElseClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 342, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [451] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 342, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [452] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 342, child_count: 7, min_times: 0, first_terminator_idx: 11, terminator_count: 0, _padding: 0 }, // [453] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 349, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [454] Ref(CaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 349, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [455] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(7), first_child_idx: 349, child_count: 1, min_times: 0, first_terminator_idx: 8, terminator_count: 2, _padding: 0 }, // [456] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 350, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [457] Ref(WhenClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 350, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [458] Ref(ElseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 350, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [459] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(7), first_child_idx: 350, child_count: 0, min_times: 0, first_terminator_idx: 10, terminator_count: 1, _padding: 0 }, // [460] Ref(ElseClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 350, child_count: 0, min_times: 0, first_terminator_idx: 10, terminator_count: 0, _padding: 0 }, // [461] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 350, child_count: 0, min_times: 0, first_terminator_idx: 11, terminator_count: 0, _padding: 0 }, // [462] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 350, child_count: 0, min_times: 0, first_terminator_idx: 11, terminator_count: 0, _padding: 0 }, // [463] Ref(ComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 350, child_count: 0, min_times: 0, first_terminator_idx: 11, terminator_count: 0, _padding: 0 }, // [464] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 350, child_count: 0, min_times: 0, first_terminator_idx: 11, terminator_count: 0, _padding: 0 }, // [465] Ref(BinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 350, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [466] StringParser("CASE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 350, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [467] StringParser("CAST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 350, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [468] StringParser("::")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 350, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [469] StringParser("CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 350, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [470] StringParser("CATALOG_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 350, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [471] StringParser("CCSID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 350, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [472] StringParser("CEIL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 350, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [473] StringParser("CEILING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 350, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [474] StringParser("CHAIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 350, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [475] StringParser("CHANGE")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 350, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [476] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 350, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [477] StringParser("CHAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 350, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [478] StringParser("CHAR_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 350, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [479] StringParser("CHARACTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 350, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [480] StringParser("CHARACTER_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 350, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [481] StringParser("CHARACTER_SET_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 350, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [482] StringParser("CHARACTER_SET_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 350, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [483] StringParser("CHARACTER_SET_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 350, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [484] StringParser("CHARACTERISTICS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 350, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [485] StringParser("CHARACTERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 350, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [486] StringParser("CHECK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 350, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [487] StringParser("CHECKED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 350, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [488] StringParser("CHECKPOINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 350, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [489] StringParser("CHECKSUM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 350, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [490] StringParser("CLASS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 350, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [491] StringParser("CLASS_ORIGIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 350, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [492] StringParser("CLIENT_ACCTNG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 350, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [493] StringParser("CLIENT_APPLNAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 350, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [494] StringParser("CLIENT_USERID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 350, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [495] StringParser("CLIENT_WRKSTNNAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 350, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [496] StringParser("CLOB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 350, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [497] StringParser("CLONE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 350, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [498] StringParser("CLOSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 350, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [499] StringParser("CLUSTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 350, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [500] StringParser("CLUSTERED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 350, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [501] StringParser("COALESCE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 350, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [502] StringParser("COBOL")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 350, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [503] Token("raw")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 350, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [504] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 350, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [505] StringParser("COLLATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 350, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [506] StringParser("COLLATION")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 350, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [507] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 352, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [508] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 352, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 1, _padding: 0 }, // [509] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 354, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [510] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 354, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [511] Ref(ObjectReferenceDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 354, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [512] Ref(ObjectReferenceTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 354, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [513] StringParser("COLLATION_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 354, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [514] StringParser("COLLATION_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 354, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [515] StringParser("COLLATION_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 354, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [516] StringParser("COLLECT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 354, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [517] StringParser("COLLECTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 354, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [518] StringParser("COLLID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 354, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [519] StringParser(":")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 354, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [520] StringParser(":")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 354, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [521] StringParser(":")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 354, child_count: 4, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [522] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 358, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [523] Ref(ShorthandCastSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 358, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [524] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 358, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [525] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 358, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [526] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 358, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [527] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 360, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [528] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 362, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [529] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 362, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [530] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 362, child_count: 10, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [531] OneOf(10 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 372, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [532] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 374, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [533] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 374, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [534] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 374, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [535] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 376, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [536] Ref(CheckKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 376, child_count: 3, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [537] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 379, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [538] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 379, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [539] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 381, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [540] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 381, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [541] Ref(ColumnConstraintDefaultGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 381, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [542] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 383, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [543] Ref(PrimaryKeyGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 383, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [544] Ref(NotEnforcedGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 383, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [545] Ref(UniqueKeyGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 383, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [546] Ref(AutoIncrementGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 383, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [547] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 385, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [548] Ref(ReferenceDefinitionGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 385, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [549] Ref(NotEnforcedGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 385, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [550] Ref(CommentClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 385, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [551] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 387, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [552] Ref(CollateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 387, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [553] Ref(CollationReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 387, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [554] Ref(ColumnGeneratedGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 387, child_count: 4, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [555] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 391, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [556] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 391, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [557] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 391, child_count: 3, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [558] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 394, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [559] Anything
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 394, child_count: 1, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [560] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 395, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [561] Ref(ColumnConstraintSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 395, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [562] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 395, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [563] StringParser("COLUMN")
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 395, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 1, _padding: 0 }, // [564] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 397, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [565] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 397, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [566] Ref(ObjectReferenceDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 397, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [567] Ref(ObjectReferenceTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 397, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [568] StringParser("COLUMN_NAME")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 397, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [569] Nothing
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 397, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [570] Ref(ColumnsExpressionNameGrammar)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 397, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [571] Nothing
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 397, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [572] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 397, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [573] StringParser("COLUMNS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 397, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [574] StringParser(",")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 397, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [575] StringParser("COMMAND_FUNCTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 397, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [576] StringParser("COMMAND_FUNCTION_CODE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 397, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [577] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 399, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [578] Ref(CommentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 399, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [579] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 399, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [580] StringParser("COMMENT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 399, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [581] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 402, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [582] Ref(CommentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 402, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [583] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 402, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [584] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 404, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [585] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 407, child_count: 4, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [586] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 411, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [587] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 413, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [588] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 413, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [589] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 413, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [590] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 415, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [591] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 415, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [592] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 415, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [593] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 417, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [594] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 417, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [595] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 417, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [596] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 419, child_count: 12, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [597] OneOf(12 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 431, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [598] Ref(AliasKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 431, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [599] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 431, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [600] Ref(PackageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 431, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [601] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 431, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [602] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 431, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [603] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 431, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [604] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 431, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [605] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 431, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [606] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 431, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [607] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 431, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [608] Ref(VariableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 431, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [609] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 431, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [610] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 431, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [611] Ref(IsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 431, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [612] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 431, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [613] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 433, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [614] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 433, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [615] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 436, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [616] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 438, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [617] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 441, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [618] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 441, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [619] Ref(IsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 441, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [620] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 441, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [621] Token("comment")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 441, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [622] StringParser("COMMIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 441, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [623] StringParser("COMMITTED")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 441, child_count: 8, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [624] OneOf(8 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 449, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [625] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 449, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [626] Ref(GreaterThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 449, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [627] Ref(LessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 449, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [628] Ref(GreaterThanOrEqualToSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 449, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [629] Ref(LessThanOrEqualToSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 449, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [630] Ref(NotEqualToSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 449, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [631] Ref(LikeOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 449, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [632] Ref(IsDistinctFromGrammar)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 449, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [633] Token("comparison_operator")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 449, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [634] StringParser("COMPLETION")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 449, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [635] Token("binary_operator")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 449, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [636] Token("comparison_operator")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 449, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [637] StringParser("COMPRESS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 449, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [638] StringParser("COMPUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 449, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [639] StringParser("CONCAT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 449, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [640] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 451, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [641] Ref(PipeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 451, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [642] Ref(PipeSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 451, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [643] StringParser("CONDITION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 451, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [644] StringParser("CONDITION_NUMBER")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 451, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [645] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 451, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [646] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 454, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [647] Ref(JoinTypeKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 454, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [648] Ref(ConditionalCrossJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 454, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [649] Ref(NonStandardJoinTypeKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 454, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [650] StringParser("CONNECT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 454, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [651] StringParser("CONNECTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 454, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [652] StringParser("CONNECTION_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 454, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [653] StringParser("CONSTRAINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 454, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [654] StringParser("CONSTRAINT_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 454, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [655] StringParser("CONSTRAINT_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 454, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [656] StringParser("CONSTRAINT_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 454, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [657] StringParser("CONSTRAINTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 454, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [658] StringParser("CONSTRUCTOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 454, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [659] StringParser("CONTAINS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 454, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [660] StringParser("CONTAINSTABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 454, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [661] StringParser("CONTINUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 454, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [662] StringParser("CONVERSION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 454, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [663] StringParser("CONVERT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 454, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [664] StringParser("COPY")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 454, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [665] AnySetOf(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 456, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [666] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 459, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [667] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 461, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [668] Ref(IncludingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 461, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [669] Ref(ExcludingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 461, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [670] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 461, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [671] Ref(DefaultsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 461, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [672] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 464, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [673] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 466, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [674] Ref(IncludingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 466, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [675] Ref(ExcludingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 466, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [676] Ref(IdentityKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 466, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [677] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 468, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [678] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 468, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [679] Ref(AttributesKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 468, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [680] StringParser("CORR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 468, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [681] StringParser("CORRESPONDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 468, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [682] StringParser("COUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 468, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [683] StringParser("COUNT_BIG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 468, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [684] StringParser("COVAR_POP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 468, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [685] StringParser("COVAR_SAMP")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 468, child_count: 10, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [686] Sequence(10 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 478, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [687] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 478, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [688] Ref(CastKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 478, child_count: 5, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [689] Bracketed(3 elements -> 3 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 483, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [690] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 483, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [691] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 483, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [692] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 483, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [693] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 483, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [694] Ref(SpecificKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 483, child_count: 4, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [695] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 487, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [696] Ref(RoutineKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 487, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [697] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 487, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [698] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 487, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [699] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 489, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [700] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 492, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [701] Ref(InstanceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 492, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [702] Ref(StaticKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 492, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [703] Ref(ConstructorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 492, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [704] Ref(MethodKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 492, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [705] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 492, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [706] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 492, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [707] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 494, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [708] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 494, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [709] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 494, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [710] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 496, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [711] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 496, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [712] Ref(AssignmentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 496, child_count: 4, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [713] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 500, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [714] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 500, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [715] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 500, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [716] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 500, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [717] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 500, child_count: 9, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [718] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 509, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [719] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 509, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [720] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 509, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [721] Ref(TemporaryGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 509, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [722] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 509, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [723] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 509, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [724] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 509, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [725] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 509, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [726] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 511, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [727] Ref(ReturnsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 511, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [728] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 511, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [729] Ref(FunctionDefinitionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 511, child_count: 8, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [730] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 519, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [731] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 519, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [732] Ref(UniqueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 519, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [733] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 519, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [734] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 519, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [735] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 519, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [736] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 519, child_count: 1, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [737] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 520, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [738] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 523, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [739] Delimited(2 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 525, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [740] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 527, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [741] Ref(IndexColumnDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 527, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [742] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 530, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [743] Ref(Business_timeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 530, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [744] Ref(WithoutKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 530, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [745] Ref(OverlapsKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 530, child_count: 13, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [746] AnySetOf(13 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 543, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [747] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 545, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [748] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 545, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [749] Ref(PartitionedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 545, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [750] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 547, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [751] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 547, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [752] Ref(TablespaceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 547, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [753] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 549, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [754] Ref(SpecificationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 549, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [755] Ref(OnlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 549, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [756] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 551, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [757] Ref(IncludeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 551, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [758] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 554, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [759] Delimited(2 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 556, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [760] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 558, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [761] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 558, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [762] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 558, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [763] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 561, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [764] Ref(XmlIndexSpecificationGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 561, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [765] Ref(ClusterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 561, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [766] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 564, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [767] Ref(ExtendKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 564, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [768] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 564, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [769] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 566, child_count: 4, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [770] Bracketed(2 elements -> 2 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 570, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [771] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 570, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [772] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 573, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [773] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 575, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [774] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 575, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [775] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 577, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [776] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 579, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [777] Ref(PctfreeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 579, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [778] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 579, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [779] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 582, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [780] Ref(Level2KeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 582, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [781] Ref(PctfreeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 582, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [782] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 582, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [783] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 584, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [784] Ref(MinpctusedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 584, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [785] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 584, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [786] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 587, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [787] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 589, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [788] Ref(AllowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 589, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [789] Ref(DisallowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 589, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [790] Ref(ReverseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 589, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [791] Ref(ScansKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 589, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [792] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 592, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [793] Ref(PageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 592, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [794] Ref(SplitKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 592, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [795] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 595, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [796] Ref(SymmetricKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 595, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [797] Ref(HighKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 595, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [798] Ref(LowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 595, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [799] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 598, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [800] Ref(CollectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 598, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [801] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 600, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [802] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 602, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [803] Ref(SampledKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 602, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [804] Ref(UnsampledKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 602, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [805] Ref(DetailedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 602, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [806] Ref(StatisticsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 602, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [807] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 604, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [808] Ref(CompressKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 604, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [809] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 606, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [810] Ref(YesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 606, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [811] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 606, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [812] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 609, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [813] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 611, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [814] Ref(IncludeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 611, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [815] Ref(ExcludeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 611, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [816] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 611, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [817] Ref(KeysKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 611, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [818] StringParser("CREATE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 611, child_count: 8, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [819] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 619, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [820] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 619, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [821] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 619, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [822] Ref(ModelKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 619, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [823] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 619, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [824] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 619, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [825] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 621, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [826] Ref(OptionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 621, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [827] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 624, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [828] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 626, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [829] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 629, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [830] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 629, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [831] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 629, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [832] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 631, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [833] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 631, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [834] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 634, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [835] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 636, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [836] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 636, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [837] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 636, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [838] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 636, child_count: 4, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [839] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 640, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [840] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 640, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [841] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 640, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [842] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 640, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [843] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 640, child_count: 4, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [844] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 644, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [845] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 644, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [846] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 644, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [847] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 644, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [848] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 644, child_count: 7, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [849] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 651, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [850] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 654, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [851] Ref(IncrementKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 654, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [852] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 654, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [853] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 654, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [854] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 657, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [855] Ref(StartKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 657, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [856] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 657, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [857] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 657, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [858] Ref(SequenceMinValueGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 657, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [859] Ref(SequenceMaxValueGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 657, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [860] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 659, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [861] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 661, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [862] Ref(CacheKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 661, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [863] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 661, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [864] Ref(NocacheKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 661, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [865] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 663, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [866] Ref(CycleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 663, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [867] Ref(NocycleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 663, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [868] Ref(OrderNoOrderGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 663, child_count: 4, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [869] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 667, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [870] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 667, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [871] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 667, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [872] Ref(SequenceReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 667, child_count: 1, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [873] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 668, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [874] Ref(CreateSequenceOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 668, child_count: 8, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [875] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 676, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [876] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 676, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [877] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 676, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [878] Ref(TemporaryTransientGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 676, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [879] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 676, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [880] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 676, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [881] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 676, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [882] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 679, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [883] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 681, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [884] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 684, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [885] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 686, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [886] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 688, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [887] Ref(TableConstraintSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 688, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [888] Ref(ColumnDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 688, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [889] Ref(CommentClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 688, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [890] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 690, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [891] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 690, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [892] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 692, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [893] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 695, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [894] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 695, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [895] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 697, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [896] Ref(LikeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 697, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [897] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 697, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [898] Ref(TableEndClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 697, child_count: 9, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [899] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 706, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [900] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 706, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [901] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 706, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [902] Ref(TriggerReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 706, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [903] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 709, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [904] Ref(BeforeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 709, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [905] Ref(AfterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 709, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [906] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 711, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [907] Ref(InsteadKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 711, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [908] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 711, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 1, _padding: 0 }, // [909] Delimited(3 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 713, child_count: 3, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [910] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 716, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [911] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 716, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [912] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 716, child_count: 3, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [913] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 719, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [914] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 719, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [915] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 719, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 2, _padding: 0 }, // [916] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 721, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [917] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 721, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [918] Ref(OrKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 721, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [919] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 721, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [920] Ref(OrKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 721, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [921] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 721, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [922] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 721, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [923] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 721, child_count: 5, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [924] AnyNumberOf(5 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 726, child_count: 9, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [925] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 735, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [926] Ref(ReferencingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 735, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [927] Ref(OldKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 735, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [928] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 735, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [929] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 735, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [930] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 735, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [931] Ref(NewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 735, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [932] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 735, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [933] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 735, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [934] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 735, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [935] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 737, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [936] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 737, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [937] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 737, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [938] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 739, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [939] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 741, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [940] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 741, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [941] Ref(DeferrableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 741, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [942] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 743, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [943] Ref(DeferrableKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 743, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [944] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 745, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [945] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 747, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [946] Ref(InitiallyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 747, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [947] Ref(ImmediateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 747, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [948] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 749, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [949] Ref(InitiallyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 749, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [950] Ref(DeferredKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 749, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [951] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 752, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [952] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 752, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [953] Ref(EachKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 752, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [954] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 754, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [955] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 754, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [956] Ref(StatementKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 754, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [957] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 756, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [958] Ref(WhenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 756, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [959] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 759, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [960] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 759, child_count: 4, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [961] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 763, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [962] Ref(ExecuteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 763, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [963] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 763, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [964] Ref(FunctionNameIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 763, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [965] Ref(FunctionContentsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 763, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [966] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 766, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [967] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 766, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [968] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 766, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [969] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 766, child_count: 10, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [970] Sequence(10 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 776, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [971] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 776, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [972] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 776, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [973] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 776, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [974] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 776, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [975] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 776, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [976] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 776, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [977] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 776, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [978] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 778, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [979] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 781, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [980] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 781, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [981] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 781, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [982] Ref(WithNoSchemaBindingClauseSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 781, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [983] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 783, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [984] Ref(RowMovementClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 783, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [985] Ref(ViewCheckOptionClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 783, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [986] StringParser("CREATEDB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 783, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [987] StringParser("CREATEROLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 783, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [988] StringParser("CREATEUSER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 783, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [989] StringParser("CROSS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 783, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [990] StringParser("CS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 783, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [991] StringParser("CSV")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 783, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [992] StringParser("CUBE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 783, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [993] StringParser("CUBE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 783, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [994] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 785, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [995] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 787, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [996] Ref(CubeFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 787, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [997] Ref(RollupFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 787, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [998] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 790, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [999] Ref(GroupingExpressionList)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 790, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1000] StringParser("CUME_DIST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 790, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1001] StringParser("CURRENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 790, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1002] StringParser("CURRENT_DATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 790, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1003] StringParser("CURRENT_DEFAULT_TRANSFORM_GROUP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 790, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1004] StringParser("CURRENT_LC_CTYPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 790, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1005] StringParser("CURRENT_PATH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 790, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1006] StringParser("CURRENT_ROLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 790, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1007] StringParser("CURRENT_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 790, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1008] StringParser("CURRENT_SERVER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 790, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1009] StringParser("CURRENT_TIME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 790, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1010] StringParser("CURRENT_TIMESTAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 790, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1011] StringParser("CURRENT_TIMEZONE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 790, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1012] StringParser("CURRENT_TRANSFORM_GROUP_FOR_TYPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 790, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1013] StringParser("CURRENT_USER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 790, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1014] StringParser("CURSOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 790, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1015] StringParser("CURSOR_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 790, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1016] StringParser("CYCLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 790, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1017] StringParser("DATA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 790, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1018] StringParser("DATABASE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 790, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1019] StringParser("DATABASES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 790, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1020] StringParser("DATAPARTITIONNAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 790, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1021] StringParser("DATAPARTITIONNUM")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 790, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1022] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 792, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1023] RegexParser
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 792, child_count: 1, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1024] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 793, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1025] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 793, child_count: 4, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1026] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 797, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1027] Ref(TimeWithTZGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 797, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1028] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 799, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1029] Ref(DoubleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 799, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1030] Ref(PrecisionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 799, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1031] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 802, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1032] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 804, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1033] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 806, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1034] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 808, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1035] Ref(CharacterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 808, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1036] Ref(BinaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 808, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1037] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 810, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1038] Ref(VaryingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 810, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1039] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 812, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1040] Ref(LargeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 812, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1041] Ref(ObjectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 812, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1042] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(1), first_child_idx: 814, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1043] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 816, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1044] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 816, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1045] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 816, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1046] Ref(DatatypeIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 816, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1047] Ref(BracketedArguments)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 816, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1048] AnyNumberOf(2 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 818, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1049] Ref(UnsignedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 818, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1050] Ref(CharCharacterSetGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 818, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1051] Ref(ArrayTypeSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 818, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1052] StringParser("DATE")
    GrammarInst { variant: GrammarVariant::MultiStringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 818, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1053] MultiStringParser(1 templates)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 818, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1054] Ref(DatePartFunctionName)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 818, child_count: 1, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1055] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 819, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1056] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 822, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1057] Delimited(2 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 824, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1058] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 826, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1059] Ref(DatetimeUnitSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 826, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1060] Ref(FunctionContentsGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 826, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1061] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 828, child_count: 4, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1062] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1063] Ref(DateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1064] Ref(TimeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1065] Ref(TimestampKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1066] Ref(IntervalKeywordSegment)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1067] TypedParser("single_quote")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1068] StringParser("DATETIME")
    GrammarInst { variant: GrammarVariant::MultiStringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1069] MultiStringParser(11 templates)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1070] StringParser("DATETIME_INTERVAL_CODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1071] StringParser("DATETIME_INTERVAL_PRECISION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1072] StringParser("DAY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1073] StringParser("DAY_HOUR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1074] StringParser("DAY_MICROSECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1075] StringParser("DAY_MINUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1076] StringParser("DAY_SECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1077] StringParser("DAYOFMONTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1078] StringParser("DAYOFWEEK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1079] StringParser("DAYOFYEAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1080] StringParser("DAYS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1081] StringParser("DB2GENERAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1082] StringParser("DB2GENRL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1083] StringParser("DB2SQL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1084] StringParser("DBCC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1085] StringParser("DBINFO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1086] StringParser("DBPARTITIONNAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1087] StringParser("DBPARTITIONNUM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1088] StringParser("DEALLOCATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1089] StringParser("DEC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1090] StringParser("DECFLOAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1091] StringParser("DECIMAL")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 832, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1092] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 835, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1093] Ref(DistributeKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 835, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1094] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 837, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1095] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 837, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1096] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 837, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1097] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 839, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1098] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 841, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1099] Ref(HashKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 841, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1100] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 844, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1101] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 846, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1102] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 846, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1103] Ref(RandomKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 846, child_count: 7, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1104] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 853, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1105] Ref(DeclareKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 853, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1106] Ref(GlobalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 853, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1107] Ref(TemporaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 853, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1108] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 853, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1109] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 853, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1110] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 856, child_count: 1, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1111] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 857, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1112] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 860, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1113] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 862, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1114] Ref(ColumnDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 862, child_count: 4, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1115] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 866, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1116] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 866, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1117] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 868, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1118] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 871, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1119] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 871, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1120] Ref(WithDataClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 871, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1121] Ref(CopyOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 871, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1122] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 874, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1123] Ref(LikeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 874, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1124] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 874, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1125] Ref(CopyOptionsSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 874, child_count: 6, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1126] AnySetOf(6 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 880, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1127] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 883, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1128] Ref(OrganizeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 883, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1129] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 883, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1130] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 885, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1131] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 885, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1132] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 885, child_count: 1, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1133] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 886, child_count: 4, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1134] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 890, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1135] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 890, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1136] Ref(CommitKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 890, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1137] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 892, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1138] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 892, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1139] Ref(PreserveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 892, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1140] Ref(RowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 892, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1141] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 894, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1142] Ref(LoggedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 894, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1143] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 897, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1144] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 897, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1145] Ref(LoggedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 897, child_count: 4, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1146] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 901, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1147] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 901, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1148] Ref(RollbackKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 901, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1149] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 903, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1150] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 903, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1151] Ref(PreserveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 903, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1152] Ref(RowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 903, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1153] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 905, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1154] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 905, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1155] Ref(ReplaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 905, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1156] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 907, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1157] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 907, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1158] Ref(TablespaceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 907, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1159] Ref(DeclareDistributionClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 907, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1160] StringParser("DECLARE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 907, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1161] StringParser("DEFAULT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 907, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1162] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 909, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1163] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 909, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1164] Ref(ValuesKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 909, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1165] StringParser("DEFAULTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 909, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1166] StringParser("DEFERRABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 909, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1167] StringParser("DEFERRED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 909, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1168] StringParser("DEFINED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 909, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1169] StringParser("DEFINER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 909, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1170] StringParser("DEFINITION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 909, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1171] StringParser("DEGREE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 909, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1172] StringParser("DELAY_KEY_WRITE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 909, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1173] StringParser("DELAYED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 909, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1174] StringParser("DELETE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 909, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1175] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 912, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1176] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 912, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1177] Ref(FromClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 912, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1178] Ref(WhereClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 912, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1179] Ref(SemicolonSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 912, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1180] StringParser("DELIMITER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 912, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1181] StringParser("DELIMITERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 912, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1182] StringParser("DENSE_RANK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 912, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1183] StringParser("DENSERANK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 912, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1184] StringParser("DENY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 912, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1185] StringParser("DEPTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 912, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1186] StringParser("DEREF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 912, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1187] StringParser("DERIVED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 912, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1188] StringParser("DESC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 912, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1189] StringParser("DESCRIBE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 912, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1190] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 915, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1191] Ref(DescribeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 915, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1192] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 915, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1193] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 915, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1194] StringParser("DESCRIPTOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 915, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1195] StringParser("DESTROY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 915, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1196] StringParser("DESTRUCTOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 915, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1197] StringParser("DETAILED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 915, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1198] StringParser("DETERMINISTIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 915, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1199] StringParser("DIAGNOSTICS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 915, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1200] StringParser("DICTIONARY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 915, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1201] StringParser("DISABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 915, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1202] StringParser("DISALLOW")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 915, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1203] StringParser("DISCONNECT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 915, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1204] StringParser("DISK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 915, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1205] StringParser("DISPATCH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 915, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1206] StringParser("DISTINCT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 915, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1207] StringParser("DISTINCTROW")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 915, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1208] StringParser("DISTRIBUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 915, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1209] StringParser("DISTRIBUTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 915, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1210] StringParser("DIV")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 915, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1211] StringParser("/")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 915, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1212] StringParser("DO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 915, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1213] StringParser("DOCUMENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 915, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1214] StringParser("DOMAIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 915, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1215] StringParser(".")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 915, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1216] StringParser("DOUBLE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 915, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1217] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 917, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1218] Ref(RestrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 917, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1219] Ref(CascadeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 917, child_count: 4, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1220] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 921, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1221] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 921, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1222] Ref(CastKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 921, child_count: 5, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1223] Bracketed(3 elements -> 3 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 926, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1224] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 926, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1225] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 926, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1226] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 926, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1227] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 926, child_count: 5, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1228] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 931, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1229] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 931, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1230] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 931, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1231] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 931, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1232] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 931, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1233] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 931, child_count: 4, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1234] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 935, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1235] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 935, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1236] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 935, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1237] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 935, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1238] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 935, child_count: 5, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1239] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 940, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1240] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 940, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1241] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 940, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1242] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 940, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1243] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 940, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1244] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 940, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1245] StringParser("DROP")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 940, child_count: 4, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1246] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 944, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1247] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 944, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1248] Ref(ModelKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 944, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1249] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 944, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1250] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 944, child_count: 4, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1251] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 948, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1252] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 948, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1253] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 948, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1254] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 948, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1255] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 948, child_count: 5, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1256] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 953, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1257] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 953, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1258] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 953, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1259] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 953, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1260] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 953, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1261] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 953, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1262] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 956, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1263] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 956, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1264] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 956, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1265] Ref(SequenceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 956, child_count: 6, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1266] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 962, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1267] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 962, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1268] Ref(TemporaryGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 962, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1269] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 962, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1270] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 962, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1271] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 964, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1272] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 964, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1273] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 964, child_count: 4, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1274] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 968, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1275] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 968, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1276] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 968, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1277] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 968, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1278] Ref(TriggerReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 968, child_count: 5, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1279] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 973, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1280] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 973, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1281] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 973, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1282] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 973, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1283] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 973, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1284] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 973, child_count: 4, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1285] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 977, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1286] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 977, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1287] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 977, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1288] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 977, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1289] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 977, child_count: 5, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1290] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 982, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1291] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 982, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1292] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 982, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1293] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 982, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1294] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 982, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1295] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 982, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1296] StringParser("DSSIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 982, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1297] StringParser("DUMMY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 982, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1298] StringParser("DUMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 982, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1299] StringParser("DYNAMIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 982, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1300] StringParser("DYNAMIC_FUNCTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 982, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1301] StringParser("DYNAMIC_FUNCTION_CODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 982, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1302] StringParser("EACH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 982, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1303] StringParser("EDITPROC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 982, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1304] StringParser("ELEMENT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 982, child_count: 4, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1305] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 986, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1306] Ref(ElseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 986, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1307] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 986, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1308] StringParser("ELSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 986, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1309] StringParser("ELSEIF")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 986, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1310] Bracketed(0 elements -> 0 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 988, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1311] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 990, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1312] Ref(StructTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 990, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1313] Ref(EmptyStructLiteralBracketsSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 990, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1314] StringParser("ENABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 990, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1315] StringParser("ENCLOSED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 990, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1316] StringParser("ENCODING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 990, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1317] StringParser("ENCRYPTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 990, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1318] StringParser("ENCRYPTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 990, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1319] StringParser("END-EXEC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 990, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1320] StringParser("}")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 990, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1321] StringParser("END")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 990, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1322] StringParser("ENDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 990, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1323] StringParser("ENUM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 990, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1324] StringParser("EQUALS")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 990, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1325] Ref(RawEqualsSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 990, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1326] StringParser("ERASE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 990, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1327] StringParser("ERRLVL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 990, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1328] StringParser("ESCAPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 990, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1329] StringParser("ESCAPED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 990, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1330] StringParser("EVERY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 990, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1331] StringParser("EXCEPT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 990, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1332] StringParser("EXCEPTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 990, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1333] StringParser("EXCLUDE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 990, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1334] StringParser("EXCLUDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 990, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1335] StringParser("EXCLUSIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 990, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1336] StringParser("EXEC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 990, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1337] StringParser("EXECUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 990, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1338] StringParser("EXECUTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 990, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1339] StringParser("EXISTING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 990, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1340] StringParser("EXISTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 990, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1341] StringParser("EXIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 990, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1342] StringParser("EXP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 990, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1343] StringParser("EXPLAIN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 990, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1344] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 992, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1345] Ref(ExplainKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 992, child_count: 4, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1346] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 996, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1347] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 996, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1348] Ref(InsertStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 996, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1349] Ref(UpdateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 996, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1350] Ref(DeleteStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 996, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1351] Ref(Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 996, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1352] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 998, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1353] Ref(Tail_Recurse_Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 998, child_count: 1, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1354] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 999, child_count: 10, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1355] OneOf(10 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1009, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1356] Ref(LikeExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1009, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1357] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1011, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1358] Ref(BinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1011, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1359] Ref(Tail_Recurse_Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1011, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1360] Ref(InOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1011, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1361] Ref(QuantifiedComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1011, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1362] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1014, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1363] Ref(IsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1014, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1364] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1014, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1365] Ref(IsClauseGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1014, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1366] Ref(IsNullGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1014, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1367] Ref(NotNullGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1014, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1368] Ref(CollateGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1014, child_count: 5, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1369] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1019, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1370] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1019, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1371] Ref(BetweenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1019, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1372] Ref(Expression_B_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1019, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1373] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1019, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1374] Ref(Tail_Recurse_Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1019, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1375] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1021, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1376] Ref(PatternMatchingGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1021, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1377] Ref(Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1021, child_count: 4, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1378] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 1025, child_count: 1, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1379] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1026, child_count: 1, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1380] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1027, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1381] Ref(QualifiedNumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1027, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1382] Ref(TildeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1027, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1383] Ref(NotOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1027, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1384] Ref(PriorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1027, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1385] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1029, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1386] Ref(Tail_Recurse_Expression_B_Grammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1029, child_count: 1, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1387] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1030, child_count: 1, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1388] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1031, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1389] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1033, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1390] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1036, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1391] Ref(ArithmeticBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1036, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1392] Ref(StringBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1036, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1393] Ref(ComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1036, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1394] Ref(Tail_Recurse_Expression_B_Grammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1036, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1395] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 1038, child_count: 1, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1396] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1039, child_count: 1, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1397] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1040, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1398] Ref(QualifiedNumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1040, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1399] Ref(TildeSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1040, child_count: 4, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1400] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1044, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1401] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1046, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1402] Ref(ExistsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1046, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1403] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1049, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1404] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1049, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1405] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1051, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1406] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1053, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1407] Ref(Expression_D_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1053, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1408] Ref(CaseExpressionSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1053, child_count: 1, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1409] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1054, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1410] Ref(TimeZoneGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1054, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1411] Ref(ShorthandCastSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1054, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1412] Ref(LabeledDurationGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1054, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1413] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1056, child_count: 9, min_times: 0, first_terminator_idx: 19, terminator_count: 1, _padding: 0 }, // [1414] OneOf(9 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1065, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1415] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1065, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1416] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 1065, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1417] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1068, child_count: 4, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1418] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1072, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1419] Ref(LateralColumnAliasExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1072, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1420] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1072, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1421] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1072, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1422] Delimited(5 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1074, child_count: 5, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1423] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1079, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1424] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1079, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1425] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1079, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1426] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1079, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1427] Ref(LocalAliasSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1079, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1428] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1079, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1429] Ref(Expression_D_Potential_Select_Statement_Without_Brackets)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1079, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1430] Ref(SingleIdentifierWildcardSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1079, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1431] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1081, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1432] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1083, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1433] Ref(StructTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1083, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1434] Ref(MapTypeSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1083, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1435] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1086, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1436] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1088, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1437] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1088, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1438] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1090, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1439] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1090, child_count: 5, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1440] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1095, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1441] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1095, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1442] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1095, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1443] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1095, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1444] Ref(NullLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1095, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1445] Ref(DateTimeLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1095, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1446] Ref(LocalAliasSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1095, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1447] Ref(ListComprehensionGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1095, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1448] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1095, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1449] Ref(AccessorGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1095, child_count: 7, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1450] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1102, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1451] Ref(SelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1102, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1452] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1102, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1453] Ref(IntervalExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1102, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1454] Ref(TypedStructLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1102, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1455] Ref(ArrayExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1102, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1456] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1102, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1457] Ref(OverlapsClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1102, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1458] StringParser("EXTEND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1102, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1459] StringParser("EXTENDED")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1102, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1460] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1102, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1461] StringParser("EXTENSION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1102, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1462] StringParser("EXTERNAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1102, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1463] StringParser("EXTRACT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1102, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1464] StringParser("FALSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1102, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1465] StringParser("FALSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1102, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1466] StringParser("FEDERATED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1102, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1467] StringParser("FENCED")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1102, child_count: 5, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1468] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1107, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1469] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1107, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1470] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1109, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1471] Ref(FirstKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1109, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1472] Ref(NextKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1109, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1473] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1111, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1474] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 1111, child_count: 1, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1475] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1112, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1476] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1112, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1477] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1114, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1478] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1114, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1479] Ref(RowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1114, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1480] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1116, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1481] Ref(OnlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1116, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1482] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1118, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1483] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1118, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1484] Ref(TiesKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1118, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1485] StringParser("FETCH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1118, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1486] StringParser("FIELDPROC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1118, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1487] StringParser("FIELDS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1118, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1488] StringParser("FILE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1118, child_count: 3, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1489] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1121, child_count: 1, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1490] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1122, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1491] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(12), first_child_idx: 1122, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1492] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1124, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1493] Ref(StatementSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1124, child_count: 1, min_times: 1, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1494] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1125, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1495] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1125, child_count: 1, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1496] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1126, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1497] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1126, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1498] StringParser("FILLFACTOR")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1126, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1499] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1128, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1500] Ref(FilterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1128, child_count: 3, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1501] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1131, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1502] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1133, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1503] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1133, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1504] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1133, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1505] StringParser("FILTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1133, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1506] StringParser("FINAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1133, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1507] StringParser("FIRST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1133, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1508] StringParser("FLOAT4")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1133, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1509] StringParser("FLOAT8")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1133, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1510] StringParser("FLOAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1133, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1511] StringParser("FLOOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1133, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1512] StringParser("FLUSH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1133, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1513] StringParser("FOLLOWING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1133, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1514] StringParser("FOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1133, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1515] StringParser("FORCE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1133, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1516] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1135, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1517] Ref(ForeignKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1135, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1518] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1135, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1519] StringParser("FOREIGN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1135, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1520] StringParser("FORMAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1135, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1521] StringParser("FORTRAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1135, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1522] StringParser("FORWARD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1135, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1523] StringParser("FOUND")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1135, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1524] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1137, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1525] Ref(FrameClauseUnitGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1137, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1526] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1139, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1527] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1141, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1528] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1143, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1529] Ref(CurrentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1143, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1530] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1143, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1531] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1145, child_count: 4, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1532] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1149, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1533] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1149, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1534] Ref(IntervalExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1149, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1535] Ref(UnboundedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1149, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1536] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1149, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1537] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1151, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1538] Ref(PrecedingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1151, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1539] Ref(FollowingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1151, child_count: 4, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1540] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1155, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1541] Ref(BetweenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1155, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1542] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1155, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1543] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1157, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1544] Ref(RowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1157, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1545] Ref(RangeKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1157, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1546] StringParser("FREE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1157, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1547] StringParser("FREETEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1157, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1548] StringParser("FREETEXTTABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1157, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1549] StringParser("FREEZE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1157, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1550] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1159, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1551] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1159, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1552] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1161, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1553] Ref(FromExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1161, child_count: 17, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1554] OneOf(17 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1178, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1555] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1178, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1556] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1178, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1557] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1180, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1558] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1180, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1559] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1180, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1560] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1182, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1561] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1182, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1562] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1182, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1563] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1182, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1564] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1182, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1565] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1182, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1566] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1182, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1567] Ref(WithNoSchemaBindingClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1182, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1568] Ref(WithDataClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1182, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1569] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1182, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1570] Ref(OffsetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1182, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1571] Ref(OffsetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1182, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1572] Ref(ReadOnlyClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1182, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1573] Ref(IsolationClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1182, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1574] Ref(RowMovementClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1182, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1575] Ref(ViewCheckOptionClauseSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1182, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1576] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1184, child_count: 7, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1577] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1191, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1578] Ref(PreTableFunctionKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1191, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1579] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1193, child_count: 3, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1580] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1196, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1581] Ref(TableExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1196, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1582] Ref(TemporalQuerySegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(69), first_child_idx: 1196, child_count: 1, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1583] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1197, child_count: 4, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1584] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1201, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1585] Ref(FromClauseTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1201, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1586] Ref(SamplingExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1201, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1587] Ref(JoinLikeClauseGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1201, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1588] Ref(JoinClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1201, child_count: 3, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1589] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1204, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1590] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1204, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1591] Ref(OffsetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1204, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1592] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1204, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1593] Ref(SamplingExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1204, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1594] Ref(PostTableExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1204, child_count: 3, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1595] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1207, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1596] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1209, child_count: 1, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1597] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1210, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1598] Ref(JoinClauseSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1210, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1599] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1212, child_count: 3, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1600] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1215, child_count: 6, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1601] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1221, child_count: 3, min_times: 0, first_terminator_idx: 20, terminator_count: 2, _padding: 0 }, // [1602] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1224, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1603] Ref(MLTableExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1224, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1604] Ref(FromExpressionElementSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1224, child_count: 3, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1605] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1227, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1606] Ref(FromExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1227, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1607] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1229, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1608] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1229, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1609] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1229, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1610] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1231, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1611] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1231, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1612] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 1231, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1613] Meta(conditional: indented_joins=True -> indent)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1231, child_count: 1, min_times: 0, first_terminator_idx: 22, terminator_count: 2, _padding: 0 }, // [1614] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1232, child_count: 1, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1615] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1233, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1616] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1235, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1617] Ref(JoinClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1235, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1618] Ref(JoinLikeClauseGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1235, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1619] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1237, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1620] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1237, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1621] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1237, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1622] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1239, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1623] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1239, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1624] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 1239, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1625] Meta(conditional: indented_joins=True -> dedent)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1239, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1626] StringParser("FROM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1239, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1627] StringParser("FULL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1239, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1628] StringParser("FULLTEXT")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1239, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1629] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1241, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1630] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1241, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1631] Ref(NamedArgumentSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1241, child_count: 11, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1632] AnyNumberOf(11 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1252, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1633] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1252, child_count: 3, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1634] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1255, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1635] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1255, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1636] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1255, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1637] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1255, child_count: 4, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1638] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1259, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1639] Ref(TrimParametersGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(69), first_child_idx: 1259, child_count: 1, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1640] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1260, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1641] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1260, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1642] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1260, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1643] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1260, child_count: 3, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1644] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1263, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1645] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1265, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1646] Ref(DatetimeUnitSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1265, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1647] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1265, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1648] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1265, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1649] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1265, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1650] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1267, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1651] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1267, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1652] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1269, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1653] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1269, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1654] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1271, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1655] Ref(FunctionContentsExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1271, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1656] Ref(AggregateOrderByClause)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1271, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1657] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1273, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1658] Ref(SeparatorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1273, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1659] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1273, child_count: 3, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1660] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1276, child_count: 3, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1661] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1279, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1662] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1279, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1663] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1279, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1664] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1279, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1665] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1279, child_count: 3, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1666] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1282, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1667] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1282, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1668] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1282, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1669] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1282, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1670] Ref(IgnoreRespectNullsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1282, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1671] Ref(IndexColumnDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1282, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1672] Ref(EmptyStructLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1282, child_count: 1, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1673] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1283, child_count: 3, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1674] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1286, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1675] Ref(FunctionContentsGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1286, child_count: 3, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1676] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1289, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1677] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1289, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1678] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1289, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1679] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1291, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1680] Ref(LanguageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1291, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1681] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1291, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1682] StringParser("FUNCTION")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1291, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1683] TypedParser("word")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1291, child_count: 2, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1684] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1293, child_count: 1, min_times: 0, first_terminator_idx: 24, terminator_count: 1, _padding: 0 }, // [1685] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1294, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1686] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1296, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1687] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1296, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1688] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1296, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1689] Ref(BracketedSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1296, child_count: 2, min_times: 0, first_terminator_idx: 25, terminator_count: 1, _padding: 0 }, // [1690] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1298, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1691] Ref(FunctionNameIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1298, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1692] Ref(QuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1298, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1693] Ref(BracketedSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1298, child_count: 2, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1694] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1300, child_count: 2, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1695] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1302, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1696] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1302, child_count: 2, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1697] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1304, child_count: 2, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1698] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1306, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1699] Ref(AnyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1306, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1700] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1306, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1701] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1306, child_count: 2, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1702] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1308, child_count: 2, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1703] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1310, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1704] Ref(AnyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1310, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1705] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1310, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1706] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1310, child_count: 3, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1707] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1313, child_count: 2, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1708] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1315, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1709] Ref(FunctionParameterGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1315, child_count: 3, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1710] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1318, child_count: 2, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1711] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1320, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1712] Ref(DatePartFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1320, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1713] Ref(DateTimeFunctionContentsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1320, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1714] Ref(ColumnsExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1320, child_count: 2, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1715] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1322, child_count: 2, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1716] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 1324, child_count: 1, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1717] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1325, child_count: 3, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1718] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1328, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1719] Ref(DatePartFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1328, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1720] Ref(ColumnsExpressionFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1328, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1721] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1328, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1722] Ref(FunctionContentsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1328, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1723] Ref(PostFunctionGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1328, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1724] StringParser("FUNCTIONS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1328, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1725] StringParser("FUSION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1328, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1726] StringParser("FUTURE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1328, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1727] StringParser("G")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1328, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1728] StringParser("GENERAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1328, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1729] StringParser("GENERATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1328, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1730] StringParser("GENERATED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1328, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1731] StringParser("GET")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1328, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1732] TypedParser("glob_operator")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1328, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1733] StringParser("GLOBAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1328, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1734] StringParser("GO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1328, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1735] StringParser("GOTO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1328, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1736] StringParser("GRANT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1328, child_count: 6, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1737] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1334, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1738] Ref(GrantKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1334, child_count: 4, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1739] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1338, child_count: 3, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1740] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1341, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1741] Ref(AccessPermissionsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1341, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1742] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1341, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1743] Ref(AccessObjectSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1341, child_count: 2, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1744] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1343, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1745] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1343, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1746] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1343, child_count: 4, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1747] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1347, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1748] Ref(OwnershipKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1347, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1749] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1347, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1750] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1347, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1751] Ref(UserReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1347, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1752] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1347, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1753] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1347, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1754] Ref(AccessTargetSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1347, child_count: 3, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1755] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1350, child_count: 3, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1756] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1353, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1757] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1353, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1758] Ref(GrantKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1353, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1759] Ref(OptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1353, child_count: 3, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1760] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1356, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1761] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1356, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1762] Ref(AdminKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1356, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1763] Ref(OptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1356, child_count: 3, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1764] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1359, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1765] Ref(CopyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1359, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1766] Ref(CurrentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1359, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1767] Ref(GrantsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1359, child_count: 3, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1768] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1362, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1769] Ref(GrantedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1362, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1770] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1362, child_count: 3, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1771] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1365, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1772] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1365, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1773] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1365, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1774] Ref(UserReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1365, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1775] StringParser("GRANTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1365, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1776] StringParser("GRANTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1365, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1777] StringParser("GRAPHIC")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1365, child_count: 2, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1778] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1367, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1779] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1367, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1780] Ref(RawEqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1367, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1781] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1367, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1782] StringParser("GREATEST")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1367, child_count: 5, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1783] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1372, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1784] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1372, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1785] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1372, child_count: 4, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1786] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1376, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1787] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1376, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1788] Ref(GroupingSetsClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1376, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1789] Ref(CubeRollupClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1376, child_count: 1, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1790] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1377, child_count: 2, min_times: 0, first_terminator_idx: 26, terminator_count: 1, _padding: 0 }, // [1791] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1379, child_count: 3, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1792] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1382, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1793] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1382, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1794] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1382, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1795] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1382, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1796] Ref(GroupByClauseTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1382, child_count: 11, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1797] OneOf(11 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1393, child_count: 2, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1798] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1395, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1799] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1395, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1800] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1395, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1801] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1395, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1802] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1395, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1803] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1395, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1804] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1395, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1805] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1395, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1806] Ref(OffsetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1395, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1807] Ref(ReadOnlyClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1395, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1808] Ref(IsolationClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1395, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1809] Ref(RowMovementClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1395, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1810] Ref(ViewCheckOptionClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1395, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1811] StringParser("GROUP")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1395, child_count: 1, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1812] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1396, child_count: 2, min_times: 0, first_terminator_idx: 27, terminator_count: 1, _padding: 0 }, // [1813] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1398, child_count: 4, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1814] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1402, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1815] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1402, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1816] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1402, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1817] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1402, child_count: 2, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1818] Bracketed(0 elements -> 0 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1404, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1819] Ref(GroupByClauseTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1404, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1820] StringParser("GROUPING")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1404, child_count: 3, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1821] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1407, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1822] Ref(GroupingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1407, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1823] Ref(SetsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1407, child_count: 3, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1824] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1410, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1825] Delimited(2 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1412, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1826] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1414, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1827] Ref(CubeRollupClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1414, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1828] Ref(GroupingExpressionList)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1414, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1829] StringParser("HANDLER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1414, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1830] StringParser("HASH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1414, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1831] StringParser("HASHED_VALUE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1414, child_count: 4, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1832] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1418, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1833] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1418, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1834] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1420, child_count: 3, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1835] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1423, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1836] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1423, child_count: 10, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1837] OneOf(10 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1433, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1838] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1435, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1839] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1435, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1840] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1435, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1841] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1435, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1842] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1435, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1843] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1435, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1844] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1435, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1845] Ref(OffsetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1435, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1846] Ref(ReadOnlyClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1435, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1847] Ref(IsolationClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1435, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1848] Ref(RowMovementClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1435, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1849] Ref(ViewCheckOptionClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1435, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1850] StringParser("HAVING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1435, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1851] StringParser("HEADER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1435, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1852] StringParser("HEAP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1435, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1853] StringParser("HIERARCHY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1435, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1854] StringParser("HIGH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1435, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1855] StringParser("HIGH_PRIORITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1435, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1856] StringParser("HINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1435, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1857] StringParser("HOLD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1435, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1858] StringParser("HOLDLOCK")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1435, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1859] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1435, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1860] StringParser("HOST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1435, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1861] StringParser("HOSTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1435, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1862] StringParser("HOUR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1435, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1863] StringParser("HOUR_MICROSECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1435, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1864] StringParser("HOUR_MINUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1435, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1865] StringParser("HOUR_SECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1435, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1866] StringParser("HOURS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1435, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1867] StringParser("IDENTIFIED")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1435, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1868] Token("identifier")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1435, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1869] StringParser("IDENTITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1435, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1870] StringParser("IDENTITY_INSERT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1435, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1871] StringParser("IDENTITYCOL")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1435, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1872] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1437, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1873] Ref(IfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1437, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1874] Ref(ExistsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1437, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1875] StringParser("IF")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1437, child_count: 3, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1876] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1440, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1877] Ref(IfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1440, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1878] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1440, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1879] Ref(ExistsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1440, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1880] StringParser("IGNORE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1440, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1881] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1442, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1882] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1444, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1883] Ref(IgnoreKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1444, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1884] Ref(RespectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1444, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1885] Ref(NullsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1444, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1886] StringParser("ILIKE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1444, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1887] StringParser("IMMEDIATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1444, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1888] StringParser("IMMUTABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1444, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1889] StringParser("IMPLEMENTATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1444, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1890] StringParser("IMPLICIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1444, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1891] StringParser("IMPORT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1444, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1892] StringParser("IMPORTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1444, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1893] StringParser("IN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1444, child_count: 3, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1894] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1447, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1895] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1447, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1896] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1447, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1897] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 1449, child_count: 3, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1898] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1452, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1899] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1454, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1900] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1456, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1901] Ref(Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1456, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1902] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1456, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1903] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1456, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1904] StringParser("INCLUDE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1456, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1905] StringParser("INCLUDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1456, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1906] StringParser("INCLUSIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1456, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1907] StringParser("INCREMENT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1456, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1908] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1458, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1909] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1460, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1910] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1460, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1911] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1460, child_count: 3, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1912] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1463, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1913] Ref(AscKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1463, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1914] Ref(DescKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1463, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1915] Ref(RandomKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1463, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1916] StringParser("INDEX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1463, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1917] StringParser("INDICATOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1463, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1918] StringParser("INDICATORS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1463, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1919] StringParser("INF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1463, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1920] StringParser("INFILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1463, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1921] StringParser("INFINITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1463, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1922] StringParser("INFIX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1463, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1923] StringParser("INHERIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1463, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1924] StringParser("INHERITS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1463, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1925] StringParser("INITIAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1463, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1926] StringParser("INITIALIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1463, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1927] StringParser("INITIALLY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1463, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1928] StringParser("INNER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1463, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1929] StringParser("INOUT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1463, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1930] StringParser("INPUT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1463, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1931] StringParser("INSENSITIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1463, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1932] StringParser("INSERT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1463, child_count: 5, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1933] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1468, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1934] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1468, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1935] Ref(OverwriteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1468, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1936] Ref(IntoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1468, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1937] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1468, child_count: 3, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1938] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1471, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1939] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1471, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1940] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1473, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1941] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1473, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1942] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1473, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1943] Ref(DefaultValuesGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1473, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1944] StringParser("INSERT_ID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1473, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1945] StringParser("INSTANCE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1473, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1946] StringParser("INSTANTIABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1473, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1947] StringParser("INSTEAD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1473, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1948] StringParser("INT1")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1473, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1949] StringParser("INT2")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1473, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1950] StringParser("INT3")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1473, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1951] StringParser("INT4")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1473, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1952] StringParser("INT8")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1473, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1953] StringParser("INT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1473, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1954] StringParser("INTEGER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1473, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1955] StringParser("INTEGRATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1473, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1956] StringParser("INTEGRATIONS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1473, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1957] StringParser("INTEGRITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1473, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1958] StringParser("INTERSECT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1473, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1959] StringParser("INTERSECTION")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1473, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1960] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1475, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1961] Ref(IntervalKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1475, child_count: 3, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1962] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1478, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1963] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1480, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1964] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1480, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1965] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1482, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1966] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1482, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1967] Ref(DatetimeUnitSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1482, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1968] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1482, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1969] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1484, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1970] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1484, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1971] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1486, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1972] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1486, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1973] Ref(DatetimeUnitSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1486, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1974] StringParser("INTERVAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1486, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1975] StringParser("INTO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1486, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1976] StringParser("INVOKER")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1486, child_count: 5, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1977] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1491, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1978] Ref(NullLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1491, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1979] Ref(NanLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1491, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1980] Ref(UnknownLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1491, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1981] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1491, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1982] Ref(NormalizedGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1491, child_count: 4, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1983] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1495, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1984] Ref(IsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1495, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1985] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1495, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1986] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1495, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1987] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1495, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1988] StringParser("IS")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1495, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1989] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1495, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1990] StringParser("ISAM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1495, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1991] StringParser("ISNULL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1495, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1992] StringParser("ISOBID")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1495, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1993] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1497, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1994] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1497, child_count: 3, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1995] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1500, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1996] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1502, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1997] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1504, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1998] Ref(RrKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1504, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1999] Ref(RsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1504, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2000] Ref(LockClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1504, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2001] Ref(CsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1504, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2002] Ref(UrKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1504, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2003] StringParser("ISOLATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1504, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2004] StringParser("ITERATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1504, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2005] StringParser("JAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1504, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2006] StringParser("JAVA")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1504, child_count: 3, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2007] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1507, child_count: 7, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2008] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1514, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2009] Ref(ConditionalJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1514, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2010] Ref(JoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1514, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2011] Ref(FromExpressionElementSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1514, child_count: 1, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2012] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1515, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2013] Ref(NestedJoinGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1515, child_count: 4, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2014] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 1519, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2015] Meta(conditional: indented_using_on=True -> indent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1519, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2016] Ref(MatchConditionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1519, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2017] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1521, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2018] Ref(JoinOnConditionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1521, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2019] Ref(JoinUsingConditionGrammar)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 1521, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2020] Meta(conditional: indented_using_on=True -> dedent)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1521, child_count: 6, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2021] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1527, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2022] Ref(UnconditionalJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1527, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2023] Ref(JoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1527, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2024] Ref(FromExpressionElementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1527, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2025] Ref(MatchConditionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1527, child_count: 4, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2026] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1531, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2027] Ref(ExtendedNaturalJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1531, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2028] Ref(FromExpressionElementSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1531, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2029] StringParser("JOIN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1531, child_count: 1, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2030] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1532, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2031] Ref(JoinKeywordSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1532, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2032] Nothing
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1532, child_count: 4, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2033] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1536, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2034] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 1536, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2035] Meta(conditional: indented_on_contents=True -> implicit_indent)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1536, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2036] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1538, child_count: 3, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2037] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1541, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2038] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 1541, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2039] Meta(conditional: indented_on_contents=True -> dedent)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1541, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2040] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1543, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2041] Ref(InnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1543, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2042] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1545, child_count: 3, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2043] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1548, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2044] Ref(FullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1548, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2045] Ref(LeftKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1548, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2046] Ref(RightKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1548, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2047] Ref(OuterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1548, child_count: 4, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2048] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1552, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2049] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 1552, child_count: 3, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2050] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1555, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2051] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1557, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2052] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1557, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2053] StringParser("JSON")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1557, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2054] StringParser("K")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1557, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2055] StringParser("KEEP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1557, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2056] StringParser("KEY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1557, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2057] StringParser("KEY_MEMBER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1557, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2058] StringParser("KEY_TYPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1557, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2059] StringParser("KEYS")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1557, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2060] Token("keyword")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1557, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2061] StringParser("KILL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1557, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2062] StringParser("LABEL")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1557, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2063] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1559, child_count: 5, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2064] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1564, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2065] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1564, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2066] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1564, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2067] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1564, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2068] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1564, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2069] Ref(Expression_D_Grammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1564, child_count: 14, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2070] OneOf(14 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1578, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2071] Ref(DayKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1578, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2072] Ref(DaysKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1578, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2073] Ref(HourKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1578, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2074] Ref(HoursKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1578, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2075] Ref(MicrosecondKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1578, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2076] Ref(MicrosecondsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1578, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2077] Ref(MinuteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1578, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2078] Ref(MinutesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1578, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2079] Ref(MonthKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1578, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2080] Ref(MonthsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1578, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2081] Ref(SecondKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1578, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2082] Ref(SecondsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1578, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2083] Ref(YearKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1578, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2084] Ref(YearsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1578, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2085] StringParser("LANCOMPILER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1578, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2086] StringParser("LANGUAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1578, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2087] StringParser("LARGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1578, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2088] StringParser("LAST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1578, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2089] StringParser("LAST_INSERT_ID")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1578, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2090] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1578, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2091] StringParser("LATERAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1578, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2092] StringParser("LC_CTYPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1578, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2093] StringParser("LC_MESSAGES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1578, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2094] StringParser("LC_TIME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1578, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2095] StringParser("LEADING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1578, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2096] StringParser("LEAST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1578, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2097] StringParser("LEAVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1578, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2098] StringParser("LEFT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1578, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2099] StringParser("LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1578, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2100] StringParser("LESS")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1578, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2101] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1580, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2102] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1580, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2103] Ref(RawEqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1580, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2104] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1580, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2105] StringParser("LEVEL2")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1580, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2106] StringParser("LEVEL")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1580, child_count: 3, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2107] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1583, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2108] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1585, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2109] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1585, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2110] Ref(LikeGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1585, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2111] Ref(Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1585, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2112] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1587, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2113] Ref(EscapeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1587, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2114] Ref(Tail_Recurse_Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1587, child_count: 3, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2115] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1590, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2116] Ref(LikeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1590, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2117] Ref(RlikeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1590, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2118] Ref(IlikeKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1590, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2119] StringParser("LIKE")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1590, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2120] TypedParser("like_operator")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1590, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2121] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1592, child_count: 5, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2122] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1597, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2123] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1597, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2124] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1599, child_count: 3, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2125] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1602, child_count: 3, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2126] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1605, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2127] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1605, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2128] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1605, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2129] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1605, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2130] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1607, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2131] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1609, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2132] Ref(OffsetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1609, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2133] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1611, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2134] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1611, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2135] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1611, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2136] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1613, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2137] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1613, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2138] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1613, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2139] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1615, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2140] Ref(OffsetClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1615, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2141] Ref(FetchClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1615, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2142] StringParser("LIMIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1615, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2143] StringParser("LINENO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1615, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2144] StringParser("LINES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1615, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2145] StringParser("LINKTYPE")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1615, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2146] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1615, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2147] StringParser("LISTEN")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1615, child_count: 9, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2148] OneOf(9 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1624, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2149] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1624, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2150] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1624, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2151] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1624, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2152] Ref(QualifiedNumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1624, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2153] Ref(NullLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1624, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2154] Ref(DateTimeLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1624, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2155] Ref(ArrayLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1624, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2156] Ref(TypedArrayLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1624, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2157] Ref(ObjectLiteralSegment)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1624, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2158] Token("literal")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1624, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2159] Token("literal")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1624, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2160] StringParser("LN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1624, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2161] StringParser("LOAD")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1624, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2162] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1624, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2163] StringParser("LOCAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1624, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2164] StringParser("LOCALDATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1624, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2165] StringParser("LOCALE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1624, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2166] StringParser("LOCALTIME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1624, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2167] StringParser("LOCALTIMESTAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1624, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2168] StringParser("LOCATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1624, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2169] StringParser("LOCATOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1624, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2170] StringParser("LOCATORS")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1624, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2171] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1626, child_count: 5, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2172] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1631, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2173] Ref(UseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1631, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2174] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1631, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2175] Ref(KeepKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1631, child_count: 3, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2176] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1634, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2177] Ref(ExclusiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1634, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2178] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1634, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2179] Ref(ShareKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1634, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2180] Ref(LocksKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1634, child_count: 3, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2181] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1637, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2182] Ref(KeepKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1637, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2183] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1637, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2184] Ref(LocksKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1637, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2185] StringParser("LOCK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1637, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2186] StringParser("LOCKMAX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1637, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2187] StringParser("LOCKS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1637, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2188] StringParser("LOCKSIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1637, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2189] StringParser("LOGGED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1637, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2190] StringParser("LOGIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1637, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2191] StringParser("LOGS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1637, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2192] StringParser("LONG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1637, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2193] StringParser("LONGBLOB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1637, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2194] StringParser("LONGTEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1637, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2195] StringParser("LOOP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1637, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2196] StringParser("LOW")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1637, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2197] StringParser("LOW_PRIORITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1637, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2198] StringParser("LOWER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1637, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2199] StringParser("M")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1637, child_count: 4, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2200] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1641, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2201] Ref(MlKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1641, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2202] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1641, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2203] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1641, child_count: 4, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2204] Bracketed(2 elements -> 2 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1645, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2205] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1647, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2206] Ref(ModelKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1647, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2207] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1647, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2208] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1649, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2209] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1649, child_count: 3, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2210] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1652, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2211] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1652, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2212] StringParser("MAINTAINED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1652, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2213] StringParser("MANAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1652, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2214] StringParser("MAP")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1652, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2215] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1652, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2216] StringParser("MASKING")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1652, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2217] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1652, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2218] StringParser("MATCH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1652, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2219] StringParser("MATCHED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1652, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2220] StringParser("MATERIALIZED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1652, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2221] StringParser("MAX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1652, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2222] StringParser("MAX_ROWS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1652, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2223] StringParser("MAXEXTENTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1652, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2224] StringParser("MAXVALUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1652, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2225] StringParser("MDC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1652, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2226] StringParser("MEDIUMBLOB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1652, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2227] StringParser("MEDIUMINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1652, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2228] StringParser("MEDIUMTEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1652, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2229] StringParser("MEMBER")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1652, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2230] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1652, child_count: 5, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2231] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1657, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2232] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1657, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2233] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1657, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2234] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1657, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2235] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1659, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2236] Ref(MergeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1659, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2237] Ref(IntoKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1659, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2238] StringParser("MERGE")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1659, child_count: 2, min_times: 1, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2239] AnyNumberOf(2 elements, min=1)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1661, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2240] Ref(MergeMatchedClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1661, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2241] Ref(MergeNotMatchedClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1661, child_count: 7, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2242] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1668, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2243] Ref(WhenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1668, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2244] Ref(MatchedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1668, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2245] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1670, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2246] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1670, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2247] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1670, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2248] Ref(ThenKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1670, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2249] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1672, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2250] Ref(MergeUpdateClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1672, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2251] Ref(MergeDeleteClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1672, child_count: 8, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2252] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1680, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2253] Ref(WhenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1680, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2254] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1680, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2255] Ref(MatchedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1680, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2256] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1682, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2257] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1682, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2258] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1682, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2259] Ref(ThenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1682, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2260] Ref(MergeInsertClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1682, child_count: 12, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2261] Sequence(12 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1694, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2262] Ref(MergeIntoLiteralGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1694, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2263] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1696, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2264] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1696, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2265] Ref(AliasedTableReferenceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1696, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2266] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1696, child_count: 4, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2267] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1700, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2268] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1700, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2269] Ref(AliasedTableReferenceGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1700, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2270] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1702, child_count: 3, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2271] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1705, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2272] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1705, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2273] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1705, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2274] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1707, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2275] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1707, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2276] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 1707, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2277] Meta(conditional: indented_using_on=True -> indent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1707, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2278] Ref(JoinOnConditionSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 1707, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2279] Meta(conditional: indented_using_on=True -> dedent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1707, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2280] Ref(MergeMatchSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1707, child_count: 4, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2281] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1711, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2282] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1711, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2283] Ref(SetClauseListSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1711, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2284] StringParser("MESSAGE_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1711, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2285] StringParser("MESSAGE_OCTET_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1711, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2286] StringParser("MESSAGE_TEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1711, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2287] StringParser("METHOD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1711, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2288] StringParser("MICROSECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1711, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2289] StringParser("MICROSECONDS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1711, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2290] StringParser("MIDDLEINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1711, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2291] StringParser("MILLISECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1711, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2292] StringParser("MIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1711, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2293] StringParser("MIN_ROWS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1711, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2294] StringParser("MINPCTUSED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1711, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2295] StringParser("MINUS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1711, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2296] StringParser("-")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1711, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2297] StringParser("MINUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1711, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2298] StringParser("MINUTE_MICROSECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1711, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2299] StringParser("MINUTE_SECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1711, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2300] StringParser("MINUTES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1711, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2301] StringParser("MINVALUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1711, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2302] StringParser("ML")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1711, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2303] StringParser("MLSLABEL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1711, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2304] StringParser("MOD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1711, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2305] StringParser("MODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1711, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2306] StringParser("MODEL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1711, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2307] StringParser("MODIFIES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1711, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2308] StringParser("MODIFY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1711, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2309] StringParser("MODULE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1711, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2310] StringParser("%")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1711, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2311] StringParser("MONITOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1711, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2312] StringParser("MONTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1711, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2313] StringParser("MONTHNAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1711, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2314] StringParser("MONTHS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1711, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2315] StringParser("MORE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1711, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2316] StringParser("MOVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1711, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2317] StringParser("MOVEMENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1711, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2318] StringParser("*")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1711, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2319] StringParser("MULTISET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1711, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2320] StringParser("MUMPS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1711, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2321] StringParser("MYISAM")
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1711, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2322] RegexParser
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1711, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2323] StringParser("NAME")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1711, child_count: 3, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2324] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1714, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2325] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1714, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2326] Ref(RightArrowSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1714, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2327] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1714, child_count: 3, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2328] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1717, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2329] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1717, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2330] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1717, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2331] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1719, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2332] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 1719, child_count: 3, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2333] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1722, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2334] Ref(WindowSpecificationSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1722, child_count: 4, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2335] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1726, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2336] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1726, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2337] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1728, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2338] Ref(NamedWindowExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1728, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2339] StringParser("NAMES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1728, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2340] StringParser("NAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1728, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2341] StringParser("NAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1728, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2342] StringParser("NATIONAL")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1728, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2343] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1728, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2344] StringParser("NCHAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1728, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2345] StringParser("NCLOB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1728, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2346] StringParser("-")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1728, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2347] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1728, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2348] StringParser("NESTING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1728, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2349] StringParser("NEW")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1728, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2350] StringParser("NEW_TABLE")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1728, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2351] Token("newline")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1728, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2352] StringParser("NEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1728, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2353] StringParser("NEXTVAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1728, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2354] StringParser("NO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1728, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2355] StringParser("NO_WRITE_TO_BINLOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1728, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2356] StringParser("NOAUDIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1728, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2357] StringParser("NOCACHE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1728, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2358] StringParser("NOCHECK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1728, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2359] StringParser("NOCOMPRESS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1728, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2360] StringParser("NOCREATEDB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1728, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2361] StringParser("NOCREATEROLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1728, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2362] StringParser("NOCREATEUSER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1728, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2363] StringParser("NOCYCLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1728, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2364] StringParser("NODENAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1728, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2365] StringParser("NODENUMBER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1728, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2366] StringParser("NOINHERIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1728, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2367] StringParser("NOLOGIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1728, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2368] StringParser("NOMAXVALUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1728, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2369] StringParser("NOMINVALUE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1728, child_count: 6, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2370] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1734, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2371] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1734, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2372] Ref(UnorderedSelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1734, child_count: 3, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2373] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1737, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2374] Ref(SelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1737, child_count: 3, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2375] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1740, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2376] Ref(WithCompoundStatementSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1740, child_count: 3, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2377] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1743, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2378] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1743, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2379] Ref(BracketedSetExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1743, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2380] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1743, child_count: 4, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2381] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1747, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2382] Ref(UpdateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1747, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2383] Ref(InsertStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1747, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2384] Ref(DeleteStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1747, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2385] Ref(MergeStatementSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1747, child_count: 3, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2386] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1750, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2387] Ref(SetExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1750, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2388] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1752, child_count: 3, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2389] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1755, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2390] Ref(SelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1755, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2391] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1755, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2392] StringParser("NONCLUSTERED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1755, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2393] StringParser("NONE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1755, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2394] StringParser("NOORDER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1755, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2395] StringParser("NORMALIZE")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1755, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2396] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1755, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2397] StringParser("NORMALIZED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1755, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2398] StringParser("NOSUPERUSER")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1755, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2399] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1755, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2400] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1757, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2401] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1759, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2402] Ref(RawNotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1759, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2403] Ref(RawEqualsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1759, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2404] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1761, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2405] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1761, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2406] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1761, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2407] StringParser("NOT")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1761, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2408] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1761, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2409] StringParser("NOT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1761, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2410] StringParser("NOTHING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1761, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2411] StringParser("NOTIFY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1761, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2412] StringParser("NOTNULL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1761, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2413] StringParser("NOWAIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1761, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2414] StringParser("NULL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1761, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2415] StringParser("NULL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1761, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2416] StringParser("NULLABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1761, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2417] StringParser("NULLIF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1761, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2418] StringParser("NULLS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1761, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2419] StringParser("NUMBER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1761, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2420] StringParser("NUMERIC")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1761, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2421] TypedParser("numeric_literal")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1761, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2422] StringParser("NUMPARTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1761, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2423] StringParser("OBID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1761, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2424] StringParser("OBJECT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1761, child_count: 3, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2425] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1764, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2426] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1764, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2427] Ref(ColonSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1764, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2428] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1764, child_count: 3, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2429] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1767, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2430] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1769, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2431] Ref(ObjectLiteralElementSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1769, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2432] StringParser("{")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1769, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2433] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1771, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2434] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1771, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2435] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1773, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2436] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1773, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2437] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1773, child_count: 12, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2438] OneOf(12 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1785, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2439] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1785, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2440] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1785, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2441] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1785, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2442] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1785, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2443] Ref(CastOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1785, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2444] Ref(StartSquareBracketSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1785, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2445] Ref(StartBracketSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1785, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2446] Ref(BinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1785, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2447] Ref(ColonSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1785, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2448] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1785, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2449] Ref(JoinLikeClauseGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1785, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2450] StringParser("OBJECTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1785, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2451] StringParser("OCTET_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1785, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2452] StringParser("OCTETS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1785, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2453] StringParser("OF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1785, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2454] StringParser("OFF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1785, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2455] StringParser("OFFLINE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1785, child_count: 3, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2456] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1788, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2457] Ref(OffsetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1788, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2458] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1790, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2459] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1790, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2460] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1790, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2461] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1792, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2462] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1792, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2463] Ref(RowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1792, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2464] StringParser("OFFSET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1792, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2465] StringParser("OFFSETS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1792, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2466] StringParser("OIDS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1792, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2467] StringParser("OLD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1792, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2468] StringParser("OLD_TABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1792, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2469] StringParser("ON")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1792, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2470] StringParser("ONLINE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1792, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2471] StringParser("ONLY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1792, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2472] StringParser("OPEN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1792, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2473] StringParser("OPENDATASOURCE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1792, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2474] StringParser("OPENQUERY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1792, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2475] StringParser("OPENROWSET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1792, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2476] StringParser("OPENXML")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1792, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2477] StringParser("OPERATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1792, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2478] StringParser("OPERATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1792, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2479] StringParser("OPERATOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1792, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2480] StringParser("OPTIMIZATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1792, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2481] StringParser("OPTIMIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1792, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2482] StringParser("OPTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1792, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2483] StringParser("OPTIONALLY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1792, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2484] StringParser("OPTIONS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1792, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2485] StringParser("OR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1792, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2486] StringParser("OR")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1792, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2487] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1794, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2488] Ref(OrKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1794, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2489] Ref(ReplaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1794, child_count: 5, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2490] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1799, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2491] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1799, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2492] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1799, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 2, _padding: 0 }, // [2493] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1801, child_count: 4, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2494] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1805, child_count: 3, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2495] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1808, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2496] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1808, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2497] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1808, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2498] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1808, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2499] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1810, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2500] Ref(AscKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1810, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2501] Ref(DescKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1810, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2502] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1812, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2503] Ref(NullsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1812, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2504] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1814, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2505] Ref(FirstKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1814, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2506] Ref(LastKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1814, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2507] Ref(WithFillSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1814, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2508] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1814, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2509] Ref(FrameClauseUnitGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1814, child_count: 12, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2510] OneOf(12 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1826, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2511] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1826, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2512] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1826, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2513] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1826, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2514] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1826, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2515] Ref(FrameClauseUnitGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1826, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2516] Ref(SeparatorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1826, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2517] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1826, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2518] Ref(OffsetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1826, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2519] Ref(ReadOnlyClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1826, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2520] Ref(IsolationClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1826, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2521] Ref(RowMovementClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1826, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2522] Ref(ViewCheckOptionClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1826, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2523] StringParser("ORDER")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1826, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2524] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1828, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2525] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1828, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2526] Ref(NoorderKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1828, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2527] StringParser("ORDERING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1828, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2528] StringParser("ORDINALITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1828, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2529] StringParser("ORGANIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1828, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2530] StringParser("OTHERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1828, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2531] StringParser("OUT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1828, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2532] StringParser("OUTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1828, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2533] StringParser("OUTFILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1828, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2534] StringParser("OUTPUT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1828, child_count: 5, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2535] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1833, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2536] Ref(IgnoreRespectNullsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1833, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2537] Ref(OverKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1833, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2538] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1835, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2539] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 1835, child_count: 3, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2540] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1838, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2541] Ref(WindowSpecificationSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1838, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2542] StringParser("OVER")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1838, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2543] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1840, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2544] Ref(OverlapsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1840, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2545] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1842, child_count: 1, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2546] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1843, child_count: 5, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2547] Bracketed(3 elements -> 3 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1848, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2548] Ref(DateTimeLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1848, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2549] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1848, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2550] Ref(DateTimeLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1848, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2551] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1848, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2552] StringParser("OVERLAPS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1848, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2553] StringParser("OVERLAY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1848, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2554] StringParser("OVERRIDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1848, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2555] StringParser("OVERWRITE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1848, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2556] StringParser("OWNER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1848, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2557] StringParser("OWNERSHIP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1848, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2558] StringParser("PACK_KEYS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1848, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2559] StringParser("PACKAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1848, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2560] StringParser("PAD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1848, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2561] StringParser("PADDED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1848, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2562] StringParser("PAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1848, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2563] StringParser("PAGESIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1848, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2564] StringParser("PARAMETER")
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1848, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2565] RegexParser
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1848, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2566] StringParser("?")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1848, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2567] StringParser("PARAMETER_MODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1848, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2568] StringParser("PARAMETER_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1848, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2569] StringParser("PARAMETER_ORDINAL_POSITION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1848, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2570] StringParser("PARAMETER_SPECIFIC_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1848, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2571] StringParser("PARAMETER_SPECIFIC_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1848, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2572] StringParser("PARAMETER_SPECIFIC_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1848, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2573] StringParser("PARAMETERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1848, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2574] StringParser("PART")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1848, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2575] StringParser("PARTIAL")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1848, child_count: 5, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2576] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1853, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2577] Ref(PartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1853, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2578] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1853, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2579] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1855, child_count: 3, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2580] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1858, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2581] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1860, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2582] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1860, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2583] StringParser("PARTITION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1860, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2584] StringParser("PARTITIONED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1860, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2585] StringParser("PARTITIONING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1860, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2586] StringParser("PARTITIONS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1860, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2587] StringParser("PASCAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1860, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2588] StringParser("PASSWORD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1860, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2589] StringParser("PATH")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1860, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2590] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1862, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2591] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1864, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2592] Ref(SlashSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1864, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2593] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1866, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2594] TypedParser("word")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1866, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2595] Ref(SlashSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1866, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2596] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1866, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2597] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1866, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2598] StringParser("PCTFREE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1866, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2599] StringParser("PERCENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1866, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2600] StringParser("PERCENT_RANK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1866, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2601] StringParser("PERCENTILE_CONT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1866, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2602] StringParser("PERCENTILE_DISC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1866, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2603] StringParser("PIECESIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1866, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2604] StringParser("PIPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1866, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2605] StringParser("|")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1866, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2606] StringParser("PLACING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1866, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2607] StringParser("PLAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1866, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2608] StringParser("PLI")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1866, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2609] StringParser("+")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1866, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2610] StringParser("POLICY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1866, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2611] StringParser("POSITION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1866, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2612] StringParser("+")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1866, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2613] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1868, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2614] Ref(OverClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1868, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2615] Ref(WithinGroupClauseSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1868, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2616] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1868, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2617] StringParser("POSTFIX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1868, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2618] StringParser("POWER")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1868, child_count: 1, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2619] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1869, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2620] Ref(LateralKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1869, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2621] StringParser("PRECEDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1869, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2622] StringParser("PRECISION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1869, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2623] StringParser("PREFIX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1869, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2624] StringParser("PREORDER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1869, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2625] StringParser("PREPARE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1869, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2626] StringParser("PREPARED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1869, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2627] StringParser("PRESERVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1869, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2628] StringParser("PREVVAL")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1869, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2629] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1871, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2630] Ref(PrimaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1871, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2631] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1871, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2632] StringParser("PRIMARY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1871, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2633] StringParser("PRINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1871, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2634] StringParser("PRIOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1871, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2635] StringParser("PRIQTY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1871, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2636] StringParser("PRIVILEGES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1871, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2637] StringParser("PROC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1871, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2638] StringParser("PROCEDURAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1871, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2639] StringParser("PROCEDURE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1871, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2640] StringParser("PROCEDURES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1871, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2641] StringParser("PROCESS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1871, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2642] StringParser("PROCESSLIST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1871, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2643] StringParser("PROFILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1871, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2644] StringParser("PROGRAM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1871, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2645] StringParser("PSID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1871, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2646] StringParser("PUBLIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1871, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2647] StringParser("PURGE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1871, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2648] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1873, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2649] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1873, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2650] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1873, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2651] StringParser("QUALIFY")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1873, child_count: 3, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2652] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1876, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2653] Ref(ComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1876, child_count: 3, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2654] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1879, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2655] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1879, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2656] Ref(AnyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1879, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2657] Ref(SomeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 1879, child_count: 3, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2658] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1882, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2659] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1882, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2660] StringParser("QUARTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1882, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2661] StringParser("QUERY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1882, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2662] StringParser("QUERYNO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1882, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2663] StringParser("QUOTE")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1882, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2664] TypedParser("double_quote")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1882, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2665] TypedParser("single_quote")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1882, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2666] StringParser("RAID0")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1882, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2667] StringParser("RAISERROR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1882, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2668] StringParser("RANDOM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1882, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2669] StringParser("RANGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1882, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2670] StringParser("RANK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1882, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2671] StringParser("=")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1882, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2672] StringParser(">")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1882, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2673] StringParser("RAW")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1882, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2674] StringParser("<")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1882, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2675] StringParser("!")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1882, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2676] StringParser("READ")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1882, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2677] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1884, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2678] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1884, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2679] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1886, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2680] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1888, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2681] Ref(ReadKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1888, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2682] Ref(OnlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1888, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2683] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1890, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2684] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1890, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2685] Ref(OnlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1890, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2686] StringParser("READS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1890, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2687] StringParser("READTEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1890, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2688] StringParser("REAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1890, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2689] StringParser("RECHECK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1890, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2690] StringParser("RECONFIGURE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1890, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2691] StringParser("RECOVERY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1890, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2692] StringParser("RECURSIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1890, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2693] StringParser("REF")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1890, child_count: 5, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2694] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1895, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2695] Ref(ReferencesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1895, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2696] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1895, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2697] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1895, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2698] Ref(ReferenceMatchGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1895, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2699] AnySetOf(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1897, child_count: 3, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2700] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1900, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2701] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1900, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2702] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1900, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2703] Ref(ReferentialActionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1900, child_count: 3, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2704] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1903, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2705] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1903, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2706] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1903, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2707] Ref(ReferentialActionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1903, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2708] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1905, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2709] Ref(MatchKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1905, child_count: 3, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2710] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1908, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2711] Ref(FullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1908, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2712] Ref(PartialKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1908, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2713] Ref(SimpleKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1908, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2714] StringParser("REFERENCE_USAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1908, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2715] StringParser("REFERENCES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1908, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2716] StringParser("REFERENCING")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1908, child_count: 5, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2717] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1913, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2718] Ref(RestrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1913, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2719] Ref(CascadeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1913, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2720] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1915, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2721] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1915, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2722] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1915, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2723] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1917, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2724] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1917, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2725] Ref(ActionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1917, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2726] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1919, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2727] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1919, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2728] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1919, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2729] StringParser("REFRESH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1919, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2730] StringParser("REGEXP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1919, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2731] StringParser("REGR_AVGX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1919, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2732] StringParser("REGR_AVGY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1919, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2733] StringParser("REGR_COUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1919, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2734] StringParser("REGR_INTERCEPT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1919, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2735] StringParser("REGR_R2")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1919, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2736] StringParser("REGR_SLOPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1919, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2737] StringParser("REGR_SXX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1919, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2738] StringParser("REGR_SXY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1919, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2739] StringParser("REGR_SYY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1919, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2740] StringParser("REINDEX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1919, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2741] StringParser("REJECT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1919, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2742] StringParser("RELATIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1919, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2743] StringParser("RELEASE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1919, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2744] StringParser("RELOAD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1919, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2745] StringParser("RENAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1919, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2746] StringParser("REPEAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1919, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2747] StringParser("REPEATABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1919, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2748] StringParser("REPLACE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1919, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2749] StringParser("REPLICATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1919, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2750] StringParser("REQUIRE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1919, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2751] StringParser("RESET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1919, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2752] StringParser("RESIGNAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1919, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2753] StringParser("RESOURCE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1919, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2754] StringParser("RESPECT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1919, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2755] StringParser("RESTART")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1919, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2756] StringParser("RESTORE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1919, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2757] StringParser("RESTRICT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1919, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2758] StringParser("RESULT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1919, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2759] StringParser("RESULT_SET_LOCATOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1919, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2760] StringParser("RETURN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1919, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2761] StringParser("RETURNED_CARDINALITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1919, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2762] StringParser("RETURNED_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1919, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2763] StringParser("RETURNED_OCTET_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1919, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2764] StringParser("RETURNED_SQLSTATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1919, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2765] StringParser("RETURNS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1919, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2766] StringParser("REVERSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1919, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2767] StringParser("REVOKE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1919, child_count: 6, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2768] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1925, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2769] Ref(RevokeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1925, child_count: 3, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2770] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1928, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2771] Ref(GrantKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1928, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2772] Ref(OptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1928, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2773] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1928, child_count: 4, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2774] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1932, child_count: 3, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2775] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1935, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2776] Ref(AccessPermissionsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1935, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2777] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1935, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2778] Ref(AccessObjectSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1935, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2779] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1937, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2780] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1937, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2781] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1937, child_count: 4, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2782] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1941, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2783] Ref(OwnershipKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1941, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2784] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1941, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2785] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1941, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2786] Ref(UserReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1941, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2787] Ref(UserReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1941, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2788] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1941, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2789] Ref(AccessTargetSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1941, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2790] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1941, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2791] StringParser("=>")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1941, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2792] StringParser("RIGHT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1941, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2793] StringParser("RLIKE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1941, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2794] StringParser("ROLE")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1941, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2795] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1941, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2796] StringParser("ROLES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1941, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2797] StringParser("ROLLBACK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1941, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2798] StringParser("ROLLOUT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1941, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2799] StringParser("ROLLUP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1941, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2800] StringParser("ROLLUP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1941, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2801] StringParser("ROUND_CEILING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1941, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2802] StringParser("ROUND_DOWN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1941, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2803] StringParser("ROUND_FLOOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1941, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2804] StringParser("ROUND_HALF_DOWN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1941, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2805] StringParser("ROUND_HALF_EVEN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1941, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2806] StringParser("ROUND_HALF_UP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1941, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2807] StringParser("ROUND_UP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1941, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2808] StringParser("ROUNDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1941, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2809] StringParser("ROUTINE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1941, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2810] StringParser("ROUTINE_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1941, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2811] StringParser("ROUTINE_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1941, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2812] StringParser("ROUTINE_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1941, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2813] StringParser("ROUTINES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1941, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2814] StringParser("ROW")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1941, child_count: 4, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2815] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1945, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2816] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1945, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2817] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1945, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2818] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1945, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2819] Ref(MovementKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1945, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2820] StringParser("ROW_COUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1945, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2821] StringParser("ROW_NUMBER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1945, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2822] StringParser("ROWCOUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1945, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2823] StringParser("ROWGUIDCOL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1945, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2824] StringParser("ROWID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1945, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2825] StringParser("ROWNUM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1945, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2826] StringParser("ROWNUMBER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1945, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2827] StringParser("ROWS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1945, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2828] StringParser("ROWSET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1945, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2829] StringParser("RR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1945, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2830] StringParser("RRN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1945, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2831] StringParser("RS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1945, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2832] StringParser("RULE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1945, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2833] StringParser("RUN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1945, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2834] StringParser("SAMPLED")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1945, child_count: 4, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2835] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1949, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2836] Ref(TablesampleKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1949, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2837] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1951, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2838] Ref(BernoulliKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1951, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2839] Ref(SystemKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1951, child_count: 3, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2840] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1954, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2841] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1954, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2842] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1956, child_count: 1, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2843] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1957, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2844] Ref(RepeatableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1957, child_count: 3, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2845] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1960, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2846] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1960, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2847] StringParser("SAVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1960, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2848] StringParser("SAVEPOINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1960, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2849] StringParser("SCALE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1960, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2850] StringParser("SCANS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1960, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2851] StringParser("SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1960, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2852] StringParser("SCHEMA_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1960, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2853] StringParser("SCHEMAS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1960, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2854] StringParser("SCOPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1960, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2855] StringParser("SCOPE_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1960, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2856] StringParser("SCOPE_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1960, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2857] StringParser("SCOPE_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1960, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2858] StringParser("SCRATCHPAD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1960, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2859] StringParser("SCROLL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1960, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2860] StringParser("SEARCH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1960, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2861] StringParser("SECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1960, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2862] StringParser("SECOND_MICROSECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1960, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2863] StringParser("SECONDS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1960, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2864] StringParser("SECQTY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1960, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2865] StringParser("SECTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1960, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2866] StringParser("SECURITY")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1960, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2867] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1962, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2868] Ref(WildcardExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1962, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2869] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1964, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2870] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1964, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2871] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1964, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2872] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1966, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2873] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1966, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2874] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::GreedyOnceStarted, flags: GrammarFlags::from_bits(4), first_child_idx: 1966, child_count: 5, min_times: 0, first_terminator_idx: 30, terminator_count: 1, _padding: 0 }, // [2875] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1971, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2876] Ref(SelectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1971, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2877] Ref(SelectClauseModifierSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(12), first_child_idx: 1971, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2878] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1973, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2879] Ref(SelectClauseElementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1973, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2880] Ref(SelectClauseTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1973, child_count: 11, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2881] OneOf(11 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 1984, child_count: 1, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2882] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::PrecededBy, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1985, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2883] PrecededBy(preceding_sequences=(('IS', 'DISTINCT'), ('IS', 'NOT', 'DISTINCT')))
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1985, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2884] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1985, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2885] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1987, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2886] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1987, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2887] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1987, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2888] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1987, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2889] Ref(OverlapsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1987, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2890] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1987, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2891] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1987, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2892] Ref(ReadOnlyClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1987, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2893] Ref(IsolationClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1987, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2894] Ref(RowMovementClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1987, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2895] Ref(ViewCheckOptionClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1987, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2896] StringParser("SELECT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::GreedyOnceStarted, flags: GrammarFlags::from_bits(4), first_child_idx: 1987, child_count: 14, min_times: 0, first_terminator_idx: 31, terminator_count: 5, _padding: 0 }, // [2897] Sequence(14 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2001, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2898] Ref(SelectClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2001, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2899] Ref(FromClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2001, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2900] Ref(WhereClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2001, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2901] Ref(GroupByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2001, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2902] Ref(HavingClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2001, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2903] Ref(OverlapsClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2001, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2904] Ref(NamedWindowSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2001, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2905] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2001, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2906] Ref(OffsetClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2001, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2907] Ref(FetchClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2001, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2908] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2001, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2909] Ref(NamedWindowSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2001, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2910] Ref(ReadOnlyClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2001, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2911] Ref(IsolationClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2001, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2912] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2001, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2913] Ref(WithNoSchemaBindingClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2001, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2914] Ref(WithDataClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2001, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2915] Ref(RowMovementClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2001, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2916] Ref(ViewCheckOptionClauseSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2001, child_count: 4, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2917] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2005, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2918] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2007, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2919] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2010, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2920] Ref(WithCompoundStatementSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2010, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2921] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2012, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2922] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2015, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2923] Ref(WithCompoundNonSelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2015, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2924] Ref(NonWithSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2015, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2925] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2018, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2926] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2018, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2927] StringParser("SELF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2018, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2928] StringParser(";")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2018, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2929] StringParser("SENSITIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2018, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2930] StringParser("SEPARATOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2018, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2931] StringParser("SEQUENCE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2018, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2932] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2020, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2933] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2022, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2934] Ref(MaxvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2022, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2935] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2022, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2936] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2024, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2937] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2024, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2938] Ref(MaxvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2024, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2939] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2026, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2940] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2028, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2941] Ref(MinvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2028, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2942] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2028, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2943] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2030, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2944] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2030, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2945] Ref(MinvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2030, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2946] StringParser("SEQUENCES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2030, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2947] StringParser("SERIALIZABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2030, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2948] StringParser("SERVER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2030, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2949] StringParser("SERVER_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2030, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2950] StringParser("SESSION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2030, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2951] StringParser("SESSION_USER")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2030, child_count: 4, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2952] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2034, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2953] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2034, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2954] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2036, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2955] Ref(SetClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2036, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2956] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2039, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2957] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2039, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2958] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2039, child_count: 7, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2959] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2046, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2960] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2046, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2961] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2046, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2962] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2046, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2963] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2046, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2964] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2046, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2965] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2046, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2966] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2046, child_count: 7, min_times: 0, first_terminator_idx: 36, terminator_count: 2, _padding: 0 }, // [2967] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2053, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2968] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2053, child_count: 1, min_times: 1, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2969] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2054, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2970] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2056, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2971] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2056, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2972] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2056, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2973] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2056, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2974] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2056, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2975] Ref(NamedWindowSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2056, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2976] Ref(ReadOnlyClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2056, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2977] Ref(IsolationClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2056, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2978] Ref(RowMovementClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2056, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2979] Ref(ViewCheckOptionClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2056, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2980] StringParser("SET")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2056, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2981] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2059, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2982] Ref(UnionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2059, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2983] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2061, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2984] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2063, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2985] Ref(IntersectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2063, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2986] Ref(ExceptKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2063, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2987] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2063, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2988] Ref(MinusKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2063, child_count: 4, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2989] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2067, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2990] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2067, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2991] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2067, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2992] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2067, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2993] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2067, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2994] StringParser("SETOF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2067, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2995] StringParser("SETS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2067, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2996] StringParser("SETUSER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2067, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2997] StringParser("SHARE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2067, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2998] StringParser("SHARES")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2067, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [2999] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2069, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3000] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2071, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3001] Ref(Expression_D_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2071, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3002] Ref(CaseExpressionSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2071, child_count: 1, min_times: 1, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3003] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2072, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3004] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2075, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3005] Ref(CastOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2075, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3006] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2075, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3007] Ref(TimeZoneGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2075, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3008] StringParser("SHOW")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2075, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3009] StringParser("SHUTDOWN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2075, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3010] StringParser("SIGNAL")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2075, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3011] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2077, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3012] Ref(PositiveSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2077, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3013] Ref(NegativeSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2077, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3014] StringParser("SIMILAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2077, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3015] StringParser("SIMPLE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2077, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 1, _padding: 0 }, // [3016] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2079, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3017] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2079, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3018] Ref(QuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2079, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3019] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2079, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3020] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2081, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3021] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2081, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3022] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2084, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3023] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2084, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3024] Ref(ObjectReferenceDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2084, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3025] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2084, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3026] TypedParser("single_quote")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2084, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3027] StringParser("SIZE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2084, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3028] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2086, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3029] Ref(ArrayTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2086, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3030] Ref(ArrayAccessorSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2086, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3031] StringParser("/")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2086, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3032] StringParser(":")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2086, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3033] StringParser("SMALLINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2086, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3034] StringParser("SNAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2086, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3035] StringParser("SNAPSHOT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2086, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3036] StringParser("SOME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2086, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3037] StringParser("SONAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2086, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3038] StringParser("SOURCE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2086, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3039] StringParser("SPACE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2086, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3040] StringParser("SPATIAL")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2086, child_count: 12, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3041] OneOf(12 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2098, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3042] Ref(Current_dateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2098, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3043] Ref(Current_pathKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2098, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3044] Ref(Current_schemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2098, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3045] Ref(Current_serverKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2098, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3046] Ref(Current_timeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2098, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3047] Ref(Current_timestampKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2098, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3048] Ref(Current_timezoneKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2098, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3049] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2098, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3050] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2098, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3051] Ref(System_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2098, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3052] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2098, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3053] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2100, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3054] Ref(CurrentKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2100, child_count: 31, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3055] OneOf(31 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2131, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3056] Ref(Client_acctngKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2131, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3057] Ref(Client_applnameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2131, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3058] Ref(Client_useridKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2131, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3059] Ref(Client_wrkstnnameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2131, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3060] Ref(DateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2131, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3061] Ref(DbpartitionnumKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2131, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3062] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2134, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3063] Ref(DecfloatKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2134, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3064] Ref(RoundingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2134, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3065] Ref(ModeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2134, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3066] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2137, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3067] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2137, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3068] Ref(TransformKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2137, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3069] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2137, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3070] Ref(DegreeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2137, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3071] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2139, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3072] Ref(ExplainKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2139, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3073] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2141, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3074] Ref(ModeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2141, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3075] Ref(SnapshotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2141, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3076] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2143, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3077] Ref(FederatedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2143, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3078] Ref(AsynchronyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2143, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3079] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2146, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3080] Ref(ImplicitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2146, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3081] Ref(XmlparseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2146, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3082] Ref(OptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2146, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3083] Ref(IsolationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2146, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3084] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2148, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3085] Ref(LocaleKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2148, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3086] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2150, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3087] Ref(Lc_messagesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2150, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3088] Ref(Lc_timeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2150, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3089] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2152, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3090] Ref(LockKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2152, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3091] Ref(TimeoutKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2152, child_count: 5, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3092] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2157, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3093] Ref(MaintainedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2157, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3094] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2157, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3095] Ref(TypesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2157, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3096] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2157, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3097] Ref(OptimizationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2157, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3098] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2160, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3099] Ref(MdcKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2160, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3100] Ref(RolloutKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2160, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3101] Ref(ModeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2160, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3102] Ref(MemberKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2160, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3103] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2162, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3104] Ref(OptimizationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2162, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3105] Ref(ProfileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2162, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3106] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2164, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3107] Ref(PackageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2164, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3108] Ref(PathKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2164, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3109] Ref(PathKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2164, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3110] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2166, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3111] Ref(QueryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2166, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3112] Ref(OptimizationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2166, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3113] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2168, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3114] Ref(RefreshKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2168, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3115] Ref(AgeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2168, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3116] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2168, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3117] Ref(ServerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2168, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3118] Ref(Sql_ccflagsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2168, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3119] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2170, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3120] Ref(TemporalKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2170, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3121] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2172, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3122] Ref(Business_timeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2172, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3123] Ref(System_timeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2172, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3124] Ref(TimeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2172, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3125] Ref(TimestampKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2172, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3126] Ref(TimezoneKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2172, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3127] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2172, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3128] StringParser("SPECIFIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2172, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3129] StringParser("SPECIFIC_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2172, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3130] StringParser("SPECIFICATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2172, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3131] StringParser("SPECIFICTYPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2172, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3132] StringParser("SPLIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2172, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3133] StringParser("SQL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2172, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3134] StringParser("SQL_BIG_RESULT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2172, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3135] StringParser("SQL_BIG_SELECTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2172, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3136] StringParser("SQL_BIG_TABLES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2172, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3137] StringParser("SQL_CALC_FOUND_ROWS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2172, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3138] StringParser("SQL_CCFLAGS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2172, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3139] StringParser("SQL_LOG_OFF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2172, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3140] StringParser("SQL_LOG_UPDATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2172, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3141] StringParser("SQL_LOW_PRIORITY_UPDATES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2172, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3142] StringParser("SQL_SELECT_LIMIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2172, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3143] StringParser("SQL_SMALL_RESULT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2172, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3144] StringParser("SQL_WARNINGS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2172, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3145] StringParser("SQLCA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2172, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3146] StringParser("SQLCODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2172, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3147] StringParser("SQLERROR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2172, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3148] StringParser("SQLEXCEPTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2172, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3149] StringParser("SQLID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2172, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3150] StringParser("SQLSTATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2172, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3151] StringParser("SQLWARNING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2172, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3152] StringParser("SQRT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2172, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3153] StringParser("SSL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2172, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3154] StringParser("STABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2172, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3155] StringParser("STACKED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2172, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3156] StringParser("STAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2172, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3157] StringParser("STAGES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2172, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3158] StringParser("STANDARD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2172, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3159] StringParser("*")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2172, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3160] StringParser("START")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2172, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3161] StringParser("STARTING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2172, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3162] StringParser("STARTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2172, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3163] StringParser("STATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2172, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3164] StringParser("STATEMENT")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2172, child_count: 42, min_times: 0, first_terminator_idx: 39, terminator_count: 1, _padding: 0 }, // [3165] OneOf(42 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2214, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3166] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2214, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3167] Ref(MergeStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2214, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3168] Ref(InsertStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2214, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3169] Ref(TransactionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2214, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3170] Ref(DropTableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2214, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3171] Ref(DropViewStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2214, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3172] Ref(CreateUserStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2214, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3173] Ref(DropUserStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2214, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3174] Ref(TruncateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2214, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3175] Ref(AccessStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2214, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3176] Ref(CreateTableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2214, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3177] Ref(CreateRoleStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2214, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3178] Ref(DropRoleStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2214, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3179] Ref(AlterTableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2214, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3180] Ref(CreateSchemaStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2214, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3181] Ref(SetSchemaStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2214, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3182] Ref(DropSchemaStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2214, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3183] Ref(DropTypeStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2214, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3184] Ref(CreateDatabaseStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2214, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3185] Ref(DropDatabaseStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2214, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3186] Ref(CreateIndexStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2214, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3187] Ref(DropIndexStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2214, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3188] Ref(CreateViewStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2214, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3189] Ref(DeleteStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2214, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3190] Ref(UpdateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2214, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3191] Ref(CreateCastStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2214, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3192] Ref(DropCastStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2214, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3193] Ref(CreateFunctionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2214, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3194] Ref(DropFunctionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2214, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3195] Ref(CreateModelStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2214, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3196] Ref(DropModelStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2214, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3197] Ref(DescribeStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2214, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3198] Ref(UseStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2214, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3199] Ref(ExplainStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2214, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3200] Ref(CreateSequenceStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2214, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3201] Ref(AlterSequenceStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2214, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3202] Ref(DropSequenceStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2214, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3203] Ref(CreateTriggerStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2214, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3204] Ref(DropTriggerStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2214, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3205] Ref(CallStoredProcedureSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2214, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3206] Ref(CommentOnStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2214, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3207] Ref(DeclareGlobalTempTableSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2214, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [3208] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2214, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3209] StringParser("STATIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2214, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3210] StringParser("STATISTICS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2214, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3211] StringParser("STATMENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2214, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3212] StringParser("STAY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2214, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3213] StringParser("STDDEV_POP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2214, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3214] StringParser("STDDEV_SAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2214, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3215] StringParser("STDIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2214, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3216] StringParser("STDOUT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2214, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3217] StringParser("STOGROUP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2214, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3218] StringParser("STORAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2214, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3219] StringParser("STORES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2214, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3220] StringParser("STRAIGHT_JOIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2214, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3221] StringParser("STREAM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2214, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3222] StringParser("STREAMS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2214, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3223] StringParser("STRICT")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2214, child_count: 1, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3224] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2215, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3225] Ref(ConcatSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2215, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3226] StringParser("STRING")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2215, child_count: 3, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3227] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2218, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3228] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2220, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3229] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2222, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3230] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2222, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3231] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2222, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3232] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2222, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3233] StringParser("STRUCTURE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2222, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3234] StringParser("STYLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2222, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3235] StringParser("SUBCLASS_ORIGIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2222, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3236] StringParser("SUBLIST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2222, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3237] StringParser("SUBMULTISET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2222, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3238] StringParser("SUBSTRING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2222, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3239] StringParser("SUCCESSFUL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2222, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3240] StringParser("SUM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2222, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3241] StringParser("SUMMARY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2222, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3242] StringParser("SUPERUSER")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2222, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3243] Token("symbol")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2222, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3244] StringParser("SYMMETRIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2222, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3245] StringParser("SYNONYM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2222, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3246] StringParser("SYSDATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2222, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3247] StringParser("SYSFUN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2222, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3248] StringParser("SYSIBM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2222, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3249] StringParser("SYSID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2222, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3250] StringParser("SYSPROC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2222, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3251] StringParser("SYSTEM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2222, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3252] StringParser("SYSTEM_TIME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2222, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3253] StringParser("SYSTEM_USER")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2222, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3254] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2224, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3255] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2226, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3256] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2226, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3257] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2226, child_count: 3, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3258] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2229, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3259] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2231, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3260] Ref(UniqueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2231, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3261] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2231, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3262] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2233, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3263] Ref(PrimaryKeyGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2233, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3264] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2233, child_count: 3, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3265] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2236, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3266] Ref(ForeignKeyGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2236, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3267] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2236, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3268] Ref(ReferenceDefinitionGrammar)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2236, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3269] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2236, child_count: 6, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3270] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2242, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3271] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2242, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3272] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2242, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3273] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2242, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3274] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2242, child_count: 3, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3275] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2245, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3276] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2245, child_count: 3, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3277] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2248, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3278] Ref(MergeStatementSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2248, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3279] StringParser("TABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2248, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3280] StringParser("TABLE_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2248, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3281] StringParser("TABLES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2248, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3282] StringParser("TABLESAMPLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2248, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3283] StringParser("TABLESPACE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2248, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3284] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2250, child_count: 1, min_times: 0, first_terminator_idx: 40, terminator_count: 1, _padding: 0 }, // [3285] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2251, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3286] Ref(Expression_A_Unary_Operator_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2251, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [3287] Ref(BinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2251, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3288] Ref(Expression_C_Grammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2251, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3289] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2253, child_count: 1, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3290] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2254, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3291] Ref(Expression_B_Unary_Operator_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2254, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3292] Ref(Expression_C_Grammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2254, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3293] StringParser("TASK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2254, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3294] StringParser("TASKS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2254, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3295] StringParser("TEMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2254, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3296] StringParser("TEMPLATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2254, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3297] StringParser("TEMPORAL")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2254, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3298] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2254, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3299] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2256, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3300] Ref(TempKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2256, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3301] Ref(TemporaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2256, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3302] StringParser("TEMPORARY")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2256, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3303] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2258, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3304] Ref(TransientKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2258, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3305] Ref(TemporaryGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2258, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3306] StringParser("TERMINATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2258, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3307] StringParser("TERMINATED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2258, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3308] StringParser("TEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2258, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3309] StringParser("TEXTSIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2258, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3310] StringParser("THAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2258, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3311] StringParser("THEN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2258, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3312] StringParser("TIES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2258, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3313] StringParser("~")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2258, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3314] StringParser("TIME")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2258, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3315] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2261, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3316] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2263, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3317] Ref(TimeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2263, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3318] Ref(TimestampKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2263, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3319] Ref(BracketedArguments)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2263, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3320] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2266, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3321] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2268, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3322] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2268, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3323] Ref(WithoutKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2268, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3324] Ref(TimeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2268, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3325] Ref(ZoneKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2268, child_count: 1, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3326] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2269, child_count: 4, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3327] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2273, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3328] Ref(AtKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2273, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3329] Ref(TimeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2273, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3330] Ref(ZoneKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2273, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3331] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2273, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3332] StringParser("TIMEOUT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2273, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3333] StringParser("TIMESTAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2273, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3334] StringParser("TIMEZONE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2273, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3335] StringParser("TIMEZONE_HOUR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2273, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3336] StringParser("TIMEZONE_MINUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2273, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3337] StringParser("TINYBLOB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2273, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3338] StringParser("TINYINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2273, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3339] StringParser("TINYTEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2273, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3340] StringParser("TO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2273, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3341] StringParser("TOAST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2273, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3342] StringParser("TOP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2273, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3343] StringParser("TOP_LEVEL_COUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2273, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3344] StringParser("TRAILING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2273, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3345] StringParser("TRAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2273, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3346] StringParser("TRANSACTION")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2273, child_count: 4, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3347] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2277, child_count: 5, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3348] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2282, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3349] Ref(StartKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2282, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3350] Ref(BeginKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2282, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3351] Ref(CommitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2282, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3352] Ref(RollbackKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2282, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3353] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2282, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3354] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2284, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3355] Ref(TransactionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2284, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3356] Ref(WorkKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2284, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3357] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2286, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3358] Ref(NameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2286, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3359] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2286, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3360] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2289, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3361] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2289, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3362] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2289, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3363] Ref(ChainKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2289, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3364] StringParser("TRANSACTION_ACTIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2289, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3365] StringParser("TRANSACTIONS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2289, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3366] StringParser("TRANSACTIONS_COMMITTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2289, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3367] StringParser("TRANSACTIONS_ROLLED_BACK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2289, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3368] StringParser("TRANSFORM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2289, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3369] StringParser("TRANSFORMS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2289, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3370] StringParser("TRANSIENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2289, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3371] StringParser("TRANSLATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2289, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3372] StringParser("TRANSLATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2289, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3373] StringParser("TREAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2289, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3374] StringParser("TRIGGER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2289, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3375] StringParser("TRIGGER_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2289, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3376] StringParser("TRIGGER_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2289, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3377] StringParser("TRIGGER_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2289, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3378] StringParser("TRIM")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2289, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3379] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2292, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3380] Ref(BothKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2292, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3381] Ref(LeadingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2292, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3382] Ref(TrailingKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2292, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3383] StringParser("TRUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2292, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3384] StringParser("TRUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2292, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3385] StringParser("TRUNCATE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2292, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3386] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2295, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3387] Ref(TruncateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2295, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3388] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2295, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3389] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2295, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3390] StringParser("TRUSTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2295, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3391] StringParser("TSEQUAL")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2295, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3392] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2298, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3393] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2300, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3394] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2300, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3395] StringParser("TYPE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2300, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3396] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2302, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3397] Ref(ArrayTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2302, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3398] Ref(ArrayLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2302, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3399] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2304, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3400] Ref(StructTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2304, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3401] Ref(StructLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2304, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3402] StringParser("TYPES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2304, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3403] StringParser("UESCAPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2304, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3404] StringParser("UID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2304, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3405] StringParser("UNBOUNDED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2304, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3406] StringParser("UNCOMMITTED")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2304, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3407] Ref(CrossKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2304, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3408] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2307, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3409] Ref(NaturalJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2307, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3410] Ref(UnconditionalCrossJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2307, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3411] Ref(HorizontalJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2307, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3412] StringParser("UNDER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2307, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3413] StringParser("UNDO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2307, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3414] StringParser("UNENCRYPTED")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2307, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3415] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2309, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3416] Ref(UnionKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2309, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3417] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2311, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3418] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2311, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3419] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2311, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3420] StringParser("UNION")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2311, child_count: 1, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3421] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2312, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3422] Ref(UniqueKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2312, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3423] StringParser("UNIQUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2312, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3424] StringParser("UNKNOWN")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2312, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3425] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2312, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3426] StringParser("UNLISTEN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2312, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3427] StringParser("UNLOCK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2312, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3428] StringParser("UNNAMED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2312, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3429] StringParser("UNNEST")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::GreedyOnceStarted, flags: GrammarFlags::from_bits(4), first_child_idx: 2312, child_count: 7, min_times: 0, first_terminator_idx: 41, terminator_count: 9, _padding: 0 }, // [3430] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2319, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3431] Ref(SelectClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2319, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3432] Ref(FromClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2319, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3433] Ref(WhereClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2319, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3434] Ref(GroupByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2319, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3435] Ref(HavingClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2319, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3436] Ref(OverlapsClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2319, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3437] Ref(NamedWindowSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2319, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3438] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2319, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3439] Ref(WithNoSchemaBindingClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2319, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3440] Ref(WithDataClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2319, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3441] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2319, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3442] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2319, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3443] Ref(ReadOnlyClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2319, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3444] Ref(IsolationClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2319, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3445] Ref(RowMovementClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2319, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3446] Ref(ViewCheckOptionClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2319, child_count: 2, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3447] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2321, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3448] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2321, child_count: 1, min_times: 1, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3449] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2322, child_count: 2, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3450] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2324, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3451] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2324, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3452] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2324, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3453] StringParser("UNSAMPLED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2324, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3454] StringParser("UNSIGNED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2324, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3455] StringParser("UNTIL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2324, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3456] StringParser("UPDATE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2324, child_count: 8, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3457] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2332, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3458] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2332, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3459] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(69), first_child_idx: 2332, child_count: 1, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3460] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2333, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3461] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2333, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3462] Ref(SetClauseListSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2333, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3463] Ref(FromClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2333, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3464] Ref(WhereClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2333, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3465] StringParser("UPDATETEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2333, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3466] StringParser("UPPER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2333, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3467] StringParser("UR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2333, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3468] StringParser("USAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2333, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3469] StringParser("USE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2333, child_count: 2, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3470] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2335, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3471] Ref(UseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2335, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3472] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2335, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3473] StringParser("USE_ANY_ROLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2335, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3474] StringParser("USER")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2335, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3475] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2335, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3476] StringParser("USER_DEFINED_TYPE_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2335, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3477] StringParser("USER_DEFINED_TYPE_CODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2335, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3478] StringParser("USER_DEFINED_TYPE_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2335, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3479] StringParser("USER_DEFINED_TYPE_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2335, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3480] StringParser("USERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2335, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3481] StringParser("USING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2335, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3482] StringParser("UTC_DATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2335, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3483] StringParser("UTC_TIME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2335, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3484] StringParser("UTC_TIMESTAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2335, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3485] StringParser("VACUUM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2335, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3486] StringParser("VALID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2335, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3487] StringParser("VALIDATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2335, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3488] StringParser("VALIDATOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2335, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3489] StringParser("VALIDPROC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2335, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3490] StringParser("VALUE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2335, child_count: 4, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3491] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2339, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3492] Ref(ValuesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2339, child_count: 2, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3493] Delimited(3 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2341, child_count: 3, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3494] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 2344, child_count: 3, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3495] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2347, child_count: 2, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3496] Delimited(2 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2349, child_count: 2, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3497] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2351, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3498] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2351, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3499] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2351, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3500] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2351, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3501] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2351, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3502] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2351, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3503] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2351, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3504] StringParser("VALUES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2351, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3505] StringParser("VAR_POP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2351, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3506] StringParser("VAR_SAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2351, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3507] StringParser("VARBINARY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2351, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3508] StringParser("VARCHAR2")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2351, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3509] StringParser("VARCHAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2351, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3510] StringParser("VARCHARACTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2351, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3511] StringParser("VARIABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2351, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3512] StringParser("VARIABLES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2351, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3513] StringParser("VARIANT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2351, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3514] StringParser("VARYING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2351, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3515] StringParser("VCAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2351, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3516] StringParser("VERBOSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2351, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3517] StringParser("VERSION")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2351, child_count: 4, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3518] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2355, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3519] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2355, child_count: 2, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3520] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2357, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3521] Ref(CascadedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2357, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3522] Ref(LocalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2357, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3523] Ref(CheckKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2357, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3524] Ref(OptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2357, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3525] StringParser("VIEW")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2357, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3526] StringParser("VIEWS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2357, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3527] StringParser("VOLATILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2357, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3528] StringParser("VOLUMES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2357, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3529] StringParser("WAITFOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2357, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3530] StringParser("WAREHOUSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2357, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3531] StringParser("WAREHOUSES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2357, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3532] StringParser("WEEK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2357, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3533] StringParser("WEEKDAY")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2357, child_count: 8, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3534] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2365, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3535] Ref(WhenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2365, child_count: 3, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3536] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2368, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3537] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 2368, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3538] Meta(conditional: indented_then=True -> indent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2368, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3539] Ref(ThenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 2368, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3540] Meta(conditional: indented_then_contents=True -> implicit_indent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2368, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3541] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 2368, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3542] Meta(conditional: indented_then_contents=True -> dedent)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 2368, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3543] Meta(conditional: indented_then=True -> dedent)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2368, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3544] StringParser("WHEN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2368, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3545] StringParser("WHENEVER")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2368, child_count: 4, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3546] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2372, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3547] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2372, child_count: 2, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3548] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2374, child_count: 3, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3549] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2377, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3550] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2377, child_count: 13, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3551] OneOf(13 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2390, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3552] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2390, child_count: 2, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3553] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2392, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3554] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2392, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3555] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2392, child_count: 2, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3556] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2394, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3557] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2394, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3558] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2394, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3559] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2394, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3560] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2394, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3561] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2394, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3562] Ref(OverlapsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2394, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3563] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2394, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3564] Ref(OffsetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2394, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3565] Ref(ReadOnlyClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2394, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3566] Ref(IsolationClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2394, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3567] Ref(RowMovementClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2394, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3568] Ref(ViewCheckOptionClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2394, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3569] StringParser("WHERE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2394, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3570] StringParser("WHILE")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2394, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3571] Token("whitespace")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2394, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3572] StringParser("WIDTH_BUCKET")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2394, child_count: 1, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3573] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2395, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3574] Ref(WildcardIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2395, child_count: 2, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3575] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2397, child_count: 1, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3576] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2398, child_count: 2, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3577] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2400, child_count: 2, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3578] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2402, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3579] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2402, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3580] Ref(ObjectReferenceDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2402, child_count: 2, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3581] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2404, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3582] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2404, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3583] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2404, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3584] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2404, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3585] StringParser("WINDOW")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2404, child_count: 4, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3586] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(69), first_child_idx: 2408, child_count: 1, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3587] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2409, child_count: 2, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3588] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2411, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3589] Ref(PartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2411, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3590] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2411, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3591] Ref(PartitionClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2411, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3592] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2411, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3593] Ref(FrameClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2411, child_count: 6, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [3594] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2417, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3595] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2417, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3596] Ref(RecursiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 2417, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3597] Meta(conditional: indented_ctes=True -> indent)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2417, child_count: 2, min_times: 0, first_terminator_idx: 50, terminator_count: 1, _padding: 0 }, // [3598] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2419, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3599] Ref(CTEDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2419, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3600] Ref(SelectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 2419, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [3601] Meta(conditional: indented_ctes=True -> dedent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2419, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [3602] Ref(NonWithNonSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2419, child_count: 6, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3603] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2425, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [3604] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2425, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [3605] Ref(RecursiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 2425, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [3606] Meta(conditional: indented_ctes=True -> indent)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2425, child_count: 2, min_times: 0, first_terminator_idx: 51, terminator_count: 1, _padding: 0 }, // [3607] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2427, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [3608] Ref(CTEDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2427, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [3609] Ref(SelectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 2427, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3610] Meta(conditional: indented_ctes=True -> dedent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2427, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3611] Ref(NonWithSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2427, child_count: 3, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3612] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2430, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3613] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2430, child_count: 1, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3614] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2431, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3615] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2431, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3616] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2431, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3617] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2431, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3618] StringParser("WITH")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2431, child_count: 4, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3619] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2435, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3620] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2435, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3621] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2435, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3622] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2435, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3623] Ref(BindingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2435, child_count: 3, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3624] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2438, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3625] Ref(WithinKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2438, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3626] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 2438, child_count: 3, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3627] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2441, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3628] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2441, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3629] StringParser("WITHIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2441, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3630] StringParser("WITHOUT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2441, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3631] StringParser("WLM")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2441, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3632] Token("word")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2441, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3633] StringParser("WORK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2441, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3634] StringParser("WRAPPER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2441, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3635] StringParser("WRITE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2441, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3636] StringParser("WRITETEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2441, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3637] StringParser("X509")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2441, child_count: 6, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3638] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2447, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3639] Ref(GenerateKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2447, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3640] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2449, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3641] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2449, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3642] Ref(KeysKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2449, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3643] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2449, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3644] Ref(XmlpatternKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2449, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3645] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2449, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3646] Ref(XmlTypeClauseGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2449, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3647] StringParser("XML")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2449, child_count: 4, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3648] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2453, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3649] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2453, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3650] Ref(SqlKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2453, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3651] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2453, child_count: 3, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3652] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2456, child_count: 2, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3653] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2458, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3654] Ref(IgnoreKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2458, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3655] Ref(RejectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2458, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3656] Ref(InvalidKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2458, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3657] Ref(ValuesKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2458, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3658] StringParser("XMLELEMENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2458, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3659] StringParser("XMLEXISTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2458, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3660] StringParser("XMLNAMESPACES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2458, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3661] StringParser("XMLPARSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2458, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3662] StringParser("XMLPATTERN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2458, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3663] StringParser("XOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2458, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3664] StringParser("YAML")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2458, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3665] StringParser("YEAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2458, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3666] StringParser("YEAR_MONTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2458, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3667] StringParser("YEARS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2458, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3668] StringParser("YES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2458, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3669] StringParser("ZEROFILL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2458, child_count: 0, min_times: 0, first_terminator_idx: 52, terminator_count: 0, _padding: 0 }, // [3670] StringParser("ZONE")
];

pub static CHILD_IDS: &[u32] = &[
    5, 6, 61, 7, 51, 8, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
    21, 28, 33, 38, 39, 44, 9, 10, 22, 23, 24, 25, 26, 27, 29, 30,
    31, 32, 34, 35, 36, 37, 40, 41, 42, 43, 45, 46, 47, 48, 49, 50,
    52, 58, 53, 54, 57, 55, 56, 62, 63, 64, 66, 74, 78, 81, 84, 89,
    67, 68, 69, 70, 71, 72, 73, 75, 76, 77, 79, 80, 82, 83, 85, 86,
    87, 88, 90, 133, 91, 100, 105, 108, 109, 110, 111, 112, 113, 114, 115, 116,
    117, 118, 119, 120, 121, 122, 123, 124, 125, 126, 127, 128, 129, 130, 92, 93,
    94, 95, 98, 99, 96, 97, 101, 102, 103, 104, 106, 107, 131, 132, 135, 58,
    138, 139, 140, 141, 142, 143, 144, 145, 146, 147, 148, 151, 154, 157, 149, 150,
    152, 153, 155, 156, 158, 159, 161, 162, 163, 164, 165, 166, 167, 168, 169, 170,
    171, 174, 177, 180, 172, 173, 175, 176, 178, 179, 181, 182, 184, 185, 187, 200,
    188, 58, 189, 192, 195, 198, 199, 190, 191, 193, 194, 196, 197, 201, 58, 203,
    216, 217, 218, 226, 219, 225, 220, 221, 222, 223, 224, 229, 230, 237, 241, 242,
    243, 248, 251, 238, 239, 240, 244, 247, 245, 246, 249, 250, 253, 254, 255, 256,
    257, 259, 260, 261, 262, 264, 270, 283, 284, 265, 266, 267, 268, 269, 271, 274,
    275, 276, 272, 273, 277, 282, 278, 281, 279, 280, 285, 286, 289, 287, 288, 291,
    292, 293, 294, 295, 58, 306, 307, 308, 309, 310, 311, 312, 313, 314, 315, 317,
    322, 323, 318, 321, 319, 320, 327, 322, 323, 328, 58, 331, 348, 358, 359, 360,
    361, 362, 363, 364, 365, 366, 378, 379, 380, 381, 389, 390, 393, 394, 399, 400,
    403, 404, 407, 223, 224, 408, 58, 410, 223, 224, 411, 58, 414, 223, 224, 423,
    223, 224, 425, 426, 427, 428, 429, 223, 224, 433, 434, 435, 436, 443, 453, 444,
    445, 446, 450, 226, 452, 447, 454, 455, 445, 456, 460, 226, 462, 457, 508, 509,
    510, 511, 523, 524, 525, 526, 528, 531, 529, 530, 532, 535, 539, 542, 545, 546,
    547, 550, 551, 554, 533, 534, 536, 537, 538, 223, 224, 540, 541, 543, 544, 548,
    549, 552, 553, 556, 557, 558, 560, 559, 223, 224, 561, 565, 566, 578, 579, 582,
    583, 584, 585, 613, 586, 611, 612, 587, 590, 593, 596, 588, 589, 591, 592, 594,
    595, 597, 610, 598, 599, 600, 601, 602, 603, 604, 605, 606, 607, 608, 609, 614,
    615, 616, 223, 224, 617, 58, 618, 619, 620, 625, 626, 627, 628, 629, 630, 631,
    632, 641, 642, 647, 648, 649, 666, 672, 667, 670, 671, 668, 669, 673, 676, 677,
    674, 675, 678, 679, 687, 688, 689, 693, 694, 695, 705, 706, 707, 710, 690, 691,
    692, 223, 224, 696, 697, 698, 699, 700, 704, 701, 702, 703, 708, 709, 711, 712,
    714, 715, 716, 717, 719, 720, 721, 722, 723, 724, 725, 726, 729, 727, 728, 731,
    732, 733, 734, 735, 736, 737, 746, 738, 739, 223, 224, 740, 58, 741, 742, 743,
    744, 745, 747, 750, 753, 756, 763, 776, 779, 783, 786, 792, 799, 807, 812, 748,
    749, 751, 752, 754, 755, 757, 758, 759, 223, 224, 760, 58, 761, 762, 764, 765,
    766, 767, 768, 769, 770, 775, 771, 772, 223, 224, 773, 223, 224, 774, 58, 771,
    772, 777, 778, 780, 781, 782, 784, 785, 787, 790, 791, 788, 789, 793, 794, 795,
    796, 797, 798, 800, 801, 806, 802, 805, 803, 804, 808, 809, 810, 811, 813, 816,
    817, 814, 815, 820, 821, 822, 823, 824, 825, 837, 838, 826, 827, 828, 223, 224,
    829, 58, 830, 831, 832, 833, 834, 835, 322, 323, 836, 58, 840, 841, 842, 843,
    845, 846, 847, 848, 850, 854, 858, 859, 860, 865, 868, 851, 852, 853, 855, 856,
    857, 861, 864, 862, 863, 866, 867, 870, 871, 872, 873, 874, 876, 877, 878, 879,
    880, 881, 882, 898, 883, 890, 895, 884, 889, 885, 223, 224, 886, 58, 887, 888,
    891, 892, 893, 894, 894, 223, 224, 896, 897, 900, 901, 902, 903, 909, 922, 923,
    924, 961, 904, 905, 906, 907, 908, 910, 920, 911, 912, 913, 914, 915, 916, 917,
    58, 925, 935, 938, 951, 957, 926, 927, 928, 929, 930, 931, 932, 933, 934, 936,
    937, 939, 942, 940, 941, 943, 944, 945, 948, 946, 947, 949, 950, 952, 953, 954,
    955, 956, 958, 959, 960, 223, 224, 962, 963, 964, 965, 967, 968, 969, 971, 972,
    973, 974, 975, 976, 977, 978, 982, 983, 979, 981, 980, 223, 224, 984, 985, 995,
    998, 996, 997, 999, 223, 224, 1023, 1024, 1025, 1027, 1028, 1031, 1051, 1029, 1030, 1032,
    1047, 1048, 1033, 1042, 1034, 1037, 1035, 1036, 1038, 1039, 1040, 1041, 1043, 1046, 1044, 1045,
    1049, 1050, 1056, 1057, 223, 224, 1058, 58, 1059, 1060, 1062, 1067, 1063, 1064, 1065, 1066,
    1093, 1094, 1097, 1095, 1096, 1098, 1103, 1099, 1100, 1101, 223, 224, 1102, 58, 1105, 1106,
    1107, 1108, 1109, 1110, 1126, 1111, 1115, 1122, 1112, 1113, 223, 224, 1114, 58, 1116, 1117,
    1120, 1121, 1118, 1119, 1119, 223, 224, 1123, 1124, 1125, 1127, 1133, 1141, 1153, 1156, 1159,
    1128, 1129, 1130, 1131, 1132, 1134, 1135, 1136, 1137, 1140, 1138, 1139, 1142, 1143, 1144, 1145,
    1146, 1147, 1148, 1149, 1152, 1150, 1151, 1154, 1155, 1157, 1158, 1163, 1164, 1176, 1177, 1178,
    1191, 1192, 1193, 1218, 1219, 1221, 1222, 1223, 1227, 1224, 1225, 1226, 223, 224, 1229, 1230,
    1231, 1232, 1233, 1235, 1236, 1237, 1238, 1240, 1241, 1242, 1243, 1244, 1247, 1248, 1249, 1250,
    1252, 1253, 1254, 1255, 1257, 1258, 1259, 1260, 1261, 1263, 1264, 1265, 1267, 1268, 1269, 1270,
    1271, 1273, 1272, 58, 1275, 1276, 1277, 1278, 1280, 1281, 1282, 1283, 1284, 1286, 1287, 1288,
    1289, 1291, 1292, 1293, 1294, 1295, 1306, 445, 1307, 226, 223, 224, 1312, 1313, 1345, 1346,
    1347, 1348, 1349, 1350, 1353, 1354, 1355, 1356, 1357, 1360, 1361, 1362, 1366, 1367, 1368, 1369,
    1375, 1358, 1359, 1363, 1364, 1365, 1370, 1371, 1372, 1373, 1374, 1376, 1377, 1379, 1382, 1383,
    1384, 1380, 1381, 1386, 1387, 1388, 1389, 1390, 1394, 1391, 1392, 1393, 1396, 1399, 1397, 1398,
    1401, 1405, 1411, 1412, 1402, 1403, 1404, 223, 224, 1406, 1409, 1407, 1408, 1410, 1414, 1449,
    1415, 1416, 1417, 1429, 1430, 1431, 1438, 1446, 1447, 1418, 223, 224, 1419, 1420, 1421, 1422,
    1423, 58, 1424, 1425, 1426, 1427, 1428, 1432, 1435, 1433, 1434, 1436, 223, 224, 1437, 58,
    1439, 1440, 1441, 1442, 1443, 1444, 1445, 1451, 1452, 1453, 1454, 1455, 1456, 1457, 1469, 1470,
    1473, 1477, 1480, 1471, 1472, 1474, 1475, 1476, 1478, 1479, 1481, 1482, 1483, 1484, 1490, 1492,
    1496, 1491, 1493, 1494, 1495, 1497, 1500, 1501, 1502, 223, 224, 1503, 1504, 1517, 1518, 1525,
    1526, 1527, 1540, 1528, 1531, 1529, 1530, 1532, 1537, 1533, 1534, 1535, 1536, 1538, 1539, 1541,
    1527, 1542, 1527, 1544, 1545, 1551, 1552, 1553, 58, 1555, 1556, 1557, 1560, 1563, 1564, 1565,
    1566, 1567, 1568, 1569, 1570, 1571, 1572, 1573, 1574, 1575, 1558, 1559, 1561, 1562, 1577, 1595,
    1578, 1579, 1582, 1583, 1589, 1593, 1594, 1580, 1581, 1581, 223, 224, 1584, 1585, 1586, 1587,
    1588, 1590, 1591, 1592, 1596, 223, 224, 1577, 1597, 1598, 1600, 1601, 1601, 223, 224, 445,
    1602, 226, 1613, 1614, 1625, 1603, 1604, 1605, 1606, 223, 224, 1608, 1609, 1611, 1612, 1615,
    1616, 1617, 1618, 1620, 1621, 1623, 1624, 1630, 1631, 1633, 1634, 1638, 1644, 1650, 1656, 1657,
    1660, 1670, 1671, 1672, 1635, 1636, 1637, 1639, 1640, 1642, 1643, 1641, 1645, 1648, 1649, 1646,
    1647, 1651, 1652, 1653, 1654, 1655, 58, 1658, 1659, 1661, 1665, 1666, 1662, 1663, 1664, 1667,
    1668, 1669, 1674, 1675, 223, 224, 1677, 1678, 1679, 1680, 1681, 1685, 1690, 1686, 1687, 1688,
    1691, 1692, 1695, 1702, 1696, 1697, 1698, 1701, 1699, 1700, 1703, 1706, 1704, 1705, 1708, 223,
    224, 1709, 58, 1711, 1714, 1715, 1712, 1713, 1716, 1723, 1717, 1722, 1718, 1719, 1720, 1721,
    1738, 1739, 1753, 1754, 1755, 1768, 1740, 1744, 1747, 1752, 1741, 1742, 1743, 1745, 1746, 1748,
    1749, 1750, 1751, 1756, 1760, 1764, 1757, 1758, 1759, 1761, 1762, 1763, 1765, 1766, 1767, 1769,
    1770, 1771, 1772, 1773, 1774, 1779, 1780, 1784, 1785, 216, 1786, 226, 1787, 1788, 1789, 1790,
    1791, 1792, 58, 1793, 1794, 1795, 1798, 1801, 1802, 1803, 1804, 1805, 1806, 1807, 1808, 1809,
    1810, 1799, 1800, 1813, 1814, 58, 1815, 1816, 1817, 1818, 223, 224, 1822, 1823, 1824, 1825,
    223, 224, 1826, 58, 1827, 1828, 1833, 445, 1834, 226, 1835, 1836, 1836, 223, 224, 1838,
    1841, 1842, 1843, 1844, 1845, 1846, 1847, 1848, 1849, 1839, 1840, 1873, 1874, 1877, 1878, 1879,
    1882, 1885, 1883, 1884, 1895, 1896, 1897, 1898, 1903, 1899, 223, 224, 1900, 1902, 1901, 58,
    1909, 1912, 1910, 1911, 1913, 1914, 1915, 1934, 1935, 1936, 1937, 1938, 1939, 1940, 1943, 1941,
    1942, 1961, 1962, 1963, 1968, 1969, 1964, 1965, 1966, 1967, 1970, 1971, 1972, 1973, 1978, 1979,
    1980, 1981, 1982, 1984, 1985, 1986, 1987, 1994, 1995, 1996, 2001, 2002, 1997, 2000, 1998, 1999,
    2008, 2021, 2026, 2009, 2010, 216, 2011, 2012, 226, 2014, 2013, 2015, 2016, 2017, 2020, 2018,
    2019, 2022, 2023, 216, 2024, 2025, 226, 2027, 216, 2028, 226, 2031, 2034, 2035, 2036, 2039,
    2037, 2038, 2038, 223, 224, 2041, 2042, 2043, 2047, 2044, 2045, 2046, 2049, 216, 2050, 226,
    2051, 223, 224, 2052, 58, 2064, 2070, 2065, 2066, 2067, 2068, 2069, 2071, 2072, 2073, 2074,
    2075, 2076, 2077, 2078, 2079, 2080, 2081, 2082, 2083, 2084, 2102, 2103, 2108, 2111, 2112, 2109,
    2110, 2113, 2114, 2116, 2117, 2118, 2122, 2139, 2123, 216, 2124, 2130, 226, 2125, 2126, 2126,
    223, 224, 2127, 2128, 2129, 2131, 2136, 2132, 2133, 2134, 2135, 2137, 2138, 2140, 2141, 2149,
    2150, 2151, 2152, 2153, 2154, 2155, 2156, 2157, 2172, 2181, 2173, 2174, 2175, 2176, 2180, 2177,
    2178, 2179, 2182, 2183, 2184, 2201, 2202, 2203, 2204, 2205, 2208, 223, 224, 2206, 2207, 2209,
    2210, 2211, 223, 224, 2232, 216, 2233, 226, 2234, 2236, 2237, 2240, 2241, 2243, 2244, 2245,
    2248, 216, 2249, 226, 2246, 2247, 2250, 2251, 2253, 2254, 2255, 2256, 2259, 216, 2260, 226,
    2257, 2258, 2262, 216, 2263, 226, 2266, 216, 2267, 226, 2277, 2278, 2279, 2280, 2264, 2265,
    2268, 2269, 2270, 2274, 2271, 2273, 2272, 223, 224, 2275, 2276, 2282, 216, 2283, 226, 2325,
    2326, 2327, 2329, 2330, 2331, 2332, 2333, 2334, 223, 224, 2336, 216, 2337, 226, 2338, 58,
    2371, 2372, 2373, 2375, 2377, 2379, 2374, 223, 224, 2376, 223, 224, 2378, 223, 224, 2382,
    2383, 2384, 2385, 2387, 2388, 2391, 2389, 2390, 2390, 223, 224, 2401, 2404, 2402, 2403, 2405,
    2406, 2426, 2427, 2428, 2430, 2432, 1320, 2431, 58, 2434, 2435, 2436, 2437, 2439, 2440, 2441,
    2442, 2443, 2444, 2445, 2446, 2447, 2448, 2449, 412, 2457, 2458, 2461, 2459, 2460, 2462, 2463,
    2488, 2489, 2491, 2492, 216, 2493, 226, 2494, 58, 2495, 2499, 2502, 2507, 2496, 2497, 2498,
    2500, 2501, 2503, 2504, 2505, 2506, 2511, 2512, 2513, 2514, 2515, 2516, 2517, 2518, 2519, 2520,
    2521, 2522, 2525, 2526, 216, 2536, 2537, 2538, 226, 2539, 2540, 2541, 223, 224, 2544, 2545,
    2546, 2551, 2547, 2548, 2549, 2550, 223, 224, 2577, 2578, 216, 2579, 226, 2580, 2581, 2581,
    223, 224, 2582, 58, 2591, 2596, 2592, 2593, 2594, 2595, 2614, 2615, 2620, 2630, 2631, 2649,
    2650, 2653, 2654, 2658, 2655, 2656, 2657, 2659, 223, 224, 2678, 2679, 2680, 2683, 2681, 2682,
    2684, 2685, 2695, 2696, 2697, 2698, 2699, 2700, 2704, 2701, 2702, 2703, 2705, 2706, 2707, 2709,
    2710, 2711, 2712, 2713, 2718, 2719, 2720, 2723, 2726, 2721, 2722, 2724, 2725, 2727, 2728, 2769,
    2770, 2774, 2788, 2789, 2790, 2771, 2772, 2773, 2775, 2779, 2782, 2787, 2776, 2777, 2778, 2780,
    2781, 2783, 2784, 2785, 2786, 2816, 2817, 2818, 2819, 2836, 2837, 2840, 2842, 2838, 2839, 2841,
    223, 224, 2843, 2845, 2844, 2846, 223, 224, 2868, 2869, 2870, 2871, 2873, 2874, 2876, 2877,
    216, 2878, 226, 2879, 58, 2882, 2884, 2885, 2888, 2889, 2890, 2891, 2892, 2893, 2894, 2895,
    2883, 2886, 2887, 2898, 2899, 2900, 2901, 2902, 2903, 2904, 2905, 2906, 2907, 2908, 2909, 2910,
    2911, 2918, 2921, 2924, 2925, 2919, 2920, 2920, 223, 224, 2922, 2923, 2923, 223, 224, 2926,
    223, 224, 2933, 2936, 2934, 2935, 2937, 2938, 2940, 2943, 2941, 2942, 2944, 2945, 2953, 216,
    2954, 226, 2955, 58, 2957, 2958, 2959, 2960, 2961, 2962, 2963, 2964, 2965, 2966, 2968, 2969,
    2973, 2974, 2975, 2976, 2977, 2970, 2971, 2972, 2982, 2983, 2988, 2984, 2987, 2985, 2986, 2990,
    2991, 2992, 2993, 3000, 3003, 3001, 3002, 3004, 3005, 3006, 3007, 3012, 3013, 3017, 3018, 3021,
    58, 3023, 3024, 3025, 3029, 3030, 3042, 3043, 3044, 3045, 3046, 3047, 3048, 3049, 3050, 3051,
    3052, 3053, 3054, 3055, 3056, 3057, 3058, 3059, 3060, 3061, 3062, 3066, 3070, 3071, 3076, 3079,
    3083, 3084, 3089, 3092, 3098, 3102, 3103, 3106, 3109, 3110, 3113, 3116, 3117, 3118, 3119, 3124,
    3125, 3126, 3127, 3063, 3064, 3065, 3067, 3068, 3069, 3072, 3073, 3074, 3075, 3077, 3078, 3080,
    3081, 3082, 3085, 3086, 3087, 3088, 3090, 3091, 3093, 3094, 3095, 3096, 3097, 3099, 3100, 3101,
    3104, 3105, 3107, 3108, 3111, 3112, 3114, 3115, 3120, 3121, 3122, 3123, 3166, 3167, 3168, 3169,
    3170, 3171, 3172, 3173, 3174, 3175, 3176, 3177, 3178, 3179, 3180, 3181, 3182, 3183, 3184, 3185,
    3186, 3187, 3188, 3189, 3190, 3191, 3192, 3193, 3194, 3195, 3196, 3197, 3198, 3199, 3200, 3201,
    3202, 3203, 3204, 3205, 3206, 3207, 3225, 3228, 223, 224, 3229, 58, 3230, 3231, 3255, 3258,
    3256, 3257, 3259, 3262, 3265, 3260, 3261, 3263, 3264, 3266, 3267, 3268, 3271, 3272, 3273, 3274,
    3275, 3277, 3276, 223, 224, 3278, 223, 224, 3285, 3288, 3286, 3290, 3292, 3291, 3300, 3301,
    3304, 3305, 3316, 3319, 3320, 3317, 3318, 3321, 3324, 3325, 3322, 3323, 3327, 3328, 3329, 3330,
    3331, 3348, 3354, 3357, 3360, 3349, 3350, 3351, 3352, 3353, 3355, 3356, 3358, 3359, 3361, 3362,
    3363, 3380, 3381, 3382, 3387, 3388, 3389, 3393, 223, 224, 3394, 58, 3397, 3398, 3400, 3401,
    3409, 3410, 3411, 3416, 3417, 3418, 3419, 3422, 3431, 3432, 3433, 3434, 3435, 3436, 3437, 3448,
    3449, 3450, 3451, 3452, 3458, 216, 3459, 3460, 226, 3462, 3463, 3464, 3461, 3471, 3472, 3492,
    3493, 3502, 3503, 3494, 58, 3495, 3500, 3501, 3496, 223, 224, 3497, 58, 3498, 3499, 3519,
    3520, 3523, 3524, 3521, 3522, 3535, 3536, 3538, 3539, 3540, 3541, 3542, 3543, 445, 3537, 226,
    3547, 445, 3548, 226, 3549, 3550, 3550, 223, 224, 3552, 3553, 3556, 3559, 3560, 3561, 3562,
    3563, 3564, 3565, 3566, 3567, 3568, 3554, 3555, 3557, 3558, 3574, 3576, 3584, 3577, 3578, 3581,
    3579, 3580, 3582, 3583, 3587, 3591, 3592, 3593, 3588, 3589, 3590, 3595, 3596, 3597, 3598, 3601,
    3602, 3599, 58, 3604, 3605, 3606, 3607, 3610, 3611, 3608, 58, 3613, 3614, 3616, 3615, 3620,
    3621, 3622, 3623, 3625, 3626, 3627, 3628, 223, 224, 3639, 3640, 3643, 3644, 3645, 3646, 3641,
    3642, 3649, 3650, 3651, 3652, 3653, 3656, 3657, 3654, 3655,
];

pub static TERMINATORS: &[u32] = &[
    59, 60, 136, 367, 368, 448, 449, 451, 458, 459, 461, 463, 464, 465, 512, 567,
    918, 919, 921, 1448, 1607, 1610, 1619, 1622, 1689, 1693, 1796, 1819, 2508, 2509, 2880, 2912,
    2913, 2914, 2915, 2916, 2978, 2979, 3019, 3208, 3287, 3438, 3439, 3440, 3441, 3442, 3443, 3444,
    3445, 3446, 3600, 3609,
];

pub static STRINGS: &[&str] = &[
    "ABORT", // [0]
    "keyword", // [1]
    "KeywordSegment", // [2]
    "base", // [3]
    "raw", // [4]
    "word", // [5]
    "ABS", // [6]
    "ABSOLUTE", // [7]
    "ACCESS", // [8]
    "AccountKeywordSegment", // [9]
    "ACCOUNT", // [10]
    "ALL", // [11]
    "CATALOG", // [12]
    "DATABASE", // [13]
    "DOMAIN", // [14]
    "EXTERNAL", // [15]
    "FILE", // [16]
    "FOREIGN", // [17]
    "FUNCTION", // [18]
    "FUTURE", // [19]
    "INTEGRATION", // [20]
    "LANGUAGE", // [21]
    "MASKING", // [22]
    "MATERIALIZED", // [23]
    "PROCEDURE", // [24]
    "RESOURCE", // [25]
    "ROLE", // [26]
    "ROUTINE", // [27]
    "SCHEMA", // [28]
    "SEQUENCE", // [29]
    "STAGE", // [30]
    "STREAM", // [31]
    "TABLE", // [32]
    "TABLESPACE", // [33]
    "TASK", // [34]
    "TYPE", // [35]
    "VIEW", // [36]
    "WAREHOUSE", // [37]
    "ResourceKeywordSegment", // [38]
    "MonitorKeywordSegment", // [39]
    "MONITOR", // [40]
    "WarehouseKeywordSegment", // [41]
    "DatabaseKeywordSegment", // [42]
    "DomainKeywordSegment", // [43]
    "IntegrationKeywordSegment", // [44]
    "LanguageKeywordSegment", // [45]
    "SchemaKeywordSegment", // [46]
    "CatalogKeywordSegment", // [47]
    "RoleKeywordSegment", // [48]
    "TablespaceKeywordSegment", // [49]
    "TypeKeywordSegment", // [50]
    "ForeignKeywordSegment", // [51]
    "DATA", // [52]
    "SERVER", // [53]
    "ServerKeywordSegment", // [54]
    "DataKeywordSegment", // [55]
    "WrapperKeywordSegment", // [56]
    "WRAPPER", // [57]
    "AllKeywordSegment", // [58]
    "SchemasKeywordSegment", // [59]
    "SCHEMAS", // [60]
    "InKeywordSegment", // [61]
    "IN", // [62]
    "FutureKeywordSegment", // [63]
    "AccessSchemaObjectSegment", // [64]
    "access_schema_object", // [65]
    "AccessSchemaPluralObjectSegment", // [66]
    "FUNCTIONS", // [67]
    "PROCEDURES", // [68]
    "ROUTINES", // [69]
    "SEQUENCES", // [70]
    "STAGES", // [71]
    "STREAMS", // [72]
    "TABLES", // [73]
    "TASKS", // [74]
    "VIEWS", // [75]
    "access_schema_plural_object", // [76]
    "ObjectReferenceSegment", // [77]
    "object_reference", // [78]
    "FunctionNameSegment", // [79]
    "function_name", // [80]
    "FunctionParameterListGrammar", // [81]
    "(", // [82]
    "function_parameter_list", // [83]
    "WildcardIdentifierSegment", // [84]
    "wildcard_identifier", // [85]
    "CommaSegment", // [86]
    ",", // [87]
    "ToKeywordSegment", // [88]
    "TO", // [89]
    "FromKeywordSegment", // [90]
    "FROM", // [91]
    "LARGE", // [92]
    "LargeKeywordSegment", // [93]
    "ObjectKeywordSegment", // [94]
    "OBJECT", // [95]
    "NumericLiteralSegment", // [96]
    "numeric_literal", // [97]
    "APPLY", // [98]
    "CONNECT", // [99]
    "CREATE", // [100]
    "DELETE", // [101]
    "EXECUTE", // [102]
    "IMPORTED", // [103]
    "INSERT", // [104]
    "MANAGE", // [105]
    "MODIFY", // [106]
    "OPERATE", // [107]
    "OWNERSHIP", // [108]
    "READ", // [109]
    "REFERENCES", // [110]
    "REFERENCE_USAGE", // [111]
    "SELECT", // [112]
    "TEMP", // [113]
    "TEMPORARY", // [114]
    "TRIGGER", // [115]
    "TRUNCATE", // [116]
    "UPDATE", // [117]
    "USAGE", // [118]
    "USE", // [119]
    "USE_ANY_ROLE", // [120]
    "WRITE", // [121]
    "CreateKeywordSegment", // [122]
    "USER", // [123]
    "UserKeywordSegment", // [124]
    "ApplyKeywordSegment", // [125]
    "MaskingKeywordSegment", // [126]
    "PolicyKeywordSegment", // [127]
    "POLICY", // [128]
    "ExecuteKeywordSegment", // [129]
    "TaskKeywordSegment", // [130]
    "ManageKeywordSegment", // [131]
    "GrantsKeywordSegment", // [132]
    "GRANTS", // [133]
    "EXECUTION", // [134]
    "ExecutionKeywordSegment", // [135]
    "UsageKeywordSegment", // [136]
    "PIPE", // [137]
    "PipeKeywordSegment", // [138]
    "UseKeywordSegment", // [139]
    "ImportedKeywordSegment", // [140]
    "PrivilegesKeywordSegment", // [141]
    "PRIVILEGES", // [142]
    "ConnectKeywordSegment", // [143]
    "DeleteKeywordSegment", // [144]
    "InsertKeywordSegment", // [145]
    "ModifyKeywordSegment", // [146]
    "OperateKeywordSegment", // [147]
    "OwnershipKeywordSegment", // [148]
    "ReadKeywordSegment", // [149]
    "Reference_usageKeywordSegment", // [150]
    "ReferencesKeywordSegment", // [151]
    "SelectKeywordSegment", // [152]
    "TempKeywordSegment", // [153]
    "TemporaryKeywordSegment", // [154]
    "TriggerKeywordSegment", // [155]
    "TruncateKeywordSegment", // [156]
    "UpdateKeywordSegment", // [157]
    "Use_any_roleKeywordSegment", // [158]
    "WriteKeywordSegment", // [159]
    "BracketedColumnReferenceListGrammar", // [160]
    "AccessPermissionSegment", // [161]
    "access_permission", // [162]
    "OnKeywordSegment", // [163]
    "ON", // [164]
    "TableKeywordSegment", // [165]
    "ViewKeywordSegment", // [166]
    "StageKeywordSegment", // [167]
    "FunctionKeywordSegment", // [168]
    "ProcedureKeywordSegment", // [169]
    "RoutineKeywordSegment", // [170]
    "SequenceKeywordSegment", // [171]
    "StreamKeywordSegment", // [172]
    "MaterializedKeywordSegment", // [173]
    "ExternalKeywordSegment", // [174]
    "FileKeywordSegment", // [175]
    "FormatKeywordSegment", // [176]
    "FORMAT", // [177]
    "TablesKeywordSegment", // [178]
    "ViewsKeywordSegment", // [179]
    "StagesKeywordSegment", // [180]
    "FunctionsKeywordSegment", // [181]
    "ProceduresKeywordSegment", // [182]
    "RoutinesKeywordSegment", // [183]
    "SequencesKeywordSegment", // [184]
    "StreamsKeywordSegment", // [185]
    "TasksKeywordSegment", // [186]
    "PoliciesKeywordSegment", // [187]
    "FormatsKeywordSegment", // [188]
    "GRANT", // [189]
    "REVOKE", // [190]
    "GrantStatementSegment", // [191]
    "grant_statement", // [192]
    "RevokeStatementSegment", // [193]
    "revoke_statement", // [194]
    "UserReferenceSegment", // [195]
    "user_reference", // [196]
    "GROUP", // [197]
    "GroupKeywordSegment", // [198]
    "RoleReferenceSegment", // [199]
    "role_reference", // [200]
    "PublicKeywordSegment", // [201]
    "PUBLIC", // [202]
    "FunctionSegment", // [203]
    "function", // [204]
    "[", // [205]
    "ArrayAccessorSegment", // [206]
    "array_accessor", // [207]
    "ACCOUNTS", // [208]
    "ACTION", // [209]
    "ACTIVATE", // [210]
    "ADA", // [211]
    "ADD", // [212]
    "ADMIN", // [213]
    "AFTER", // [214]
    "AGE", // [215]
    "AGGREGATE", // [216]
    "OrderByClauseSegment", // [217]
    "ORDER", // [218]
    "orderby_clause", // [219]
    "indent", // [220]
    "Indent", // [221]
    "meta", // [222]
    "AsAliasOperatorSegment", // [223]
    "AS", // [224]
    "alias_operator", // [225]
    "SingleIdentifierGrammar", // [226]
    "SingleIdentifierListSegment", // [227]
    "identifier_list", // [228]
    "start_bracket", // [229]
    "SymbolSegment", // [230]
    "symbol", // [231]
    ")", // [232]
    "end_bracket", // [233]
    "SingleQuotedIdentifierSegment", // [234]
    "single_quote", // [235]
    "dedent", // [236]
    "Dedent", // [237]
    "ALIAS", // [238]
    "TableReferenceSegment", // [239]
    "table_reference", // [240]
    "AliasExpressionSegment", // [241]
    "alias_expression", // [242]
    "ALLOCATE", // [243]
    "ALLOW", // [244]
    "ALSO", // [245]
    "ALTER", // [246]
    "CACHE", // [247]
    "CYCLE", // [248]
    "INCREMENT", // [249]
    "MAXVALUE", // [250]
    "MINVALUE", // [251]
    "NO", // [252]
    "NOCACHE", // [253]
    "NOCYCLE", // [254]
    "NOORDER", // [255]
    "IncrementKeywordSegment", // [256]
    "ByKeywordSegment", // [257]
    "BY", // [258]
    "SequenceMinValueGrammar", // [259]
    "SequenceMaxValueGrammar", // [260]
    "CacheKeywordSegment", // [261]
    "NocacheKeywordSegment", // [262]
    "CycleKeywordSegment", // [263]
    "NocycleKeywordSegment", // [264]
    "OrderNoOrderGrammar", // [265]
    "AlterKeywordSegment", // [266]
    "SequenceReferenceSegment", // [267]
    "sequence_reference", // [268]
    "AlterSequenceOptionsSegment", // [269]
    "alter_sequence_options_segment", // [270]
    "DROP", // [271]
    "DropKeywordSegment", // [272]
    "ColumnKeywordSegment", // [273]
    "COLUMN", // [274]
    "DropBehaviorGrammar", // [275]
    "CASCADE", // [276]
    "RESTRICT", // [277]
    "ParameterNameSegment", // [278]
    "EqualsSegment", // [279]
    "=", // [280]
    "comparison_operator", // [281]
    "LiteralGrammar", // [282]
    "NakedIdentifierSegment", // [283]
    "AddKeywordSegment", // [284]
    "ColumnDefinitionSegment", // [285]
    "column_definition", // [286]
    "FIRST", // [287]
    "FirstKeywordSegment", // [288]
    "AfterKeywordSegment", // [289]
    "ColumnReferenceSegment", // [290]
    "column_reference", // [291]
    "AlterTableDropColumnGrammar", // [292]
    "RENAME", // [293]
    "RenameKeywordSegment", // [294]
    "AsKeywordSegment", // [295]
    "AlterTableOptionsGrammar", // [296]
    "ALWAYS", // [297]
    "&", // [298]
    "ampersand", // [299]
    "ANALYSE", // [300]
    "ANALYZE", // [301]
    "AND", // [302]
    "binary_operator", // [303]
    "BinaryOperatorSegment", // [304]
    "ANY", // [305]
    "ARE", // [306]
    "%", // [307]
    "*", // [308]
    "+", // [309]
    "-", // [310]
    "/", // [311]
    "<", // [312]
    ">", // [313]
    "^", // [314]
    "|", // [315]
    "PlusSegment", // [316]
    "MinusSegment", // [317]
    "DivideSegment", // [318]
    "MultiplySegment", // [319]
    "ModuloSegment", // [320]
    "BitwiseAndSegment", // [321]
    "BitwiseOrSegment", // [322]
    "BitwiseXorSegment", // [323]
    "BitwiseLShiftSegment", // [324]
    "BitwiseRShiftSegment", // [325]
    "ExpressionSegment", // [326]
    "expression", // [327]
    "SliceSegment", // [328]
    ":", // [329]
    "start_square_bracket", // [330]
    "]", // [331]
    "end_square_bracket", // [332]
    "ARRAY", // [333]
    "BaseExpressionElementGrammar", // [334]
    "ASC", // [335]
    "ASENSITIVE", // [336]
    "ASSERTION", // [337]
    "ASSIGNMENT", // [338]
    "ASSOCIATE", // [339]
    "ASUTIME", // [340]
    "ASYMMETRIC", // [341]
    "ASYNCHRONY", // [342]
    "AT", // [343]
    "ATOMIC", // [344]
    "ATTRIBUTE", // [345]
    "ATTRIBUTES", // [346]
    "AUDIT", // [347]
    "AUTHORIZATION", // [348]
    "AUTO_INCREMENT", // [349]
    "Auto_incrementKeywordSegment", // [350]
    "AUX", // [351]
    "AUXILIARY", // [352]
    "AVG", // [353]
    "AVG_ROW_LENGTH", // [354]
    "BACKUP", // [355]
    "BACKWARD", // [356]
    "SpecialRegisterGrammar", // [357]
    "CURRENT", // [358]
    "CURRENT_DATE", // [359]
    "CURRENT_PATH", // [360]
    "CURRENT_SCHEMA", // [361]
    "CURRENT_SERVER", // [362]
    "CURRENT_TIME", // [363]
    "CURRENT_TIMESTAMP", // [364]
    "CURRENT_TIMEZONE", // [365]
    "CURRENT_USER", // [366]
    "SESSION_USER", // [367]
    "SYSTEM_USER", // [368]
    "BareFunctionSegment", // [369]
    "bare_function", // [370]
    "IntervalExpressionSegment", // [371]
    "INTERVAL", // [372]
    "interval_expression", // [373]
    "DatatypeSegment", // [374]
    "data_type", // [375]
    "file", // [376]
    "BaseFileSegment", // [377]
    "BaseSegment", // [378]
    "BEFORE", // [379]
    "BEGIN", // [380]
    "BERNOULLI", // [381]
    "BETWEEN", // [382]
    "BIGINT", // [383]
    "BINARY", // [384]
    "!", // [385]
    "IS", // [386]
    "OR", // [387]
    "like_operator", // [388]
    "ArithmeticBinaryOperatorGrammar", // [389]
    "StringBinaryOperatorGrammar", // [390]
    "BooleanBinaryOperatorGrammar", // [391]
    "ComparisonOperatorGrammar", // [392]
    "BINDING", // [393]
    "BIT", // [394]
    "BIT_LENGTH", // [395]
    "BITVAR", // [396]
    "AmpersandSegment", // [397]
    "RawLessThanSegment", // [398]
    "PipeSegment", // [399]
    "RawGreaterThanSegment", // [400]
    "BLOB", // [401]
    "BOOL", // [402]
    "AndOperatorGrammar", // [403]
    "OrOperatorGrammar", // [404]
    "BOOLEAN", // [405]
    "FALSE", // [406]
    "TRUE", // [407]
    "TrueSegment", // [408]
    "FalseSegment", // [409]
    "BOTH", // [410]
    "bracketed", // [411]
    "BracketedSegment", // [412]
    "SetExpressionSegment", // [413]
    "VALUES", // [414]
    "set_expression", // [415]
    "BREADTH", // [416]
    "BREAK", // [417]
    "BROWSE", // [418]
    "BUFFERPOOL", // [419]
    "BULK", // [420]
    "BUSINESS_TIME", // [421]
    "CTEColumnList", // [422]
    "cte_column_list", // [423]
    "SelectableGrammar", // [424]
    "WITH", // [425]
    "CALL", // [426]
    "CallKeywordSegment", // [427]
    "CALLED", // [428]
    "CAPTURE", // [429]
    "CARDINALITY", // [430]
    "CASCADED", // [431]
    "CASE", // [432]
    "CaseKeywordSegment", // [433]
    "implicit_indent", // [434]
    "ImplicitIndent", // [435]
    "WHEN", // [436]
    "WhenClauseSegment", // [437]
    "when_clause", // [438]
    "ElseKeywordSegment", // [439]
    "ELSE", // [440]
    "EndKeywordSegment", // [441]
    "END", // [442]
    "ElseClauseSegment", // [443]
    "else_clause", // [444]
    "BinaryOperatorGrammar", // [445]
    "CAST", // [446]
    "::", // [447]
    "casting_operator", // [448]
    "CATALOG_NAME", // [449]
    "CCSID", // [450]
    "CEIL", // [451]
    "CEILING", // [452]
    "CHAIN", // [453]
    "CHANGE", // [454]
    "CHAR", // [455]
    "CHAR_LENGTH", // [456]
    "CHARACTER", // [457]
    "CHARACTER_LENGTH", // [458]
    "CHARACTER_SET_CATALOG", // [459]
    "CHARACTER_SET_NAME", // [460]
    "CHARACTER_SET_SCHEMA", // [461]
    "CHARACTERISTICS", // [462]
    "CHARACTERS", // [463]
    "CHECK", // [464]
    "CHECKED", // [465]
    "CHECKPOINT", // [466]
    "CHECKSUM", // [467]
    "CLASS", // [468]
    "CLASS_ORIGIN", // [469]
    "CLIENT_ACCTNG", // [470]
    "CLIENT_APPLNAME", // [471]
    "CLIENT_USERID", // [472]
    "CLIENT_WRKSTNNAME", // [473]
    "CLOB", // [474]
    "CLONE", // [475]
    "CLOSE", // [476]
    "CLUSTER", // [477]
    "CLUSTERED", // [478]
    "COALESCE", // [479]
    "COBOL", // [480]
    "CodeSegment", // [481]
    "COLLATE", // [482]
    "COLLATION", // [483]
    "QuotedLiteralSegment", // [484]
    "ObjectReferenceDelimiterGrammar", // [485]
    ".", // [486]
    "ObjectReferenceTerminatorGrammar", // [487]
    "COLLATION_CATALOG", // [488]
    "COLLATION_NAME", // [489]
    "COLLATION_SCHEMA", // [490]
    "COLLECT", // [491]
    "COLLECTION", // [492]
    "COLLID", // [493]
    "colon_delimiter", // [494]
    "colon_prefix", // [495]
    "colon", // [496]
    "ShorthandCastSegment", // [497]
    "cast_expression", // [498]
    "CONSTRAINT", // [499]
    "ConstraintKeywordSegment", // [500]
    "NOT", // [501]
    "NULL", // [502]
    "NotKeywordSegment", // [503]
    "NullKeywordSegment", // [504]
    "CheckKeywordSegment", // [505]
    "DEFAULT", // [506]
    "DefaultKeywordSegment", // [507]
    "ColumnConstraintDefaultGrammar", // [508]
    "PRIMARY", // [509]
    "PrimaryKeyGrammar", // [510]
    "NotEnforcedGrammar", // [511]
    "UniqueKeyGrammar", // [512]
    "UNIQUE", // [513]
    "AutoIncrementGrammar", // [514]
    "ReferenceDefinitionGrammar", // [515]
    "CommentClauseSegment", // [516]
    "COMMENT", // [517]
    "comment_clause", // [518]
    "CollateKeywordSegment", // [519]
    "CollationReferenceSegment", // [520]
    "collation_reference", // [521]
    "ColumnGeneratedGrammar", // [522]
    "ColumnConstraintSegment", // [523]
    "column_constraint_segment", // [524]
    "COLUMN_NAME", // [525]
    "ColumnsExpressionNameGrammar", // [526]
    "COLUMNS", // [527]
    "comma", // [528]
    "COMMAND_FUNCTION", // [529]
    "COMMAND_FUNCTION_CODE", // [530]
    "CommentKeywordSegment", // [531]
    "INDEX", // [532]
    "PACKAGE", // [533]
    "VARIABLE", // [534]
    "IndexKeywordSegment", // [535]
    "IndexReferenceSegment", // [536]
    "index_reference", // [537]
    "SchemaReferenceSegment", // [538]
    "schema_reference", // [539]
    "AliasKeywordSegment", // [540]
    "PackageKeywordSegment", // [541]
    "VariableKeywordSegment", // [542]
    "IsKeywordSegment", // [543]
    "comment", // [544]
    "CommentSegment", // [545]
    "COMMIT", // [546]
    "COMMITTED", // [547]
    "GreaterThanSegment", // [548]
    "LessThanSegment", // [549]
    "GreaterThanOrEqualToSegment", // [550]
    "LessThanOrEqualToSegment", // [551]
    "NotEqualToSegment", // [552]
    "LikeOperatorSegment", // [553]
    "IsDistinctFromGrammar", // [554]
    "ComparisonOperatorSegment", // [555]
    "COMPLETION", // [556]
    "CompositeBinaryOperatorSegment", // [557]
    "CompositeComparisonOperatorSegment", // [558]
    "COMPRESS", // [559]
    "COMPUTE", // [560]
    "CONCAT", // [561]
    "CONDITION", // [562]
    "CONDITION_NUMBER", // [563]
    "JoinTypeKeywordsGrammar", // [564]
    "FULL", // [565]
    "INNER", // [566]
    "LEFT", // [567]
    "RIGHT", // [568]
    "ConditionalCrossJoinKeywordsGrammar", // [569]
    "NonStandardJoinTypeKeywordsGrammar", // [570]
    "CONNECTION", // [571]
    "CONNECTION_NAME", // [572]
    "CONSTRAINT_CATALOG", // [573]
    "CONSTRAINT_NAME", // [574]
    "CONSTRAINT_SCHEMA", // [575]
    "CONSTRAINTS", // [576]
    "CONSTRUCTOR", // [577]
    "CONTAINS", // [578]
    "CONTAINSTABLE", // [579]
    "CONTINUE", // [580]
    "CONVERSION", // [581]
    "CONVERT", // [582]
    "COPY", // [583]
    "EXCLUDING", // [584]
    "INCLUDING", // [585]
    "IncludingKeywordSegment", // [586]
    "ExcludingKeywordSegment", // [587]
    "DefaultsKeywordSegment", // [588]
    "DEFAULTS", // [589]
    "IdentityKeywordSegment", // [590]
    "IDENTITY", // [591]
    "AttributesKeywordSegment", // [592]
    "CORR", // [593]
    "CORRESPONDING", // [594]
    "COUNT", // [595]
    "COUNT_BIG", // [596]
    "COVAR_POP", // [597]
    "COVAR_SAMP", // [598]
    "CastKeywordSegment", // [599]
    "WithKeywordSegment", // [600]
    "SpecificKeywordSegment", // [601]
    "SPECIFIC", // [602]
    "INSTANCE", // [603]
    "METHOD", // [604]
    "STATIC", // [605]
    "InstanceKeywordSegment", // [606]
    "StaticKeywordSegment", // [607]
    "ConstructorKeywordSegment", // [608]
    "MethodKeywordSegment", // [609]
    "FOR", // [610]
    "ForKeywordSegment", // [611]
    "AssignmentKeywordSegment", // [612]
    "IfNotExistsGrammar", // [613]
    "IF", // [614]
    "DatabaseReferenceSegment", // [615]
    "database_reference", // [616]
    "OrReplaceGrammar", // [617]
    "TemporaryGrammar", // [618]
    "RETURNS", // [619]
    "ReturnsKeywordSegment", // [620]
    "FunctionDefinitionGrammar", // [621]
    "function_definition", // [622]
    "UniqueKeywordSegment", // [623]
    "IndexColumnDefinitionSegment", // [624]
    "index_column_definition", // [625]
    "Business_timeKeywordSegment", // [626]
    "WithoutKeywordSegment", // [627]
    "WITHOUT", // [628]
    "OverlapsKeywordSegment", // [629]
    "OVERLAPS", // [630]
    "DISALLOW", // [631]
    "EXCLUDE", // [632]
    "EXTEND", // [633]
    "GENERATE", // [634]
    "INCLUDE", // [635]
    "LEVEL2", // [636]
    "MINPCTUSED", // [637]
    "PAGE", // [638]
    "PARTITIONED", // [639]
    "PCTFREE", // [640]
    "SPECIFICATION", // [641]
    "PartitionedKeywordSegment", // [642]
    "TablespaceReferenceSegment", // [643]
    "tablespace_reference", // [644]
    "SpecificationKeywordSegment", // [645]
    "OnlyKeywordSegment", // [646]
    "ONLY", // [647]
    "IncludeKeywordSegment", // [648]
    "XmlIndexSpecificationGrammar", // [649]
    "ClusterKeywordSegment", // [650]
    "ExtendKeywordSegment", // [651]
    "UsingKeywordSegment", // [652]
    "USING", // [653]
    "PctfreeKeywordSegment", // [654]
    "Level2KeywordSegment", // [655]
    "MinpctusedKeywordSegment", // [656]
    "AllowKeywordSegment", // [657]
    "DisallowKeywordSegment", // [658]
    "ReverseKeywordSegment", // [659]
    "REVERSE", // [660]
    "ScansKeywordSegment", // [661]
    "SCANS", // [662]
    "PageKeywordSegment", // [663]
    "SplitKeywordSegment", // [664]
    "SPLIT", // [665]
    "HIGH", // [666]
    "LOW", // [667]
    "SYMMETRIC", // [668]
    "SymmetricKeywordSegment", // [669]
    "HighKeywordSegment", // [670]
    "LowKeywordSegment", // [671]
    "CollectKeywordSegment", // [672]
    "DETAILED", // [673]
    "SAMPLED", // [674]
    "UNSAMPLED", // [675]
    "SampledKeywordSegment", // [676]
    "UnsampledKeywordSegment", // [677]
    "DetailedKeywordSegment", // [678]
    "StatisticsKeywordSegment", // [679]
    "STATISTICS", // [680]
    "CompressKeywordSegment", // [681]
    "YES", // [682]
    "YesKeywordSegment", // [683]
    "NoKeywordSegment", // [684]
    "ExcludeKeywordSegment", // [685]
    "KeysKeywordSegment", // [686]
    "KEYS", // [687]
    "ModelKeywordSegment", // [688]
    "MODEL", // [689]
    "OPTIONS", // [690]
    "OptionsKeywordSegment", // [691]
    "START", // [692]
    "StartKeywordSegment", // [693]
    "CreateSequenceOptionsSegment", // [694]
    "create_sequence_options_segment", // [695]
    "TemporaryTransientGrammar", // [696]
    "TRANSIENT", // [697]
    "LIKE", // [698]
    "TableConstraintSegment", // [699]
    "table_constraint", // [700]
    "LikeKeywordSegment", // [701]
    "TableEndClauseSegment", // [702]
    "table_end_clause_segment", // [703]
    "TriggerReferenceSegment", // [704]
    "trigger_reference", // [705]
    "INSTEAD", // [706]
    "BeforeKeywordSegment", // [707]
    "InsteadKeywordSegment", // [708]
    "OfKeywordSegment", // [709]
    "OF", // [710]
    "OrKeywordSegment", // [711]
    "DEFERRABLE", // [712]
    "INITIALLY", // [713]
    "REFERENCING", // [714]
    "ReferencingKeywordSegment", // [715]
    "OldKeywordSegment", // [716]
    "OLD", // [717]
    "RowKeywordSegment", // [718]
    "ROW", // [719]
    "NewKeywordSegment", // [720]
    "NEW", // [721]
    "DeferrableKeywordSegment", // [722]
    "InitiallyKeywordSegment", // [723]
    "ImmediateKeywordSegment", // [724]
    "IMMEDIATE", // [725]
    "DeferredKeywordSegment", // [726]
    "DEFERRED", // [727]
    "EachKeywordSegment", // [728]
    "EACH", // [729]
    "STATEMENT", // [730]
    "StatementKeywordSegment", // [731]
    "WhenKeywordSegment", // [732]
    "FunctionNameIdentifierSegment", // [733]
    "FunctionContentsSegment", // [734]
    "function_contents", // [735]
    "WithNoSchemaBindingClauseSegment", // [736]
    "with_no_schema_binding_clause", // [737]
    "RowMovementClauseSegment", // [738]
    "row_movement_clause", // [739]
    "ViewCheckOptionClauseSegment", // [740]
    "check_option_clause", // [741]
    "CREATEDB", // [742]
    "CREATEROLE", // [743]
    "CREATEUSER", // [744]
    "CROSS", // [745]
    "CS", // [746]
    "CSV", // [747]
    "CUBE", // [748]
    "function_name_identifier", // [749]
    "ROLLUP", // [750]
    "CubeFunctionNameSegment", // [751]
    "RollupFunctionNameSegment", // [752]
    "GroupingExpressionList", // [753]
    "grouping_expression_list", // [754]
    "CUME_DIST", // [755]
    "CURRENT_DEFAULT_TRANSFORM_GROUP", // [756]
    "CURRENT_LC_CTYPE", // [757]
    "CURRENT_ROLE", // [758]
    "CURRENT_TRANSFORM_GROUP_FOR_TYPE", // [759]
    "CURSOR", // [760]
    "CURSOR_NAME", // [761]
    "DATABASES", // [762]
    "DATAPARTITIONNAME", // [763]
    "DATAPARTITIONNUM", // [764]
    "data_type_identifier", // [765]
    "TimeWithTZGrammar", // [766]
    "TIME", // [767]
    "TIMESTAMP", // [768]
    "DOUBLE", // [769]
    "DoubleKeywordSegment", // [770]
    "PrecisionKeywordSegment", // [771]
    "PRECISION", // [772]
    "CharacterKeywordSegment", // [773]
    "BinaryKeywordSegment", // [774]
    "VARYING", // [775]
    "VaryingKeywordSegment", // [776]
    "DotSegment", // [777]
    "DatatypeIdentifierSegment", // [778]
    "BracketedArguments", // [779]
    "bracketed_arguments", // [780]
    "UnsignedKeywordSegment", // [781]
    "UNSIGNED", // [782]
    "CharCharacterSetGrammar", // [783]
    "ArrayTypeSegment", // [784]
    "array_type", // [785]
    "DATE", // [786]
    "DATEADD", // [787]
    "DatePartFunctionName", // [788]
    "DatetimeUnitSegment", // [789]
    "DAY", // [790]
    "DAYOFYEAR", // [791]
    "HOUR", // [792]
    "MILLISECOND", // [793]
    "MINUTE", // [794]
    "MONTH", // [795]
    "QUARTER", // [796]
    "SECOND", // [797]
    "WEEK", // [798]
    "WEEKDAY", // [799]
    "YEAR", // [800]
    "FunctionContentsGrammar", // [801]
    "DateKeywordSegment", // [802]
    "TimeKeywordSegment", // [803]
    "TimestampKeywordSegment", // [804]
    "IntervalKeywordSegment", // [805]
    "date_constructor_literal", // [806]
    "literal", // [807]
    "LiteralSegment", // [808]
    "DATETIME", // [809]
    "date_part", // [810]
    "DATETIME_INTERVAL_CODE", // [811]
    "DATETIME_INTERVAL_PRECISION", // [812]
    "DAY_HOUR", // [813]
    "DAY_MICROSECOND", // [814]
    "DAY_MINUTE", // [815]
    "DAY_SECOND", // [816]
    "DAYOFMONTH", // [817]
    "DAYOFWEEK", // [818]
    "DAYS", // [819]
    "DB2GENERAL", // [820]
    "DB2GENRL", // [821]
    "DB2SQL", // [822]
    "DBCC", // [823]
    "DBINFO", // [824]
    "DBPARTITIONNAME", // [825]
    "DBPARTITIONNUM", // [826]
    "DEALLOCATE", // [827]
    "DEC", // [828]
    "DECFLOAT", // [829]
    "DECIMAL", // [830]
    "DISTRIBUTE", // [831]
    "DistributeKeywordSegment", // [832]
    "HASH", // [833]
    "RANDOM", // [834]
    "HashKeywordSegment", // [835]
    "RandomKeywordSegment", // [836]
    "DECLARE", // [837]
    "DeclareKeywordSegment", // [838]
    "GlobalKeywordSegment", // [839]
    "GLOBAL", // [840]
    "WithDataClauseSegment", // [841]
    "with_data_clause", // [842]
    "CopyOptionsSegment", // [843]
    "copy_options", // [844]
    "LOGGED", // [845]
    "ORGANIZE", // [846]
    "OrganizeKeywordSegment", // [847]
    "CommitKeywordSegment", // [848]
    "PRESERVE", // [849]
    "PreserveKeywordSegment", // [850]
    "RowsKeywordSegment", // [851]
    "ROWS", // [852]
    "LoggedKeywordSegment", // [853]
    "RollbackKeywordSegment", // [854]
    "ROLLBACK", // [855]
    "ReplaceKeywordSegment", // [856]
    "REPLACE", // [857]
    "DeclareDistributionClauseSegment", // [858]
    "distribution_clause", // [859]
    "ValuesKeywordSegment", // [860]
    "DEFINED", // [861]
    "DEFINER", // [862]
    "DEFINITION", // [863]
    "DEGREE", // [864]
    "DELAY_KEY_WRITE", // [865]
    "DELAYED", // [866]
    "FromClauseSegment", // [867]
    "from_clause", // [868]
    "WhereClauseSegment", // [869]
    "WHERE", // [870]
    "where_clause", // [871]
    "SemicolonSegment", // [872]
    ";", // [873]
    "DELIMITER", // [874]
    "DELIMITERS", // [875]
    "DENSE_RANK", // [876]
    "DENSERANK", // [877]
    "DENY", // [878]
    "DEPTH", // [879]
    "DEREF", // [880]
    "DERIVED", // [881]
    "DESC", // [882]
    "DESCRIBE", // [883]
    "DescribeKeywordSegment", // [884]
    "DESCRIPTOR", // [885]
    "DESTROY", // [886]
    "DESTRUCTOR", // [887]
    "DETERMINISTIC", // [888]
    "DIAGNOSTICS", // [889]
    "DICTIONARY", // [890]
    "DISABLE", // [891]
    "DISCONNECT", // [892]
    "DISK", // [893]
    "DISPATCH", // [894]
    "DISTINCT", // [895]
    "DISTINCTROW", // [896]
    "DISTRIBUTED", // [897]
    "DIV", // [898]
    "DO", // [899]
    "DOCUMENT", // [900]
    "dot", // [901]
    "RestrictKeywordSegment", // [902]
    "CascadeKeywordSegment", // [903]
    "IfExistsGrammar", // [904]
    "DSSIZE", // [905]
    "DUMMY", // [906]
    "DUMP", // [907]
    "DYNAMIC", // [908]
    "DYNAMIC_FUNCTION", // [909]
    "DYNAMIC_FUNCTION_CODE", // [910]
    "EDITPROC", // [911]
    "ELEMENT", // [912]
    "ELSEIF", // [913]
    "StructTypeSegment", // [914]
    "struct_type", // [915]
    "EmptyStructLiteralBracketsSegment", // [916]
    "struct_literal", // [917]
    "ENABLE", // [918]
    "ENCLOSED", // [919]
    "ENCODING", // [920]
    "ENCRYPTED", // [921]
    "ENCRYPTION", // [922]
    "END-EXEC", // [923]
    "}", // [924]
    "end_curly_bracket", // [925]
    "ENDING", // [926]
    "ENUM", // [927]
    "EQUALS", // [928]
    "RawEqualsSegment", // [929]
    "ERASE", // [930]
    "ERRLVL", // [931]
    "ESCAPE", // [932]
    "ESCAPED", // [933]
    "EVERY", // [934]
    "EXCEPT", // [935]
    "EXCEPTION", // [936]
    "EXCLUSIVE", // [937]
    "EXEC", // [938]
    "EXISTING", // [939]
    "EXISTS", // [940]
    "EXIT", // [941]
    "EXP", // [942]
    "EXPLAIN", // [943]
    "ExplainKeywordSegment", // [944]
    "InsertStatementSegment", // [945]
    "insert_statement", // [946]
    "UpdateStatementSegment", // [947]
    "update_statement", // [948]
    "DeleteStatementSegment", // [949]
    "delete_statement", // [950]
    "Expression_A_Grammar", // [951]
    "Tail_Recurse_Expression_A_Grammar", // [952]
    "LikeExpressionGrammar", // [953]
    "ILIKE", // [954]
    "RLIKE", // [955]
    "InOperatorGrammar", // [956]
    "QuantifiedComparisonOperatorGrammar", // [957]
    "IsClauseGrammar", // [958]
    "IsNullGrammar", // [959]
    "NotNullGrammar", // [960]
    "CollateGrammar", // [961]
    "BetweenKeywordSegment", // [962]
    "Expression_B_Grammar", // [963]
    "AndKeywordSegment", // [964]
    "PatternMatchingGrammar", // [965]
    "PRIOR", // [966]
    "~", // [967]
    "SignedSegmentGrammar", // [968]
    "QualifiedNumericLiteralSegment", // [969]
    "TildeSegment", // [970]
    "NotOperatorGrammar", // [971]
    "PriorKeywordSegment", // [972]
    "Tail_Recurse_Expression_B_Grammar", // [973]
    "ExistsKeywordSegment", // [974]
    "Expression_D_Grammar", // [975]
    "CaseExpressionSegment", // [976]
    "case_expression", // [977]
    "TimeZoneGrammar", // [978]
    "time_zone_grammar", // [979]
    "LabeledDurationGrammar", // [980]
    "LateralColumnAliasExpressionGrammar", // [981]
    "LocalAliasSegment", // [982]
    "local_alias_segment", // [983]
    "Expression_D_Potential_Select_Statement_Without_Brackets", // [984]
    "SingleIdentifierWildcardSegment", // [985]
    "MapTypeSegment", // [986]
    "map_type", // [987]
    "BooleanLiteralGrammar", // [988]
    "NullLiteralSegment", // [989]
    "DateTimeLiteralGrammar", // [990]
    "ListComprehensionGrammar", // [991]
    "AccessorGrammar", // [992]
    "SelectStatementSegment", // [993]
    "select_statement", // [994]
    "TypedStructLiteralSegment", // [995]
    "typed_struct_literal", // [996]
    "ArrayExpressionSegment", // [997]
    "array_expression", // [998]
    "OverlapsClauseSegment", // [999]
    "overlaps_clause", // [1000]
    "EXTENDED", // [1001]
    "EXTENSION", // [1002]
    "EXTRACT", // [1003]
    "boolean_literal", // [1004]
    "LiteralKeywordSegment", // [1005]
    "FEDERATED", // [1006]
    "FENCED", // [1007]
    "FETCH", // [1008]
    "FetchKeywordSegment", // [1009]
    "NEXT", // [1010]
    "NextKeywordSegment", // [1011]
    "TiesKeywordSegment", // [1012]
    "TIES", // [1013]
    "FIELDPROC", // [1014]
    "FIELDS", // [1015]
    "MERGE", // [1016]
    "SET", // [1017]
    "DelimiterGrammar", // [1018]
    "StatementSegment", // [1019]
    "statement", // [1020]
    "FILLFACTOR", // [1021]
    "FILTER", // [1022]
    "FilterKeywordSegment", // [1023]
    "WhereKeywordSegment", // [1024]
    "FINAL", // [1025]
    "FLOAT4", // [1026]
    "FLOAT8", // [1027]
    "FLOAT", // [1028]
    "FLOOR", // [1029]
    "FLUSH", // [1030]
    "FOLLOWING", // [1031]
    "FORCE", // [1032]
    "KeyKeywordSegment", // [1033]
    "KEY", // [1034]
    "FORTRAN", // [1035]
    "FORWARD", // [1036]
    "FOUND", // [1037]
    "RANGE", // [1038]
    "FrameClauseUnitGrammar", // [1039]
    "CurrentKeywordSegment", // [1040]
    "UnboundedKeywordSegment", // [1041]
    "UNBOUNDED", // [1042]
    "PRECEDING", // [1043]
    "PrecedingKeywordSegment", // [1044]
    "FollowingKeywordSegment", // [1045]
    "RangeKeywordSegment", // [1046]
    "FREE", // [1047]
    "FREETEXT", // [1048]
    "FREETEXTTABLE", // [1049]
    "FREEZE", // [1050]
    "FromExpressionSegment", // [1051]
    "from_expression", // [1052]
    "HAVING", // [1053]
    "INTERSECT", // [1054]
    "LIMIT", // [1055]
    "MINUS", // [1056]
    "OFFSET", // [1057]
    "QUALIFY", // [1058]
    "UNION", // [1059]
    "WINDOW", // [1060]
    "LimitClauseSegment", // [1061]
    "limit_clause", // [1062]
    "OrderKeywordSegment", // [1063]
    "HavingKeywordSegment", // [1064]
    "QualifyKeywordSegment", // [1065]
    "WindowKeywordSegment", // [1066]
    "SetOperatorSegment", // [1067]
    "set_operator", // [1068]
    "OffsetKeywordSegment", // [1069]
    "ReadOnlyClauseSegment", // [1070]
    "read_only_clause", // [1071]
    "IsolationClauseSegment", // [1072]
    "isolation_clause", // [1073]
    "PreTableFunctionKeywordsGrammar", // [1074]
    "LATERAL", // [1075]
    "TableExpressionSegment", // [1076]
    "table_expression", // [1077]
    "TemporalQuerySegment", // [1078]
    "temporal_query", // [1079]
    "FromClauseTerminatorGrammar", // [1080]
    "SamplingExpressionSegment", // [1081]
    "TABLESAMPLE", // [1082]
    "sample_expression", // [1083]
    "JoinLikeClauseGrammar", // [1084]
    "JoinClauseSegment", // [1085]
    "join_clause", // [1086]
    "PostTableExpressionGrammar", // [1087]
    "MLTableExpressionSegment", // [1088]
    "ML", // [1089]
    "ml_table_expression", // [1090]
    "FromExpressionElementSegment", // [1091]
    "from_expression_element", // [1092]
    "indented_joins", // [1093]
    "FULLTEXT", // [1094]
    "NamedArgumentSegment", // [1095]
    "named_argument", // [1096]
    "LEADING", // [1097]
    "TRAILING", // [1098]
    "TrimParametersGrammar", // [1099]
    "DistinctKeywordSegment", // [1100]
    "StarSegment", // [1101]
    "FunctionContentsExpressionGrammar", // [1102]
    "AggregateOrderByClause", // [1103]
    "aggregate_order_by", // [1104]
    "SEPARATOR", // [1105]
    "SeparatorKeywordSegment", // [1106]
    "IgnoreRespectNullsGrammar", // [1107]
    "IGNORE", // [1108]
    "RESPECT", // [1109]
    "EmptyStructLiteralSegment", // [1110]
    "WordSegment", // [1111]
    "double_quote", // [1112]
    "QuotedIdentifierSegment", // [1113]
    "AnyKeywordSegment", // [1114]
    "FunctionParameterGrammar", // [1115]
    "DatePartFunctionNameSegment", // [1116]
    "DateTimeFunctionContentsSegment", // [1117]
    "ColumnsExpressionGrammar", // [1118]
    "ColumnsExpressionFunctionNameSegment", // [1119]
    "ValuesClauseSegment", // [1120]
    "values_clause", // [1121]
    "PostFunctionGrammar", // [1122]
    "FUSION", // [1123]
    "G", // [1124]
    "GENERAL", // [1125]
    "GENERATED", // [1126]
    "GET", // [1127]
    "glob_operator", // [1128]
    "GO", // [1129]
    "GOTO", // [1130]
    "GrantKeywordSegment", // [1131]
    "AccessPermissionsSegment", // [1132]
    "access_permissions", // [1133]
    "AccessObjectSegment", // [1134]
    "access_object", // [1135]
    "AccessTargetSegment", // [1136]
    "access_target", // [1137]
    "OptionKeywordSegment", // [1138]
    "OPTION", // [1139]
    "AdminKeywordSegment", // [1140]
    "CopyKeywordSegment", // [1141]
    "GRANTED", // [1142]
    "GrantedKeywordSegment", // [1143]
    "Current_userKeywordSegment", // [1144]
    "Session_userKeywordSegment", // [1145]
    "GRAPHIC", // [1146]
    "GREATEST", // [1147]
    "GroupingSetsClauseSegment", // [1148]
    "GROUPING", // [1149]
    "grouping_sets_clause", // [1150]
    "CubeRollupClauseSegment", // [1151]
    "cube_rollup_clause", // [1152]
    "GroupByClauseTerminatorGrammar", // [1153]
    "GroupingKeywordSegment", // [1154]
    "SetsKeywordSegment", // [1155]
    "SETS", // [1156]
    "HANDLER", // [1157]
    "HASHED_VALUE", // [1158]
    "HEADER", // [1159]
    "HEAP", // [1160]
    "HIERARCHY", // [1161]
    "HIGH_PRIORITY", // [1162]
    "HINT", // [1163]
    "HOLD", // [1164]
    "HOLDLOCK", // [1165]
    "HOST", // [1166]
    "HOSTS", // [1167]
    "HOUR_MICROSECOND", // [1168]
    "HOUR_MINUTE", // [1169]
    "HOUR_SECOND", // [1170]
    "HOURS", // [1171]
    "IDENTIFIED", // [1172]
    "identifier", // [1173]
    "IdentifierSegment", // [1174]
    "IDENTITY_INSERT", // [1175]
    "IDENTITYCOL", // [1176]
    "IfKeywordSegment", // [1177]
    "IgnoreKeywordSegment", // [1178]
    "RespectKeywordSegment", // [1179]
    "NullsKeywordSegment", // [1180]
    "NULLS", // [1181]
    "IMMUTABLE", // [1182]
    "IMPLEMENTATION", // [1183]
    "IMPLICIT", // [1184]
    "IMPORT", // [1185]
    "INCLUSIVE", // [1186]
    "AscKeywordSegment", // [1187]
    "DescKeywordSegment", // [1188]
    "INDICATOR", // [1189]
    "INDICATORS", // [1190]
    "INF", // [1191]
    "INFILE", // [1192]
    "INFINITY", // [1193]
    "INFIX", // [1194]
    "INHERIT", // [1195]
    "INHERITS", // [1196]
    "INITIAL", // [1197]
    "INITIALIZE", // [1198]
    "INOUT", // [1199]
    "INPUT", // [1200]
    "INSENSITIVE", // [1201]
    "OverwriteKeywordSegment", // [1202]
    "OVERWRITE", // [1203]
    "IntoKeywordSegment", // [1204]
    "INTO", // [1205]
    "DefaultValuesGrammar", // [1206]
    "INSERT_ID", // [1207]
    "INSTANTIABLE", // [1208]
    "INT1", // [1209]
    "INT2", // [1210]
    "INT3", // [1211]
    "INT4", // [1212]
    "INT8", // [1213]
    "INT", // [1214]
    "INTEGER", // [1215]
    "INTEGRATIONS", // [1216]
    "INTEGRITY", // [1217]
    "INTERSECTION", // [1218]
    "INVOKER", // [1219]
    "NanLiteralSegment", // [1220]
    "NAN", // [1221]
    "UnknownLiteralSegment", // [1222]
    "NormalizedGrammar", // [1223]
    "ISAM", // [1224]
    "ISNULL", // [1225]
    "ISOBID", // [1226]
    "RR", // [1227]
    "RS", // [1228]
    "UR", // [1229]
    "RrKeywordSegment", // [1230]
    "RsKeywordSegment", // [1231]
    "LockClauseSegment", // [1232]
    "KEEP", // [1233]
    "lock_clause", // [1234]
    "CsKeywordSegment", // [1235]
    "UrKeywordSegment", // [1236]
    "ISOLATION", // [1237]
    "ITERATE", // [1238]
    "JAR", // [1239]
    "JAVA", // [1240]
    "ConditionalJoinKeywordsGrammar", // [1241]
    "JoinKeywordsGrammar", // [1242]
    "JOIN", // [1243]
    "NestedJoinGrammar", // [1244]
    "indented_using_on", // [1245]
    "MatchConditionSegment", // [1246]
    "match_condition", // [1247]
    "JoinOnConditionSegment", // [1248]
    "join_on_condition", // [1249]
    "JoinUsingConditionGrammar", // [1250]
    "UnconditionalJoinKeywordsGrammar", // [1251]
    "ExtendedNaturalJoinKeywordsGrammar", // [1252]
    "JoinKeywordSegment", // [1253]
    "indented_on_contents", // [1254]
    "InnerKeywordSegment", // [1255]
    "FullKeywordSegment", // [1256]
    "LeftKeywordSegment", // [1257]
    "RightKeywordSegment", // [1258]
    "OuterKeywordSegment", // [1259]
    "OUTER", // [1260]
    "JSON", // [1261]
    "K", // [1262]
    "KEY_MEMBER", // [1263]
    "KEY_TYPE", // [1264]
    "KILL", // [1265]
    "LABEL", // [1266]
    "MICROSECOND", // [1267]
    "MICROSECONDS", // [1268]
    "MINUTES", // [1269]
    "MONTHS", // [1270]
    "SECONDS", // [1271]
    "YEARS", // [1272]
    "DayKeywordSegment", // [1273]
    "DaysKeywordSegment", // [1274]
    "HourKeywordSegment", // [1275]
    "HoursKeywordSegment", // [1276]
    "MicrosecondKeywordSegment", // [1277]
    "MicrosecondsKeywordSegment", // [1278]
    "MinuteKeywordSegment", // [1279]
    "MinutesKeywordSegment", // [1280]
    "MonthKeywordSegment", // [1281]
    "MonthsKeywordSegment", // [1282]
    "SecondKeywordSegment", // [1283]
    "SecondsKeywordSegment", // [1284]
    "YearKeywordSegment", // [1285]
    "YearsKeywordSegment", // [1286]
    "LANCOMPILER", // [1287]
    "LAST", // [1288]
    "LAST_INSERT_ID", // [1289]
    "LC_CTYPE", // [1290]
    "LC_MESSAGES", // [1291]
    "LC_TIME", // [1292]
    "LEAST", // [1293]
    "LEAVE", // [1294]
    "LENGTH", // [1295]
    "LESS", // [1296]
    "LEVEL", // [1297]
    "LikeGrammar", // [1298]
    "EscapeKeywordSegment", // [1299]
    "RlikeKeywordSegment", // [1300]
    "IlikeKeywordSegment", // [1301]
    "LimitKeywordSegment", // [1302]
    "OffsetClauseSegment", // [1303]
    "offset_clause", // [1304]
    "FetchClauseSegment", // [1305]
    "fetch_clause", // [1306]
    "LINENO", // [1307]
    "LINES", // [1308]
    "LINKTYPE", // [1309]
    "LISTEN", // [1310]
    "ArrayLiteralSegment", // [1311]
    "array_literal", // [1312]
    "TypedArrayLiteralSegment", // [1313]
    "typed_array_literal", // [1314]
    "ObjectLiteralSegment", // [1315]
    "{", // [1316]
    "object_literal", // [1317]
    "LN", // [1318]
    "LOAD", // [1319]
    "LOCAL", // [1320]
    "LOCALDATE", // [1321]
    "LOCALE", // [1322]
    "LOCALTIME", // [1323]
    "LOCALTIMESTAMP", // [1324]
    "LOCATION", // [1325]
    "LOCATOR", // [1326]
    "LOCATORS", // [1327]
    "KeepKeywordSegment", // [1328]
    "SHARE", // [1329]
    "ExclusiveKeywordSegment", // [1330]
    "ShareKeywordSegment", // [1331]
    "LocksKeywordSegment", // [1332]
    "LOCKS", // [1333]
    "LOCK", // [1334]
    "LOCKMAX", // [1335]
    "LOCKSIZE", // [1336]
    "LOGIN", // [1337]
    "LOGS", // [1338]
    "LONG", // [1339]
    "LONGBLOB", // [1340]
    "LONGTEXT", // [1341]
    "LOOP", // [1342]
    "LOW_PRIORITY", // [1343]
    "LOWER", // [1344]
    "M", // [1345]
    "MlKeywordSegment", // [1346]
    "MAINTAINED", // [1347]
    "MAP", // [1348]
    "MATCH", // [1349]
    "MATCHED", // [1350]
    "MAX", // [1351]
    "MAX_ROWS", // [1352]
    "MAXEXTENTS", // [1353]
    "MDC", // [1354]
    "MEDIUMBLOB", // [1355]
    "MEDIUMINT", // [1356]
    "MEDIUMTEXT", // [1357]
    "MEMBER", // [1358]
    "MergeKeywordSegment", // [1359]
    "MergeMatchedClauseSegment", // [1360]
    "merge_when_matched_clause", // [1361]
    "MergeNotMatchedClauseSegment", // [1362]
    "merge_when_not_matched_clause", // [1363]
    "MatchedKeywordSegment", // [1364]
    "ThenKeywordSegment", // [1365]
    "THEN", // [1366]
    "MergeUpdateClauseSegment", // [1367]
    "merge_update_clause", // [1368]
    "MergeDeleteClauseSegment", // [1369]
    "merge_delete_clause", // [1370]
    "MergeInsertClauseSegment", // [1371]
    "merge_insert_clause", // [1372]
    "MergeIntoLiteralGrammar", // [1373]
    "AliasedTableReferenceGrammar", // [1374]
    "MergeMatchSegment", // [1375]
    "merge_match", // [1376]
    "SetClauseListSegment", // [1377]
    "set_clause_list", // [1378]
    "MESSAGE_LENGTH", // [1379]
    "MESSAGE_OCTET_LENGTH", // [1380]
    "MESSAGE_TEXT", // [1381]
    "MIDDLEINT", // [1382]
    "MIN", // [1383]
    "MIN_ROWS", // [1384]
    "MINUTE_MICROSECOND", // [1385]
    "MINUTE_SECOND", // [1386]
    "MLSLABEL", // [1387]
    "MOD", // [1388]
    "MODE", // [1389]
    "MODIFIES", // [1390]
    "MODULE", // [1391]
    "MONTHNAME", // [1392]
    "MORE", // [1393]
    "MOVE", // [1394]
    "MOVEMENT", // [1395]
    "MULTISET", // [1396]
    "MUMPS", // [1397]
    "MYISAM", // [1398]
    "naked_identifier", // [1399]
    "NAME", // [1400]
    "RightArrowSegment", // [1401]
    "=>", // [1402]
    "WindowSpecificationSegment", // [1403]
    "window_specification", // [1404]
    "NamedWindowExpressionSegment", // [1405]
    "named_window_expression", // [1406]
    "NAMES", // [1407]
    "null_literal", // [1408]
    "NATIONAL", // [1409]
    "NCHAR", // [1410]
    "NCLOB", // [1411]
    "sign_indicator", // [1412]
    "NESTING", // [1413]
    "NEW_TABLE", // [1414]
    "newline", // [1415]
    "NewlineSegment", // [1416]
    "NEXTVAL", // [1417]
    "NO_WRITE_TO_BINLOG", // [1418]
    "NOAUDIT", // [1419]
    "NOCHECK", // [1420]
    "NOCOMPRESS", // [1421]
    "NOCREATEDB", // [1422]
    "NOCREATEROLE", // [1423]
    "NOCREATEUSER", // [1424]
    "NODENAME", // [1425]
    "NODENUMBER", // [1426]
    "NOINHERIT", // [1427]
    "NOLOGIN", // [1428]
    "NOMAXVALUE", // [1429]
    "NOMINVALUE", // [1430]
    "UnorderedSelectStatementSegment", // [1431]
    "WithCompoundStatementSegment", // [1432]
    "with_compound_statement", // [1433]
    "NonSetSelectableGrammar", // [1434]
    "BracketedSetExpressionGrammar", // [1435]
    "MergeStatementSegment", // [1436]
    "merge_statement", // [1437]
    "NONCLUSTERED", // [1438]
    "NONE", // [1439]
    "NORMALIZE", // [1440]
    "NORMALIZED", // [1441]
    "NOSUPERUSER", // [1442]
    "RawNotSegment", // [1443]
    "NOTHING", // [1444]
    "NOTIFY", // [1445]
    "NOTNULL", // [1446]
    "NOWAIT", // [1447]
    "NULLABLE", // [1448]
    "NULLIF", // [1449]
    "NUMBER", // [1450]
    "NUMERIC", // [1451]
    "NUMPARTS", // [1452]
    "OBID", // [1453]
    "ColonSegment", // [1454]
    "ObjectLiteralElementSegment", // [1455]
    "object_literal_element", // [1456]
    "start_curly_bracket", // [1457]
    "CastOperatorSegment", // [1458]
    "StartSquareBracketSegment", // [1459]
    "StartBracketSegment", // [1460]
    "OBJECTS", // [1461]
    "OCTET_LENGTH", // [1462]
    "OCTETS", // [1463]
    "OFF", // [1464]
    "OFFLINE", // [1465]
    "OFFSETS", // [1466]
    "OIDS", // [1467]
    "OLD_TABLE", // [1468]
    "ONLINE", // [1469]
    "OPEN", // [1470]
    "OPENDATASOURCE", // [1471]
    "OPENQUERY", // [1472]
    "OPENROWSET", // [1473]
    "OPENXML", // [1474]
    "OPERATION", // [1475]
    "OPERATOR", // [1476]
    "OPTIMIZATION", // [1477]
    "OPTIMIZE", // [1478]
    "OPTIONALLY", // [1479]
    "LastKeywordSegment", // [1480]
    "WithFillSegment", // [1481]
    "with_fill", // [1482]
    "NoorderKeywordSegment", // [1483]
    "ORDERING", // [1484]
    "ORDINALITY", // [1485]
    "OTHERS", // [1486]
    "OUT", // [1487]
    "OUTFILE", // [1488]
    "OUTPUT", // [1489]
    "OverKeywordSegment", // [1490]
    "OVER", // [1491]
    "OVERLAY", // [1492]
    "OVERRIDING", // [1493]
    "OWNER", // [1494]
    "PACK_KEYS", // [1495]
    "PAD", // [1496]
    "PADDED", // [1497]
    "PAGESIZE", // [1498]
    "PARAMETER", // [1499]
    "parameter", // [1500]
    "?", // [1501]
    "PARAMETER_MODE", // [1502]
    "PARAMETER_NAME", // [1503]
    "PARAMETER_ORDINAL_POSITION", // [1504]
    "PARAMETER_SPECIFIC_CATALOG", // [1505]
    "PARAMETER_SPECIFIC_NAME", // [1506]
    "PARAMETER_SPECIFIC_SCHEMA", // [1507]
    "PARAMETERS", // [1508]
    "PART", // [1509]
    "PARTIAL", // [1510]
    "PARTITION", // [1511]
    "PartitionKeywordSegment", // [1512]
    "PARTITIONING", // [1513]
    "PARTITIONS", // [1514]
    "PASCAL", // [1515]
    "PASSWORD", // [1516]
    "PATH", // [1517]
    "SlashSegment", // [1518]
    "path_segment", // [1519]
    "PERCENT", // [1520]
    "PERCENT_RANK", // [1521]
    "PERCENTILE_CONT", // [1522]
    "PERCENTILE_DISC", // [1523]
    "PIECESIZE", // [1524]
    "pipe", // [1525]
    "PLACING", // [1526]
    "PLAN", // [1527]
    "PLI", // [1528]
    "POSITION", // [1529]
    "OverClauseSegment", // [1530]
    "over_clause", // [1531]
    "WithinGroupClauseSegment", // [1532]
    "WITHIN", // [1533]
    "withingroup_clause", // [1534]
    "POSTFIX", // [1535]
    "POWER", // [1536]
    "LateralKeywordSegment", // [1537]
    "PREFIX", // [1538]
    "PREORDER", // [1539]
    "PREPARE", // [1540]
    "PREPARED", // [1541]
    "PREVVAL", // [1542]
    "PrimaryKeywordSegment", // [1543]
    "PRINT", // [1544]
    "PRIQTY", // [1545]
    "PROC", // [1546]
    "PROCEDURAL", // [1547]
    "PROCESS", // [1548]
    "PROCESSLIST", // [1549]
    "PROFILE", // [1550]
    "PROGRAM", // [1551]
    "PSID", // [1552]
    "PURGE", // [1553]
    "SOME", // [1554]
    "SomeKeywordSegment", // [1555]
    "QUERY", // [1556]
    "QUERYNO", // [1557]
    "QUOTE", // [1558]
    "quoted_identifier", // [1559]
    "quoted_literal", // [1560]
    "RAID0", // [1561]
    "RAISERROR", // [1562]
    "RANK", // [1563]
    "raw_comparison_operator", // [1564]
    "RAW", // [1565]
    "READS", // [1566]
    "READTEXT", // [1567]
    "REAL", // [1568]
    "RECHECK", // [1569]
    "RECONFIGURE", // [1570]
    "RECOVERY", // [1571]
    "RECURSIVE", // [1572]
    "REF", // [1573]
    "ReferenceMatchGrammar", // [1574]
    "ReferentialActionGrammar", // [1575]
    "MatchKeywordSegment", // [1576]
    "SIMPLE", // [1577]
    "PartialKeywordSegment", // [1578]
    "SimpleKeywordSegment", // [1579]
    "SetKeywordSegment", // [1580]
    "ActionKeywordSegment", // [1581]
    "REFRESH", // [1582]
    "REGEXP", // [1583]
    "REGR_AVGX", // [1584]
    "REGR_AVGY", // [1585]
    "REGR_COUNT", // [1586]
    "REGR_INTERCEPT", // [1587]
    "REGR_R2", // [1588]
    "REGR_SLOPE", // [1589]
    "REGR_SXX", // [1590]
    "REGR_SXY", // [1591]
    "REGR_SYY", // [1592]
    "REINDEX", // [1593]
    "REJECT", // [1594]
    "RELATIVE", // [1595]
    "RELEASE", // [1596]
    "RELOAD", // [1597]
    "REPEAT", // [1598]
    "REPEATABLE", // [1599]
    "REPLICATION", // [1600]
    "REQUIRE", // [1601]
    "RESET", // [1602]
    "RESIGNAL", // [1603]
    "RESTART", // [1604]
    "RESTORE", // [1605]
    "RESULT", // [1606]
    "RESULT_SET_LOCATOR", // [1607]
    "RETURN", // [1608]
    "RETURNED_CARDINALITY", // [1609]
    "RETURNED_LENGTH", // [1610]
    "RETURNED_OCTET_LENGTH", // [1611]
    "RETURNED_SQLSTATE", // [1612]
    "RevokeKeywordSegment", // [1613]
    "right_arrow", // [1614]
    "ROLES", // [1615]
    "ROLLOUT", // [1616]
    "ROUND_CEILING", // [1617]
    "ROUND_DOWN", // [1618]
    "ROUND_FLOOR", // [1619]
    "ROUND_HALF_DOWN", // [1620]
    "ROUND_HALF_EVEN", // [1621]
    "ROUND_HALF_UP", // [1622]
    "ROUND_UP", // [1623]
    "ROUNDING", // [1624]
    "ROUTINE_CATALOG", // [1625]
    "ROUTINE_NAME", // [1626]
    "ROUTINE_SCHEMA", // [1627]
    "MovementKeywordSegment", // [1628]
    "ROW_COUNT", // [1629]
    "ROW_NUMBER", // [1630]
    "ROWCOUNT", // [1631]
    "ROWGUIDCOL", // [1632]
    "ROWID", // [1633]
    "ROWNUM", // [1634]
    "ROWNUMBER", // [1635]
    "ROWSET", // [1636]
    "RRN", // [1637]
    "RULE", // [1638]
    "RUN", // [1639]
    "TablesampleKeywordSegment", // [1640]
    "SYSTEM", // [1641]
    "BernoulliKeywordSegment", // [1642]
    "SystemKeywordSegment", // [1643]
    "RepeatableKeywordSegment", // [1644]
    "SAVE", // [1645]
    "SAVEPOINT", // [1646]
    "SCALE", // [1647]
    "SCHEMA_NAME", // [1648]
    "SCOPE", // [1649]
    "SCOPE_CATALOG", // [1650]
    "SCOPE_NAME", // [1651]
    "SCOPE_SCHEMA", // [1652]
    "SCRATCHPAD", // [1653]
    "SCROLL", // [1654]
    "SEARCH", // [1655]
    "SECOND_MICROSECOND", // [1656]
    "SECQTY", // [1657]
    "SECTION", // [1658]
    "SECURITY", // [1659]
    "WildcardExpressionSegment", // [1660]
    "wildcard_expression", // [1661]
    "SelectClauseModifierSegment", // [1662]
    "select_clause_modifier", // [1663]
    "SelectClauseElementSegment", // [1664]
    "select_clause_element", // [1665]
    "SelectClauseTerminatorGrammar", // [1666]
    "SelectClauseSegment", // [1667]
    "select_clause", // [1668]
    "GroupByClauseSegment", // [1669]
    "groupby_clause", // [1670]
    "HavingClauseSegment", // [1671]
    "having_clause", // [1672]
    "NamedWindowSegment", // [1673]
    "named_window", // [1674]
    "WithCompoundNonSelectStatementSegment", // [1675]
    "NonWithSelectableGrammar", // [1676]
    "SELF", // [1677]
    "statement_terminator", // [1678]
    "SENSITIVE", // [1679]
    "MaxvalueKeywordSegment", // [1680]
    "MinvalueKeywordSegment", // [1681]
    "SERIALIZABLE", // [1682]
    "SERVER_NAME", // [1683]
    "SESSION", // [1684]
    "SetClauseSegment", // [1685]
    "set_clause", // [1686]
    "UnionGrammar", // [1687]
    "IntersectKeywordSegment", // [1688]
    "ExceptKeywordSegment", // [1689]
    "MinusKeywordSegment", // [1690]
    "SETOF", // [1691]
    "SETUSER", // [1692]
    "SHARES", // [1693]
    "SHOW", // [1694]
    "SHUTDOWN", // [1695]
    "SIGNAL", // [1696]
    "PositiveSegment", // [1697]
    "NegativeSegment", // [1698]
    "SIMILAR", // [1699]
    "SIZE", // [1700]
    "slash", // [1701]
    "slice", // [1702]
    "SMALLINT", // [1703]
    "SNAN", // [1704]
    "SNAPSHOT", // [1705]
    "SONAME", // [1706]
    "SOURCE", // [1707]
    "SPACE", // [1708]
    "SPATIAL", // [1709]
    "Current_dateKeywordSegment", // [1710]
    "Current_pathKeywordSegment", // [1711]
    "Current_schemaKeywordSegment", // [1712]
    "Current_serverKeywordSegment", // [1713]
    "Current_timeKeywordSegment", // [1714]
    "Current_timestampKeywordSegment", // [1715]
    "Current_timezoneKeywordSegment", // [1716]
    "System_userKeywordSegment", // [1717]
    "SQL_CCFLAGS", // [1718]
    "TEMPORAL", // [1719]
    "TIMEZONE", // [1720]
    "Client_acctngKeywordSegment", // [1721]
    "Client_applnameKeywordSegment", // [1722]
    "Client_useridKeywordSegment", // [1723]
    "Client_wrkstnnameKeywordSegment", // [1724]
    "DbpartitionnumKeywordSegment", // [1725]
    "DecfloatKeywordSegment", // [1726]
    "RoundingKeywordSegment", // [1727]
    "ModeKeywordSegment", // [1728]
    "TransformKeywordSegment", // [1729]
    "TRANSFORM", // [1730]
    "DegreeKeywordSegment", // [1731]
    "SnapshotKeywordSegment", // [1732]
    "FederatedKeywordSegment", // [1733]
    "AsynchronyKeywordSegment", // [1734]
    "ImplicitKeywordSegment", // [1735]
    "XmlparseKeywordSegment", // [1736]
    "XMLPARSE", // [1737]
    "IsolationKeywordSegment", // [1738]
    "LocaleKeywordSegment", // [1739]
    "Lc_messagesKeywordSegment", // [1740]
    "Lc_timeKeywordSegment", // [1741]
    "LockKeywordSegment", // [1742]
    "TimeoutKeywordSegment", // [1743]
    "TIMEOUT", // [1744]
    "MaintainedKeywordSegment", // [1745]
    "TypesKeywordSegment", // [1746]
    "TYPES", // [1747]
    "OptimizationKeywordSegment", // [1748]
    "MdcKeywordSegment", // [1749]
    "RolloutKeywordSegment", // [1750]
    "MemberKeywordSegment", // [1751]
    "ProfileKeywordSegment", // [1752]
    "PathKeywordSegment", // [1753]
    "QueryKeywordSegment", // [1754]
    "RefreshKeywordSegment", // [1755]
    "AgeKeywordSegment", // [1756]
    "Sql_ccflagsKeywordSegment", // [1757]
    "TemporalKeywordSegment", // [1758]
    "SYSTEM_TIME", // [1759]
    "System_timeKeywordSegment", // [1760]
    "TimezoneKeywordSegment", // [1761]
    "SPECIFIC_NAME", // [1762]
    "SPECIFICTYPE", // [1763]
    "SQL", // [1764]
    "SQL_BIG_RESULT", // [1765]
    "SQL_BIG_SELECTS", // [1766]
    "SQL_BIG_TABLES", // [1767]
    "SQL_CALC_FOUND_ROWS", // [1768]
    "SQL_LOG_OFF", // [1769]
    "SQL_LOG_UPDATE", // [1770]
    "SQL_LOW_PRIORITY_UPDATES", // [1771]
    "SQL_SELECT_LIMIT", // [1772]
    "SQL_SMALL_RESULT", // [1773]
    "SQL_WARNINGS", // [1774]
    "SQLCA", // [1775]
    "SQLCODE", // [1776]
    "SQLERROR", // [1777]
    "SQLEXCEPTION", // [1778]
    "SQLID", // [1779]
    "SQLSTATE", // [1780]
    "SQLWARNING", // [1781]
    "SQRT", // [1782]
    "SSL", // [1783]
    "STABLE", // [1784]
    "STACKED", // [1785]
    "STANDARD", // [1786]
    "star", // [1787]
    "STARTING", // [1788]
    "STARTS", // [1789]
    "STATE", // [1790]
    "TransactionStatementSegment", // [1791]
    "transaction_statement", // [1792]
    "DropTableStatementSegment", // [1793]
    "drop_table_statement", // [1794]
    "DropViewStatementSegment", // [1795]
    "drop_view_statement", // [1796]
    "CreateUserStatementSegment", // [1797]
    "create_user_statement", // [1798]
    "DropUserStatementSegment", // [1799]
    "drop_user_statement", // [1800]
    "TruncateStatementSegment", // [1801]
    "truncate_table", // [1802]
    "AccessStatementSegment", // [1803]
    "access_statement", // [1804]
    "CreateTableStatementSegment", // [1805]
    "create_table_statement", // [1806]
    "CreateRoleStatementSegment", // [1807]
    "create_role_statement", // [1808]
    "DropRoleStatementSegment", // [1809]
    "drop_role_statement", // [1810]
    "AlterTableStatementSegment", // [1811]
    "alter_table_statement", // [1812]
    "CreateSchemaStatementSegment", // [1813]
    "create_schema_statement", // [1814]
    "SetSchemaStatementSegment", // [1815]
    "set_schema_statement", // [1816]
    "DropSchemaStatementSegment", // [1817]
    "drop_schema_statement", // [1818]
    "DropTypeStatementSegment", // [1819]
    "drop_type_statement", // [1820]
    "CreateDatabaseStatementSegment", // [1821]
    "create_database_statement", // [1822]
    "DropDatabaseStatementSegment", // [1823]
    "drop_database_statement", // [1824]
    "CreateIndexStatementSegment", // [1825]
    "create_index_statement", // [1826]
    "DropIndexStatementSegment", // [1827]
    "drop_index_statement", // [1828]
    "CreateViewStatementSegment", // [1829]
    "create_view_statement", // [1830]
    "CreateCastStatementSegment", // [1831]
    "create_cast_statement", // [1832]
    "DropCastStatementSegment", // [1833]
    "drop_cast_statement", // [1834]
    "CreateFunctionStatementSegment", // [1835]
    "create_function_statement", // [1836]
    "DropFunctionStatementSegment", // [1837]
    "drop_function_statement", // [1838]
    "CreateModelStatementSegment", // [1839]
    "create_model_statement", // [1840]
    "DropModelStatementSegment", // [1841]
    "drop_MODELstatement", // [1842]
    "DescribeStatementSegment", // [1843]
    "describe_statement", // [1844]
    "UseStatementSegment", // [1845]
    "use_statement", // [1846]
    "ExplainStatementSegment", // [1847]
    "explain_statement", // [1848]
    "CreateSequenceStatementSegment", // [1849]
    "create_sequence_statement", // [1850]
    "AlterSequenceStatementSegment", // [1851]
    "alter_sequence_statement", // [1852]
    "DropSequenceStatementSegment", // [1853]
    "drop_sequence_statement", // [1854]
    "CreateTriggerStatementSegment", // [1855]
    "create_trigger", // [1856]
    "DropTriggerStatementSegment", // [1857]
    "drop_trigger", // [1858]
    "CallStoredProcedureSegment", // [1859]
    "call_segment", // [1860]
    "CommentOnStatementSegment", // [1861]
    "DeclareGlobalTempTableSegment", // [1862]
    "declare_temp_table", // [1863]
    "STATMENT", // [1864]
    "STAY", // [1865]
    "STDDEV_POP", // [1866]
    "STDDEV_SAMP", // [1867]
    "STDIN", // [1868]
    "STDOUT", // [1869]
    "STOGROUP", // [1870]
    "STORAGE", // [1871]
    "STORES", // [1872]
    "STRAIGHT_JOIN", // [1873]
    "STRICT", // [1874]
    "ConcatSegment", // [1875]
    "STRING", // [1876]
    "STRUCTURE", // [1877]
    "STYLE", // [1878]
    "SUBCLASS_ORIGIN", // [1879]
    "SUBLIST", // [1880]
    "SUBMULTISET", // [1881]
    "SUBSTRING", // [1882]
    "SUCCESSFUL", // [1883]
    "SUM", // [1884]
    "SUMMARY", // [1885]
    "SUPERUSER", // [1886]
    "SYNONYM", // [1887]
    "SYSDATE", // [1888]
    "SYSFUN", // [1889]
    "SYSIBM", // [1890]
    "SYSID", // [1891]
    "SYSPROC", // [1892]
    "ForeignKeyGrammar", // [1893]
    "TABLE_NAME", // [1894]
    "Expression_A_Unary_Operator_Grammar", // [1895]
    "Expression_C_Grammar", // [1896]
    "Expression_B_Unary_Operator_Grammar", // [1897]
    "TEMPLATE", // [1898]
    "TransientKeywordSegment", // [1899]
    "TERMINATE", // [1900]
    "TERMINATED", // [1901]
    "TEXT", // [1902]
    "TEXTSIZE", // [1903]
    "THAN", // [1904]
    "tilde", // [1905]
    "ZoneKeywordSegment", // [1906]
    "ZONE", // [1907]
    "AtKeywordSegment", // [1908]
    "TIMEZONE_HOUR", // [1909]
    "TIMEZONE_MINUTE", // [1910]
    "TINYBLOB", // [1911]
    "TINYINT", // [1912]
    "TINYTEXT", // [1913]
    "TOAST", // [1914]
    "TOP", // [1915]
    "TOP_LEVEL_COUNT", // [1916]
    "TRAN", // [1917]
    "TRANSACTION", // [1918]
    "BeginKeywordSegment", // [1919]
    "WORK", // [1920]
    "TransactionKeywordSegment", // [1921]
    "WorkKeywordSegment", // [1922]
    "NameKeywordSegment", // [1923]
    "ChainKeywordSegment", // [1924]
    "TRANSACTION_ACTIVE", // [1925]
    "TRANSACTIONS", // [1926]
    "TRANSACTIONS_COMMITTED", // [1927]
    "TRANSACTIONS_ROLLED_BACK", // [1928]
    "TRANSFORMS", // [1929]
    "TRANSLATE", // [1930]
    "TRANSLATION", // [1931]
    "TREAT", // [1932]
    "TRIGGER_CATALOG", // [1933]
    "TRIGGER_NAME", // [1934]
    "TRIGGER_SCHEMA", // [1935]
    "TRIM", // [1936]
    "BothKeywordSegment", // [1937]
    "LeadingKeywordSegment", // [1938]
    "TrailingKeywordSegment", // [1939]
    "TRUSTED", // [1940]
    "TSEQUAL", // [1941]
    "StructLiteralSegment", // [1942]
    "UESCAPE", // [1943]
    "UID", // [1944]
    "UNCOMMITTED", // [1945]
    "CrossKeywordSegment", // [1946]
    "NaturalJoinKeywordsGrammar", // [1947]
    "UnconditionalCrossJoinKeywordsGrammar", // [1948]
    "HorizontalJoinKeywordsGrammar", // [1949]
    "UNDER", // [1950]
    "UNDO", // [1951]
    "UNENCRYPTED", // [1952]
    "UnionKeywordSegment", // [1953]
    "UNKNOWN", // [1954]
    "UNLISTEN", // [1955]
    "UNLOCK", // [1956]
    "UNNAMED", // [1957]
    "UNNEST", // [1958]
    "UNTIL", // [1959]
    "UPDATETEXT", // [1960]
    "UPPER", // [1961]
    "USER_DEFINED_TYPE_CATALOG", // [1962]
    "USER_DEFINED_TYPE_CODE", // [1963]
    "USER_DEFINED_TYPE_NAME", // [1964]
    "USER_DEFINED_TYPE_SCHEMA", // [1965]
    "USERS", // [1966]
    "UTC_DATE", // [1967]
    "UTC_TIME", // [1968]
    "UTC_TIMESTAMP", // [1969]
    "VACUUM", // [1970]
    "VALID", // [1971]
    "VALIDATE", // [1972]
    "VALIDATOR", // [1973]
    "VALIDPROC", // [1974]
    "VALUE", // [1975]
    "VAR_POP", // [1976]
    "VAR_SAMP", // [1977]
    "VARBINARY", // [1978]
    "VARCHAR2", // [1979]
    "VARCHAR", // [1980]
    "VARCHARACTER", // [1981]
    "VARIABLES", // [1982]
    "VARIANT", // [1983]
    "VCAT", // [1984]
    "VERBOSE", // [1985]
    "VERSION", // [1986]
    "CascadedKeywordSegment", // [1987]
    "LocalKeywordSegment", // [1988]
    "VOLATILE", // [1989]
    "VOLUMES", // [1990]
    "WAITFOR", // [1991]
    "WAREHOUSES", // [1992]
    "indented_then", // [1993]
    "indented_then_contents", // [1994]
    "WHENEVER", // [1995]
    "WHILE", // [1996]
    "whitespace", // [1997]
    "WhitespaceSegment", // [1998]
    "WIDTH_BUCKET", // [1999]
    "PartitionClauseSegment", // [2000]
    "partitionby_clause", // [2001]
    "FrameClauseSegment", // [2002]
    "frame_clause", // [2003]
    "RecursiveKeywordSegment", // [2004]
    "indented_ctes", // [2005]
    "CTEDefinitionSegment", // [2006]
    "common_table_expression", // [2007]
    "NonWithNonSelectableGrammar", // [2008]
    "BindingKeywordSegment", // [2009]
    "WithinKeywordSegment", // [2010]
    "WLM", // [2011]
    "WRITETEXT", // [2012]
    "X509", // [2013]
    "GenerateKeywordSegment", // [2014]
    "XmlpatternKeywordSegment", // [2015]
    "XMLPATTERN", // [2016]
    "XmlTypeClauseGrammar", // [2017]
    "XML", // [2018]
    "SqlKeywordSegment", // [2019]
    "RejectKeywordSegment", // [2020]
    "InvalidKeywordSegment", // [2021]
    "XMLELEMENT", // [2022]
    "XMLEXISTS", // [2023]
    "XMLNAMESPACES", // [2024]
    "XOR", // [2025]
    "YAML", // [2026]
    "YEAR_MONTH", // [2027]
    "ZEROFILL", // [2028]
];

pub static AUX_DATA: &[u32] = &[
    0, 1, 2, 1, 1, 4, 3, 1, 4, 5, 6, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 7, 1, 2, 1, 1, 4, 3, 1, 4, 5, 8, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 0, 0, 1, 0, 0, 0,
    0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 1, 0, 0,
    0, 4294967295, 4294967295, 0, 10, 1, 2, 1, 1, 4, 3, 1, 4, 5, 208, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 209, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 210, 1, 2, 1, 1, 4, 3, 1, 4, 5, 211, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 212, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    213, 1, 2, 1, 1, 4, 3, 1, 4, 5, 214, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 215, 1, 2, 1, 1, 4, 3, 1, 4, 5, 216, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 82, 229, 230, 1, 229, 3, 3, 4,
    231, 232, 233, 230, 1, 233, 3, 3, 4, 231, 1, 2, 1, 0, 238, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 11, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 243, 1, 2, 1, 1, 4, 3, 1, 4, 5, 244, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 245, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    246, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 0, 0, 4294967295, 4294967295,
    0, 0, 0, 0, 0, 0, 0, 1, 0, 297, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 298, 299, 230, 1, 299, 3, 3, 4, 231, 300, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 301, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    302, 1, 2, 1, 1, 4, 3, 1, 4, 5, 302, 303, 304, 1, 303, 3,
    3, 303, 4, 305, 1, 2, 1, 1, 4, 3, 1, 4, 5, 98, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 306, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 0, 1, 0, 205, 330, 230, 1, 330, 3, 3, 4, 231, 331, 332,
    230, 1, 332, 3, 3, 4, 231, 1, 2, 0, 333, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1, 0, 1, 2, 0, 224, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 335, 1, 2, 1, 1, 4, 3, 1, 4, 5, 336, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 337, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 338, 1, 2, 1, 1, 4, 3, 1, 4, 5, 339, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 340, 1, 2, 1, 1, 4, 3, 1, 4, 5, 341,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 342, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 343, 1, 2, 1, 1, 4, 3, 1, 4, 5, 344, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 345, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 346, 1, 2, 1, 1, 4, 3, 1, 4, 5, 347, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 348, 1, 2, 1, 1, 4, 3, 1, 4, 5, 349,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 351, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 352, 1, 2, 1, 1, 4, 3, 1, 4, 5, 353, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 354, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 355, 1, 2, 1, 1, 4, 3, 1, 4, 5, 356, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 376, 377, 0, 2, 3, 376, 3, 378, 0, 1,
    3, 379, 1, 2, 1, 1, 4, 3, 1, 4, 5, 380, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 381, 1, 2, 1, 1, 4, 3, 1, 4, 5, 382,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 383, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 384, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 303, 304,
    9, 3, 3, 303, 4, 393, 1, 2, 1, 1, 4, 3, 1, 4, 5, 394,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 395, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 396, 1, 2, 1, 1, 4, 3, 1, 4, 5, 314, 303, 230,
    1, 303, 3, 3, 4, 231, 401, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    402, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 405, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 410, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1, 0, 1, 2, 1, 1, 0, 1, 2, 1, 411, 412, 0, 2, 3, 411,
    1, 2, 1, 416, 1, 2, 1, 1, 4, 3, 1, 4, 5, 417, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 418, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 419, 1, 2, 1, 1, 4, 3, 1, 4, 5, 420, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 421, 1, 2, 1, 1, 4, 3, 1, 4, 5, 258,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 1, 1, 2, 1, 247,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 426, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 428, 1, 2, 1, 1, 4, 3, 1, 4, 5, 429, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 430, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 276, 1, 2, 1, 1, 4, 3, 1, 4, 5, 431, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 4294967295, 4294967295, 0, 0, 4294967295, 4294967295, 0, 0, 432,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 446, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 447, 448, 230, 1, 448, 3, 3, 4, 231, 12, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 449, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    450, 1, 2, 1, 1, 4, 3, 1, 4, 5, 451, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 452, 1, 2, 1, 1, 4, 3, 1, 4, 5, 453, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 454, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 455, 1, 2, 1, 1, 4, 3, 1, 4, 5, 456, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 457, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    458, 1, 2, 1, 1, 4, 3, 1, 4, 5, 459, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 460, 1, 2, 1, 1, 4, 3, 1, 4, 5, 461, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 462, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 463, 1, 2, 1, 1, 4, 3, 1, 4, 5, 464, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 465, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    466, 1, 2, 1, 1, 4, 3, 1, 4, 5, 467, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 468, 1, 2, 1, 1, 4, 3, 1, 4, 5, 469, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 470, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 471, 1, 2, 1, 1, 4, 3, 1, 4, 5, 472, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 473, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    474, 1, 2, 1, 1, 4, 3, 1, 4, 5, 475, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 476, 1, 2, 1, 1, 4, 3, 1, 4, 5, 477, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 478, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 479, 1, 2, 1, 1, 4, 3, 1, 4, 5, 480, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 4, 481, 9, 2, 3, 4, 482, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 483, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1, 0, 0, 488, 1, 2, 1, 1, 4, 3, 1, 4, 5, 489, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 490, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 491, 1, 2, 1, 1, 4, 3, 1, 4, 5, 492, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 493, 1, 2, 1, 1, 4, 3, 1, 4, 5, 329,
    494, 230, 1, 494, 3, 3, 4, 231, 329, 495, 230, 1, 495, 3, 3, 4,
    231, 329, 496, 230, 1, 496, 3, 3, 4, 231, 0, 1, 2, 1, 0, 1,
    2, 1, 0, 4294967295, 4294967295, 0, 274, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1, 0, 525, 1, 2, 1, 1, 4, 3, 1, 4, 5, 527, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 87, 528, 230, 1, 528, 3, 3, 4, 231, 529,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 530, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 517, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 1,
    0, 1, 2, 1, 0, 544, 545, 10, 3, 3, 544, 4, 546, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 547, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 281, 555, 9, 3, 3, 281, 4, 556, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 303, 557, 0, 2, 3, 303, 281, 558, 0, 2, 3, 281, 559, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 560, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 561, 1, 2, 1, 1, 4, 3, 1, 4, 5, 562, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 563, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 99, 1, 2, 1, 1, 4, 3, 1, 4, 5, 571, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 572, 1, 2, 1, 1, 4, 3, 1, 4, 5, 499,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 573, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 574, 1, 2, 1, 1, 4, 3, 1, 4, 5, 575, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 576, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 577, 1, 2, 1, 1, 4, 3, 1, 4, 5, 578, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 579, 1, 2, 1, 1, 4, 3, 1, 4, 5, 580,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 581, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 582, 1, 2, 1, 1, 4, 3, 1, 4, 5, 583, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 0, 0, 0, 4294967295, 1, 0, 593, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 594, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 595, 1, 2, 1, 1, 4, 3, 1, 4, 5, 596, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 597, 1, 2, 1, 1, 4, 3, 1, 4, 5, 598,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 3, 4, 1, 0, 0, 0, 1,
    0, 1, 2, 1, 0, 1, 0, 1, 2, 1, 1, 0, 1, 2, 1, 2,
    3, 1, 0, 0, 0, 0, 0, 0, 0, 0, 4294967295, 1, 0, 100, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1, 0, 1, 2, 0, 0, 1, 0, 1,
    2, 1, 0, 0, 0, 0, 4294967295, 4294967295, 0, 0, 1, 0, 1, 2, 1, 1,
    2, 1, 0, 0, 0, 1, 0, 0, 1, 0, 0, 0, 0, 1, 2, 1,
    0, 4294967295, 4294967295, 0, 1, 2, 1, 0, 0, 742, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 743, 1, 2, 1, 1, 4, 3, 1, 4, 5, 744, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 745, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 746, 1, 2, 1, 1, 4, 3, 1, 4, 5, 747, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 748, 749, 481, 1, 749, 2, 3, 4, 748, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 0, 1, 2, 1, 755, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 358, 1, 2, 1, 1, 4, 3, 1, 4, 5, 359,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 756, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 757, 1, 2, 1, 1, 4, 3, 1, 4, 5, 360, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 758, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 361, 1, 2, 1, 1, 4, 3, 1, 4, 5, 362, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 363, 1, 2, 1, 1, 4, 3, 1, 4, 5, 364,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 365, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 759, 1, 2, 1, 1, 4, 3, 1, 4, 5, 366, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 760, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 761, 1, 2, 1, 1, 4, 3, 1, 4, 5, 248, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 52, 1, 2, 1, 1, 4, 3, 1, 4, 5, 13,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 762, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 763, 1, 2, 1, 1, 4, 3, 1, 4, 5, 764, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 0, 1, 765, 481, 1, 765, 2, 3, 4,
    0, 0, 0, 0, 0, 4294967295, 4294967295, 0, 0, 786, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 787, 2275, 1, 749, 481, 1, 749, 2, 3, 4, 0, 1, 0,
    1, 2, 1, 0, 235, 806, 808, 3, 806, 807, 235, 3, 3, 807, 4, 809,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 790, 791, 792, 793, 794, 795, 796,
    797, 798, 799, 800, 2313, 11, 810, 481, 1, 810, 2, 3, 4, 811, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 812, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 790, 1, 2, 1, 1, 4, 3, 1, 4, 5, 813, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 814, 1, 2, 1, 1, 4, 3, 1, 4, 5, 815,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 816, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 817, 1, 2, 1, 1, 4, 3, 1, 4, 5, 818, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 791, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 819, 1, 2, 1, 1, 4, 3, 1, 4, 5, 820, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 821, 1, 2, 1, 1, 4, 3, 1, 4, 5, 822,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 823, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 824, 1, 2, 1, 1, 4, 3, 1, 4, 5, 825, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 826, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 827, 1, 2, 1, 1, 4, 3, 1, 4, 5, 828, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 829, 1, 2, 1, 1, 4, 3, 1, 4, 5, 830,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 0, 1, 2, 1, 0,
    1, 0, 1, 2, 1, 1, 2, 1, 0, 0, 0, 0, 0, 0, 0, 0,
    4294967295, 1, 0, 837, 1, 2, 1, 1, 4, 3, 1, 4, 5, 506, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 589, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 712, 1, 2, 1, 1, 4, 3, 1, 4, 5, 727, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 861, 1, 2, 1, 1, 4, 3, 1, 4, 5, 862,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 863, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 864, 1, 2, 1, 1, 4, 3, 1, 4, 5, 865, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 866, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 101, 1, 2, 1, 1, 4, 3, 1, 4, 5, 874, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 875, 1, 2, 1, 1, 4, 3, 1, 4, 5, 876,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 877, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 878, 1, 2, 1, 1, 4, 3, 1, 4, 5, 879, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 880, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 881, 1, 2, 1, 1, 4, 3, 1, 4, 5, 882, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 883, 1, 2, 1, 1, 4, 3, 1, 4, 5, 885,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 886, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 887, 1, 2, 1, 1, 4, 3, 1, 4, 5, 673, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 888, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 889, 1, 2, 1, 1, 4, 3, 1, 4, 5, 890, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 891, 1, 2, 1, 1, 4, 3, 1, 4, 5, 631,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 892, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 893, 1, 2, 1, 1, 4, 3, 1, 4, 5, 894, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 895, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 896, 1, 2, 1, 1, 4, 3, 1, 4, 5, 831, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 897, 1, 2, 1, 1, 4, 3, 1, 4, 5, 898,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 311, 303, 230, 1, 303, 3, 3,
    4, 231, 899, 1, 2, 1, 1, 4, 3, 1, 4, 5, 900, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 14, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    486, 901, 230, 1, 901, 3, 3, 4, 231, 769, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 3, 4, 1, 271, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1, 0, 905, 1, 2, 1, 1, 4, 3, 1, 4, 5, 906, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 907, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 908, 1, 2, 1, 1, 4, 3, 1, 4, 5, 909, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 910, 1, 2, 1, 1, 4, 3, 1, 4, 5, 729,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 911, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 912, 1, 2, 1, 1, 4, 3, 1, 4, 5, 440, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 913, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 1, 1, 918, 1, 2, 1, 1, 4, 3, 1, 4, 5, 919, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 920, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 921, 1, 2, 1, 1, 4, 3, 1, 4, 5, 922, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 923, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    924, 925, 230, 1, 925, 3, 3, 4, 231, 442, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 926, 1, 2, 1, 1, 4, 3, 1, 4, 5, 927, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 928, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 930, 1, 2, 1, 1, 4, 3, 1, 4, 5, 931, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 932, 1, 2, 1, 1, 4, 3, 1, 4, 5, 933,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 934, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 935, 1, 2, 1, 1, 4, 3, 1, 4, 5, 936, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 632, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 584, 1, 2, 1, 1, 4, 3, 1, 4, 5, 937, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 938, 1, 2, 1, 1, 4, 3, 1, 4, 5, 102,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 134, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 939, 1, 2, 1, 1, 4, 3, 1, 4, 5, 940, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 941, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 942, 1, 2, 1, 1, 4, 3, 1, 4, 5, 943, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 0, 0, 4294967295, 4294967295, 0, 0, 0, 0, 0, 4294967295,
    4294967295, 0, 0, 1, 2, 1, 0, 0, 4294967295, 4294967295, 0, 0, 0, 1, 0, 0,
    1, 2, 1, 0, 1, 0, 1, 2, 1, 0, 0, 0, 633, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1001, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1002, 1, 2, 1, 1, 4, 3, 1, 4, 5, 15, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1003, 1, 2, 1, 1, 4, 3, 1, 4, 5, 406, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 406, 1004, 1005, 1, 1004, 5, 3, 1,
    807, 4, 5, 1006, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1007, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 0, 0, 0, 0, 1008, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1014, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1015,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 16, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 4294967295, 4294967295, 0, 1, 4294967295, 4294967295, 0, 1, 0, 0, 4294967295, 4294967295,
    0, 1021, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 1, 1022, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1025, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 287, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1026, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1027, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1028, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1029, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1030, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1031, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 610, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1032, 1, 2, 1, 1, 4, 3, 1, 4, 5, 17, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 177, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1035, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1036, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1037, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0,
    0, 0, 0, 1047, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1048, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1049, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1050, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 0, 1, 2,
    1, 0, 0, 0, 4294967295, 4294967295, 0, 1, 2, 1, 0, 1, 2, 1, 0, 220,
    1093, 1, 0, 0, 4294967295, 4294967295, 0, 236, 1093, 1, 1, 2, 1, 0, 91, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 565, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1094, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 1, 0,
    0, 0, 0, 0, 4294967295, 4294967295, 0, 1, 2, 1, 18, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 5, 749, 1111, 2, 749, 5, 3, 3, 4, 5, 0, 4294967295,
    4294967295, 0, 0, 0, 0, 0, 1, 0, 1, 2, 1, 0, 0, 67, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1123, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 19, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1124, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1125, 1, 2, 1, 1, 4, 3, 1, 4, 5, 634,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1126, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1127, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1128, 1128, 555,
    3, 1128, 281, 1128, 3, 3, 281, 4, 840, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1129, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1130, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 189, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 0, 0, 1142, 1, 2, 1, 1, 4, 3, 1, 4, 5, 133, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1146, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1147, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 0, 0, 0,
    197, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 1, 0, 1, 0,
    1149, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 0, 1, 2, 1,
    1157, 1, 2, 1, 1, 4, 3, 1, 4, 5, 833, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1158, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2,
    1, 0, 0, 1053, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1159, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1160, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1161, 1, 2, 1, 1, 4, 3, 1, 4, 5, 666, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1162, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1163,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1164, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1165, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1166, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1167, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 792, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1168, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1169, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1170,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1171, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1172, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1173, 1174, 9,
    3, 3, 1173, 4, 591, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1175, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1176, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 614, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1108, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 954, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 725, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1182, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1183, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1184,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1185, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 103, 1, 2, 1, 1, 4, 3, 1, 4, 5, 62, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1, 0, 0, 1, 2, 1, 0, 635, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 585, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1186, 1, 2, 1, 1, 4, 3, 1, 4, 5, 249, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 0, 532, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1189, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1190, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1191, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1192, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1193, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1194, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1195, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1196, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1197, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1198, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 713, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    566, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1199, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1200, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1201, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 104, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 0, 1207, 1, 2, 1, 1, 4, 3, 1, 4, 5, 603, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1208, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 706, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1209, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1210, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1211,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1212, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1213, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1214, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1215, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 20, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1216, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1217, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1054,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1218, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 0, 0, 372, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1205, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1219, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 0, 386, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1224,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1225, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1226, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 1237,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1238, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1239, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1240, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 0, 4294967295, 4294967295, 0, 220, 1245, 1, 0, 236,
    1245, 1, 0, 1243, 1, 2, 1, 1, 4, 3, 1, 4, 5, 434, 1254, 1,
    1, 2, 1, 0, 236, 1254, 1, 0, 0, 1, 0, 1, 2, 1, 1261, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1262, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1233, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1034, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1263, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1264, 1, 2, 1, 1, 4, 3, 1, 4, 5, 687, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1, 2, 9, 4, 3, 1, 4, 5, 1265, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1266, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 0, 1287, 1, 2, 1, 1, 4, 3, 1, 4, 5, 21, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 92, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1288, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1289, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1075, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1290, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1291, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1292, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1097, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1293, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1294, 1, 2, 1, 1, 4, 3, 1, 4, 5, 567, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1295, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1296, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 636, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1297, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 698, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 388, 388, 555, 3, 388, 281, 388, 3, 3,
    281, 4, 0, 1, 2, 1, 0, 0, 0, 0, 1055, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1307, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1308, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1309, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1310, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 807, 1005, 9,
    5, 3, 1, 807, 4, 5, 807, 808, 9, 3, 3, 807, 4, 1318, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1319, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1320, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1321, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1322, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1323,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1324, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1325, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1326, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1327, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 0, 1334, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1335, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1333, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1336, 1, 2, 1, 1, 4, 3, 1, 4, 5, 845, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1337, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1338,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1339, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1340, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1341, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1342, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 667, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1343, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1344, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1345,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 1, 2, 3, 1, 1347,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 105, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1348, 1, 2, 1, 1, 4, 3, 1, 4, 5, 22, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1349, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1350, 1, 2, 1, 1, 4, 3, 1, 4, 5, 23, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1351, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1352,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1353, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 250, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1354, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1355, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1356, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1357, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1358, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1016,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 4294967295, 4294967295, 0, 0, 0, 1,
    2, 1, 0, 220, 1245, 1, 236, 1245, 1, 1379, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1380, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1381, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 604, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1267, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1268, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1382, 1, 2, 1, 1, 4, 3, 1, 4, 5, 793,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1383, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1384, 1, 2, 1, 1, 4, 3, 1, 4, 5, 637, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1056, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 310, 303, 230, 1, 303, 3, 3, 4, 231, 794, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1385, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1386, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1269, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 251, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1089, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1387, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1388, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1389, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 689, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1390, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 106, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1391, 1, 2, 1, 1, 4, 3, 1, 4, 5, 307, 303, 230, 1,
    303, 3, 3, 4, 231, 40, 1, 2, 1, 1, 4, 3, 1, 4, 5, 795,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1392, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1270, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1393, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1394, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1395, 1, 2, 1, 1, 4, 3, 1, 4, 5, 308, 303, 230, 1, 303,
    3, 3, 4, 231, 1396, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1397, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1398, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2, 3, 1399, 1174, 1, 1399, 3, 3, 1173, 4, 1400, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1, 2, 1, 0, 1, 0, 1407, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1221, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1221, 1408, 1005, 1, 1408, 5, 3, 1, 807, 4, 5, 1409, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1410, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1411,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 310, 1412, 230, 1, 1412, 3, 3,
    4, 231, 1413, 1, 2, 1, 1, 4, 3, 1, 4, 5, 721, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1414, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1415, 1416, 12, 3, 3, 1415, 4, 1010, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1417, 1, 2, 1, 1, 4, 3, 1, 4, 5, 252, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1418, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1419,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 253, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1420, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1421, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1422, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1423, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1424, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 254, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1425,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1426, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1427, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1428, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1429, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1430, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 1, 1, 2,
    1, 1, 2, 1, 0, 0, 1, 2, 1, 0, 0, 1438, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1439, 1, 2, 1, 1, 4, 3, 1, 4, 5, 255,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1440, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1441, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1442, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 0, 501, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 501, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1444, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1445, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1446, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1447, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 502, 1, 2, 1, 1, 4, 3, 1, 4, 5, 502, 1408,
    1005, 1, 1408, 5, 3, 1, 807, 4, 5, 1448, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1449, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1181, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1450, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1451, 1, 2, 1, 1, 4, 3, 1, 4, 5, 97, 97, 808, 3, 97,
    807, 97, 3, 3, 807, 4, 1452, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1453, 1, 2, 1, 1, 4, 3, 1, 4, 5, 95, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1, 0, 1316, 1457, 230, 1, 1457, 3, 3, 4, 231, 1,
    2, 0, 0, 0, 1461, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1462, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1463, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 710, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1464, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1465, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 0, 1057, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1466, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1467, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    717, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1468, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 164, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1469, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 647, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1470, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1471, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1472, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1473, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1474, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 107, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1475, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1476, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1477, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1478, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1139, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1479, 1, 2, 1, 1, 4, 3, 1, 4, 5, 690, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 387, 1, 2, 1, 1, 4, 3, 1, 4, 5, 387, 303,
    304, 1, 303, 3, 3, 303, 4, 0, 0, 0, 1, 0, 0, 218, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 0, 1484, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1485, 1, 2, 1, 1, 4, 3, 1, 4, 5, 846, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1486, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1487, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1260, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1488, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1489, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 1, 0, 1491, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 3, 4, 1, 0, 630, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1492, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1493, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1203, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1494, 1, 2, 1, 1, 4, 3, 1, 4, 5, 108, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1495, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    533, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1496, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1497, 1, 2, 1, 1, 4, 3, 1, 4, 5, 638, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1498, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1499, 1, 2, 1, 1, 4, 3, 1, 4, 5, 4, 4294967295, 1500, 481,
    1, 1500, 2, 3, 4, 1501, 1500, 230, 1, 1500, 3, 3, 4, 231, 1502, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1503, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1504, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1505, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1506, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1507, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1508, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1509, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1510, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 1, 2, 1, 0, 1511, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 639, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1513, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1514, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1515, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1516, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1517, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 5, 1519, 1111, 2, 1519, 5, 3, 3, 4, 5, 1, 0,
    0, 640, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1520, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1521, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1522,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1523, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1524, 1, 2, 1, 1, 4, 3, 1, 4, 5, 137, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 315, 1525, 230, 1, 1525, 3, 3, 4, 231,
    1526, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1527, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1528, 1, 2, 1, 1, 4, 3, 1, 4, 5, 309, 303,
    230, 1, 303, 3, 3, 4, 231, 128, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1529, 1, 2, 1, 1, 4, 3, 1, 4, 5, 309, 1412, 230, 1, 1412,
    3, 3, 4, 231, 0, 1535, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1536,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1043, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 772, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1538, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1539, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1540, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1541, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 849, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1542, 1, 2, 1, 1, 4, 3, 1, 4, 5, 509, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1544, 1, 2, 1, 1, 4, 3, 1, 4, 5, 966, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1545, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 142, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1546, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1547, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    24, 1, 2, 1, 1, 4, 3, 1, 4, 5, 68, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1548, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1549, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1550, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1551, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1552, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 202, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1553, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1058, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 0, 1, 2, 1, 796, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1556, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1557, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1558, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1112, 1559, 1174, 3, 1559, 1173, 1112, 3, 3, 1173, 4, 235, 1560, 808, 3, 1560,
    807, 235, 3, 3, 807, 4, 1561, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1562, 1, 2, 1, 1, 4, 3, 1, 4, 5, 834, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1038, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1563, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 280, 1564, 230, 1, 1564, 3, 3, 4,
    231, 313, 1564, 230, 1, 1564, 3, 3, 4, 231, 1565, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 312, 1564, 230, 1, 1564, 3, 3, 4, 231, 385, 1564, 230,
    1, 1564, 3, 3, 4, 231, 109, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 1566, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1567, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1568, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1569,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1570, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1571, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1572, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1573, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 4294967295, 1, 0, 0, 111, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    110, 1, 2, 1, 1, 4, 3, 1, 4, 5, 714, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 0, 1582, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1583,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1584, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1585, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1586, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1587, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1588, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1589, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1590, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1591,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1592, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1593, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1594, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1595, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1596, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1597, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 293, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1598,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1599, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 857, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1600, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1601, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1602, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1603, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 25, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1109,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1604, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1605, 1, 2, 1, 1, 4, 3, 1, 4, 5, 277, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1606, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1607, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1608, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1609, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1610,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1611, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1612, 1, 2, 1, 1, 4, 3, 1, 4, 5, 619, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 660, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 190, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1402, 1614, 230, 1,
    1614, 3, 3, 4, 231, 568, 1, 2, 1, 1, 4, 3, 1, 4, 5, 955,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 26, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1615, 1, 2, 1, 1, 4, 3, 1, 4, 5, 855, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1616, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 750, 749, 481, 1, 749, 2, 3, 4, 750, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1617, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1618, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1619, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1620, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1621, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1622, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1623,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1624, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 27, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1625, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1626, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1627, 1, 2, 1, 1, 4, 3, 1, 4, 5, 69, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 719, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1629,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1630, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1631, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1632, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1633, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1634, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1635, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 852, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1636,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1227, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1637, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1228, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1638, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1639, 1, 2, 1, 1, 4, 3, 1, 4, 5, 674, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 1, 2, 1, 0, 1, 2, 1, 1645, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1646, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1647, 1, 2, 1, 1, 4, 3, 1, 4, 5, 662, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 28, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1648,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 60, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1649, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1650, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1651, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1652, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1653, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1654, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1655,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 797, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1656, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1271, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1657, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1658, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1659, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 0, 1, 0, 2, 9246, 2, 9248, 3, 386, 895,
    386, 501, 895, 0, 112, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2,
    1, 0, 1, 2, 1, 0, 1, 2, 1, 0, 1677, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 873, 1678, 230, 1, 1678, 3, 3, 4, 231, 1679, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1105, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 29, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 70, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1682, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 53, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1683, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1684, 1, 2, 1, 1, 4, 3, 1, 4, 5, 367,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 0, 1, 4294967295, 4294967295, 0,
    1017, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 1691, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1156, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1692, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1329, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1693, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1,
    4294967295, 4294967295, 0, 1694, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1695, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1696, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 1699, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1577, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 1, 0, 235, 1559, 1174, 3, 1559, 1173, 235,
    3, 3, 1173, 4, 1700, 1, 2, 1, 1, 4, 3, 1, 4, 5, 311, 1701,
    230, 1, 1701, 3, 3, 4, 231, 329, 1702, 230, 1, 1702, 3, 3, 4, 231,
    1703, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1704, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1705, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1554, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1706, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1707, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1708, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1709, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 0, 0, 0, 0, 602, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1762,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 641, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1763, 1, 2, 1, 1, 4, 3, 1, 4, 5, 665, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1764, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1765, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1766, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1767, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1768,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1718, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1769, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1770, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1771, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1772, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1773, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1774, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1775,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1776, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1777, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1778, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1779, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1780, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1781, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1782, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1783,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1784, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1785, 1, 2, 1, 1, 4, 3, 1, 4, 5, 30, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 71, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1786, 1, 2, 1, 1, 4, 3, 1, 4, 5, 308, 1787, 230, 1, 1787,
    3, 3, 4, 231, 692, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1788, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1789, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1790, 1, 2, 1, 1, 4, 3, 1, 4, 5, 730, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 605, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 680, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1864, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1865, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1866,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1867, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1868, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1869, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1870, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1871, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1872, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1873, 1, 2, 1, 1, 4, 3, 1, 4, 5, 31,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 72, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1874, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1876, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 1, 2, 1, 1877, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1878, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1879, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1880, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1881, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1882,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1883, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1884, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1885, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1886, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 231, 230, 9, 3, 3, 4, 231, 668, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1887, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1888, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1889, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1890, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1891, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1892, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1641, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1759, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 368, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 2, 1,
    1, 2, 1, 0, 32, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1894, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 73, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1082, 1, 2, 1, 1, 4, 3, 1, 4, 5, 33, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 4294967295, 4294967295, 0, 0, 4294967295, 4294967295, 0, 34, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 74, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 113, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1898, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1719, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 114, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1900, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1901, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1902, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1903, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1904, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1366, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1013, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 967, 1905, 230, 1, 1905, 3, 3, 4, 231, 767, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 0, 0, 4294967295, 4294967295, 0, 1744, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 768, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1720,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1909, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1910, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1911, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1912, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1913, 1, 2, 1, 1, 4, 3, 1, 4, 5, 89, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1914, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1915,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1916, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1098, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1917, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1918, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 0, 1925, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1926, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1927, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1928, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1730, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1929, 1, 2, 1, 1, 4, 3, 1, 4, 5, 697,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1930, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1931, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1932, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 115, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1933, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1934, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1935, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1936,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 407, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 407, 1004, 1005, 1, 1004, 5, 3, 1, 807, 4, 5, 116,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1940, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1941, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 1,
    2, 1, 35, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1747, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1943, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1944, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1042, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1945, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1950,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1951, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1952, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1059, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 513, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1954, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1955, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1956, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1957, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1958, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1, 4294967295, 4294967295, 0, 675, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 782, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1959, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 117, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1960, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1961, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1229, 1, 2, 1, 1, 4, 3, 1, 4, 5, 118, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 119, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 120, 1, 2, 1, 1, 4, 3, 1, 4, 5, 123, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1962, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1963, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1964, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1965, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1966, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 653, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1967, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1968, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1969, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1970, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1971, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1972, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1973, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1974, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1975, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 0, 1,
    2, 1, 0, 1, 0, 414, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1976,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1977, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1978, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1979, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1980, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1981, 1, 2, 1, 1, 4, 3, 1, 4, 5, 534, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1982, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1983,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 775, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1984, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1985, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1986, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 36, 1, 2, 1, 1, 4, 3, 1, 4, 5, 75, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1989, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1990, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1991, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 37, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1992, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 798, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 799, 1, 2, 1, 1, 4, 3, 1, 4, 5, 220, 1993, 1, 434,
    1994, 1, 236, 1994, 1, 236, 1993, 1, 436, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1995, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 1, 0,
    0, 870, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1996, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1997, 1998, 12, 3, 3, 4, 1997, 1999, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 0, 4294967295, 4294967295, 0, 1060, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 220, 2005, 1, 1, 0, 236, 2005, 1, 220, 2005,
    1, 1, 0, 236, 2005, 1, 425, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1, 2, 1, 1533, 1, 2, 1, 1, 4, 3, 1, 4, 5, 628, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2011, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 5, 1111, 9, 3, 3, 4, 5, 1920, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 57, 1, 2, 1, 1, 4, 3, 1, 4, 5, 121, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2012, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2013, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 2018, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 2022, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2023, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2024, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1737, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2016, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2025, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2026, 1, 2, 1, 1, 4, 3, 1, 4, 5, 800, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2027, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1272, 1, 2, 1, 1, 4, 3, 1, 4, 5, 682, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2028, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1907, 1,
    2, 1, 1, 4, 3, 1, 4, 5,
];

pub static AUX_DATA_OFFSETS: &[u32] = &[
    0, 10, 20, 30, 46, 9, 0, 42, 0, 38, 39, 41, 42, 43, 44, 45,
    46, 47, 48, 49, 50, 0, 51, 40, 54, 0, 55, 56, 0, 58, 59, 61,
    42, 0, 63, 59, 61, 42, 64, 0, 58, 66, 61, 46, 0, 63, 66, 61,
    41, 42, 46, 44, 43, 77, 0, 79, 81, 84, 86, 88, 90, 0, 93, 94,
    96, 52, 0, 122, 47, 48, 124, 41, 42, 44, 0, 125, 126, 127, 0, 129,
    130, 0, 131, 132, 0, 39, 48, 135, 136, 0, 51, 0, 122, 49, 46, 0,
    126, 127, 138, 64, 0, 139, 50, 46, 47, 0, 140, 141, 125, 143, 122, 144,
    129, 145, 146, 39, 147, 148, 149, 150, 151, 152, 153, 154, 155, 156, 157, 136,
    158, 159, 0, 58, 141, 160, 53, 161, 163, 55, 165, 166, 167, 168, 169, 170,
    171, 172, 130, 46, 0, 126, 127, 0, 173, 166, 0, 174, 165, 0, 175, 176,
    56, 178, 179, 180, 181, 182, 183, 184, 185, 186, 59, 0, 126, 187, 0, 173,
    179, 0, 174, 178, 0, 175, 188, 57, 191, 193, 63, 59, 58, 0, 124, 195,
    0, 198, 77, 0, 48, 199, 201, 199, 61, 203, 64, 206, 68, 78, 88, 98,
    108, 118, 128, 138, 148, 158, 217, 0, 220, 223, 189, 0, 226, 186, 227, 168,
    177, 234, 236, 190, 0, 239, 241, 200, 210, 220, 230, 240, 252, 0, 256, 257,
    96, 259, 260, 250, 0, 261, 96, 262, 251, 263, 264, 265, 0, 266, 171, 267,
    253, 269, 0, 272, 273, 226, 275, 262, 0, 278, 279, 257, 282, 283, 0, 258,
    284, 146, 273, 285, 260, 0, 259, 288, 289, 290, 160, 292, 0, 294, 261, 295,
    88, 239, 0, 266, 165, 239, 263, 296, 265, 275, 284, 294, 304, 314, 323, 333,
    343, 353, 316, 317, 318, 319, 320, 321, 322, 323, 324, 325, 375, 355, 354, 96,
    326, 328, 357, 366, 0, 378, 390, 388, 334, 0, 0, 295, 393, 403, 413, 423,
    433, 443, 453, 463, 473, 483, 493, 503, 513, 523, 533, 0, 350, 543, 553, 563,
    573, 583, 593, 603, 357, 613, 282, 369, 371, 203, 290, 326, 0, 374, 282, 86,
    295, 614, 620, 625, 635, 645, 655, 665, 675, 685, 389, 390, 391, 392, 686, 693,
    703, 713, 723, 397, 0, 398, 398, 399, 0, 400, 400, 733, 742, 752, 762, 403,
    404, 763, 773, 408, 409, 774, 786, 784, 282, 791, 789, 290, 794, 800, 413, 803,
    813, 823, 833, 843, 853, 863, 873, 227, 0, 226, 422, 295, 876, 424, 879, 889,
    0, 427, 899, 203, 79, 900, 910, 920, 930, 940, 958, 0, 433, 434, 950, 437,
    439, 441, 443, 441, 441, 0, 433, 326, 954, 437, 439, 441, 443, 441, 441, 392,
    86, 445, 959, 969, 979, 988, 998, 1008, 1018, 1028, 1038, 1048, 0, 1058, 1068, 1078,
    1088, 1098, 1108, 1118, 1128, 1138, 1148, 1158, 1168, 1178, 1188, 1198, 1208, 1218, 1228, 1238,
    1248, 1258, 1268, 1278, 1288, 1298, 1308, 1318, 0, 1324, 1334, 1346, 484, 1344, 226, 485,
    487, 1347, 1357, 1367, 1377, 1387, 1397, 1407, 1416, 1425, 1434, 497, 282, 203, 369, 0,
    0, 500, 77, 1438, 0, 503, 504, 0, 505, 1435, 326, 0, 507, 508, 0, 510,
    511, 512, 514, 0, 515, 511, 516, 0, 519, 520, 522, 0, 226, 374, 1439, 0,
    1442, 523, 0, 1446, 1456, 226, 485, 487, 1458, 0, 526, 0, 0, 1468, 1478, 1487,
    1497, 0, 531, 484, 1507, 0, 531, 163, 1524, 0, 1518, 0, 273, 290, 0, 535,
    536, 0, 46, 538, 0, 1517, 540, 168, 541, 169, 48, 171, 165, 49, 155, 50,
    542, 166, 77, 543, 484, 0, 239, 1521, 1519, 0, 226, 543, 484, 1525, 1532, 1542,
    1552, 279, 548, 549, 550, 551, 552, 553, 554, 1553, 1560, 1570, 1576, 1582, 1592, 1602,
    0, 399, 399, 1612, 1622, 0, 1632, 564, 569, 570, 1633, 1643, 1653, 1663, 1673, 1683,
    1693, 1703, 1713, 1723, 1733, 1743, 1753, 1763, 1773, 1785, 0, 1783, 586, 587, 273, 588,
    0, 1784, 586, 587, 590, 0, 273, 592, 1789, 1799, 1809, 1819, 1829, 1839, 0, 122,
    599, 1849, 374, 295, 374, 600, 601, 1853, 170, 168, 169, 0, 1852, 606, 607, 608,
    609, 79, 81, 0, 611, 77, 0, 295, 612, 0, 122, 42, 613, 615, 0, 122,
    617, 618, 168, 613, 79, 81, 0, 620, 374, 621, 0, 122, 623, 535, 536, 163,
    239, 0, 1857, 1855, 1854, 624, 0, 626, 627, 629, 1881, 0, 503, 642, 0, 61,
    643, 0, 645, 646, 0, 648, 1863, 1861, 1860, 226, 326, 1875, 649, 650, 0, 651,
    652, 1874, 1871, 536, 1868, 1866, 334, 0, 0, 654, 96, 0, 655, 654, 96, 0,
    656, 96, 0, 1876, 657, 658, 659, 661, 0, 663, 664, 1877, 669, 670, 671, 0,
    672, 0, 1878, 676, 677, 678, 679, 0, 681, 1879, 683, 684, 0, 1880, 648, 685,
    504, 686, 1885, 0, 122, 617, 688, 613, 77, 0, 691, 1903, 1901, 0, 278, 279,
    1900, 282, 1897, 1895, 484, 295, 424, 0, 122, 48, 613, 199, 0, 122, 46, 613,
    538, 1908, 0, 256, 257, 96, 0, 693, 600, 96, 259, 260, 1906, 0, 261, 96,
    262, 1907, 263, 264, 265, 0, 122, 171, 267, 1909, 694, 0, 122, 617, 696, 165,
    613, 239, 1923, 0, 1916, 1914, 1913, 699, 285, 516, 0, 295, 1922, 1919, 424, 0,
    701, 239, 702, 0, 122, 155, 704, 1924, 707, 289, 0, 708, 709, 1928, 1927, 145,
    144, 0, 157, 709, 1925, 290, 711, 163, 711, 163, 163, 239, 1936, 0, 715, 716,
    718, 295, 278, 720, 718, 295, 278, 0, 90, 239, 1931, 0, 503, 722, 0, 722,
    1930, 0, 723, 724, 0, 723, 726, 0, 611, 728, 1932, 718, 731, 0, 732, 1933,
    326, 0, 129, 169, 733, 734, 0, 122, 124, 199, 0, 122, 617, 166, 613, 239,
    160, 295, 1943, 1940, 424, 424, 736, 1944, 738, 740, 1945, 1955, 1965, 1975, 1985, 1995,
    2005, 2013, 0, 2023, 751, 752, 2024, 753, 2027, 2037, 2047, 2057, 2067, 2077, 2087, 2097,
    2107, 2117, 2127, 2137, 2147, 2157, 2167, 2177, 2187, 2197, 2207, 2217, 2227, 2237, 2256, 2247,
    226, 283, 2264, 766, 0, 770, 771, 0, 2259, 0, 2257, 773, 774, 2258, 776, 0,
    93, 94, 0, 0, 226, 777, 778, 779, 2260, 781, 783, 784, 2265, 2276, 788, 0,
    2288, 2286, 2285, 789, 801, 0, 2291, 802, 803, 804, 805, 2292, 2303, 2324, 2333, 2343,
    2353, 2363, 2373, 2383, 2393, 2403, 2413, 2423, 2433, 2443, 2453, 2463, 2473, 2483, 2493, 2503,
    2513, 2523, 2533, 2543, 0, 832, 2553, 257, 163, 2559, 0, 835, 2556, 2554, 290, 836,
    0, 838, 839, 154, 165, 239, 2569, 0, 2562, 2560, 285, 0, 295, 2568, 2565, 424,
    841, 843, 0, 701, 239, 843, 2575, 0, 847, 257, 2570, 718, 273, 2572, 0, 163,
    848, 2571, 144, 850, 851, 2574, 853, 0, 503, 853, 0, 163, 854, 2573, 144, 850,
    851, 0, 600, 856, 0, 61, 643, 858, 2579, 2589, 0, 507, 860, 2599, 2609, 2619,
    2629, 2639, 2649, 2659, 2669, 2679, 2689, 0, 144, 867, 869, 872, 2699, 2709, 2719, 2729,
    2739, 2749, 2759, 2769, 2779, 2789, 0, 884, 283, 77, 2799, 2809, 2819, 2829, 2839, 2849,
    2859, 2869, 2879, 2889, 2899, 2909, 2919, 2929, 2939, 2949, 2959, 2969, 2978, 2988, 2998, 3008,
    3017, 3027, 902, 903, 0, 272, 599, 3028, 374, 295, 374, 275, 0, 272, 42, 904,
    615, 275, 0, 272, 168, 904, 79, 0, 272, 535, 904, 536, 275, 3031, 0, 272,
    688, 904, 77, 0, 272, 48, 904, 226, 0, 272, 46, 904, 538, 275, 0, 272,
    171, 267, 0, 272, 618, 165, 904, 3041, 239, 275, 0, 272, 155, 904, 704, 0,
    272, 50, 904, 77, 275, 0, 272, 124, 904, 199, 0, 272, 166, 904, 239, 275,
    3043, 3053, 3063, 3073, 3083, 3093, 3103, 3113, 3123, 0, 439, 326, 3133, 3143, 3153, 0,
    914, 916, 3156, 3166, 3176, 3186, 3196, 3206, 3216, 3225, 3235, 3245, 3255, 929, 3265, 3275,
    3285, 3295, 3305, 3315, 3325, 3335, 3345, 3355, 3365, 3375, 3385, 3395, 3405, 3415, 3425, 3435,
    0, 944, 3445, 424, 945, 947, 949, 951, 0, 952, 3447, 3446, 953, 0, 445, 952,
    956, 957, 0, 543, 503, 958, 959, 960, 961, 0, 503, 962, 963, 964, 952, 0,
    965, 951, 3451, 968, 0, 969, 970, 971, 972, 0, 973, 3454, 3453, 0, 3452, 389,
    390, 392, 973, 3458, 968, 0, 969, 970, 3467, 0, 974, 3459, 424, 0, 3462, 975,
    976, 3463, 978, 497, 980, 0, 3482, 369, 203, 3472, 3471, 981, 326, 424, 3469, 3468,
    282, 290, 203, 982, 326, 984, 985, 0, 3475, 914, 986, 3478, 3476, 326, 0, 374,
    3481, 484, 96, 988, 989, 990, 982, 991, 86, 992, 3483, 993, 282, 371, 995, 997,
    290, 999, 3484, 3494, 0, 3504, 3514, 3524, 3534, 3544, 3555, 3565, 0, 1009, 3575, 288,
    1011, 3576, 96, 326, 718, 3577, 718, 851, 3578, 646, 0, 600, 1012, 3579, 3589, 3599,
    3609, 0, 3619, 1018, 3627, 1019, 3623, 1018, 3629, 1018, 3633, 0, 1023, 3643, 0, 1024,
    326, 3646, 3656, 3666, 3676, 3686, 3696, 3706, 3716, 3726, 3736, 3746, 0, 51, 1033, 3756,
    3766, 3776, 3786, 3796, 0, 1039, 3809, 3808, 0, 1040, 718, 0, 3806, 96, 371, 1041,
    290, 3807, 1044, 1045, 0, 962, 964, 3810, 851, 1046, 3811, 3821, 3831, 3841, 0, 90,
    3851, 1051, 3853, 1024, 1061, 0, 198, 257, 0, 1063, 257, 1064, 1065, 1066, 1067, 736,
    841, 1009, 1069, 1069, 1070, 1072, 738, 740, 3866, 0, 1074, 3857, 3854, 1076, 1078, 241,
    3858, 1080, 1081, 1084, 1085, 0, 600, 1069, 241, 1081, 1087, 3863, 0, 3859, 1085, 3885,
    3882, 0, 3870, 1088, 1091, 3867, 1051, 0, 1063, 257, 0, 198, 257, 3871, 3875, 0,
    3874, 1085, 1084, 0, 1063, 257, 0, 198, 257, 3879, 3886, 3896, 3906, 3916, 326, 1095,
    3923, 326, 0, 326, 295, 374, 0, 1099, 326, 90, 90, 326, 0, 3917, 789, 326,
    90, 326, 0, 1100, 3920, 1101, 3918, 1102, 1103, 0, 1106, 282, 0, 3921, 484, 226,
    290, 61, 3922, 484, 226, 290, 1107, 624, 1110, 0, 3927, 801, 0, 295, 484, 0,
    45, 283, 3930, 3940, 0, 3950, 0, 226, 777, 412, 3954, 733, 1113, 412, 3957, 0,
    278, 3955, 0, 1114, 50, 374, 3956, 0, 1114, 50, 374, 3960, 3958, 1115, 3964, 0,
    1116, 1117, 1118, 0, 0, 79, 3963, 1116, 1119, 1120, 734, 1122, 3965, 3975, 3985, 3995,
    4005, 4015, 4025, 4035, 4045, 4056, 4066, 4076, 4086, 0, 1131, 4096, 0, 1132, 163, 1134,
    0, 48, 199, 0, 148, 163, 124, 195, 77, 88, 1136, 4097, 0, 600, 1131, 1138,
    0, 600, 1140, 1138, 0, 1141, 1040, 132, 0, 1143, 257, 4098, 1144, 1145, 195, 4099,
    4109, 4119, 0, 400, 929, 400, 4129, 0, 198, 257, 4142, 58, 1148, 1151, 0, 4140,
    4139, 290, 96, 326, 1153, 4143, 0, 1063, 257, 1061, 1064, 1065, 1066, 1009, 1069, 1070,
    1072, 738, 740, 4144, 0, 4158, 4157, 290, 96, 326, 4154, 1153, 4160, 0, 1154, 1155,
    4173, 4171, 4170, 1151, 753, 4176, 4186, 4196, 0, 1064, 4209, 4206, 326, 4210, 0, 1063,
    257, 1061, 1065, 1066, 1009, 1069, 1070, 1072, 738, 740, 4211, 4221, 4231, 4241, 4251, 4261,
    4271, 4281, 4291, 0, 4301, 4311, 4321, 4331, 4341, 4351, 4361, 4371, 4381, 4388, 4398, 4408,
    0, 1177, 974, 4418, 0, 1177, 503, 974, 4428, 0, 4438, 1178, 1179, 1180, 4439, 4449,
    4459, 4469, 4479, 4489, 4499, 4509, 0, 503, 61, 4525, 4522, 4521, 4519, 951, 424, 203,
    4526, 4536, 4546, 4556, 0, 4566, 226, 326, 4567, 1187, 1188, 836, 4568, 4578, 4588, 4598,
    4608, 4618, 4628, 4638, 4648, 4658, 4668, 4678, 4688, 4698, 4708, 4718, 4728, 0, 145, 1202,
    1204, 239, 4738, 424, 0, 160, 424, 1206, 4739, 4749, 4759, 4769, 4779, 4789, 4799, 4809,
    4819, 4829, 4839, 4849, 4859, 4869, 4879, 4889, 0, 805, 4901, 0, 96, 4899, 484, 789,
    484, 0, 484, 4900, 484, 789, 4902, 4912, 4922, 4932, 989, 1220, 1222, 988, 1223, 0,
    543, 503, 1100, 90, 4933, 0, 4943, 4953, 4963, 0, 600, 4974, 0, 4973, 1230, 1231,
    1232, 1235, 1236, 4975, 4985, 4995, 5005, 5026, 0, 1241, 1242, 1091, 5015, 1244, 0, 5019,
    1246, 5022, 1248, 1250, 5023, 0, 1251, 1242, 1091, 1246, 0, 1252, 1091, 5027, 0, 1253,
    0, 0, 163, 5037, 5043, 5040, 326, 5044, 5048, 1255, 0, 5047, 1256, 1257, 1258, 1259,
    0, 652, 5051, 5049, 226, 5054, 5064, 5074, 5084, 5094, 5104, 5114, 5124, 5132, 5142, 0,
    5152, 282, 369, 203, 290, 975, 5153, 1273, 1274, 1275, 1276, 1277, 1278, 1279, 1280, 1281,
    1282, 1283, 1284, 1285, 1286, 5154, 5164, 5174, 5184, 5194, 0, 5204, 5214, 5224, 5234, 5244,
    5254, 5264, 5274, 5284, 5294, 0, 398, 929, 398, 5304, 5314, 0, 0, 503, 1298, 951,
    0, 1299, 952, 5324, 701, 1300, 1301, 5325, 5335, 5353, 0, 1302, 5350, 5347, 5346, 96,
    326, 58, 5352, 0, 1069, 5351, 96, 326, 0, 86, 96, 0, 1303, 1305, 5354, 5364,
    5374, 5384, 0, 5394, 5404, 484, 96, 988, 969, 989, 990, 1311, 1313, 1315, 5405, 5414,
    5421, 5431, 0, 5441, 5451, 5461, 5471, 5481, 5491, 5501, 5511, 5522, 0, 139, 964, 1328,
    5521, 1330, 157, 1331, 1332, 0, 1328, 157, 1332, 5523, 5533, 5543, 5553, 5563, 5573, 5583,
    5593, 5603, 5613, 5623, 5633, 5643, 5653, 5663, 0, 1346, 777, 226, 5676, 0, 688, 77,
    0, 86, 5673, 424, 5679, 5689, 5699, 0, 5709, 0, 5719, 5729, 5739, 5749, 5759, 5769,
    5779, 5789, 5799, 5809, 5819, 5829, 144, 0, 145, 160, 1120, 0, 1359, 1204, 5839, 5849,
    1360, 1362, 0, 732, 1364, 0, 964, 326, 1365, 5853, 1367, 1369, 0, 732, 503, 1364,
    0, 964, 326, 1365, 1371, 0, 1373, 5854, 239, 1374, 652, 5858, 239, 1374, 0, 5855,
    424, 241, 0, 203, 241, 5859, 1248, 5862, 1375, 0, 157, 1377, 5865, 5875, 5885, 5895,
    5905, 5915, 5925, 5935, 5945, 5955, 5965, 5975, 5985, 5994, 6004, 6014, 6024, 6034, 6044, 6054,
    6064, 6074, 6084, 6094, 6104, 6114, 6124, 6133, 6143, 6153, 6163, 6173, 6183, 6193, 6203, 6212,
    6222, 6232, 6242, 6252, 0, 283, 1401, 326, 0, 226, 295, 6265, 226, 6262, 1403, 0,
    1066, 6266, 1405, 6268, 6278, 6288, 6299, 0, 6309, 6319, 6329, 0, 6338, 6348, 6358, 6368,
    6375, 6385, 6395, 6405, 6415, 6425, 6435, 6445, 6455, 6465, 6475, 6485, 6495, 6505, 6515, 6525,
    6535, 6545, 6564, 1120, 1431, 6555, 993, 6558, 1432, 6561, 1434, 1435, 0, 6565, 947, 945,
    949, 1436, 6570, 413, 6569, 6566, 993, 1434, 6571, 6581, 6591, 6601, 0, 6611, 6621, 0,
    6631, 0, 1443, 929, 0, 398, 400, 6632, 0, 6642, 6652, 6662, 6672, 6682, 6692, 6702,
    6713, 6723, 6733, 6743, 6753, 6763, 6774, 6784, 6794, 0, 484, 1454, 334, 6815, 6804, 1455,
    6806, 6818, 777, 0, 777, 777, 6819, 163, 295, 652, 86, 1458, 1459, 1460, 445, 1454,
    1018, 1084, 6820, 6830, 6840, 6850, 6860, 6870, 0, 1069, 6880, 96, 326, 6881, 718, 851,
    6882, 6892, 6902, 6912, 6922, 6932, 6942, 6952, 6962, 6972, 6982, 6992, 7002, 7012, 7022, 7032,
    7042, 7052, 7062, 7072, 7082, 7092, 7102, 0, 711, 856, 0, 1063, 257, 7114, 0, 7111,
    290, 96, 326, 7112, 1187, 1188, 0, 1180, 7113, 288, 1480, 1481, 1061, 1039, 7116, 1061,
    1064, 1065, 1066, 1039, 1106, 1009, 1069, 1070, 1072, 738, 740, 7117, 7127, 1063, 1483, 7128,
    7138, 7148, 7158, 7168, 7178, 7188, 7198, 0, 1107, 1490, 7211, 226, 7208, 1403, 7212, 0,
    629, 7225, 0, 7222, 990, 86, 990, 290, 7226, 7236, 7246, 7256, 7266, 7276, 7286, 7296,
    7306, 7316, 7326, 7336, 7346, 7356, 7365, 7374, 7384, 7394, 7404, 7414, 7424, 7434, 7444, 7454,
    0, 1512, 257, 7469, 7466, 7464, 326, 7470, 7480, 7490, 7500, 7510, 7520, 7530, 7552, 0,
    1518, 7550, 7540, 1518, 484, 0, 7553, 7563, 7573, 7583, 7593, 7603, 7613, 7623, 7632, 7642,
    7652, 7662, 7671, 7681, 7691, 7700, 1530, 1532, 0, 7701, 7711, 7721, 1537, 7722, 7732, 7742,
    7752, 7762, 7772, 7782, 7792, 0, 1543, 1033, 7802, 7812, 7822, 7832, 7842, 7852, 7862, 7872,
    7882, 7892, 7902, 7912, 7922, 7932, 7942, 7952, 0, 968, 96, 7962, 0, 392, 7972, 58,
    1114, 1555, 7973, 424, 7976, 7986, 7996, 8006, 8016, 8027, 8038, 8048, 8058, 8068, 8078, 8088,
    8097, 8106, 8116, 8125, 8134, 0, 611, 8144, 0, 149, 646, 0, 1009, 646, 8145, 8155,
    8165, 8175, 8185, 8195, 8205, 8215, 0, 151, 239, 160, 1574, 8225, 0, 163, 144, 1575,
    0, 163, 157, 1575, 0, 1576, 8229, 1256, 1578, 1579, 8230, 8240, 8250, 8260, 902, 903,
    0, 1580, 504, 0, 684, 1581, 0, 1580, 507, 8261, 8271, 8281, 8291, 8301, 8311, 8321,
    8331, 8341, 8351, 8361, 8371, 8381, 8391, 8401, 8411, 8421, 8431, 8441, 8451, 8461, 8471, 8481,
    8491, 8501, 8511, 8521, 8531, 8541, 8551, 8561, 8571, 8581, 8591, 8601, 8611, 8621, 8631, 8641,
    0, 1613, 0, 1131, 1138, 611, 8651, 0, 1132, 163, 1134, 0, 48, 199, 0, 148,
    163, 124, 195, 195, 90, 1136, 275, 8652, 8661, 8671, 8681, 226, 8691, 8701, 8711, 8721,
    8729, 8739, 8749, 8759, 8769, 8779, 8789, 8799, 8809, 8819, 8829, 8839, 8849, 8859, 8869, 0,
    600, 684, 718, 1628, 8879, 8889, 8899, 8909, 8919, 8929, 8939, 8949, 8959, 8969, 8979, 8989,
    8999, 9009, 9019, 0, 1640, 9029, 1642, 1643, 9030, 96, 0, 9033, 1644, 9034, 96, 9037,
    9047, 9057, 9067, 9077, 9087, 9097, 9107, 9117, 9127, 9137, 9147, 9157, 9167, 9177, 9187, 9197,
    9207, 9217, 9227, 9237, 1660, 0, 334, 241, 9238, 1100, 58, 0, 152, 1662, 9239, 1664,
    1666, 9251, 90, 9241, 1024, 0, 1063, 257, 1061, 629, 1067, 1009, 1070, 1072, 738, 740,
    9252, 0, 1667, 867, 869, 1669, 1671, 999, 1673, 217, 1303, 1305, 1061, 1673, 1070, 1072,
    1067, 736, 841, 738, 740, 9273, 9265, 9262, 1432, 9269, 9266, 1675, 1676, 9270, 424, 9274,
    9284, 9293, 9303, 9313, 9323, 0, 1680, 96, 0, 684, 1680, 9324, 0, 1681, 96, 0,
    684, 1681, 9325, 9335, 9345, 9355, 9365, 9375, 0, 1580, 9385, 1685, 0, 290, 279, 9387,
    282, 369, 203, 290, 326, 1120, 507, 0, 1434, 9388, 0, 1067, 1434, 217, 1061, 1673,
    1070, 1072, 738, 740, 9392, 9403, 1687, 0, 9402, 1688, 1689, 58, 1690, 0, 1580, 46,
    613, 538, 9404, 9414, 9424, 9434, 9444, 0, 9454, 975, 976, 9455, 0, 1458, 374, 978,
    9459, 9469, 9479, 9489, 1697, 1698, 9490, 9500, 9510, 283, 1113, 777, 9511, 226, 0, 226,
    485, 1101, 9513, 9524, 0, 784, 206, 9534, 9543, 9552, 9562, 9572, 9582, 9592, 9602, 9612,
    9622, 9636, 1710, 1711, 1712, 1713, 1714, 1715, 1716, 1144, 1145, 1717, 124, 0, 1040, 9635,
    1721, 1722, 1723, 1724, 802, 1725, 0, 1726, 1727, 1728, 0, 507, 1729, 198, 1731, 0,
    944, 9632, 1728, 1732, 0, 1733, 1734, 0, 1735, 1736, 1138, 1738, 0, 1739, 9633, 1740,
    1741, 0, 1742, 1743, 0, 1745, 165, 1746, 611, 1748, 0, 1749, 1750, 1728, 1751, 0,
    1748, 1752, 0, 541, 1753, 1753, 0, 1754, 1748, 0, 1755, 1756, 46, 54, 1757, 0,
    1758, 9634, 626, 1760, 803, 804, 1761, 124, 9637, 9647, 9657, 9667, 9677, 9687, 9697, 9707,
    9717, 9727, 9737, 9747, 9757, 9767, 9777, 9787, 9797, 9807, 9817, 9827, 9837, 9847, 9857, 9867,
    9877, 9887, 9897, 9907, 9917, 9927, 9937, 9947, 9956, 9966, 9976, 9986, 9996, 10006, 424, 1436,
    945, 1791, 1793, 1795, 1797, 1799, 1801, 1803, 1805, 1807, 1809, 1811, 1813, 1815, 1817, 1819,
    1821, 1823, 1825, 1827, 1829, 949, 947, 1831, 1833, 1835, 1837, 1839, 1841, 1843, 1845, 1847,
    1849, 1851, 1853, 1855, 1857, 1859, 1861, 1862, 1018, 10007, 10017, 10027, 10037, 10047, 10057, 10067,
    10077, 10087, 10097, 10107, 10117, 10127, 10137, 10147, 10157, 1875, 10158, 10170, 10168, 0, 334, 241,
    0, 10173, 10183, 10193, 10203, 10213, 10223, 10233, 10243, 10253, 10263, 10273, 10280, 10290, 10300, 10310,
    10320, 10330, 10340, 10350, 10360, 10370, 0, 0, 500, 77, 10380, 0, 623, 160, 0, 510,
    160, 0, 1893, 160, 515, 0, 10387, 1120, 369, 203, 239, 10381, 424, 10384, 1436, 10388,
    10398, 10408, 10418, 10428, 0, 10438, 1895, 445, 1896, 0, 10442, 1897, 1896, 10446, 10456, 10466,
    10476, 10486, 0, 10496, 153, 154, 10497, 10507, 1899, 618, 10508, 10518, 10528, 10538, 10548, 10558,
    10568, 10578, 10587, 0, 10597, 803, 804, 779, 0, 10598, 600, 627, 803, 1906, 10599, 0,
    1908, 803, 1906, 326, 10603, 10613, 10623, 10633, 10643, 10653, 10663, 10673, 10683, 10693, 10703, 10713,
    10723, 10733, 10743, 0, 10753, 693, 1919, 848, 854, 441, 10754, 1921, 1922, 0, 1923, 226,
    0, 964, 684, 1924, 10755, 10765, 10775, 10785, 10795, 10805, 10815, 10825, 10835, 10845, 10855, 10865,
    10875, 10885, 10895, 10905, 1937, 1938, 1939, 10906, 10916, 10927, 0, 156, 165, 239, 10937, 10947,
    10959, 10957, 334, 10962, 0, 784, 1311, 0, 914, 1942, 10972, 10982, 10992, 11002, 11012, 1946,
    11022, 1947, 1948, 1949, 11023, 11033, 11043, 0, 1953, 11053, 1100, 58, 11054, 0, 623, 11064,
    11074, 0, 11084, 11094, 11104, 11114, 0, 1667, 867, 869, 1669, 1671, 999, 1673, 1067, 736,
    841, 217, 1061, 1070, 1072, 738, 740, 0, 1434, 11124, 0, 1067, 1434, 11128, 11138, 11148,
    11158, 0, 157, 239, 241, 1580, 1377, 867, 869, 11168, 11178, 11188, 11198, 11208, 0, 139,
    615, 11218, 11228, 226, 11238, 11248, 11258, 11268, 11278, 11288, 11298, 11308, 11318, 11328, 11338, 11348,
    11358, 11368, 11378, 0, 860, 11395, 11394, 11391, 11389, 11388, 507, 326, 507, 326, 217, 1061,
    11397, 11407, 11417, 11427, 11437, 11447, 11457, 11467, 11477, 11487, 11497, 11507, 11517, 11527, 0, 600,
    11537, 1987, 1988, 505, 1138, 11538, 11548, 11558, 11568, 11578, 11588, 11598, 11608, 11618, 0, 732,
    0, 326, 11628, 1365, 11631, 326, 11634, 11637, 11640, 11650, 0, 1024, 11663, 11660, 326, 11664,
    1061, 0, 198, 257, 0, 1063, 257, 1064, 1065, 1066, 629, 1009, 1069, 1070, 1072, 738,
    740, 11665, 11675, 11685, 11692, 0, 84, 0, 11703, 11702, 0, 226, 485, 0, 1101, 777,
    1101, 11707, 0, 226, 11717, 1512, 1063, 2000, 217, 2002, 0, 600, 2004, 11718, 11721, 2006,
    152, 11723, 2008, 0, 600, 2004, 11726, 11729, 2006, 152, 11731, 1676, 0, 600, 0, 684,
    55, 0, 11734, 0, 600, 684, 46, 2009, 0, 2010, 198, 11744, 217, 11747, 11757, 11767,
    11777, 11784, 11794, 11804, 11814, 11824, 0, 2014, 11834, 1033, 686, 652, 2015, 484, 2017, 11835,
    0, 295, 2019, 374, 0, 11845, 1178, 2020, 2021, 860, 11846, 11856, 11866, 11876, 11886, 11896,
    11906, 11916, 11926, 11936, 11946, 11956, 11966,
];

pub static SEGMENT_TYPE_OFFSETS: &[u32] = &[
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 65, 4294967295, 4294967295, 76, 4294967295, 4294967295, 4294967295, 4294967295, 76, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 78, 4294967295, 80, 83, 85, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 65, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 162, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 192, 194, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 196,
    4294967295, 4294967295, 78, 4294967295, 4294967295, 200, 4294967295, 200, 4294967295, 204, 4294967295, 207, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 219, 4294967295, 220, 225, 4294967295, 4294967295, 4294967295, 4294967295, 228, 4294967295,
    4294967295, 4294967295, 236, 4294967295, 4294967295, 240, 242, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 268,
    4294967295, 270, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 281, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 286, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 291, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 240, 4294967295, 4294967295, 4294967295, 240, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 303, 303, 4294967295, 303, 303, 4294967295, 4294967295, 4294967295, 4294967295,
    327, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 370, 373, 204, 291, 327, 4294967295, 375, 4294967295, 4294967295,
    4294967295, 376, 3, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 303, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 291, 411, 4294967295, 415, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 228, 4294967295, 4294967295, 423, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 204, 80, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 220, 4294967295, 438,
    4294967295, 4294967295, 444, 4294967295, 4294967295, 4294967295, 4294967295, 327, 4294967295, 438, 4294967295, 4294967295, 444, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 498, 4294967295, 204, 370, 4294967295,
    4294967295, 4294967295, 78, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 327, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 518, 4294967295, 4294967295, 521, 4294967295, 4294967295, 4294967295, 375, 4294967295, 4294967295,
    4294967295, 524, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 291, 4294967295, 4294967295,
    537, 4294967295, 4294967295, 539, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 78, 4294967295, 4294967295, 4294967295, 240, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 544, 4294967295, 4294967295,
    4294967295, 281, 281, 281, 281, 281, 281, 4294967295, 4294967295, 281, 4294967295, 303, 281, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 375, 4294967295, 375, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 80, 83, 4294967295, 4294967295, 78, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 616, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 80, 83, 4294967295, 4294967295, 375, 622, 4294967295, 4294967295, 4294967295, 4294967295, 537, 4294967295,
    240, 4294967295, 4294967295, 4294967295, 4294967295, 625, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    644, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 327, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 537, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 78, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 281,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 200, 4294967295, 4294967295, 4294967295, 4294967295,
    539, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 268, 4294967295, 695, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 240, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 700, 286, 518, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 240, 703, 4294967295, 4294967295, 4294967295, 705, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 291, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 240, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 240, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    327, 4294967295, 4294967295, 4294967295, 4294967295, 735, 4294967295, 4294967295, 4294967295, 200, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 240,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 737, 4294967295, 739, 741, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 80, 80, 4294967295, 754, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 780, 4294967295, 4294967295, 4294967295, 785, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 291, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 240, 4294967295, 4294967295, 4294967295, 4294967295, 286, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    842, 844, 4294967295, 4294967295, 240, 844, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 644, 859, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 868, 871, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 78, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 375, 4294967295, 375, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    616, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 80, 4294967295, 4294967295, 4294967295, 4294967295, 537, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 78, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 539, 4294967295, 4294967295, 4294967295,
    4294967295, 268, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 240, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 705, 4294967295,
    4294967295, 4294967295, 4294967295, 78, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 200, 4294967295, 4294967295, 4294967295, 4294967295, 240, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 327, 4294967295, 4294967295, 4294967295, 4294967295,
    915, 917, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 946, 948, 950, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 97, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 97, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    977, 4294967295, 979, 498, 4294967295, 4294967295, 4294967295, 370, 204, 4294967295, 4294967295, 4294967295, 327, 4294967295, 4294967295, 4294967295,
    4294967295, 291, 204, 983, 327, 4294967295, 85, 4294967295, 4294967295, 915, 987, 4294967295, 4294967295, 327, 4294967295, 375,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 983, 4294967295, 4294967295, 4294967295, 4294967295, 994, 4294967295, 373, 996, 998,
    291, 1000, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 327, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1020, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    327, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 373, 4294967295,
    291, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1052, 4294967295, 4294967295, 1062, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1068, 737,
    842, 4294967295, 4294967295, 4294967295, 1071, 1073, 739, 741, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1077, 1079, 242,
    4294967295, 4294967295, 1083, 4294967295, 1086, 4294967295, 4294967295, 4294967295, 242, 1083, 4294967295, 4294967295, 4294967295, 4294967295, 1086, 4294967295,
    4294967295, 4294967295, 4294967295, 1090, 1092, 4294967295, 1052, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 220, 4294967295, 4294967295,
    4294967295, 1086, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 236, 4294967295, 4294967295, 4294967295, 4294967295, 327, 1096,
    4294967295, 327, 4294967295, 327, 4294967295, 375, 4294967295, 4294967295, 327, 4294967295, 4294967295, 327, 4294967295, 4294967295, 4294967295, 327,
    4294967295, 327, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1104, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    291, 4294967295, 4294967295, 4294967295, 4294967295, 291, 4294967295, 625, 996, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 411, 4294967295, 4294967295, 4294967295, 411, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 375, 4294967295, 4294967295, 4294967295, 4294967295, 375, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    80, 735, 4294967295, 4294967295, 4294967295, 80, 4294967295, 80, 80, 1121, 735, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1133, 4294967295, 1135,
    4294967295, 4294967295, 200, 4294967295, 4294967295, 4294967295, 4294967295, 196, 78, 4294967295, 1137, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 196, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1150, 1152, 4294967295, 4294967295,
    4294967295, 291, 4294967295, 327, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1062, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1071,
    1073, 739, 741, 4294967295, 4294967295, 4294967295, 4294967295, 291, 4294967295, 327, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1152, 754, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 327, 4294967295, 4294967295, 4294967295,
    4294967295, 1062, 4294967295, 4294967295, 4294967295, 4294967295, 1071, 1073, 739, 741, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1173, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 204,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 327, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 240, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    1234, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1092, 4294967295, 4294967295, 4294967295, 220,
    1247, 4294967295, 1249, 4294967295, 236, 4294967295, 4294967295, 4294967295, 1092, 1247, 4294967295, 4294967295, 1092, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 220, 4294967295, 4294967295, 327, 236, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 370, 204, 291, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    327, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 327, 4294967295, 4294967295, 4294967295, 4294967295, 1304, 1306, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 97, 4294967295, 4294967295, 1312, 1314, 1317, 807, 807,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 78,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1121, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    1361, 1363, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 327, 4294967295, 4294967295, 1368, 1370, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 327, 4294967295, 1372, 4294967295, 4294967295, 4294967295, 240, 4294967295, 4294967295, 4294967295, 240, 4294967295, 4294967295, 4294967295,
    4294967295, 242, 4294967295, 204, 242, 220, 1249, 236, 1376, 4294967295, 4294967295, 1378, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 327, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1404, 4294967295,
    4294967295, 4294967295, 1406, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1415,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1121, 994, 4294967295, 994, 4294967295, 1433, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 948, 946,
    950, 1437, 4294967295, 415, 4294967295, 4294967295, 994, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1456,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 327, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    291, 4294967295, 327, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1482, 1062, 4294967295, 4294967295, 1062,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1071, 1073, 739, 741, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1404, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 291, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 327, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1531, 1534, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 240, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1133, 4294967295, 1135, 4294967295, 4294967295, 200, 4294967295, 4294967295,
    4294967295, 4294967295, 196, 196, 4294967295, 1137, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 1661, 4294967295, 4294967295, 242, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1663, 4294967295, 1665,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1062, 4294967295, 1068, 4294967295, 1071, 1073, 739, 741,
    4294967295, 4294967295, 1668, 868, 871, 1670, 1672, 1000, 1674, 219, 1304, 1306, 1062, 1674, 1071, 1073,
    1068, 737, 842, 739, 741, 4294967295, 4294967295, 4294967295, 1433, 4294967295, 4294967295, 1433, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1686, 4294967295, 291, 281, 4294967295,
    4294967295, 370, 204, 291, 327, 1121, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1068, 4294967295, 219, 1062, 1674,
    1071, 1073, 739, 741, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 539, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 977, 4294967295, 4294967295, 4294967295, 375, 979,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 785, 207, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1437,
    946, 1792, 1794, 1796, 1798, 1800, 1802, 1804, 1806, 1808, 1810, 1812, 1814, 1816, 1818, 1820,
    1822, 1824, 1826, 1828, 1830, 950, 948, 1832, 1834, 1836, 1838, 1840, 1842, 1844, 1846, 1848,
    1850, 1852, 1854, 1856, 1858, 1860, 518, 1863, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 303, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 242,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 231, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 78, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1121, 370, 204, 240, 4294967295, 4294967295, 4294967295, 1437, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 780, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 327, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 240, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 785, 1312, 4294967295, 915, 917, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1668, 868, 871, 1670, 1672, 1000, 1674, 1068, 737,
    842, 219, 1062, 1071, 1073, 739, 741, 4294967295, 4294967295, 4294967295, 4294967295, 1068, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 240, 242, 4294967295, 1378, 868, 871, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    616, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 327, 4294967295, 327, 219, 1062,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 327, 220, 4294967295, 220, 327, 236, 236, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 327, 4294967295,
    1062, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1071, 1073, 739,
    741, 4294967295, 4294967295, 1997, 4294967295, 4294967295, 85, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2001, 219, 2003, 4294967295, 4294967295, 4294967295, 220, 4294967295, 2007,
    4294967295, 236, 4294967295, 4294967295, 4294967295, 4294967295, 220, 4294967295, 2007, 4294967295, 236, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 219, 4294967295, 4294967295, 4294967295,
    5, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 375, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
];

pub static SEGMENT_CLASS_OFFSETS: &[u32] = &[
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 64, 4294967295, 4294967295, 66, 4294967295, 4294967295, 4294967295, 4294967295, 66, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 79, 81, 84, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 64, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 161, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 191, 193, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 195,
    4294967295, 4294967295, 77, 4294967295, 4294967295, 199, 4294967295, 199, 4294967295, 203, 4294967295, 206, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 217, 4294967295, 221, 223, 4294967295, 4294967295, 4294967295, 4294967295, 227, 4294967295,
    4294967295, 4294967295, 237, 4294967295, 4294967295, 239, 241, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 267,
    4294967295, 269, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 279, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 285, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 290, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 239, 4294967295, 4294967295, 4294967295, 239, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 321, 322, 4294967295, 324, 325, 4294967295, 4294967295, 4294967295, 4294967295,
    326, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 369, 371, 203, 290, 326, 4294967295, 374, 4294967295, 4294967295,
    4294967295, 377, 378, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 304, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 290, 412, 4294967295, 413, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 227, 4294967295, 4294967295, 422, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 203, 79, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 435, 4294967295, 437,
    4294967295, 4294967295, 443, 4294967295, 4294967295, 4294967295, 4294967295, 326, 4294967295, 437, 4294967295, 4294967295, 443, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 481, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 497, 4294967295, 203, 369, 4294967295,
    4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 326, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 516, 4294967295, 4294967295, 520, 4294967295, 4294967295, 4294967295, 374, 4294967295, 4294967295,
    4294967295, 523, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 290, 4294967295, 4294967295,
    536, 4294967295, 4294967295, 538, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 239, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 545, 4294967295, 4294967295,
    4294967295, 279, 548, 549, 550, 551, 552, 4294967295, 4294967295, 555, 4294967295, 557, 558, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 374, 4294967295, 374, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 79, 81, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 615, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 79, 81, 4294967295, 4294967295, 374, 621, 4294967295, 4294967295, 4294967295, 4294967295, 536, 4294967295,
    239, 4294967295, 4294967295, 4294967295, 4294967295, 624, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    643, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 326, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 536, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 279,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 199, 4294967295, 4294967295, 4294967295, 4294967295,
    538, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 267, 4294967295, 694, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 239, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 699, 285, 516, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 239, 702, 4294967295, 4294967295, 4294967295, 704, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 290, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 239, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 239, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    326, 4294967295, 4294967295, 4294967295, 4294967295, 734, 4294967295, 4294967295, 4294967295, 199, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 239,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 736, 4294967295, 738, 740, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 751, 752, 4294967295, 753, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 779, 4294967295, 4294967295, 4294967295, 784, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 290, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 239, 4294967295, 4294967295, 4294967295, 4294967295, 285, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    841, 843, 4294967295, 4294967295, 239, 843, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 643, 858, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 867, 869, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 374, 4294967295, 374, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    615, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 79, 4294967295, 4294967295, 4294967295, 4294967295, 536, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 538, 4294967295, 4294967295, 4294967295,
    4294967295, 267, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 239, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 704, 4294967295,
    4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 199, 4294967295, 4294967295, 4294967295, 4294967295, 239, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 326, 4294967295, 4294967295, 4294967295, 4294967295,
    914, 916, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 945, 947, 949, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 969, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 969, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    976, 4294967295, 978, 497, 4294967295, 4294967295, 4294967295, 369, 203, 4294967295, 4294967295, 4294967295, 326, 4294967295, 4294967295, 4294967295,
    4294967295, 290, 203, 982, 326, 4294967295, 985, 4294967295, 4294967295, 914, 986, 4294967295, 4294967295, 326, 4294967295, 374,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 982, 4294967295, 4294967295, 4294967295, 4294967295, 993, 4294967295, 371, 995, 997,
    290, 999, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 326, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1019, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    326, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 371, 4294967295,
    290, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1051, 4294967295, 4294967295, 1061, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1067, 736,
    841, 4294967295, 4294967295, 4294967295, 1070, 1072, 738, 740, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1076, 1078, 241,
    4294967295, 4294967295, 1081, 4294967295, 1085, 4294967295, 4294967295, 4294967295, 241, 1081, 4294967295, 4294967295, 4294967295, 4294967295, 1085, 4294967295,
    4294967295, 4294967295, 4294967295, 1088, 1091, 4294967295, 1051, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 221, 4294967295, 4294967295,
    4294967295, 1085, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 237, 4294967295, 4294967295, 4294967295, 4294967295, 326, 1095,
    4294967295, 326, 4294967295, 326, 4294967295, 374, 4294967295, 4294967295, 326, 4294967295, 4294967295, 326, 4294967295, 4294967295, 4294967295, 326,
    4294967295, 326, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1103, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    290, 4294967295, 4294967295, 4294967295, 4294967295, 290, 4294967295, 624, 1110, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 412, 4294967295, 4294967295, 4294967295, 412, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 374, 4294967295, 4294967295, 4294967295, 4294967295, 374, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    1116, 1117, 4294967295, 4294967295, 4294967295, 79, 4294967295, 1116, 1119, 1120, 734, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1132, 4294967295, 1134,
    4294967295, 4294967295, 199, 4294967295, 4294967295, 4294967295, 4294967295, 195, 77, 4294967295, 1136, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 195, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1148, 1151, 4294967295, 4294967295,
    4294967295, 290, 4294967295, 326, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1061, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1070,
    1072, 738, 740, 4294967295, 4294967295, 4294967295, 4294967295, 290, 4294967295, 326, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1151, 753, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 326, 4294967295, 4294967295, 4294967295,
    4294967295, 1061, 4294967295, 4294967295, 4294967295, 4294967295, 1070, 1072, 738, 740, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1174, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 203,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 326, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 239, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    1232, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1091, 4294967295, 4294967295, 4294967295, 221,
    1246, 4294967295, 1248, 4294967295, 237, 4294967295, 4294967295, 4294967295, 1091, 1246, 4294967295, 4294967295, 1091, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 435, 4294967295, 4294967295, 326, 237, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 369, 203, 290, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    326, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 326, 4294967295, 4294967295, 4294967295, 4294967295, 1303, 1305, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 969, 4294967295, 4294967295, 1311, 1313, 1315, 1005, 808,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1120, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    1360, 1362, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 326, 4294967295, 4294967295, 1367, 1369, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 326, 4294967295, 1371, 4294967295, 4294967295, 4294967295, 239, 4294967295, 4294967295, 4294967295, 239, 4294967295, 4294967295, 4294967295,
    4294967295, 241, 4294967295, 203, 241, 221, 1248, 237, 1375, 4294967295, 4294967295, 1377, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 326, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1403, 4294967295,
    4294967295, 4294967295, 1405, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1416,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1120, 1431, 4294967295, 993, 4294967295, 1432, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 947, 945,
    949, 1436, 4294967295, 413, 4294967295, 4294967295, 993, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1455,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 326, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    290, 4294967295, 326, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1481, 1061, 4294967295, 4294967295, 1061,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1070, 1072, 738, 740, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1403, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 290, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 326, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1530, 1532, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 239, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1132, 4294967295, 1134, 4294967295, 4294967295, 199, 4294967295, 4294967295,
    4294967295, 4294967295, 195, 195, 4294967295, 1136, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 1660, 4294967295, 4294967295, 241, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1662, 4294967295, 1664,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1061, 4294967295, 1067, 4294967295, 1070, 1072, 738, 740,
    4294967295, 4294967295, 1667, 867, 869, 1669, 1671, 999, 1673, 217, 1303, 1305, 1061, 1673, 1070, 1072,
    1067, 736, 841, 738, 740, 4294967295, 4294967295, 4294967295, 1432, 4294967295, 4294967295, 1675, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1685, 4294967295, 290, 279, 4294967295,
    4294967295, 369, 203, 290, 326, 1120, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1067, 4294967295, 217, 1061, 1673,
    1070, 1072, 738, 740, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 538, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 976, 4294967295, 4294967295, 4294967295, 374, 978,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 784, 206, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1436,
    945, 1791, 1793, 1795, 1797, 1799, 1801, 1803, 1805, 1807, 1809, 1811, 1813, 1815, 1817, 1819,
    1821, 1823, 1825, 1827, 1829, 949, 947, 1831, 1833, 1835, 1837, 1839, 1841, 1843, 1845, 1847,
    1849, 1851, 1853, 1855, 1857, 1859, 1861, 1862, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1875, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 241,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 230, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1120, 369, 203, 239, 4294967295, 4294967295, 4294967295, 1436, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 779, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 326, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 239, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 784, 1311, 4294967295, 914, 1942, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1667, 867, 869, 1669, 1671, 999, 1673, 1067, 736,
    841, 217, 1061, 1070, 1072, 738, 740, 4294967295, 4294967295, 4294967295, 4294967295, 1067, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 239, 241, 4294967295, 1377, 867, 869, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    615, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 326, 4294967295, 326, 217, 1061,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 326, 221, 4294967295, 435, 326, 237, 237, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 326, 4294967295,
    1061, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1070, 1072, 738,
    740, 4294967295, 4294967295, 1998, 4294967295, 4294967295, 84, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2000, 217, 2002, 4294967295, 4294967295, 4294967295, 221, 4294967295, 2006,
    4294967295, 237, 4294967295, 4294967295, 4294967295, 4294967295, 221, 4294967295, 2006, 4294967295, 237, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 217, 4294967295, 4294967295, 4294967295,
    1111, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 374, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
];

pub static CASEFOLD_SPARSE: &[(u32, u8)] = &[
    (2322, 1),
];

pub static TRIM_CHARS_SPARSE: &[(u32, u32, u8)] = &[
];

pub static TRIM_CHARS_DATA: &[u32] = &[
];

pub static SEGMENT_CLASS_TYPES_SPARSE: &[(u32, u32, u8)] = &[
    (38, 0, 2),
    (41, 2, 2),
    (46, 4, 2),
    (53, 6, 2),
    (55, 8, 2),
    (56, 10, 2),
    (57, 12, 3),
    (99, 15, 2),
    (135, 17, 2),
    (184, 19, 2),
    (185, 21, 2),
    (191, 23, 3),
    (194, 26, 2),
    (197, 28, 3),
    (199, 31, 3),
    (201, 34, 2),
    (203, 36, 2),
    (214, 38, 2),
    (216, 40, 4),
    (217, 44, 2),
    (222, 46, 2),
    (226, 48, 5),
    (229, 53, 3),
    (230, 56, 2),
    (255, 58, 3),
    (257, 61, 2),
    (266, 63, 2),
    (275, 65, 2),
    (281, 67, 3),
    (289, 70, 3),
    (293, 73, 3),
    (311, 76, 2),
    (312, 78, 2),
    (314, 80, 2),
    (315, 82, 2),
    (320, 84, 2),
    (359, 86, 2),
    (360, 88, 2),
    (361, 90, 2),
    (362, 92, 3),
    (363, 95, 2),
    (365, 97, 2),
    (369, 99, 2),
    (370, 101, 1),
    (382, 102, 3),
    (411, 105, 3),
    (412, 108, 2),
    (414, 110, 2),
    (423, 112, 2),
    (426, 114, 2),
    (435, 116, 2),
    (436, 118, 2),
    (445, 120, 4),
    (447, 124, 2),
    (450, 126, 2),
    (455, 128, 2),
    (457, 130, 2),
    (460, 132, 2),
    (503, 134, 2),
    (523, 136, 2),
    (525, 138, 2),
    (526, 140, 2),
    (530, 142, 2),
    (538, 144, 2),
    (550, 146, 2),
    (553, 148, 3),
    (557, 151, 2),
    (561, 153, 2),
    (589, 155, 3),
    (592, 158, 3),
    (595, 161, 3),
    (610, 164, 2),
    (614, 166, 3),
    (621, 169, 3),
    (625, 172, 2),
    (626, 174, 2),
    (627, 176, 2),
    (628, 178, 2),
    (629, 180, 2),
    (630, 182, 2),
    (633, 184, 3),
    (635, 187, 2),
    (636, 189, 2),
    (690, 191, 2),
    (692, 193, 2),
    (705, 195, 2),
    (706, 197, 2),
    (709, 199, 2),
    (717, 201, 3),
    (724, 204, 2),
    (725, 206, 2),
    (728, 208, 2),
    (729, 210, 2),
    (734, 212, 3),
    (736, 215, 3),
    (741, 218, 2),
    (752, 220, 3),
    (762, 223, 2),
    (771, 225, 3),
    (824, 228, 2),
    (831, 230, 2),
    (843, 232, 3),
    (848, 235, 3),
    (872, 238, 3),
    (874, 241, 2),
    (881, 243, 3),
    (887, 246, 2),
    (888, 248, 2),
    (889, 250, 2),
    (897, 252, 3),
    (898, 255, 2),
    (902, 257, 3),
    (917, 260, 3),
    (923, 263, 3),
    (937, 266, 3),
    (960, 269, 2),
    (965, 271, 2),
    (969, 273, 3),
    (975, 276, 3),
    (982, 279, 2),
    (984, 281, 2),
    (985, 283, 2),
    (996, 285, 2),
    (997, 287, 2),
    (999, 289, 2),
    (1047, 291, 2),
    (1051, 293, 2),
    (1102, 295, 3),
    (1109, 298, 3),
    (1114, 301, 2),
    (1120, 303, 2),
    (1121, 305, 2),
    (1124, 307, 3),
    (1125, 310, 2),
    (1158, 312, 3),
    (1159, 315, 2),
    (1177, 317, 2),
    (1178, 319, 2),
    (1193, 321, 2),
    (1224, 323, 2),
    (1226, 325, 2),
    (1232, 327, 3),
    (1238, 330, 2),
    (1243, 332, 3),
    (1250, 335, 2),
    (1260, 337, 3),
    (1265, 340, 3),
    (1272, 343, 3),
    (1278, 346, 3),
    (1283, 349, 2),
    (1289, 351, 3),
    (1294, 354, 3),
    (1307, 357, 2),
    (1312, 359, 2),
    (1313, 361, 2),
    (1348, 363, 2),
    (1349, 365, 2),
    (1350, 367, 2),
    (1381, 369, 2),
    (1398, 371, 2),
    (1408, 373, 2),
    (1410, 375, 2),
    (1411, 377, 2),
    (1415, 379, 2),
    (1416, 381, 2),
    (1420, 383, 2),
    (1425, 385, 3),
    (1426, 388, 2),
    (1427, 390, 2),
    (1428, 392, 2),
    (1430, 394, 3),
    (1433, 397, 2),
    (1434, 399, 2),
    (1437, 401, 2),
    (1439, 403, 2),
    (1446, 405, 2),
    (1451, 407, 2),
    (1453, 409, 2),
    (1454, 411, 2),
    (1455, 413, 2),
    (1456, 415, 3),
    (1457, 418, 2),
    (1475, 420, 2),
    (1493, 422, 2),
    (1504, 424, 2),
    (1534, 426, 2),
    (1536, 428, 3),
    (1553, 431, 2),
    (1556, 433, 2),
    (1566, 435, 2),
    (1567, 437, 2),
    (1568, 439, 2),
    (1572, 441, 2),
    (1573, 443, 2),
    (1574, 445, 2),
    (1575, 447, 2),
    (1581, 449, 2),
    (1582, 451, 2),
    (1583, 457, 2),
    (1586, 453, 2),
    (1588, 455, 2),
    (1592, 459, 2),
    (1593, 461, 2),
    (1598, 463, 2),
    (1603, 465, 2),
    (1604, 467, 2),
    (1606, 469, 2),
    (1617, 471, 2),
    (1630, 473, 2),
    (1631, 475, 2),
    (1633, 477, 2),
    (1635, 479, 2),
    (1637, 481, 2),
    (1640, 483, 2),
    (1643, 485, 2),
    (1647, 487, 2),
    (1649, 489, 2),
    (1656, 491, 2),
    (1664, 493, 3),
    (1669, 496, 3),
    (1671, 499, 2),
    (1672, 501, 2),
    (1689, 503, 2),
    (1693, 505, 2),
    (1701, 507, 2),
    (1706, 509, 2),
    (1712, 511, 2),
    (1713, 513, 2),
    (1717, 521, 2),
    (1719, 515, 2),
    (1720, 517, 2),
    (1721, 519, 2),
    (1722, 523, 2),
    (1741, 525, 2),
    (1743, 527, 2),
    (1746, 529, 3),
    (1751, 532, 3),
    (1752, 535, 2),
    (1754, 537, 2),
    (1774, 539, 3),
    (1788, 542, 2),
    (1789, 544, 2),
    (1793, 546, 3),
    (1795, 549, 2),
    (1801, 551, 2),
    (1807, 553, 2),
    (1808, 555, 2),
    (1809, 557, 2),
    (1810, 559, 2),
    (1815, 561, 3),
    (1817, 564, 2),
    (1827, 566, 2),
    (1828, 568, 2),
    (1836, 570, 2),
    (1841, 572, 2),
    (1846, 574, 2),
    (1847, 576, 2),
    (1848, 578, 2),
    (1849, 580, 2),
    (1868, 582, 3),
    (1903, 585, 2),
    (1911, 587, 2),
    (1937, 589, 3),
    (2000, 592, 2),
    (2011, 594, 2),
    (2016, 596, 2),
    (2018, 598, 2),
    (2024, 600, 2),
    (2025, 602, 2),
    (2028, 604, 2),
    (2038, 606, 2),
    (2060, 608, 4),
    (2066, 612, 2),
    (2067, 614, 2),
    (2068, 616, 3),
    (2128, 619, 2),
    (2135, 621, 2),
    (2140, 623, 2),
    (2141, 625, 2),
    (2152, 627, 2),
    (2155, 629, 2),
    (2156, 631, 2),
    (2157, 633, 2),
    (2158, 635, 5),
    (2159, 640, 3),
    (2207, 643, 2),
    (2234, 645, 2),
    (2240, 647, 2),
    (2241, 649, 2),
    (2247, 651, 2),
    (2250, 653, 2),
    (2251, 655, 2),
    (2258, 657, 2),
    (2260, 659, 2),
    (2264, 661, 3),
    (2268, 664, 3),
    (2273, 667, 2),
    (2275, 669, 2),
    (2276, 671, 2),
    (2278, 673, 2),
    (2280, 675, 2),
    (2283, 677, 2),
    (2327, 679, 2),
    (2334, 681, 2),
    (2338, 683, 2),
    (2351, 685, 3),
    (2371, 688, 2),
    (2372, 690, 2),
    (2374, 692, 2),
    (2376, 694, 2),
    (2382, 696, 2),
    (2383, 698, 2),
    (2384, 700, 2),
    (2385, 702, 2),
    (2387, 704, 2),
    (2390, 706, 2),
    (2431, 708, 2),
    (2460, 710, 2),
    (2496, 712, 3),
    (2498, 715, 2),
    (2507, 717, 2),
    (2508, 719, 2),
    (2511, 721, 2),
    (2519, 723, 2),
    (2520, 725, 2),
    (2521, 727, 2),
    (2522, 729, 2),
    (2541, 731, 2),
    (2551, 733, 3),
    (2582, 736, 2),
    (2614, 738, 2),
    (2615, 740, 2),
    (2696, 742, 3),
    (2776, 745, 2),
    (2778, 747, 2),
    (2781, 749, 3),
    (2786, 752, 3),
    (2787, 755, 3),
    (2789, 758, 2),
    (2868, 760, 2),
    (2871, 762, 2),
    (2877, 764, 2),
    (2879, 766, 2),
    (2888, 768, 2),
    (2890, 770, 2),
    (2892, 772, 2),
    (2893, 774, 2),
    (2894, 776, 2),
    (2895, 778, 2),
    (2898, 780, 2),
    (2899, 782, 2),
    (2900, 784, 2),
    (2901, 786, 2),
    (2902, 788, 2),
    (2903, 790, 2),
    (2904, 792, 2),
    (2905, 794, 2),
    (2906, 796, 2),
    (2907, 798, 2),
    (2908, 800, 2),
    (2909, 802, 2),
    (2910, 804, 2),
    (2911, 806, 2),
    (2912, 808, 2),
    (2913, 810, 2),
    (2914, 812, 2),
    (2915, 814, 2),
    (2916, 816, 2),
    (2920, 818, 2),
    (2923, 820, 2),
    (2955, 822, 2),
    (2957, 824, 3),
    (2958, 827, 2),
    (2961, 829, 2),
    (2962, 831, 2),
    (2963, 833, 3),
    (2964, 836, 2),
    (2965, 838, 2),
    (2971, 840, 2),
    (2973, 842, 2),
    (2974, 844, 2),
    (2975, 846, 2),
    (2976, 848, 2),
    (2977, 850, 2),
    (2978, 852, 2),
    (2979, 854, 2),
    (2993, 856, 3),
    (3002, 859, 2),
    (3006, 861, 2),
    (3007, 863, 2),
    (3029, 865, 2),
    (3030, 867, 2),
    (3167, 869, 2),
    (3168, 871, 2),
    (3169, 873, 2),
    (3170, 875, 2),
    (3171, 877, 2),
    (3172, 879, 2),
    (3173, 881, 2),
    (3174, 883, 2),
    (3175, 885, 2),
    (3176, 887, 2),
    (3177, 889, 2),
    (3178, 891, 2),
    (3179, 893, 2),
    (3180, 895, 2),
    (3181, 897, 2),
    (3182, 899, 2),
    (3183, 901, 2),
    (3184, 903, 2),
    (3185, 905, 2),
    (3186, 907, 2),
    (3187, 909, 2),
    (3188, 911, 2),
    (3189, 913, 2),
    (3190, 915, 2),
    (3191, 917, 2),
    (3192, 919, 2),
    (3193, 921, 2),
    (3194, 923, 2),
    (3195, 925, 2),
    (3196, 927, 2),
    (3197, 929, 2),
    (3198, 931, 2),
    (3199, 933, 2),
    (3200, 935, 2),
    (3201, 937, 2),
    (3202, 939, 2),
    (3203, 941, 2),
    (3204, 943, 2),
    (3205, 945, 2),
    (3206, 947, 2),
    (3207, 949, 2),
    (3225, 951, 2),
    (3231, 953, 2),
    (3243, 955, 3),
    (3257, 958, 2),
    (3271, 960, 2),
    (3272, 962, 2),
    (3273, 964, 2),
    (3274, 966, 3),
    (3278, 969, 2),
    (3319, 971, 2),
    (3331, 973, 2),
    (3389, 975, 3),
    (3397, 978, 2),
    (3398, 980, 2),
    (3400, 982, 2),
    (3401, 984, 2),
    (3431, 986, 2),
    (3432, 988, 2),
    (3433, 990, 2),
    (3434, 992, 2),
    (3435, 994, 2),
    (3436, 996, 2),
    (3437, 998, 2),
    (3438, 1000, 2),
    (3439, 1002, 2),
    (3440, 1004, 2),
    (3441, 1006, 2),
    (3442, 1008, 2),
    (3443, 1010, 2),
    (3444, 1012, 2),
    (3445, 1014, 2),
    (3446, 1016, 2),
    (3451, 1018, 2),
    (3459, 1020, 3),
    (3460, 1023, 2),
    (3462, 1025, 2),
    (3463, 1027, 2),
    (3464, 1029, 2),
    (3472, 1031, 3),
    (3499, 1034, 2),
    (3501, 1036, 2),
    (3502, 1038, 2),
    (3503, 1040, 2),
    (3537, 1042, 2),
    (3541, 1044, 2),
    (3550, 1046, 2),
    (3552, 1048, 2),
    (3565, 1050, 2),
    (3566, 1052, 2),
    (3567, 1054, 2),
    (3568, 1056, 2),
    (3571, 1058, 3),
    (3574, 1061, 3),
    (3591, 1064, 2),
    (3592, 1066, 2),
    (3593, 1068, 2),
    (3599, 1070, 2),
    (3608, 1072, 2),
    (3628, 1074, 2),
    (3632, 1076, 3),
    (3651, 1079, 2),
];

pub static SEGMENT_CLASS_TYPES_DATA: &[u32] = &[
    65, 3, 76, 3, 76, 3, 3, 78, 3, 80, 3, 83, 3, 78, 85, 65,
    3, 162, 3, 3, 192, 3, 194, 3, 78, 196, 3, 78, 3, 78, 200, 3,
    78, 200, 3, 204, 207, 3, 3, 219, 3, 220, 222, 4, 225, 3, 3, 228,
    3, 236, 220, 222, 4, 3, 78, 240, 242, 3, 3, 78, 268, 270, 3, 3,
    281, 3, 286, 3, 291, 78, 3, 78, 240, 3, 78, 240, 3, 303, 3, 303,
    3, 303, 3, 303, 3, 327, 370, 3, 3, 373, 3, 204, 3, 291, 78, 3,
    327, 3, 375, 3, 376, 3, 3, 303, 4, 3, 291, 78, 3, 411, 3, 415,
    3, 228, 3, 423, 3, 204, 3, 80, 3, 220, 222, 4, 3, 438, 3, 444,
    3, 327, 3, 438, 3, 444, 3, 4, 3, 498, 3, 204, 370, 3, 3, 78,
    3, 327, 3, 518, 3, 521, 78, 3, 375, 3, 524, 3, 291, 78, 3, 537,
    78, 3, 78, 539, 3, 78, 3, 78, 240, 3, 544, 4, 3, 281, 3, 281,
    3, 281, 3, 281, 3, 281, 3, 281, 3, 281, 4, 3, 303, 3, 281, 3,
    375, 3, 375, 3, 80, 3, 83, 3, 78, 3, 616, 78, 3, 80, 3, 83,
    3, 375, 3, 622, 3, 537, 78, 3, 78, 240, 3, 625, 3, 78, 644, 3,
    327, 3, 537, 78, 3, 78, 3, 281, 3, 78, 200, 3, 78, 539, 3, 78,
    268, 3, 695, 3, 78, 240, 3, 700, 3, 286, 3, 518, 3, 78, 240, 3,
    703, 3, 78, 705, 3, 291, 78, 3, 78, 240, 3, 78, 240, 3, 327, 3,
    735, 3, 78, 200, 3, 78, 240, 3, 737, 3, 739, 3, 741, 3, 80, 3,
    80, 3, 754, 3, 780, 785, 3, 3, 291, 78, 3, 78, 240, 3, 286, 3,
    842, 3, 844, 3, 78, 240, 3, 844, 3, 78, 644, 3, 859, 3, 868, 3,
    871, 3, 78, 3, 375, 3, 375, 3, 616, 78, 3, 80, 3, 537, 78, 3,
    78, 3, 78, 539, 3, 78, 268, 3, 78, 240, 3, 78, 705, 3, 78, 3,
    78, 200, 3, 78, 240, 3, 327, 3, 915, 3, 917, 3, 946, 3, 948, 3,
    950, 3, 97, 3, 97, 3, 977, 3, 979, 3, 498, 370, 3, 3, 204, 3,
    327, 3, 291, 78, 3, 204, 3, 983, 3, 327, 3, 78, 85, 3, 915, 3,
    987, 3, 327, 3, 375, 3, 983, 3, 994, 3, 373, 3, 996, 998, 3, 3,
    291, 78, 3, 1000, 3, 327, 3, 1020, 3, 327, 3, 373, 3, 291, 78, 3,
    1052, 3, 1062, 3, 1068, 3, 737, 3, 842, 3, 1071, 3, 1073, 3, 739, 3,
    741, 3, 1077, 3, 1079, 3, 1083, 3, 1086, 242, 3, 242, 3, 3, 1083, 3,
    1086, 3, 1090, 3, 1092, 3, 1052, 3, 1086, 3, 327, 3, 1096, 3, 327, 3,
    327, 3, 375, 3, 327, 3, 327, 3, 327, 3, 327, 1104, 3, 3, 291, 78,
    3, 291, 78, 3, 625, 3, 996, 3, 411, 3, 411, 3, 375, 3, 375, 3,
    80, 3, 735, 3, 80, 3, 80, 3, 1121, 3, 80, 3, 735, 1133, 3, 1135,
    3, 3, 78, 200, 3, 78, 196, 3, 78, 1137, 3, 3, 78, 196, 3, 1150,
    3, 1152, 3, 291, 78, 3, 327, 3, 1062, 3, 1071, 3, 1073, 3, 739, 3,
    741, 3, 291, 78, 3, 327, 3, 1152, 3, 754, 3, 327, 3, 1062, 3, 1071,
    3, 1073, 3, 739, 3, 741, 3, 1173, 4, 3, 204, 3, 327, 3, 78, 240,
    3, 1234, 3, 1092, 3, 1247, 3, 1249, 3, 1092, 3, 1247, 3, 1092, 3, 327,
    3, 1, 4, 5, 370, 3, 3, 204, 3, 291, 78, 3, 327, 3, 327, 3,
    1304, 3, 1306, 3, 97, 1312, 3, 3, 1314, 3, 1317, 3, 1, 807, 4, 5,
    3, 807, 4, 3, 78, 3, 1121, 3, 1361, 3, 1363, 3, 327, 3, 1368, 3,
    1370, 3, 327, 3, 1372, 3, 78, 240, 3, 78, 240, 242, 3, 3, 204, 242,
    3, 3, 1249, 3, 1376, 3, 1378, 3, 327, 3, 1404, 3, 1406, 3, 1415, 4,
    3, 1121, 3, 994, 3, 994, 3, 1433, 3, 948, 3, 946, 3, 950, 3, 1437,
    3, 415, 3, 994, 3, 1456, 3, 327, 3, 291, 78, 3, 327, 3, 1482, 3,
    1062, 3, 1062, 3, 1071, 3, 1073, 3, 739, 3, 741, 3, 1404, 3, 291, 78,
    3, 327, 3, 1531, 3, 1534, 3, 78, 240, 1133, 3, 1135, 3, 3, 78, 200,
    3, 78, 196, 3, 78, 196, 1137, 3, 3, 1661, 242, 3, 3, 1663, 3, 1665,
    3, 1062, 3, 1068, 3, 1071, 3, 1073, 3, 739, 3, 741, 3, 1668, 3, 868,
    3, 871, 3, 1670, 3, 1672, 3, 1000, 3, 1674, 3, 219, 3, 1304, 3, 1306,
    3, 1062, 3, 1674, 3, 1071, 3, 1073, 3, 1068, 3, 737, 3, 842, 3, 739,
    3, 741, 3, 1433, 3, 1433, 3, 1686, 3, 291, 78, 3, 281, 370, 3, 3,
    204, 3, 291, 78, 3, 327, 3, 1121, 3, 1068, 3, 219, 3, 1062, 3, 1674,
    3, 1071, 3, 1073, 3, 739, 3, 741, 3, 78, 539, 3, 977, 3, 375, 3,
    979, 785, 3, 207, 3, 3, 1437, 3, 946, 3, 1792, 3, 1794, 3, 1796, 3,
    1798, 3, 1800, 3, 1802, 1804, 3, 3, 1806, 3, 1808, 3, 1810, 1812, 3, 3,
    1814, 3, 1816, 3, 1818, 3, 1820, 3, 1822, 3, 1824, 3, 1826, 3, 1828, 3,
    1830, 3, 950, 3, 948, 3, 1832, 3, 1834, 3, 1836, 3, 1838, 3, 1840, 3,
    1842, 3, 1844, 3, 1846, 3, 1848, 3, 1850, 1852, 3, 3, 1854, 3, 1856, 3,
    1858, 3, 1860, 3, 518, 3, 1863, 3, 303, 242, 3, 3, 4, 231, 3, 78,
    3, 1121, 370, 3, 3, 204, 3, 78, 240, 3, 1437, 3, 780, 3, 327, 3,
    78, 240, 785, 3, 1312, 3, 3, 915, 3, 917, 3, 1668, 3, 868, 3, 871,
    3, 1670, 3, 1672, 3, 1000, 3, 1674, 3, 1068, 3, 737, 3, 842, 3, 219,
    3, 1062, 3, 1071, 3, 1073, 3, 739, 3, 741, 3, 1068, 3, 78, 240, 242,
    3, 3, 1378, 3, 868, 3, 871, 3, 616, 78, 3, 327, 3, 327, 3, 219,
    3, 1062, 3, 327, 3, 327, 3, 327, 3, 1062, 3, 1071, 3, 1073, 3, 739,
    3, 741, 3, 4, 1997, 3, 78, 85, 3, 2001, 3, 219, 3, 2003, 3, 2007,
    3, 2007, 3, 219, 3, 4, 5, 3, 375,
];

pub static REGEX_PATTERNS: &[&str] = &[
    r#"[A-Z_][A-Z0-9_]*"#, // [0]
    r#"^(NOT)$"#, // [1]
    r#"[A-Z0-9_#]*[A-Z#][A-Z0-9_#]*"#, // [2]
    r#"^(CASE|CROSS|FULL|IGNORE|INNER|INTERVAL|JOIN|LEFT|NOT|NULL|ON|ORDER|OUTER|PARTITION|RESPECT|RIGHT|ROWS|SELECT|SET|UNION|USING)$"#, // [3]
    r#"\"?[A-Z][A-Z0-9_]*\"?"#, // [4]
];

pub static HINT_STRING_INDICES: &[u32] = &[
    0, 6, 7, 8, 10, 11, 12, 13, 14, 15,
    16, 17, 18, 19, 20, 21, 22, 23, 24, 25,
    26, 27, 28, 29, 30, 31, 32, 33, 34, 35,
    36, 37, 25, 40, 37, 13, 14, 20, 21, 28,
    12, 26, 33, 35, 17, 52, 53, 53, 52, 57,
    11, 60, 62, 19, 15, 16, 18, 22, 23, 24,
    27, 28, 29, 30, 31, 32, 34, 36, 15, 16,
    67, 22, 23, 68, 69, 60, 70, 71, 72, 73,
    74, 75, 13, 28, 82, 87, 89, 91, 92, 95,
    97, 11, 98, 99, 100, 101, 102, 103, 104, 105,
    106, 40, 107, 108, 109, 110, 111, 112, 113, 114,
    115, 116, 117, 118, 119, 120, 121, 100, 13, 20,
    26, 123, 37, 123, 98, 22, 128, 102, 34, 105,
    133, 134, 118, 134, 118, 11, 98, 99, 100, 101,
    102, 103, 104, 106, 40, 107, 108, 109, 110, 111,
    112, 113, 114, 115, 116, 117, 118, 119, 120, 121,
    15, 16, 18, 22, 23, 137, 24, 27, 28, 29,
    30, 31, 32, 34, 36, 137, 119, 12, 28, 103,
    142, 99, 101, 104, 106, 107, 108, 109, 111, 110,
    112, 113, 114, 115, 116, 117, 120, 121, 164, 32,
    36, 30, 18, 24, 27, 29, 31, 23, 15, 16,
    177, 73, 75, 71, 67, 68, 69, 70, 72, 74,
    189, 190, 189, 190, 197, 202, 205, 208, 209, 210,
    211, 212, 213, 214, 215, 216, 218, 224, 232, 235,
    238, 243, 244, 245, 246, 247, 248, 249, 250, 251,
    252, 253, 254, 255, 218, 249, 258, 251, 252, 250,
    252, 247, 253, 247, 253, 248, 254, 248, 254, 255,
    218, 271, 274, 276, 277, 280, 212, 106, 82, 214,
    287, 214, 287, 287, 293, 224, 89, 297, 298, 300,
    301, 302, 305, 306, 307, 298, 308, 309, 310, 311,
    312, 313, 314, 315, 309, 310, 311, 308, 307, 315,
    314, 312, 313, 329, 331, 333, 335, 336, 337, 338,
    339, 340, 341, 342, 343, 344, 345, 346, 347, 348,
    349, 351, 352, 353, 354, 355, 356, 358, 359, 360,
    361, 362, 363, 364, 365, 366, 367, 368, 123, 372,
    379, 380, 381, 382, 383, 384, 385, 307, 298, 308,
    309, 310, 311, 312, 280, 313, 302, 386, 387, 314,
    315, 388, 302, 387, 385, 312, 280, 313, 386, 388,
    393, 394, 395, 396, 401, 402, 387, 405, 406, 407,
    407, 406, 410, 82, 112, 414, 416, 417, 418, 419,
    420, 421, 82, 112, 414, 425, 426, 428, 429, 430,
    276, 431, 432, 436, 440, 442, 446, 447, 449, 450,
    451, 452, 453, 454, 455, 456, 457, 458, 459, 460,
    461, 462, 463, 464, 465, 466, 467, 468, 469, 470,
    471, 472, 473, 474, 475, 476, 477, 478, 479, 480,
    482, 483, 486, 488, 489, 490, 491, 492, 493, 499,
    501, 502, 501, 502, 506, 509, 513, 517, 525, 527,
    529, 530, 238, 274, 18, 532, 533, 24, 26, 28,
    29, 32, 33, 115, 35, 534, 36, 532, 238, 18,
    533, 24, 26, 29, 32, 33, 115, 35, 534, 36,
    533, 534, 386, 546, 547, 385, 312, 388, 556, 559,
    560, 561, 562, 563, 565, 566, 567, 568, 571, 572,
    573, 574, 575, 576, 577, 578, 579, 580, 581, 582,
    583, 584, 585, 585, 584, 589, 591, 593, 594, 595,
    596, 597, 598, 425, 602, 577, 18, 603, 604, 24,
    27, 605, 577, 603, 604, 605, 577, 603, 605, 603,
    605, 604, 610, 614, 113, 114, 619, 628, 630, 244,
    477, 491, 559, 631, 632, 633, 634, 62, 635, 636,
    637, 501, 638, 639, 640, 641, 501, 639, 639, 641,
    647, 635, 477, 633, 634, 634, 633, 653, 640, 636,
    637, 244, 631, 631, 660, 662, 638, 665, 666, 667,
    668, 668, 666, 667, 673, 674, 675, 674, 675, 674,
    675, 673, 680, 252, 682, 682, 252, 632, 635, 632,
    687, 689, 690, 247, 248, 249, 250, 251, 252, 253,
    254, 255, 218, 692, 692, 113, 114, 697, 82, 224,
    698, 499, 17, 509, 513, 698, 214, 379, 706, 706,
    710, 101, 104, 117, 712, 610, 91, 713, 501, 714,
    436, 714, 717, 719, 721, 712, 713, 501, 712, 712,
    713, 713, 725, 727, 729, 719, 730, 730, 5, 742,
    743, 744, 745, 746, 747, 748, 748, 750, 750, 755,
    358, 359, 756, 757, 360, 758, 361, 362, 363, 364,
    365, 759, 366, 760, 761, 762, 763, 764, 767, 768,
    769, 772, 384, 457, 92, 775, 775, 782, 786, 787,
    790, 791, 792, 793, 794, 795, 796, 797, 798, 799,
    800, 786, 372, 767, 768, 767, 768, 809, 811, 812,
    790, 813, 814, 815, 816, 817, 818, 791, 819, 820,
    821, 822, 823, 824, 825, 826, 827, 828, 829, 830,
    831, 258, 164, 82, 833, 834, 82, 833, 833, 834,
    837, 840, 831, 62, 845, 501, 164, 846, 425, 846,
    274, 719, 101, 849, 849, 852, 845, 501, 845, 855,
    857, 414, 861, 862, 863, 864, 865, 866, 870, 873,
    874, 875, 876, 877, 878, 879, 880, 881, 882, 883,
    885, 886, 887, 888, 889, 890, 891, 892, 893, 894,
    895, 896, 897, 898, 899, 900, 277, 905, 906, 907,
    908, 909, 910, 911, 912, 913, 918, 919, 920, 921,
    922, 923, 924, 926, 927, 928, 930, 931, 932, 933,
    934, 935, 936, 937, 938, 939, 940, 941, 942, 943,
    82, 101, 104, 112, 117, 414, 425, 954, 698, 501,
    955, 62, 501, 382, 501, 309, 310, 501, 966, 967,
    309, 310, 967, 966, 385, 307, 298, 308, 309, 310,
    311, 312, 280, 313, 386, 314, 315, 388, 309, 310,
    967, 786, 406, 372, 502, 767, 768, 407, 97, 235,
    1001, 1002, 1003, 1006, 1007, 1008, 287, 1010, 1010, 719,
    852, 647, 425, 1013, 1014, 1015, 82, 873, 246, 380,
    426, 517, 546, 100, 837, 101, 883, 271, 442, 943,
    189, 104, 1016, 190, 855, 112, 1017, 692, 116, 117,
    119, 414, 425, 82, 246, 380, 426, 517, 546, 100,
    837, 101, 883, 271, 442, 943, 189, 104, 1016, 190,
    855, 112, 1017, 692, 116, 117, 119, 414, 425, 1021,
    1022, 1025, 1026, 1027, 1028, 1029, 1030, 1031, 1032, 1034,
    1035, 1036, 1037, 1038, 852, 1042, 1031, 1043, 1043, 1038,
    1047, 1048, 1049, 1050, 935, 1008, 610, 197, 1053, 1054,
    1055, 1056, 1057, 218, 1058, 1059, 870, 1060, 425, 1008,
    1055, 1057, 1053, 1058, 1060, 935, 1054, 1056, 1059, 1057,
    1075, 1082, 1089, 565, 1094, 410, 1097, 1098, 1105, 1108,
    1109, 1112, 5, 1112, 1123, 1124, 1125, 1126, 1127, 1128,
    1129, 1130, 583, 425, 1139, 1142, 367, 1146, 1147, 1149,
    1008, 610, 1053, 1055, 1057, 218, 1058, 1060, 425, 1156,
    1157, 1158, 1008, 610, 1055, 1057, 218, 1058, 1060, 425,
    1159, 1160, 1161, 1162, 1163, 1164, 1165, 1166, 1167, 792,
    1168, 1169, 1170, 1171, 1172, 1175, 1176, 1108, 1109, 1181,
    954, 1182, 1183, 1184, 1185, 1186, 335, 882, 834, 1189,
    1190, 1191, 1192, 1193, 1194, 1195, 1196, 1197, 1198, 566,
    1199, 1200, 1201, 1203, 1205, 82, 506, 112, 414, 425,
    1207, 1208, 1209, 1210, 1211, 1212, 1213, 1214, 1215, 1216,
    1217, 1054, 1218, 97, 235, 790, 791, 792, 793, 794,
    795, 796, 797, 798, 799, 800, 235, 1219, 1221, 1224,
    1225, 1226, 746, 1227, 1228, 1229, 1227, 1228, 1227, 1228,
    1233, 119, 1229, 1237, 1238, 1239, 1240, 1243, 164, 653,
    565, 567, 568, 567, 568, 1260, 1261, 1262, 1233, 1263,
    1264, 1265, 1266, 790, 819, 792, 1171, 1267, 1268, 794,
    1269, 795, 1270, 797, 1271, 800, 1272, 1267, 1268, 794,
    1269, 795, 1270, 797, 1271, 800, 1272, 1287, 1288, 1289,
    1290, 1291, 1292, 1097, 1293, 1294, 1295, 1296, 1297, 954,
    698, 955, 955, 1055, 87, 1057, 1008, 1057, 1307, 1308,
    1309, 1310, 1316, 1318, 1319, 1320, 1321, 1322, 1323, 1324,
    1325, 1326, 1327, 937, 1329, 117, 1329, 1333, 1334, 1335,
    1336, 1337, 1338, 1339, 1340, 1341, 1342, 1343, 1344, 1345,
    1347, 1348, 1349, 1350, 1351, 1352, 1353, 250, 1354, 1355,
    1356, 1357, 1358, 1016, 1366, 101, 117, 1017, 1379, 1380,
    1381, 1382, 793, 1383, 1384, 1056, 1385, 1386, 251, 1387,
    1388, 1389, 1390, 1391, 1392, 1393, 1394, 1395, 1396, 1397,
    1398, 1400, 1402, 1407, 1409, 1410, 1411, 1413, 1414, 1417,
    1418, 1419, 1420, 1421, 1422, 1423, 1424, 1425, 1426, 1427,
    1428, 1429, 1430, 101, 104, 1016, 117, 82, 112, 1438,
    1439, 255, 1440, 1441, 1442, 385, 1444, 1445, 1446, 1447,
    1448, 1449, 1450, 1451, 1452, 1453, 1461, 1462, 1463, 1464,
    1465, 1466, 1467, 1468, 1469, 1470, 1471, 1472, 1473, 1474,
    1475, 1476, 1477, 1478, 1479, 335, 882, 287, 1288, 1008,
    610, 1053, 1055, 1057, 1058, 1038, 852, 1105, 1060, 425,
    1484, 1485, 1486, 1487, 1488, 1489, 1491, 1492, 1493, 1494,
    1495, 1496, 1497, 1498, 1499, 1501, 1502, 1503, 1504, 1505,
    1506, 1507, 1508, 1509, 1510, 1511, 1513, 1514, 1515, 1516,
    1517, 311, 235, 1520, 1521, 1522, 1523, 1524, 1526, 1527,
    1528, 1529, 1533, 1535, 1536, 1538, 1539, 1540, 1541, 1542,
    1544, 1545, 1546, 1547, 1548, 1549, 1550, 1551, 1552, 1553,
    11, 305, 1554, 1554, 796, 1556, 1557, 1558, 1561, 1562,
    1563, 1565, 1008, 109, 1566, 1567, 1568, 1569, 1570, 1571,
    1572, 1573, 276, 252, 277, 1017, 565, 1510, 1577, 1577,
    1582, 1583, 1584, 1585, 1586, 1587, 1588, 1589, 1590, 1591,
    1592, 1593, 1594, 1595, 1596, 1597, 1598, 1599, 1600, 1601,
    1602, 1603, 1604, 1605, 1606, 1607, 1608, 1609, 1610, 1611,
    1612, 1615, 1616, 1617, 1618, 1619, 1620, 1621, 1622, 1623,
    1624, 1625, 1626, 1627, 1629, 1630, 1631, 1632, 1633, 1634,
    1635, 1636, 1637, 1638, 1639, 381, 1641, 1641, 1645, 1646,
    1647, 1648, 1649, 1650, 1651, 1652, 1653, 1654, 1655, 1656,
    1657, 1658, 1659, 11, 895, 935, 1008, 610, 91, 1054,
    1055, 1056, 1057, 218, 630, 1059, 870, 425, 82, 425,
    1677, 1679, 1682, 1683, 1684, 1059, 935, 1054, 1691, 1692,
    1693, 1694, 1695, 1696, 1699, 1700, 1703, 1704, 1705, 1706,
    1707, 1708, 1709, 368, 470, 471, 472, 473, 786, 826,
    829, 506, 864, 943, 1006, 1184, 1237, 1322, 1334, 1347,
    1354, 1358, 1477, 533, 1517, 1556, 1582, 28, 53, 1718,
    1719, 767, 768, 1720, 123, 1730, 1389, 1705, 1737, 1291,
    1292, 1744, 1747, 1718, 1719, 421, 1759, 1759, 1720, 1762,
    1763, 1764, 1765, 1766, 1767, 1768, 1769, 1770, 1771, 1772,
    1773, 1774, 1775, 1776, 1777, 1778, 1779, 1780, 1781, 1782,
    1783, 1784, 1785, 1786, 1788, 1789, 1790, 380, 546, 442,
    855, 692, 1864, 1865, 1866, 1867, 1868, 1869, 1870, 1871,
    1872, 1873, 1874, 1876, 1877, 1878, 1879, 1880, 1881, 1882,
    1883, 1884, 1885, 1886, 1887, 1888, 1889, 1890, 1891, 1892,
    17, 509, 513, 1894, 1898, 697, 1900, 1901, 1902, 1903,
    1904, 425, 628, 1907, 1909, 1910, 1911, 1912, 1913, 1914,
    1915, 1916, 1098, 1917, 1918, 1918, 1920, 1920, 1925, 1926,
    1927, 1928, 1929, 1930, 1931, 1932, 1933, 1934, 1935, 1936,
    1940, 1941, 1943, 1944, 1945, 1950, 1951, 1952, 1954, 1955,
    1956, 1957, 1958, 1959, 1960, 1961, 1962, 1963, 1964, 1965,
    1966, 1967, 1968, 1969, 1970, 1971, 1972, 1973, 1974, 1975,
    1976, 1977, 1978, 1979, 1980, 1981, 1982, 1983, 1984, 1985,
    1986, 431, 1320, 1989, 1990, 1991, 1992, 798, 799, 1995,
    1008, 610, 197, 1053, 1055, 1057, 218, 630, 1058, 1060,
    425, 1996, 1999, 218, 1511, 2011, 2012, 2013, 1034, 687,
    2016, 2018, 1108, 1594, 2022, 2023, 2024, 2025, 2026, 2027,
    2028,
];

pub static SIMPLE_HINTS: &[SimpleHintData] = &[
    SimpleHintData::empty(), // [0] reserved for no hint
    SimpleHintData { raw_values_start: 0, raw_values_count: 1, token_types_start: 1, token_types_count: 0 }, // [1]
    SimpleHintData { raw_values_start: 1, raw_values_count: 1, token_types_start: 2, token_types_count: 0 }, // [2]
    SimpleHintData { raw_values_start: 2, raw_values_count: 1, token_types_start: 3, token_types_count: 0 }, // [3]
    SimpleHintData { raw_values_start: 3, raw_values_count: 1, token_types_start: 4, token_types_count: 0 }, // [4]
    SimpleHintData { raw_values_start: 4, raw_values_count: 1, token_types_start: 5, token_types_count: 0 }, // [5]
    SimpleHintData { raw_values_start: 5, raw_values_count: 27, token_types_start: 32, token_types_count: 0 }, // [6]
    SimpleHintData { raw_values_start: 32, raw_values_count: 1, token_types_start: 33, token_types_count: 0 }, // [7]
    SimpleHintData { raw_values_start: 33, raw_values_count: 1, token_types_start: 34, token_types_count: 0 }, // [8]
    SimpleHintData { raw_values_start: 34, raw_values_count: 1, token_types_start: 35, token_types_count: 0 }, // [9]
    SimpleHintData { raw_values_start: 35, raw_values_count: 1, token_types_start: 36, token_types_count: 0 }, // [10]
    SimpleHintData { raw_values_start: 36, raw_values_count: 1, token_types_start: 37, token_types_count: 0 }, // [11]
    SimpleHintData { raw_values_start: 37, raw_values_count: 1, token_types_start: 38, token_types_count: 0 }, // [12]
    SimpleHintData { raw_values_start: 38, raw_values_count: 1, token_types_start: 39, token_types_count: 0 }, // [13]
    SimpleHintData { raw_values_start: 39, raw_values_count: 1, token_types_start: 40, token_types_count: 0 }, // [14]
    SimpleHintData { raw_values_start: 40, raw_values_count: 1, token_types_start: 41, token_types_count: 0 }, // [15]
    SimpleHintData { raw_values_start: 41, raw_values_count: 1, token_types_start: 42, token_types_count: 0 }, // [16]
    SimpleHintData { raw_values_start: 42, raw_values_count: 1, token_types_start: 43, token_types_count: 0 }, // [17]
    SimpleHintData { raw_values_start: 43, raw_values_count: 1, token_types_start: 44, token_types_count: 0 }, // [18]
    SimpleHintData { raw_values_start: 44, raw_values_count: 1, token_types_start: 45, token_types_count: 0 }, // [19]
    SimpleHintData { raw_values_start: 45, raw_values_count: 2, token_types_start: 47, token_types_count: 0 }, // [20]
    SimpleHintData { raw_values_start: 47, raw_values_count: 1, token_types_start: 48, token_types_count: 0 }, // [21]
    SimpleHintData { raw_values_start: 48, raw_values_count: 1, token_types_start: 49, token_types_count: 0 }, // [22]
    SimpleHintData { raw_values_start: 49, raw_values_count: 1, token_types_start: 50, token_types_count: 0 }, // [23]
    SimpleHintData { raw_values_start: 50, raw_values_count: 1, token_types_start: 51, token_types_count: 0 }, // [24]
    SimpleHintData { raw_values_start: 51, raw_values_count: 1, token_types_start: 52, token_types_count: 0 }, // [25]
    SimpleHintData { raw_values_start: 52, raw_values_count: 1, token_types_start: 53, token_types_count: 0 }, // [26]
    SimpleHintData { raw_values_start: 53, raw_values_count: 1, token_types_start: 54, token_types_count: 0 }, // [27]
    SimpleHintData { raw_values_start: 54, raw_values_count: 14, token_types_start: 68, token_types_count: 0 }, // [28]
    SimpleHintData { raw_values_start: 68, raw_values_count: 14, token_types_start: 82, token_types_count: 0 }, // [29]
    SimpleHintData { raw_values_start: 82, raw_values_count: 2, token_types_start: 84, token_types_count: 0 }, // [30]
    SimpleHintData { raw_values_start: 84, raw_values_count: 1, token_types_start: 85, token_types_count: 0 }, // [31]
    SimpleHintData { raw_values_start: 85, raw_values_count: 1, token_types_start: 86, token_types_count: 0 }, // [32]
    SimpleHintData { raw_values_start: 86, raw_values_count: 1, token_types_start: 87, token_types_count: 0 }, // [33]
    SimpleHintData { raw_values_start: 87, raw_values_count: 1, token_types_start: 88, token_types_count: 0 }, // [34]
    SimpleHintData { raw_values_start: 88, raw_values_count: 1, token_types_start: 89, token_types_count: 0 }, // [35]
    SimpleHintData { raw_values_start: 89, raw_values_count: 1, token_types_start: 90, token_types_count: 0 }, // [36]
    SimpleHintData { raw_values_start: 90, raw_values_count: 0, token_types_start: 90, token_types_count: 1 }, // [37]
    SimpleHintData { raw_values_start: 91, raw_values_count: 26, token_types_start: 117, token_types_count: 0 }, // [38]
    SimpleHintData { raw_values_start: 117, raw_values_count: 1, token_types_start: 118, token_types_count: 0 }, // [39]
    SimpleHintData { raw_values_start: 118, raw_values_count: 5, token_types_start: 123, token_types_count: 0 }, // [40]
    SimpleHintData { raw_values_start: 123, raw_values_count: 1, token_types_start: 124, token_types_count: 0 }, // [41]
    SimpleHintData { raw_values_start: 124, raw_values_count: 1, token_types_start: 125, token_types_count: 0 }, // [42]
    SimpleHintData { raw_values_start: 125, raw_values_count: 1, token_types_start: 126, token_types_count: 0 }, // [43]
    SimpleHintData { raw_values_start: 126, raw_values_count: 1, token_types_start: 127, token_types_count: 0 }, // [44]
    SimpleHintData { raw_values_start: 127, raw_values_count: 1, token_types_start: 128, token_types_count: 0 }, // [45]
    SimpleHintData { raw_values_start: 128, raw_values_count: 1, token_types_start: 129, token_types_count: 0 }, // [46]
    SimpleHintData { raw_values_start: 129, raw_values_count: 1, token_types_start: 130, token_types_count: 0 }, // [47]
    SimpleHintData { raw_values_start: 130, raw_values_count: 1, token_types_start: 131, token_types_count: 0 }, // [48]
    SimpleHintData { raw_values_start: 131, raw_values_count: 2, token_types_start: 133, token_types_count: 0 }, // [49]
    SimpleHintData { raw_values_start: 133, raw_values_count: 1, token_types_start: 134, token_types_count: 0 }, // [50]
    SimpleHintData { raw_values_start: 134, raw_values_count: 1, token_types_start: 135, token_types_count: 0 }, // [51]
    SimpleHintData { raw_values_start: 135, raw_values_count: 25, token_types_start: 160, token_types_count: 0 }, // [52]
    SimpleHintData { raw_values_start: 160, raw_values_count: 15, token_types_start: 175, token_types_count: 0 }, // [53]
    SimpleHintData { raw_values_start: 175, raw_values_count: 1, token_types_start: 176, token_types_count: 0 }, // [54]
    SimpleHintData { raw_values_start: 176, raw_values_count: 1, token_types_start: 177, token_types_count: 0 }, // [55]
    SimpleHintData { raw_values_start: 177, raw_values_count: 2, token_types_start: 179, token_types_count: 0 }, // [56]
    SimpleHintData { raw_values_start: 179, raw_values_count: 1, token_types_start: 180, token_types_count: 0 }, // [57]
    SimpleHintData { raw_values_start: 180, raw_values_count: 1, token_types_start: 181, token_types_count: 0 }, // [58]
    SimpleHintData { raw_values_start: 181, raw_values_count: 1, token_types_start: 182, token_types_count: 0 }, // [59]
    SimpleHintData { raw_values_start: 182, raw_values_count: 1, token_types_start: 183, token_types_count: 0 }, // [60]
    SimpleHintData { raw_values_start: 183, raw_values_count: 1, token_types_start: 184, token_types_count: 0 }, // [61]
    SimpleHintData { raw_values_start: 184, raw_values_count: 1, token_types_start: 185, token_types_count: 0 }, // [62]
    SimpleHintData { raw_values_start: 185, raw_values_count: 1, token_types_start: 186, token_types_count: 0 }, // [63]
    SimpleHintData { raw_values_start: 186, raw_values_count: 1, token_types_start: 187, token_types_count: 0 }, // [64]
    SimpleHintData { raw_values_start: 187, raw_values_count: 1, token_types_start: 188, token_types_count: 0 }, // [65]
    SimpleHintData { raw_values_start: 188, raw_values_count: 1, token_types_start: 189, token_types_count: 0 }, // [66]
    SimpleHintData { raw_values_start: 189, raw_values_count: 1, token_types_start: 190, token_types_count: 0 }, // [67]
    SimpleHintData { raw_values_start: 190, raw_values_count: 1, token_types_start: 191, token_types_count: 0 }, // [68]
    SimpleHintData { raw_values_start: 191, raw_values_count: 1, token_types_start: 192, token_types_count: 0 }, // [69]
    SimpleHintData { raw_values_start: 192, raw_values_count: 1, token_types_start: 193, token_types_count: 0 }, // [70]
    SimpleHintData { raw_values_start: 193, raw_values_count: 1, token_types_start: 194, token_types_count: 0 }, // [71]
    SimpleHintData { raw_values_start: 194, raw_values_count: 1, token_types_start: 195, token_types_count: 0 }, // [72]
    SimpleHintData { raw_values_start: 195, raw_values_count: 1, token_types_start: 196, token_types_count: 0 }, // [73]
    SimpleHintData { raw_values_start: 196, raw_values_count: 1, token_types_start: 197, token_types_count: 0 }, // [74]
    SimpleHintData { raw_values_start: 197, raw_values_count: 1, token_types_start: 198, token_types_count: 0 }, // [75]
    SimpleHintData { raw_values_start: 198, raw_values_count: 1, token_types_start: 199, token_types_count: 0 }, // [76]
    SimpleHintData { raw_values_start: 199, raw_values_count: 1, token_types_start: 200, token_types_count: 0 }, // [77]
    SimpleHintData { raw_values_start: 200, raw_values_count: 1, token_types_start: 201, token_types_count: 0 }, // [78]
    SimpleHintData { raw_values_start: 201, raw_values_count: 1, token_types_start: 202, token_types_count: 0 }, // [79]
    SimpleHintData { raw_values_start: 202, raw_values_count: 1, token_types_start: 203, token_types_count: 0 }, // [80]
    SimpleHintData { raw_values_start: 203, raw_values_count: 1, token_types_start: 204, token_types_count: 0 }, // [81]
    SimpleHintData { raw_values_start: 204, raw_values_count: 1, token_types_start: 205, token_types_count: 0 }, // [82]
    SimpleHintData { raw_values_start: 205, raw_values_count: 1, token_types_start: 206, token_types_count: 0 }, // [83]
    SimpleHintData { raw_values_start: 206, raw_values_count: 1, token_types_start: 207, token_types_count: 0 }, // [84]
    SimpleHintData { raw_values_start: 207, raw_values_count: 1, token_types_start: 208, token_types_count: 0 }, // [85]
    SimpleHintData { raw_values_start: 208, raw_values_count: 1, token_types_start: 209, token_types_count: 0 }, // [86]
    SimpleHintData { raw_values_start: 209, raw_values_count: 1, token_types_start: 210, token_types_count: 0 }, // [87]
    SimpleHintData { raw_values_start: 210, raw_values_count: 1, token_types_start: 211, token_types_count: 0 }, // [88]
    SimpleHintData { raw_values_start: 211, raw_values_count: 1, token_types_start: 212, token_types_count: 0 }, // [89]
    SimpleHintData { raw_values_start: 212, raw_values_count: 1, token_types_start: 213, token_types_count: 0 }, // [90]
    SimpleHintData { raw_values_start: 213, raw_values_count: 1, token_types_start: 214, token_types_count: 0 }, // [91]
    SimpleHintData { raw_values_start: 214, raw_values_count: 1, token_types_start: 215, token_types_count: 0 }, // [92]
    SimpleHintData { raw_values_start: 215, raw_values_count: 1, token_types_start: 216, token_types_count: 0 }, // [93]
    SimpleHintData { raw_values_start: 216, raw_values_count: 1, token_types_start: 217, token_types_count: 0 }, // [94]
    SimpleHintData { raw_values_start: 217, raw_values_count: 1, token_types_start: 218, token_types_count: 0 }, // [95]
    SimpleHintData { raw_values_start: 218, raw_values_count: 1, token_types_start: 219, token_types_count: 0 }, // [96]
    SimpleHintData { raw_values_start: 219, raw_values_count: 1, token_types_start: 220, token_types_count: 0 }, // [97]
    SimpleHintData { raw_values_start: 220, raw_values_count: 2, token_types_start: 222, token_types_count: 0 }, // [98]
    SimpleHintData { raw_values_start: 222, raw_values_count: 1, token_types_start: 223, token_types_count: 0 }, // [99]
    SimpleHintData { raw_values_start: 223, raw_values_count: 1, token_types_start: 224, token_types_count: 0 }, // [100]
    SimpleHintData { raw_values_start: 224, raw_values_count: 1, token_types_start: 225, token_types_count: 0 }, // [101]
    SimpleHintData { raw_values_start: 225, raw_values_count: 1, token_types_start: 226, token_types_count: 0 }, // [102]
    SimpleHintData { raw_values_start: 226, raw_values_count: 1, token_types_start: 227, token_types_count: 0 }, // [103]
    SimpleHintData { raw_values_start: 227, raw_values_count: 1, token_types_start: 228, token_types_count: 0 }, // [104]
    SimpleHintData { raw_values_start: 228, raw_values_count: 1, token_types_start: 229, token_types_count: 0 }, // [105]
    SimpleHintData { raw_values_start: 229, raw_values_count: 1, token_types_start: 230, token_types_count: 0 }, // [106]
    SimpleHintData { raw_values_start: 230, raw_values_count: 1, token_types_start: 231, token_types_count: 0 }, // [107]
    SimpleHintData { raw_values_start: 231, raw_values_count: 1, token_types_start: 232, token_types_count: 0 }, // [108]
    SimpleHintData { raw_values_start: 232, raw_values_count: 1, token_types_start: 233, token_types_count: 0 }, // [109]
    SimpleHintData { raw_values_start: 233, raw_values_count: 1, token_types_start: 234, token_types_count: 0 }, // [110]
    SimpleHintData { raw_values_start: 234, raw_values_count: 1, token_types_start: 235, token_types_count: 0 }, // [111]
    SimpleHintData { raw_values_start: 235, raw_values_count: 1, token_types_start: 236, token_types_count: 0 }, // [112]
    SimpleHintData { raw_values_start: 236, raw_values_count: 1, token_types_start: 237, token_types_count: 0 }, // [113]
    SimpleHintData { raw_values_start: 237, raw_values_count: 1, token_types_start: 238, token_types_count: 0 }, // [114]
    SimpleHintData { raw_values_start: 238, raw_values_count: 1, token_types_start: 239, token_types_count: 0 }, // [115]
    SimpleHintData { raw_values_start: 239, raw_values_count: 0, token_types_start: 239, token_types_count: 1 }, // [116]
    SimpleHintData { raw_values_start: 240, raw_values_count: 1, token_types_start: 241, token_types_count: 0 }, // [117]
    SimpleHintData { raw_values_start: 241, raw_values_count: 1, token_types_start: 242, token_types_count: 0 }, // [118]
    SimpleHintData { raw_values_start: 242, raw_values_count: 1, token_types_start: 243, token_types_count: 0 }, // [119]
    SimpleHintData { raw_values_start: 243, raw_values_count: 1, token_types_start: 244, token_types_count: 0 }, // [120]
    SimpleHintData { raw_values_start: 244, raw_values_count: 1, token_types_start: 245, token_types_count: 0 }, // [121]
    SimpleHintData { raw_values_start: 245, raw_values_count: 10, token_types_start: 255, token_types_count: 0 }, // [122]
    SimpleHintData { raw_values_start: 255, raw_values_count: 1, token_types_start: 256, token_types_count: 0 }, // [123]
    SimpleHintData { raw_values_start: 256, raw_values_count: 1, token_types_start: 257, token_types_count: 0 }, // [124]
    SimpleHintData { raw_values_start: 257, raw_values_count: 2, token_types_start: 259, token_types_count: 0 }, // [125]
    SimpleHintData { raw_values_start: 259, raw_values_count: 2, token_types_start: 261, token_types_count: 0 }, // [126]
    SimpleHintData { raw_values_start: 261, raw_values_count: 2, token_types_start: 263, token_types_count: 0 }, // [127]
    SimpleHintData { raw_values_start: 263, raw_values_count: 1, token_types_start: 264, token_types_count: 0 }, // [128]
    SimpleHintData { raw_values_start: 264, raw_values_count: 1, token_types_start: 265, token_types_count: 0 }, // [129]
    SimpleHintData { raw_values_start: 265, raw_values_count: 2, token_types_start: 267, token_types_count: 0 }, // [130]
    SimpleHintData { raw_values_start: 267, raw_values_count: 1, token_types_start: 268, token_types_count: 0 }, // [131]
    SimpleHintData { raw_values_start: 268, raw_values_count: 1, token_types_start: 269, token_types_count: 0 }, // [132]
    SimpleHintData { raw_values_start: 269, raw_values_count: 2, token_types_start: 271, token_types_count: 0 }, // [133]
    SimpleHintData { raw_values_start: 271, raw_values_count: 1, token_types_start: 272, token_types_count: 0 }, // [134]
    SimpleHintData { raw_values_start: 272, raw_values_count: 1, token_types_start: 273, token_types_count: 0 }, // [135]
    SimpleHintData { raw_values_start: 273, raw_values_count: 2, token_types_start: 275, token_types_count: 0 }, // [136]
    SimpleHintData { raw_values_start: 275, raw_values_count: 1, token_types_start: 276, token_types_count: 0 }, // [137]
    SimpleHintData { raw_values_start: 276, raw_values_count: 2, token_types_start: 278, token_types_count: 0 }, // [138]
    SimpleHintData { raw_values_start: 278, raw_values_count: 3, token_types_start: 281, token_types_count: 0 }, // [139]
    SimpleHintData { raw_values_start: 281, raw_values_count: 2, token_types_start: 283, token_types_count: 0 }, // [140]
    SimpleHintData { raw_values_start: 283, raw_values_count: 1, token_types_start: 284, token_types_count: 0 }, // [141]
    SimpleHintData { raw_values_start: 284, raw_values_count: 1, token_types_start: 285, token_types_count: 0 }, // [142]
    SimpleHintData { raw_values_start: 285, raw_values_count: 2, token_types_start: 287, token_types_count: 0 }, // [143]
    SimpleHintData { raw_values_start: 287, raw_values_count: 1, token_types_start: 288, token_types_count: 0 }, // [144]
    SimpleHintData { raw_values_start: 288, raw_values_count: 1, token_types_start: 289, token_types_count: 0 }, // [145]
    SimpleHintData { raw_values_start: 289, raw_values_count: 1, token_types_start: 290, token_types_count: 0 }, // [146]
    SimpleHintData { raw_values_start: 290, raw_values_count: 1, token_types_start: 291, token_types_count: 0 }, // [147]
    SimpleHintData { raw_values_start: 291, raw_values_count: 1, token_types_start: 292, token_types_count: 0 }, // [148]
    SimpleHintData { raw_values_start: 292, raw_values_count: 1, token_types_start: 293, token_types_count: 0 }, // [149]
    SimpleHintData { raw_values_start: 293, raw_values_count: 1, token_types_start: 294, token_types_count: 0 }, // [150]
    SimpleHintData { raw_values_start: 294, raw_values_count: 10, token_types_start: 304, token_types_count: 0 }, // [151]
    SimpleHintData { raw_values_start: 304, raw_values_count: 1, token_types_start: 305, token_types_count: 0 }, // [152]
    SimpleHintData { raw_values_start: 305, raw_values_count: 1, token_types_start: 306, token_types_count: 0 }, // [153]
    SimpleHintData { raw_values_start: 306, raw_values_count: 1, token_types_start: 307, token_types_count: 0 }, // [154]
    SimpleHintData { raw_values_start: 307, raw_values_count: 1, token_types_start: 308, token_types_count: 0 }, // [155]
    SimpleHintData { raw_values_start: 308, raw_values_count: 1, token_types_start: 309, token_types_count: 0 }, // [156]
    SimpleHintData { raw_values_start: 309, raw_values_count: 1, token_types_start: 310, token_types_count: 0 }, // [157]
    SimpleHintData { raw_values_start: 310, raw_values_count: 1, token_types_start: 311, token_types_count: 0 }, // [158]
    SimpleHintData { raw_values_start: 311, raw_values_count: 1, token_types_start: 312, token_types_count: 0 }, // [159]
    SimpleHintData { raw_values_start: 312, raw_values_count: 1, token_types_start: 313, token_types_count: 0 }, // [160]
    SimpleHintData { raw_values_start: 313, raw_values_count: 1, token_types_start: 314, token_types_count: 0 }, // [161]
    SimpleHintData { raw_values_start: 314, raw_values_count: 1, token_types_start: 315, token_types_count: 0 }, // [162]
    SimpleHintData { raw_values_start: 315, raw_values_count: 1, token_types_start: 316, token_types_count: 0 }, // [163]
    SimpleHintData { raw_values_start: 316, raw_values_count: 1, token_types_start: 317, token_types_count: 0 }, // [164]
    SimpleHintData { raw_values_start: 317, raw_values_count: 1, token_types_start: 318, token_types_count: 0 }, // [165]
    SimpleHintData { raw_values_start: 318, raw_values_count: 1, token_types_start: 319, token_types_count: 0 }, // [166]
    SimpleHintData { raw_values_start: 319, raw_values_count: 1, token_types_start: 320, token_types_count: 0 }, // [167]
    SimpleHintData { raw_values_start: 320, raw_values_count: 1, token_types_start: 321, token_types_count: 0 }, // [168]
    SimpleHintData { raw_values_start: 321, raw_values_count: 1, token_types_start: 322, token_types_count: 0 }, // [169]
    SimpleHintData { raw_values_start: 322, raw_values_count: 1, token_types_start: 323, token_types_count: 0 }, // [170]
    SimpleHintData { raw_values_start: 323, raw_values_count: 1, token_types_start: 324, token_types_count: 0 }, // [171]
    SimpleHintData { raw_values_start: 324, raw_values_count: 1, token_types_start: 325, token_types_count: 0 }, // [172]
    SimpleHintData { raw_values_start: 325, raw_values_count: 1, token_types_start: 326, token_types_count: 0 }, // [173]
    SimpleHintData { raw_values_start: 326, raw_values_count: 1, token_types_start: 327, token_types_count: 0 }, // [174]
    SimpleHintData { raw_values_start: 327, raw_values_count: 1, token_types_start: 328, token_types_count: 0 }, // [175]
    SimpleHintData { raw_values_start: 328, raw_values_count: 1, token_types_start: 329, token_types_count: 0 }, // [176]
    SimpleHintData { raw_values_start: 329, raw_values_count: 1, token_types_start: 330, token_types_count: 0 }, // [177]
    SimpleHintData { raw_values_start: 330, raw_values_count: 1, token_types_start: 331, token_types_count: 0 }, // [178]
    SimpleHintData { raw_values_start: 331, raw_values_count: 1, token_types_start: 332, token_types_count: 0 }, // [179]
    SimpleHintData { raw_values_start: 332, raw_values_count: 1, token_types_start: 333, token_types_count: 0 }, // [180]
    SimpleHintData { raw_values_start: 333, raw_values_count: 1, token_types_start: 334, token_types_count: 0 }, // [181]
    SimpleHintData { raw_values_start: 334, raw_values_count: 1, token_types_start: 335, token_types_count: 0 }, // [182]
    SimpleHintData { raw_values_start: 335, raw_values_count: 1, token_types_start: 336, token_types_count: 0 }, // [183]
    SimpleHintData { raw_values_start: 336, raw_values_count: 1, token_types_start: 337, token_types_count: 0 }, // [184]
    SimpleHintData { raw_values_start: 337, raw_values_count: 12, token_types_start: 349, token_types_count: 0 }, // [185]
    SimpleHintData { raw_values_start: 349, raw_values_count: 1, token_types_start: 350, token_types_count: 0 }, // [186]
    SimpleHintData { raw_values_start: 350, raw_values_count: 1, token_types_start: 351, token_types_count: 0 }, // [187]
    SimpleHintData { raw_values_start: 351, raw_values_count: 1, token_types_start: 352, token_types_count: 0 }, // [188]
    SimpleHintData { raw_values_start: 352, raw_values_count: 1, token_types_start: 353, token_types_count: 0 }, // [189]
    SimpleHintData { raw_values_start: 353, raw_values_count: 1, token_types_start: 354, token_types_count: 0 }, // [190]
    SimpleHintData { raw_values_start: 354, raw_values_count: 1, token_types_start: 355, token_types_count: 0 }, // [191]
    SimpleHintData { raw_values_start: 355, raw_values_count: 1, token_types_start: 356, token_types_count: 0 }, // [192]
    SimpleHintData { raw_values_start: 356, raw_values_count: 15, token_types_start: 371, token_types_count: 1 }, // [193]
    SimpleHintData { raw_values_start: 372, raw_values_count: 2, token_types_start: 374, token_types_count: 0 }, // [194]
    SimpleHintData { raw_values_start: 374, raw_values_count: 5, token_types_start: 379, token_types_count: 1 }, // [195]
    SimpleHintData { raw_values_start: 380, raw_values_count: 1, token_types_start: 381, token_types_count: 0 }, // [196]
    SimpleHintData { raw_values_start: 381, raw_values_count: 1, token_types_start: 382, token_types_count: 0 }, // [197]
    SimpleHintData { raw_values_start: 382, raw_values_count: 1, token_types_start: 383, token_types_count: 0 }, // [198]
    SimpleHintData { raw_values_start: 383, raw_values_count: 1, token_types_start: 384, token_types_count: 0 }, // [199]
    SimpleHintData { raw_values_start: 384, raw_values_count: 1, token_types_start: 385, token_types_count: 0 }, // [200]
    SimpleHintData { raw_values_start: 385, raw_values_count: 1, token_types_start: 386, token_types_count: 0 }, // [201]
    SimpleHintData { raw_values_start: 386, raw_values_count: 1, token_types_start: 387, token_types_count: 0 }, // [202]
    SimpleHintData { raw_values_start: 387, raw_values_count: 1, token_types_start: 388, token_types_count: 0 }, // [203]
    SimpleHintData { raw_values_start: 388, raw_values_count: 2, token_types_start: 390, token_types_count: 0 }, // [204]
    SimpleHintData { raw_values_start: 390, raw_values_count: 1, token_types_start: 391, token_types_count: 0 }, // [205]
    SimpleHintData { raw_values_start: 391, raw_values_count: 1, token_types_start: 392, token_types_count: 0 }, // [206]
    SimpleHintData { raw_values_start: 392, raw_values_count: 1, token_types_start: 393, token_types_count: 0 }, // [207]
    SimpleHintData { raw_values_start: 393, raw_values_count: 3, token_types_start: 396, token_types_count: 0 }, // [208]
    SimpleHintData { raw_values_start: 396, raw_values_count: 1, token_types_start: 397, token_types_count: 0 }, // [209]
    SimpleHintData { raw_values_start: 397, raw_values_count: 1, token_types_start: 398, token_types_count: 0 }, // [210]
    SimpleHintData { raw_values_start: 398, raw_values_count: 1, token_types_start: 399, token_types_count: 0 }, // [211]
    SimpleHintData { raw_values_start: 399, raw_values_count: 1, token_types_start: 400, token_types_count: 0 }, // [212]
    SimpleHintData { raw_values_start: 400, raw_values_count: 1, token_types_start: 401, token_types_count: 0 }, // [213]
    SimpleHintData { raw_values_start: 401, raw_values_count: 1, token_types_start: 402, token_types_count: 0 }, // [214]
    SimpleHintData { raw_values_start: 402, raw_values_count: 4, token_types_start: 406, token_types_count: 0 }, // [215]
    SimpleHintData { raw_values_start: 406, raw_values_count: 1, token_types_start: 407, token_types_count: 0 }, // [216]
    SimpleHintData { raw_values_start: 407, raw_values_count: 1, token_types_start: 408, token_types_count: 0 }, // [217]
    SimpleHintData { raw_values_start: 408, raw_values_count: 1, token_types_start: 409, token_types_count: 0 }, // [218]
    SimpleHintData { raw_values_start: 409, raw_values_count: 1, token_types_start: 410, token_types_count: 0 }, // [219]
    SimpleHintData { raw_values_start: 410, raw_values_count: 1, token_types_start: 411, token_types_count: 0 }, // [220]
    SimpleHintData { raw_values_start: 411, raw_values_count: 1, token_types_start: 412, token_types_count: 0 }, // [221]
    SimpleHintData { raw_values_start: 412, raw_values_count: 1, token_types_start: 413, token_types_count: 0 }, // [222]
    SimpleHintData { raw_values_start: 413, raw_values_count: 1, token_types_start: 414, token_types_count: 0 }, // [223]
    SimpleHintData { raw_values_start: 414, raw_values_count: 1, token_types_start: 415, token_types_count: 0 }, // [224]
    SimpleHintData { raw_values_start: 415, raw_values_count: 1, token_types_start: 416, token_types_count: 0 }, // [225]
    SimpleHintData { raw_values_start: 416, raw_values_count: 1, token_types_start: 417, token_types_count: 0 }, // [226]
    SimpleHintData { raw_values_start: 417, raw_values_count: 1, token_types_start: 418, token_types_count: 0 }, // [227]
    SimpleHintData { raw_values_start: 418, raw_values_count: 1, token_types_start: 419, token_types_count: 0 }, // [228]
    SimpleHintData { raw_values_start: 419, raw_values_count: 1, token_types_start: 420, token_types_count: 0 }, // [229]
    SimpleHintData { raw_values_start: 420, raw_values_count: 1, token_types_start: 421, token_types_count: 0 }, // [230]
    SimpleHintData { raw_values_start: 421, raw_values_count: 1, token_types_start: 422, token_types_count: 0 }, // [231]
    SimpleHintData { raw_values_start: 422, raw_values_count: 1, token_types_start: 423, token_types_count: 0 }, // [232]
    SimpleHintData { raw_values_start: 423, raw_values_count: 1, token_types_start: 424, token_types_count: 0 }, // [233]
    SimpleHintData { raw_values_start: 424, raw_values_count: 1, token_types_start: 425, token_types_count: 0 }, // [234]
    SimpleHintData { raw_values_start: 425, raw_values_count: 1, token_types_start: 426, token_types_count: 0 }, // [235]
    SimpleHintData { raw_values_start: 426, raw_values_count: 1, token_types_start: 427, token_types_count: 0 }, // [236]
    SimpleHintData { raw_values_start: 427, raw_values_count: 1, token_types_start: 428, token_types_count: 0 }, // [237]
    SimpleHintData { raw_values_start: 428, raw_values_count: 1, token_types_start: 429, token_types_count: 0 }, // [238]
    SimpleHintData { raw_values_start: 429, raw_values_count: 1, token_types_start: 430, token_types_count: 0 }, // [239]
    SimpleHintData { raw_values_start: 430, raw_values_count: 1, token_types_start: 431, token_types_count: 0 }, // [240]
    SimpleHintData { raw_values_start: 431, raw_values_count: 1, token_types_start: 432, token_types_count: 0 }, // [241]
    SimpleHintData { raw_values_start: 432, raw_values_count: 1, token_types_start: 433, token_types_count: 0 }, // [242]
    SimpleHintData { raw_values_start: 433, raw_values_count: 1, token_types_start: 434, token_types_count: 0 }, // [243]
    SimpleHintData { raw_values_start: 434, raw_values_count: 1, token_types_start: 435, token_types_count: 0 }, // [244]
    SimpleHintData { raw_values_start: 435, raw_values_count: 1, token_types_start: 436, token_types_count: 0 }, // [245]
    SimpleHintData { raw_values_start: 436, raw_values_count: 1, token_types_start: 437, token_types_count: 0 }, // [246]
    SimpleHintData { raw_values_start: 437, raw_values_count: 1, token_types_start: 438, token_types_count: 0 }, // [247]
    SimpleHintData { raw_values_start: 438, raw_values_count: 1, token_types_start: 439, token_types_count: 0 }, // [248]
    SimpleHintData { raw_values_start: 439, raw_values_count: 1, token_types_start: 440, token_types_count: 0 }, // [249]
    SimpleHintData { raw_values_start: 440, raw_values_count: 1, token_types_start: 441, token_types_count: 0 }, // [250]
    SimpleHintData { raw_values_start: 441, raw_values_count: 1, token_types_start: 442, token_types_count: 0 }, // [251]
    SimpleHintData { raw_values_start: 442, raw_values_count: 1, token_types_start: 443, token_types_count: 0 }, // [252]
    SimpleHintData { raw_values_start: 443, raw_values_count: 1, token_types_start: 444, token_types_count: 0 }, // [253]
    SimpleHintData { raw_values_start: 444, raw_values_count: 1, token_types_start: 445, token_types_count: 0 }, // [254]
    SimpleHintData { raw_values_start: 445, raw_values_count: 1, token_types_start: 446, token_types_count: 0 }, // [255]
    SimpleHintData { raw_values_start: 446, raw_values_count: 1, token_types_start: 447, token_types_count: 0 }, // [256]
    SimpleHintData { raw_values_start: 447, raw_values_count: 1, token_types_start: 448, token_types_count: 0 }, // [257]
    SimpleHintData { raw_values_start: 448, raw_values_count: 1, token_types_start: 449, token_types_count: 0 }, // [258]
    SimpleHintData { raw_values_start: 449, raw_values_count: 1, token_types_start: 450, token_types_count: 0 }, // [259]
    SimpleHintData { raw_values_start: 450, raw_values_count: 1, token_types_start: 451, token_types_count: 0 }, // [260]
    SimpleHintData { raw_values_start: 451, raw_values_count: 1, token_types_start: 452, token_types_count: 0 }, // [261]
    SimpleHintData { raw_values_start: 452, raw_values_count: 1, token_types_start: 453, token_types_count: 0 }, // [262]
    SimpleHintData { raw_values_start: 453, raw_values_count: 1, token_types_start: 454, token_types_count: 0 }, // [263]
    SimpleHintData { raw_values_start: 454, raw_values_count: 1, token_types_start: 455, token_types_count: 0 }, // [264]
    SimpleHintData { raw_values_start: 455, raw_values_count: 1, token_types_start: 456, token_types_count: 0 }, // [265]
    SimpleHintData { raw_values_start: 456, raw_values_count: 1, token_types_start: 457, token_types_count: 0 }, // [266]
    SimpleHintData { raw_values_start: 457, raw_values_count: 1, token_types_start: 458, token_types_count: 0 }, // [267]
    SimpleHintData { raw_values_start: 458, raw_values_count: 1, token_types_start: 459, token_types_count: 0 }, // [268]
    SimpleHintData { raw_values_start: 459, raw_values_count: 1, token_types_start: 460, token_types_count: 0 }, // [269]
    SimpleHintData { raw_values_start: 460, raw_values_count: 2, token_types_start: 462, token_types_count: 0 }, // [270]
    SimpleHintData { raw_values_start: 462, raw_values_count: 1, token_types_start: 463, token_types_count: 0 }, // [271]
    SimpleHintData { raw_values_start: 463, raw_values_count: 1, token_types_start: 464, token_types_count: 0 }, // [272]
    SimpleHintData { raw_values_start: 464, raw_values_count: 1, token_types_start: 465, token_types_count: 0 }, // [273]
    SimpleHintData { raw_values_start: 465, raw_values_count: 1, token_types_start: 466, token_types_count: 0 }, // [274]
    SimpleHintData { raw_values_start: 466, raw_values_count: 1, token_types_start: 467, token_types_count: 0 }, // [275]
    SimpleHintData { raw_values_start: 467, raw_values_count: 1, token_types_start: 468, token_types_count: 0 }, // [276]
    SimpleHintData { raw_values_start: 468, raw_values_count: 1, token_types_start: 469, token_types_count: 0 }, // [277]
    SimpleHintData { raw_values_start: 469, raw_values_count: 1, token_types_start: 470, token_types_count: 0 }, // [278]
    SimpleHintData { raw_values_start: 470, raw_values_count: 1, token_types_start: 471, token_types_count: 0 }, // [279]
    SimpleHintData { raw_values_start: 471, raw_values_count: 1, token_types_start: 472, token_types_count: 0 }, // [280]
    SimpleHintData { raw_values_start: 472, raw_values_count: 15, token_types_start: 487, token_types_count: 0 }, // [281]
    SimpleHintData { raw_values_start: 487, raw_values_count: 1, token_types_start: 488, token_types_count: 0 }, // [282]
    SimpleHintData { raw_values_start: 488, raw_values_count: 12, token_types_start: 500, token_types_count: 0 }, // [283]
    SimpleHintData { raw_values_start: 500, raw_values_count: 1, token_types_start: 501, token_types_count: 0 }, // [284]
    SimpleHintData { raw_values_start: 501, raw_values_count: 1, token_types_start: 502, token_types_count: 0 }, // [285]
    SimpleHintData { raw_values_start: 502, raw_values_count: 1, token_types_start: 503, token_types_count: 0 }, // [286]
    SimpleHintData { raw_values_start: 503, raw_values_count: 1, token_types_start: 504, token_types_count: 0 }, // [287]
    SimpleHintData { raw_values_start: 504, raw_values_count: 1, token_types_start: 505, token_types_count: 0 }, // [288]
    SimpleHintData { raw_values_start: 505, raw_values_count: 2, token_types_start: 507, token_types_count: 0 }, // [289]
    SimpleHintData { raw_values_start: 507, raw_values_count: 0, token_types_start: 507, token_types_count: 1 }, // [290]
    SimpleHintData { raw_values_start: 508, raw_values_count: 1, token_types_start: 509, token_types_count: 0 }, // [291]
    SimpleHintData { raw_values_start: 509, raw_values_count: 1, token_types_start: 510, token_types_count: 0 }, // [292]
    SimpleHintData { raw_values_start: 510, raw_values_count: 1, token_types_start: 511, token_types_count: 0 }, // [293]
    SimpleHintData { raw_values_start: 511, raw_values_count: 1, token_types_start: 512, token_types_count: 0 }, // [294]
    SimpleHintData { raw_values_start: 512, raw_values_count: 1, token_types_start: 513, token_types_count: 0 }, // [295]
    SimpleHintData { raw_values_start: 513, raw_values_count: 1, token_types_start: 514, token_types_count: 0 }, // [296]
    SimpleHintData { raw_values_start: 514, raw_values_count: 4, token_types_start: 518, token_types_count: 0 }, // [297]
    SimpleHintData { raw_values_start: 518, raw_values_count: 1, token_types_start: 519, token_types_count: 0 }, // [298]
    SimpleHintData { raw_values_start: 519, raw_values_count: 1, token_types_start: 520, token_types_count: 0 }, // [299]
    SimpleHintData { raw_values_start: 520, raw_values_count: 1, token_types_start: 521, token_types_count: 0 }, // [300]
    SimpleHintData { raw_values_start: 521, raw_values_count: 1, token_types_start: 522, token_types_count: 0 }, // [301]
    SimpleHintData { raw_values_start: 522, raw_values_count: 1, token_types_start: 523, token_types_count: 0 }, // [302]
    SimpleHintData { raw_values_start: 523, raw_values_count: 1, token_types_start: 524, token_types_count: 0 }, // [303]
    SimpleHintData { raw_values_start: 524, raw_values_count: 1, token_types_start: 525, token_types_count: 0 }, // [304]
    SimpleHintData { raw_values_start: 525, raw_values_count: 1, token_types_start: 526, token_types_count: 0 }, // [305]
    SimpleHintData { raw_values_start: 526, raw_values_count: 1, token_types_start: 527, token_types_count: 0 }, // [306]
    SimpleHintData { raw_values_start: 527, raw_values_count: 1, token_types_start: 528, token_types_count: 0 }, // [307]
    SimpleHintData { raw_values_start: 528, raw_values_count: 1, token_types_start: 529, token_types_count: 0 }, // [308]
    SimpleHintData { raw_values_start: 529, raw_values_count: 1, token_types_start: 530, token_types_count: 0 }, // [309]
    SimpleHintData { raw_values_start: 530, raw_values_count: 1, token_types_start: 531, token_types_count: 0 }, // [310]
    SimpleHintData { raw_values_start: 531, raw_values_count: 2, token_types_start: 533, token_types_count: 0 }, // [311]
    SimpleHintData { raw_values_start: 533, raw_values_count: 1, token_types_start: 534, token_types_count: 0 }, // [312]
    SimpleHintData { raw_values_start: 534, raw_values_count: 1, token_types_start: 535, token_types_count: 0 }, // [313]
    SimpleHintData { raw_values_start: 535, raw_values_count: 1, token_types_start: 536, token_types_count: 0 }, // [314]
    SimpleHintData { raw_values_start: 536, raw_values_count: 1, token_types_start: 537, token_types_count: 0 }, // [315]
    SimpleHintData { raw_values_start: 537, raw_values_count: 1, token_types_start: 538, token_types_count: 0 }, // [316]
    SimpleHintData { raw_values_start: 538, raw_values_count: 1, token_types_start: 539, token_types_count: 0 }, // [317]
    SimpleHintData { raw_values_start: 539, raw_values_count: 1, token_types_start: 540, token_types_count: 0 }, // [318]
    SimpleHintData { raw_values_start: 540, raw_values_count: 1, token_types_start: 541, token_types_count: 0 }, // [319]
    SimpleHintData { raw_values_start: 541, raw_values_count: 1, token_types_start: 542, token_types_count: 0 }, // [320]
    SimpleHintData { raw_values_start: 542, raw_values_count: 1, token_types_start: 543, token_types_count: 0 }, // [321]
    SimpleHintData { raw_values_start: 543, raw_values_count: 1, token_types_start: 544, token_types_count: 0 }, // [322]
    SimpleHintData { raw_values_start: 544, raw_values_count: 1, token_types_start: 545, token_types_count: 0 }, // [323]
    SimpleHintData { raw_values_start: 545, raw_values_count: 7, token_types_start: 552, token_types_count: 0 }, // [324]
    SimpleHintData { raw_values_start: 552, raw_values_count: 4, token_types_start: 556, token_types_count: 0 }, // [325]
    SimpleHintData { raw_values_start: 556, raw_values_count: 3, token_types_start: 559, token_types_count: 0 }, // [326]
    SimpleHintData { raw_values_start: 559, raw_values_count: 1, token_types_start: 560, token_types_count: 0 }, // [327]
    SimpleHintData { raw_values_start: 560, raw_values_count: 1, token_types_start: 561, token_types_count: 0 }, // [328]
    SimpleHintData { raw_values_start: 561, raw_values_count: 1, token_types_start: 562, token_types_count: 0 }, // [329]
    SimpleHintData { raw_values_start: 562, raw_values_count: 1, token_types_start: 563, token_types_count: 0 }, // [330]
    SimpleHintData { raw_values_start: 563, raw_values_count: 1, token_types_start: 564, token_types_count: 0 }, // [331]
    SimpleHintData { raw_values_start: 564, raw_values_count: 2, token_types_start: 566, token_types_count: 0 }, // [332]
    SimpleHintData { raw_values_start: 566, raw_values_count: 1, token_types_start: 567, token_types_count: 0 }, // [333]
    SimpleHintData { raw_values_start: 567, raw_values_count: 1, token_types_start: 568, token_types_count: 0 }, // [334]
    SimpleHintData { raw_values_start: 568, raw_values_count: 1, token_types_start: 569, token_types_count: 0 }, // [335]
    SimpleHintData { raw_values_start: 569, raw_values_count: 17, token_types_start: 586, token_types_count: 0 }, // [336]
    SimpleHintData { raw_values_start: 586, raw_values_count: 2, token_types_start: 588, token_types_count: 0 }, // [337]
    SimpleHintData { raw_values_start: 588, raw_values_count: 1, token_types_start: 589, token_types_count: 0 }, // [338]
    SimpleHintData { raw_values_start: 589, raw_values_count: 1, token_types_start: 590, token_types_count: 0 }, // [339]
    SimpleHintData { raw_values_start: 590, raw_values_count: 1, token_types_start: 591, token_types_count: 0 }, // [340]
    SimpleHintData { raw_values_start: 591, raw_values_count: 1, token_types_start: 592, token_types_count: 0 }, // [341]
    SimpleHintData { raw_values_start: 592, raw_values_count: 3, token_types_start: 595, token_types_count: 0 }, // [342]
    SimpleHintData { raw_values_start: 595, raw_values_count: 1, token_types_start: 596, token_types_count: 0 }, // [343]
    SimpleHintData { raw_values_start: 596, raw_values_count: 1, token_types_start: 597, token_types_count: 0 }, // [344]
    SimpleHintData { raw_values_start: 597, raw_values_count: 1, token_types_start: 598, token_types_count: 0 }, // [345]
    SimpleHintData { raw_values_start: 598, raw_values_count: 1, token_types_start: 599, token_types_count: 0 }, // [346]
    SimpleHintData { raw_values_start: 599, raw_values_count: 1, token_types_start: 600, token_types_count: 0 }, // [347]
    SimpleHintData { raw_values_start: 600, raw_values_count: 1, token_types_start: 601, token_types_count: 0 }, // [348]
    SimpleHintData { raw_values_start: 601, raw_values_count: 2, token_types_start: 603, token_types_count: 0 }, // [349]
    SimpleHintData { raw_values_start: 603, raw_values_count: 1, token_types_start: 604, token_types_count: 0 }, // [350]
    SimpleHintData { raw_values_start: 604, raw_values_count: 1, token_types_start: 605, token_types_count: 0 }, // [351]
    SimpleHintData { raw_values_start: 605, raw_values_count: 1, token_types_start: 606, token_types_count: 0 }, // [352]
    SimpleHintData { raw_values_start: 606, raw_values_count: 1, token_types_start: 607, token_types_count: 0 }, // [353]
    SimpleHintData { raw_values_start: 607, raw_values_count: 1, token_types_start: 608, token_types_count: 0 }, // [354]
    SimpleHintData { raw_values_start: 608, raw_values_count: 3, token_types_start: 611, token_types_count: 0 }, // [355]
    SimpleHintData { raw_values_start: 611, raw_values_count: 1, token_types_start: 612, token_types_count: 0 }, // [356]
    SimpleHintData { raw_values_start: 612, raw_values_count: 1, token_types_start: 613, token_types_count: 0 }, // [357]
    SimpleHintData { raw_values_start: 613, raw_values_count: 1, token_types_start: 614, token_types_count: 0 }, // [358]
    SimpleHintData { raw_values_start: 614, raw_values_count: 3, token_types_start: 617, token_types_count: 0 }, // [359]
    SimpleHintData { raw_values_start: 617, raw_values_count: 2, token_types_start: 619, token_types_count: 0 }, // [360]
    SimpleHintData { raw_values_start: 619, raw_values_count: 1, token_types_start: 620, token_types_count: 0 }, // [361]
    SimpleHintData { raw_values_start: 620, raw_values_count: 1, token_types_start: 621, token_types_count: 0 }, // [362]
    SimpleHintData { raw_values_start: 621, raw_values_count: 1, token_types_start: 622, token_types_count: 0 }, // [363]
    SimpleHintData { raw_values_start: 622, raw_values_count: 1, token_types_start: 623, token_types_count: 0 }, // [364]
    SimpleHintData { raw_values_start: 623, raw_values_count: 2, token_types_start: 625, token_types_count: 0 }, // [365]
    SimpleHintData { raw_values_start: 625, raw_values_count: 1, token_types_start: 626, token_types_count: 0 }, // [366]
    SimpleHintData { raw_values_start: 626, raw_values_count: 1, token_types_start: 627, token_types_count: 0 }, // [367]
    SimpleHintData { raw_values_start: 627, raw_values_count: 2, token_types_start: 629, token_types_count: 0 }, // [368]
    SimpleHintData { raw_values_start: 629, raw_values_count: 1, token_types_start: 630, token_types_count: 0 }, // [369]
    SimpleHintData { raw_values_start: 630, raw_values_count: 1, token_types_start: 631, token_types_count: 0 }, // [370]
    SimpleHintData { raw_values_start: 631, raw_values_count: 1, token_types_start: 632, token_types_count: 0 }, // [371]
    SimpleHintData { raw_values_start: 632, raw_values_count: 1, token_types_start: 633, token_types_count: 0 }, // [372]
    SimpleHintData { raw_values_start: 633, raw_values_count: 11, token_types_start: 644, token_types_count: 0 }, // [373]
    SimpleHintData { raw_values_start: 644, raw_values_count: 1, token_types_start: 645, token_types_count: 0 }, // [374]
    SimpleHintData { raw_values_start: 645, raw_values_count: 3, token_types_start: 648, token_types_count: 0 }, // [375]
    SimpleHintData { raw_values_start: 648, raw_values_count: 3, token_types_start: 651, token_types_count: 0 }, // [376]
    SimpleHintData { raw_values_start: 651, raw_values_count: 4, token_types_start: 655, token_types_count: 0 }, // [377]
    SimpleHintData { raw_values_start: 655, raw_values_count: 1, token_types_start: 656, token_types_count: 0 }, // [378]
    SimpleHintData { raw_values_start: 656, raw_values_count: 3, token_types_start: 659, token_types_count: 0 }, // [379]
    SimpleHintData { raw_values_start: 659, raw_values_count: 1, token_types_start: 660, token_types_count: 0 }, // [380]
    SimpleHintData { raw_values_start: 660, raw_values_count: 1, token_types_start: 661, token_types_count: 0 }, // [381]
    SimpleHintData { raw_values_start: 661, raw_values_count: 3, token_types_start: 664, token_types_count: 0 }, // [382]
    SimpleHintData { raw_values_start: 664, raw_values_count: 7, token_types_start: 671, token_types_count: 0 }, // [383]
    SimpleHintData { raw_values_start: 671, raw_values_count: 1, token_types_start: 672, token_types_count: 0 }, // [384]
    SimpleHintData { raw_values_start: 672, raw_values_count: 1, token_types_start: 673, token_types_count: 0 }, // [385]
    SimpleHintData { raw_values_start: 673, raw_values_count: 1, token_types_start: 674, token_types_count: 0 }, // [386]
    SimpleHintData { raw_values_start: 674, raw_values_count: 1, token_types_start: 675, token_types_count: 0 }, // [387]
    SimpleHintData { raw_values_start: 675, raw_values_count: 3, token_types_start: 678, token_types_count: 0 }, // [388]
    SimpleHintData { raw_values_start: 678, raw_values_count: 1, token_types_start: 679, token_types_count: 0 }, // [389]
    SimpleHintData { raw_values_start: 679, raw_values_count: 2, token_types_start: 681, token_types_count: 0 }, // [390]
    SimpleHintData { raw_values_start: 681, raw_values_count: 1, token_types_start: 682, token_types_count: 0 }, // [391]
    SimpleHintData { raw_values_start: 682, raw_values_count: 1, token_types_start: 683, token_types_count: 0 }, // [392]
    SimpleHintData { raw_values_start: 683, raw_values_count: 1, token_types_start: 684, token_types_count: 0 }, // [393]
    SimpleHintData { raw_values_start: 684, raw_values_count: 1, token_types_start: 685, token_types_count: 0 }, // [394]
    SimpleHintData { raw_values_start: 685, raw_values_count: 2, token_types_start: 687, token_types_count: 0 }, // [395]
    SimpleHintData { raw_values_start: 687, raw_values_count: 1, token_types_start: 688, token_types_count: 0 }, // [396]
    SimpleHintData { raw_values_start: 688, raw_values_count: 0, token_types_start: 688, token_types_count: 1 }, // [397]
    SimpleHintData { raw_values_start: 689, raw_values_count: 1, token_types_start: 690, token_types_count: 0 }, // [398]
    SimpleHintData { raw_values_start: 690, raw_values_count: 1, token_types_start: 691, token_types_count: 0 }, // [399]
    SimpleHintData { raw_values_start: 691, raw_values_count: 1, token_types_start: 692, token_types_count: 0 }, // [400]
    SimpleHintData { raw_values_start: 692, raw_values_count: 1, token_types_start: 693, token_types_count: 0 }, // [401]
    SimpleHintData { raw_values_start: 693, raw_values_count: 1, token_types_start: 694, token_types_count: 0 }, // [402]
    SimpleHintData { raw_values_start: 694, raw_values_count: 1, token_types_start: 695, token_types_count: 0 }, // [403]
    SimpleHintData { raw_values_start: 695, raw_values_count: 1, token_types_start: 696, token_types_count: 0 }, // [404]
    SimpleHintData { raw_values_start: 696, raw_values_count: 2, token_types_start: 698, token_types_count: 0 }, // [405]
    SimpleHintData { raw_values_start: 698, raw_values_count: 1, token_types_start: 699, token_types_count: 0 }, // [406]
    SimpleHintData { raw_values_start: 699, raw_values_count: 1, token_types_start: 700, token_types_count: 0 }, // [407]
    SimpleHintData { raw_values_start: 700, raw_values_count: 1, token_types_start: 701, token_types_count: 0 }, // [408]
    SimpleHintData { raw_values_start: 701, raw_values_count: 1, token_types_start: 702, token_types_count: 0 }, // [409]
    SimpleHintData { raw_values_start: 702, raw_values_count: 1, token_types_start: 703, token_types_count: 0 }, // [410]
    SimpleHintData { raw_values_start: 703, raw_values_count: 1, token_types_start: 704, token_types_count: 0 }, // [411]
    SimpleHintData { raw_values_start: 704, raw_values_count: 1, token_types_start: 705, token_types_count: 0 }, // [412]
    SimpleHintData { raw_values_start: 705, raw_values_count: 1, token_types_start: 706, token_types_count: 0 }, // [413]
    SimpleHintData { raw_values_start: 706, raw_values_count: 1, token_types_start: 707, token_types_count: 0 }, // [414]
    SimpleHintData { raw_values_start: 707, raw_values_count: 1, token_types_start: 708, token_types_count: 0 }, // [415]
    SimpleHintData { raw_values_start: 708, raw_values_count: 1, token_types_start: 709, token_types_count: 0 }, // [416]
    SimpleHintData { raw_values_start: 709, raw_values_count: 1, token_types_start: 710, token_types_count: 0 }, // [417]
    SimpleHintData { raw_values_start: 710, raw_values_count: 1, token_types_start: 711, token_types_count: 0 }, // [418]
    SimpleHintData { raw_values_start: 711, raw_values_count: 1, token_types_start: 712, token_types_count: 0 }, // [419]
    SimpleHintData { raw_values_start: 712, raw_values_count: 1, token_types_start: 713, token_types_count: 0 }, // [420]
    SimpleHintData { raw_values_start: 713, raw_values_count: 1, token_types_start: 714, token_types_count: 0 }, // [421]
    SimpleHintData { raw_values_start: 714, raw_values_count: 1, token_types_start: 715, token_types_count: 0 }, // [422]
    SimpleHintData { raw_values_start: 715, raw_values_count: 1, token_types_start: 716, token_types_count: 0 }, // [423]
    SimpleHintData { raw_values_start: 716, raw_values_count: 1, token_types_start: 717, token_types_count: 0 }, // [424]
    SimpleHintData { raw_values_start: 717, raw_values_count: 1, token_types_start: 718, token_types_count: 0 }, // [425]
    SimpleHintData { raw_values_start: 718, raw_values_count: 2, token_types_start: 720, token_types_count: 0 }, // [426]
    SimpleHintData { raw_values_start: 720, raw_values_count: 1, token_types_start: 721, token_types_count: 0 }, // [427]
    SimpleHintData { raw_values_start: 721, raw_values_count: 1, token_types_start: 722, token_types_count: 0 }, // [428]
    SimpleHintData { raw_values_start: 722, raw_values_count: 2, token_types_start: 724, token_types_count: 0 }, // [429]
    SimpleHintData { raw_values_start: 724, raw_values_count: 2, token_types_start: 726, token_types_count: 0 }, // [430]
    SimpleHintData { raw_values_start: 726, raw_values_count: 1, token_types_start: 727, token_types_count: 0 }, // [431]
    SimpleHintData { raw_values_start: 727, raw_values_count: 1, token_types_start: 728, token_types_count: 0 }, // [432]
    SimpleHintData { raw_values_start: 728, raw_values_count: 1, token_types_start: 729, token_types_count: 0 }, // [433]
    SimpleHintData { raw_values_start: 729, raw_values_count: 1, token_types_start: 730, token_types_count: 0 }, // [434]
    SimpleHintData { raw_values_start: 730, raw_values_count: 11, token_types_start: 741, token_types_count: 0 }, // [435]
    SimpleHintData { raw_values_start: 741, raw_values_count: 4, token_types_start: 745, token_types_count: 0 }, // [436]
    SimpleHintData { raw_values_start: 745, raw_values_count: 1, token_types_start: 746, token_types_count: 0 }, // [437]
    SimpleHintData { raw_values_start: 746, raw_values_count: 1, token_types_start: 747, token_types_count: 0 }, // [438]
    SimpleHintData { raw_values_start: 747, raw_values_count: 1, token_types_start: 748, token_types_count: 0 }, // [439]
    SimpleHintData { raw_values_start: 748, raw_values_count: 1, token_types_start: 749, token_types_count: 0 }, // [440]
    SimpleHintData { raw_values_start: 749, raw_values_count: 1, token_types_start: 750, token_types_count: 0 }, // [441]
    SimpleHintData { raw_values_start: 750, raw_values_count: 1, token_types_start: 751, token_types_count: 0 }, // [442]
    SimpleHintData { raw_values_start: 751, raw_values_count: 1, token_types_start: 752, token_types_count: 0 }, // [443]
    SimpleHintData { raw_values_start: 752, raw_values_count: 1, token_types_start: 753, token_types_count: 0 }, // [444]
    SimpleHintData { raw_values_start: 753, raw_values_count: 1, token_types_start: 754, token_types_count: 0 }, // [445]
    SimpleHintData { raw_values_start: 754, raw_values_count: 1, token_types_start: 755, token_types_count: 0 }, // [446]
    SimpleHintData { raw_values_start: 755, raw_values_count: 1, token_types_start: 756, token_types_count: 0 }, // [447]
    SimpleHintData { raw_values_start: 756, raw_values_count: 1, token_types_start: 757, token_types_count: 0 }, // [448]
    SimpleHintData { raw_values_start: 757, raw_values_count: 1, token_types_start: 758, token_types_count: 0 }, // [449]
    SimpleHintData { raw_values_start: 758, raw_values_count: 1, token_types_start: 759, token_types_count: 0 }, // [450]
    SimpleHintData { raw_values_start: 759, raw_values_count: 1, token_types_start: 760, token_types_count: 0 }, // [451]
    SimpleHintData { raw_values_start: 760, raw_values_count: 1, token_types_start: 761, token_types_count: 0 }, // [452]
    SimpleHintData { raw_values_start: 761, raw_values_count: 1, token_types_start: 762, token_types_count: 0 }, // [453]
    SimpleHintData { raw_values_start: 762, raw_values_count: 1, token_types_start: 763, token_types_count: 0 }, // [454]
    SimpleHintData { raw_values_start: 763, raw_values_count: 1, token_types_start: 764, token_types_count: 0 }, // [455]
    SimpleHintData { raw_values_start: 764, raw_values_count: 1, token_types_start: 765, token_types_count: 0 }, // [456]
    SimpleHintData { raw_values_start: 765, raw_values_count: 1, token_types_start: 766, token_types_count: 0 }, // [457]
    SimpleHintData { raw_values_start: 766, raw_values_count: 1, token_types_start: 767, token_types_count: 0 }, // [458]
    SimpleHintData { raw_values_start: 767, raw_values_count: 1, token_types_start: 768, token_types_count: 0 }, // [459]
    SimpleHintData { raw_values_start: 768, raw_values_count: 1, token_types_start: 769, token_types_count: 0 }, // [460]
    SimpleHintData { raw_values_start: 769, raw_values_count: 1, token_types_start: 770, token_types_count: 0 }, // [461]
    SimpleHintData { raw_values_start: 770, raw_values_count: 1, token_types_start: 771, token_types_count: 0 }, // [462]
    SimpleHintData { raw_values_start: 771, raw_values_count: 2, token_types_start: 773, token_types_count: 0 }, // [463]
    SimpleHintData { raw_values_start: 773, raw_values_count: 3, token_types_start: 776, token_types_count: 0 }, // [464]
    SimpleHintData { raw_values_start: 776, raw_values_count: 2, token_types_start: 778, token_types_count: 0 }, // [465]
    SimpleHintData { raw_values_start: 778, raw_values_count: 1, token_types_start: 779, token_types_count: 0 }, // [466]
    SimpleHintData { raw_values_start: 779, raw_values_count: 1, token_types_start: 780, token_types_count: 0 }, // [467]
    SimpleHintData { raw_values_start: 780, raw_values_count: 1, token_types_start: 781, token_types_count: 0 }, // [468]
    SimpleHintData { raw_values_start: 781, raw_values_count: 1, token_types_start: 782, token_types_count: 0 }, // [469]
    SimpleHintData { raw_values_start: 782, raw_values_count: 7, token_types_start: 789, token_types_count: 0 }, // [470]
    SimpleHintData { raw_values_start: 789, raw_values_count: 1, token_types_start: 790, token_types_count: 0 }, // [471]
    SimpleHintData { raw_values_start: 790, raw_values_count: 2, token_types_start: 792, token_types_count: 0 }, // [472]
    SimpleHintData { raw_values_start: 792, raw_values_count: 2, token_types_start: 794, token_types_count: 0 }, // [473]
    SimpleHintData { raw_values_start: 794, raw_values_count: 1, token_types_start: 795, token_types_count: 0 }, // [474]
    SimpleHintData { raw_values_start: 795, raw_values_count: 1, token_types_start: 796, token_types_count: 0 }, // [475]
    SimpleHintData { raw_values_start: 796, raw_values_count: 2, token_types_start: 798, token_types_count: 0 }, // [476]
    SimpleHintData { raw_values_start: 798, raw_values_count: 1, token_types_start: 799, token_types_count: 0 }, // [477]
    SimpleHintData { raw_values_start: 799, raw_values_count: 1, token_types_start: 800, token_types_count: 0 }, // [478]
    SimpleHintData { raw_values_start: 800, raw_values_count: 1, token_types_start: 801, token_types_count: 0 }, // [479]
    SimpleHintData { raw_values_start: 801, raw_values_count: 1, token_types_start: 802, token_types_count: 0 }, // [480]
    SimpleHintData { raw_values_start: 802, raw_values_count: 1, token_types_start: 803, token_types_count: 0 }, // [481]
    SimpleHintData { raw_values_start: 803, raw_values_count: 1, token_types_start: 804, token_types_count: 0 }, // [482]
    SimpleHintData { raw_values_start: 804, raw_values_count: 1, token_types_start: 805, token_types_count: 0 }, // [483]
    SimpleHintData { raw_values_start: 805, raw_values_count: 1, token_types_start: 806, token_types_count: 0 }, // [484]
    SimpleHintData { raw_values_start: 806, raw_values_count: 1, token_types_start: 807, token_types_count: 0 }, // [485]
    SimpleHintData { raw_values_start: 807, raw_values_count: 1, token_types_start: 808, token_types_count: 0 }, // [486]
    SimpleHintData { raw_values_start: 808, raw_values_count: 1, token_types_start: 809, token_types_count: 0 }, // [487]
    SimpleHintData { raw_values_start: 809, raw_values_count: 1, token_types_start: 810, token_types_count: 0 }, // [488]
    SimpleHintData { raw_values_start: 810, raw_values_count: 1, token_types_start: 811, token_types_count: 0 }, // [489]
    SimpleHintData { raw_values_start: 811, raw_values_count: 1, token_types_start: 812, token_types_count: 0 }, // [490]
    SimpleHintData { raw_values_start: 812, raw_values_count: 1, token_types_start: 813, token_types_count: 0 }, // [491]
    SimpleHintData { raw_values_start: 813, raw_values_count: 1, token_types_start: 814, token_types_count: 0 }, // [492]
    SimpleHintData { raw_values_start: 814, raw_values_count: 1, token_types_start: 815, token_types_count: 0 }, // [493]
    SimpleHintData { raw_values_start: 815, raw_values_count: 1, token_types_start: 816, token_types_count: 0 }, // [494]
    SimpleHintData { raw_values_start: 816, raw_values_count: 1, token_types_start: 817, token_types_count: 0 }, // [495]
    SimpleHintData { raw_values_start: 817, raw_values_count: 1, token_types_start: 818, token_types_count: 0 }, // [496]
    SimpleHintData { raw_values_start: 818, raw_values_count: 1, token_types_start: 819, token_types_count: 0 }, // [497]
    SimpleHintData { raw_values_start: 819, raw_values_count: 1, token_types_start: 820, token_types_count: 0 }, // [498]
    SimpleHintData { raw_values_start: 820, raw_values_count: 1, token_types_start: 821, token_types_count: 0 }, // [499]
    SimpleHintData { raw_values_start: 821, raw_values_count: 1, token_types_start: 822, token_types_count: 0 }, // [500]
    SimpleHintData { raw_values_start: 822, raw_values_count: 1, token_types_start: 823, token_types_count: 0 }, // [501]
    SimpleHintData { raw_values_start: 823, raw_values_count: 1, token_types_start: 824, token_types_count: 0 }, // [502]
    SimpleHintData { raw_values_start: 824, raw_values_count: 1, token_types_start: 825, token_types_count: 0 }, // [503]
    SimpleHintData { raw_values_start: 825, raw_values_count: 1, token_types_start: 826, token_types_count: 0 }, // [504]
    SimpleHintData { raw_values_start: 826, raw_values_count: 1, token_types_start: 827, token_types_count: 0 }, // [505]
    SimpleHintData { raw_values_start: 827, raw_values_count: 1, token_types_start: 828, token_types_count: 0 }, // [506]
    SimpleHintData { raw_values_start: 828, raw_values_count: 1, token_types_start: 829, token_types_count: 0 }, // [507]
    SimpleHintData { raw_values_start: 829, raw_values_count: 1, token_types_start: 830, token_types_count: 0 }, // [508]
    SimpleHintData { raw_values_start: 830, raw_values_count: 1, token_types_start: 831, token_types_count: 0 }, // [509]
    SimpleHintData { raw_values_start: 831, raw_values_count: 1, token_types_start: 832, token_types_count: 0 }, // [510]
    SimpleHintData { raw_values_start: 832, raw_values_count: 1, token_types_start: 833, token_types_count: 0 }, // [511]
    SimpleHintData { raw_values_start: 833, raw_values_count: 1, token_types_start: 834, token_types_count: 0 }, // [512]
    SimpleHintData { raw_values_start: 834, raw_values_count: 1, token_types_start: 835, token_types_count: 0 }, // [513]
    SimpleHintData { raw_values_start: 835, raw_values_count: 1, token_types_start: 836, token_types_count: 0 }, // [514]
    SimpleHintData { raw_values_start: 836, raw_values_count: 1, token_types_start: 837, token_types_count: 0 }, // [515]
    SimpleHintData { raw_values_start: 837, raw_values_count: 1, token_types_start: 838, token_types_count: 0 }, // [516]
    SimpleHintData { raw_values_start: 838, raw_values_count: 1, token_types_start: 839, token_types_count: 0 }, // [517]
    SimpleHintData { raw_values_start: 839, raw_values_count: 1, token_types_start: 840, token_types_count: 0 }, // [518]
    SimpleHintData { raw_values_start: 840, raw_values_count: 1, token_types_start: 841, token_types_count: 0 }, // [519]
    SimpleHintData { raw_values_start: 841, raw_values_count: 1, token_types_start: 842, token_types_count: 0 }, // [520]
    SimpleHintData { raw_values_start: 842, raw_values_count: 1, token_types_start: 843, token_types_count: 0 }, // [521]
    SimpleHintData { raw_values_start: 843, raw_values_count: 1, token_types_start: 844, token_types_count: 0 }, // [522]
    SimpleHintData { raw_values_start: 844, raw_values_count: 1, token_types_start: 845, token_types_count: 0 }, // [523]
    SimpleHintData { raw_values_start: 845, raw_values_count: 1, token_types_start: 846, token_types_count: 0 }, // [524]
    SimpleHintData { raw_values_start: 846, raw_values_count: 1, token_types_start: 847, token_types_count: 0 }, // [525]
    SimpleHintData { raw_values_start: 847, raw_values_count: 1, token_types_start: 848, token_types_count: 0 }, // [526]
    SimpleHintData { raw_values_start: 848, raw_values_count: 1, token_types_start: 849, token_types_count: 0 }, // [527]
    SimpleHintData { raw_values_start: 849, raw_values_count: 1, token_types_start: 850, token_types_count: 0 }, // [528]
    SimpleHintData { raw_values_start: 850, raw_values_count: 1, token_types_start: 851, token_types_count: 0 }, // [529]
    SimpleHintData { raw_values_start: 851, raw_values_count: 1, token_types_start: 852, token_types_count: 0 }, // [530]
    SimpleHintData { raw_values_start: 852, raw_values_count: 1, token_types_start: 853, token_types_count: 0 }, // [531]
    SimpleHintData { raw_values_start: 853, raw_values_count: 1, token_types_start: 854, token_types_count: 0 }, // [532]
    SimpleHintData { raw_values_start: 854, raw_values_count: 1, token_types_start: 855, token_types_count: 0 }, // [533]
    SimpleHintData { raw_values_start: 855, raw_values_count: 1, token_types_start: 856, token_types_count: 0 }, // [534]
    SimpleHintData { raw_values_start: 856, raw_values_count: 1, token_types_start: 857, token_types_count: 0 }, // [535]
    SimpleHintData { raw_values_start: 857, raw_values_count: 1, token_types_start: 858, token_types_count: 0 }, // [536]
    SimpleHintData { raw_values_start: 858, raw_values_count: 1, token_types_start: 859, token_types_count: 0 }, // [537]
    SimpleHintData { raw_values_start: 859, raw_values_count: 1, token_types_start: 860, token_types_count: 0 }, // [538]
    SimpleHintData { raw_values_start: 860, raw_values_count: 1, token_types_start: 861, token_types_count: 0 }, // [539]
    SimpleHintData { raw_values_start: 861, raw_values_count: 1, token_types_start: 862, token_types_count: 0 }, // [540]
    SimpleHintData { raw_values_start: 862, raw_values_count: 1, token_types_start: 863, token_types_count: 0 }, // [541]
    SimpleHintData { raw_values_start: 863, raw_values_count: 1, token_types_start: 864, token_types_count: 0 }, // [542]
    SimpleHintData { raw_values_start: 864, raw_values_count: 1, token_types_start: 865, token_types_count: 0 }, // [543]
    SimpleHintData { raw_values_start: 865, raw_values_count: 1, token_types_start: 866, token_types_count: 0 }, // [544]
    SimpleHintData { raw_values_start: 866, raw_values_count: 1, token_types_start: 867, token_types_count: 0 }, // [545]
    SimpleHintData { raw_values_start: 867, raw_values_count: 1, token_types_start: 868, token_types_count: 0 }, // [546]
    SimpleHintData { raw_values_start: 868, raw_values_count: 1, token_types_start: 869, token_types_count: 0 }, // [547]
    SimpleHintData { raw_values_start: 869, raw_values_count: 1, token_types_start: 870, token_types_count: 0 }, // [548]
    SimpleHintData { raw_values_start: 870, raw_values_count: 7, token_types_start: 877, token_types_count: 0 }, // [549]
    SimpleHintData { raw_values_start: 877, raw_values_count: 4, token_types_start: 881, token_types_count: 0 }, // [550]
    SimpleHintData { raw_values_start: 881, raw_values_count: 2, token_types_start: 883, token_types_count: 0 }, // [551]
    SimpleHintData { raw_values_start: 883, raw_values_count: 2, token_types_start: 885, token_types_count: 0 }, // [552]
    SimpleHintData { raw_values_start: 885, raw_values_count: 5, token_types_start: 890, token_types_count: 0 }, // [553]
    SimpleHintData { raw_values_start: 890, raw_values_count: 2, token_types_start: 892, token_types_count: 0 }, // [554]
    SimpleHintData { raw_values_start: 892, raw_values_count: 1, token_types_start: 893, token_types_count: 0 }, // [555]
    SimpleHintData { raw_values_start: 893, raw_values_count: 1, token_types_start: 894, token_types_count: 0 }, // [556]
    SimpleHintData { raw_values_start: 894, raw_values_count: 13, token_types_start: 907, token_types_count: 1 }, // [557]
    SimpleHintData { raw_values_start: 908, raw_values_count: 3, token_types_start: 911, token_types_count: 0 }, // [558]
    SimpleHintData { raw_values_start: 911, raw_values_count: 7, token_types_start: 918, token_types_count: 2 }, // [559]
    SimpleHintData { raw_values_start: 920, raw_values_count: 1, token_types_start: 921, token_types_count: 0 }, // [560]
    SimpleHintData { raw_values_start: 921, raw_values_count: 1, token_types_start: 922, token_types_count: 0 }, // [561]
    SimpleHintData { raw_values_start: 922, raw_values_count: 1, token_types_start: 923, token_types_count: 0 }, // [562]
    SimpleHintData { raw_values_start: 923, raw_values_count: 1, token_types_start: 924, token_types_count: 0 }, // [563]
    SimpleHintData { raw_values_start: 924, raw_values_count: 1, token_types_start: 925, token_types_count: 0 }, // [564]
    SimpleHintData { raw_values_start: 925, raw_values_count: 1, token_types_start: 926, token_types_count: 0 }, // [565]
    SimpleHintData { raw_values_start: 926, raw_values_count: 2, token_types_start: 928, token_types_count: 0 }, // [566]
    SimpleHintData { raw_values_start: 928, raw_values_count: 1, token_types_start: 929, token_types_count: 0 }, // [567]
    SimpleHintData { raw_values_start: 929, raw_values_count: 2, token_types_start: 931, token_types_count: 0 }, // [568]
    SimpleHintData { raw_values_start: 931, raw_values_count: 2, token_types_start: 933, token_types_count: 0 }, // [569]
    SimpleHintData { raw_values_start: 933, raw_values_count: 1, token_types_start: 934, token_types_count: 0 }, // [570]
    SimpleHintData { raw_values_start: 934, raw_values_count: 1, token_types_start: 935, token_types_count: 0 }, // [571]
    SimpleHintData { raw_values_start: 935, raw_values_count: 1, token_types_start: 936, token_types_count: 0 }, // [572]
    SimpleHintData { raw_values_start: 936, raw_values_count: 27, token_types_start: 963, token_types_count: 0 }, // [573]
    SimpleHintData { raw_values_start: 963, raw_values_count: 26, token_types_start: 989, token_types_count: 0 }, // [574]
    SimpleHintData { raw_values_start: 989, raw_values_count: 1, token_types_start: 990, token_types_count: 0 }, // [575]
    SimpleHintData { raw_values_start: 990, raw_values_count: 1, token_types_start: 991, token_types_count: 0 }, // [576]
    SimpleHintData { raw_values_start: 991, raw_values_count: 1, token_types_start: 992, token_types_count: 0 }, // [577]
    SimpleHintData { raw_values_start: 992, raw_values_count: 1, token_types_start: 993, token_types_count: 0 }, // [578]
    SimpleHintData { raw_values_start: 993, raw_values_count: 1, token_types_start: 994, token_types_count: 0 }, // [579]
    SimpleHintData { raw_values_start: 994, raw_values_count: 1, token_types_start: 995, token_types_count: 0 }, // [580]
    SimpleHintData { raw_values_start: 995, raw_values_count: 1, token_types_start: 996, token_types_count: 0 }, // [581]
    SimpleHintData { raw_values_start: 996, raw_values_count: 1, token_types_start: 997, token_types_count: 0 }, // [582]
    SimpleHintData { raw_values_start: 997, raw_values_count: 1, token_types_start: 998, token_types_count: 0 }, // [583]
    SimpleHintData { raw_values_start: 998, raw_values_count: 1, token_types_start: 999, token_types_count: 0 }, // [584]
    SimpleHintData { raw_values_start: 999, raw_values_count: 1, token_types_start: 1000, token_types_count: 0 }, // [585]
    SimpleHintData { raw_values_start: 1000, raw_values_count: 1, token_types_start: 1001, token_types_count: 0 }, // [586]
    SimpleHintData { raw_values_start: 1001, raw_values_count: 1, token_types_start: 1002, token_types_count: 0 }, // [587]
    SimpleHintData { raw_values_start: 1002, raw_values_count: 1, token_types_start: 1003, token_types_count: 0 }, // [588]
    SimpleHintData { raw_values_start: 1003, raw_values_count: 2, token_types_start: 1005, token_types_count: 0 }, // [589]
    SimpleHintData { raw_values_start: 1005, raw_values_count: 1, token_types_start: 1006, token_types_count: 0 }, // [590]
    SimpleHintData { raw_values_start: 1006, raw_values_count: 2, token_types_start: 1008, token_types_count: 0 }, // [591]
    SimpleHintData { raw_values_start: 1008, raw_values_count: 1, token_types_start: 1009, token_types_count: 0 }, // [592]
    SimpleHintData { raw_values_start: 1009, raw_values_count: 1, token_types_start: 1010, token_types_count: 0 }, // [593]
    SimpleHintData { raw_values_start: 1010, raw_values_count: 1, token_types_start: 1011, token_types_count: 0 }, // [594]
    SimpleHintData { raw_values_start: 1011, raw_values_count: 1, token_types_start: 1012, token_types_count: 0 }, // [595]
    SimpleHintData { raw_values_start: 1012, raw_values_count: 1, token_types_start: 1013, token_types_count: 0 }, // [596]
    SimpleHintData { raw_values_start: 1013, raw_values_count: 1, token_types_start: 1014, token_types_count: 0 }, // [597]
    SimpleHintData { raw_values_start: 1014, raw_values_count: 15, token_types_start: 1029, token_types_count: 0 }, // [598]
    SimpleHintData { raw_values_start: 1029, raw_values_count: 3, token_types_start: 1032, token_types_count: 0 }, // [599]
    SimpleHintData { raw_values_start: 1032, raw_values_count: 1, token_types_start: 1033, token_types_count: 0 }, // [600]
    SimpleHintData { raw_values_start: 1033, raw_values_count: 1, token_types_start: 1034, token_types_count: 0 }, // [601]
    SimpleHintData { raw_values_start: 1034, raw_values_count: 1, token_types_start: 1035, token_types_count: 0 }, // [602]
    SimpleHintData { raw_values_start: 1035, raw_values_count: 4, token_types_start: 1039, token_types_count: 0 }, // [603]
    SimpleHintData { raw_values_start: 1039, raw_values_count: 1, token_types_start: 1040, token_types_count: 0 }, // [604]
    SimpleHintData { raw_values_start: 1040, raw_values_count: 1, token_types_start: 1041, token_types_count: 0 }, // [605]
    SimpleHintData { raw_values_start: 1041, raw_values_count: 1, token_types_start: 1042, token_types_count: 0 }, // [606]
    SimpleHintData { raw_values_start: 1042, raw_values_count: 1, token_types_start: 1043, token_types_count: 0 }, // [607]
    SimpleHintData { raw_values_start: 1043, raw_values_count: 1, token_types_start: 1044, token_types_count: 0 }, // [608]
    SimpleHintData { raw_values_start: 1044, raw_values_count: 1, token_types_start: 1045, token_types_count: 0 }, // [609]
    SimpleHintData { raw_values_start: 1045, raw_values_count: 3, token_types_start: 1048, token_types_count: 0 }, // [610]
    SimpleHintData { raw_values_start: 1048, raw_values_count: 1, token_types_start: 1049, token_types_count: 0 }, // [611]
    SimpleHintData { raw_values_start: 1049, raw_values_count: 2, token_types_start: 1051, token_types_count: 0 }, // [612]
    SimpleHintData { raw_values_start: 1051, raw_values_count: 0, token_types_start: 1051, token_types_count: 2 }, // [613]
    SimpleHintData { raw_values_start: 1053, raw_values_count: 0, token_types_start: 1053, token_types_count: 1 }, // [614]
    SimpleHintData { raw_values_start: 1054, raw_values_count: 1, token_types_start: 1055, token_types_count: 0 }, // [615]
    SimpleHintData { raw_values_start: 1055, raw_values_count: 1, token_types_start: 1056, token_types_count: 0 }, // [616]
    SimpleHintData { raw_values_start: 1056, raw_values_count: 1, token_types_start: 1057, token_types_count: 0 }, // [617]
    SimpleHintData { raw_values_start: 1057, raw_values_count: 1, token_types_start: 1058, token_types_count: 0 }, // [618]
    SimpleHintData { raw_values_start: 1058, raw_values_count: 1, token_types_start: 1059, token_types_count: 0 }, // [619]
    SimpleHintData { raw_values_start: 1059, raw_values_count: 0, token_types_start: 1059, token_types_count: 1 }, // [620]
    SimpleHintData { raw_values_start: 1060, raw_values_count: 1, token_types_start: 1061, token_types_count: 0 }, // [621]
    SimpleHintData { raw_values_start: 1061, raw_values_count: 1, token_types_start: 1062, token_types_count: 0 }, // [622]
    SimpleHintData { raw_values_start: 1062, raw_values_count: 2, token_types_start: 1064, token_types_count: 0 }, // [623]
    SimpleHintData { raw_values_start: 1064, raw_values_count: 1, token_types_start: 1065, token_types_count: 0 }, // [624]
    SimpleHintData { raw_values_start: 1065, raw_values_count: 1, token_types_start: 1066, token_types_count: 0 }, // [625]
    SimpleHintData { raw_values_start: 1066, raw_values_count: 1, token_types_start: 1067, token_types_count: 0 }, // [626]
    SimpleHintData { raw_values_start: 1067, raw_values_count: 1, token_types_start: 1068, token_types_count: 0 }, // [627]
    SimpleHintData { raw_values_start: 1068, raw_values_count: 1, token_types_start: 1069, token_types_count: 0 }, // [628]
    SimpleHintData { raw_values_start: 1069, raw_values_count: 1, token_types_start: 1070, token_types_count: 0 }, // [629]
    SimpleHintData { raw_values_start: 1070, raw_values_count: 9, token_types_start: 1079, token_types_count: 0 }, // [630]
    SimpleHintData { raw_values_start: 1079, raw_values_count: 1, token_types_start: 1080, token_types_count: 0 }, // [631]
    SimpleHintData { raw_values_start: 1080, raw_values_count: 1, token_types_start: 1081, token_types_count: 0 }, // [632]
    SimpleHintData { raw_values_start: 1081, raw_values_count: 1, token_types_start: 1082, token_types_count: 0 }, // [633]
    SimpleHintData { raw_values_start: 1082, raw_values_count: 8, token_types_start: 1090, token_types_count: 0 }, // [634]
    SimpleHintData { raw_values_start: 1090, raw_values_count: 1, token_types_start: 1091, token_types_count: 0 }, // [635]
    SimpleHintData { raw_values_start: 1091, raw_values_count: 1, token_types_start: 1092, token_types_count: 0 }, // [636]
    SimpleHintData { raw_values_start: 1092, raw_values_count: 1, token_types_start: 1093, token_types_count: 0 }, // [637]
    SimpleHintData { raw_values_start: 1093, raw_values_count: 1, token_types_start: 1094, token_types_count: 0 }, // [638]
    SimpleHintData { raw_values_start: 1094, raw_values_count: 1, token_types_start: 1095, token_types_count: 0 }, // [639]
    SimpleHintData { raw_values_start: 1095, raw_values_count: 1, token_types_start: 1096, token_types_count: 0 }, // [640]
    SimpleHintData { raw_values_start: 1096, raw_values_count: 1, token_types_start: 1097, token_types_count: 0 }, // [641]
    SimpleHintData { raw_values_start: 1097, raw_values_count: 1, token_types_start: 1098, token_types_count: 0 }, // [642]
    SimpleHintData { raw_values_start: 1098, raw_values_count: 1, token_types_start: 1099, token_types_count: 0 }, // [643]
    SimpleHintData { raw_values_start: 1099, raw_values_count: 1, token_types_start: 1100, token_types_count: 0 }, // [644]
    SimpleHintData { raw_values_start: 1100, raw_values_count: 1, token_types_start: 1101, token_types_count: 0 }, // [645]
    SimpleHintData { raw_values_start: 1101, raw_values_count: 1, token_types_start: 1102, token_types_count: 0 }, // [646]
    SimpleHintData { raw_values_start: 1102, raw_values_count: 1, token_types_start: 1103, token_types_count: 0 }, // [647]
    SimpleHintData { raw_values_start: 1103, raw_values_count: 1, token_types_start: 1104, token_types_count: 0 }, // [648]
    SimpleHintData { raw_values_start: 1104, raw_values_count: 1, token_types_start: 1105, token_types_count: 0 }, // [649]
    SimpleHintData { raw_values_start: 1105, raw_values_count: 1, token_types_start: 1106, token_types_count: 0 }, // [650]
    SimpleHintData { raw_values_start: 1106, raw_values_count: 1, token_types_start: 1107, token_types_count: 0 }, // [651]
    SimpleHintData { raw_values_start: 1107, raw_values_count: 1, token_types_start: 1108, token_types_count: 0 }, // [652]
    SimpleHintData { raw_values_start: 1108, raw_values_count: 1, token_types_start: 1109, token_types_count: 0 }, // [653]
    SimpleHintData { raw_values_start: 1109, raw_values_count: 1, token_types_start: 1110, token_types_count: 0 }, // [654]
    SimpleHintData { raw_values_start: 1110, raw_values_count: 1, token_types_start: 1111, token_types_count: 0 }, // [655]
    SimpleHintData { raw_values_start: 1111, raw_values_count: 1, token_types_start: 1112, token_types_count: 0 }, // [656]
    SimpleHintData { raw_values_start: 1112, raw_values_count: 1, token_types_start: 1113, token_types_count: 0 }, // [657]
    SimpleHintData { raw_values_start: 1113, raw_values_count: 1, token_types_start: 1114, token_types_count: 0 }, // [658]
    SimpleHintData { raw_values_start: 1114, raw_values_count: 1, token_types_start: 1115, token_types_count: 0 }, // [659]
    SimpleHintData { raw_values_start: 1115, raw_values_count: 1, token_types_start: 1116, token_types_count: 0 }, // [660]
    SimpleHintData { raw_values_start: 1116, raw_values_count: 3, token_types_start: 1119, token_types_count: 0 }, // [661]
    SimpleHintData { raw_values_start: 1119, raw_values_count: 1, token_types_start: 1120, token_types_count: 0 }, // [662]
    SimpleHintData { raw_values_start: 1120, raw_values_count: 1, token_types_start: 1121, token_types_count: 0 }, // [663]
    SimpleHintData { raw_values_start: 1121, raw_values_count: 1, token_types_start: 1122, token_types_count: 0 }, // [664]
    SimpleHintData { raw_values_start: 1122, raw_values_count: 1, token_types_start: 1123, token_types_count: 0 }, // [665]
    SimpleHintData { raw_values_start: 1123, raw_values_count: 1, token_types_start: 1124, token_types_count: 0 }, // [666]
    SimpleHintData { raw_values_start: 1124, raw_values_count: 1, token_types_start: 1125, token_types_count: 0 }, // [667]
    SimpleHintData { raw_values_start: 1125, raw_values_count: 1, token_types_start: 1126, token_types_count: 0 }, // [668]
    SimpleHintData { raw_values_start: 1126, raw_values_count: 1, token_types_start: 1127, token_types_count: 0 }, // [669]
    SimpleHintData { raw_values_start: 1127, raw_values_count: 1, token_types_start: 1128, token_types_count: 0 }, // [670]
    SimpleHintData { raw_values_start: 1128, raw_values_count: 1, token_types_start: 1129, token_types_count: 0 }, // [671]
    SimpleHintData { raw_values_start: 1129, raw_values_count: 1, token_types_start: 1130, token_types_count: 0 }, // [672]
    SimpleHintData { raw_values_start: 1130, raw_values_count: 1, token_types_start: 1131, token_types_count: 0 }, // [673]
    SimpleHintData { raw_values_start: 1131, raw_values_count: 1, token_types_start: 1132, token_types_count: 0 }, // [674]
    SimpleHintData { raw_values_start: 1132, raw_values_count: 1, token_types_start: 1133, token_types_count: 0 }, // [675]
    SimpleHintData { raw_values_start: 1133, raw_values_count: 1, token_types_start: 1134, token_types_count: 0 }, // [676]
    SimpleHintData { raw_values_start: 1134, raw_values_count: 1, token_types_start: 1135, token_types_count: 0 }, // [677]
    SimpleHintData { raw_values_start: 1135, raw_values_count: 5, token_types_start: 1140, token_types_count: 0 }, // [678]
    SimpleHintData { raw_values_start: 1140, raw_values_count: 1, token_types_start: 1141, token_types_count: 0 }, // [679]
    SimpleHintData { raw_values_start: 1141, raw_values_count: 1, token_types_start: 1142, token_types_count: 0 }, // [680]
    SimpleHintData { raw_values_start: 1142, raw_values_count: 1, token_types_start: 1143, token_types_count: 0 }, // [681]
    SimpleHintData { raw_values_start: 1143, raw_values_count: 1, token_types_start: 1144, token_types_count: 0 }, // [682]
    SimpleHintData { raw_values_start: 1144, raw_values_count: 1, token_types_start: 1145, token_types_count: 0 }, // [683]
    SimpleHintData { raw_values_start: 1145, raw_values_count: 1, token_types_start: 1146, token_types_count: 0 }, // [684]
    SimpleHintData { raw_values_start: 1146, raw_values_count: 1, token_types_start: 1147, token_types_count: 0 }, // [685]
    SimpleHintData { raw_values_start: 1147, raw_values_count: 1, token_types_start: 1148, token_types_count: 0 }, // [686]
    SimpleHintData { raw_values_start: 1148, raw_values_count: 1, token_types_start: 1149, token_types_count: 0 }, // [687]
    SimpleHintData { raw_values_start: 1149, raw_values_count: 1, token_types_start: 1150, token_types_count: 0 }, // [688]
    SimpleHintData { raw_values_start: 1150, raw_values_count: 1, token_types_start: 1151, token_types_count: 0 }, // [689]
    SimpleHintData { raw_values_start: 1151, raw_values_count: 1, token_types_start: 1152, token_types_count: 0 }, // [690]
    SimpleHintData { raw_values_start: 1152, raw_values_count: 1, token_types_start: 1153, token_types_count: 0 }, // [691]
    SimpleHintData { raw_values_start: 1153, raw_values_count: 0, token_types_start: 1153, token_types_count: 2 }, // [692]
    SimpleHintData { raw_values_start: 1155, raw_values_count: 11, token_types_start: 1166, token_types_count: 1 }, // [693]
    SimpleHintData { raw_values_start: 1167, raw_values_count: 1, token_types_start: 1168, token_types_count: 0 }, // [694]
    SimpleHintData { raw_values_start: 1168, raw_values_count: 1, token_types_start: 1169, token_types_count: 0 }, // [695]
    SimpleHintData { raw_values_start: 1169, raw_values_count: 1, token_types_start: 1170, token_types_count: 0 }, // [696]
    SimpleHintData { raw_values_start: 1170, raw_values_count: 1, token_types_start: 1171, token_types_count: 0 }, // [697]
    SimpleHintData { raw_values_start: 1171, raw_values_count: 1, token_types_start: 1172, token_types_count: 0 }, // [698]
    SimpleHintData { raw_values_start: 1172, raw_values_count: 4, token_types_start: 1176, token_types_count: 0 }, // [699]
    SimpleHintData { raw_values_start: 1176, raw_values_count: 2, token_types_start: 1178, token_types_count: 0 }, // [700]
    SimpleHintData { raw_values_start: 1178, raw_values_count: 1, token_types_start: 1179, token_types_count: 0 }, // [701]
    SimpleHintData { raw_values_start: 1179, raw_values_count: 1, token_types_start: 1180, token_types_count: 0 }, // [702]
    SimpleHintData { raw_values_start: 1180, raw_values_count: 2, token_types_start: 1182, token_types_count: 0 }, // [703]
    SimpleHintData { raw_values_start: 1182, raw_values_count: 1, token_types_start: 1183, token_types_count: 0 }, // [704]
    SimpleHintData { raw_values_start: 1183, raw_values_count: 1, token_types_start: 1184, token_types_count: 0 }, // [705]
    SimpleHintData { raw_values_start: 1184, raw_values_count: 1, token_types_start: 1185, token_types_count: 0 }, // [706]
    SimpleHintData { raw_values_start: 1185, raw_values_count: 1, token_types_start: 1186, token_types_count: 0 }, // [707]
    SimpleHintData { raw_values_start: 1186, raw_values_count: 1, token_types_start: 1187, token_types_count: 0 }, // [708]
    SimpleHintData { raw_values_start: 1187, raw_values_count: 1, token_types_start: 1188, token_types_count: 0 }, // [709]
    SimpleHintData { raw_values_start: 1188, raw_values_count: 2, token_types_start: 1190, token_types_count: 0 }, // [710]
    SimpleHintData { raw_values_start: 1190, raw_values_count: 3, token_types_start: 1193, token_types_count: 0 }, // [711]
    SimpleHintData { raw_values_start: 1193, raw_values_count: 1, token_types_start: 1194, token_types_count: 0 }, // [712]
    SimpleHintData { raw_values_start: 1194, raw_values_count: 1, token_types_start: 1195, token_types_count: 0 }, // [713]
    SimpleHintData { raw_values_start: 1195, raw_values_count: 1, token_types_start: 1196, token_types_count: 0 }, // [714]
    SimpleHintData { raw_values_start: 1196, raw_values_count: 1, token_types_start: 1197, token_types_count: 0 }, // [715]
    SimpleHintData { raw_values_start: 1197, raw_values_count: 1, token_types_start: 1198, token_types_count: 0 }, // [716]
    SimpleHintData { raw_values_start: 1198, raw_values_count: 1, token_types_start: 1199, token_types_count: 0 }, // [717]
    SimpleHintData { raw_values_start: 1199, raw_values_count: 1, token_types_start: 1200, token_types_count: 0 }, // [718]
    SimpleHintData { raw_values_start: 1200, raw_values_count: 1, token_types_start: 1201, token_types_count: 0 }, // [719]
    SimpleHintData { raw_values_start: 1201, raw_values_count: 1, token_types_start: 1202, token_types_count: 0 }, // [720]
    SimpleHintData { raw_values_start: 1202, raw_values_count: 1, token_types_start: 1203, token_types_count: 0 }, // [721]
    SimpleHintData { raw_values_start: 1203, raw_values_count: 14, token_types_start: 1217, token_types_count: 0 }, // [722]
    SimpleHintData { raw_values_start: 1217, raw_values_count: 1, token_types_start: 1218, token_types_count: 0 }, // [723]
    SimpleHintData { raw_values_start: 1218, raw_values_count: 1, token_types_start: 1219, token_types_count: 0 }, // [724]
    SimpleHintData { raw_values_start: 1219, raw_values_count: 1, token_types_start: 1220, token_types_count: 0 }, // [725]
    SimpleHintData { raw_values_start: 1220, raw_values_count: 1, token_types_start: 1221, token_types_count: 0 }, // [726]
    SimpleHintData { raw_values_start: 1221, raw_values_count: 1, token_types_start: 1222, token_types_count: 0 }, // [727]
    SimpleHintData { raw_values_start: 1222, raw_values_count: 1, token_types_start: 1223, token_types_count: 0 }, // [728]
    SimpleHintData { raw_values_start: 1223, raw_values_count: 1, token_types_start: 1224, token_types_count: 0 }, // [729]
    SimpleHintData { raw_values_start: 1224, raw_values_count: 1, token_types_start: 1225, token_types_count: 0 }, // [730]
    SimpleHintData { raw_values_start: 1225, raw_values_count: 1, token_types_start: 1226, token_types_count: 0 }, // [731]
    SimpleHintData { raw_values_start: 1226, raw_values_count: 1, token_types_start: 1227, token_types_count: 0 }, // [732]
    SimpleHintData { raw_values_start: 1227, raw_values_count: 1, token_types_start: 1228, token_types_count: 0 }, // [733]
    SimpleHintData { raw_values_start: 1228, raw_values_count: 1, token_types_start: 1229, token_types_count: 0 }, // [734]
    SimpleHintData { raw_values_start: 1229, raw_values_count: 1, token_types_start: 1230, token_types_count: 0 }, // [735]
    SimpleHintData { raw_values_start: 1230, raw_values_count: 1, token_types_start: 1231, token_types_count: 0 }, // [736]
    SimpleHintData { raw_values_start: 1231, raw_values_count: 1, token_types_start: 1232, token_types_count: 0 }, // [737]
    SimpleHintData { raw_values_start: 1232, raw_values_count: 1, token_types_start: 1233, token_types_count: 0 }, // [738]
    SimpleHintData { raw_values_start: 1233, raw_values_count: 1, token_types_start: 1234, token_types_count: 0 }, // [739]
    SimpleHintData { raw_values_start: 1234, raw_values_count: 1, token_types_start: 1235, token_types_count: 0 }, // [740]
    SimpleHintData { raw_values_start: 1235, raw_values_count: 1, token_types_start: 1236, token_types_count: 0 }, // [741]
    SimpleHintData { raw_values_start: 1236, raw_values_count: 1, token_types_start: 1237, token_types_count: 0 }, // [742]
    SimpleHintData { raw_values_start: 1237, raw_values_count: 1, token_types_start: 1238, token_types_count: 0 }, // [743]
    SimpleHintData { raw_values_start: 1238, raw_values_count: 1, token_types_start: 1239, token_types_count: 0 }, // [744]
    SimpleHintData { raw_values_start: 1239, raw_values_count: 3, token_types_start: 1242, token_types_count: 0 }, // [745]
    SimpleHintData { raw_values_start: 1242, raw_values_count: 1, token_types_start: 1243, token_types_count: 0 }, // [746]
    SimpleHintData { raw_values_start: 1243, raw_values_count: 1, token_types_start: 1244, token_types_count: 0 }, // [747]
    SimpleHintData { raw_values_start: 1244, raw_values_count: 2, token_types_start: 1246, token_types_count: 0 }, // [748]
    SimpleHintData { raw_values_start: 1246, raw_values_count: 2, token_types_start: 1248, token_types_count: 0 }, // [749]
    SimpleHintData { raw_values_start: 1248, raw_values_count: 1, token_types_start: 1249, token_types_count: 0 }, // [750]
    SimpleHintData { raw_values_start: 1249, raw_values_count: 1, token_types_start: 1250, token_types_count: 0 }, // [751]
    SimpleHintData { raw_values_start: 1250, raw_values_count: 1, token_types_start: 1251, token_types_count: 0 }, // [752]
    SimpleHintData { raw_values_start: 1251, raw_values_count: 1, token_types_start: 1252, token_types_count: 0 }, // [753]
    SimpleHintData { raw_values_start: 1252, raw_values_count: 1, token_types_start: 1253, token_types_count: 0 }, // [754]
    SimpleHintData { raw_values_start: 1253, raw_values_count: 1, token_types_start: 1254, token_types_count: 0 }, // [755]
    SimpleHintData { raw_values_start: 1254, raw_values_count: 1, token_types_start: 1255, token_types_count: 0 }, // [756]
    SimpleHintData { raw_values_start: 1255, raw_values_count: 1, token_types_start: 1256, token_types_count: 0 }, // [757]
    SimpleHintData { raw_values_start: 1256, raw_values_count: 1, token_types_start: 1257, token_types_count: 0 }, // [758]
    SimpleHintData { raw_values_start: 1257, raw_values_count: 1, token_types_start: 1258, token_types_count: 0 }, // [759]
    SimpleHintData { raw_values_start: 1258, raw_values_count: 1, token_types_start: 1259, token_types_count: 0 }, // [760]
    SimpleHintData { raw_values_start: 1259, raw_values_count: 1, token_types_start: 1260, token_types_count: 0 }, // [761]
    SimpleHintData { raw_values_start: 1260, raw_values_count: 1, token_types_start: 1261, token_types_count: 0 }, // [762]
    SimpleHintData { raw_values_start: 1261, raw_values_count: 1, token_types_start: 1262, token_types_count: 0 }, // [763]
    SimpleHintData { raw_values_start: 1262, raw_values_count: 1, token_types_start: 1263, token_types_count: 0 }, // [764]
    SimpleHintData { raw_values_start: 1263, raw_values_count: 3, token_types_start: 1266, token_types_count: 0 }, // [765]
    SimpleHintData { raw_values_start: 1266, raw_values_count: 1, token_types_start: 1267, token_types_count: 0 }, // [766]
    SimpleHintData { raw_values_start: 1267, raw_values_count: 1, token_types_start: 1268, token_types_count: 0 }, // [767]
    SimpleHintData { raw_values_start: 1268, raw_values_count: 1, token_types_start: 1269, token_types_count: 0 }, // [768]
    SimpleHintData { raw_values_start: 1269, raw_values_count: 1, token_types_start: 1270, token_types_count: 0 }, // [769]
    SimpleHintData { raw_values_start: 1270, raw_values_count: 1, token_types_start: 1271, token_types_count: 0 }, // [770]
    SimpleHintData { raw_values_start: 1271, raw_values_count: 1, token_types_start: 1272, token_types_count: 0 }, // [771]
    SimpleHintData { raw_values_start: 1272, raw_values_count: 1, token_types_start: 1273, token_types_count: 0 }, // [772]
    SimpleHintData { raw_values_start: 1273, raw_values_count: 1, token_types_start: 1274, token_types_count: 0 }, // [773]
    SimpleHintData { raw_values_start: 1274, raw_values_count: 1, token_types_start: 1275, token_types_count: 0 }, // [774]
    SimpleHintData { raw_values_start: 1275, raw_values_count: 1, token_types_start: 1276, token_types_count: 0 }, // [775]
    SimpleHintData { raw_values_start: 1276, raw_values_count: 1, token_types_start: 1277, token_types_count: 0 }, // [776]
    SimpleHintData { raw_values_start: 1277, raw_values_count: 1, token_types_start: 1278, token_types_count: 0 }, // [777]
    SimpleHintData { raw_values_start: 1278, raw_values_count: 1, token_types_start: 1279, token_types_count: 0 }, // [778]
    SimpleHintData { raw_values_start: 1279, raw_values_count: 1, token_types_start: 1280, token_types_count: 0 }, // [779]
    SimpleHintData { raw_values_start: 1280, raw_values_count: 1, token_types_start: 1281, token_types_count: 0 }, // [780]
    SimpleHintData { raw_values_start: 1281, raw_values_count: 1, token_types_start: 1282, token_types_count: 0 }, // [781]
    SimpleHintData { raw_values_start: 1282, raw_values_count: 1, token_types_start: 1283, token_types_count: 0 }, // [782]
    SimpleHintData { raw_values_start: 1283, raw_values_count: 1, token_types_start: 1284, token_types_count: 0 }, // [783]
    SimpleHintData { raw_values_start: 1284, raw_values_count: 1, token_types_start: 1285, token_types_count: 0 }, // [784]
    SimpleHintData { raw_values_start: 1285, raw_values_count: 1, token_types_start: 1286, token_types_count: 0 }, // [785]
    SimpleHintData { raw_values_start: 1286, raw_values_count: 1, token_types_start: 1287, token_types_count: 0 }, // [786]
    SimpleHintData { raw_values_start: 1287, raw_values_count: 1, token_types_start: 1288, token_types_count: 0 }, // [787]
    SimpleHintData { raw_values_start: 1288, raw_values_count: 1, token_types_start: 1289, token_types_count: 0 }, // [788]
    SimpleHintData { raw_values_start: 1289, raw_values_count: 1, token_types_start: 1290, token_types_count: 0 }, // [789]
    SimpleHintData { raw_values_start: 1290, raw_values_count: 1, token_types_start: 1291, token_types_count: 0 }, // [790]
    SimpleHintData { raw_values_start: 1291, raw_values_count: 1, token_types_start: 1292, token_types_count: 0 }, // [791]
    SimpleHintData { raw_values_start: 1292, raw_values_count: 1, token_types_start: 1293, token_types_count: 0 }, // [792]
    SimpleHintData { raw_values_start: 1293, raw_values_count: 1, token_types_start: 1294, token_types_count: 0 }, // [793]
    SimpleHintData { raw_values_start: 1294, raw_values_count: 1, token_types_start: 1295, token_types_count: 0 }, // [794]
    SimpleHintData { raw_values_start: 1295, raw_values_count: 2, token_types_start: 1297, token_types_count: 0 }, // [795]
    SimpleHintData { raw_values_start: 1297, raw_values_count: 1, token_types_start: 1298, token_types_count: 0 }, // [796]
    SimpleHintData { raw_values_start: 1298, raw_values_count: 1, token_types_start: 1299, token_types_count: 0 }, // [797]
    SimpleHintData { raw_values_start: 1299, raw_values_count: 1, token_types_start: 1300, token_types_count: 0 }, // [798]
    SimpleHintData { raw_values_start: 1300, raw_values_count: 1, token_types_start: 1301, token_types_count: 0 }, // [799]
    SimpleHintData { raw_values_start: 1301, raw_values_count: 1, token_types_start: 1302, token_types_count: 0 }, // [800]
    SimpleHintData { raw_values_start: 1302, raw_values_count: 1, token_types_start: 1303, token_types_count: 0 }, // [801]
    SimpleHintData { raw_values_start: 1303, raw_values_count: 1, token_types_start: 1304, token_types_count: 0 }, // [802]
    SimpleHintData { raw_values_start: 1304, raw_values_count: 1, token_types_start: 1305, token_types_count: 0 }, // [803]
    SimpleHintData { raw_values_start: 1305, raw_values_count: 1, token_types_start: 1306, token_types_count: 0 }, // [804]
    SimpleHintData { raw_values_start: 1306, raw_values_count: 1, token_types_start: 1307, token_types_count: 0 }, // [805]
    SimpleHintData { raw_values_start: 1307, raw_values_count: 1, token_types_start: 1308, token_types_count: 0 }, // [806]
    SimpleHintData { raw_values_start: 1308, raw_values_count: 1, token_types_start: 1309, token_types_count: 0 }, // [807]
    SimpleHintData { raw_values_start: 1309, raw_values_count: 1, token_types_start: 1310, token_types_count: 0 }, // [808]
    SimpleHintData { raw_values_start: 1310, raw_values_count: 1, token_types_start: 1311, token_types_count: 0 }, // [809]
    SimpleHintData { raw_values_start: 1311, raw_values_count: 1, token_types_start: 1312, token_types_count: 0 }, // [810]
    SimpleHintData { raw_values_start: 1312, raw_values_count: 1, token_types_start: 1313, token_types_count: 0 }, // [811]
    SimpleHintData { raw_values_start: 1313, raw_values_count: 1, token_types_start: 1314, token_types_count: 0 }, // [812]
    SimpleHintData { raw_values_start: 1314, raw_values_count: 1, token_types_start: 1315, token_types_count: 0 }, // [813]
    SimpleHintData { raw_values_start: 1315, raw_values_count: 1, token_types_start: 1316, token_types_count: 0 }, // [814]
    SimpleHintData { raw_values_start: 1316, raw_values_count: 1, token_types_start: 1317, token_types_count: 0 }, // [815]
    SimpleHintData { raw_values_start: 1317, raw_values_count: 1, token_types_start: 1318, token_types_count: 0 }, // [816]
    SimpleHintData { raw_values_start: 1318, raw_values_count: 1, token_types_start: 1319, token_types_count: 0 }, // [817]
    SimpleHintData { raw_values_start: 1319, raw_values_count: 1, token_types_start: 1320, token_types_count: 0 }, // [818]
    SimpleHintData { raw_values_start: 1320, raw_values_count: 1, token_types_start: 1321, token_types_count: 0 }, // [819]
    SimpleHintData { raw_values_start: 1321, raw_values_count: 1, token_types_start: 1322, token_types_count: 0 }, // [820]
    SimpleHintData { raw_values_start: 1322, raw_values_count: 1, token_types_start: 1323, token_types_count: 0 }, // [821]
    SimpleHintData { raw_values_start: 1323, raw_values_count: 1, token_types_start: 1324, token_types_count: 0 }, // [822]
    SimpleHintData { raw_values_start: 1324, raw_values_count: 1, token_types_start: 1325, token_types_count: 0 }, // [823]
    SimpleHintData { raw_values_start: 1325, raw_values_count: 1, token_types_start: 1326, token_types_count: 0 }, // [824]
    SimpleHintData { raw_values_start: 1326, raw_values_count: 1, token_types_start: 1327, token_types_count: 0 }, // [825]
    SimpleHintData { raw_values_start: 1327, raw_values_count: 1, token_types_start: 1328, token_types_count: 0 }, // [826]
    SimpleHintData { raw_values_start: 1328, raw_values_count: 1, token_types_start: 1329, token_types_count: 0 }, // [827]
    SimpleHintData { raw_values_start: 1329, raw_values_count: 1, token_types_start: 1330, token_types_count: 0 }, // [828]
    SimpleHintData { raw_values_start: 1330, raw_values_count: 1, token_types_start: 1331, token_types_count: 0 }, // [829]
    SimpleHintData { raw_values_start: 1331, raw_values_count: 1, token_types_start: 1332, token_types_count: 0 }, // [830]
    SimpleHintData { raw_values_start: 1332, raw_values_count: 1, token_types_start: 1333, token_types_count: 0 }, // [831]
    SimpleHintData { raw_values_start: 1333, raw_values_count: 1, token_types_start: 1334, token_types_count: 0 }, // [832]
    SimpleHintData { raw_values_start: 1334, raw_values_count: 1, token_types_start: 1335, token_types_count: 0 }, // [833]
    SimpleHintData { raw_values_start: 1335, raw_values_count: 1, token_types_start: 1336, token_types_count: 0 }, // [834]
    SimpleHintData { raw_values_start: 1336, raw_values_count: 1, token_types_start: 1337, token_types_count: 0 }, // [835]
    SimpleHintData { raw_values_start: 1337, raw_values_count: 1, token_types_start: 1338, token_types_count: 0 }, // [836]
    SimpleHintData { raw_values_start: 1338, raw_values_count: 1, token_types_start: 1339, token_types_count: 0 }, // [837]
    SimpleHintData { raw_values_start: 1339, raw_values_count: 1, token_types_start: 1340, token_types_count: 0 }, // [838]
    SimpleHintData { raw_values_start: 1340, raw_values_count: 1, token_types_start: 1341, token_types_count: 0 }, // [839]
    SimpleHintData { raw_values_start: 1341, raw_values_count: 1, token_types_start: 1342, token_types_count: 0 }, // [840]
    SimpleHintData { raw_values_start: 1342, raw_values_count: 1, token_types_start: 1343, token_types_count: 0 }, // [841]
    SimpleHintData { raw_values_start: 1343, raw_values_count: 4, token_types_start: 1347, token_types_count: 0 }, // [842]
    SimpleHintData { raw_values_start: 1347, raw_values_count: 2, token_types_start: 1349, token_types_count: 0 }, // [843]
    SimpleHintData { raw_values_start: 1349, raw_values_count: 1, token_types_start: 1350, token_types_count: 0 }, // [844]
    SimpleHintData { raw_values_start: 1350, raw_values_count: 1, token_types_start: 1351, token_types_count: 0 }, // [845]
    SimpleHintData { raw_values_start: 1351, raw_values_count: 1, token_types_start: 1352, token_types_count: 0 }, // [846]
    SimpleHintData { raw_values_start: 1352, raw_values_count: 1, token_types_start: 1353, token_types_count: 0 }, // [847]
    SimpleHintData { raw_values_start: 1353, raw_values_count: 1, token_types_start: 1354, token_types_count: 0 }, // [848]
    SimpleHintData { raw_values_start: 1354, raw_values_count: 1, token_types_start: 1355, token_types_count: 0 }, // [849]
    SimpleHintData { raw_values_start: 1355, raw_values_count: 1, token_types_start: 1356, token_types_count: 0 }, // [850]
    SimpleHintData { raw_values_start: 1356, raw_values_count: 1, token_types_start: 1357, token_types_count: 0 }, // [851]
    SimpleHintData { raw_values_start: 1357, raw_values_count: 1, token_types_start: 1358, token_types_count: 0 }, // [852]
    SimpleHintData { raw_values_start: 1358, raw_values_count: 1, token_types_start: 1359, token_types_count: 0 }, // [853]
    SimpleHintData { raw_values_start: 1359, raw_values_count: 1, token_types_start: 1360, token_types_count: 0 }, // [854]
    SimpleHintData { raw_values_start: 1360, raw_values_count: 1, token_types_start: 1361, token_types_count: 0 }, // [855]
    SimpleHintData { raw_values_start: 1361, raw_values_count: 1, token_types_start: 1362, token_types_count: 0 }, // [856]
    SimpleHintData { raw_values_start: 1362, raw_values_count: 1, token_types_start: 1363, token_types_count: 0 }, // [857]
    SimpleHintData { raw_values_start: 1363, raw_values_count: 1, token_types_start: 1364, token_types_count: 0 }, // [858]
    SimpleHintData { raw_values_start: 1364, raw_values_count: 1, token_types_start: 1365, token_types_count: 0 }, // [859]
    SimpleHintData { raw_values_start: 1365, raw_values_count: 1, token_types_start: 1366, token_types_count: 0 }, // [860]
    SimpleHintData { raw_values_start: 1366, raw_values_count: 1, token_types_start: 1367, token_types_count: 0 }, // [861]
    SimpleHintData { raw_values_start: 1367, raw_values_count: 1, token_types_start: 1368, token_types_count: 0 }, // [862]
    SimpleHintData { raw_values_start: 1368, raw_values_count: 1, token_types_start: 1369, token_types_count: 0 }, // [863]
    SimpleHintData { raw_values_start: 1369, raw_values_count: 1, token_types_start: 1370, token_types_count: 0 }, // [864]
    SimpleHintData { raw_values_start: 1370, raw_values_count: 1, token_types_start: 1371, token_types_count: 0 }, // [865]
    SimpleHintData { raw_values_start: 1371, raw_values_count: 1, token_types_start: 1372, token_types_count: 0 }, // [866]
    SimpleHintData { raw_values_start: 1372, raw_values_count: 1, token_types_start: 1373, token_types_count: 0 }, // [867]
    SimpleHintData { raw_values_start: 1373, raw_values_count: 1, token_types_start: 1374, token_types_count: 0 }, // [868]
    SimpleHintData { raw_values_start: 1374, raw_values_count: 1, token_types_start: 1375, token_types_count: 0 }, // [869]
    SimpleHintData { raw_values_start: 1375, raw_values_count: 1, token_types_start: 1376, token_types_count: 0 }, // [870]
    SimpleHintData { raw_values_start: 1376, raw_values_count: 1, token_types_start: 1377, token_types_count: 0 }, // [871]
    SimpleHintData { raw_values_start: 1377, raw_values_count: 1, token_types_start: 1378, token_types_count: 0 }, // [872]
    SimpleHintData { raw_values_start: 1378, raw_values_count: 1, token_types_start: 1379, token_types_count: 0 }, // [873]
    SimpleHintData { raw_values_start: 1379, raw_values_count: 1, token_types_start: 1380, token_types_count: 0 }, // [874]
    SimpleHintData { raw_values_start: 1380, raw_values_count: 1, token_types_start: 1381, token_types_count: 0 }, // [875]
    SimpleHintData { raw_values_start: 1381, raw_values_count: 1, token_types_start: 1382, token_types_count: 0 }, // [876]
    SimpleHintData { raw_values_start: 1382, raw_values_count: 1, token_types_start: 1383, token_types_count: 0 }, // [877]
    SimpleHintData { raw_values_start: 1383, raw_values_count: 1, token_types_start: 1384, token_types_count: 0 }, // [878]
    SimpleHintData { raw_values_start: 1384, raw_values_count: 1, token_types_start: 1385, token_types_count: 0 }, // [879]
    SimpleHintData { raw_values_start: 1385, raw_values_count: 2, token_types_start: 1387, token_types_count: 0 }, // [880]
    SimpleHintData { raw_values_start: 1387, raw_values_count: 2, token_types_start: 1389, token_types_count: 0 }, // [881]
    SimpleHintData { raw_values_start: 1389, raw_values_count: 11, token_types_start: 1400, token_types_count: 0 }, // [882]
    SimpleHintData { raw_values_start: 1400, raw_values_count: 1, token_types_start: 1401, token_types_count: 0 }, // [883]
    SimpleHintData { raw_values_start: 1401, raw_values_count: 1, token_types_start: 1402, token_types_count: 0 }, // [884]
    SimpleHintData { raw_values_start: 1402, raw_values_count: 1, token_types_start: 1403, token_types_count: 0 }, // [885]
    SimpleHintData { raw_values_start: 1403, raw_values_count: 1, token_types_start: 1404, token_types_count: 0 }, // [886]
    SimpleHintData { raw_values_start: 1404, raw_values_count: 1, token_types_start: 1405, token_types_count: 0 }, // [887]
    SimpleHintData { raw_values_start: 1405, raw_values_count: 1, token_types_start: 1406, token_types_count: 0 }, // [888]
    SimpleHintData { raw_values_start: 1406, raw_values_count: 1, token_types_start: 1407, token_types_count: 0 }, // [889]
    SimpleHintData { raw_values_start: 1407, raw_values_count: 1, token_types_start: 1408, token_types_count: 0 }, // [890]
    SimpleHintData { raw_values_start: 1408, raw_values_count: 1, token_types_start: 1409, token_types_count: 0 }, // [891]
    SimpleHintData { raw_values_start: 1409, raw_values_count: 1, token_types_start: 1410, token_types_count: 0 }, // [892]
    SimpleHintData { raw_values_start: 1410, raw_values_count: 1, token_types_start: 1411, token_types_count: 0 }, // [893]
    SimpleHintData { raw_values_start: 1411, raw_values_count: 1, token_types_start: 1412, token_types_count: 0 }, // [894]
    SimpleHintData { raw_values_start: 1412, raw_values_count: 1, token_types_start: 1413, token_types_count: 0 }, // [895]
    SimpleHintData { raw_values_start: 1413, raw_values_count: 1, token_types_start: 1414, token_types_count: 0 }, // [896]
    SimpleHintData { raw_values_start: 1414, raw_values_count: 1, token_types_start: 1415, token_types_count: 0 }, // [897]
    SimpleHintData { raw_values_start: 1415, raw_values_count: 1, token_types_start: 1416, token_types_count: 0 }, // [898]
    SimpleHintData { raw_values_start: 1416, raw_values_count: 1, token_types_start: 1417, token_types_count: 0 }, // [899]
    SimpleHintData { raw_values_start: 1417, raw_values_count: 1, token_types_start: 1418, token_types_count: 0 }, // [900]
    SimpleHintData { raw_values_start: 1418, raw_values_count: 1, token_types_start: 1419, token_types_count: 0 }, // [901]
    SimpleHintData { raw_values_start: 1419, raw_values_count: 1, token_types_start: 1420, token_types_count: 0 }, // [902]
    SimpleHintData { raw_values_start: 1420, raw_values_count: 1, token_types_start: 1421, token_types_count: 0 }, // [903]
    SimpleHintData { raw_values_start: 1421, raw_values_count: 1, token_types_start: 1422, token_types_count: 0 }, // [904]
    SimpleHintData { raw_values_start: 1422, raw_values_count: 1, token_types_start: 1423, token_types_count: 0 }, // [905]
    SimpleHintData { raw_values_start: 1423, raw_values_count: 1, token_types_start: 1424, token_types_count: 0 }, // [906]
    SimpleHintData { raw_values_start: 1424, raw_values_count: 1, token_types_start: 1425, token_types_count: 0 }, // [907]
    SimpleHintData { raw_values_start: 1425, raw_values_count: 1, token_types_start: 1426, token_types_count: 0 }, // [908]
    SimpleHintData { raw_values_start: 1426, raw_values_count: 1, token_types_start: 1427, token_types_count: 0 }, // [909]
    SimpleHintData { raw_values_start: 1427, raw_values_count: 1, token_types_start: 1428, token_types_count: 0 }, // [910]
    SimpleHintData { raw_values_start: 1428, raw_values_count: 1, token_types_start: 1429, token_types_count: 0 }, // [911]
    SimpleHintData { raw_values_start: 1429, raw_values_count: 1, token_types_start: 1430, token_types_count: 0 }, // [912]
    SimpleHintData { raw_values_start: 1430, raw_values_count: 1, token_types_start: 1431, token_types_count: 0 }, // [913]
    SimpleHintData { raw_values_start: 1431, raw_values_count: 1, token_types_start: 1432, token_types_count: 1 }, // [914]
    SimpleHintData { raw_values_start: 1433, raw_values_count: 1, token_types_start: 1434, token_types_count: 0 }, // [915]
    SimpleHintData { raw_values_start: 1434, raw_values_count: 1, token_types_start: 1435, token_types_count: 0 }, // [916]
    SimpleHintData { raw_values_start: 1435, raw_values_count: 1, token_types_start: 1436, token_types_count: 0 }, // [917]
    SimpleHintData { raw_values_start: 1436, raw_values_count: 1, token_types_start: 1437, token_types_count: 0 }, // [918]
    SimpleHintData { raw_values_start: 1437, raw_values_count: 1, token_types_start: 1438, token_types_count: 0 }, // [919]
    SimpleHintData { raw_values_start: 1438, raw_values_count: 1, token_types_start: 1439, token_types_count: 0 }, // [920]
    SimpleHintData { raw_values_start: 1439, raw_values_count: 1, token_types_start: 1440, token_types_count: 0 }, // [921]
    SimpleHintData { raw_values_start: 1440, raw_values_count: 1, token_types_start: 1441, token_types_count: 0 }, // [922]
    SimpleHintData { raw_values_start: 1441, raw_values_count: 1, token_types_start: 1442, token_types_count: 0 }, // [923]
    SimpleHintData { raw_values_start: 1442, raw_values_count: 1, token_types_start: 1443, token_types_count: 0 }, // [924]
    SimpleHintData { raw_values_start: 1443, raw_values_count: 1, token_types_start: 1444, token_types_count: 0 }, // [925]
    SimpleHintData { raw_values_start: 1444, raw_values_count: 1, token_types_start: 1445, token_types_count: 0 }, // [926]
    SimpleHintData { raw_values_start: 1445, raw_values_count: 1, token_types_start: 1446, token_types_count: 0 }, // [927]
    SimpleHintData { raw_values_start: 1446, raw_values_count: 1, token_types_start: 1447, token_types_count: 0 }, // [928]
    SimpleHintData { raw_values_start: 1447, raw_values_count: 1, token_types_start: 1448, token_types_count: 0 }, // [929]
    SimpleHintData { raw_values_start: 1448, raw_values_count: 1, token_types_start: 1449, token_types_count: 0 }, // [930]
    SimpleHintData { raw_values_start: 1449, raw_values_count: 1, token_types_start: 1450, token_types_count: 0 }, // [931]
    SimpleHintData { raw_values_start: 1450, raw_values_count: 1, token_types_start: 1451, token_types_count: 0 }, // [932]
    SimpleHintData { raw_values_start: 1451, raw_values_count: 1, token_types_start: 1452, token_types_count: 0 }, // [933]
    SimpleHintData { raw_values_start: 1452, raw_values_count: 1, token_types_start: 1453, token_types_count: 0 }, // [934]
    SimpleHintData { raw_values_start: 1453, raw_values_count: 1, token_types_start: 1454, token_types_count: 0 }, // [935]
    SimpleHintData { raw_values_start: 1454, raw_values_count: 1, token_types_start: 1455, token_types_count: 0 }, // [936]
    SimpleHintData { raw_values_start: 1455, raw_values_count: 1, token_types_start: 1456, token_types_count: 0 }, // [937]
    SimpleHintData { raw_values_start: 1456, raw_values_count: 1, token_types_start: 1457, token_types_count: 0 }, // [938]
    SimpleHintData { raw_values_start: 1457, raw_values_count: 1, token_types_start: 1458, token_types_count: 0 }, // [939]
    SimpleHintData { raw_values_start: 1458, raw_values_count: 1, token_types_start: 1459, token_types_count: 0 }, // [940]
    SimpleHintData { raw_values_start: 1459, raw_values_count: 1, token_types_start: 1460, token_types_count: 0 }, // [941]
    SimpleHintData { raw_values_start: 1460, raw_values_count: 3, token_types_start: 1463, token_types_count: 0 }, // [942]
    SimpleHintData { raw_values_start: 1463, raw_values_count: 1, token_types_start: 1464, token_types_count: 0 }, // [943]
    SimpleHintData { raw_values_start: 1464, raw_values_count: 1, token_types_start: 1465, token_types_count: 0 }, // [944]
    SimpleHintData { raw_values_start: 1465, raw_values_count: 1, token_types_start: 1466, token_types_count: 0 }, // [945]
    SimpleHintData { raw_values_start: 1466, raw_values_count: 1, token_types_start: 1467, token_types_count: 0 }, // [946]
    SimpleHintData { raw_values_start: 1467, raw_values_count: 1, token_types_start: 1468, token_types_count: 0 }, // [947]
    SimpleHintData { raw_values_start: 1468, raw_values_count: 1, token_types_start: 1469, token_types_count: 0 }, // [948]
    SimpleHintData { raw_values_start: 1469, raw_values_count: 1, token_types_start: 1470, token_types_count: 0 }, // [949]
    SimpleHintData { raw_values_start: 1470, raw_values_count: 1, token_types_start: 1471, token_types_count: 0 }, // [950]
    SimpleHintData { raw_values_start: 1471, raw_values_count: 1, token_types_start: 1472, token_types_count: 0 }, // [951]
    SimpleHintData { raw_values_start: 1472, raw_values_count: 2, token_types_start: 1474, token_types_count: 0 }, // [952]
    SimpleHintData { raw_values_start: 1474, raw_values_count: 1, token_types_start: 1475, token_types_count: 0 }, // [953]
    SimpleHintData { raw_values_start: 1475, raw_values_count: 1, token_types_start: 1476, token_types_count: 0 }, // [954]
    SimpleHintData { raw_values_start: 1476, raw_values_count: 1, token_types_start: 1477, token_types_count: 0 }, // [955]
    SimpleHintData { raw_values_start: 1477, raw_values_count: 1, token_types_start: 1478, token_types_count: 0 }, // [956]
    SimpleHintData { raw_values_start: 1478, raw_values_count: 1, token_types_start: 1479, token_types_count: 0 }, // [957]
    SimpleHintData { raw_values_start: 1479, raw_values_count: 1, token_types_start: 1480, token_types_count: 0 }, // [958]
    SimpleHintData { raw_values_start: 1480, raw_values_count: 1, token_types_start: 1481, token_types_count: 0 }, // [959]
    SimpleHintData { raw_values_start: 1481, raw_values_count: 1, token_types_start: 1482, token_types_count: 0 }, // [960]
    SimpleHintData { raw_values_start: 1482, raw_values_count: 4, token_types_start: 1486, token_types_count: 0 }, // [961]
    SimpleHintData { raw_values_start: 1486, raw_values_count: 3, token_types_start: 1489, token_types_count: 0 }, // [962]
    SimpleHintData { raw_values_start: 1489, raw_values_count: 1, token_types_start: 1490, token_types_count: 0 }, // [963]
    SimpleHintData { raw_values_start: 1490, raw_values_count: 1, token_types_start: 1491, token_types_count: 0 }, // [964]
    SimpleHintData { raw_values_start: 1491, raw_values_count: 1, token_types_start: 1492, token_types_count: 0 }, // [965]
    SimpleHintData { raw_values_start: 1492, raw_values_count: 1, token_types_start: 1493, token_types_count: 0 }, // [966]
    SimpleHintData { raw_values_start: 1493, raw_values_count: 1, token_types_start: 1494, token_types_count: 0 }, // [967]
    SimpleHintData { raw_values_start: 1494, raw_values_count: 1, token_types_start: 1495, token_types_count: 0 }, // [968]
    SimpleHintData { raw_values_start: 1495, raw_values_count: 1, token_types_start: 1496, token_types_count: 0 }, // [969]
    SimpleHintData { raw_values_start: 1496, raw_values_count: 1, token_types_start: 1497, token_types_count: 0 }, // [970]
    SimpleHintData { raw_values_start: 1497, raw_values_count: 1, token_types_start: 1498, token_types_count: 0 }, // [971]
    SimpleHintData { raw_values_start: 1498, raw_values_count: 1, token_types_start: 1499, token_types_count: 0 }, // [972]
    SimpleHintData { raw_values_start: 1499, raw_values_count: 1, token_types_start: 1500, token_types_count: 0 }, // [973]
    SimpleHintData { raw_values_start: 1500, raw_values_count: 1, token_types_start: 1501, token_types_count: 0 }, // [974]
    SimpleHintData { raw_values_start: 1501, raw_values_count: 1, token_types_start: 1502, token_types_count: 0 }, // [975]
    SimpleHintData { raw_values_start: 1502, raw_values_count: 1, token_types_start: 1503, token_types_count: 0 }, // [976]
    SimpleHintData { raw_values_start: 1503, raw_values_count: 1, token_types_start: 1504, token_types_count: 0 }, // [977]
    SimpleHintData { raw_values_start: 1504, raw_values_count: 1, token_types_start: 1505, token_types_count: 0 }, // [978]
    SimpleHintData { raw_values_start: 1505, raw_values_count: 1, token_types_start: 1506, token_types_count: 0 }, // [979]
    SimpleHintData { raw_values_start: 1506, raw_values_count: 1, token_types_start: 1507, token_types_count: 0 }, // [980]
    SimpleHintData { raw_values_start: 1507, raw_values_count: 1, token_types_start: 1508, token_types_count: 0 }, // [981]
    SimpleHintData { raw_values_start: 1508, raw_values_count: 1, token_types_start: 1509, token_types_count: 0 }, // [982]
    SimpleHintData { raw_values_start: 1509, raw_values_count: 1, token_types_start: 1510, token_types_count: 0 }, // [983]
    SimpleHintData { raw_values_start: 1510, raw_values_count: 1, token_types_start: 1511, token_types_count: 0 }, // [984]
    SimpleHintData { raw_values_start: 1511, raw_values_count: 1, token_types_start: 1512, token_types_count: 0 }, // [985]
    SimpleHintData { raw_values_start: 1512, raw_values_count: 1, token_types_start: 1513, token_types_count: 0 }, // [986]
    SimpleHintData { raw_values_start: 1513, raw_values_count: 1, token_types_start: 1514, token_types_count: 0 }, // [987]
    SimpleHintData { raw_values_start: 1514, raw_values_count: 1, token_types_start: 1515, token_types_count: 0 }, // [988]
    SimpleHintData { raw_values_start: 1515, raw_values_count: 1, token_types_start: 1516, token_types_count: 0 }, // [989]
    SimpleHintData { raw_values_start: 1516, raw_values_count: 1, token_types_start: 1517, token_types_count: 0 }, // [990]
    SimpleHintData { raw_values_start: 1517, raw_values_count: 1, token_types_start: 1518, token_types_count: 0 }, // [991]
    SimpleHintData { raw_values_start: 1518, raw_values_count: 1, token_types_start: 1519, token_types_count: 0 }, // [992]
    SimpleHintData { raw_values_start: 1519, raw_values_count: 1, token_types_start: 1520, token_types_count: 0 }, // [993]
    SimpleHintData { raw_values_start: 1520, raw_values_count: 1, token_types_start: 1521, token_types_count: 0 }, // [994]
    SimpleHintData { raw_values_start: 1521, raw_values_count: 1, token_types_start: 1522, token_types_count: 0 }, // [995]
    SimpleHintData { raw_values_start: 1522, raw_values_count: 1, token_types_start: 1523, token_types_count: 0 }, // [996]
    SimpleHintData { raw_values_start: 1523, raw_values_count: 1, token_types_start: 1524, token_types_count: 0 }, // [997]
    SimpleHintData { raw_values_start: 1524, raw_values_count: 1, token_types_start: 1525, token_types_count: 0 }, // [998]
    SimpleHintData { raw_values_start: 1525, raw_values_count: 1, token_types_start: 1526, token_types_count: 0 }, // [999]
    SimpleHintData { raw_values_start: 1526, raw_values_count: 1, token_types_start: 1527, token_types_count: 0 }, // [1000]
    SimpleHintData { raw_values_start: 1527, raw_values_count: 1, token_types_start: 1528, token_types_count: 0 }, // [1001]
    SimpleHintData { raw_values_start: 1528, raw_values_count: 1, token_types_start: 1529, token_types_count: 0 }, // [1002]
    SimpleHintData { raw_values_start: 1529, raw_values_count: 1, token_types_start: 1530, token_types_count: 0 }, // [1003]
    SimpleHintData { raw_values_start: 1530, raw_values_count: 1, token_types_start: 1531, token_types_count: 0 }, // [1004]
    SimpleHintData { raw_values_start: 1531, raw_values_count: 1, token_types_start: 1532, token_types_count: 0 }, // [1005]
    SimpleHintData { raw_values_start: 1532, raw_values_count: 1, token_types_start: 1533, token_types_count: 0 }, // [1006]
    SimpleHintData { raw_values_start: 1533, raw_values_count: 1, token_types_start: 1534, token_types_count: 0 }, // [1007]
    SimpleHintData { raw_values_start: 1534, raw_values_count: 1, token_types_start: 1535, token_types_count: 0 }, // [1008]
    SimpleHintData { raw_values_start: 1535, raw_values_count: 1, token_types_start: 1536, token_types_count: 0 }, // [1009]
    SimpleHintData { raw_values_start: 1536, raw_values_count: 1, token_types_start: 1537, token_types_count: 0 }, // [1010]
    SimpleHintData { raw_values_start: 1537, raw_values_count: 1, token_types_start: 1538, token_types_count: 0 }, // [1011]
    SimpleHintData { raw_values_start: 1538, raw_values_count: 1, token_types_start: 1539, token_types_count: 0 }, // [1012]
    SimpleHintData { raw_values_start: 1539, raw_values_count: 1, token_types_start: 1540, token_types_count: 0 }, // [1013]
    SimpleHintData { raw_values_start: 1540, raw_values_count: 1, token_types_start: 1541, token_types_count: 0 }, // [1014]
    SimpleHintData { raw_values_start: 1541, raw_values_count: 1, token_types_start: 1542, token_types_count: 0 }, // [1015]
    SimpleHintData { raw_values_start: 1542, raw_values_count: 1, token_types_start: 1543, token_types_count: 0 }, // [1016]
    SimpleHintData { raw_values_start: 1543, raw_values_count: 1, token_types_start: 1544, token_types_count: 0 }, // [1017]
    SimpleHintData { raw_values_start: 1544, raw_values_count: 1, token_types_start: 1545, token_types_count: 0 }, // [1018]
    SimpleHintData { raw_values_start: 1545, raw_values_count: 2, token_types_start: 1547, token_types_count: 0 }, // [1019]
    SimpleHintData { raw_values_start: 1547, raw_values_count: 1, token_types_start: 1548, token_types_count: 0 }, // [1020]
    SimpleHintData { raw_values_start: 1548, raw_values_count: 1, token_types_start: 1549, token_types_count: 0 }, // [1021]
    SimpleHintData { raw_values_start: 1549, raw_values_count: 1, token_types_start: 1550, token_types_count: 0 }, // [1022]
    SimpleHintData { raw_values_start: 1550, raw_values_count: 1, token_types_start: 1551, token_types_count: 0 }, // [1023]
    SimpleHintData { raw_values_start: 1551, raw_values_count: 1, token_types_start: 1552, token_types_count: 0 }, // [1024]
    SimpleHintData { raw_values_start: 1552, raw_values_count: 1, token_types_start: 1553, token_types_count: 0 }, // [1025]
    SimpleHintData { raw_values_start: 1553, raw_values_count: 1, token_types_start: 1554, token_types_count: 0 }, // [1026]
    SimpleHintData { raw_values_start: 1554, raw_values_count: 1, token_types_start: 1555, token_types_count: 0 }, // [1027]
    SimpleHintData { raw_values_start: 1555, raw_values_count: 1, token_types_start: 1556, token_types_count: 0 }, // [1028]
    SimpleHintData { raw_values_start: 1556, raw_values_count: 1, token_types_start: 1557, token_types_count: 0 }, // [1029]
    SimpleHintData { raw_values_start: 1557, raw_values_count: 1, token_types_start: 1558, token_types_count: 0 }, // [1030]
    SimpleHintData { raw_values_start: 1558, raw_values_count: 1, token_types_start: 1559, token_types_count: 0 }, // [1031]
    SimpleHintData { raw_values_start: 1559, raw_values_count: 1, token_types_start: 1560, token_types_count: 0 }, // [1032]
    SimpleHintData { raw_values_start: 1560, raw_values_count: 1, token_types_start: 1561, token_types_count: 0 }, // [1033]
    SimpleHintData { raw_values_start: 1561, raw_values_count: 1, token_types_start: 1562, token_types_count: 0 }, // [1034]
    SimpleHintData { raw_values_start: 1562, raw_values_count: 1, token_types_start: 1563, token_types_count: 0 }, // [1035]
    SimpleHintData { raw_values_start: 1563, raw_values_count: 2, token_types_start: 1565, token_types_count: 0 }, // [1036]
    SimpleHintData { raw_values_start: 1565, raw_values_count: 13, token_types_start: 1578, token_types_count: 0 }, // [1037]
    SimpleHintData { raw_values_start: 1578, raw_values_count: 2, token_types_start: 1580, token_types_count: 0 }, // [1038]
    SimpleHintData { raw_values_start: 1580, raw_values_count: 1, token_types_start: 1581, token_types_count: 0 }, // [1039]
    SimpleHintData { raw_values_start: 1581, raw_values_count: 1, token_types_start: 1582, token_types_count: 0 }, // [1040]
    SimpleHintData { raw_values_start: 1582, raw_values_count: 1, token_types_start: 1583, token_types_count: 0 }, // [1041]
    SimpleHintData { raw_values_start: 1583, raw_values_count: 1, token_types_start: 1584, token_types_count: 0 }, // [1042]
    SimpleHintData { raw_values_start: 1584, raw_values_count: 1, token_types_start: 1585, token_types_count: 0 }, // [1043]
    SimpleHintData { raw_values_start: 1585, raw_values_count: 1, token_types_start: 1586, token_types_count: 0 }, // [1044]
    SimpleHintData { raw_values_start: 1586, raw_values_count: 2, token_types_start: 1588, token_types_count: 0 }, // [1045]
    SimpleHintData { raw_values_start: 1588, raw_values_count: 1, token_types_start: 1589, token_types_count: 0 }, // [1046]
    SimpleHintData { raw_values_start: 1589, raw_values_count: 1, token_types_start: 1590, token_types_count: 0 }, // [1047]
    SimpleHintData { raw_values_start: 1590, raw_values_count: 1, token_types_start: 1591, token_types_count: 0 }, // [1048]
    SimpleHintData { raw_values_start: 1591, raw_values_count: 1, token_types_start: 1592, token_types_count: 0 }, // [1049]
    SimpleHintData { raw_values_start: 1592, raw_values_count: 1, token_types_start: 1593, token_types_count: 0 }, // [1050]
    SimpleHintData { raw_values_start: 1593, raw_values_count: 1, token_types_start: 1594, token_types_count: 0 }, // [1051]
    SimpleHintData { raw_values_start: 1594, raw_values_count: 1, token_types_start: 1595, token_types_count: 0 }, // [1052]
    SimpleHintData { raw_values_start: 1595, raw_values_count: 1, token_types_start: 1596, token_types_count: 0 }, // [1053]
    SimpleHintData { raw_values_start: 1596, raw_values_count: 1, token_types_start: 1597, token_types_count: 0 }, // [1054]
    SimpleHintData { raw_values_start: 1597, raw_values_count: 1, token_types_start: 1598, token_types_count: 0 }, // [1055]
    SimpleHintData { raw_values_start: 1598, raw_values_count: 1, token_types_start: 1599, token_types_count: 0 }, // [1056]
    SimpleHintData { raw_values_start: 1599, raw_values_count: 1, token_types_start: 1600, token_types_count: 0 }, // [1057]
    SimpleHintData { raw_values_start: 1600, raw_values_count: 1, token_types_start: 1601, token_types_count: 0 }, // [1058]
    SimpleHintData { raw_values_start: 1601, raw_values_count: 1, token_types_start: 1602, token_types_count: 0 }, // [1059]
    SimpleHintData { raw_values_start: 1602, raw_values_count: 1, token_types_start: 1603, token_types_count: 0 }, // [1060]
    SimpleHintData { raw_values_start: 1603, raw_values_count: 1, token_types_start: 1604, token_types_count: 0 }, // [1061]
    SimpleHintData { raw_values_start: 1604, raw_values_count: 31, token_types_start: 1635, token_types_count: 0 }, // [1062]
    SimpleHintData { raw_values_start: 1635, raw_values_count: 1, token_types_start: 1636, token_types_count: 0 }, // [1063]
    SimpleHintData { raw_values_start: 1636, raw_values_count: 2, token_types_start: 1638, token_types_count: 0 }, // [1064]
    SimpleHintData { raw_values_start: 1638, raw_values_count: 1, token_types_start: 1639, token_types_count: 0 }, // [1065]
    SimpleHintData { raw_values_start: 1639, raw_values_count: 2, token_types_start: 1641, token_types_count: 0 }, // [1066]
    SimpleHintData { raw_values_start: 1641, raw_values_count: 1, token_types_start: 1642, token_types_count: 0 }, // [1067]
    SimpleHintData { raw_values_start: 1642, raw_values_count: 1, token_types_start: 1643, token_types_count: 0 }, // [1068]
    SimpleHintData { raw_values_start: 1643, raw_values_count: 1, token_types_start: 1644, token_types_count: 0 }, // [1069]
    SimpleHintData { raw_values_start: 1644, raw_values_count: 1, token_types_start: 1645, token_types_count: 0 }, // [1070]
    SimpleHintData { raw_values_start: 1645, raw_values_count: 2, token_types_start: 1647, token_types_count: 0 }, // [1071]
    SimpleHintData { raw_values_start: 1647, raw_values_count: 1, token_types_start: 1648, token_types_count: 0 }, // [1072]
    SimpleHintData { raw_values_start: 1648, raw_values_count: 1, token_types_start: 1649, token_types_count: 0 }, // [1073]
    SimpleHintData { raw_values_start: 1649, raw_values_count: 1, token_types_start: 1650, token_types_count: 0 }, // [1074]
    SimpleHintData { raw_values_start: 1650, raw_values_count: 1, token_types_start: 1651, token_types_count: 0 }, // [1075]
    SimpleHintData { raw_values_start: 1651, raw_values_count: 1, token_types_start: 1652, token_types_count: 0 }, // [1076]
    SimpleHintData { raw_values_start: 1652, raw_values_count: 1, token_types_start: 1653, token_types_count: 0 }, // [1077]
    SimpleHintData { raw_values_start: 1653, raw_values_count: 1, token_types_start: 1654, token_types_count: 0 }, // [1078]
    SimpleHintData { raw_values_start: 1654, raw_values_count: 1, token_types_start: 1655, token_types_count: 0 }, // [1079]
    SimpleHintData { raw_values_start: 1655, raw_values_count: 1, token_types_start: 1656, token_types_count: 0 }, // [1080]
    SimpleHintData { raw_values_start: 1656, raw_values_count: 1, token_types_start: 1657, token_types_count: 0 }, // [1081]
    SimpleHintData { raw_values_start: 1657, raw_values_count: 1, token_types_start: 1658, token_types_count: 0 }, // [1082]
    SimpleHintData { raw_values_start: 1658, raw_values_count: 1, token_types_start: 1659, token_types_count: 0 }, // [1083]
    SimpleHintData { raw_values_start: 1659, raw_values_count: 1, token_types_start: 1660, token_types_count: 0 }, // [1084]
    SimpleHintData { raw_values_start: 1660, raw_values_count: 1, token_types_start: 1661, token_types_count: 0 }, // [1085]
    SimpleHintData { raw_values_start: 1661, raw_values_count: 1, token_types_start: 1662, token_types_count: 0 }, // [1086]
    SimpleHintData { raw_values_start: 1662, raw_values_count: 1, token_types_start: 1663, token_types_count: 0 }, // [1087]
    SimpleHintData { raw_values_start: 1663, raw_values_count: 1, token_types_start: 1664, token_types_count: 0 }, // [1088]
    SimpleHintData { raw_values_start: 1664, raw_values_count: 1, token_types_start: 1665, token_types_count: 0 }, // [1089]
    SimpleHintData { raw_values_start: 1665, raw_values_count: 1, token_types_start: 1666, token_types_count: 0 }, // [1090]
    SimpleHintData { raw_values_start: 1666, raw_values_count: 1, token_types_start: 1667, token_types_count: 0 }, // [1091]
    SimpleHintData { raw_values_start: 1667, raw_values_count: 1, token_types_start: 1668, token_types_count: 0 }, // [1092]
    SimpleHintData { raw_values_start: 1668, raw_values_count: 1, token_types_start: 1669, token_types_count: 0 }, // [1093]
    SimpleHintData { raw_values_start: 1669, raw_values_count: 1, token_types_start: 1670, token_types_count: 0 }, // [1094]
    SimpleHintData { raw_values_start: 1670, raw_values_count: 1, token_types_start: 1671, token_types_count: 0 }, // [1095]
    SimpleHintData { raw_values_start: 1671, raw_values_count: 1, token_types_start: 1672, token_types_count: 0 }, // [1096]
    SimpleHintData { raw_values_start: 1672, raw_values_count: 1, token_types_start: 1673, token_types_count: 0 }, // [1097]
    SimpleHintData { raw_values_start: 1673, raw_values_count: 1, token_types_start: 1674, token_types_count: 0 }, // [1098]
    SimpleHintData { raw_values_start: 1674, raw_values_count: 1, token_types_start: 1675, token_types_count: 0 }, // [1099]
    SimpleHintData { raw_values_start: 1675, raw_values_count: 1, token_types_start: 1676, token_types_count: 0 }, // [1100]
    SimpleHintData { raw_values_start: 1676, raw_values_count: 1, token_types_start: 1677, token_types_count: 0 }, // [1101]
    SimpleHintData { raw_values_start: 1677, raw_values_count: 5, token_types_start: 1682, token_types_count: 0 }, // [1102]
    SimpleHintData { raw_values_start: 1682, raw_values_count: 1, token_types_start: 1683, token_types_count: 0 }, // [1103]
    SimpleHintData { raw_values_start: 1683, raw_values_count: 1, token_types_start: 1684, token_types_count: 0 }, // [1104]
    SimpleHintData { raw_values_start: 1684, raw_values_count: 1, token_types_start: 1685, token_types_count: 0 }, // [1105]
    SimpleHintData { raw_values_start: 1685, raw_values_count: 1, token_types_start: 1686, token_types_count: 0 }, // [1106]
    SimpleHintData { raw_values_start: 1686, raw_values_count: 1, token_types_start: 1687, token_types_count: 0 }, // [1107]
    SimpleHintData { raw_values_start: 1687, raw_values_count: 1, token_types_start: 1688, token_types_count: 0 }, // [1108]
    SimpleHintData { raw_values_start: 1688, raw_values_count: 1, token_types_start: 1689, token_types_count: 0 }, // [1109]
    SimpleHintData { raw_values_start: 1689, raw_values_count: 1, token_types_start: 1690, token_types_count: 0 }, // [1110]
    SimpleHintData { raw_values_start: 1690, raw_values_count: 1, token_types_start: 1691, token_types_count: 0 }, // [1111]
    SimpleHintData { raw_values_start: 1691, raw_values_count: 1, token_types_start: 1692, token_types_count: 0 }, // [1112]
    SimpleHintData { raw_values_start: 1692, raw_values_count: 1, token_types_start: 1693, token_types_count: 0 }, // [1113]
    SimpleHintData { raw_values_start: 1693, raw_values_count: 1, token_types_start: 1694, token_types_count: 0 }, // [1114]
    SimpleHintData { raw_values_start: 1694, raw_values_count: 1, token_types_start: 1695, token_types_count: 0 }, // [1115]
    SimpleHintData { raw_values_start: 1695, raw_values_count: 1, token_types_start: 1696, token_types_count: 0 }, // [1116]
    SimpleHintData { raw_values_start: 1696, raw_values_count: 1, token_types_start: 1697, token_types_count: 0 }, // [1117]
    SimpleHintData { raw_values_start: 1697, raw_values_count: 1, token_types_start: 1698, token_types_count: 0 }, // [1118]
    SimpleHintData { raw_values_start: 1698, raw_values_count: 1, token_types_start: 1699, token_types_count: 0 }, // [1119]
    SimpleHintData { raw_values_start: 1699, raw_values_count: 1, token_types_start: 1700, token_types_count: 0 }, // [1120]
    SimpleHintData { raw_values_start: 1700, raw_values_count: 1, token_types_start: 1701, token_types_count: 0 }, // [1121]
    SimpleHintData { raw_values_start: 1701, raw_values_count: 1, token_types_start: 1702, token_types_count: 0 }, // [1122]
    SimpleHintData { raw_values_start: 1702, raw_values_count: 1, token_types_start: 1703, token_types_count: 0 }, // [1123]
    SimpleHintData { raw_values_start: 1703, raw_values_count: 1, token_types_start: 1704, token_types_count: 0 }, // [1124]
    SimpleHintData { raw_values_start: 1704, raw_values_count: 1, token_types_start: 1705, token_types_count: 0 }, // [1125]
    SimpleHintData { raw_values_start: 1705, raw_values_count: 1, token_types_start: 1706, token_types_count: 0 }, // [1126]
    SimpleHintData { raw_values_start: 1706, raw_values_count: 1, token_types_start: 1707, token_types_count: 0 }, // [1127]
    SimpleHintData { raw_values_start: 1707, raw_values_count: 1, token_types_start: 1708, token_types_count: 0 }, // [1128]
    SimpleHintData { raw_values_start: 1708, raw_values_count: 1, token_types_start: 1709, token_types_count: 0 }, // [1129]
    SimpleHintData { raw_values_start: 1709, raw_values_count: 1, token_types_start: 1710, token_types_count: 0 }, // [1130]
    SimpleHintData { raw_values_start: 1710, raw_values_count: 3, token_types_start: 1713, token_types_count: 0 }, // [1131]
    SimpleHintData { raw_values_start: 1713, raw_values_count: 1, token_types_start: 1714, token_types_count: 0 }, // [1132]
    SimpleHintData { raw_values_start: 1714, raw_values_count: 1, token_types_start: 1715, token_types_count: 0 }, // [1133]
    SimpleHintData { raw_values_start: 1715, raw_values_count: 1, token_types_start: 1716, token_types_count: 0 }, // [1134]
    SimpleHintData { raw_values_start: 1716, raw_values_count: 1, token_types_start: 1717, token_types_count: 0 }, // [1135]
    SimpleHintData { raw_values_start: 1717, raw_values_count: 1, token_types_start: 1718, token_types_count: 0 }, // [1136]
    SimpleHintData { raw_values_start: 1718, raw_values_count: 1, token_types_start: 1719, token_types_count: 0 }, // [1137]
    SimpleHintData { raw_values_start: 1719, raw_values_count: 1, token_types_start: 1720, token_types_count: 0 }, // [1138]
    SimpleHintData { raw_values_start: 1720, raw_values_count: 1, token_types_start: 1721, token_types_count: 0 }, // [1139]
    SimpleHintData { raw_values_start: 1721, raw_values_count: 2, token_types_start: 1723, token_types_count: 0 }, // [1140]
    SimpleHintData { raw_values_start: 1723, raw_values_count: 1, token_types_start: 1724, token_types_count: 0 }, // [1141]
    SimpleHintData { raw_values_start: 1724, raw_values_count: 1, token_types_start: 1725, token_types_count: 0 }, // [1142]
    SimpleHintData { raw_values_start: 1725, raw_values_count: 1, token_types_start: 1726, token_types_count: 0 }, // [1143]
    SimpleHintData { raw_values_start: 1726, raw_values_count: 1, token_types_start: 1727, token_types_count: 0 }, // [1144]
    SimpleHintData { raw_values_start: 1727, raw_values_count: 1, token_types_start: 1728, token_types_count: 0 }, // [1145]
    SimpleHintData { raw_values_start: 1728, raw_values_count: 1, token_types_start: 1729, token_types_count: 0 }, // [1146]
    SimpleHintData { raw_values_start: 1729, raw_values_count: 1, token_types_start: 1730, token_types_count: 0 }, // [1147]
    SimpleHintData { raw_values_start: 1730, raw_values_count: 1, token_types_start: 1731, token_types_count: 0 }, // [1148]
    SimpleHintData { raw_values_start: 1731, raw_values_count: 1, token_types_start: 1732, token_types_count: 0 }, // [1149]
    SimpleHintData { raw_values_start: 1732, raw_values_count: 1, token_types_start: 1733, token_types_count: 0 }, // [1150]
    SimpleHintData { raw_values_start: 1733, raw_values_count: 1, token_types_start: 1734, token_types_count: 0 }, // [1151]
    SimpleHintData { raw_values_start: 1734, raw_values_count: 1, token_types_start: 1735, token_types_count: 0 }, // [1152]
    SimpleHintData { raw_values_start: 1735, raw_values_count: 2, token_types_start: 1737, token_types_count: 0 }, // [1153]
    SimpleHintData { raw_values_start: 1737, raw_values_count: 1, token_types_start: 1738, token_types_count: 0 }, // [1154]
    SimpleHintData { raw_values_start: 1738, raw_values_count: 1, token_types_start: 1739, token_types_count: 0 }, // [1155]
    SimpleHintData { raw_values_start: 1739, raw_values_count: 1, token_types_start: 1740, token_types_count: 0 }, // [1156]
    SimpleHintData { raw_values_start: 1740, raw_values_count: 1, token_types_start: 1741, token_types_count: 0 }, // [1157]
    SimpleHintData { raw_values_start: 1741, raw_values_count: 1, token_types_start: 1742, token_types_count: 0 }, // [1158]
    SimpleHintData { raw_values_start: 1742, raw_values_count: 1, token_types_start: 1743, token_types_count: 0 }, // [1159]
    SimpleHintData { raw_values_start: 1743, raw_values_count: 1, token_types_start: 1744, token_types_count: 0 }, // [1160]
    SimpleHintData { raw_values_start: 1744, raw_values_count: 1, token_types_start: 1745, token_types_count: 0 }, // [1161]
    SimpleHintData { raw_values_start: 1745, raw_values_count: 1, token_types_start: 1746, token_types_count: 0 }, // [1162]
    SimpleHintData { raw_values_start: 1746, raw_values_count: 1, token_types_start: 1747, token_types_count: 0 }, // [1163]
    SimpleHintData { raw_values_start: 1747, raw_values_count: 1, token_types_start: 1748, token_types_count: 0 }, // [1164]
    SimpleHintData { raw_values_start: 1748, raw_values_count: 1, token_types_start: 1749, token_types_count: 0 }, // [1165]
    SimpleHintData { raw_values_start: 1749, raw_values_count: 1, token_types_start: 1750, token_types_count: 0 }, // [1166]
    SimpleHintData { raw_values_start: 1750, raw_values_count: 1, token_types_start: 1751, token_types_count: 0 }, // [1167]
    SimpleHintData { raw_values_start: 1751, raw_values_count: 1, token_types_start: 1752, token_types_count: 0 }, // [1168]
    SimpleHintData { raw_values_start: 1752, raw_values_count: 1, token_types_start: 1753, token_types_count: 0 }, // [1169]
    SimpleHintData { raw_values_start: 1753, raw_values_count: 1, token_types_start: 1754, token_types_count: 0 }, // [1170]
    SimpleHintData { raw_values_start: 1754, raw_values_count: 1, token_types_start: 1755, token_types_count: 0 }, // [1171]
    SimpleHintData { raw_values_start: 1755, raw_values_count: 1, token_types_start: 1756, token_types_count: 0 }, // [1172]
    SimpleHintData { raw_values_start: 1756, raw_values_count: 1, token_types_start: 1757, token_types_count: 0 }, // [1173]
    SimpleHintData { raw_values_start: 1757, raw_values_count: 1, token_types_start: 1758, token_types_count: 0 }, // [1174]
    SimpleHintData { raw_values_start: 1758, raw_values_count: 1, token_types_start: 1759, token_types_count: 0 }, // [1175]
    SimpleHintData { raw_values_start: 1759, raw_values_count: 1, token_types_start: 1760, token_types_count: 0 }, // [1176]
    SimpleHintData { raw_values_start: 1760, raw_values_count: 1, token_types_start: 1761, token_types_count: 0 }, // [1177]
    SimpleHintData { raw_values_start: 1761, raw_values_count: 1, token_types_start: 1762, token_types_count: 0 }, // [1178]
    SimpleHintData { raw_values_start: 1762, raw_values_count: 1, token_types_start: 1763, token_types_count: 0 }, // [1179]
    SimpleHintData { raw_values_start: 1763, raw_values_count: 1, token_types_start: 1764, token_types_count: 0 }, // [1180]
    SimpleHintData { raw_values_start: 1764, raw_values_count: 1, token_types_start: 1765, token_types_count: 0 }, // [1181]
    SimpleHintData { raw_values_start: 1765, raw_values_count: 1, token_types_start: 1766, token_types_count: 0 }, // [1182]
    SimpleHintData { raw_values_start: 1766, raw_values_count: 1, token_types_start: 1767, token_types_count: 0 }, // [1183]
    SimpleHintData { raw_values_start: 1767, raw_values_count: 1, token_types_start: 1768, token_types_count: 0 }, // [1184]
    SimpleHintData { raw_values_start: 1768, raw_values_count: 1, token_types_start: 1769, token_types_count: 0 }, // [1185]
    SimpleHintData { raw_values_start: 1769, raw_values_count: 1, token_types_start: 1770, token_types_count: 0 }, // [1186]
    SimpleHintData { raw_values_start: 1770, raw_values_count: 1, token_types_start: 1771, token_types_count: 0 }, // [1187]
    SimpleHintData { raw_values_start: 1771, raw_values_count: 1, token_types_start: 1772, token_types_count: 0 }, // [1188]
    SimpleHintData { raw_values_start: 1772, raw_values_count: 1, token_types_start: 1773, token_types_count: 0 }, // [1189]
    SimpleHintData { raw_values_start: 1773, raw_values_count: 1, token_types_start: 1774, token_types_count: 0 }, // [1190]
    SimpleHintData { raw_values_start: 1774, raw_values_count: 1, token_types_start: 1775, token_types_count: 0 }, // [1191]
    SimpleHintData { raw_values_start: 1775, raw_values_count: 1, token_types_start: 1776, token_types_count: 0 }, // [1192]
    SimpleHintData { raw_values_start: 1776, raw_values_count: 1, token_types_start: 1777, token_types_count: 0 }, // [1193]
    SimpleHintData { raw_values_start: 1777, raw_values_count: 1, token_types_start: 1778, token_types_count: 0 }, // [1194]
    SimpleHintData { raw_values_start: 1778, raw_values_count: 1, token_types_start: 1779, token_types_count: 0 }, // [1195]
    SimpleHintData { raw_values_start: 1779, raw_values_count: 1, token_types_start: 1780, token_types_count: 0 }, // [1196]
    SimpleHintData { raw_values_start: 1780, raw_values_count: 1, token_types_start: 1781, token_types_count: 0 }, // [1197]
    SimpleHintData { raw_values_start: 1781, raw_values_count: 1, token_types_start: 1782, token_types_count: 0 }, // [1198]
    SimpleHintData { raw_values_start: 1782, raw_values_count: 1, token_types_start: 1783, token_types_count: 0 }, // [1199]
    SimpleHintData { raw_values_start: 1783, raw_values_count: 1, token_types_start: 1784, token_types_count: 0 }, // [1200]
    SimpleHintData { raw_values_start: 1784, raw_values_count: 1, token_types_start: 1785, token_types_count: 0 }, // [1201]
    SimpleHintData { raw_values_start: 1785, raw_values_count: 1, token_types_start: 1786, token_types_count: 0 }, // [1202]
    SimpleHintData { raw_values_start: 1786, raw_values_count: 1, token_types_start: 1787, token_types_count: 0 }, // [1203]
    SimpleHintData { raw_values_start: 1787, raw_values_count: 1, token_types_start: 1788, token_types_count: 0 }, // [1204]
    SimpleHintData { raw_values_start: 1788, raw_values_count: 1, token_types_start: 1789, token_types_count: 0 }, // [1205]
    SimpleHintData { raw_values_start: 1789, raw_values_count: 1, token_types_start: 1790, token_types_count: 0 }, // [1206]
    SimpleHintData { raw_values_start: 1790, raw_values_count: 1, token_types_start: 1791, token_types_count: 0 }, // [1207]
    SimpleHintData { raw_values_start: 1791, raw_values_count: 2, token_types_start: 1793, token_types_count: 0 }, // [1208]
    SimpleHintData { raw_values_start: 1793, raw_values_count: 1, token_types_start: 1794, token_types_count: 0 }, // [1209]
    SimpleHintData { raw_values_start: 1794, raw_values_count: 1, token_types_start: 1795, token_types_count: 0 }, // [1210]
    SimpleHintData { raw_values_start: 1795, raw_values_count: 1, token_types_start: 1796, token_types_count: 0 }, // [1211]
    SimpleHintData { raw_values_start: 1796, raw_values_count: 1, token_types_start: 1797, token_types_count: 0 }, // [1212]
    SimpleHintData { raw_values_start: 1797, raw_values_count: 1, token_types_start: 1798, token_types_count: 0 }, // [1213]
    SimpleHintData { raw_values_start: 1798, raw_values_count: 1, token_types_start: 1799, token_types_count: 0 }, // [1214]
    SimpleHintData { raw_values_start: 1799, raw_values_count: 1, token_types_start: 1800, token_types_count: 0 }, // [1215]
    SimpleHintData { raw_values_start: 1800, raw_values_count: 11, token_types_start: 1811, token_types_count: 0 }, // [1216]
    SimpleHintData { raw_values_start: 1811, raw_values_count: 1, token_types_start: 1812, token_types_count: 0 }, // [1217]
    SimpleHintData { raw_values_start: 1812, raw_values_count: 1, token_types_start: 1813, token_types_count: 0 }, // [1218]
    SimpleHintData { raw_values_start: 1813, raw_values_count: 2, token_types_start: 1815, token_types_count: 0 }, // [1219]
    SimpleHintData { raw_values_start: 1815, raw_values_count: 1, token_types_start: 1816, token_types_count: 0 }, // [1220]
    SimpleHintData { raw_values_start: 1816, raw_values_count: 1, token_types_start: 1817, token_types_count: 0 }, // [1221]
    SimpleHintData { raw_values_start: 1817, raw_values_count: 1, token_types_start: 1818, token_types_count: 0 }, // [1222]
    SimpleHintData { raw_values_start: 1818, raw_values_count: 2, token_types_start: 1820, token_types_count: 0 }, // [1223]
    SimpleHintData { raw_values_start: 1820, raw_values_count: 1, token_types_start: 1821, token_types_count: 0 }, // [1224]
    SimpleHintData { raw_values_start: 1821, raw_values_count: 1, token_types_start: 1822, token_types_count: 0 }, // [1225]
    SimpleHintData { raw_values_start: 1822, raw_values_count: 2, token_types_start: 1824, token_types_count: 0 }, // [1226]
    SimpleHintData { raw_values_start: 1824, raw_values_count: 1, token_types_start: 1825, token_types_count: 0 }, // [1227]
    SimpleHintData { raw_values_start: 1825, raw_values_count: 1, token_types_start: 1826, token_types_count: 0 }, // [1228]
    SimpleHintData { raw_values_start: 1826, raw_values_count: 1, token_types_start: 1827, token_types_count: 0 }, // [1229]
    SimpleHintData { raw_values_start: 1827, raw_values_count: 1, token_types_start: 1828, token_types_count: 0 }, // [1230]
    SimpleHintData { raw_values_start: 1828, raw_values_count: 1, token_types_start: 1829, token_types_count: 0 }, // [1231]
    SimpleHintData { raw_values_start: 1829, raw_values_count: 1, token_types_start: 1830, token_types_count: 0 }, // [1232]
    SimpleHintData { raw_values_start: 1830, raw_values_count: 1, token_types_start: 1831, token_types_count: 0 }, // [1233]
];

pub static SIMPLE_HINT_INDICES: &[u32] = &[
    1, 2, 3, 4, 0, 5, 0, 6, 7, 7, 8, 9, 10, 11, 12, 13,
    14, 15, 16, 17, 18, 19, 19, 20, 21, 22, 22, 23, 24, 24, 25, 26,
    10, 27, 27, 25, 26, 10, 28, 24, 24, 29, 26, 14, 27, 27, 29, 26,
    30, 10, 14, 0, 0, 0, 0, 0, 31, 0, 32, 33, 34, 35, 35, 36,
    37, 38, 39, 39, 40, 16, 41, 9, 10, 12, 42, 42, 43, 44, 45, 45,
    46, 47, 47, 48, 8, 8, 49, 50, 51, 52, 52, 39, 39, 53, 14, 43,
    43, 44, 54, 28, 55, 55, 56, 14, 15, 57, 57, 58, 42, 59, 39, 60,
    45, 61, 62, 8, 63, 64, 65, 66, 67, 68, 69, 70, 71, 72, 73, 51,
    74, 75, 24, 24, 58, 31, 38, 38, 76, 28, 77, 78, 79, 80, 81, 82,
    83, 84, 46, 14, 43, 43, 44, 85, 85, 78, 86, 86, 77, 87, 87, 88,
    29, 89, 90, 91, 92, 93, 94, 95, 96, 97, 25, 43, 43, 0, 85, 85,
    90, 86, 86, 89, 87, 87, 0, 98, 99, 100, 0, 0, 0, 41, 41, 0,
    101, 101, 0, 16, 16, 0, 102, 0, 0, 0, 103, 103, 5, 104, 105, 106,
    107, 108, 109, 110, 111, 112, 113, 0, 0, 114, 0, 0, 0, 31, 0, 31,
    115, 116, 0, 117, 0, 0, 0, 24, 118, 119, 120, 121, 122, 123, 123, 124,
    37, 125, 126, 127, 128, 128, 37, 129, 130, 131, 132, 133, 121, 121, 83, 0,
    122, 122, 134, 134, 135, 0, 136, 0, 0, 0, 137, 0, 0, 0, 138, 138,
    108, 62, 135, 0, 139, 140, 140, 141, 110, 0, 31, 134, 142, 142, 143, 114,
    33, 0, 121, 121, 77, 0, 0, 0, 144, 145, 146, 147, 148, 148, 149, 42,
    150, 151, 152, 153, 154, 155, 156, 145, 157, 158, 159, 160, 103, 0, 0, 37,
    0, 161, 103, 162, 0, 163, 103, 0, 0, 0, 114, 114, 114, 164, 165, 166,
    167, 168, 169, 170, 171, 172, 173, 174, 175, 176, 177, 178, 178, 178, 179, 180,
    181, 182, 183, 184, 185, 0, 0, 185, 186, 0, 0, 0, 0, 0, 0, 32,
    114, 0, 0, 187, 188, 189, 190, 191, 192, 193, 151, 157, 194, 195, 0, 196,
    197, 198, 199, 145, 159, 159, 159, 157, 160, 160, 160, 158, 200, 201, 194, 148,
    202, 203, 204, 205, 206, 207, 31, 0, 0, 31, 0, 0, 0, 31, 208, 209,
    210, 211, 212, 213, 214, 124, 31, 0, 0, 0, 31, 114, 31, 215, 128, 216,
    216, 216, 0, 0, 0, 217, 218, 219, 220, 221, 222, 222, 222, 0, 223, 223,
    224, 225, 224, 225, 225, 222, 222, 0, 223, 223, 224, 225, 224, 225, 225, 195,
    32, 193, 222, 226, 227, 15, 228, 229, 230, 231, 232, 233, 0, 234, 235, 236,
    237, 238, 239, 240, 241, 242, 243, 244, 245, 246, 247, 248, 249, 250, 251, 252,
    253, 254, 255, 256, 257, 258, 259, 0, 0, 260, 261, 0, 116, 0, 0, 262,
    0, 263, 264, 265, 266, 267, 268, 161, 161, 161, 0, 0, 0, 0, 185, 0,
    269, 269, 0, 0, 270, 271, 272, 243, 243, 31, 0, 273, 273, 0, 274, 274,
    0, 275, 178, 67, 67, 0, 276, 260, 260, 0, 0, 0, 0, 0, 31, 0,
    0, 0, 0, 135, 0, 0, 262, 0, 277, 0, 0, 0, 0, 278, 32, 279,
    280, 276, 276, 116, 276, 276, 276, 76, 0, 281, 281, 135, 135, 0, 282, 282,
    0, 14, 14, 0, 283, 283, 117, 80, 284, 81, 16, 83, 77, 17, 71, 18,
    285, 78, 0, 286, 116, 0, 0, 31, 0, 0, 0, 286, 116, 0, 287, 288,
    195, 137, 160, 159, 160, 159, 289, 290, 286, 0, 291, 0, 0, 292, 293, 294,
    157, 157, 157, 295, 296, 0, 0, 297, 0, 0, 59, 298, 299, 269, 300, 301,
    302, 303, 304, 305, 306, 307, 308, 309, 310, 311, 311, 311, 312, 313, 135, 314,
    311, 311, 312, 313, 315, 135, 135, 175, 316, 317, 318, 319, 320, 321, 39, 39,
    226, 31, 0, 114, 0, 322, 323, 324, 82, 80, 81, 325, 326, 327, 328, 304,
    329, 0, 31, 330, 330, 0, 114, 114, 167, 39, 39, 10, 331, 0, 39, 39,
    202, 332, 80, 331, 0, 31, 333, 333, 0, 114, 39, 39, 275, 282, 0, 76,
    0, 31, 31, 0, 0, 0, 214, 214, 334, 335, 336, 337, 271, 338, 26, 26,
    0, 339, 339, 340, 341, 341, 31, 0, 0, 0, 0, 342, 343, 256, 344, 344,
    345, 0, 31, 0, 31, 0, 0, 0, 346, 346, 37, 347, 347, 346, 37, 348,
    348, 37, 349, 349, 119, 350, 351, 352, 353, 353, 354, 355, 356, 357, 358, 266,
    266, 359, 360, 361, 362, 363, 364, 292, 292, 365, 366, 367, 368, 368, 341, 369,
    272, 370, 39, 39, 39, 202, 371, 331, 0, 372, 372, 31, 0, 0, 0, 137,
    0, 0, 103, 116, 116, 114, 215, 39, 39, 16, 331, 0, 39, 39, 14, 331,
    0, 373, 123, 123, 124, 37, 374, 374, 322, 37, 125, 126, 127, 128, 128, 37,
    129, 130, 131, 132, 133, 39, 39, 83, 0, 373, 373, 39, 39, 202, 375, 77,
    331, 0, 376, 31, 31, 0, 0, 377, 0, 276, 114, 114, 215, 31, 215, 378,
    378, 0, 0, 39, 39, 71, 0, 379, 187, 110, 380, 380, 381, 382, 382, 61,
    60, 73, 73, 381, 0, 0, 202, 76, 202, 76, 76, 0, 383, 384, 384, 385,
    386, 114, 0, 387, 386, 114, 0, 34, 34, 0, 388, 271, 271, 389, 390, 389,
    391, 391, 391, 392, 391, 391, 393, 330, 330, 394, 395, 386, 396, 223, 223, 31,
    0, 45, 45, 81, 397, 31, 39, 39, 41, 0, 39, 39, 202, 78, 331, 0,
    31, 114, 215, 31, 215, 215, 322, 322, 322, 322, 398, 399, 400, 401, 402, 403,
    404, 404, 405, 405, 404, 406, 31, 0, 407, 408, 409, 410, 411, 412, 413, 414,
    415, 416, 417, 418, 419, 420, 421, 422, 131, 22, 10, 423, 424, 425, 0, 0,
    0, 0, 0, 426, 427, 427, 428, 0, 0, 429, 429, 236, 192, 430, 431, 35,
    35, 36, 0, 0, 0, 262, 0, 31, 0, 432, 0, 0, 433, 434, 434, 31,
    31, 0, 0, 435, 0, 436, 436, 433, 437, 438, 186, 116, 439, 435, 440, 441,
    442, 443, 444, 445, 446, 447, 448, 449, 450, 451, 452, 453, 454, 455, 456, 457,
    458, 459, 460, 461, 462, 462, 463, 124, 76, 464, 465, 466, 31, 0, 0, 467,
    468, 468, 469, 70, 77, 0, 376, 31, 31, 0, 0, 114, 114, 215, 31, 215,
    322, 311, 378, 378, 0, 311, 470, 471, 471, 124, 472, 386, 135, 76, 76, 76,
    287, 473, 60, 474, 475, 476, 477, 271, 271, 477, 76, 76, 478, 473, 60, 474,
    475, 322, 322, 479, 26, 26, 0, 462, 468, 273, 273, 273, 480, 314, 389, 393,
    481, 482, 483, 484, 485, 486, 60, 60, 60, 34, 487, 488, 489, 490, 491, 492,
    493, 494, 495, 496, 497, 498, 498, 498, 0, 0, 499, 500, 501, 363, 502, 503,
    504, 505, 350, 506, 507, 508, 509, 510, 462, 511, 512, 154, 513, 514, 11, 262,
    427, 136, 515, 220, 134, 134, 226, 31, 0, 114, 0, 136, 134, 134, 10, 331,
    0, 136, 134, 134, 80, 331, 0, 134, 134, 282, 331, 0, 136, 134, 134, 134,
    371, 331, 0, 134, 134, 16, 331, 0, 134, 134, 14, 331, 0, 136, 134, 134,
    83, 0, 134, 134, 332, 77, 331, 0, 0, 136, 134, 134, 71, 331, 0, 134,
    134, 18, 331, 0, 136, 134, 134, 41, 331, 0, 134, 134, 78, 331, 0, 136,
    516, 517, 518, 519, 520, 521, 394, 522, 523, 224, 224, 0, 224, 524, 31, 0,
    0, 31, 525, 526, 527, 528, 529, 530, 531, 225, 532, 533, 534, 137, 535, 536,
    537, 538, 539, 540, 541, 369, 313, 542, 543, 45, 50, 544, 545, 546, 547, 548,
    548, 548, 549, 215, 61, 73, 60, 0, 0, 0, 0, 0, 550, 193, 193, 0,
    551, 195, 286, 286, 271, 0, 0, 0, 0, 552, 271, 190, 0, 148, 0, 0,
    0, 0, 553, 554, 554, 554, 555, 271, 556, 0, 0, 557, 557, 557, 557, 151,
    157, 195, 0, 558, 554, 554, 554, 555, 0, 545, 545, 31, 215, 0, 0, 0,
    222, 172, 172, 0, 0, 0, 0, 185, 0, 31, 0, 0, 0, 215, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 31, 0, 0, 0, 0,
    559, 116, 37, 204, 272, 436, 0, 0, 32, 103, 0, 68, 0, 186, 0, 0,
    0, 335, 344, 560, 0, 561, 86, 562, 206, 206, 563, 564, 565, 565, 566, 141,
    567, 0, 37, 0, 386, 568, 386, 475, 569, 340, 322, 322, 570, 565, 571, 572,
    87, 573, 488, 488, 574, 574, 488, 488, 488, 488, 575, 576, 576, 31, 487, 487,
    0, 576, 577, 141, 578, 579, 580, 581, 582, 583, 330, 584, 19, 19, 585, 19,
    88, 586, 587, 588, 589, 589, 0, 0, 408, 408, 386, 0, 0, 37, 186, 590,
    0, 591, 592, 583, 190, 190, 148, 589, 475, 593, 594, 595, 596, 597, 34, 34,
    0, 0, 598, 487, 599, 101, 101, 124, 113, 113, 124, 600, 601, 602, 603, 322,
    322, 565, 604, 604, 330, 322, 322, 322, 0, 0, 605, 0, 31, 0, 0, 0,
    0, 598, 606, 0, 0, 322, 322, 604, 0, 606, 0, 31, 0, 0, 0, 0,
    31, 0, 0, 607, 0, 31, 0, 113, 113, 124, 101, 101, 124, 0, 0, 0,
    0, 0, 0, 113, 113, 124, 101, 101, 124, 0, 34, 608, 609, 0, 0, 0,
    0, 0, 0, 0, 114, 0, 610, 610, 0, 34, 34, 0, 0, 0, 435, 0,
    34, 0, 0, 509, 0, 155, 0, 0, 113, 611, 611, 0, 0, 0, 116, 0,
    0, 26, 0, 116, 0, 0, 612, 0, 0, 31, 31, 0, 114, 114, 116, 13,
    13, 0, 80, 397, 0, 0, 0, 0, 262, 31, 613, 397, 614, 31, 0, 0,
    0, 0, 149, 149, 18, 0, 0, 149, 149, 18, 0, 31, 0, 0, 0, 434,
    434, 31, 0, 0, 0, 0, 0, 434, 0, 480, 31, 0, 92, 615, 27, 616,
    617, 343, 618, 619, 620, 469, 621, 622, 99, 99, 99, 0, 38, 38, 76, 0,
    16, 16, 0, 64, 64, 76, 41, 0, 0, 33, 0, 623, 322, 322, 99, 624,
    322, 322, 109, 624, 310, 310, 408, 48, 625, 625, 124, 0, 420, 626, 0, 625,
    48, 627, 160, 160, 137, 160, 628, 101, 101, 124, 0, 24, 629, 405, 0, 0,
    0, 0, 37, 0, 630, 630, 113, 113, 124, 599, 600, 601, 602, 565, 604, 330,
    322, 322, 322, 101, 0, 0, 0, 0, 37, 0, 31, 630, 629, 629, 629, 631,
    31, 0, 0, 405, 0, 632, 466, 633, 600, 600, 0, 31, 0, 634, 113, 113,
    124, 599, 601, 602, 565, 604, 330, 322, 322, 322, 600, 635, 636, 637, 357, 638,
    639, 640, 641, 0, 642, 643, 644, 645, 646, 647, 648, 649, 0, 315, 650, 651,
    331, 331, 545, 331, 331, 331, 271, 545, 652, 612, 612, 652, 653, 654, 655, 392,
    656, 657, 658, 659, 57, 26, 551, 271, 26, 0, 31, 0, 0, 0, 215, 0,
    341, 312, 660, 123, 0, 0, 0, 0, 661, 164, 497, 467, 282, 662, 663, 664,
    665, 666, 667, 668, 669, 670, 671, 391, 672, 673, 674, 675, 61, 61, 61, 676,
    677, 0, 678, 215, 31, 31, 215, 273, 679, 327, 680, 380, 681, 682, 683, 684,
    685, 686, 687, 12, 688, 689, 690, 691, 186, 186, 692, 37, 37, 693, 116, 435,
    116, 116, 116, 693, 116, 435, 186, 677, 694, 0, 272, 695, 0, 204, 0, 286,
    286, 271, 509, 34, 286, 0, 696, 697, 698, 322, 322, 699, 700, 700, 701, 702,
    703, 402, 704, 705, 706, 707, 708, 0, 0, 0, 709, 0, 0, 0, 0, 0,
    0, 710, 76, 345, 0, 0, 0, 709, 0, 0, 0, 0, 0, 709, 709, 709,
    0, 76, 76, 0, 0, 31, 0, 0, 297, 672, 711, 711, 608, 712, 713, 714,
    345, 345, 31, 0, 0, 715, 716, 717, 585, 718, 719, 370, 0, 720, 721, 0,
    0, 0, 185, 0, 0, 0, 722, 442, 450, 644, 648, 723, 724, 725, 726, 727,
    728, 729, 730, 731, 732, 733, 13, 35, 734, 735, 0, 605, 736, 737, 738, 739,
    740, 741, 712, 742, 743, 159, 159, 137, 159, 347, 744, 550, 550, 271, 745, 0,
    537, 537, 0, 745, 378, 746, 655, 378, 290, 599, 747, 747, 0, 31, 0, 37,
    0, 24, 748, 604, 604, 0, 37, 0, 32, 32, 37, 749, 604, 565, 747, 750,
    751, 752, 0, 753, 0, 116, 37, 204, 554, 272, 436, 103, 0, 754, 0, 0,
    755, 756, 0, 757, 758, 759, 760, 761, 762, 763, 764, 703, 55, 55, 148, 717,
    765, 542, 73, 766, 767, 717, 717, 73, 767, 768, 769, 767, 770, 477, 771, 772,
    773, 774, 775, 776, 358, 777, 778, 779, 607, 607, 262, 0, 31, 371, 371, 0,
    32, 32, 31, 215, 780, 47, 781, 0, 43, 0, 782, 783, 85, 784, 785, 786,
    787, 788, 789, 790, 791, 792, 60, 61, 61, 31, 480, 793, 793, 677, 793, 223,
    223, 223, 223, 223, 783, 148, 148, 0, 794, 795, 73, 60, 223, 223, 271, 783,
    148, 148, 0, 794, 61, 793, 793, 0, 0, 0, 345, 0, 0, 0, 31, 31,
    215, 0, 0, 0, 0, 0, 76, 0, 223, 73, 73, 796, 797, 798, 799, 329,
    723, 724, 800, 801, 802, 803, 348, 804, 153, 725, 805, 806, 726, 807, 607, 808,
    809, 810, 371, 811, 62, 812, 156, 8, 727, 813, 728, 814, 815, 816, 155, 817,
    818, 819, 0, 820, 0, 0, 821, 0, 0, 0, 114, 0, 0, 31, 0, 602,
    602, 0, 0, 822, 695, 695, 823, 0, 824, 825, 153, 0, 826, 387, 827, 0,
    567, 828, 367, 829, 830, 129, 831, 832, 833, 834, 835, 132, 836, 837, 838, 839,
    840, 841, 208, 480, 68, 31, 68, 31, 322, 31, 208, 31, 0, 842, 73, 61,
    60, 793, 208, 208, 843, 31, 68, 208, 844, 845, 846, 847, 0, 848, 849, 0,
    289, 850, 850, 137, 159, 159, 160, 271, 0, 271, 851, 852, 853, 854, 272, 272,
    855, 856, 654, 857, 858, 37, 859, 860, 36, 116, 116, 161, 0, 754, 116, 116,
    754, 262, 262, 262, 262, 262, 0, 76, 114, 345, 32, 227, 103, 31, 193, 161,
    488, 0, 861, 862, 863, 381, 864, 865, 604, 604, 0, 37, 0, 568, 386, 475,
    604, 866, 867, 385, 868, 76, 869, 340, 870, 871, 872, 873, 874, 63, 875, 876,
    877, 878, 624, 879, 372, 202, 202, 202, 202, 479, 113, 113, 124, 0, 0, 0,
    0, 37, 0, 880, 164, 497, 654, 654, 881, 141, 734, 0, 599, 589, 882, 599,
    600, 601, 602, 589, 611, 565, 604, 330, 322, 322, 322, 113, 133, 113, 846, 883,
    884, 471, 885, 886, 714, 887, 888, 0, 612, 889, 0, 0, 31, 0, 889, 335,
    335, 0, 31, 31, 436, 32, 436, 0, 335, 890, 891, 676, 892, 64, 893, 284,
    894, 895, 353, 896, 897, 0, 898, 899, 900, 901, 902, 903, 904, 905, 906, 907,
    908, 908, 124, 0, 31, 0, 0, 908, 338, 909, 910, 911, 912, 913, 914, 154,
    154, 397, 397, 154, 116, 0, 346, 915, 916, 917, 918, 919, 54, 157, 920, 921,
    922, 152, 44, 923, 152, 0, 0, 924, 0, 925, 926, 605, 605, 592, 428, 927,
    928, 929, 930, 474, 931, 274, 274, 585, 274, 932, 556, 933, 58, 934, 935, 81,
    93, 936, 937, 938, 939, 940, 102, 941, 554, 554, 37, 601, 195, 195, 942, 24,
    149, 943, 31, 215, 944, 945, 946, 947, 614, 116, 948, 949, 467, 593, 950, 137,
    160, 951, 159, 850, 65, 330, 330, 952, 65, 65, 340, 565, 565, 340, 953, 954,
    955, 956, 957, 958, 959, 960, 67, 67, 0, 31, 782, 76, 76, 76, 60, 961,
    76, 76, 73, 961, 782, 782, 962, 608, 907, 963, 66, 67, 384, 961, 515, 220,
    796, 796, 272, 367, 367, 105, 796, 796, 273, 964, 965, 966, 967, 968, 969, 970,
    971, 972, 973, 974, 975, 976, 977, 978, 979, 142, 980, 981, 479, 982, 983, 984,
    985, 7, 653, 986, 987, 515, 988, 989, 990, 991, 992, 993, 994, 333, 351, 100,
    100, 100, 99, 99, 624, 330, 0, 38, 38, 76, 0, 16, 16, 0, 64, 64,
    76, 41, 0, 0, 34, 0, 136, 821, 713, 746, 16, 0, 995, 478, 996, 406,
    406, 997, 998, 999, 1000, 1001, 1002, 1003, 1004, 82, 1005, 1006, 1007, 94, 386, 322,
    322, 367, 386, 816, 1008, 1009, 1010, 1011, 1012, 1013, 1014, 475, 1015, 701, 1016, 702,
    1017, 1018, 361, 606, 606, 1019, 189, 1020, 31, 37, 981, 981, 981, 31, 37, 1021,
    1022, 1023, 352, 14, 1024, 25, 1025, 1026, 1027, 1028, 1029, 1030, 1031, 729, 1032, 730,
    1033, 1034, 1035, 0, 0, 0, 0, 0, 1036, 509, 24, 68, 68, 1036, 0, 0,
    1037, 1037, 34, 0, 487, 113, 113, 124, 599, 335, 603, 565, 330, 322, 322, 322,
    68, 68, 68, 34, 487, 101, 600, 335, 602, 113, 604, 565, 599, 602, 330, 322,
    603, 322, 322, 322, 322, 215, 1038, 31, 322, 1038, 31, 322, 208, 31, 215, 1039,
    488, 1040, 611, 83, 126, 787, 787, 37, 367, 367, 787, 125, 807, 807, 37, 367,
    367, 807, 95, 1041, 21, 1042, 1043, 626, 796, 796, 0, 0, 0, 0, 137, 0,
    0, 185, 0, 0, 0, 480, 273, 208, 208, 603, 603, 603, 208, 113, 599, 602,
    330, 322, 322, 322, 796, 603, 1044, 1045, 1045, 690, 540, 24, 804, 796, 796, 14,
    331, 0, 1046, 631, 1047, 766, 1048, 0, 0, 0, 222, 227, 227, 227, 0, 172,
    1049, 1050, 1051, 554, 152, 153, 1052, 963, 0, 0, 614, 262, 0, 0, 0, 0,
    262, 155, 116, 1053, 0, 0, 103, 154, 161, 1054, 1055, 1056, 943, 1057, 1058, 1059,
    1060, 185, 409, 412, 414, 415, 416, 417, 418, 420, 626, 1061, 41, 408, 408, 1062,
    249, 250, 251, 252, 433, 457, 460, 460, 1004, 810, 273, 273, 1063, 101, 484, 548,
    548, 1064, 810, 1056, 563, 563, 171, 658, 658, 1065, 624, 705, 759, 759, 1066, 737,
    738, 768, 768, 1067, 780, 780, 77, 1068, 330, 877, 788, 788, 996, 810, 792, 877,
    877, 938, 284, 284, 913, 913, 945, 945, 877, 964, 964, 111, 14, 21, 1069, 1070,
    1070, 1071, 214, 1072, 437, 438, 1073, 41, 323, 1074, 339, 1075, 354, 1076, 1077, 1078,
    1079, 1080, 1069, 1081, 1082, 1083, 1084, 1085, 1086, 1087, 1088, 1089, 1090, 1091, 1092, 1093,
    1094, 1095, 1096, 1097, 79, 91, 1098, 155, 374, 1099, 1100, 1101, 396, 574, 215, 793,
    61, 1102, 134, 134, 39, 134, 72, 98, 39, 39, 134, 121, 39, 796, 134, 134,
    39, 134, 39, 134, 39, 60, 73, 39, 134, 39, 134, 39, 134, 498, 55, 548,
    39, 121, 134, 39, 134, 216, 276, 468, 488, 328, 364, 1103, 1104, 1105, 1106, 1107,
    1108, 1109, 1110, 1111, 1112, 84, 96, 1113, 157, 157, 1114, 31, 0, 0, 0, 0,
    0, 1115, 1116, 1117, 1118, 1119, 1120, 1121, 1122, 1123, 1124, 0, 356, 1125, 1126, 1127,
    1128, 1129, 1130, 1020, 1072, 1061, 377, 269, 269, 0, 1131, 275, 275, 31, 274, 274,
    31, 19, 19, 31, 67, 0, 0, 480, 185, 0, 0, 31, 215, 31, 793, 77,
    1132, 89, 606, 17, 0, 553, 553, 193, 0, 0, 558, 558, 0, 46, 97, 69,
    1133, 1070, 0, 332, 69, 70, 70, 375, 1134, 332, 1135, 1136, 1137, 1138, 1139, 794,
    570, 555, 437, 426, 426, 437, 438, 31, 1140, 1140, 322, 334, 437, 1141, 172, 172,
    172, 437, 1141, 0, 1067, 438, 1073, 1142, 1143, 1144, 1145, 1146, 33, 1147, 1148, 1149,
    1150, 1151, 1152, 1102, 1102, 374, 188, 287, 478, 225, 1153, 1152, 1154, 820, 820, 0,
    148, 148, 367, 232, 1155, 1156, 1157, 1158, 1063, 1159, 1134, 1160, 1161, 1162, 71, 1163,
    1164, 1165, 1166, 610, 207, 739, 1150, 205, 205, 72, 72, 72, 77, 0, 1167, 1168,
    31, 0, 0, 18, 0, 0, 103, 0, 0, 31, 1068, 1169, 1170, 590, 1171, 401,
    0, 0, 401, 0, 1172, 1173, 1174, 1044, 1044, 1036, 509, 24, 1044, 275, 275, 275,
    1175, 0, 1176, 1177, 1178, 1179, 68, 68, 34, 487, 101, 600, 335, 602, 603, 322,
    322, 113, 599, 330, 322, 322, 322, 208, 208, 603, 603, 603, 208, 362, 432, 1180,
    73, 73, 73, 0, 0, 796, 796, 34, 487, 1181, 1182, 704, 51, 55, 55, 55,
    0, 74, 41, 0, 1183, 1184, 1185, 1186, 1187, 345, 1188, 1189, 1190, 1191, 1192, 1193,
    1194, 1195, 1196, 480, 480, 0, 0, 31, 0, 0, 273, 0, 273, 0, 113, 599,
    480, 1197, 1198, 1199, 1200, 1201, 1202, 285, 1203, 1204, 431, 1205, 1206, 1207, 322, 322,
    1208, 221, 757, 243, 624, 78, 90, 1209, 1210, 1211, 9, 1212, 1213, 1214, 223, 223,
    0, 0, 0, 794, 0, 0, 0, 0, 223, 1215, 487, 487, 0, 31, 0, 1216,
    599, 101, 101, 124, 113, 113, 124, 600, 601, 602, 335, 565, 604, 330, 322, 322,
    322, 487, 1217, 0, 1218, 0, 0, 0, 0, 0, 0, 0, 262, 155, 155, 262,
    155, 602, 0, 0, 1219, 908, 113, 908, 113, 589, 322, 322, 959, 0, 0, 0,
    68, 0, 842, 322, 322, 959, 0, 0, 0, 68, 0, 208, 322, 322, 367, 367,
    22, 0, 322, 322, 322, 367, 14, 196, 924, 924, 101, 31, 113, 924, 334, 1220,
    0, 1154, 23, 75, 1221, 1222, 343, 343, 1223, 585, 370, 345, 1224, 116, 114, 1225,
    114, 114, 1076, 0, 1226, 1226, 652, 976, 0, 480, 1227, 1228, 1229, 1065, 1224, 1230,
    1231, 731, 1232, 732, 366, 1233, 1141,
];

pub fn get_db2_segment_grammar(name: &str) -> Option<RootGrammar> {
    match name {
        "AbortKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(0), tables: &DB2_TABLES }),
        "AbsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1), tables: &DB2_TABLES }),
        "AbsoluteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2), tables: &DB2_TABLES }),
        "AccessKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3), tables: &DB2_TABLES }),
        "AccessObjectSegment" => Some(RootGrammar { grammar_id: GrammarId(4), tables: &DB2_TABLES }),
        "AccessPermissionSegment" => Some(RootGrammar { grammar_id: GrammarId(65), tables: &DB2_TABLES }),
        "AccessPermissionsSegment" => Some(RootGrammar { grammar_id: GrammarId(134), tables: &DB2_TABLES }),
        "AccessSchemaObjectSegment" => Some(RootGrammar { grammar_id: GrammarId(137), tables: &DB2_TABLES }),
        "AccessSchemaPluralObjectSegment" => Some(RootGrammar { grammar_id: GrammarId(160), tables: &DB2_TABLES }),
        "AccessStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(183), tables: &DB2_TABLES }),
        "AccessTargetSegment" => Some(RootGrammar { grammar_id: GrammarId(186), tables: &DB2_TABLES }),
        "AccessorGrammar" => Some(RootGrammar { grammar_id: GrammarId(202), tables: &DB2_TABLES }),
        "AccountKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(204), tables: &DB2_TABLES }),
        "AccountsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(205), tables: &DB2_TABLES }),
        "ActionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(206), tables: &DB2_TABLES }),
        "ActivateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(207), tables: &DB2_TABLES }),
        "AdaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(208), tables: &DB2_TABLES }),
        "AddKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(209), tables: &DB2_TABLES }),
        "AdminKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(210), tables: &DB2_TABLES }),
        "AfterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(211), tables: &DB2_TABLES }),
        "AgeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(212), tables: &DB2_TABLES }),
        "AggregateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(213), tables: &DB2_TABLES }),
        "AggregateOrderByClause" => Some(RootGrammar { grammar_id: GrammarId(214), tables: &DB2_TABLES }),
        "AliasExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(215), tables: &DB2_TABLES }),
        "AliasKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(227), tables: &DB2_TABLES }),
        "AliasedTableReferenceGrammar" => Some(RootGrammar { grammar_id: GrammarId(228), tables: &DB2_TABLES }),
        "AllKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(231), tables: &DB2_TABLES }),
        "AllocateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(232), tables: &DB2_TABLES }),
        "AllowKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(233), tables: &DB2_TABLES }),
        "AlsoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(234), tables: &DB2_TABLES }),
        "AlterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(235), tables: &DB2_TABLES }),
        "AlterSequenceOptionsSegment" => Some(RootGrammar { grammar_id: GrammarId(236), tables: &DB2_TABLES }),
        "AlterSequenceStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(252), tables: &DB2_TABLES }),
        "AlterTableDropColumnGrammar" => Some(RootGrammar { grammar_id: GrammarId(258), tables: &DB2_TABLES }),
        "AlterTableOptionsGrammar" => Some(RootGrammar { grammar_id: GrammarId(263), tables: &DB2_TABLES }),
        "AlterTableStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(290), tables: &DB2_TABLES }),
        "AlwaysKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(296), tables: &DB2_TABLES }),
        "AmpersandSegment" => Some(RootGrammar { grammar_id: GrammarId(297), tables: &DB2_TABLES }),
        "AnalyseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(298), tables: &DB2_TABLES }),
        "AnalyzeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(299), tables: &DB2_TABLES }),
        "AndKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(300), tables: &DB2_TABLES }),
        "AndOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(301), tables: &DB2_TABLES }),
        "AnyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(302), tables: &DB2_TABLES }),
        "ApplyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(303), tables: &DB2_TABLES }),
        "AreKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(304), tables: &DB2_TABLES }),
        "ArithmeticBinaryOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(305), tables: &DB2_TABLES }),
        "ArrayAccessorSegment" => Some(RootGrammar { grammar_id: GrammarId(316), tables: &DB2_TABLES }),
        "ArrayExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(324), tables: &DB2_TABLES }),
        "ArrayKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(325), tables: &DB2_TABLES }),
        "ArrayLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(326), tables: &DB2_TABLES }),
        "ArrayTypeSegment" => Some(RootGrammar { grammar_id: GrammarId(329), tables: &DB2_TABLES }),
        "AsAliasOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(330), tables: &DB2_TABLES }),
        "AsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(332), tables: &DB2_TABLES }),
        "AscKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(333), tables: &DB2_TABLES }),
        "AsensitiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(334), tables: &DB2_TABLES }),
        "AssertionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(335), tables: &DB2_TABLES }),
        "AssignmentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(336), tables: &DB2_TABLES }),
        "AssociateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(337), tables: &DB2_TABLES }),
        "AsutimeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(338), tables: &DB2_TABLES }),
        "AsymmetricKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(339), tables: &DB2_TABLES }),
        "AsynchronyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(340), tables: &DB2_TABLES }),
        "AtKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(341), tables: &DB2_TABLES }),
        "AtomicKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(342), tables: &DB2_TABLES }),
        "AttributeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(343), tables: &DB2_TABLES }),
        "AttributesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(344), tables: &DB2_TABLES }),
        "AuditKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(345), tables: &DB2_TABLES }),
        "AuthorizationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(346), tables: &DB2_TABLES }),
        "AutoIncrementGrammar" => Some(RootGrammar { grammar_id: GrammarId(347), tables: &DB2_TABLES }),
        "Auto_incrementKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(349), tables: &DB2_TABLES }),
        "AuxKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(350), tables: &DB2_TABLES }),
        "AuxiliaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(351), tables: &DB2_TABLES }),
        "AvgKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(352), tables: &DB2_TABLES }),
        "Avg_row_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(353), tables: &DB2_TABLES }),
        "BackupKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(354), tables: &DB2_TABLES }),
        "BackwardKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(355), tables: &DB2_TABLES }),
        "BareFunctionSegment" => Some(RootGrammar { grammar_id: GrammarId(356), tables: &DB2_TABLES }),
        "BaseExpressionElementGrammar" => Some(RootGrammar { grammar_id: GrammarId(357), tables: &DB2_TABLES }),
        "BaseFileSegment" => Some(RootGrammar { grammar_id: GrammarId(369), tables: &DB2_TABLES }),
        "BaseSegment" => Some(RootGrammar { grammar_id: GrammarId(370), tables: &DB2_TABLES }),
        "BeforeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(371), tables: &DB2_TABLES }),
        "BeginKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(372), tables: &DB2_TABLES }),
        "BernoulliKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(373), tables: &DB2_TABLES }),
        "BetweenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(374), tables: &DB2_TABLES }),
        "BigintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(375), tables: &DB2_TABLES }),
        "BinaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(376), tables: &DB2_TABLES }),
        "BinaryOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(377), tables: &DB2_TABLES }),
        "BinaryOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(382), tables: &DB2_TABLES }),
        "BindingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(383), tables: &DB2_TABLES }),
        "BitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(384), tables: &DB2_TABLES }),
        "Bit_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(385), tables: &DB2_TABLES }),
        "BitvarKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(386), tables: &DB2_TABLES }),
        "BitwiseAndSegment" => Some(RootGrammar { grammar_id: GrammarId(387), tables: &DB2_TABLES }),
        "BitwiseLShiftSegment" => Some(RootGrammar { grammar_id: GrammarId(388), tables: &DB2_TABLES }),
        "BitwiseOrSegment" => Some(RootGrammar { grammar_id: GrammarId(391), tables: &DB2_TABLES }),
        "BitwiseRShiftSegment" => Some(RootGrammar { grammar_id: GrammarId(392), tables: &DB2_TABLES }),
        "BitwiseXorSegment" => Some(RootGrammar { grammar_id: GrammarId(395), tables: &DB2_TABLES }),
        "BlobKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(396), tables: &DB2_TABLES }),
        "BoolKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(397), tables: &DB2_TABLES }),
        "BooleanBinaryOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(398), tables: &DB2_TABLES }),
        "BooleanKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(401), tables: &DB2_TABLES }),
        "BooleanLiteralGrammar" => Some(RootGrammar { grammar_id: GrammarId(402), tables: &DB2_TABLES }),
        "BothKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(405), tables: &DB2_TABLES }),
        "BracketedArguments" => Some(RootGrammar { grammar_id: GrammarId(406), tables: &DB2_TABLES }),
        "BracketedColumnReferenceListGrammar" => Some(RootGrammar { grammar_id: GrammarId(409), tables: &DB2_TABLES }),
        "BracketedSegment" => Some(RootGrammar { grammar_id: GrammarId(412), tables: &DB2_TABLES }),
        "BracketedSetExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(413), tables: &DB2_TABLES }),
        "BreadthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(415), tables: &DB2_TABLES }),
        "BreakKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(416), tables: &DB2_TABLES }),
        "BrowseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(417), tables: &DB2_TABLES }),
        "BufferpoolKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(418), tables: &DB2_TABLES }),
        "BulkKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(419), tables: &DB2_TABLES }),
        "Business_timeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(420), tables: &DB2_TABLES }),
        "ByKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(421), tables: &DB2_TABLES }),
        "CTEColumnList" => Some(RootGrammar { grammar_id: GrammarId(422), tables: &DB2_TABLES }),
        "CTEDefinitionSegment" => Some(RootGrammar { grammar_id: GrammarId(424), tables: &DB2_TABLES }),
        "CacheKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(430), tables: &DB2_TABLES }),
        "CallKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(431), tables: &DB2_TABLES }),
        "CallStoredProcedureSegment" => Some(RootGrammar { grammar_id: GrammarId(432), tables: &DB2_TABLES }),
        "CalledKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(437), tables: &DB2_TABLES }),
        "CaptureKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(438), tables: &DB2_TABLES }),
        "CardinalityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(439), tables: &DB2_TABLES }),
        "CascadeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(440), tables: &DB2_TABLES }),
        "CascadedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(441), tables: &DB2_TABLES }),
        "CaseExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(442), tables: &DB2_TABLES }),
        "CaseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(466), tables: &DB2_TABLES }),
        "CastKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(467), tables: &DB2_TABLES }),
        "CastOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(468), tables: &DB2_TABLES }),
        "CatalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(469), tables: &DB2_TABLES }),
        "Catalog_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(470), tables: &DB2_TABLES }),
        "CcsidKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(471), tables: &DB2_TABLES }),
        "CeilKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(472), tables: &DB2_TABLES }),
        "CeilingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(473), tables: &DB2_TABLES }),
        "ChainKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(474), tables: &DB2_TABLES }),
        "ChangeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(475), tables: &DB2_TABLES }),
        "CharCharacterSetGrammar" => Some(RootGrammar { grammar_id: GrammarId(476), tables: &DB2_TABLES }),
        "CharKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(477), tables: &DB2_TABLES }),
        "Char_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(478), tables: &DB2_TABLES }),
        "CharacterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(479), tables: &DB2_TABLES }),
        "Character_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(480), tables: &DB2_TABLES }),
        "Character_set_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(481), tables: &DB2_TABLES }),
        "Character_set_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(482), tables: &DB2_TABLES }),
        "Character_set_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(483), tables: &DB2_TABLES }),
        "CharacteristicsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(484), tables: &DB2_TABLES }),
        "CharactersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(485), tables: &DB2_TABLES }),
        "CheckKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(486), tables: &DB2_TABLES }),
        "CheckedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(487), tables: &DB2_TABLES }),
        "CheckpointKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(488), tables: &DB2_TABLES }),
        "ChecksumKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(489), tables: &DB2_TABLES }),
        "ClassKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(490), tables: &DB2_TABLES }),
        "Class_originKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(491), tables: &DB2_TABLES }),
        "Client_acctngKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(492), tables: &DB2_TABLES }),
        "Client_applnameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(493), tables: &DB2_TABLES }),
        "Client_useridKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(494), tables: &DB2_TABLES }),
        "Client_wrkstnnameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(495), tables: &DB2_TABLES }),
        "ClobKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(496), tables: &DB2_TABLES }),
        "CloneKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(497), tables: &DB2_TABLES }),
        "CloseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(498), tables: &DB2_TABLES }),
        "ClusterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(499), tables: &DB2_TABLES }),
        "ClusteredKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(500), tables: &DB2_TABLES }),
        "CoalesceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(501), tables: &DB2_TABLES }),
        "CobolKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(502), tables: &DB2_TABLES }),
        "CodeSegment" => Some(RootGrammar { grammar_id: GrammarId(503), tables: &DB2_TABLES }),
        "CollateGrammar" => Some(RootGrammar { grammar_id: GrammarId(504), tables: &DB2_TABLES }),
        "CollateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(505), tables: &DB2_TABLES }),
        "CollationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(506), tables: &DB2_TABLES }),
        "CollationReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(507), tables: &DB2_TABLES }),
        "Collation_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(513), tables: &DB2_TABLES }),
        "Collation_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(514), tables: &DB2_TABLES }),
        "Collation_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(515), tables: &DB2_TABLES }),
        "CollectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(516), tables: &DB2_TABLES }),
        "CollectionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(517), tables: &DB2_TABLES }),
        "CollidKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(518), tables: &DB2_TABLES }),
        "ColonDelimiterSegment" => Some(RootGrammar { grammar_id: GrammarId(519), tables: &DB2_TABLES }),
        "ColonPrefixSegment" => Some(RootGrammar { grammar_id: GrammarId(520), tables: &DB2_TABLES }),
        "ColonSegment" => Some(RootGrammar { grammar_id: GrammarId(521), tables: &DB2_TABLES }),
        "ColumnConstraintDefaultGrammar" => Some(RootGrammar { grammar_id: GrammarId(522), tables: &DB2_TABLES }),
        "ColumnConstraintSegment" => Some(RootGrammar { grammar_id: GrammarId(527), tables: &DB2_TABLES }),
        "ColumnDefinitionSegment" => Some(RootGrammar { grammar_id: GrammarId(555), tables: &DB2_TABLES }),
        "ColumnGeneratedGrammar" => Some(RootGrammar { grammar_id: GrammarId(562), tables: &DB2_TABLES }),
        "ColumnKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(563), tables: &DB2_TABLES }),
        "ColumnReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(564), tables: &DB2_TABLES }),
        "Column_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(568), tables: &DB2_TABLES }),
        "ColumnsExpressionFunctionContentsSegment" => Some(RootGrammar { grammar_id: GrammarId(569), tables: &DB2_TABLES }),
        "ColumnsExpressionFunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(570), tables: &DB2_TABLES }),
        "ColumnsExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(571), tables: &DB2_TABLES }),
        "ColumnsExpressionNameGrammar" => Some(RootGrammar { grammar_id: GrammarId(572), tables: &DB2_TABLES }),
        "ColumnsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(573), tables: &DB2_TABLES }),
        "CommaSegment" => Some(RootGrammar { grammar_id: GrammarId(574), tables: &DB2_TABLES }),
        "Command_functionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(575), tables: &DB2_TABLES }),
        "Command_function_codeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(576), tables: &DB2_TABLES }),
        "CommentClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(577), tables: &DB2_TABLES }),
        "CommentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(580), tables: &DB2_TABLES }),
        "CommentOnStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(581), tables: &DB2_TABLES }),
        "CommentSegment" => Some(RootGrammar { grammar_id: GrammarId(621), tables: &DB2_TABLES }),
        "CommitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(622), tables: &DB2_TABLES }),
        "CommittedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(623), tables: &DB2_TABLES }),
        "ComparisonOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(624), tables: &DB2_TABLES }),
        "ComparisonOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(633), tables: &DB2_TABLES }),
        "CompletionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(634), tables: &DB2_TABLES }),
        "CompositeBinaryOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(635), tables: &DB2_TABLES }),
        "CompositeComparisonOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(636), tables: &DB2_TABLES }),
        "CompressKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(637), tables: &DB2_TABLES }),
        "ComputeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(638), tables: &DB2_TABLES }),
        "ConcatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(639), tables: &DB2_TABLES }),
        "ConcatSegment" => Some(RootGrammar { grammar_id: GrammarId(640), tables: &DB2_TABLES }),
        "ConditionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(643), tables: &DB2_TABLES }),
        "Condition_numberKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(644), tables: &DB2_TABLES }),
        "ConditionalCrossJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(645), tables: &DB2_TABLES }),
        "ConditionalJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(646), tables: &DB2_TABLES }),
        "ConnectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(650), tables: &DB2_TABLES }),
        "ConnectionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(651), tables: &DB2_TABLES }),
        "Connection_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(652), tables: &DB2_TABLES }),
        "ConstraintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(653), tables: &DB2_TABLES }),
        "Constraint_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(654), tables: &DB2_TABLES }),
        "Constraint_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(655), tables: &DB2_TABLES }),
        "Constraint_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(656), tables: &DB2_TABLES }),
        "ConstraintsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(657), tables: &DB2_TABLES }),
        "ConstructorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(658), tables: &DB2_TABLES }),
        "ContainsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(659), tables: &DB2_TABLES }),
        "ContainstableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(660), tables: &DB2_TABLES }),
        "ContinueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(661), tables: &DB2_TABLES }),
        "ConversionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(662), tables: &DB2_TABLES }),
        "ConvertKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(663), tables: &DB2_TABLES }),
        "CopyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(664), tables: &DB2_TABLES }),
        "CopyOptionsSegment" => Some(RootGrammar { grammar_id: GrammarId(665), tables: &DB2_TABLES }),
        "CorrKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(680), tables: &DB2_TABLES }),
        "CorrespondingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(681), tables: &DB2_TABLES }),
        "CountKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(682), tables: &DB2_TABLES }),
        "Count_bigKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(683), tables: &DB2_TABLES }),
        "Covar_popKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(684), tables: &DB2_TABLES }),
        "Covar_sampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(685), tables: &DB2_TABLES }),
        "CreateCastStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(686), tables: &DB2_TABLES }),
        "CreateDatabaseStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(713), tables: &DB2_TABLES }),
        "CreateFunctionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(718), tables: &DB2_TABLES }),
        "CreateIndexStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(730), tables: &DB2_TABLES }),
        "CreateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(818), tables: &DB2_TABLES }),
        "CreateModelStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(819), tables: &DB2_TABLES }),
        "CreateRoleStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(839), tables: &DB2_TABLES }),
        "CreateSchemaStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(844), tables: &DB2_TABLES }),
        "CreateSequenceOptionsSegment" => Some(RootGrammar { grammar_id: GrammarId(849), tables: &DB2_TABLES }),
        "CreateSequenceStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(869), tables: &DB2_TABLES }),
        "CreateTableStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(875), tables: &DB2_TABLES }),
        "CreateTriggerStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(899), tables: &DB2_TABLES }),
        "CreateUserStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(966), tables: &DB2_TABLES }),
        "CreateViewStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(970), tables: &DB2_TABLES }),
        "CreatedbKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(986), tables: &DB2_TABLES }),
        "CreateroleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(987), tables: &DB2_TABLES }),
        "CreateuserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(988), tables: &DB2_TABLES }),
        "CrossKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(989), tables: &DB2_TABLES }),
        "CsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(990), tables: &DB2_TABLES }),
        "CsvKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(991), tables: &DB2_TABLES }),
        "CubeFunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(992), tables: &DB2_TABLES }),
        "CubeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(993), tables: &DB2_TABLES }),
        "CubeRollupClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(994), tables: &DB2_TABLES }),
        "Cume_distKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1000), tables: &DB2_TABLES }),
        "CurrentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1001), tables: &DB2_TABLES }),
        "Current_dateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1002), tables: &DB2_TABLES }),
        "Current_default_transform_groupKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1003), tables: &DB2_TABLES }),
        "Current_lc_ctypeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1004), tables: &DB2_TABLES }),
        "Current_pathKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1005), tables: &DB2_TABLES }),
        "Current_roleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1006), tables: &DB2_TABLES }),
        "Current_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1007), tables: &DB2_TABLES }),
        "Current_serverKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1008), tables: &DB2_TABLES }),
        "Current_timeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1009), tables: &DB2_TABLES }),
        "Current_timestampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1010), tables: &DB2_TABLES }),
        "Current_timezoneKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1011), tables: &DB2_TABLES }),
        "Current_transform_group_for_typeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1012), tables: &DB2_TABLES }),
        "Current_userKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1013), tables: &DB2_TABLES }),
        "CursorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1014), tables: &DB2_TABLES }),
        "Cursor_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1015), tables: &DB2_TABLES }),
        "CycleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1016), tables: &DB2_TABLES }),
        "DataKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1017), tables: &DB2_TABLES }),
        "DatabaseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1018), tables: &DB2_TABLES }),
        "DatabaseReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(564), tables: &DB2_TABLES }),
        "DatabasesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1019), tables: &DB2_TABLES }),
        "DatapartitionnameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1020), tables: &DB2_TABLES }),
        "DatapartitionnumKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1021), tables: &DB2_TABLES }),
        "DatatypeIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(1022), tables: &DB2_TABLES }),
        "DatatypeSegment" => Some(RootGrammar { grammar_id: GrammarId(1026), tables: &DB2_TABLES }),
        "DateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1052), tables: &DB2_TABLES }),
        "DatePartFunctionName" => Some(RootGrammar { grammar_id: GrammarId(1053), tables: &DB2_TABLES }),
        "DatePartFunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(1054), tables: &DB2_TABLES }),
        "DateTimeFunctionContentsSegment" => Some(RootGrammar { grammar_id: GrammarId(1055), tables: &DB2_TABLES }),
        "DateTimeLiteralGrammar" => Some(RootGrammar { grammar_id: GrammarId(1061), tables: &DB2_TABLES }),
        "DatetimeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1068), tables: &DB2_TABLES }),
        "DatetimeUnitSegment" => Some(RootGrammar { grammar_id: GrammarId(1069), tables: &DB2_TABLES }),
        "Datetime_interval_codeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1070), tables: &DB2_TABLES }),
        "Datetime_interval_precisionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1071), tables: &DB2_TABLES }),
        "DayKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1072), tables: &DB2_TABLES }),
        "Day_hourKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1073), tables: &DB2_TABLES }),
        "Day_microsecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1074), tables: &DB2_TABLES }),
        "Day_minuteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1075), tables: &DB2_TABLES }),
        "Day_secondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1076), tables: &DB2_TABLES }),
        "DayofmonthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1077), tables: &DB2_TABLES }),
        "DayofweekKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1078), tables: &DB2_TABLES }),
        "DayofyearKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1079), tables: &DB2_TABLES }),
        "DaysKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1080), tables: &DB2_TABLES }),
        "Db2generalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1081), tables: &DB2_TABLES }),
        "Db2genrlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1082), tables: &DB2_TABLES }),
        "Db2sqlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1083), tables: &DB2_TABLES }),
        "DbccKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1084), tables: &DB2_TABLES }),
        "DbinfoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1085), tables: &DB2_TABLES }),
        "DbpartitionnameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1086), tables: &DB2_TABLES }),
        "DbpartitionnumKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1087), tables: &DB2_TABLES }),
        "DeallocateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1088), tables: &DB2_TABLES }),
        "DecKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1089), tables: &DB2_TABLES }),
        "DecfloatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1090), tables: &DB2_TABLES }),
        "DecimalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1091), tables: &DB2_TABLES }),
        "DeclareDistributionClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1092), tables: &DB2_TABLES }),
        "DeclareGlobalTempTableSegment" => Some(RootGrammar { grammar_id: GrammarId(1104), tables: &DB2_TABLES }),
        "DeclareKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1160), tables: &DB2_TABLES }),
        "Dedent" => Some(RootGrammar { grammar_id: GrammarId(226), tables: &DB2_TABLES }),
        "DefaultKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1161), tables: &DB2_TABLES }),
        "DefaultValuesGrammar" => Some(RootGrammar { grammar_id: GrammarId(1162), tables: &DB2_TABLES }),
        "DefaultsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1165), tables: &DB2_TABLES }),
        "DeferrableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1166), tables: &DB2_TABLES }),
        "DeferredKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1167), tables: &DB2_TABLES }),
        "DefinedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1168), tables: &DB2_TABLES }),
        "DefinerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1169), tables: &DB2_TABLES }),
        "DefinitionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1170), tables: &DB2_TABLES }),
        "DegreeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1171), tables: &DB2_TABLES }),
        "Delay_key_writeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1172), tables: &DB2_TABLES }),
        "DelayedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1173), tables: &DB2_TABLES }),
        "DeleteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1174), tables: &DB2_TABLES }),
        "DeleteStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1175), tables: &DB2_TABLES }),
        "DelimiterGrammar" => Some(RootGrammar { grammar_id: GrammarId(1179), tables: &DB2_TABLES }),
        "DelimiterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1180), tables: &DB2_TABLES }),
        "DelimitersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1181), tables: &DB2_TABLES }),
        "Dense_rankKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1182), tables: &DB2_TABLES }),
        "DenserankKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1183), tables: &DB2_TABLES }),
        "DenyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1184), tables: &DB2_TABLES }),
        "DepthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1185), tables: &DB2_TABLES }),
        "DerefKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1186), tables: &DB2_TABLES }),
        "DerivedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1187), tables: &DB2_TABLES }),
        "DescKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1188), tables: &DB2_TABLES }),
        "DescribeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1189), tables: &DB2_TABLES }),
        "DescribeStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1190), tables: &DB2_TABLES }),
        "DescriptorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1194), tables: &DB2_TABLES }),
        "DestroyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1195), tables: &DB2_TABLES }),
        "DestructorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1196), tables: &DB2_TABLES }),
        "DetailedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1197), tables: &DB2_TABLES }),
        "DeterministicKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1198), tables: &DB2_TABLES }),
        "DiagnosticsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1199), tables: &DB2_TABLES }),
        "DictionaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1200), tables: &DB2_TABLES }),
        "DisableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1201), tables: &DB2_TABLES }),
        "DisallowKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1202), tables: &DB2_TABLES }),
        "DisconnectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1203), tables: &DB2_TABLES }),
        "DiskKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1204), tables: &DB2_TABLES }),
        "DispatchKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1205), tables: &DB2_TABLES }),
        "DistinctKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1206), tables: &DB2_TABLES }),
        "DistinctrowKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1207), tables: &DB2_TABLES }),
        "DistributeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1208), tables: &DB2_TABLES }),
        "DistributedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1209), tables: &DB2_TABLES }),
        "DivKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1210), tables: &DB2_TABLES }),
        "DivideSegment" => Some(RootGrammar { grammar_id: GrammarId(1211), tables: &DB2_TABLES }),
        "DoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1212), tables: &DB2_TABLES }),
        "DocumentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1213), tables: &DB2_TABLES }),
        "DomainKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1214), tables: &DB2_TABLES }),
        "DotSegment" => Some(RootGrammar { grammar_id: GrammarId(1215), tables: &DB2_TABLES }),
        "DoubleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1216), tables: &DB2_TABLES }),
        "DropBehaviorGrammar" => Some(RootGrammar { grammar_id: GrammarId(1217), tables: &DB2_TABLES }),
        "DropCastStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1220), tables: &DB2_TABLES }),
        "DropDatabaseStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1228), tables: &DB2_TABLES }),
        "DropFunctionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1234), tables: &DB2_TABLES }),
        "DropIndexStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1239), tables: &DB2_TABLES }),
        "DropKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1245), tables: &DB2_TABLES }),
        "DropModelStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1246), tables: &DB2_TABLES }),
        "DropRoleStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1251), tables: &DB2_TABLES }),
        "DropSchemaStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1256), tables: &DB2_TABLES }),
        "DropSequenceStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1262), tables: &DB2_TABLES }),
        "DropTableStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1266), tables: &DB2_TABLES }),
        "DropTriggerStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1274), tables: &DB2_TABLES }),
        "DropTypeStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1279), tables: &DB2_TABLES }),
        "DropUserStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1285), tables: &DB2_TABLES }),
        "DropViewStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1290), tables: &DB2_TABLES }),
        "DssizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1296), tables: &DB2_TABLES }),
        "DummyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1297), tables: &DB2_TABLES }),
        "DumpKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1298), tables: &DB2_TABLES }),
        "DynamicKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1299), tables: &DB2_TABLES }),
        "Dynamic_functionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1300), tables: &DB2_TABLES }),
        "Dynamic_function_codeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1301), tables: &DB2_TABLES }),
        "EachKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1302), tables: &DB2_TABLES }),
        "EditprocKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1303), tables: &DB2_TABLES }),
        "ElementKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1304), tables: &DB2_TABLES }),
        "ElseClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1305), tables: &DB2_TABLES }),
        "ElseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1308), tables: &DB2_TABLES }),
        "ElseifKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1309), tables: &DB2_TABLES }),
        "EmptyStructLiteralBracketsSegment" => Some(RootGrammar { grammar_id: GrammarId(1310), tables: &DB2_TABLES }),
        "EmptyStructLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(1311), tables: &DB2_TABLES }),
        "EnableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1314), tables: &DB2_TABLES }),
        "EnclosedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1315), tables: &DB2_TABLES }),
        "EncodingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1316), tables: &DB2_TABLES }),
        "EncryptedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1317), tables: &DB2_TABLES }),
        "EncryptionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1318), tables: &DB2_TABLES }),
        "End-execKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1319), tables: &DB2_TABLES }),
        "EndBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(224), tables: &DB2_TABLES }),
        "EndCurlyBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(1320), tables: &DB2_TABLES }),
        "EndKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1321), tables: &DB2_TABLES }),
        "EndSquareBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(323), tables: &DB2_TABLES }),
        "EndingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1322), tables: &DB2_TABLES }),
        "EnumKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1323), tables: &DB2_TABLES }),
        "EqualsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1324), tables: &DB2_TABLES }),
        "EqualsSegment" => Some(RootGrammar { grammar_id: GrammarId(1325), tables: &DB2_TABLES }),
        "EraseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1326), tables: &DB2_TABLES }),
        "ErrlvlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1327), tables: &DB2_TABLES }),
        "EscapeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1328), tables: &DB2_TABLES }),
        "EscapedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1329), tables: &DB2_TABLES }),
        "EveryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1330), tables: &DB2_TABLES }),
        "ExceptKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1331), tables: &DB2_TABLES }),
        "ExceptionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1332), tables: &DB2_TABLES }),
        "ExcludeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1333), tables: &DB2_TABLES }),
        "ExcludingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1334), tables: &DB2_TABLES }),
        "ExclusiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1335), tables: &DB2_TABLES }),
        "ExecKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1336), tables: &DB2_TABLES }),
        "ExecuteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1337), tables: &DB2_TABLES }),
        "ExecutionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1338), tables: &DB2_TABLES }),
        "ExistingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1339), tables: &DB2_TABLES }),
        "ExistsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1340), tables: &DB2_TABLES }),
        "ExitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1341), tables: &DB2_TABLES }),
        "ExpKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1342), tables: &DB2_TABLES }),
        "ExplainKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1343), tables: &DB2_TABLES }),
        "ExplainStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1344), tables: &DB2_TABLES }),
        "ExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(1351), tables: &DB2_TABLES }),
        "Expression_A_Grammar" => Some(RootGrammar { grammar_id: GrammarId(1352), tables: &DB2_TABLES }),
        "Expression_A_Unary_Operator_Grammar" => Some(RootGrammar { grammar_id: GrammarId(1378), tables: &DB2_TABLES }),
        "Expression_B_Grammar" => Some(RootGrammar { grammar_id: GrammarId(1385), tables: &DB2_TABLES }),
        "Expression_B_Unary_Operator_Grammar" => Some(RootGrammar { grammar_id: GrammarId(1395), tables: &DB2_TABLES }),
        "Expression_C_Grammar" => Some(RootGrammar { grammar_id: GrammarId(1400), tables: &DB2_TABLES }),
        "Expression_D_Grammar" => Some(RootGrammar { grammar_id: GrammarId(1413), tables: &DB2_TABLES }),
        "Expression_D_Potential_Select_Statement_Without_Brackets" => Some(RootGrammar { grammar_id: GrammarId(1450), tables: &DB2_TABLES }),
        "ExtendKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1458), tables: &DB2_TABLES }),
        "ExtendedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1459), tables: &DB2_TABLES }),
        "ExtendedNaturalJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(1460), tables: &DB2_TABLES }),
        "ExtensionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1461), tables: &DB2_TABLES }),
        "ExtensionReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(564), tables: &DB2_TABLES }),
        "ExternalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1462), tables: &DB2_TABLES }),
        "ExtractKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1463), tables: &DB2_TABLES }),
        "FalseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1464), tables: &DB2_TABLES }),
        "FalseSegment" => Some(RootGrammar { grammar_id: GrammarId(1465), tables: &DB2_TABLES }),
        "FederatedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1466), tables: &DB2_TABLES }),
        "FencedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1467), tables: &DB2_TABLES }),
        "FetchClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1468), tables: &DB2_TABLES }),
        "FetchKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1485), tables: &DB2_TABLES }),
        "FieldprocKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1486), tables: &DB2_TABLES }),
        "FieldsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1487), tables: &DB2_TABLES }),
        "FileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1488), tables: &DB2_TABLES }),
        "FileSegment" => Some(RootGrammar { grammar_id: GrammarId(1489), tables: &DB2_TABLES }),
        "FillfactorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1498), tables: &DB2_TABLES }),
        "FilterClauseGrammar" => Some(RootGrammar { grammar_id: GrammarId(1499), tables: &DB2_TABLES }),
        "FilterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1505), tables: &DB2_TABLES }),
        "FinalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1506), tables: &DB2_TABLES }),
        "FirstKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1507), tables: &DB2_TABLES }),
        "Float4KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1508), tables: &DB2_TABLES }),
        "Float8KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1509), tables: &DB2_TABLES }),
        "FloatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1510), tables: &DB2_TABLES }),
        "FloorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1511), tables: &DB2_TABLES }),
        "FlushKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1512), tables: &DB2_TABLES }),
        "FollowingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1513), tables: &DB2_TABLES }),
        "ForKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1514), tables: &DB2_TABLES }),
        "ForceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1515), tables: &DB2_TABLES }),
        "ForeignKeyGrammar" => Some(RootGrammar { grammar_id: GrammarId(1516), tables: &DB2_TABLES }),
        "ForeignKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1519), tables: &DB2_TABLES }),
        "FormatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1520), tables: &DB2_TABLES }),
        "FortranKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1521), tables: &DB2_TABLES }),
        "ForwardKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1522), tables: &DB2_TABLES }),
        "FoundKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1523), tables: &DB2_TABLES }),
        "FrameClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1524), tables: &DB2_TABLES }),
        "FrameClauseUnitGrammar" => Some(RootGrammar { grammar_id: GrammarId(1543), tables: &DB2_TABLES }),
        "FreeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1546), tables: &DB2_TABLES }),
        "FreetextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1547), tables: &DB2_TABLES }),
        "FreetexttableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1548), tables: &DB2_TABLES }),
        "FreezeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1549), tables: &DB2_TABLES }),
        "FromClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1550), tables: &DB2_TABLES }),
        "FromClauseTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(1554), tables: &DB2_TABLES }),
        "FromExpressionElementSegment" => Some(RootGrammar { grammar_id: GrammarId(1576), tables: &DB2_TABLES }),
        "FromExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(1599), tables: &DB2_TABLES }),
        "FromKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1626), tables: &DB2_TABLES }),
        "FullKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1627), tables: &DB2_TABLES }),
        "FulltextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1628), tables: &DB2_TABLES }),
        "FunctionContentsExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(1629), tables: &DB2_TABLES }),
        "FunctionContentsGrammar" => Some(RootGrammar { grammar_id: GrammarId(1632), tables: &DB2_TABLES }),
        "FunctionContentsSegment" => Some(RootGrammar { grammar_id: GrammarId(1673), tables: &DB2_TABLES }),
        "FunctionDefinitionGrammar" => Some(RootGrammar { grammar_id: GrammarId(1676), tables: &DB2_TABLES }),
        "FunctionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1682), tables: &DB2_TABLES }),
        "FunctionNameIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(1683), tables: &DB2_TABLES }),
        "FunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(1684), tables: &DB2_TABLES }),
        "FunctionParameterGrammar" => Some(RootGrammar { grammar_id: GrammarId(1694), tables: &DB2_TABLES }),
        "FunctionParameterListGrammar" => Some(RootGrammar { grammar_id: GrammarId(1707), tables: &DB2_TABLES }),
        "FunctionSegment" => Some(RootGrammar { grammar_id: GrammarId(1710), tables: &DB2_TABLES }),
        "FunctionsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1724), tables: &DB2_TABLES }),
        "FusionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1725), tables: &DB2_TABLES }),
        "FutureKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1726), tables: &DB2_TABLES }),
        "GKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1727), tables: &DB2_TABLES }),
        "GeneralKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1728), tables: &DB2_TABLES }),
        "GenerateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1729), tables: &DB2_TABLES }),
        "GeneratedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1730), tables: &DB2_TABLES }),
        "GetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1731), tables: &DB2_TABLES }),
        "GlobOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(1732), tables: &DB2_TABLES }),
        "GlobalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1733), tables: &DB2_TABLES }),
        "GoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1734), tables: &DB2_TABLES }),
        "GotoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1735), tables: &DB2_TABLES }),
        "GrantKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1736), tables: &DB2_TABLES }),
        "GrantStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1737), tables: &DB2_TABLES }),
        "GrantedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1775), tables: &DB2_TABLES }),
        "GrantsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1776), tables: &DB2_TABLES }),
        "GraphicKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1777), tables: &DB2_TABLES }),
        "GreaterThanOrEqualToSegment" => Some(RootGrammar { grammar_id: GrammarId(1778), tables: &DB2_TABLES }),
        "GreaterThanSegment" => Some(RootGrammar { grammar_id: GrammarId(1781), tables: &DB2_TABLES }),
        "GreatestKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1782), tables: &DB2_TABLES }),
        "GroupByClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1783), tables: &DB2_TABLES }),
        "GroupByClauseTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(1797), tables: &DB2_TABLES }),
        "GroupKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1811), tables: &DB2_TABLES }),
        "GroupingExpressionList" => Some(RootGrammar { grammar_id: GrammarId(1812), tables: &DB2_TABLES }),
        "GroupingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1820), tables: &DB2_TABLES }),
        "GroupingSetsClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1821), tables: &DB2_TABLES }),
        "HandlerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1829), tables: &DB2_TABLES }),
        "HashKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1830), tables: &DB2_TABLES }),
        "Hashed_valueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1831), tables: &DB2_TABLES }),
        "HavingClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1832), tables: &DB2_TABLES }),
        "HavingClauseTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(1837), tables: &DB2_TABLES }),
        "HavingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1850), tables: &DB2_TABLES }),
        "HeaderKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1851), tables: &DB2_TABLES }),
        "HeapKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1852), tables: &DB2_TABLES }),
        "HierarchyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1853), tables: &DB2_TABLES }),
        "HighKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1854), tables: &DB2_TABLES }),
        "High_priorityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1855), tables: &DB2_TABLES }),
        "HintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1856), tables: &DB2_TABLES }),
        "HoldKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1857), tables: &DB2_TABLES }),
        "HoldlockKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1858), tables: &DB2_TABLES }),
        "HorizontalJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(1859), tables: &DB2_TABLES }),
        "HostKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1860), tables: &DB2_TABLES }),
        "HostsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1861), tables: &DB2_TABLES }),
        "HourKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1862), tables: &DB2_TABLES }),
        "Hour_microsecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1863), tables: &DB2_TABLES }),
        "Hour_minuteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1864), tables: &DB2_TABLES }),
        "Hour_secondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1865), tables: &DB2_TABLES }),
        "HoursKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1866), tables: &DB2_TABLES }),
        "IdentifiedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1867), tables: &DB2_TABLES }),
        "IdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(1868), tables: &DB2_TABLES }),
        "IdentityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1869), tables: &DB2_TABLES }),
        "Identity_insertKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1870), tables: &DB2_TABLES }),
        "IdentitycolKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1871), tables: &DB2_TABLES }),
        "IfExistsGrammar" => Some(RootGrammar { grammar_id: GrammarId(1872), tables: &DB2_TABLES }),
        "IfKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1875), tables: &DB2_TABLES }),
        "IfNotExistsGrammar" => Some(RootGrammar { grammar_id: GrammarId(1876), tables: &DB2_TABLES }),
        "IgnoreKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1880), tables: &DB2_TABLES }),
        "IgnoreRespectNullsGrammar" => Some(RootGrammar { grammar_id: GrammarId(1881), tables: &DB2_TABLES }),
        "IlikeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1886), tables: &DB2_TABLES }),
        "ImmediateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1887), tables: &DB2_TABLES }),
        "ImmutableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1888), tables: &DB2_TABLES }),
        "ImplementationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1889), tables: &DB2_TABLES }),
        "ImplicitIndent" => Some(RootGrammar { grammar_id: GrammarId(445), tables: &DB2_TABLES }),
        "ImplicitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1890), tables: &DB2_TABLES }),
        "ImportKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1891), tables: &DB2_TABLES }),
        "ImportedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1892), tables: &DB2_TABLES }),
        "InKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1893), tables: &DB2_TABLES }),
        "InOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(1894), tables: &DB2_TABLES }),
        "IncludeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1904), tables: &DB2_TABLES }),
        "IncludingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1905), tables: &DB2_TABLES }),
        "InclusiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1906), tables: &DB2_TABLES }),
        "IncrementKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1907), tables: &DB2_TABLES }),
        "Indent" => Some(RootGrammar { grammar_id: GrammarId(216), tables: &DB2_TABLES }),
        "IndexColumnDefinitionSegment" => Some(RootGrammar { grammar_id: GrammarId(1908), tables: &DB2_TABLES }),
        "IndexKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1916), tables: &DB2_TABLES }),
        "IndexReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(564), tables: &DB2_TABLES }),
        "IndicatorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1917), tables: &DB2_TABLES }),
        "IndicatorsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1918), tables: &DB2_TABLES }),
        "InfKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1919), tables: &DB2_TABLES }),
        "InfileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1920), tables: &DB2_TABLES }),
        "InfinityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1921), tables: &DB2_TABLES }),
        "InfixKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1922), tables: &DB2_TABLES }),
        "InheritKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1923), tables: &DB2_TABLES }),
        "InheritsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1924), tables: &DB2_TABLES }),
        "InitialKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1925), tables: &DB2_TABLES }),
        "InitializeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1926), tables: &DB2_TABLES }),
        "InitiallyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1927), tables: &DB2_TABLES }),
        "InnerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1928), tables: &DB2_TABLES }),
        "InoutKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1929), tables: &DB2_TABLES }),
        "InputKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1930), tables: &DB2_TABLES }),
        "InsensitiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1931), tables: &DB2_TABLES }),
        "InsertKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1932), tables: &DB2_TABLES }),
        "InsertStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1933), tables: &DB2_TABLES }),
        "Insert_idKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1944), tables: &DB2_TABLES }),
        "InstanceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1945), tables: &DB2_TABLES }),
        "InstantiableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1946), tables: &DB2_TABLES }),
        "InsteadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1947), tables: &DB2_TABLES }),
        "Int1KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1948), tables: &DB2_TABLES }),
        "Int2KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1949), tables: &DB2_TABLES }),
        "Int3KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1950), tables: &DB2_TABLES }),
        "Int4KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1951), tables: &DB2_TABLES }),
        "Int8KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1952), tables: &DB2_TABLES }),
        "IntKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1953), tables: &DB2_TABLES }),
        "IntegerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1954), tables: &DB2_TABLES }),
        "IntegrationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1955), tables: &DB2_TABLES }),
        "IntegrationsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1956), tables: &DB2_TABLES }),
        "IntegrityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1957), tables: &DB2_TABLES }),
        "IntersectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1958), tables: &DB2_TABLES }),
        "IntersectionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1959), tables: &DB2_TABLES }),
        "IntervalExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(1960), tables: &DB2_TABLES }),
        "IntervalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1974), tables: &DB2_TABLES }),
        "IntoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1975), tables: &DB2_TABLES }),
        "InvokerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1976), tables: &DB2_TABLES }),
        "IsClauseGrammar" => Some(RootGrammar { grammar_id: GrammarId(1977), tables: &DB2_TABLES }),
        "IsDistinctFromGrammar" => Some(RootGrammar { grammar_id: GrammarId(1983), tables: &DB2_TABLES }),
        "IsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1988), tables: &DB2_TABLES }),
        "IsNullGrammar" => Some(RootGrammar { grammar_id: GrammarId(1989), tables: &DB2_TABLES }),
        "IsamKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1990), tables: &DB2_TABLES }),
        "IsnullKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1991), tables: &DB2_TABLES }),
        "IsobidKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1992), tables: &DB2_TABLES }),
        "IsolationClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1993), tables: &DB2_TABLES }),
        "IsolationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2003), tables: &DB2_TABLES }),
        "IterateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2004), tables: &DB2_TABLES }),
        "JarKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2005), tables: &DB2_TABLES }),
        "JavaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2006), tables: &DB2_TABLES }),
        "JoinClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2007), tables: &DB2_TABLES }),
        "JoinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2029), tables: &DB2_TABLES }),
        "JoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(2030), tables: &DB2_TABLES }),
        "JoinLikeClauseGrammar" => Some(RootGrammar { grammar_id: GrammarId(2032), tables: &DB2_TABLES }),
        "JoinOnConditionSegment" => Some(RootGrammar { grammar_id: GrammarId(2033), tables: &DB2_TABLES }),
        "JoinTypeKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(2040), tables: &DB2_TABLES }),
        "JoinUsingConditionGrammar" => Some(RootGrammar { grammar_id: GrammarId(2048), tables: &DB2_TABLES }),
        "JsonKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2053), tables: &DB2_TABLES }),
        "KKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2054), tables: &DB2_TABLES }),
        "KeepKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2055), tables: &DB2_TABLES }),
        "KeyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2056), tables: &DB2_TABLES }),
        "Key_memberKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2057), tables: &DB2_TABLES }),
        "Key_typeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2058), tables: &DB2_TABLES }),
        "KeysKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2059), tables: &DB2_TABLES }),
        "KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2060), tables: &DB2_TABLES }),
        "KillKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2061), tables: &DB2_TABLES }),
        "LabelKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2062), tables: &DB2_TABLES }),
        "LabeledDurationGrammar" => Some(RootGrammar { grammar_id: GrammarId(2063), tables: &DB2_TABLES }),
        "LancompilerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2085), tables: &DB2_TABLES }),
        "LanguageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2086), tables: &DB2_TABLES }),
        "LargeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2087), tables: &DB2_TABLES }),
        "LastKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2088), tables: &DB2_TABLES }),
        "Last_insert_idKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2089), tables: &DB2_TABLES }),
        "LateralColumnAliasExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(2090), tables: &DB2_TABLES }),
        "LateralKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2091), tables: &DB2_TABLES }),
        "Lc_ctypeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2092), tables: &DB2_TABLES }),
        "Lc_messagesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2093), tables: &DB2_TABLES }),
        "Lc_timeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2094), tables: &DB2_TABLES }),
        "LeadingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2095), tables: &DB2_TABLES }),
        "LeastKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2096), tables: &DB2_TABLES }),
        "LeaveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2097), tables: &DB2_TABLES }),
        "LeftKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2098), tables: &DB2_TABLES }),
        "LengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2099), tables: &DB2_TABLES }),
        "LessKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2100), tables: &DB2_TABLES }),
        "LessThanOrEqualToSegment" => Some(RootGrammar { grammar_id: GrammarId(2101), tables: &DB2_TABLES }),
        "LessThanSegment" => Some(RootGrammar { grammar_id: GrammarId(2104), tables: &DB2_TABLES }),
        "Level2KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2105), tables: &DB2_TABLES }),
        "LevelKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2106), tables: &DB2_TABLES }),
        "LikeExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(2107), tables: &DB2_TABLES }),
        "LikeGrammar" => Some(RootGrammar { grammar_id: GrammarId(2115), tables: &DB2_TABLES }),
        "LikeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2119), tables: &DB2_TABLES }),
        "LikeOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(2120), tables: &DB2_TABLES }),
        "LimitClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2121), tables: &DB2_TABLES }),
        "LimitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2142), tables: &DB2_TABLES }),
        "LinenoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2143), tables: &DB2_TABLES }),
        "LinesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2144), tables: &DB2_TABLES }),
        "LinktypeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2145), tables: &DB2_TABLES }),
        "ListComprehensionGrammar" => Some(RootGrammar { grammar_id: GrammarId(2146), tables: &DB2_TABLES }),
        "ListenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2147), tables: &DB2_TABLES }),
        "LiteralGrammar" => Some(RootGrammar { grammar_id: GrammarId(2148), tables: &DB2_TABLES }),
        "LiteralKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2158), tables: &DB2_TABLES }),
        "LiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(2159), tables: &DB2_TABLES }),
        "LnKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2160), tables: &DB2_TABLES }),
        "LoadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2161), tables: &DB2_TABLES }),
        "LocalAliasSegment" => Some(RootGrammar { grammar_id: GrammarId(2162), tables: &DB2_TABLES }),
        "LocalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2163), tables: &DB2_TABLES }),
        "LocaldateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2164), tables: &DB2_TABLES }),
        "LocaleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2165), tables: &DB2_TABLES }),
        "LocaltimeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2166), tables: &DB2_TABLES }),
        "LocaltimestampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2167), tables: &DB2_TABLES }),
        "LocationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2168), tables: &DB2_TABLES }),
        "LocatorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2169), tables: &DB2_TABLES }),
        "LocatorsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2170), tables: &DB2_TABLES }),
        "LockClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2171), tables: &DB2_TABLES }),
        "LockKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2185), tables: &DB2_TABLES }),
        "LockmaxKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2186), tables: &DB2_TABLES }),
        "LocksKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2187), tables: &DB2_TABLES }),
        "LocksizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2188), tables: &DB2_TABLES }),
        "LoggedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2189), tables: &DB2_TABLES }),
        "LoginKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2190), tables: &DB2_TABLES }),
        "LogsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2191), tables: &DB2_TABLES }),
        "LongKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2192), tables: &DB2_TABLES }),
        "LongblobKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2193), tables: &DB2_TABLES }),
        "LongtextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2194), tables: &DB2_TABLES }),
        "LoopKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2195), tables: &DB2_TABLES }),
        "LowKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2196), tables: &DB2_TABLES }),
        "Low_priorityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2197), tables: &DB2_TABLES }),
        "LowerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2198), tables: &DB2_TABLES }),
        "MKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2199), tables: &DB2_TABLES }),
        "MLTableExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(2200), tables: &DB2_TABLES }),
        "MaintainedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2212), tables: &DB2_TABLES }),
        "ManageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2213), tables: &DB2_TABLES }),
        "MapKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2214), tables: &DB2_TABLES }),
        "MapTypeSegment" => Some(RootGrammar { grammar_id: GrammarId(2215), tables: &DB2_TABLES }),
        "MaskingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2216), tables: &DB2_TABLES }),
        "MatchConditionSegment" => Some(RootGrammar { grammar_id: GrammarId(2217), tables: &DB2_TABLES }),
        "MatchKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2218), tables: &DB2_TABLES }),
        "MatchedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2219), tables: &DB2_TABLES }),
        "MaterializedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2220), tables: &DB2_TABLES }),
        "MaxKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2221), tables: &DB2_TABLES }),
        "Max_rowsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2222), tables: &DB2_TABLES }),
        "MaxextentsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2223), tables: &DB2_TABLES }),
        "MaxvalueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2224), tables: &DB2_TABLES }),
        "MdcKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2225), tables: &DB2_TABLES }),
        "MediumblobKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2226), tables: &DB2_TABLES }),
        "MediumintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2227), tables: &DB2_TABLES }),
        "MediumtextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2228), tables: &DB2_TABLES }),
        "MemberKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2229), tables: &DB2_TABLES }),
        "MergeDeleteClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2230), tables: &DB2_TABLES }),
        "MergeInsertClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2231), tables: &DB2_TABLES }),
        "MergeIntoLiteralGrammar" => Some(RootGrammar { grammar_id: GrammarId(2235), tables: &DB2_TABLES }),
        "MergeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2238), tables: &DB2_TABLES }),
        "MergeMatchSegment" => Some(RootGrammar { grammar_id: GrammarId(2239), tables: &DB2_TABLES }),
        "MergeMatchedClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2242), tables: &DB2_TABLES }),
        "MergeNotMatchedClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2252), tables: &DB2_TABLES }),
        "MergeStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2261), tables: &DB2_TABLES }),
        "MergeUpdateClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2281), tables: &DB2_TABLES }),
        "Message_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2284), tables: &DB2_TABLES }),
        "Message_octet_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2285), tables: &DB2_TABLES }),
        "Message_textKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2286), tables: &DB2_TABLES }),
        "MethodKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2287), tables: &DB2_TABLES }),
        "MicrosecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2288), tables: &DB2_TABLES }),
        "MicrosecondsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2289), tables: &DB2_TABLES }),
        "MiddleintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2290), tables: &DB2_TABLES }),
        "MillisecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2291), tables: &DB2_TABLES }),
        "MinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2292), tables: &DB2_TABLES }),
        "Min_rowsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2293), tables: &DB2_TABLES }),
        "MinpctusedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2294), tables: &DB2_TABLES }),
        "MinusKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2295), tables: &DB2_TABLES }),
        "MinusSegment" => Some(RootGrammar { grammar_id: GrammarId(2296), tables: &DB2_TABLES }),
        "MinuteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2297), tables: &DB2_TABLES }),
        "Minute_microsecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2298), tables: &DB2_TABLES }),
        "Minute_secondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2299), tables: &DB2_TABLES }),
        "MinutesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2300), tables: &DB2_TABLES }),
        "MinvalueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2301), tables: &DB2_TABLES }),
        "MlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2302), tables: &DB2_TABLES }),
        "MlslabelKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2303), tables: &DB2_TABLES }),
        "ModKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2304), tables: &DB2_TABLES }),
        "ModeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2305), tables: &DB2_TABLES }),
        "ModelKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2306), tables: &DB2_TABLES }),
        "ModifiesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2307), tables: &DB2_TABLES }),
        "ModifyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2308), tables: &DB2_TABLES }),
        "ModuleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2309), tables: &DB2_TABLES }),
        "ModuloSegment" => Some(RootGrammar { grammar_id: GrammarId(2310), tables: &DB2_TABLES }),
        "MonitorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2311), tables: &DB2_TABLES }),
        "MonthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2312), tables: &DB2_TABLES }),
        "MonthnameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2313), tables: &DB2_TABLES }),
        "MonthsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2314), tables: &DB2_TABLES }),
        "MoreKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2315), tables: &DB2_TABLES }),
        "MoveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2316), tables: &DB2_TABLES }),
        "MovementKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2317), tables: &DB2_TABLES }),
        "MultiplySegment" => Some(RootGrammar { grammar_id: GrammarId(2318), tables: &DB2_TABLES }),
        "MultisetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2319), tables: &DB2_TABLES }),
        "MumpsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2320), tables: &DB2_TABLES }),
        "MyisamKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2321), tables: &DB2_TABLES }),
        "NakedIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(2322), tables: &DB2_TABLES }),
        "NameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2323), tables: &DB2_TABLES }),
        "NamedArgumentSegment" => Some(RootGrammar { grammar_id: GrammarId(2324), tables: &DB2_TABLES }),
        "NamedWindowExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(2328), tables: &DB2_TABLES }),
        "NamedWindowSegment" => Some(RootGrammar { grammar_id: GrammarId(2335), tables: &DB2_TABLES }),
        "NamesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2339), tables: &DB2_TABLES }),
        "NanKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2340), tables: &DB2_TABLES }),
        "NanLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(2341), tables: &DB2_TABLES }),
        "NationalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2342), tables: &DB2_TABLES }),
        "NaturalJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(2343), tables: &DB2_TABLES }),
        "NcharKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2344), tables: &DB2_TABLES }),
        "NclobKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2345), tables: &DB2_TABLES }),
        "NegativeSegment" => Some(RootGrammar { grammar_id: GrammarId(2346), tables: &DB2_TABLES }),
        "NestedJoinGrammar" => Some(RootGrammar { grammar_id: GrammarId(2347), tables: &DB2_TABLES }),
        "NestingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2348), tables: &DB2_TABLES }),
        "NewKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2349), tables: &DB2_TABLES }),
        "New_tableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2350), tables: &DB2_TABLES }),
        "NewlineSegment" => Some(RootGrammar { grammar_id: GrammarId(2351), tables: &DB2_TABLES }),
        "NextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2352), tables: &DB2_TABLES }),
        "NextvalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2353), tables: &DB2_TABLES }),
        "NoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2354), tables: &DB2_TABLES }),
        "No_write_to_binlogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2355), tables: &DB2_TABLES }),
        "NoauditKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2356), tables: &DB2_TABLES }),
        "NocacheKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2357), tables: &DB2_TABLES }),
        "NocheckKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2358), tables: &DB2_TABLES }),
        "NocompressKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2359), tables: &DB2_TABLES }),
        "NocreatedbKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2360), tables: &DB2_TABLES }),
        "NocreateroleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2361), tables: &DB2_TABLES }),
        "NocreateuserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2362), tables: &DB2_TABLES }),
        "NocycleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2363), tables: &DB2_TABLES }),
        "NodenameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2364), tables: &DB2_TABLES }),
        "NodenumberKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2365), tables: &DB2_TABLES }),
        "NoinheritKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2366), tables: &DB2_TABLES }),
        "NologinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2367), tables: &DB2_TABLES }),
        "NomaxvalueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2368), tables: &DB2_TABLES }),
        "NominvalueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2369), tables: &DB2_TABLES }),
        "NonSetSelectableGrammar" => Some(RootGrammar { grammar_id: GrammarId(2370), tables: &DB2_TABLES }),
        "NonStandardJoinTypeKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(2380), tables: &DB2_TABLES }),
        "NonWithNonSelectableGrammar" => Some(RootGrammar { grammar_id: GrammarId(2381), tables: &DB2_TABLES }),
        "NonWithSelectableGrammar" => Some(RootGrammar { grammar_id: GrammarId(2386), tables: &DB2_TABLES }),
        "NonclusteredKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2392), tables: &DB2_TABLES }),
        "NoneKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2393), tables: &DB2_TABLES }),
        "NoorderKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2394), tables: &DB2_TABLES }),
        "NormalizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2395), tables: &DB2_TABLES }),
        "NormalizedGrammar" => Some(RootGrammar { grammar_id: GrammarId(2396), tables: &DB2_TABLES }),
        "NormalizedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2397), tables: &DB2_TABLES }),
        "NosuperuserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2398), tables: &DB2_TABLES }),
        "NotEnforcedGrammar" => Some(RootGrammar { grammar_id: GrammarId(2399), tables: &DB2_TABLES }),
        "NotEqualToSegment" => Some(RootGrammar { grammar_id: GrammarId(2400), tables: &DB2_TABLES }),
        "NotKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2407), tables: &DB2_TABLES }),
        "NotNullGrammar" => Some(RootGrammar { grammar_id: GrammarId(2408), tables: &DB2_TABLES }),
        "NotOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(2409), tables: &DB2_TABLES }),
        "NothingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2410), tables: &DB2_TABLES }),
        "NotifyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2411), tables: &DB2_TABLES }),
        "NotnullKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2412), tables: &DB2_TABLES }),
        "NowaitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2413), tables: &DB2_TABLES }),
        "NullKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2414), tables: &DB2_TABLES }),
        "NullLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(2415), tables: &DB2_TABLES }),
        "NullableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2416), tables: &DB2_TABLES }),
        "NullifKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2417), tables: &DB2_TABLES }),
        "NullsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2418), tables: &DB2_TABLES }),
        "NumberKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2419), tables: &DB2_TABLES }),
        "NumericKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2420), tables: &DB2_TABLES }),
        "NumericLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(2421), tables: &DB2_TABLES }),
        "NumpartsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2422), tables: &DB2_TABLES }),
        "ObidKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2423), tables: &DB2_TABLES }),
        "ObjectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2424), tables: &DB2_TABLES }),
        "ObjectLiteralElementSegment" => Some(RootGrammar { grammar_id: GrammarId(2425), tables: &DB2_TABLES }),
        "ObjectLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(2429), tables: &DB2_TABLES }),
        "ObjectReferenceDelimiterGrammar" => Some(RootGrammar { grammar_id: GrammarId(2433), tables: &DB2_TABLES }),
        "ObjectReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(564), tables: &DB2_TABLES }),
        "ObjectReferenceTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(2438), tables: &DB2_TABLES }),
        "ObjectsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2450), tables: &DB2_TABLES }),
        "Octet_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2451), tables: &DB2_TABLES }),
        "OctetsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2452), tables: &DB2_TABLES }),
        "OfKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2453), tables: &DB2_TABLES }),
        "OffKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2454), tables: &DB2_TABLES }),
        "OfflineKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2455), tables: &DB2_TABLES }),
        "OffsetClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2456), tables: &DB2_TABLES }),
        "OffsetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2464), tables: &DB2_TABLES }),
        "OffsetsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2465), tables: &DB2_TABLES }),
        "OidsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2466), tables: &DB2_TABLES }),
        "OldKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2467), tables: &DB2_TABLES }),
        "Old_tableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2468), tables: &DB2_TABLES }),
        "OnKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2469), tables: &DB2_TABLES }),
        "OnlineKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2470), tables: &DB2_TABLES }),
        "OnlyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2471), tables: &DB2_TABLES }),
        "OpenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2472), tables: &DB2_TABLES }),
        "OpendatasourceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2473), tables: &DB2_TABLES }),
        "OpenqueryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2474), tables: &DB2_TABLES }),
        "OpenrowsetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2475), tables: &DB2_TABLES }),
        "OpenxmlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2476), tables: &DB2_TABLES }),
        "OperateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2477), tables: &DB2_TABLES }),
        "OperationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2478), tables: &DB2_TABLES }),
        "OperatorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2479), tables: &DB2_TABLES }),
        "OptimizationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2480), tables: &DB2_TABLES }),
        "OptimizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2481), tables: &DB2_TABLES }),
        "OptionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2482), tables: &DB2_TABLES }),
        "OptionallyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2483), tables: &DB2_TABLES }),
        "OptionsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2484), tables: &DB2_TABLES }),
        "OrKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2485), tables: &DB2_TABLES }),
        "OrOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(2486), tables: &DB2_TABLES }),
        "OrReplaceGrammar" => Some(RootGrammar { grammar_id: GrammarId(2487), tables: &DB2_TABLES }),
        "OrderByClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2490), tables: &DB2_TABLES }),
        "OrderByClauseTerminators" => Some(RootGrammar { grammar_id: GrammarId(2510), tables: &DB2_TABLES }),
        "OrderKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2523), tables: &DB2_TABLES }),
        "OrderNoOrderGrammar" => Some(RootGrammar { grammar_id: GrammarId(2524), tables: &DB2_TABLES }),
        "OrderingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2527), tables: &DB2_TABLES }),
        "OrdinalityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2528), tables: &DB2_TABLES }),
        "OrganizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2529), tables: &DB2_TABLES }),
        "OthersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2530), tables: &DB2_TABLES }),
        "OutKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2531), tables: &DB2_TABLES }),
        "OuterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2532), tables: &DB2_TABLES }),
        "OutfileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2533), tables: &DB2_TABLES }),
        "OutputKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2534), tables: &DB2_TABLES }),
        "OverClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2535), tables: &DB2_TABLES }),
        "OverKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2542), tables: &DB2_TABLES }),
        "OverlapsClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2543), tables: &DB2_TABLES }),
        "OverlapsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2552), tables: &DB2_TABLES }),
        "OverlayKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2553), tables: &DB2_TABLES }),
        "OverridingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2554), tables: &DB2_TABLES }),
        "OverwriteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2555), tables: &DB2_TABLES }),
        "OwnerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2556), tables: &DB2_TABLES }),
        "OwnershipKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2557), tables: &DB2_TABLES }),
        "Pack_keysKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2558), tables: &DB2_TABLES }),
        "PackageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2559), tables: &DB2_TABLES }),
        "PadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2560), tables: &DB2_TABLES }),
        "PaddedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2561), tables: &DB2_TABLES }),
        "PageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2562), tables: &DB2_TABLES }),
        "PagesizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2563), tables: &DB2_TABLES }),
        "ParameterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2564), tables: &DB2_TABLES }),
        "ParameterNameSegment" => Some(RootGrammar { grammar_id: GrammarId(2565), tables: &DB2_TABLES }),
        "ParameterSegment" => Some(RootGrammar { grammar_id: GrammarId(2566), tables: &DB2_TABLES }),
        "Parameter_modeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2567), tables: &DB2_TABLES }),
        "Parameter_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2568), tables: &DB2_TABLES }),
        "Parameter_ordinal_positionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2569), tables: &DB2_TABLES }),
        "Parameter_specific_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2570), tables: &DB2_TABLES }),
        "Parameter_specific_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2571), tables: &DB2_TABLES }),
        "Parameter_specific_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2572), tables: &DB2_TABLES }),
        "ParametersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2573), tables: &DB2_TABLES }),
        "PartKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2574), tables: &DB2_TABLES }),
        "PartialKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2575), tables: &DB2_TABLES }),
        "PartitionClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2576), tables: &DB2_TABLES }),
        "PartitionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2583), tables: &DB2_TABLES }),
        "PartitionedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2584), tables: &DB2_TABLES }),
        "PartitioningKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2585), tables: &DB2_TABLES }),
        "PartitionsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2586), tables: &DB2_TABLES }),
        "PascalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2587), tables: &DB2_TABLES }),
        "PasswordKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2588), tables: &DB2_TABLES }),
        "PathKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2589), tables: &DB2_TABLES }),
        "PathSegment" => Some(RootGrammar { grammar_id: GrammarId(2590), tables: &DB2_TABLES }),
        "PatternMatchingGrammar" => Some(RootGrammar { grammar_id: GrammarId(2597), tables: &DB2_TABLES }),
        "PctfreeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2598), tables: &DB2_TABLES }),
        "PercentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2599), tables: &DB2_TABLES }),
        "Percent_rankKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2600), tables: &DB2_TABLES }),
        "Percentile_contKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2601), tables: &DB2_TABLES }),
        "Percentile_discKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2602), tables: &DB2_TABLES }),
        "PiecesizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2603), tables: &DB2_TABLES }),
        "PipeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2604), tables: &DB2_TABLES }),
        "PipeSegment" => Some(RootGrammar { grammar_id: GrammarId(2605), tables: &DB2_TABLES }),
        "PlacingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2606), tables: &DB2_TABLES }),
        "PlanKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2607), tables: &DB2_TABLES }),
        "PliKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2608), tables: &DB2_TABLES }),
        "PlusSegment" => Some(RootGrammar { grammar_id: GrammarId(2609), tables: &DB2_TABLES }),
        "PolicyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2610), tables: &DB2_TABLES }),
        "PositionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2611), tables: &DB2_TABLES }),
        "PositiveSegment" => Some(RootGrammar { grammar_id: GrammarId(2612), tables: &DB2_TABLES }),
        "PostFunctionGrammar" => Some(RootGrammar { grammar_id: GrammarId(2613), tables: &DB2_TABLES }),
        "PostTableExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(2616), tables: &DB2_TABLES }),
        "PostfixKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2617), tables: &DB2_TABLES }),
        "PowerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2618), tables: &DB2_TABLES }),
        "PreTableFunctionKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(2619), tables: &DB2_TABLES }),
        "PrecedingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2621), tables: &DB2_TABLES }),
        "PrecisionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2622), tables: &DB2_TABLES }),
        "PrefixKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2623), tables: &DB2_TABLES }),
        "PreorderKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2624), tables: &DB2_TABLES }),
        "PrepareKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2625), tables: &DB2_TABLES }),
        "PreparedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2626), tables: &DB2_TABLES }),
        "PreserveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2627), tables: &DB2_TABLES }),
        "PrevvalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2628), tables: &DB2_TABLES }),
        "PrimaryKeyGrammar" => Some(RootGrammar { grammar_id: GrammarId(2629), tables: &DB2_TABLES }),
        "PrimaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2632), tables: &DB2_TABLES }),
        "PrintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2633), tables: &DB2_TABLES }),
        "PriorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2634), tables: &DB2_TABLES }),
        "PriqtyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2635), tables: &DB2_TABLES }),
        "PrivilegesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2636), tables: &DB2_TABLES }),
        "ProcKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2637), tables: &DB2_TABLES }),
        "ProceduralKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2638), tables: &DB2_TABLES }),
        "ProcedureKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2639), tables: &DB2_TABLES }),
        "ProceduresKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2640), tables: &DB2_TABLES }),
        "ProcessKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2641), tables: &DB2_TABLES }),
        "ProcesslistKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2642), tables: &DB2_TABLES }),
        "ProfileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2643), tables: &DB2_TABLES }),
        "ProgramKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2644), tables: &DB2_TABLES }),
        "PsidKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2645), tables: &DB2_TABLES }),
        "PublicKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2646), tables: &DB2_TABLES }),
        "PurgeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2647), tables: &DB2_TABLES }),
        "QualifiedNumericLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(2648), tables: &DB2_TABLES }),
        "QualifyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2651), tables: &DB2_TABLES }),
        "QuantifiedComparisonOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(2652), tables: &DB2_TABLES }),
        "QuarterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2660), tables: &DB2_TABLES }),
        "QueryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2661), tables: &DB2_TABLES }),
        "QuerynoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2662), tables: &DB2_TABLES }),
        "QuoteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2663), tables: &DB2_TABLES }),
        "QuotedIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(2664), tables: &DB2_TABLES }),
        "QuotedLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(2665), tables: &DB2_TABLES }),
        "Raid0KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2666), tables: &DB2_TABLES }),
        "RaiserrorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2667), tables: &DB2_TABLES }),
        "RandomKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2668), tables: &DB2_TABLES }),
        "RangeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2669), tables: &DB2_TABLES }),
        "RankKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2670), tables: &DB2_TABLES }),
        "RawEqualsSegment" => Some(RootGrammar { grammar_id: GrammarId(2671), tables: &DB2_TABLES }),
        "RawGreaterThanSegment" => Some(RootGrammar { grammar_id: GrammarId(2672), tables: &DB2_TABLES }),
        "RawKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2673), tables: &DB2_TABLES }),
        "RawLessThanSegment" => Some(RootGrammar { grammar_id: GrammarId(2674), tables: &DB2_TABLES }),
        "RawNotSegment" => Some(RootGrammar { grammar_id: GrammarId(2675), tables: &DB2_TABLES }),
        "ReadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2676), tables: &DB2_TABLES }),
        "ReadOnlyClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2677), tables: &DB2_TABLES }),
        "ReadsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2686), tables: &DB2_TABLES }),
        "ReadtextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2687), tables: &DB2_TABLES }),
        "RealKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2688), tables: &DB2_TABLES }),
        "RecheckKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2689), tables: &DB2_TABLES }),
        "ReconfigureKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2690), tables: &DB2_TABLES }),
        "RecoveryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2691), tables: &DB2_TABLES }),
        "RecursiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2692), tables: &DB2_TABLES }),
        "RefKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2693), tables: &DB2_TABLES }),
        "ReferenceDefinitionGrammar" => Some(RootGrammar { grammar_id: GrammarId(2694), tables: &DB2_TABLES }),
        "ReferenceMatchGrammar" => Some(RootGrammar { grammar_id: GrammarId(2708), tables: &DB2_TABLES }),
        "Reference_usageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2714), tables: &DB2_TABLES }),
        "ReferencesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2715), tables: &DB2_TABLES }),
        "ReferencingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2716), tables: &DB2_TABLES }),
        "ReferentialActionGrammar" => Some(RootGrammar { grammar_id: GrammarId(2717), tables: &DB2_TABLES }),
        "RefreshKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2729), tables: &DB2_TABLES }),
        "RegexpKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2730), tables: &DB2_TABLES }),
        "Regr_avgxKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2731), tables: &DB2_TABLES }),
        "Regr_avgyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2732), tables: &DB2_TABLES }),
        "Regr_countKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2733), tables: &DB2_TABLES }),
        "Regr_interceptKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2734), tables: &DB2_TABLES }),
        "Regr_r2KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2735), tables: &DB2_TABLES }),
        "Regr_slopeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2736), tables: &DB2_TABLES }),
        "Regr_sxxKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2737), tables: &DB2_TABLES }),
        "Regr_sxyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2738), tables: &DB2_TABLES }),
        "Regr_syyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2739), tables: &DB2_TABLES }),
        "ReindexKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2740), tables: &DB2_TABLES }),
        "RejectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2741), tables: &DB2_TABLES }),
        "RelativeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2742), tables: &DB2_TABLES }),
        "ReleaseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2743), tables: &DB2_TABLES }),
        "ReloadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2744), tables: &DB2_TABLES }),
        "RenameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2745), tables: &DB2_TABLES }),
        "RepeatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2746), tables: &DB2_TABLES }),
        "RepeatableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2747), tables: &DB2_TABLES }),
        "ReplaceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2748), tables: &DB2_TABLES }),
        "ReplicationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2749), tables: &DB2_TABLES }),
        "RequireKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2750), tables: &DB2_TABLES }),
        "ResetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2751), tables: &DB2_TABLES }),
        "ResignalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2752), tables: &DB2_TABLES }),
        "ResourceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2753), tables: &DB2_TABLES }),
        "RespectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2754), tables: &DB2_TABLES }),
        "RestartKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2755), tables: &DB2_TABLES }),
        "RestoreKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2756), tables: &DB2_TABLES }),
        "RestrictKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2757), tables: &DB2_TABLES }),
        "ResultKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2758), tables: &DB2_TABLES }),
        "Result_set_locatorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2759), tables: &DB2_TABLES }),
        "ReturnKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2760), tables: &DB2_TABLES }),
        "Returned_cardinalityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2761), tables: &DB2_TABLES }),
        "Returned_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2762), tables: &DB2_TABLES }),
        "Returned_octet_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2763), tables: &DB2_TABLES }),
        "Returned_sqlstateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2764), tables: &DB2_TABLES }),
        "ReturnsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2765), tables: &DB2_TABLES }),
        "ReverseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2766), tables: &DB2_TABLES }),
        "RevokeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2767), tables: &DB2_TABLES }),
        "RevokeStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2768), tables: &DB2_TABLES }),
        "RightArrowSegment" => Some(RootGrammar { grammar_id: GrammarId(2791), tables: &DB2_TABLES }),
        "RightKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2792), tables: &DB2_TABLES }),
        "RlikeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2793), tables: &DB2_TABLES }),
        "RoleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2794), tables: &DB2_TABLES }),
        "RoleReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(2795), tables: &DB2_TABLES }),
        "RolesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2796), tables: &DB2_TABLES }),
        "RollbackKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2797), tables: &DB2_TABLES }),
        "RolloutKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2798), tables: &DB2_TABLES }),
        "RollupFunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(2799), tables: &DB2_TABLES }),
        "RollupKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2800), tables: &DB2_TABLES }),
        "Round_ceilingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2801), tables: &DB2_TABLES }),
        "Round_downKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2802), tables: &DB2_TABLES }),
        "Round_floorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2803), tables: &DB2_TABLES }),
        "Round_half_downKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2804), tables: &DB2_TABLES }),
        "Round_half_evenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2805), tables: &DB2_TABLES }),
        "Round_half_upKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2806), tables: &DB2_TABLES }),
        "Round_upKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2807), tables: &DB2_TABLES }),
        "RoundingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2808), tables: &DB2_TABLES }),
        "RoutineKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2809), tables: &DB2_TABLES }),
        "Routine_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2810), tables: &DB2_TABLES }),
        "Routine_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2811), tables: &DB2_TABLES }),
        "Routine_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2812), tables: &DB2_TABLES }),
        "RoutinesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2813), tables: &DB2_TABLES }),
        "RowKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2814), tables: &DB2_TABLES }),
        "RowMovementClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2815), tables: &DB2_TABLES }),
        "Row_countKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2820), tables: &DB2_TABLES }),
        "Row_numberKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2821), tables: &DB2_TABLES }),
        "RowcountKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2822), tables: &DB2_TABLES }),
        "RowguidcolKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2823), tables: &DB2_TABLES }),
        "RowidKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2824), tables: &DB2_TABLES }),
        "RownumKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2825), tables: &DB2_TABLES }),
        "RownumberKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2826), tables: &DB2_TABLES }),
        "RowsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2827), tables: &DB2_TABLES }),
        "RowsetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2828), tables: &DB2_TABLES }),
        "RrKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2829), tables: &DB2_TABLES }),
        "RrnKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2830), tables: &DB2_TABLES }),
        "RsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2831), tables: &DB2_TABLES }),
        "RuleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2832), tables: &DB2_TABLES }),
        "RunKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2833), tables: &DB2_TABLES }),
        "SampledKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2834), tables: &DB2_TABLES }),
        "SamplingExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(2835), tables: &DB2_TABLES }),
        "SaveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2847), tables: &DB2_TABLES }),
        "SavepointKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2848), tables: &DB2_TABLES }),
        "ScaleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2849), tables: &DB2_TABLES }),
        "ScansKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2850), tables: &DB2_TABLES }),
        "SchemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2851), tables: &DB2_TABLES }),
        "SchemaReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(564), tables: &DB2_TABLES }),
        "Schema_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2852), tables: &DB2_TABLES }),
        "SchemasKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2853), tables: &DB2_TABLES }),
        "ScopeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2854), tables: &DB2_TABLES }),
        "Scope_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2855), tables: &DB2_TABLES }),
        "Scope_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2856), tables: &DB2_TABLES }),
        "Scope_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2857), tables: &DB2_TABLES }),
        "ScratchpadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2858), tables: &DB2_TABLES }),
        "ScrollKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2859), tables: &DB2_TABLES }),
        "SearchKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2860), tables: &DB2_TABLES }),
        "SecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2861), tables: &DB2_TABLES }),
        "Second_microsecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2862), tables: &DB2_TABLES }),
        "SecondsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2863), tables: &DB2_TABLES }),
        "SecqtyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2864), tables: &DB2_TABLES }),
        "SectionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2865), tables: &DB2_TABLES }),
        "SecurityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2866), tables: &DB2_TABLES }),
        "SelectClauseElementSegment" => Some(RootGrammar { grammar_id: GrammarId(2867), tables: &DB2_TABLES }),
        "SelectClauseModifierSegment" => Some(RootGrammar { grammar_id: GrammarId(2872), tables: &DB2_TABLES }),
        "SelectClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2875), tables: &DB2_TABLES }),
        "SelectClauseTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(2881), tables: &DB2_TABLES }),
        "SelectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2896), tables: &DB2_TABLES }),
        "SelectStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2897), tables: &DB2_TABLES }),
        "SelectableGrammar" => Some(RootGrammar { grammar_id: GrammarId(2917), tables: &DB2_TABLES }),
        "SelfKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2927), tables: &DB2_TABLES }),
        "SemicolonSegment" => Some(RootGrammar { grammar_id: GrammarId(2928), tables: &DB2_TABLES }),
        "SensitiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2929), tables: &DB2_TABLES }),
        "SeparatorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2930), tables: &DB2_TABLES }),
        "SequenceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2931), tables: &DB2_TABLES }),
        "SequenceMaxValueGrammar" => Some(RootGrammar { grammar_id: GrammarId(2932), tables: &DB2_TABLES }),
        "SequenceMinValueGrammar" => Some(RootGrammar { grammar_id: GrammarId(2939), tables: &DB2_TABLES }),
        "SequenceReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(564), tables: &DB2_TABLES }),
        "SequencesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2946), tables: &DB2_TABLES }),
        "SerializableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2947), tables: &DB2_TABLES }),
        "ServerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2948), tables: &DB2_TABLES }),
        "Server_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2949), tables: &DB2_TABLES }),
        "SessionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2950), tables: &DB2_TABLES }),
        "Session_userKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2951), tables: &DB2_TABLES }),
        "SetClauseListSegment" => Some(RootGrammar { grammar_id: GrammarId(2952), tables: &DB2_TABLES }),
        "SetClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2956), tables: &DB2_TABLES }),
        "SetExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(2967), tables: &DB2_TABLES }),
        "SetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2980), tables: &DB2_TABLES }),
        "SetOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(2981), tables: &DB2_TABLES }),
        "SetSchemaStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2989), tables: &DB2_TABLES }),
        "SetofKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2994), tables: &DB2_TABLES }),
        "SetsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2995), tables: &DB2_TABLES }),
        "SetuserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2996), tables: &DB2_TABLES }),
        "ShareKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2997), tables: &DB2_TABLES }),
        "SharesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2998), tables: &DB2_TABLES }),
        "ShorthandCastSegment" => Some(RootGrammar { grammar_id: GrammarId(2999), tables: &DB2_TABLES }),
        "ShowKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3008), tables: &DB2_TABLES }),
        "ShutdownKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3009), tables: &DB2_TABLES }),
        "SignalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3010), tables: &DB2_TABLES }),
        "SignedSegmentGrammar" => Some(RootGrammar { grammar_id: GrammarId(3011), tables: &DB2_TABLES }),
        "SimilarKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3014), tables: &DB2_TABLES }),
        "SimpleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3015), tables: &DB2_TABLES }),
        "SingleIdentifierGrammar" => Some(RootGrammar { grammar_id: GrammarId(3016), tables: &DB2_TABLES }),
        "SingleIdentifierListSegment" => Some(RootGrammar { grammar_id: GrammarId(3020), tables: &DB2_TABLES }),
        "SingleIdentifierWildcardSegment" => Some(RootGrammar { grammar_id: GrammarId(3022), tables: &DB2_TABLES }),
        "SingleQuotedIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(3026), tables: &DB2_TABLES }),
        "SizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3027), tables: &DB2_TABLES }),
        "SizedArrayTypeSegment" => Some(RootGrammar { grammar_id: GrammarId(3028), tables: &DB2_TABLES }),
        "SlashSegment" => Some(RootGrammar { grammar_id: GrammarId(3031), tables: &DB2_TABLES }),
        "SliceSegment" => Some(RootGrammar { grammar_id: GrammarId(3032), tables: &DB2_TABLES }),
        "SmallintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3033), tables: &DB2_TABLES }),
        "SnanKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3034), tables: &DB2_TABLES }),
        "SnapshotKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3035), tables: &DB2_TABLES }),
        "SomeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3036), tables: &DB2_TABLES }),
        "SonameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3037), tables: &DB2_TABLES }),
        "SourceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3038), tables: &DB2_TABLES }),
        "SpaceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3039), tables: &DB2_TABLES }),
        "SpatialKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3040), tables: &DB2_TABLES }),
        "SpecialRegisterGrammar" => Some(RootGrammar { grammar_id: GrammarId(3041), tables: &DB2_TABLES }),
        "SpecificKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3128), tables: &DB2_TABLES }),
        "Specific_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3129), tables: &DB2_TABLES }),
        "SpecificationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3130), tables: &DB2_TABLES }),
        "SpecifictypeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3131), tables: &DB2_TABLES }),
        "SplitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3132), tables: &DB2_TABLES }),
        "SqlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3133), tables: &DB2_TABLES }),
        "Sql_big_resultKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3134), tables: &DB2_TABLES }),
        "Sql_big_selectsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3135), tables: &DB2_TABLES }),
        "Sql_big_tablesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3136), tables: &DB2_TABLES }),
        "Sql_calc_found_rowsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3137), tables: &DB2_TABLES }),
        "Sql_ccflagsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3138), tables: &DB2_TABLES }),
        "Sql_log_offKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3139), tables: &DB2_TABLES }),
        "Sql_log_updateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3140), tables: &DB2_TABLES }),
        "Sql_low_priority_updatesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3141), tables: &DB2_TABLES }),
        "Sql_select_limitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3142), tables: &DB2_TABLES }),
        "Sql_small_resultKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3143), tables: &DB2_TABLES }),
        "Sql_warningsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3144), tables: &DB2_TABLES }),
        "SqlcaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3145), tables: &DB2_TABLES }),
        "SqlcodeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3146), tables: &DB2_TABLES }),
        "SqlerrorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3147), tables: &DB2_TABLES }),
        "SqlexceptionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3148), tables: &DB2_TABLES }),
        "SqlidKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3149), tables: &DB2_TABLES }),
        "SqlstateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3150), tables: &DB2_TABLES }),
        "SqlwarningKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3151), tables: &DB2_TABLES }),
        "SqrtKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3152), tables: &DB2_TABLES }),
        "SslKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3153), tables: &DB2_TABLES }),
        "StableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3154), tables: &DB2_TABLES }),
        "StackedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3155), tables: &DB2_TABLES }),
        "StageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3156), tables: &DB2_TABLES }),
        "StagesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3157), tables: &DB2_TABLES }),
        "StandardKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3158), tables: &DB2_TABLES }),
        "StarSegment" => Some(RootGrammar { grammar_id: GrammarId(3159), tables: &DB2_TABLES }),
        "StartBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(223), tables: &DB2_TABLES }),
        "StartCurlyBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(2432), tables: &DB2_TABLES }),
        "StartKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3160), tables: &DB2_TABLES }),
        "StartSquareBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(322), tables: &DB2_TABLES }),
        "StartingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3161), tables: &DB2_TABLES }),
        "StartsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3162), tables: &DB2_TABLES }),
        "StateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3163), tables: &DB2_TABLES }),
        "StatementKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3164), tables: &DB2_TABLES }),
        "StatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3165), tables: &DB2_TABLES }),
        "StaticKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3209), tables: &DB2_TABLES }),
        "StatisticsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3210), tables: &DB2_TABLES }),
        "StatmentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3211), tables: &DB2_TABLES }),
        "StayKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3212), tables: &DB2_TABLES }),
        "Stddev_popKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3213), tables: &DB2_TABLES }),
        "Stddev_sampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3214), tables: &DB2_TABLES }),
        "StdinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3215), tables: &DB2_TABLES }),
        "StdoutKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3216), tables: &DB2_TABLES }),
        "StogroupKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3217), tables: &DB2_TABLES }),
        "StorageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3218), tables: &DB2_TABLES }),
        "StoresKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3219), tables: &DB2_TABLES }),
        "Straight_joinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3220), tables: &DB2_TABLES }),
        "StreamKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3221), tables: &DB2_TABLES }),
        "StreamsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3222), tables: &DB2_TABLES }),
        "StrictKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3223), tables: &DB2_TABLES }),
        "StringBinaryOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(3224), tables: &DB2_TABLES }),
        "StringKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3226), tables: &DB2_TABLES }),
        "StructLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(3227), tables: &DB2_TABLES }),
        "StructTypeSegment" => Some(RootGrammar { grammar_id: GrammarId(3232), tables: &DB2_TABLES }),
        "StructureKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3233), tables: &DB2_TABLES }),
        "StyleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3234), tables: &DB2_TABLES }),
        "Subclass_originKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3235), tables: &DB2_TABLES }),
        "SublistKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3236), tables: &DB2_TABLES }),
        "SubmultisetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3237), tables: &DB2_TABLES }),
        "SubstringKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3238), tables: &DB2_TABLES }),
        "SuccessfulKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3239), tables: &DB2_TABLES }),
        "SumKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3240), tables: &DB2_TABLES }),
        "SummaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3241), tables: &DB2_TABLES }),
        "SuperuserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3242), tables: &DB2_TABLES }),
        "SymbolSegment" => Some(RootGrammar { grammar_id: GrammarId(3243), tables: &DB2_TABLES }),
        "SymmetricKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3244), tables: &DB2_TABLES }),
        "SynonymKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3245), tables: &DB2_TABLES }),
        "SysdateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3246), tables: &DB2_TABLES }),
        "SysfunKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3247), tables: &DB2_TABLES }),
        "SysibmKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3248), tables: &DB2_TABLES }),
        "SysidKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3249), tables: &DB2_TABLES }),
        "SysprocKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3250), tables: &DB2_TABLES }),
        "SystemKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3251), tables: &DB2_TABLES }),
        "System_timeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3252), tables: &DB2_TABLES }),
        "System_userKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3253), tables: &DB2_TABLES }),
        "TableConstraintSegment" => Some(RootGrammar { grammar_id: GrammarId(3254), tables: &DB2_TABLES }),
        "TableEndClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3269), tables: &DB2_TABLES }),
        "TableExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(3270), tables: &DB2_TABLES }),
        "TableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3279), tables: &DB2_TABLES }),
        "TableReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(564), tables: &DB2_TABLES }),
        "Table_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3280), tables: &DB2_TABLES }),
        "TablesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3281), tables: &DB2_TABLES }),
        "TablesampleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3282), tables: &DB2_TABLES }),
        "TablespaceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3283), tables: &DB2_TABLES }),
        "TablespaceReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(564), tables: &DB2_TABLES }),
        "TagReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(564), tables: &DB2_TABLES }),
        "Tail_Recurse_Expression_A_Grammar" => Some(RootGrammar { grammar_id: GrammarId(3284), tables: &DB2_TABLES }),
        "Tail_Recurse_Expression_B_Grammar" => Some(RootGrammar { grammar_id: GrammarId(3289), tables: &DB2_TABLES }),
        "TaskKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3293), tables: &DB2_TABLES }),
        "TasksKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3294), tables: &DB2_TABLES }),
        "TempKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3295), tables: &DB2_TABLES }),
        "TemplateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3296), tables: &DB2_TABLES }),
        "TemporalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3297), tables: &DB2_TABLES }),
        "TemporalQuerySegment" => Some(RootGrammar { grammar_id: GrammarId(3298), tables: &DB2_TABLES }),
        "TemporaryGrammar" => Some(RootGrammar { grammar_id: GrammarId(3299), tables: &DB2_TABLES }),
        "TemporaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3302), tables: &DB2_TABLES }),
        "TemporaryTransientGrammar" => Some(RootGrammar { grammar_id: GrammarId(3303), tables: &DB2_TABLES }),
        "TerminateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3306), tables: &DB2_TABLES }),
        "TerminatedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3307), tables: &DB2_TABLES }),
        "TextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3308), tables: &DB2_TABLES }),
        "TextsizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3309), tables: &DB2_TABLES }),
        "ThanKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3310), tables: &DB2_TABLES }),
        "ThenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3311), tables: &DB2_TABLES }),
        "TiesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3312), tables: &DB2_TABLES }),
        "TildeSegment" => Some(RootGrammar { grammar_id: GrammarId(3313), tables: &DB2_TABLES }),
        "TimeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3314), tables: &DB2_TABLES }),
        "TimeWithTZGrammar" => Some(RootGrammar { grammar_id: GrammarId(3315), tables: &DB2_TABLES }),
        "TimeZoneGrammar" => Some(RootGrammar { grammar_id: GrammarId(3326), tables: &DB2_TABLES }),
        "TimeoutKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3332), tables: &DB2_TABLES }),
        "TimestampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3333), tables: &DB2_TABLES }),
        "TimezoneKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3334), tables: &DB2_TABLES }),
        "Timezone_hourKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3335), tables: &DB2_TABLES }),
        "Timezone_minuteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3336), tables: &DB2_TABLES }),
        "TinyblobKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3337), tables: &DB2_TABLES }),
        "TinyintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3338), tables: &DB2_TABLES }),
        "TinytextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3339), tables: &DB2_TABLES }),
        "ToKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3340), tables: &DB2_TABLES }),
        "ToastKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3341), tables: &DB2_TABLES }),
        "TopKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3342), tables: &DB2_TABLES }),
        "Top_level_countKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3343), tables: &DB2_TABLES }),
        "TrailingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3344), tables: &DB2_TABLES }),
        "TranKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3345), tables: &DB2_TABLES }),
        "TransactionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3346), tables: &DB2_TABLES }),
        "TransactionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3347), tables: &DB2_TABLES }),
        "Transaction_activeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3364), tables: &DB2_TABLES }),
        "TransactionsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3365), tables: &DB2_TABLES }),
        "Transactions_committedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3366), tables: &DB2_TABLES }),
        "Transactions_rolled_backKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3367), tables: &DB2_TABLES }),
        "TransformKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3368), tables: &DB2_TABLES }),
        "TransformsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3369), tables: &DB2_TABLES }),
        "TransientKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3370), tables: &DB2_TABLES }),
        "TranslateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3371), tables: &DB2_TABLES }),
        "TranslationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3372), tables: &DB2_TABLES }),
        "TreatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3373), tables: &DB2_TABLES }),
        "TriggerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3374), tables: &DB2_TABLES }),
        "TriggerReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(564), tables: &DB2_TABLES }),
        "Trigger_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3375), tables: &DB2_TABLES }),
        "Trigger_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3376), tables: &DB2_TABLES }),
        "Trigger_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3377), tables: &DB2_TABLES }),
        "TrimKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3378), tables: &DB2_TABLES }),
        "TrimParametersGrammar" => Some(RootGrammar { grammar_id: GrammarId(3379), tables: &DB2_TABLES }),
        "TrueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3383), tables: &DB2_TABLES }),
        "TrueSegment" => Some(RootGrammar { grammar_id: GrammarId(3384), tables: &DB2_TABLES }),
        "TruncateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3385), tables: &DB2_TABLES }),
        "TruncateStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3386), tables: &DB2_TABLES }),
        "TrustedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3390), tables: &DB2_TABLES }),
        "TsequalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3391), tables: &DB2_TABLES }),
        "TupleSegment" => Some(RootGrammar { grammar_id: GrammarId(3392), tables: &DB2_TABLES }),
        "TypeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3395), tables: &DB2_TABLES }),
        "TypedArrayLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(3396), tables: &DB2_TABLES }),
        "TypedStructLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(3399), tables: &DB2_TABLES }),
        "TypesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3402), tables: &DB2_TABLES }),
        "UescapeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3403), tables: &DB2_TABLES }),
        "UidKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3404), tables: &DB2_TABLES }),
        "UnboundedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3405), tables: &DB2_TABLES }),
        "UncommittedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3406), tables: &DB2_TABLES }),
        "UnconditionalCrossJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(3407), tables: &DB2_TABLES }),
        "UnconditionalJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(3408), tables: &DB2_TABLES }),
        "UnderKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3412), tables: &DB2_TABLES }),
        "UndoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3413), tables: &DB2_TABLES }),
        "UnencryptedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3414), tables: &DB2_TABLES }),
        "UnionGrammar" => Some(RootGrammar { grammar_id: GrammarId(3415), tables: &DB2_TABLES }),
        "UnionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3420), tables: &DB2_TABLES }),
        "UniqueKeyGrammar" => Some(RootGrammar { grammar_id: GrammarId(3421), tables: &DB2_TABLES }),
        "UniqueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3423), tables: &DB2_TABLES }),
        "UnknownKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3424), tables: &DB2_TABLES }),
        "UnknownLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(3425), tables: &DB2_TABLES }),
        "UnlistenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3426), tables: &DB2_TABLES }),
        "UnlockKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3427), tables: &DB2_TABLES }),
        "UnnamedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3428), tables: &DB2_TABLES }),
        "UnnestKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3429), tables: &DB2_TABLES }),
        "UnorderedSelectStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3430), tables: &DB2_TABLES }),
        "UnorderedSetExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(3447), tables: &DB2_TABLES }),
        "UnsampledKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3453), tables: &DB2_TABLES }),
        "UnsignedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3454), tables: &DB2_TABLES }),
        "UntilKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3455), tables: &DB2_TABLES }),
        "UpdateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3456), tables: &DB2_TABLES }),
        "UpdateStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3457), tables: &DB2_TABLES }),
        "UpdatetextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3465), tables: &DB2_TABLES }),
        "UpperKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3466), tables: &DB2_TABLES }),
        "UrKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3467), tables: &DB2_TABLES }),
        "UsageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3468), tables: &DB2_TABLES }),
        "UseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3469), tables: &DB2_TABLES }),
        "UseStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3470), tables: &DB2_TABLES }),
        "Use_any_roleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3473), tables: &DB2_TABLES }),
        "UserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3474), tables: &DB2_TABLES }),
        "UserReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(3475), tables: &DB2_TABLES }),
        "User_defined_type_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3476), tables: &DB2_TABLES }),
        "User_defined_type_codeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3477), tables: &DB2_TABLES }),
        "User_defined_type_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3478), tables: &DB2_TABLES }),
        "User_defined_type_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3479), tables: &DB2_TABLES }),
        "UsersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3480), tables: &DB2_TABLES }),
        "UsingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3481), tables: &DB2_TABLES }),
        "Utc_dateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3482), tables: &DB2_TABLES }),
        "Utc_timeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3483), tables: &DB2_TABLES }),
        "Utc_timestampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3484), tables: &DB2_TABLES }),
        "VacuumKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3485), tables: &DB2_TABLES }),
        "ValidKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3486), tables: &DB2_TABLES }),
        "ValidateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3487), tables: &DB2_TABLES }),
        "ValidatorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3488), tables: &DB2_TABLES }),
        "ValidprocKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3489), tables: &DB2_TABLES }),
        "ValueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3490), tables: &DB2_TABLES }),
        "ValuesClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3491), tables: &DB2_TABLES }),
        "ValuesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3504), tables: &DB2_TABLES }),
        "Var_popKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3505), tables: &DB2_TABLES }),
        "Var_sampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3506), tables: &DB2_TABLES }),
        "VarbinaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3507), tables: &DB2_TABLES }),
        "Varchar2KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3508), tables: &DB2_TABLES }),
        "VarcharKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3509), tables: &DB2_TABLES }),
        "VarcharacterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3510), tables: &DB2_TABLES }),
        "VariableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3511), tables: &DB2_TABLES }),
        "VariablesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3512), tables: &DB2_TABLES }),
        "VariantKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3513), tables: &DB2_TABLES }),
        "VaryingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3514), tables: &DB2_TABLES }),
        "VcatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3515), tables: &DB2_TABLES }),
        "VerboseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3516), tables: &DB2_TABLES }),
        "VersionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3517), tables: &DB2_TABLES }),
        "ViewCheckOptionClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3518), tables: &DB2_TABLES }),
        "ViewKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3525), tables: &DB2_TABLES }),
        "ViewsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3526), tables: &DB2_TABLES }),
        "VolatileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3527), tables: &DB2_TABLES }),
        "VolumesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3528), tables: &DB2_TABLES }),
        "WaitforKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3529), tables: &DB2_TABLES }),
        "WarehouseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3530), tables: &DB2_TABLES }),
        "WarehousesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3531), tables: &DB2_TABLES }),
        "WeekKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3532), tables: &DB2_TABLES }),
        "WeekdayKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3533), tables: &DB2_TABLES }),
        "WhenClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3534), tables: &DB2_TABLES }),
        "WhenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3544), tables: &DB2_TABLES }),
        "WheneverKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3545), tables: &DB2_TABLES }),
        "WhereClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3546), tables: &DB2_TABLES }),
        "WhereClauseTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(3551), tables: &DB2_TABLES }),
        "WhereKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3569), tables: &DB2_TABLES }),
        "WhileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3570), tables: &DB2_TABLES }),
        "WhitespaceSegment" => Some(RootGrammar { grammar_id: GrammarId(3571), tables: &DB2_TABLES }),
        "Width_bucketKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3572), tables: &DB2_TABLES }),
        "WildcardExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(3573), tables: &DB2_TABLES }),
        "WildcardIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(3575), tables: &DB2_TABLES }),
        "WindowKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3585), tables: &DB2_TABLES }),
        "WindowSpecificationSegment" => Some(RootGrammar { grammar_id: GrammarId(3586), tables: &DB2_TABLES }),
        "WithCompoundNonSelectStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3594), tables: &DB2_TABLES }),
        "WithCompoundStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3603), tables: &DB2_TABLES }),
        "WithDataClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3612), tables: &DB2_TABLES }),
        "WithFillSegment" => Some(RootGrammar { grammar_id: GrammarId(3617), tables: &DB2_TABLES }),
        "WithKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3618), tables: &DB2_TABLES }),
        "WithNoSchemaBindingClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3619), tables: &DB2_TABLES }),
        "WithinGroupClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3624), tables: &DB2_TABLES }),
        "WithinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3629), tables: &DB2_TABLES }),
        "WithoutKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3630), tables: &DB2_TABLES }),
        "WlmKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3631), tables: &DB2_TABLES }),
        "WordSegment" => Some(RootGrammar { grammar_id: GrammarId(3632), tables: &DB2_TABLES }),
        "WorkKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3633), tables: &DB2_TABLES }),
        "WrapperKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3634), tables: &DB2_TABLES }),
        "WriteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3635), tables: &DB2_TABLES }),
        "WritetextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3636), tables: &DB2_TABLES }),
        "X509KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3637), tables: &DB2_TABLES }),
        "XmlIndexSpecificationGrammar" => Some(RootGrammar { grammar_id: GrammarId(3638), tables: &DB2_TABLES }),
        "XmlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3647), tables: &DB2_TABLES }),
        "XmlTypeClauseGrammar" => Some(RootGrammar { grammar_id: GrammarId(3648), tables: &DB2_TABLES }),
        "XmlelementKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3658), tables: &DB2_TABLES }),
        "XmlexistsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3659), tables: &DB2_TABLES }),
        "XmlnamespacesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3660), tables: &DB2_TABLES }),
        "XmlparseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3661), tables: &DB2_TABLES }),
        "XmlpatternKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3662), tables: &DB2_TABLES }),
        "XorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3663), tables: &DB2_TABLES }),
        "YamlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3664), tables: &DB2_TABLES }),
        "YearKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3665), tables: &DB2_TABLES }),
        "Year_monthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3666), tables: &DB2_TABLES }),
        "YearsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3667), tables: &DB2_TABLES }),
        "YesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3668), tables: &DB2_TABLES }),
        "ZerofillKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3669), tables: &DB2_TABLES }),
        "ZoneKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3670), tables: &DB2_TABLES }),
        _ => None,
    }
}

pub fn get_db2_segment_type(name: &str) -> Option<&'static str> {
    match name {
        "AccessObjectSegment" => Some("access_object"),
        "AccessPermissionSegment" => Some("access_permission"),
        "AccessPermissionsSegment" => Some("access_permissions"),
        "AccessSchemaObjectSegment" => Some("access_schema_object"),
        "AccessSchemaPluralObjectSegment" => Some("access_schema_plural_object"),
        "AccessStatementSegment" => Some("access_statement"),
        "AccessTargetSegment" => Some("access_target"),
        "AggregateOrderByClause" => Some("aggregate_order_by"),
        "AliasExpressionSegment" => Some("alias_expression"),
        "AlterSequenceOptionsSegment" => Some("alter_sequence_options_segment"),
        "AlterSequenceStatementSegment" => Some("alter_sequence_statement"),
        "AlterTableStatementSegment" => Some("alter_table_statement"),
        "ArrayAccessorSegment" => Some("array_accessor"),
        "ArrayExpressionSegment" => Some("array_expression"),
        "ArrayLiteralSegment" => Some("array_literal"),
        "ArrayTypeSegment" => Some("array_type"),
        "AsAliasOperatorSegment" => Some("alias_operator"),
        "BareFunctionSegment" => Some("bare_function"),
        "BaseFileSegment" => Some("file"),
        "BaseSegment" => Some("base"),
        "BinaryOperatorSegment" => Some("binary_operator"),
        "BitwiseAndSegment" => Some("binary_operator"),
        "BitwiseLShiftSegment" => Some("binary_operator"),
        "BitwiseOrSegment" => Some("binary_operator"),
        "BitwiseRShiftSegment" => Some("binary_operator"),
        "BracketedArguments" => Some("bracketed_arguments"),
        "BracketedSegment" => Some("bracketed"),
        "CTEColumnList" => Some("cte_column_list"),
        "CTEDefinitionSegment" => Some("common_table_expression"),
        "CallStoredProcedureSegment" => Some("call_segment"),
        "CaseExpressionSegment" => Some("case_expression"),
        "CodeSegment" => Some("raw"),
        "CollationReferenceSegment" => Some("collation_reference"),
        "ColumnConstraintSegment" => Some("column_constraint_segment"),
        "ColumnDefinitionSegment" => Some("column_definition"),
        "ColumnReferenceSegment" => Some("column_reference"),
        "ColumnsExpressionFunctionContentsSegment" => Some("columns_expression"),
        "ColumnsExpressionFunctionNameSegment" => Some("function_name"),
        "CommentClauseSegment" => Some("comment_clause"),
        "CommentOnStatementSegment" => Some("comment_clause"),
        "CommentSegment" => Some("comment"),
        "ComparisonOperatorSegment" => Some("comparison_operator"),
        "CompositeBinaryOperatorSegment" => Some("binary_operator"),
        "CompositeComparisonOperatorSegment" => Some("comparison_operator"),
        "ConcatSegment" => Some("binary_operator"),
        "CopyOptionsSegment" => Some("copy_options"),
        "CreateCastStatementSegment" => Some("create_cast_statement"),
        "CreateDatabaseStatementSegment" => Some("create_database_statement"),
        "CreateFunctionStatementSegment" => Some("create_function_statement"),
        "CreateIndexStatementSegment" => Some("create_index_statement"),
        "CreateModelStatementSegment" => Some("create_model_statement"),
        "CreateRoleStatementSegment" => Some("create_role_statement"),
        "CreateSchemaStatementSegment" => Some("create_schema_statement"),
        "CreateSequenceOptionsSegment" => Some("create_sequence_options_segment"),
        "CreateSequenceStatementSegment" => Some("create_sequence_statement"),
        "CreateTableStatementSegment" => Some("create_table_statement"),
        "CreateTriggerStatementSegment" => Some("create_trigger"),
        "CreateUserStatementSegment" => Some("create_user_statement"),
        "CreateViewStatementSegment" => Some("create_view_statement"),
        "CubeFunctionNameSegment" => Some("function_name"),
        "CubeRollupClauseSegment" => Some("cube_rollup_clause"),
        "DatabaseReferenceSegment" => Some("database_reference"),
        "DatatypeSegment" => Some("data_type"),
        "DatePartFunctionNameSegment" => Some("function_name"),
        "DateTimeFunctionContentsSegment" => Some("function_contents"),
        "DeclareDistributionClauseSegment" => Some("distribution_clause"),
        "DeclareGlobalTempTableSegment" => Some("declare_temp_table"),
        "Dedent" => Some("dedent"),
        "DeleteStatementSegment" => Some("delete_statement"),
        "DescribeStatementSegment" => Some("describe_statement"),
        "DropCastStatementSegment" => Some("drop_cast_statement"),
        "DropDatabaseStatementSegment" => Some("drop_database_statement"),
        "DropFunctionStatementSegment" => Some("drop_function_statement"),
        "DropIndexStatementSegment" => Some("drop_index_statement"),
        "DropModelStatementSegment" => Some("drop_MODELstatement"),
        "DropRoleStatementSegment" => Some("drop_role_statement"),
        "DropSchemaStatementSegment" => Some("drop_schema_statement"),
        "DropSequenceStatementSegment" => Some("drop_sequence_statement"),
        "DropTableStatementSegment" => Some("drop_table_statement"),
        "DropTriggerStatementSegment" => Some("drop_trigger"),
        "DropTypeStatementSegment" => Some("drop_type_statement"),
        "DropUserStatementSegment" => Some("drop_user_statement"),
        "DropViewStatementSegment" => Some("drop_view_statement"),
        "ElseClauseSegment" => Some("else_clause"),
        "EmptyStructLiteralBracketsSegment" => Some("struct_literal"),
        "EmptyStructLiteralSegment" => Some("typed_struct_literal"),
        "EqualsSegment" => Some("comparison_operator"),
        "ExplainStatementSegment" => Some("explain_statement"),
        "ExpressionSegment" => Some("expression"),
        "ExtensionReferenceSegment" => Some("extension_reference"),
        "FetchClauseSegment" => Some("fetch_clause"),
        "FileSegment" => Some("file"),
        "FrameClauseSegment" => Some("frame_clause"),
        "FromClauseSegment" => Some("from_clause"),
        "FromExpressionElementSegment" => Some("from_expression_element"),
        "FromExpressionSegment" => Some("from_expression"),
        "FunctionContentsSegment" => Some("function_contents"),
        "FunctionDefinitionGrammar" => Some("function_definition"),
        "FunctionNameSegment" => Some("function_name"),
        "FunctionParameterListGrammar" => Some("function_parameter_list"),
        "FunctionSegment" => Some("function"),
        "GrantStatementSegment" => Some("grant_statement"),
        "GreaterThanOrEqualToSegment" => Some("comparison_operator"),
        "GreaterThanSegment" => Some("comparison_operator"),
        "GroupByClauseSegment" => Some("groupby_clause"),
        "GroupingExpressionList" => Some("grouping_expression_list"),
        "GroupingSetsClauseSegment" => Some("grouping_sets_clause"),
        "HavingClauseSegment" => Some("having_clause"),
        "IdentifierSegment" => Some("identifier"),
        "ImplicitIndent" => Some("indent"),
        "Indent" => Some("indent"),
        "IndexColumnDefinitionSegment" => Some("index_column_definition"),
        "IndexReferenceSegment" => Some("index_reference"),
        "InsertStatementSegment" => Some("insert_statement"),
        "IntervalExpressionSegment" => Some("interval_expression"),
        "IsolationClauseSegment" => Some("isolation_clause"),
        "JoinClauseSegment" => Some("join_clause"),
        "JoinOnConditionSegment" => Some("join_on_condition"),
        "KeywordSegment" => Some("keyword"),
        "LessThanOrEqualToSegment" => Some("comparison_operator"),
        "LessThanSegment" => Some("comparison_operator"),
        "LimitClauseSegment" => Some("limit_clause"),
        "LiteralKeywordSegment" => Some("literal"),
        "LiteralSegment" => Some("literal"),
        "LocalAliasSegment" => Some("local_alias_segment"),
        "LockClauseSegment" => Some("lock_clause"),
        "MLTableExpressionSegment" => Some("ml_table_expression"),
        "MapTypeSegment" => Some("map_type"),
        "MatchConditionSegment" => Some("match_condition"),
        "MergeDeleteClauseSegment" => Some("merge_delete_clause"),
        "MergeInsertClauseSegment" => Some("merge_insert_clause"),
        "MergeMatchSegment" => Some("merge_match"),
        "MergeMatchedClauseSegment" => Some("merge_when_matched_clause"),
        "MergeNotMatchedClauseSegment" => Some("merge_when_not_matched_clause"),
        "MergeStatementSegment" => Some("merge_statement"),
        "MergeUpdateClauseSegment" => Some("merge_update_clause"),
        "NamedArgumentSegment" => Some("named_argument"),
        "NamedWindowExpressionSegment" => Some("named_window_expression"),
        "NamedWindowSegment" => Some("named_window"),
        "NewlineSegment" => Some("newline"),
        "NotEqualToSegment" => Some("comparison_operator"),
        "ObjectLiteralElementSegment" => Some("object_literal_element"),
        "ObjectLiteralSegment" => Some("object_literal"),
        "ObjectReferenceSegment" => Some("object_reference"),
        "OffsetClauseSegment" => Some("offset_clause"),
        "OrderByClauseSegment" => Some("orderby_clause"),
        "OverClauseSegment" => Some("over_clause"),
        "OverlapsClauseSegment" => Some("overlaps_clause"),
        "PartitionClauseSegment" => Some("partitionby_clause"),
        "PathSegment" => Some("path_segment"),
        "QualifiedNumericLiteralSegment" => Some("numeric_literal"),
        "ReadOnlyClauseSegment" => Some("read_only_clause"),
        "RevokeStatementSegment" => Some("revoke_statement"),
        "RoleReferenceSegment" => Some("role_reference"),
        "RollupFunctionNameSegment" => Some("function_name"),
        "RowMovementClauseSegment" => Some("row_movement_clause"),
        "SamplingExpressionSegment" => Some("sample_expression"),
        "SchemaReferenceSegment" => Some("schema_reference"),
        "SelectClauseElementSegment" => Some("select_clause_element"),
        "SelectClauseModifierSegment" => Some("select_clause_modifier"),
        "SelectClauseSegment" => Some("select_clause"),
        "SelectStatementSegment" => Some("select_statement"),
        "SequenceReferenceSegment" => Some("sequence_reference"),
        "SetClauseListSegment" => Some("set_clause_list"),
        "SetClauseSegment" => Some("set_clause"),
        "SetExpressionSegment" => Some("set_expression"),
        "SetOperatorSegment" => Some("set_operator"),
        "SetSchemaStatementSegment" => Some("set_schema_statement"),
        "ShorthandCastSegment" => Some("cast_expression"),
        "SingleIdentifierListSegment" => Some("identifier_list"),
        "SingleIdentifierWildcardSegment" => Some("wildcard_identifier"),
        "SizedArrayTypeSegment" => Some("sized_array_type"),
        "StatementSegment" => Some("statement"),
        "StructLiteralSegment" => Some("struct_literal"),
        "StructTypeSegment" => Some("struct_type"),
        "SymbolSegment" => Some("symbol"),
        "TableConstraintSegment" => Some("table_constraint"),
        "TableEndClauseSegment" => Some("table_end_clause_segment"),
        "TableExpressionSegment" => Some("table_expression"),
        "TableReferenceSegment" => Some("table_reference"),
        "TablespaceReferenceSegment" => Some("tablespace_reference"),
        "TagReferenceSegment" => Some("tag_reference"),
        "TemporalQuerySegment" => Some("temporal_query"),
        "TimeZoneGrammar" => Some("time_zone_grammar"),
        "TransactionStatementSegment" => Some("transaction_statement"),
        "TriggerReferenceSegment" => Some("trigger_reference"),
        "TruncateStatementSegment" => Some("truncate_table"),
        "TupleSegment" => Some("tuple"),
        "TypedArrayLiteralSegment" => Some("typed_array_literal"),
        "TypedStructLiteralSegment" => Some("typed_struct_literal"),
        "UnorderedSelectStatementSegment" => Some("select_statement"),
        "UnorderedSetExpressionSegment" => Some("set_expression"),
        "UpdateStatementSegment" => Some("update_statement"),
        "UseStatementSegment" => Some("use_statement"),
        "UserReferenceSegment" => Some("user_reference"),
        "ValuesClauseSegment" => Some("values_clause"),
        "ViewCheckOptionClauseSegment" => Some("check_option_clause"),
        "WhenClauseSegment" => Some("when_clause"),
        "WhereClauseSegment" => Some("where_clause"),
        "WhitespaceSegment" => Some("whitespace"),
        "WildcardExpressionSegment" => Some("wildcard_expression"),
        "WildcardIdentifierSegment" => Some("wildcard_identifier"),
        "WindowSpecificationSegment" => Some("window_specification"),
        "WithCompoundNonSelectStatementSegment" => Some("with_compound_statement"),
        "WithCompoundStatementSegment" => Some("with_compound_statement"),
        "WithDataClauseSegment" => Some("with_data_clause"),
        "WithFillSegment" => Some("with_fill"),
        "WithNoSchemaBindingClauseSegment" => Some("with_no_schema_binding_clause"),
        "WithinGroupClauseSegment" => Some("withingroup_clause"),
        "WordSegment" => Some("word"),
        _ => None,
    }
}

pub static DB2_TABLES: GrammarTables = GrammarTables {
    instructions: INSTRUCTIONS,
    child_ids: CHILD_IDS,
    terminators: TERMINATORS,
    strings: STRINGS,
    aux_data: AUX_DATA,
    aux_data_offsets: AUX_DATA_OFFSETS,
    regex_patterns: REGEX_PATTERNS,
    simple_hints: SIMPLE_HINTS,
    hint_string_indices: HINT_STRING_INDICES,
    simple_hint_indices: SIMPLE_HINT_INDICES,
    segment_type_offsets: SEGMENT_TYPE_OFFSETS,
    segment_class_offsets: SEGMENT_CLASS_OFFSETS,
    casefold_sparse: CASEFOLD_SPARSE,
    trim_chars_sparse: TRIM_CHARS_SPARSE,
    trim_chars_data: TRIM_CHARS_DATA,
    segment_class_types_sparse: SEGMENT_CLASS_TYPES_SPARSE,
    segment_class_types_data: SEGMENT_CLASS_TYPES_DATA,
};

pub fn get_db2_root_grammar_id() -> GrammarId {
    GrammarId(1489)
}
pub fn get_db2_root_grammar_table() -> RootGrammar {
    RootGrammar {
        grammar_id: get_db2_root_grammar_id(),
        tables: &DB2_TABLES,
    }
}
pub fn get_db2_root_grammar() -> RootGrammar {
    get_db2_root_grammar_table()
}
