/* This is a generated file! */
/* Generated by `utils/build_parsers.py` via `utils/rustify.py` */
/* This process can be run via tox: `tox -e generate-rs` */
#![cfg_attr(rustfmt, rustfmt_skip)]

use sqlfluffrs_types::{
    GrammarInst, GrammarVariant, GrammarFlags,
    ParseMode, GrammarId, GrammarTables,
    SimpleHintData, RootGrammar
};
// Flattening grammar tree...
// Validating tables...
// Validation passed!

// Table Statistics:
//   Instructions:      3591 × 20 bytes =    71820 bytes
//   Child IDs:         2531 ×  4 bytes =    10124 bytes
//   Terminators:         49 ×  4 bytes =      196 bytes
//   Strings:           1844 strings      =    24883 bytes
//   Aux Data:         10694 ×  4 bytes =    42776 bytes
//   Aux Offsets:       3591 ×  4 bytes =    14364 bytes
//   Regex Patterns:       5 patterns     =      259 bytes
//   Simple Hints:      1109 hints        =    17744 bytes
//   Hint Indices:      1787 ×  4 bytes =     7148 bytes
//   TOTAL:                                    ≈   189314 bytes

pub static INSTRUCTIONS: &[GrammarInst] = &[
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 0, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [0] StringParser("ABORT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 0, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [1] StringParser("ABS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 0, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [2] StringParser("ABSOLUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 0, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [3] StringParser("ACCESS")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 0, child_count: 3, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [4] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [5] Ref(AccountKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [6] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5, child_count: 17, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [7] OneOf(17 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 22, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [8] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [9] Ref(ResourceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [10] Ref(MonitorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [11] Ref(WarehouseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [12] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [13] Ref(DomainKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [14] Ref(IntegrationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [15] Ref(LanguageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [16] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [17] Ref(CatalogKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [18] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [19] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [20] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [21] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 26, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [22] Ref(ForeignKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 26, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [23] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 28, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [24] Ref(ServerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 28, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [25] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 30, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [26] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 30, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [27] Ref(WrapperKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 30, child_count: 4, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [28] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 34, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [29] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 34, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [30] Ref(SchemasKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 34, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [31] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 34, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [32] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 34, child_count: 4, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [33] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 38, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [34] Ref(FutureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 38, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [35] Ref(SchemasKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 38, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [36] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 38, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [37] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 38, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [38] Ref(AccessSchemaObjectSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 38, child_count: 4, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [39] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 42, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [40] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 42, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [41] Ref(AccessSchemaPluralObjectSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 42, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [42] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 42, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [43] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 42, child_count: 4, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [44] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 46, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [45] Ref(FutureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 46, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [46] Ref(AccessSchemaPluralObjectSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 46, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [47] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 46, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [48] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 48, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [49] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 48, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [50] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 48, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 2, _padding: 0 }, // [51] Delimited(3 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 50, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [52] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 53, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [53] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 53, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [54] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 55, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [55] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 55, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [56] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 55, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [57] Ref(WildcardIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 55, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [58] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 55, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [59] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 55, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [60] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 55, child_count: 3, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [61] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 58, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [62] Ref(LargeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 58, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [63] Ref(ObjectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 58, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [64] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 58, child_count: 6, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [65] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 64, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [66] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 66, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [67] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 66, child_count: 5, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [68] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 71, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [69] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 71, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [70] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 71, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [71] Ref(WarehouseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 71, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [72] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 71, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [73] Ref(IntegrationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 71, child_count: 3, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [74] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 74, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [75] Ref(ApplyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 74, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [76] Ref(MaskingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 74, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [77] Ref(PolicyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 74, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [78] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 76, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [79] Ref(ExecuteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 76, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [80] Ref(TaskKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 76, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [81] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 78, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [82] Ref(ManageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 78, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [83] Ref(GrantsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 78, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [84] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 80, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [85] Ref(MonitorKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 80, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [86] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 82, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [87] Ref(ExecutionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 82, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [88] Ref(UsageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 82, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [89] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 84, child_count: 26, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [90] OneOf(26 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 110, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [91] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 112, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [92] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 112, child_count: 4, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [93] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 116, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [94] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 116, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [95] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 118, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [96] Ref(MaskingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 118, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [97] Ref(PolicyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 118, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [98] Ref(PipeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 118, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [99] Ref(AccessSchemaObjectSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 118, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [100] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 120, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [101] Ref(UseKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 120, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [102] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 122, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [103] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 122, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [104] Ref(CatalogKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 122, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [105] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [106] Ref(ImportedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [107] Ref(PrivilegesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [108] Ref(ApplyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [109] Ref(ConnectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [110] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [111] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [112] Ref(ExecuteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [113] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [114] Ref(ModifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [115] Ref(MonitorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [116] Ref(OperateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [117] Ref(OwnershipKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [118] Ref(ReadKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [119] Ref(Reference_usageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [120] Ref(ReferencesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [121] Ref(SelectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [122] Ref(TempKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [123] Ref(TemporaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [124] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [125] Ref(TruncateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [126] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [127] Ref(UsageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [128] Ref(Use_any_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [129] Ref(WriteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [130] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 126, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [131] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 126, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [132] Ref(PrivilegesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 126, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [133] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 126, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 1, _padding: 0 }, // [134] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 128, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [135] Ref(AccessPermissionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 128, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [136] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 128, child_count: 14, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [137] OneOf(14 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [138] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [139] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [140] Ref(StageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [141] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [142] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [143] Ref(RoutineKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [144] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [145] Ref(StreamKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [146] Ref(TaskKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [147] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [148] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 144, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [149] Ref(MaskingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 144, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [150] Ref(PolicyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 144, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [151] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 146, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [152] Ref(MaterializedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 146, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [153] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 146, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [154] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 148, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [155] Ref(ExternalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 148, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [156] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 148, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [157] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 150, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [158] Ref(FileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 150, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [159] Ref(FormatKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 150, child_count: 14, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [160] OneOf(14 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [161] Ref(TablesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [162] Ref(ViewsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [163] Ref(StagesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [164] Ref(FunctionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [165] Ref(ProceduresKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [166] Ref(RoutinesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [167] Ref(SequencesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [168] Ref(StreamsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [169] Ref(TasksKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [170] Ref(SchemasKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [171] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 166, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [172] Ref(MaskingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 166, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [173] Ref(PoliciesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 166, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [174] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 168, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [175] Ref(MaterializedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 168, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [176] Ref(ViewsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 168, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [177] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 170, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [178] Ref(ExternalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 170, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [179] Ref(TablesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 170, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [180] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 172, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [181] Ref(FileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 172, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [182] Ref(FormatsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 172, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [183] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 174, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [184] Ref(GrantStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 174, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [185] Ref(RevokeStatementSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 174, child_count: 7, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [186] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 181, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [187] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 183, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [188] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 183, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [189] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 185, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [190] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 185, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [191] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 187, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [192] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 187, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [193] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 189, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [194] Ref(UserReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 189, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [195] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 191, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [196] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 191, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [197] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 193, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [198] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 193, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [199] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 195, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [200] Ref(ShareKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 195, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [201] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 197, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [202] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 197, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [203] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 199, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [204] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 199, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [205] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 201, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [206] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 201, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [207] Ref(PublicKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 201, child_count: 1, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [208] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 202, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [209] Ref(ArrayAccessorSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 202, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [210] StringParser("ACCOUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 202, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [211] StringParser("ACCOUNTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 202, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [212] StringParser("ACTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 202, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [213] StringParser("ACTIVITYCOUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 202, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [214] StringParser("ADA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 202, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [215] StringParser("ADD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 202, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [216] StringParser("ADMIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 202, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [217] StringParser("AFTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 202, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [218] StringParser("AGGREGATE")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 202, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [219] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 202, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [220] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 206, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [221] Meta("indent")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 206, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [222] Ref(AsAliasOperatorSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 206, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [223] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 208, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [224] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 210, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [225] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 210, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [226] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 213, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [227] Ref(SingleIdentifierListSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 213, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [228] StringParser("(")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 213, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [229] StringParser(")")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 213, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [230] Ref(SingleQuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 213, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [231] Meta("dedent")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 213, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [232] StringParser("ALIAS")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 213, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [233] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 215, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [234] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 215, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [235] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 215, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [236] StringParser("ALL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 215, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [237] StringParser("ALLOCATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 215, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [238] StringParser("ALSO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 215, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [239] StringParser("ALTER")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 215, child_count: 6, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [240] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 221, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [241] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 224, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [242] Ref(IncrementKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 224, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [243] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 224, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [244] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 224, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [245] Ref(SequenceMinValueGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 224, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [246] Ref(SequenceMaxValueGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 224, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [247] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 226, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [248] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 228, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [249] Ref(CacheKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 228, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [250] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 228, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [251] Ref(NocacheKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 228, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [252] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 230, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [253] Ref(CycleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 230, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [254] Ref(NocycleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 230, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [255] Ref(OrderNoOrderGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 230, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [256] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 234, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [257] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 234, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [258] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 234, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [259] Ref(SequenceReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 234, child_count: 1, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [260] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 235, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [261] Ref(AlterSequenceOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 235, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [262] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 239, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [263] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 239, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [264] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 239, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [265] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 239, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [266] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 239, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [267] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 243, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [268] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 246, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [269] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 246, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [270] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 246, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [271] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 248, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [272] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 248, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [273] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 248, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [274] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 252, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [275] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 254, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [276] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 254, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [277] Ref(ModifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 254, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [278] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 254, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [279] Ref(ColumnDefinitionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 254, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [280] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 256, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [281] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 258, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [282] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 260, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [283] Ref(FirstKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 260, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [284] Ref(AfterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 260, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [285] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 260, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [286] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 260, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [287] Ref(AlterTableDropColumnGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 260, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [288] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 263, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [289] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 263, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [290] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 265, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [291] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 265, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [292] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 265, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [293] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 265, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [294] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 269, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [295] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 269, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [296] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 269, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [297] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 269, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [298] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 271, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [299] Ref(AlterTableOptionsGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 271, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [300] StringParser("ALWAYS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 271, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [301] StringParser("&")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 271, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [302] StringParser("ANALYSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 271, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [303] StringParser("ANALYZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 271, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [304] StringParser("AND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 271, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [305] StringParser("AND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 271, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [306] StringParser("ANY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 271, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [307] StringParser("APPLY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 271, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [308] StringParser("ARE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 271, child_count: 10, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [309] OneOf(10 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 281, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [310] Ref(PlusSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 281, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [311] Ref(MinusSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 281, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [312] Ref(DivideSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 281, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [313] Ref(MultiplySegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 281, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [314] Ref(ModuloSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 281, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [315] Ref(BitwiseAndSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 281, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [316] Ref(BitwiseOrSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 281, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [317] Ref(BitwiseXorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 281, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [318] Ref(BitwiseLShiftSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 281, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [319] Ref(BitwiseRShiftSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 281, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [320] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 284, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [321] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 286, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [322] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 288, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [323] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 288, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [324] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 288, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [325] Ref(SliceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 288, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [326] StringParser("[")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 288, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [327] StringParser("]")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 288, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [328] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 288, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [329] StringParser("ARRAY")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 288, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [330] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 291, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [331] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 293, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [332] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 293, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [333] Nothing
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 293, child_count: 1, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [334] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 294, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [335] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 294, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [336] StringParser("AS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 294, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [337] StringParser("ASC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 294, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [338] StringParser("ASENSITIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 294, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [339] StringParser("ASSERTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 294, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [340] StringParser("ASSIGNMENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 294, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [341] StringParser("ASYMMETRIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 294, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [342] StringParser("AT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 294, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [343] StringParser("ATOMIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 294, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [344] StringParser("ATTRIBUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 294, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [345] StringParser("ATTRIBUTES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 294, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [346] StringParser("AUDIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 294, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [347] StringParser("AUTHORIZATION")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 294, child_count: 1, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [348] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 295, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [349] Ref(Auto_incrementKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 295, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [350] StringParser("AUTO_INCREMENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 295, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [351] StringParser("AUTOINCREMENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 295, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [352] StringParser("AVG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 295, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [353] StringParser("AVG_ROW_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 295, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [354] StringParser("BACKUP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 295, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [355] StringParser("BACKWARD")
    GrammarInst { variant: GrammarVariant::MultiStringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 295, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [356] MultiStringParser(4 templates)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 295, child_count: 7, min_times: 0, first_terminator_idx: 3, terminator_count: 2, _padding: 0 }, // [357] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 302, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [358] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 302, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [359] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 302, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [360] Ref(IntervalExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 302, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [361] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 302, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [362] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 302, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [363] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 302, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [364] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 304, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [365] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 304, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [366] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 304, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [367] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 304, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [368] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 304, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [369] Token("file")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 304, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [370] Token("base")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 304, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [371] StringParser("BEFORE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 304, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [372] StringParser("BEGIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 304, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [373] StringParser("BERNOULLI")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 304, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [374] StringParser("BETWEEN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 304, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [375] StringParser("BIGINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 304, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [376] StringParser("BINARY")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 304, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [377] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 308, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [378] Ref(ArithmeticBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 308, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [379] Ref(StringBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 308, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [380] Ref(BooleanBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 308, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [381] Ref(ComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 308, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [382] Token("binary_operator")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 308, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [383] StringParser("BINDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 308, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [384] StringParser("BIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 308, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [385] StringParser("BIT_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 308, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [386] StringParser("BITVAR")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 308, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [387] Ref(AmpersandSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 308, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [388] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 310, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [389] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 310, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [390] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 310, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [391] Ref(PipeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 310, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [392] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 312, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [393] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 312, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [394] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 312, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [395] StringParser("^")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 312, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [396] StringParser("BLOB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 312, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [397] StringParser("BOOL")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 312, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [398] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 314, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [399] Ref(AndOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 314, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [400] Ref(OrOperatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 314, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [401] StringParser("BOOLEAN")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 314, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [402] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 316, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [403] Ref(TrueSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 316, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [404] Ref(FalseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 316, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [405] StringParser("BOTH")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 316, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [406] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 319, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [407] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 321, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [408] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 321, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [409] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 324, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [410] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 326, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [411] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 326, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [412] Token("bracketed")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 326, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [413] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 329, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [414] Ref(UnorderedSetExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 329, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [415] StringParser("BREADTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 329, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [416] StringParser("BREAK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 329, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [417] StringParser("BROWSE")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 329, child_count: 3, min_times: 1, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [418] AnyNumberOf(3 elements, min=1)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 332, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [419] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 332, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [420] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 332, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [421] Ref(SlashSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 332, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [422] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 335, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [423] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 335, child_count: 14, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [424] OneOf(14 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 349, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [425] Ref(IfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 349, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [426] Ref(ThenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 349, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [427] Ref(LogonKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 349, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [428] Ref(ActivitycountKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 349, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [429] Ref(ErrorcodeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 349, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [430] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 349, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [431] Ref(LabelKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 349, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [432] Ref(GotoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 349, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [433] Ref(LogoffKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 349, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [434] Ref(ImportKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 349, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [435] Ref(ExportKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 349, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [436] Ref(RunKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 349, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [437] Ref(QuitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 349, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [438] Ref(ActivitycountKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 349, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [439] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 349, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [440] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 352, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [441] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 352, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [442] Ref(BteqKeyWordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 352, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [443] AnyNumberOf(3 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 355, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [444] Ref(BteqKeyWordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 355, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [445] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 358, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [446] Ref(FileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 358, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [447] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 358, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [448] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 360, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [449] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 360, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [450] Ref(BteqFilePathSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 360, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [451] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 362, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [452] Ref(ComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 362, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [453] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 362, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [454] StringParser("BULK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 362, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [455] StringParser("BY")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 362, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [456] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 365, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [457] Ref(SingleIdentifierListSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 365, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [458] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 369, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [459] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 369, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [460] Ref(CTEColumnList)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 369, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [461] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 369, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [462] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 372, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [463] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 372, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [464] StringParser("CACHE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 372, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [465] StringParser("CALL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 372, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [466] StringParser("CALLED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 372, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [467] StringParser("CARDINALITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 372, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [468] StringParser("CASCADE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 372, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [469] StringParser("CASCADED")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 372, child_count: 2, min_times: 0, first_terminator_idx: 11, terminator_count: 3, _padding: 0 }, // [470] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 374, child_count: 6, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [471] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 380, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [472] Ref(CaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 380, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [473] Meta("implicit_indent")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(7), first_child_idx: 380, child_count: 1, min_times: 0, first_terminator_idx: 5, terminator_count: 2, _padding: 0 }, // [474] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 381, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [475] Ref(WhenClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 381, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [476] Ref(ElseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 381, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [477] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(7), first_child_idx: 381, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 1, _padding: 0 }, // [478] Ref(ElseClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 381, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [479] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 381, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [480] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 381, child_count: 7, min_times: 0, first_terminator_idx: 11, terminator_count: 0, _padding: 0 }, // [481] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 388, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [482] Ref(CaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 388, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [483] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(7), first_child_idx: 388, child_count: 1, min_times: 0, first_terminator_idx: 8, terminator_count: 2, _padding: 0 }, // [484] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 389, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [485] Ref(WhenClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 389, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [486] Ref(ElseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 389, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [487] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(7), first_child_idx: 389, child_count: 0, min_times: 0, first_terminator_idx: 10, terminator_count: 1, _padding: 0 }, // [488] Ref(ElseClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 389, child_count: 0, min_times: 0, first_terminator_idx: 10, terminator_count: 0, _padding: 0 }, // [489] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 389, child_count: 0, min_times: 0, first_terminator_idx: 11, terminator_count: 0, _padding: 0 }, // [490] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 389, child_count: 0, min_times: 0, first_terminator_idx: 11, terminator_count: 0, _padding: 0 }, // [491] Ref(ComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 389, child_count: 0, min_times: 0, first_terminator_idx: 11, terminator_count: 0, _padding: 0 }, // [492] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 389, child_count: 0, min_times: 0, first_terminator_idx: 11, terminator_count: 0, _padding: 0 }, // [493] Ref(BinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 389, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [494] StringParser("CASE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 389, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [495] StringParser("CASESPECIFIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 389, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [496] StringParser("CAST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 389, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [497] StringParser("::")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 389, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [498] StringParser("CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 389, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [499] StringParser("CATALOG_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 389, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [500] StringParser("CEIL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 389, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [501] StringParser("CEILING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 389, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [502] StringParser("CHAIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 389, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [503] StringParser("CHANGE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 389, child_count: 3, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [504] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 392, child_count: 3, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [505] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 395, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [506] Ref(CharacterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 395, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [507] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 395, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [508] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 395, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [509] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 397, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [510] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 397, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [511] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 399, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [512] Ref(CasespecificKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 399, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [513] Ref(CsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 399, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [514] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 401, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [515] Ref(UppercaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 401, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [516] Ref(UcKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 401, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [517] StringParser("CHAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 401, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [518] StringParser("CHAR_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 401, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [519] StringParser("CHARACTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 401, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [520] StringParser("CHARACTER_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 401, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [521] StringParser("CHARACTER_SET_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 401, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [522] StringParser("CHARACTER_SET_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 401, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [523] StringParser("CHARACTER_SET_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 401, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [524] StringParser("CHARACTERISTICS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 401, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [525] StringParser("CHARACTERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 401, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [526] StringParser("CHECK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 401, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [527] StringParser("CHECKED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 401, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [528] StringParser("CHECKPOINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 401, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [529] StringParser("CHECKSUM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 401, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [530] StringParser("CLASS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 401, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [531] StringParser("CLASS_ORIGIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 401, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [532] StringParser("CLOB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 401, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [533] StringParser("CLOSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 401, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [534] StringParser("CLUSTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 401, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [535] StringParser("CLUSTERED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 401, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [536] StringParser("COALESCE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 401, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [537] StringParser("COBOL")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 401, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [538] Token("raw")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 401, child_count: 3, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [539] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 404, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [540] Ref(CharCharacterSetGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 404, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [541] StringParser("COLLATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 404, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [542] StringParser("COLLATION")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 404, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [543] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 406, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [544] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 406, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 1, _padding: 0 }, // [545] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 408, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [546] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 408, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [547] Ref(ObjectReferenceDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 408, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [548] Ref(ObjectReferenceTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 408, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [549] StringParser("COLLATION_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 408, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [550] StringParser("COLLATION_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 408, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [551] StringParser("COLLATION_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 408, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [552] StringParser("COLLECT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 408, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [553] StringParser(":")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 408, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [554] StringParser(":")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 408, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [555] StringParser(":")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 408, child_count: 4, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [556] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 412, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [557] Ref(ShorthandCastSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 412, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [558] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 412, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [559] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 412, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [560] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 412, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [561] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 414, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [562] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 416, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [563] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 416, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [564] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 416, child_count: 10, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [565] OneOf(10 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 426, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [566] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 428, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [567] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 428, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [568] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 428, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [569] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 430, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [570] Ref(CheckKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 430, child_count: 3, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [571] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 433, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [572] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 433, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [573] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 435, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [574] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 435, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [575] Ref(ColumnConstraintDefaultGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 435, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [576] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 437, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [577] Ref(PrimaryKeyGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 437, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [578] Ref(NotEnforcedGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 437, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [579] Ref(UniqueKeyGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 437, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [580] Ref(AutoIncrementGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 437, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [581] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 439, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [582] Ref(ReferenceDefinitionGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 439, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [583] Ref(NotEnforcedGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 439, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [584] Ref(CommentClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 439, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [585] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 441, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [586] Ref(CollateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 441, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [587] Ref(CollationReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 441, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [588] Ref(ColumnGeneratedGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 441, child_count: 4, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [589] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 445, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [590] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 445, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [591] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 445, child_count: 3, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [592] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 448, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [593] Anything
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 448, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [594] AnyNumberOf(2 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 450, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [595] Ref(ColumnConstraintSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 450, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [596] Ref(TdColumnConstraintSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 450, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [597] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 450, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [598] StringParser("COLUMN")
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 450, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 1, _padding: 0 }, // [599] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 452, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [600] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 452, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [601] Ref(ObjectReferenceDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 452, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [602] Ref(ObjectReferenceTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 452, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [603] StringParser("COLUMN_NAME")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 452, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [604] Nothing
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 452, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [605] Ref(ColumnsExpressionNameGrammar)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 452, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [606] Nothing
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 452, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [607] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 452, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [608] StringParser("COLUMNS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 452, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [609] StringParser(",")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 452, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [610] StringParser("COMMAND_FUNCTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 452, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [611] StringParser("COMMAND_FUNCTION_CODE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 452, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [612] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 454, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [613] Ref(CommentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 454, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [614] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 454, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [615] StringParser("COMMENT")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 454, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [616] Token("comment")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 454, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [617] StringParser("COMMIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 454, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [618] StringParser("COMMITTED")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 454, child_count: 18, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [619] OneOf(18 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 472, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [620] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 472, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [621] Ref(EqualsSegment_a)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 472, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [622] Ref(GreaterThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 472, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [623] Ref(GreaterThanSegment_a)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 472, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [624] Ref(LessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 472, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [625] Ref(LessThanSegment_a)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 472, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [626] Ref(GreaterThanOrEqualToSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 472, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [627] Ref(GreaterThanOrEqualToSegment_a)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 472, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [628] Ref(LessThanOrEqualToSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 472, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [629] Ref(LessThanOrEqualToSegment_a)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 472, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [630] Ref(NotEqualToSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 472, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [631] Ref(NotEqualToSegment_a)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 472, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [632] Ref(NotEqualToSegment_b)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 472, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [633] Ref(NotEqualToSegment_c)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 472, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [634] Ref(LikeOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 472, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [635] Ref(OverlapsOperatorSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 472, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [636] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 475, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [637] Ref(IsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 475, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [638] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 475, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [639] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 475, child_count: 4, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [640] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 479, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [641] Ref(IsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 479, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [642] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 479, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [643] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 479, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [644] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 479, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [645] Token("comparison_operator")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 479, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [646] StringParser("COMPLETION")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 479, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [647] Token("binary_operator")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 479, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [648] Token("comparison_operator")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 479, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [649] StringParser("COMPRESS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 479, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [650] StringParser("COMPUTE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 479, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [651] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 481, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [652] Ref(PipeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 481, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [653] Ref(PipeSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 481, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [654] StringParser("CONDITION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 481, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [655] StringParser("CONDITION_NUMBER")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 481, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [656] Ref(CrossKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 481, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [657] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 484, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [658] Ref(JoinTypeKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 484, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [659] Ref(ConditionalCrossJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 484, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [660] Ref(NonStandardJoinTypeKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 484, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [661] StringParser("CONNECT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 484, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [662] StringParser("CONNECTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 484, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [663] StringParser("CONNECTION_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 484, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [664] StringParser("CONSTRAINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 484, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [665] StringParser("CONSTRAINT_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 484, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [666] StringParser("CONSTRAINT_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 484, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [667] StringParser("CONSTRAINT_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 484, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [668] StringParser("CONSTRAINTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 484, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [669] StringParser("CONSTRUCTOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 484, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [670] StringParser("CONTAINS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 484, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [671] StringParser("CONTAINSTABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 484, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [672] StringParser("CONTINUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 484, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [673] StringParser("CONVERSION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 484, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [674] StringParser("CONVERT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 484, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [675] StringParser("COPY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 484, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [676] StringParser("CORR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 484, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [677] StringParser("CORRESPONDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 484, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [678] StringParser("COUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 484, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [679] StringParser("COVAR_POP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 484, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [680] StringParser("COVAR_SAMP")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 484, child_count: 10, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [681] Sequence(10 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 494, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [682] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 494, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [683] Ref(CastKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 494, child_count: 5, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [684] Bracketed(3 elements -> 3 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 499, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [685] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 499, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [686] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 499, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [687] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 499, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [688] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 499, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [689] Ref(SpecificKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 499, child_count: 4, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [690] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 503, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [691] Ref(RoutineKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 503, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [692] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 503, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [693] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 503, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [694] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 505, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [695] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 508, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [696] Ref(InstanceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 508, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [697] Ref(StaticKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 508, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [698] Ref(ConstructorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 508, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [699] Ref(MethodKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 508, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [700] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 508, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [701] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 508, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [702] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 510, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [703] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 510, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [704] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 510, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [705] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 512, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [706] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 512, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [707] Ref(AssignmentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 512, child_count: 4, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [708] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 516, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [709] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 516, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [710] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 516, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [711] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 516, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [712] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 516, child_count: 9, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [713] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 525, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [714] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 525, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [715] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 525, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [716] Ref(TemporaryGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 525, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [717] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 525, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [718] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 525, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [719] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 525, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [720] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 525, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [721] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 527, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [722] Ref(ReturnsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 527, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [723] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 527, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [724] Ref(FunctionDefinitionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 527, child_count: 9, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [725] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 536, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [726] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 536, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [727] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 536, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [728] Ref(UniqueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 536, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [729] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 536, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [730] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 536, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [731] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 536, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [732] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 536, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [733] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 536, child_count: 1, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [734] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 537, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [735] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 540, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [736] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 542, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [737] Ref(IndexColumnDefinitionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 542, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [738] StringParser("CREATE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 542, child_count: 8, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [739] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 550, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [740] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 550, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [741] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 550, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [742] Ref(ModelKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 550, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [743] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 550, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [744] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 550, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [745] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 552, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [746] Ref(OptionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 552, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [747] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 555, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [748] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 557, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [749] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 560, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [750] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 560, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [751] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 560, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [752] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 562, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [753] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 562, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [754] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 565, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [755] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 567, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [756] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 567, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [757] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 567, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [758] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 567, child_count: 4, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [759] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 571, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [760] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 571, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [761] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 571, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [762] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 571, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [763] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 571, child_count: 4, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [764] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 575, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [765] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 575, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [766] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 575, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [767] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 575, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [768] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 575, child_count: 7, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [769] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 582, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [770] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 585, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [771] Ref(IncrementKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 585, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [772] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 585, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [773] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 585, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [774] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 588, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [775] Ref(StartKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 588, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [776] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 588, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [777] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 588, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [778] Ref(SequenceMinValueGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 588, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [779] Ref(SequenceMaxValueGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 588, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [780] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 590, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [781] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 592, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [782] Ref(CacheKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 592, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [783] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 592, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [784] Ref(NocacheKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 592, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [785] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 594, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [786] Ref(CycleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 594, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [787] Ref(NocycleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 594, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [788] Ref(OrderNoOrderGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 594, child_count: 4, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [789] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 598, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [790] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 598, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [791] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 598, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [792] Ref(SequenceReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 598, child_count: 1, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [793] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 599, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [794] Ref(CreateSequenceOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 599, child_count: 9, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [795] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 608, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [796] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 608, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [797] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 608, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [798] AnySetOf(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 610, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [799] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 612, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [800] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 612, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [801] Ref(MultisetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 612, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [802] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 614, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [803] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 616, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [804] Ref(GlobalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 616, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [805] Ref(TemporaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 616, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [806] Ref(VolatileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 616, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [807] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 616, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [808] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 616, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [809] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 616, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [810] Ref(TdCreateTableOptions)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 616, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [811] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 619, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [812] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 621, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [813] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 624, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [814] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 626, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [815] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 628, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [816] Ref(ColumnDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 628, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [817] Ref(TableConstraintSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 628, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [818] Ref(CommentClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 628, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [819] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 630, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [820] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 630, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [821] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 630, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [822] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 632, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [823] Ref(LikeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 632, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [824] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 632, child_count: 1, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [825] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 633, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [826] Ref(TdTableConstraints)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 633, child_count: 9, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [827] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 642, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [828] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 642, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [829] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 642, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [830] Ref(TriggerReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 642, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [831] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 645, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [832] Ref(BeforeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 645, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [833] Ref(AfterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 645, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [834] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 647, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [835] Ref(InsteadKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 647, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [836] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 647, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 1, _padding: 0 }, // [837] Delimited(3 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 649, child_count: 3, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [838] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 652, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [839] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 652, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [840] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 652, child_count: 3, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [841] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 655, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [842] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 655, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [843] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 655, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 2, _padding: 0 }, // [844] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 657, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [845] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 657, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [846] Ref(OrKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 657, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [847] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 657, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [848] Ref(OrKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 657, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [849] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 657, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [850] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 657, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [851] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 657, child_count: 5, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [852] AnyNumberOf(5 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 662, child_count: 9, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [853] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 671, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [854] Ref(ReferencingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 671, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [855] Ref(OldKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 671, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [856] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 671, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [857] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 671, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [858] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 671, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [859] Ref(NewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 671, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [860] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 671, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [861] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 671, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [862] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 671, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [863] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 673, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [864] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 673, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [865] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 673, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [866] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 675, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [867] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 677, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [868] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 677, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [869] Ref(DeferrableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 677, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [870] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 679, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [871] Ref(DeferrableKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 679, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [872] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 681, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [873] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 683, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [874] Ref(InitiallyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 683, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [875] Ref(ImmediateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 683, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [876] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 685, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [877] Ref(InitiallyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 685, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [878] Ref(DeferredKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 685, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [879] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 688, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [880] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 688, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [881] Ref(EachKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 688, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [882] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 690, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [883] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 690, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [884] Ref(StatementKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 690, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [885] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 692, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [886] Ref(WhenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 692, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [887] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 695, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [888] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 695, child_count: 4, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [889] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 699, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [890] Ref(ExecuteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 699, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [891] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 699, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [892] Ref(FunctionNameIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 699, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [893] Ref(FunctionContentsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 699, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [894] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 702, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [895] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 702, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [896] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 702, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [897] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 702, child_count: 8, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [898] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 710, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [899] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 712, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [900] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 712, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [901] Ref(ReplaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 712, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [902] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 712, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [903] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 712, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [904] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 712, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [905] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 712, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [906] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 712, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [907] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 714, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [908] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 717, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [909] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 717, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [910] Ref(WithNoSchemaBindingClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 717, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [911] StringParser("CREATEDB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 717, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [912] StringParser("CREATEROLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 717, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [913] StringParser("CREATEUSER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 717, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [914] StringParser("CROSS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 717, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [915] StringParser("CS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 717, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [916] StringParser("CSV")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 717, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [917] StringParser("CUBE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 717, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [918] StringParser("CUBE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 717, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [919] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 719, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [920] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 721, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [921] Ref(CubeFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 721, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [922] Ref(RollupFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 721, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [923] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 724, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [924] Ref(GroupingExpressionList)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 724, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [925] StringParser("CUME_DIST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 724, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [926] StringParser("CURRENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 724, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [927] StringParser("CURRENT_DATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 724, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [928] StringParser("CURRENT_DEFAULT_TRANSFORM_GROUP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 724, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [929] StringParser("CURRENT_PATH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 724, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [930] StringParser("CURRENT_ROLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 724, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [931] StringParser("CURRENT_TIME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 724, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [932] StringParser("CURRENT_TIMESTAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 724, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [933] StringParser("CURRENT_TRANSFORM_GROUP_FOR_TYPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 724, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [934] StringParser("CURRENT_USER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 724, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [935] StringParser("CURSOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 724, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [936] StringParser("CURSOR_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 724, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [937] StringParser("CYCLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 724, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [938] StringParser("DATA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 724, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [939] StringParser("DATABASE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 724, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [940] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 726, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [941] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 726, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [942] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 726, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [943] StringParser("DATABASES")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 726, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [944] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 728, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [945] RegexParser
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 728, child_count: 1, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [946] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 729, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [947] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 729, child_count: 5, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [948] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 734, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [949] Ref(DatatypeIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 734, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [950] Ref(BracketedArguments)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 734, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [951] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 737, child_count: 1, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [952] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 738, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [953] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 740, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [954] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 740, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [955] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 742, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [956] Ref(FormatKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 742, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [957] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 742, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [958] AnySetOf(3 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 745, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [959] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 748, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [960] Ref(CharacterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 748, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [961] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 748, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [962] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 748, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [963] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 750, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [964] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 750, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [965] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 752, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [966] Ref(CasespecificKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 752, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [967] Ref(CsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 752, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [968] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 754, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [969] Ref(UppercaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 754, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [970] Ref(UcKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 754, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [971] StringParser("DATE")
    GrammarInst { variant: GrammarVariant::MultiStringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 754, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [972] MultiStringParser(1 templates)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 754, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [973] Ref(DatePartFunctionName)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 754, child_count: 1, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [974] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 755, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [975] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 758, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [976] Delimited(2 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 760, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [977] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 762, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [978] Ref(DatetimeUnitSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 762, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [979] Ref(FunctionContentsGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 762, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [980] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 764, child_count: 4, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [981] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 768, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [982] Ref(DateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 768, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [983] Ref(TimeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 768, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [984] Ref(TimestampKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 768, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [985] Ref(IntervalKeywordSegment)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 768, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [986] TypedParser("single_quote")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 768, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [987] StringParser("DATETIME")
    GrammarInst { variant: GrammarVariant::MultiStringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 768, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [988] MultiStringParser(11 templates)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 768, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [989] StringParser("DATETIME_INTERVAL_CODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 768, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [990] StringParser("DATETIME_INTERVAL_PRECISION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 768, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [991] StringParser("DAY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 768, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [992] StringParser("DAY_HOUR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 768, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [993] StringParser("DAY_MICROSECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 768, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [994] StringParser("DAY_MINUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 768, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [995] StringParser("DAY_SECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 768, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [996] StringParser("DAYOFMONTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 768, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [997] StringParser("DAYOFWEEK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 768, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [998] StringParser("DAYOFYEAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 768, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [999] StringParser("DAYS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 768, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1000] StringParser("DBCC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 768, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1001] StringParser("DEALLOCATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 768, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1002] StringParser("DEC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 768, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1003] StringParser("DECIMAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 768, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1004] StringParser("DECLARE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 768, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1005] StringParser("DEFAULT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 768, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1006] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 770, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1007] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 770, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1008] Ref(ValuesKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 770, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1009] StringParser("DEFAULTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 770, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1010] StringParser("DEFERRABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 770, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1011] StringParser("DEFERRED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 770, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1012] StringParser("DEFINED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 770, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1013] StringParser("DEFINER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 770, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1014] StringParser("DEGREE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 770, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1015] StringParser("DEL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 770, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1016] StringParser("DELAY_KEY_WRITE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 770, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1017] StringParser("DELAYED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 770, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1018] StringParser("DELETE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 770, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1019] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 773, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1020] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 775, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1021] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 775, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1022] Ref(DelKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 775, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1023] Ref(FromClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 775, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1024] Ref(WhereClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 775, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1025] Ref(SemicolonSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 775, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1026] StringParser("DELIMITER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 775, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1027] StringParser("DELIMITERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 775, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1028] StringParser("DENSE_RANK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 775, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1029] StringParser("DENY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 775, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1030] StringParser("DEPTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 775, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1031] StringParser("DEREF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 775, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1032] StringParser("DERIVED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 775, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1033] StringParser("DESC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 775, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1034] StringParser("DESCRIBE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 775, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1035] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 778, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1036] Ref(DescribeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 778, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1037] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 778, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1038] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 778, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1039] StringParser("DESCRIPTOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 778, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1040] StringParser("DESTROY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 778, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1041] StringParser("DESTRUCTOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 778, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1042] StringParser("DETERMINISTIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 778, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1043] StringParser("DIAGNOSTICS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 778, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1044] StringParser("DICTIONARY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 778, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1045] StringParser("DISABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 778, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1046] StringParser("DISCONNECT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 778, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1047] StringParser("DISK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 778, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1048] StringParser("DISPATCH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 778, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1049] StringParser("DISTINCT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 778, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1050] StringParser("DISTINCTROW")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 778, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1051] StringParser("DISTRIBUTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 778, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1052] StringParser("DIV")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 778, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1053] StringParser("/")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 778, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1054] StringParser("DO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 778, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1055] StringParser("DOMAIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 778, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1056] StringParser(".")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 778, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1057] StringParser("DOUBLE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 778, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1058] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 780, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1059] Ref(RestrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 780, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1060] Ref(CascadeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 780, child_count: 4, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1061] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 784, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1062] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 784, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1063] Ref(CastKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 784, child_count: 5, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1064] Bracketed(3 elements -> 3 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 789, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1065] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 789, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1066] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 789, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1067] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 789, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1068] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 789, child_count: 5, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1069] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 794, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1070] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 794, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1071] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 794, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1072] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 794, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1073] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 794, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1074] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 794, child_count: 4, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1075] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 798, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1076] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 798, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1077] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 798, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1078] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 798, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1079] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 798, child_count: 5, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1080] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 803, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1081] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 803, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1082] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 803, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1083] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 803, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1084] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 803, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1085] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 803, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1086] StringParser("DROP")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 803, child_count: 4, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1087] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 807, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1088] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 807, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1089] Ref(ModelKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 807, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1090] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 807, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1091] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 807, child_count: 4, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1092] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 811, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1093] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 811, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1094] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 811, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1095] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 811, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1096] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 811, child_count: 5, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1097] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 816, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1098] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 816, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1099] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 816, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1100] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 816, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1101] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 816, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1102] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 816, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1103] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 819, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1104] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 819, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1105] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 819, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1106] Ref(SequenceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 819, child_count: 6, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1107] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 825, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1108] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 825, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1109] Ref(TemporaryGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 825, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1110] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 825, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1111] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 825, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1112] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 827, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1113] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 827, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1114] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 827, child_count: 4, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1115] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 831, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1116] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 831, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1117] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 831, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1118] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 831, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1119] Ref(TriggerReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 831, child_count: 5, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1120] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 836, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1121] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 836, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1122] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 836, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1123] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 836, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1124] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 836, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1125] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 836, child_count: 4, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1126] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 840, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1127] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 840, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1128] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 840, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1129] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 840, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1130] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 840, child_count: 5, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1131] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 845, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1132] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 845, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1133] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 845, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1134] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 845, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1135] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 845, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1136] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 845, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1137] StringParser("DUAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 845, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1138] StringParser("DUMMY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 845, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1139] StringParser("DUMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 845, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1140] StringParser("DYNAMIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 845, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1141] StringParser("DYNAMIC_FUNCTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 845, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1142] StringParser("DYNAMIC_FUNCTION_CODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 845, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1143] StringParser("EACH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 845, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1144] StringParser("ELEMENT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 845, child_count: 4, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1145] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 849, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1146] Ref(ElseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 849, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1147] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 849, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1148] StringParser("ELSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 849, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1149] StringParser("ELSEIF")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 849, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1150] Bracketed(0 elements -> 0 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 851, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1151] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 853, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1152] Ref(StructTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 853, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1153] Ref(EmptyStructLiteralBracketsSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 853, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1154] StringParser("ENABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 853, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1155] StringParser("ENCLOSED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 853, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1156] StringParser("ENCODING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 853, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1157] StringParser("ENCRYPTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 853, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1158] StringParser("END-EXEC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 853, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1159] StringParser("}")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 853, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1160] StringParser("END")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 853, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1161] StringParser("ENUM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 853, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1162] StringParser("EQUALS")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 853, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1163] Ref(RawEqualsSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 853, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1164] StringParser("EQ")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 853, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1165] StringParser("ERRLVL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 853, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1166] StringParser("ERRORCODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 853, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1167] StringParser("ESCAPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 853, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1168] StringParser("ESCAPED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 853, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1169] StringParser("EVERY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 853, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1170] StringParser("EXCEPT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 853, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1171] StringParser("EXCEPTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 853, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1172] StringParser("EXCL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 853, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1173] StringParser("EXCLUDE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 853, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1174] StringParser("EXCLUDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 853, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1175] StringParser("EXCLUSIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 853, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1176] StringParser("EXEC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 853, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1177] StringParser("EXECUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 853, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1178] StringParser("EXECUTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 853, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1179] StringParser("EXISTING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 853, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1180] StringParser("EXISTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 853, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1181] StringParser("EXIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 853, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1182] StringParser("EXP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 853, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1183] StringParser("EXPLAIN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 853, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1184] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 855, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1185] Ref(ExplainKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 855, child_count: 4, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1186] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 859, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1187] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 859, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1188] Ref(InsertStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 859, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1189] Ref(UpdateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 859, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1190] Ref(DeleteStatementSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 859, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1191] StringParser("EXPORT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 859, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1192] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 861, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1193] Ref(Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 861, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1194] Ref(TeradataCastSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 861, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1195] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 863, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1196] Ref(Tail_Recurse_Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 863, child_count: 1, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1197] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 864, child_count: 10, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1198] OneOf(10 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 874, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1199] Ref(LikeExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 874, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1200] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 876, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1201] Ref(BinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 876, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1202] Ref(Tail_Recurse_Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 876, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1203] Ref(InOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 876, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1204] Ref(QuantifiedComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 876, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1205] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 879, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1206] Ref(IsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 879, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1207] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 879, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1208] Ref(IsClauseGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 879, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1209] Ref(IsNullGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 879, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1210] Ref(NotNullGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 879, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1211] Ref(CollateGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 879, child_count: 5, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1212] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 884, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1213] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 884, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1214] Ref(BetweenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 884, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1215] Ref(Expression_B_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 884, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1216] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 884, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1217] Ref(Tail_Recurse_Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 884, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1218] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 886, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1219] Ref(PatternMatchingGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 886, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1220] Ref(Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 886, child_count: 4, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1221] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 890, child_count: 1, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1222] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 891, child_count: 1, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1223] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 892, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1224] Ref(QualifiedNumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 892, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1225] Ref(TildeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 892, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1226] Ref(NotOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 892, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1227] Ref(PriorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 892, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1228] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 894, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1229] Ref(Tail_Recurse_Expression_B_Grammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 894, child_count: 1, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1230] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 895, child_count: 1, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1231] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 896, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1232] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 898, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1233] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 901, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1234] Ref(ArithmeticBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 901, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1235] Ref(StringBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 901, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1236] Ref(ComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 901, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1237] Ref(Tail_Recurse_Expression_B_Grammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 901, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1238] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 903, child_count: 1, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1239] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 904, child_count: 1, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1240] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 905, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1241] Ref(QualifiedNumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 905, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1242] Ref(TildeSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 905, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 1, _padding: 0 }, // [1243] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 908, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1244] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 910, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1245] Ref(ExistsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 910, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1246] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 913, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1247] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 913, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1248] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 915, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1249] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 917, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1250] Ref(Expression_D_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 917, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1251] Ref(CaseExpressionSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 917, child_count: 1, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1252] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 918, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1253] Ref(TimeZoneGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 918, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1254] Ref(ShorthandCastSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 918, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1255] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 918, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1256] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 920, child_count: 9, min_times: 0, first_terminator_idx: 20, terminator_count: 1, _padding: 0 }, // [1257] OneOf(9 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 929, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1258] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 929, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1259] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 929, child_count: 3, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1260] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 932, child_count: 4, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1261] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 936, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1262] Ref(LateralColumnAliasExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 936, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1263] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 936, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1264] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 936, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1265] Delimited(5 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 938, child_count: 5, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1266] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 943, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1267] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 943, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1268] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 943, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1269] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 943, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1270] Ref(LocalAliasSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 943, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1271] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 943, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1272] Ref(Expression_D_Potential_Select_Statement_Without_Brackets)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 943, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1273] Ref(SingleIdentifierWildcardSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 943, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1274] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 945, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1275] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 947, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1276] Ref(StructTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 947, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1277] Ref(MapTypeSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 947, child_count: 3, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1278] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 950, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1279] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 952, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1280] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 952, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1281] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 954, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1282] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 954, child_count: 5, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1283] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 959, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1284] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 959, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1285] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 959, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1286] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 959, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1287] Ref(NullLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 959, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1288] Ref(DateTimeLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 959, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1289] Ref(LocalAliasSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 959, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1290] Ref(ListComprehensionGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 959, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1291] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 959, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1292] Ref(AccessorGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 959, child_count: 7, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1293] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 966, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1294] Ref(SelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 966, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1295] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 966, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1296] Ref(IntervalExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 966, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1297] Ref(TypedStructLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 966, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1298] Ref(ArrayExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 966, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1299] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 966, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1300] Ref(OverlapsClauseSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 966, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1301] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 966, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1302] StringParser("EXTENSION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 966, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1303] StringParser("EXTERNAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 966, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1304] StringParser("EXTRACT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 966, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1305] StringParser("FALLBACK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 966, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1306] StringParser("FALSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 966, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1307] StringParser("FALSE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 966, child_count: 5, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1308] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 971, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1309] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 971, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1310] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 973, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1311] Ref(FirstKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 973, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1312] Ref(NextKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 973, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1313] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 975, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1314] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 975, child_count: 1, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1315] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 976, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1316] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 976, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1317] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 978, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1318] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 978, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1319] Ref(RowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 978, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1320] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 980, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1321] Ref(OnlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 980, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1322] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 982, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1323] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 982, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1324] Ref(TiesKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 982, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1325] StringParser("FETCH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 982, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1326] StringParser("FIELDS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 982, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1327] StringParser("FILE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 982, child_count: 3, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1328] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 985, child_count: 1, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1329] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 986, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1330] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(12), first_child_idx: 986, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1331] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 988, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1332] Ref(StatementSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 988, child_count: 1, min_times: 1, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1333] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 989, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1334] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 989, child_count: 1, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1335] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 990, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1336] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 990, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1337] StringParser("FILLFACTOR")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 990, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1338] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 992, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1339] Ref(FilterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 992, child_count: 3, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1340] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 995, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1341] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 997, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1342] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 997, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1343] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 997, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1344] StringParser("FILTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 997, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1345] StringParser("FINAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 997, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1346] StringParser("FIRST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 997, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1347] StringParser("FLOAT4")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 997, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1348] StringParser("FLOAT8")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 997, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1349] StringParser("FLOAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 997, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1350] StringParser("FLOOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 997, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1351] StringParser("FLUSH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 997, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1352] StringParser("FOLLOWING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 997, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1353] StringParser("FOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 997, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1354] StringParser("FORCE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 997, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1355] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 999, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1356] Ref(ForeignKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 999, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1357] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 999, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1358] StringParser("FOREIGN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 999, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1359] StringParser("FORMAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 999, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1360] StringParser("FORTRAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 999, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1361] StringParser("FORWARD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 999, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1362] StringParser("FOUND")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 999, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1363] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1001, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1364] Ref(FrameClauseUnitGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1001, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1365] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1003, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1366] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1005, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1367] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1007, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1368] Ref(CurrentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1007, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1369] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1007, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1370] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1009, child_count: 4, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1371] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1013, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1372] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1013, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1373] Ref(IntervalExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1013, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1374] Ref(UnboundedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1013, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1375] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1013, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1376] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1015, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1377] Ref(PrecedingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1015, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1378] Ref(FollowingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1015, child_count: 4, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1379] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1019, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1380] Ref(BetweenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1019, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1381] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1019, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1382] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1021, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1383] Ref(RowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1021, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1384] Ref(RangeKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1021, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1385] StringParser("FREE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1021, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1386] StringParser("FREETEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1021, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1387] StringParser("FREETEXTTABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1021, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1388] StringParser("FREEZE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1021, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1389] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1023, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1390] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1023, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1391] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1025, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1392] Ref(FromExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1025, child_count: 12, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1393] OneOf(12 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1037, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1394] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1037, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1395] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1037, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1396] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1039, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1397] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1039, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1398] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1039, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1399] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1041, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1400] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1041, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1401] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1041, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1402] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1041, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1403] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1041, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1404] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1041, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1405] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1041, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1406] Ref(WithNoSchemaBindingClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1041, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1407] Ref(WithDataClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1041, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1408] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1041, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1409] Ref(OffsetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1041, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1410] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1043, child_count: 7, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1411] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1050, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1412] Ref(PreTableFunctionKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1050, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1413] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1052, child_count: 3, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1414] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1055, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1415] Ref(TableExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1055, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1416] Ref(TemporalQuerySegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(69), first_child_idx: 1055, child_count: 1, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1417] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1056, child_count: 4, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1418] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1060, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1419] Ref(FromClauseTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1060, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1420] Ref(SamplingExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1060, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1421] Ref(JoinLikeClauseGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1060, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1422] Ref(JoinClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1060, child_count: 3, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1423] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1063, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1424] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1063, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1425] Ref(OffsetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1063, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1426] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1063, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1427] Ref(SamplingExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1063, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1428] Ref(PostTableExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1063, child_count: 3, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1429] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1066, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1430] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1068, child_count: 1, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1431] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1069, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1432] Ref(JoinClauseSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1069, child_count: 2, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1433] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1071, child_count: 3, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1434] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1074, child_count: 6, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1435] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1080, child_count: 3, min_times: 0, first_terminator_idx: 21, terminator_count: 2, _padding: 0 }, // [1436] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1083, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1437] Ref(MLTableExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1083, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1438] Ref(FromExpressionElementSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1083, child_count: 3, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1439] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1086, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1440] Ref(FromExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1086, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1441] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1088, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1442] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1088, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1443] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1088, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1444] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1090, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1445] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1090, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1446] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 1090, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1447] Meta(conditional: indented_joins=True -> indent)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1090, child_count: 1, min_times: 0, first_terminator_idx: 23, terminator_count: 2, _padding: 0 }, // [1448] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1091, child_count: 1, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1449] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1092, child_count: 2, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1450] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1094, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1451] Ref(JoinClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1094, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1452] Ref(JoinLikeClauseGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1094, child_count: 2, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1453] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1096, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1454] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1096, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1455] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1096, child_count: 2, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1456] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1098, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1457] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1098, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1458] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 1098, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1459] Meta(conditional: indented_joins=True -> dedent)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1098, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1460] StringParser("FROM")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1098, child_count: 2, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1461] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1100, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1462] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1100, child_count: 2, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1463] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1102, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1464] Ref(FromExpressionElementSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1102, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1465] StringParser("FULL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1102, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1466] StringParser("FULLTEXT")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1102, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1467] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1102, child_count: 12, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1468] AnyNumberOf(12 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1114, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1469] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1114, child_count: 3, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1470] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1117, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1471] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1117, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1472] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1117, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1473] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1117, child_count: 4, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1474] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1121, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1475] Ref(TrimParametersGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(69), first_child_idx: 1121, child_count: 1, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1476] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1122, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1477] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1122, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1478] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1122, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1479] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1122, child_count: 3, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1480] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1125, child_count: 2, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1481] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1127, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1482] Ref(DatetimeUnitSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1127, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1483] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1127, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1484] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1127, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1485] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1127, child_count: 2, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1486] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1129, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1487] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1129, child_count: 2, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1488] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1131, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1489] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1131, child_count: 2, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1490] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1133, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1491] Ref(FunctionContentsExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1133, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1492] Ref(AggregateOrderByClause)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1133, child_count: 2, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1493] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1135, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1494] Ref(SeparatorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1135, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1495] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1135, child_count: 3, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1496] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1138, child_count: 3, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1497] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1141, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1498] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1141, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1499] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1141, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1500] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1141, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1501] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1141, child_count: 3, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1502] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1144, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1503] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1144, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1504] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1144, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1505] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1144, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1506] Ref(IgnoreRespectNullsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1144, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1507] Ref(IndexColumnDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1144, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1508] Ref(EmptyStructLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1144, child_count: 3, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1509] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1147, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1510] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1147, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1511] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1147, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1512] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1147, child_count: 1, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1513] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1148, child_count: 3, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1514] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1151, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1515] Ref(FunctionContentsGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1151, child_count: 3, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1516] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1154, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1517] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1154, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1518] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1154, child_count: 2, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1519] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1156, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1520] Ref(LanguageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1156, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1521] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1156, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1522] StringParser("FUNCTION")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1156, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1523] TypedParser("word")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1156, child_count: 2, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1524] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1158, child_count: 1, min_times: 0, first_terminator_idx: 25, terminator_count: 1, _padding: 0 }, // [1525] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1159, child_count: 2, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1526] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1161, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1527] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1161, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1528] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1161, child_count: 0, min_times: 0, first_terminator_idx: 25, terminator_count: 0, _padding: 0 }, // [1529] Ref(BracketedSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1161, child_count: 2, min_times: 0, first_terminator_idx: 26, terminator_count: 1, _padding: 0 }, // [1530] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1163, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1531] Ref(FunctionNameIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1163, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1532] Ref(QuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1163, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1533] Ref(BracketedSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1163, child_count: 2, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1534] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1165, child_count: 2, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1535] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1167, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1536] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1167, child_count: 2, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1537] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1169, child_count: 2, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1538] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1171, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1539] Ref(AnyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1171, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1540] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1171, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1541] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1171, child_count: 2, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1542] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1173, child_count: 2, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1543] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1175, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1544] Ref(AnyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1175, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1545] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1175, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1546] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1175, child_count: 3, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1547] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1178, child_count: 2, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1548] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1180, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1549] Ref(FunctionParameterGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1180, child_count: 3, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1550] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1183, child_count: 2, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1551] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1185, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1552] Ref(DatePartFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1185, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1553] Ref(DateTimeFunctionContentsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1185, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1554] Ref(ColumnsExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1185, child_count: 2, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1555] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1187, child_count: 2, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1556] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 1189, child_count: 1, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1557] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1190, child_count: 3, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1558] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1193, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1559] Ref(DatePartFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1193, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1560] Ref(ColumnsExpressionFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1193, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1561] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1193, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1562] Ref(FunctionContentsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1193, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1563] Ref(PostFunctionGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1193, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1564] StringParser("FUNCTIONS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1193, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1565] StringParser("FUSION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1193, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1566] StringParser("FUTURE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1193, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1567] StringParser("G")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1193, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1568] StringParser("GENERAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1193, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1569] StringParser("GENERATED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1193, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1570] StringParser("GET")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1193, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1571] TypedParser("glob_operator")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1193, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1572] StringParser("GLOBAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1193, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1573] StringParser("GO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1193, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1574] StringParser("GOTO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1193, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1575] StringParser("GRANT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1193, child_count: 6, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1576] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1199, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1577] Ref(GrantKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1199, child_count: 4, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1578] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1203, child_count: 3, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1579] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1206, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1580] Ref(AccessPermissionsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1206, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1581] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1206, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1582] Ref(AccessObjectSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1206, child_count: 2, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1583] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1208, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1584] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1208, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1585] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1208, child_count: 4, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1586] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1212, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1587] Ref(OwnershipKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1212, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1588] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1212, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1589] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1212, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1590] Ref(UserReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1212, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1591] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1212, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1592] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1212, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1593] Ref(AccessTargetSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1212, child_count: 3, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1594] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1215, child_count: 3, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1595] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1218, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1596] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1218, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1597] Ref(GrantKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1218, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1598] Ref(OptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1218, child_count: 3, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1599] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1221, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1600] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1221, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1601] Ref(AdminKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1221, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1602] Ref(OptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1221, child_count: 3, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1603] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1224, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1604] Ref(CopyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1224, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1605] Ref(CurrentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1224, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1606] Ref(GrantsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1224, child_count: 3, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1607] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1227, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1608] Ref(GrantedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1227, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1609] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1227, child_count: 3, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1610] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1230, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1611] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1230, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1612] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1230, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1613] Ref(UserReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1230, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1614] StringParser("GRANTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1230, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1615] StringParser("GRANTS")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1230, child_count: 2, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1616] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1232, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1617] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1232, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1618] Ref(RawEqualsSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1232, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1619] StringParser("GE")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1232, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1620] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1232, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1621] StringParser("GT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1232, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1622] StringParser("GREATEST")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1232, child_count: 5, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1623] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1237, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1624] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1237, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1625] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1237, child_count: 4, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1626] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1241, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1627] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1241, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1628] Ref(GroupingSetsClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1241, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1629] Ref(CubeRollupClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1241, child_count: 1, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1630] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1242, child_count: 2, min_times: 0, first_terminator_idx: 27, terminator_count: 1, _padding: 0 }, // [1631] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1244, child_count: 3, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1632] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1247, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1633] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1247, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1634] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1247, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1635] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1247, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [1636] Ref(GroupByClauseTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1247, child_count: 6, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1637] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1253, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1638] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1255, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1639] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1255, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1640] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1255, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1641] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1255, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1642] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1255, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1643] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1255, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1644] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1255, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1645] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1255, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1646] StringParser("GROUP")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1255, child_count: 1, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1647] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1256, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 1, _padding: 0 }, // [1648] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1258, child_count: 4, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1649] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1262, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1650] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1262, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1651] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1262, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1652] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1262, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1653] Bracketed(0 elements -> 0 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1264, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [1654] Ref(GroupByClauseTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1264, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1655] StringParser("GROUPING")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1264, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1656] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1267, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1657] Ref(GroupingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1267, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1658] Ref(SetsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1267, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1659] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1270, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1660] Delimited(2 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1272, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1661] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1274, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1662] Ref(CubeRollupClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1274, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1663] Ref(GroupingExpressionList)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1274, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1664] StringParser("HANDLER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1274, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1665] StringParser("HASH")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1274, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1666] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1278, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1667] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1278, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1668] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1280, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1669] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1283, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1670] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1283, child_count: 5, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1671] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1288, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1672] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1290, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1673] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1290, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1674] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1290, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1675] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1290, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1676] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1290, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1677] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1290, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1678] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1290, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1679] StringParser("HAVING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1290, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1680] StringParser("HEADER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1290, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1681] StringParser("HEAP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1290, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1682] StringParser("HIERARCHY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1290, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1683] StringParser("HIGH_PRIORITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1290, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1684] StringParser("HOLD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1290, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1685] StringParser("HOLDLOCK")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1290, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1686] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1290, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1687] StringParser("HOST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1290, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1688] StringParser("HOSTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1290, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1689] StringParser("HOUR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1290, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1690] StringParser("HOUR_MICROSECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1290, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1691] StringParser("HOUR_MINUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1290, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1692] StringParser("HOUR_SECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1290, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1693] StringParser("IDENTIFIED")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1290, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1694] Token("identifier")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1290, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1695] StringParser("IDENTITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1290, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1696] StringParser("IDENTITY_INSERT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1290, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1697] StringParser("IDENTITYCOL")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1290, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1698] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1292, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1699] Ref(IfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1292, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1700] Ref(ExistsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1292, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1701] StringParser("IF")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1292, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1702] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1295, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1703] Ref(IfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1295, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1704] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1295, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1705] Ref(ExistsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1295, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1706] StringParser("IGNORE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1295, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1707] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1297, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1708] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1299, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1709] Ref(IgnoreKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1299, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1710] Ref(RespectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1299, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1711] Ref(NullsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1299, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1712] StringParser("ILIKE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1299, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1713] StringParser("IMMEDIATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1299, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1714] StringParser("IMMUTABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1299, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1715] StringParser("IMPLEMENTATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1299, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1716] StringParser("IMPLICIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1299, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1717] StringParser("IMPORT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1299, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1718] StringParser("IMPORTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1299, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1719] StringParser("IN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1299, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1720] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1302, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1721] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1302, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1722] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1302, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1723] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 1304, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1724] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1307, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1725] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1309, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1726] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1311, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1727] Ref(Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1311, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1728] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1311, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1729] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1311, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1730] StringParser("INCLUDE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1311, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1731] StringParser("INCLUDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1311, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1732] StringParser("INCREMENT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1311, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1733] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1313, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1734] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1313, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1735] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1315, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1736] Ref(AscKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1315, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1737] Ref(DescKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1315, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1738] StringParser("INDEX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1315, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1739] StringParser("INDICATOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1315, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1740] StringParser("INFILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1315, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1741] StringParser("INFIX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1315, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1742] StringParser("INHERIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1315, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1743] StringParser("INHERITS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1315, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1744] StringParser("INITIAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1315, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1745] StringParser("INITIALIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1315, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1746] StringParser("INITIALLY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1315, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1747] StringParser("INNER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1315, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1748] StringParser("INOUT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1315, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1749] StringParser("INPUT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1315, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1750] StringParser("INSENSITIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1315, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1751] StringParser("INSERT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1315, child_count: 5, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1752] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1320, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1753] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1320, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1754] Ref(OverwriteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1320, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1755] Ref(IntoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1320, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1756] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1320, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1757] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1323, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1758] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1323, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1759] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1325, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1760] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1325, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1761] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1325, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1762] Ref(DefaultValuesGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1325, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1763] StringParser("INSERT_ID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1325, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1764] StringParser("INSTANCE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1325, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1765] StringParser("INSTANTIABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1325, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1766] StringParser("INSTEAD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1325, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1767] StringParser("INT1")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1325, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1768] StringParser("INT2")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1325, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1769] StringParser("INT3")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1325, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1770] StringParser("INT4")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1325, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1771] StringParser("INT8")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1325, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1772] StringParser("INT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1325, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1773] StringParser("INTEGER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1325, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1774] StringParser("INTEGRATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1325, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1775] StringParser("INTEGRATIONS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1325, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1776] StringParser("INTERSECT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1325, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1777] StringParser("INTERSECTION")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1325, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1778] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1327, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1779] Ref(IntervalKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1327, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1780] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1330, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1781] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1332, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1782] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1332, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1783] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1334, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1784] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1334, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1785] Ref(DatetimeUnitSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1334, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1786] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1334, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1787] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1336, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1788] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1336, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1789] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1338, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1790] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1338, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1791] Ref(DatetimeUnitSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1338, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1792] StringParser("INTERVAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1338, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1793] StringParser("INTO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1338, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1794] StringParser("INVOKER")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1338, child_count: 5, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1795] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1343, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1796] Ref(NullLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1343, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1797] Ref(NanLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1343, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1798] Ref(UnknownLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1343, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1799] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1343, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1800] Ref(NormalizedGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1343, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1801] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1347, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1802] Ref(IsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1347, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1803] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1347, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1804] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1347, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1805] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1347, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1806] StringParser("IS")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1347, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1807] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1347, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1808] StringParser("ISAM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1347, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1809] StringParser("ISNULL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1347, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1810] StringParser("ISOLATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1347, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1811] StringParser("ITERATE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1347, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1812] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1350, child_count: 7, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1813] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1357, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1814] Ref(ConditionalJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1357, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1815] Ref(JoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1357, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1816] Ref(FromExpressionElementSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1357, child_count: 1, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1817] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1358, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1818] Ref(NestedJoinGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1358, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1819] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 1362, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1820] Meta(conditional: indented_using_on=True -> indent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1362, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1821] Ref(MatchConditionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1362, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1822] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1364, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1823] Ref(JoinOnConditionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1364, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1824] Ref(JoinUsingConditionGrammar)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 1364, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1825] Meta(conditional: indented_using_on=True -> dedent)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1364, child_count: 6, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1826] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1370, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1827] Ref(UnconditionalJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1370, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1828] Ref(JoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1370, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1829] Ref(FromExpressionElementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1370, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1830] Ref(MatchConditionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1370, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1831] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1374, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1832] Ref(ExtendedNaturalJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1374, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1833] Ref(FromExpressionElementSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1374, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1834] StringParser("JOIN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1374, child_count: 1, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1835] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1375, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1836] Ref(JoinKeywordSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1375, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1837] Nothing
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1375, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1838] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1379, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1839] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 1379, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1840] Meta(conditional: indented_on_contents=True -> implicit_indent)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1379, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1841] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1381, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1842] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1384, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1843] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 1384, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1844] Meta(conditional: indented_on_contents=True -> dedent)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1384, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1845] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1386, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1846] Ref(InnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1386, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1847] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1388, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1848] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1391, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1849] Ref(FullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1391, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1850] Ref(LeftKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1391, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1851] Ref(RightKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1391, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1852] Ref(OuterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1391, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1853] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1395, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1854] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 1395, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1855] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1398, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1856] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1400, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1857] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1400, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1858] StringParser("JOURNAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1400, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1859] StringParser("JSON")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1400, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1860] StringParser("K")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1400, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1861] StringParser("KEY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1400, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1862] StringParser("KEY_MEMBER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1400, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1863] StringParser("KEY_TYPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1400, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1864] StringParser("KEYS")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1400, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1865] Token("keyword")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1400, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1866] StringParser("KILL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1400, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1867] StringParser("LABEL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1400, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1868] StringParser("LANCOMPILER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1400, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1869] StringParser("LANGUAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1400, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1870] StringParser("LARGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1400, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1871] StringParser("LAST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1400, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1872] StringParser("LAST_INSERT_ID")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1400, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1873] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1400, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1874] StringParser("LATERAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1400, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1875] StringParser("LEADING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1400, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1876] StringParser("LEAST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1400, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1877] StringParser("LEAVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1400, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1878] StringParser("LEFT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1400, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1879] StringParser("LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1400, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1880] StringParser("LESS")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1400, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1881] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1402, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1882] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1402, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1883] Ref(RawEqualsSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1402, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1884] StringParser("LE")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1402, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1885] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1402, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1886] StringParser("LT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1402, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1887] StringParser("LEVEL")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1402, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1888] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1405, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1889] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1407, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1890] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1407, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1891] Ref(LikeGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1407, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1892] Ref(Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1407, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1893] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1409, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1894] Ref(EscapeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1409, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1895] Ref(Tail_Recurse_Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1409, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1896] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1412, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1897] Ref(LikeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1412, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1898] Ref(RlikeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1412, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1899] Ref(IlikeKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1412, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1900] StringParser("LIKE")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1412, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1901] TypedParser("like_operator")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1412, child_count: 5, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1902] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1417, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1903] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1417, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1904] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1419, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1905] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1422, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1906] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1425, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1907] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1425, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1908] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1425, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1909] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1425, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1910] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1427, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1911] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1429, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1912] Ref(OffsetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1429, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1913] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1431, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1914] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1431, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1915] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1431, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1916] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1433, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1917] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1433, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1918] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1433, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1919] StringParser("LIMIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1433, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1920] StringParser("LINENO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1433, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1921] StringParser("LINES")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1433, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1922] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1433, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1923] StringParser("LISTEN")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1433, child_count: 9, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1924] OneOf(9 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1442, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1925] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1442, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1926] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1442, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1927] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1442, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1928] Ref(QualifiedNumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1442, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1929] Ref(NullLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1442, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1930] Ref(DateTimeLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1442, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1931] Ref(ArrayLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1442, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1932] Ref(TypedArrayLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1442, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1933] Ref(ObjectLiteralSegment)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1442, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1934] Token("literal")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1442, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1935] Token("literal")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1442, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1936] StringParser("LN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1442, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1937] StringParser("LOAD")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1442, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1938] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1442, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1939] StringParser("LOCAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1442, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1940] StringParser("LOCALTIME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1442, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1941] StringParser("LOCALTIMESTAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1442, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1942] StringParser("LOCATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1442, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1943] StringParser("LOCATOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1442, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1944] StringParser("LOCK")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1442, child_count: 6, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1945] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1448, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1946] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1450, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1947] Ref(LockingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1450, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1948] Ref(LockKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1450, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1949] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1454, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1950] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1454, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1951] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1456, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1952] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1456, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1953] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1456, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1954] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1458, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1955] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1458, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1956] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1458, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1957] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1460, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1958] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1460, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1959] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1460, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1960] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1462, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1961] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1462, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1962] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1462, child_count: 8, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1963] OneOf(8 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1470, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1964] Ref(AccessKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1470, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1965] Ref(WriteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1470, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1966] Ref(ExclusiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1470, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1967] Ref(ExclKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1470, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1968] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1472, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1969] Ref(ReadKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1472, child_count: 1, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1970] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1473, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1971] Ref(OverrideKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1473, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1972] Ref(ShareKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1473, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1973] Ref(ChecksumKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1473, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1974] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1475, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1975] Ref(LoadKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1475, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1976] Ref(CommittedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1475, child_count: 1, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1977] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1476, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1978] Ref(ModeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1476, child_count: 1, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1979] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1477, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1980] Ref(NowaitKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1477, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1981] StringParser("LOCKING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1477, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1982] StringParser("LOCKS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1477, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1983] StringParser("LOGIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1477, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1984] StringParser("LOGOFF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1477, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1985] StringParser("LOGON")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1477, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1986] StringParser("LOGS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1477, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1987] StringParser("LONG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1477, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1988] StringParser("LONGBLOB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1477, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1989] StringParser("LONGTEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1477, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1990] StringParser("LOOP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1477, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1991] StringParser("LOW_PRIORITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1477, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1992] StringParser("LOWER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1477, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1993] StringParser("M")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1477, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1994] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1481, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1995] Ref(MlKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1481, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1996] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1481, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1997] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1481, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1998] Bracketed(2 elements -> 2 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1485, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [1999] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1487, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2000] Ref(ModelKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1487, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2001] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1487, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2002] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1489, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2003] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1489, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2004] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1492, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2005] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1492, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2006] StringParser("MACRO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1492, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2007] StringParser("MANAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1492, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2008] StringParser("MAP")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1492, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2009] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1492, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2010] StringParser("MASKING")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1492, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2011] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1492, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2012] StringParser("MATCH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1492, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2013] StringParser("MATCHED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1492, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2014] StringParser("MATERIALIZED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1492, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2015] StringParser("MAX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1492, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2016] StringParser("MAX_ROWS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1492, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2017] StringParser("MAXEXTENTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1492, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2018] StringParser("MAXINTERVALS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1492, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2019] StringParser("MAXVALUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1492, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2020] StringParser("MAXVALUELENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1492, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2021] StringParser("MEDIUMBLOB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1492, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2022] StringParser("MEDIUMINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1492, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2023] StringParser("MEDIUMTEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1492, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2024] StringParser("MEETS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1492, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2025] StringParser("MEMBER")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1492, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2026] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1492, child_count: 5, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2027] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1497, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2028] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1497, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2029] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1497, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2030] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1497, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2031] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1499, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2032] Ref(MergeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1499, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2033] Ref(IntoKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1499, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2034] StringParser("MERGE")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1499, child_count: 2, min_times: 1, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2035] AnyNumberOf(2 elements, min=1)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1501, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2036] Ref(MergeMatchedClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1501, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2037] Ref(MergeNotMatchedClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1501, child_count: 7, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2038] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1508, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2039] Ref(WhenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1508, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2040] Ref(MatchedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1508, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2041] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1510, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2042] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1510, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2043] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1510, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2044] Ref(ThenKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1510, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2045] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1512, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2046] Ref(MergeUpdateClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1512, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2047] Ref(MergeDeleteClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1512, child_count: 8, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2048] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1520, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2049] Ref(WhenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1520, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2050] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1520, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2051] Ref(MatchedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1520, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2052] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1522, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2053] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1522, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2054] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1522, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2055] Ref(ThenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1522, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2056] Ref(MergeInsertClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1522, child_count: 12, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2057] Sequence(12 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1534, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2058] Ref(MergeIntoLiteralGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1534, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2059] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1536, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2060] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1536, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2061] Ref(AliasedTableReferenceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1536, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2062] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1536, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2063] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1540, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2064] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1540, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2065] Ref(AliasedTableReferenceGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1540, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2066] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1542, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2067] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1545, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2068] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1545, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2069] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1545, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2070] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1547, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2071] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1547, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2072] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 1547, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2073] Meta(conditional: indented_using_on=True -> indent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1547, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2074] Ref(JoinOnConditionSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 1547, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2075] Meta(conditional: indented_using_on=True -> dedent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1547, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2076] Ref(MergeMatchSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1547, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2077] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1551, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2078] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1551, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2079] Ref(SetClauseListSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1551, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2080] StringParser("MERGEBLOCKRATIO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1551, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2081] StringParser("MESSAGE_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1551, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2082] StringParser("MESSAGE_OCTET_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1551, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2083] StringParser("MESSAGE_TEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1551, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2084] StringParser("METHOD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1551, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2085] StringParser("MIDDLEINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1551, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2086] StringParser("MILLISECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1551, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2087] StringParser("MIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1551, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2088] StringParser("MIN_ROWS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1551, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2089] StringParser("MINUS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1551, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2090] StringParser("-")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1551, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2091] StringParser("MINUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1551, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2092] StringParser("MINUTE_MICROSECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1551, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2093] StringParser("MINUTE_SECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1551, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2094] StringParser("MINVALUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1551, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2095] StringParser("ML")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1551, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2096] StringParser("MLSLABEL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1551, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2097] StringParser("MOD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1551, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2098] StringParser("MODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1551, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2099] StringParser("MODEL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1551, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2100] StringParser("MODIFIES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1551, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2101] StringParser("MODIFY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1551, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2102] StringParser("MODULE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1551, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2103] StringParser("%")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1551, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2104] StringParser("MONITOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1551, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2105] StringParser("MONTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1551, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2106] StringParser("MONTHNAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1551, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2107] StringParser("MORE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1551, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2108] StringParser("MOVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1551, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2109] StringParser("*")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1551, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2110] StringParser("MULTISET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1551, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2111] StringParser("MUMPS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1551, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2112] StringParser("MYISAM")
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1551, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2113] RegexParser
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1551, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2114] StringParser("NAME")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1551, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2115] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1554, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2116] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1554, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2117] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1554, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2118] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1556, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2119] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 1556, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2120] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1559, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2121] Ref(WindowSpecificationSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1559, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2122] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1563, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2123] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1563, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2124] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1565, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2125] Ref(NamedWindowExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1565, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2126] StringParser("NAMES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1565, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2127] StringParser("NAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1565, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2128] StringParser("NAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1565, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2129] StringParser("NATIONAL")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1565, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2130] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1567, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2131] Ref(NaturalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1567, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2132] Ref(JoinTypeKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1567, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2133] StringParser("NATURAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1567, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2134] StringParser("NCHAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1567, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2135] StringParser("NCLOB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1567, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2136] StringParser("-")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1567, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2137] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1567, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2138] StringParser("NESTING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1567, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2139] StringParser("NEW")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1567, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2140] Token("newline")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1567, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2141] StringParser("NEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1567, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2142] StringParser("NO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1567, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2143] StringParser("NO_WRITE_TO_BINLOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1567, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2144] StringParser("NOAUDIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1567, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2145] StringParser("NOCACHE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1567, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2146] StringParser("NOCHECK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1567, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2147] StringParser("NOCOMPRESS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1567, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2148] StringParser("NOCREATEDB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1567, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2149] StringParser("NOCREATEROLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1567, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2150] StringParser("NOCREATEUSER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1567, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2151] StringParser("NOCYCLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1567, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2152] StringParser("NOINHERIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1567, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2153] StringParser("NOLOGIN")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1567, child_count: 6, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2154] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1573, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2155] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1573, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2156] Ref(UnorderedSelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1573, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2157] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1576, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2158] Ref(SelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1576, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2159] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1579, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2160] Ref(WithCompoundStatementSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1579, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2161] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1582, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2162] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1582, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2163] Ref(BracketedSetExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1582, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2164] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1582, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2165] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1586, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2166] Ref(UpdateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1586, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2167] Ref(InsertStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1586, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2168] Ref(DeleteStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1586, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2169] Ref(MergeStatementSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1586, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2170] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1589, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2171] Ref(SetExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1589, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2172] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1591, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2173] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1594, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2174] Ref(SelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1594, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2175] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1594, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2176] StringParser("NONCLUSTERED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1594, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2177] StringParser("NONE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1594, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2178] StringParser("NOORDER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1594, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2179] StringParser("NORMALIZE")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1594, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2180] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1594, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2181] StringParser("NORMALIZED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1594, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2182] StringParser("NOSUPERUSER")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1594, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2183] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1594, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2184] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1596, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2185] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1598, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2186] Ref(RawNotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1598, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2187] Ref(RawEqualsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1598, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2188] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1600, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2189] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1600, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2190] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1600, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2191] StringParser("NE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1600, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2192] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1602, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2193] Ref(NotOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1602, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2194] Ref(RawEqualsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1602, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2195] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1604, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2196] Ref(BitwiseXorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1604, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2197] Ref(RawEqualsSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1604, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2198] StringParser("NOT")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1604, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2199] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1604, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2200] StringParser("NOT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1604, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2201] StringParser("NOTHING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1604, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2202] StringParser("NOTIFY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1604, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2203] StringParser("NOTNULL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1604, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2204] StringParser("NOWAIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1604, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2205] StringParser("NULL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1604, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2206] StringParser("NULL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1604, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2207] StringParser("NULLABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1604, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2208] StringParser("NULLIF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1604, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2209] StringParser("NULLS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1604, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2210] StringParser("NUMBER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1604, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2211] StringParser("NUMERIC")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1604, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2212] TypedParser("numeric_literal")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1604, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2213] StringParser("OBJECT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1604, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2214] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1607, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2215] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1607, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2216] Ref(ColonSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1607, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2217] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1607, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2218] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1610, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2219] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1612, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2220] Ref(ObjectLiteralElementSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1612, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2221] StringParser("{")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1612, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2222] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1614, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2223] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1614, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2224] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1616, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2225] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1616, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2226] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1616, child_count: 12, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2227] OneOf(12 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1628, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2228] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1628, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2229] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1628, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2230] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1628, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2231] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1628, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2232] Ref(CastOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1628, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2233] Ref(StartSquareBracketSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1628, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2234] Ref(StartBracketSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1628, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2235] Ref(BinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1628, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2236] Ref(ColonSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1628, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2237] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1628, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2238] Ref(JoinLikeClauseGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1628, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2239] StringParser("OBJECTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1628, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2240] StringParser("OCTET_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1628, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2241] StringParser("OCTETS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1628, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2242] StringParser("OF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1628, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2243] StringParser("OFF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1628, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2244] StringParser("OFFLINE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1628, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2245] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1631, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2246] Ref(OffsetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1631, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2247] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1633, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2248] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 1633, child_count: 1, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2249] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1634, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2250] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1634, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2251] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1636, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2252] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1636, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2253] Ref(RowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1636, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2254] StringParser("OFFSET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1636, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2255] StringParser("OFFSETS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1636, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2256] StringParser("OIDS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1636, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2257] StringParser("OLD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1636, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2258] StringParser("ON")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1636, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2259] StringParser("ONLINE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1636, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2260] StringParser("ONLY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1636, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2261] StringParser("OPEN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1636, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2262] StringParser("OPENDATASOURCE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1636, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2263] StringParser("OPENQUERY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1636, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2264] StringParser("OPENROWSET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1636, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2265] StringParser("OPENXML")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1636, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2266] StringParser("OPERATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1636, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2267] StringParser("OPERATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1636, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2268] StringParser("OPERATOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1636, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2269] StringParser("OPTIMIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1636, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2270] StringParser("OPTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1636, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2271] StringParser("OPTIONALLY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1636, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2272] StringParser("OPTIONS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1636, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2273] StringParser("OR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1636, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2274] StringParser("OR")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1636, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2275] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1638, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2276] Ref(OrKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1638, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2277] Ref(ReplaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1638, child_count: 5, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2278] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1643, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2279] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1643, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2280] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1643, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 2, _padding: 0 }, // [2281] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1645, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2282] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1649, child_count: 3, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2283] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1652, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2284] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1652, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2285] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1652, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2286] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1652, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2287] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1654, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2288] Ref(AscKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1654, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2289] Ref(DescKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1654, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2290] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1656, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2291] Ref(NullsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1656, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2292] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1658, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2293] Ref(FirstKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1658, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2294] Ref(LastKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1658, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2295] Ref(WithFillSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1658, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2296] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1658, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2297] Ref(FrameClauseUnitGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1658, child_count: 7, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2298] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1665, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2299] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1665, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2300] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1665, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2301] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1665, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2302] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1665, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2303] Ref(FrameClauseUnitGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1665, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2304] Ref(SeparatorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1665, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2305] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1665, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2306] StringParser("ORDER")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1665, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2307] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1667, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2308] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1667, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2309] Ref(NoorderKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1667, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2310] StringParser("ORDERING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1667, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2311] StringParser("ORDINALITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1667, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2312] StringParser("OTHERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1667, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2313] StringParser("OUT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1667, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2314] StringParser("OUTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1667, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2315] StringParser("OUTFILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1667, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2316] StringParser("OUTPUT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1667, child_count: 5, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2317] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1672, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2318] Ref(IgnoreRespectNullsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1672, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2319] Ref(OverKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1672, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2320] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1674, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2321] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 1674, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2322] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1677, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2323] Ref(WindowSpecificationSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1677, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2324] StringParser("OVER")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1677, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2325] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1679, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2326] Ref(OverlapsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1679, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2327] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1681, child_count: 1, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2328] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1682, child_count: 5, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2329] Bracketed(3 elements -> 3 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1687, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2330] Ref(DateTimeLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1687, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2331] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1687, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2332] Ref(DateTimeLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1687, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2333] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1687, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2334] StringParser("OVERLAPS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1687, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2335] StringParser("OVERLAPS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1687, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2336] StringParser("OVERLAY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1687, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2337] StringParser("OVERRIDE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1687, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2338] StringParser("OVERRIDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1687, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2339] StringParser("OVERWRITE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1687, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2340] StringParser("OWNER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1687, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2341] StringParser("OWNERSHIP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1687, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2342] StringParser("PACK_KEYS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1687, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2343] StringParser("PAD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1687, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2344] StringParser("PARAMETER")
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1687, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2345] RegexParser
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1687, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2346] StringParser("?")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1687, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2347] StringParser("PARAMETER_MODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1687, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2348] StringParser("PARAMETER_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1687, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2349] StringParser("PARAMETER_ORDINAL_POSITION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1687, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2350] StringParser("PARAMETER_SPECIFIC_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1687, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2351] StringParser("PARAMETER_SPECIFIC_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1687, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2352] StringParser("PARAMETER_SPECIFIC_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1687, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2353] StringParser("PARAMETERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1687, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2354] StringParser("PARTIAL")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1687, child_count: 5, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2355] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1692, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2356] Ref(PartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1692, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2357] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1692, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2358] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1694, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2359] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1697, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2360] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1699, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2361] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1699, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2362] StringParser("PARTITION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1699, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2363] StringParser("PASCAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1699, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2364] StringParser("PASSWORD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1699, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2365] StringParser("PATH")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1699, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2366] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1701, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2367] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1703, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2368] Ref(SlashSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1703, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2369] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1705, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2370] TypedParser("word")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1705, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2371] Ref(SlashSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1705, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2372] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1705, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2373] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1705, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2374] StringParser("PCTFREE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1705, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2375] StringParser("PERCENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1705, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2376] StringParser("PERCENT_RANK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1705, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2377] StringParser("PERCENTILE_CONT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1705, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2378] StringParser("PERCENTILE_DISC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1705, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2379] StringParser("PIPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1705, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2380] StringParser("|")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1705, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2381] StringParser("PLACING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1705, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2382] StringParser("PLAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1705, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2383] StringParser("PLI")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1705, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2384] StringParser("+")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1705, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2385] StringParser("POLICY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1705, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2386] StringParser("POSITION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1705, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2387] StringParser("+")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1705, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2388] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1707, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2389] Ref(OverClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1707, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2390] Ref(FilterClauseGrammar)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1707, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2391] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1707, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2392] StringParser("POSTFIX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1707, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2393] StringParser("POWER")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1707, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2394] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1707, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2395] StringParser("PRECEDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1707, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2396] StringParser("PRECISION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1707, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2397] StringParser("PREFIX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1707, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2398] StringParser("PREORDER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1707, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2399] StringParser("PREPARE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1707, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2400] StringParser("PREPARED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1707, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2401] StringParser("PRESERVE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1707, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2402] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1709, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2403] Ref(PrimaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1709, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2404] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1709, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2405] StringParser("PRIMARY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1709, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2406] StringParser("PRINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1709, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2407] StringParser("PRIOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1709, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2408] StringParser("PRIVILEGES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1709, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2409] StringParser("PROC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1709, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2410] StringParser("PROCEDURAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1709, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2411] StringParser("PROCEDURE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1709, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2412] StringParser("PROCEDURES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1709, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2413] StringParser("PROCESS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1709, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2414] StringParser("PROCESSLIST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1709, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2415] StringParser("PROFILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1709, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2416] StringParser("PROTECTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1709, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2417] StringParser("PUBLIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1709, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2418] StringParser("PURGE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1709, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2419] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1711, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2420] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1711, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2421] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1711, child_count: 4, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2422] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1715, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2423] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1715, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2424] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1717, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2425] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1720, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2426] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1720, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2427] StringParser("QUALIFY")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1720, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2428] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1723, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2429] Ref(ComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1723, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2430] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1726, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2431] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1726, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2432] Ref(AnyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1726, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2433] Ref(SomeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 1726, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2434] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1729, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2435] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1729, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2436] StringParser("QUARTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1729, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2437] StringParser("QUERY_BAND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1729, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2438] StringParser("QUIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1729, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2439] StringParser("QUOTE")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1729, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2440] TypedParser("double_quote")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1729, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2441] TypedParser("single_quote")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1729, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2442] StringParser("RAID0")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1729, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2443] StringParser("RAISERROR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1729, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2444] StringParser("RANGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1729, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2445] StringParser("RANK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1729, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2446] StringParser("=")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1729, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2447] StringParser(">")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1729, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2448] StringParser("RAW")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1729, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2449] StringParser("<")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1729, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2450] StringParser("!")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1729, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2451] StringParser("READ")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1729, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2452] StringParser("READS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1729, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2453] StringParser("READTEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1729, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2454] StringParser("REAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1729, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2455] StringParser("RECHECK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1729, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2456] StringParser("RECONFIGURE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1729, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2457] StringParser("RECURSIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1729, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2458] StringParser("REF")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1729, child_count: 5, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2459] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1734, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2460] Ref(ReferencesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1734, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2461] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1734, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2462] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1734, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2463] Ref(ReferenceMatchGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1734, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2464] AnySetOf(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1736, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2465] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1739, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2466] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1739, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2467] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1739, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2468] Ref(ReferentialActionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1739, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2469] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1742, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2470] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1742, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2471] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1742, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2472] Ref(ReferentialActionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1742, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2473] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1744, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2474] Ref(MatchKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1744, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2475] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1747, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2476] Ref(FullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1747, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2477] Ref(PartialKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1747, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2478] Ref(SimpleKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1747, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2479] StringParser("REFERENCE_USAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1747, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2480] StringParser("REFERENCES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1747, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2481] StringParser("REFERENCING")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1747, child_count: 5, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2482] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1752, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2483] Ref(RestrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1752, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2484] Ref(CascadeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1752, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2485] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1754, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2486] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1754, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2487] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1754, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2488] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1756, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2489] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1756, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2490] Ref(ActionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1756, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2491] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1758, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2492] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1758, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2493] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1758, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2494] StringParser("REGEXP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1758, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2495] StringParser("REGR_AVGX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1758, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2496] StringParser("REGR_AVGY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1758, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2497] StringParser("REGR_COUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1758, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2498] StringParser("REGR_INTERCEPT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1758, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2499] StringParser("REGR_R2")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1758, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2500] StringParser("REGR_SLOPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1758, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2501] StringParser("REGR_SXX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1758, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2502] StringParser("REGR_SXY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1758, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2503] StringParser("REGR_SYY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1758, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2504] StringParser("REINDEX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1758, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2505] StringParser("RELATIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1758, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2506] StringParser("RELEASE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1758, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2507] StringParser("RELOAD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1758, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2508] StringParser("RENAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1758, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2509] StringParser("REPEAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1758, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2510] StringParser("REPEATABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1758, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2511] StringParser("REPLACE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1758, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2512] StringParser("REPLICATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1758, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2513] StringParser("REQUIRE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1758, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2514] StringParser("RESET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1758, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2515] StringParser("RESIGNAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1758, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2516] StringParser("RESOURCE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1758, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2517] StringParser("RESPECT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1758, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2518] StringParser("RESTART")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1758, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2519] StringParser("RESTORE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1758, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2520] StringParser("RESTRICT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1758, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2521] StringParser("RESULT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1758, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2522] StringParser("RETURN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1758, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2523] StringParser("RETURNED_CARDINALITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1758, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2524] StringParser("RETURNED_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1758, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2525] StringParser("RETURNED_OCTET_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1758, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2526] StringParser("RETURNED_SQLSTATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1758, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2527] StringParser("RETURNS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1758, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2528] StringParser("REVOKE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1758, child_count: 6, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2529] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1764, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2530] Ref(RevokeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1764, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2531] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1767, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2532] Ref(GrantKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1767, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2533] Ref(OptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1767, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2534] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1767, child_count: 4, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2535] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1771, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2536] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1774, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2537] Ref(AccessPermissionsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1774, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2538] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1774, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2539] Ref(AccessObjectSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1774, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2540] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1776, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2541] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1776, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2542] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1776, child_count: 4, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2543] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1780, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2544] Ref(OwnershipKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1780, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2545] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1780, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2546] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1780, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2547] Ref(UserReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1780, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2548] Ref(UserReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1780, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2549] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1780, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2550] Ref(AccessTargetSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1780, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2551] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1780, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2552] StringParser("RIGHT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1780, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2553] StringParser("RLIKE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1780, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2554] StringParser("ROLE")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1780, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2555] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1780, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2556] StringParser("ROLES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1780, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2557] StringParser("ROLLBACK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1780, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2558] StringParser("ROLLUP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1780, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2559] StringParser("ROLLUP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1780, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2560] StringParser("ROUTINE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1780, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2561] StringParser("ROUTINE_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1780, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2562] StringParser("ROUTINE_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1780, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2563] StringParser("ROUTINE_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1780, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2564] StringParser("ROUTINES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1780, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2565] StringParser("ROW")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1780, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2566] StringParser("ROW_COUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1780, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2567] StringParser("ROW_NUMBER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1780, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2568] StringParser("ROWCOUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1780, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2569] StringParser("ROWGUIDCOL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1780, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2570] StringParser("ROWID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1780, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2571] StringParser("ROWNUM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1780, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2572] StringParser("ROWS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1780, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2573] StringParser("RULE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1780, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2574] StringParser("RUN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1780, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2575] StringParser("SAMPLE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1780, child_count: 4, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2576] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1784, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2577] Ref(TablesampleKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1784, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2578] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1786, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2579] Ref(BernoulliKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1786, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2580] Ref(SystemKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1786, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2581] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1789, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2582] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1789, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2583] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1791, child_count: 1, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2584] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1792, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2585] Ref(RepeatableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1792, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2586] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1795, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2587] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1795, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2588] StringParser("SAVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1795, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2589] StringParser("SAVEPOINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1795, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2590] StringParser("SCALE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1795, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2591] StringParser("SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1795, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2592] StringParser("SCHEMA_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1795, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2593] StringParser("SCHEMAS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1795, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2594] StringParser("SCOPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1795, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2595] StringParser("SCOPE_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1795, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2596] StringParser("SCOPE_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1795, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2597] StringParser("SCOPE_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1795, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2598] StringParser("SCROLL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1795, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2599] StringParser("SEARCH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1795, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2600] StringParser("SECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1795, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2601] StringParser("SECOND_MICROSECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1795, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2602] StringParser("SECTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1795, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2603] StringParser("SECURITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1795, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2604] StringParser("SEL")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1795, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2605] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1797, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2606] Ref(WildcardExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1797, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2607] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1799, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2608] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1799, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2609] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1799, child_count: 4, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2610] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1803, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2611] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1803, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2612] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1803, child_count: 4, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2613] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1807, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2614] Ref(TopKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1807, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2615] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1807, child_count: 1, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2616] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1808, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2617] Ref(PercentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1808, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2618] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1810, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2619] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1810, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2620] Ref(TiesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1810, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2621] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1812, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2622] Ref(NormalizeKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1812, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2623] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1815, child_count: 4, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2624] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1819, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2625] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1819, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2626] Ref(MeetsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1819, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2627] Ref(OrKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1819, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2628] Ref(OverlapsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1819, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2629] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1821, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2630] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1821, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2631] Ref(OverlapsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1821, child_count: 4, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2632] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1825, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2633] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1825, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2634] Ref(OverlapsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1825, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2635] Ref(OrKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1825, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2636] Ref(MeetsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::GreedyOnceStarted, flags: GrammarFlags::from_bits(4), first_child_idx: 1825, child_count: 5, min_times: 0, first_terminator_idx: 31, terminator_count: 5, _padding: 0 }, // [2637] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1830, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2638] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1832, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2639] Ref(SelectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1832, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2640] Ref(SelKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1832, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2641] Ref(SelectClauseModifierSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(12), first_child_idx: 1832, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2642] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1834, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2643] Ref(SelectClauseElementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1834, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2644] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1834, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2645] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1834, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2646] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1836, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2647] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1836, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2648] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1836, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2649] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1836, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [2650] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1836, child_count: 7, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2651] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 1843, child_count: 1, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2652] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::PrecededBy, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1844, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2653] PrecededBy(preceding_sequences=(('IS', 'DISTINCT'), ('IS', 'NOT', 'DISTINCT')))
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1844, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2654] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1844, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2655] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1846, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2656] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1846, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2657] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1846, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2658] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1846, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2659] Ref(OverlapsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1846, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2660] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1846, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2661] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1846, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2662] StringParser("SELECT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::GreedyOnceStarted, flags: GrammarFlags::from_bits(4), first_child_idx: 1846, child_count: 14, min_times: 0, first_terminator_idx: 36, terminator_count: 3, _padding: 0 }, // [2663] Sequence(14 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1860, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2664] Ref(LockingClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1860, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2665] Ref(SelectClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1860, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2666] Ref(FromClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1860, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2667] Ref(WhereClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1860, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2668] Ref(GroupByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1860, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2669] Ref(HavingClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1860, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2670] Ref(OverlapsClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1860, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2671] Ref(NamedWindowSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1860, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2672] Ref(QualifyClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1860, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2673] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1860, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2674] Ref(OffsetClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1860, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2675] Ref(FetchClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1860, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2676] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1860, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2677] Ref(NamedWindowSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1860, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2678] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1860, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2679] Ref(WithNoSchemaBindingClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1860, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2680] Ref(WithDataClauseSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1860, child_count: 4, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2681] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1864, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2682] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1866, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2683] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1869, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2684] Ref(WithCompoundStatementSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1869, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2685] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1871, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2686] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1874, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2687] Ref(WithCompoundNonSelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1874, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2688] Ref(NonWithSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1874, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2689] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1877, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2690] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1877, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2691] StringParser("SELF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1877, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2692] StringParser(";")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1877, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2693] StringParser("SENSITIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1877, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2694] StringParser("SEPARATOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1877, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2695] StringParser("SEQUENCE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1877, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2696] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1879, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2697] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1881, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2698] Ref(MaxvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1881, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2699] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1881, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2700] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1883, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2701] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1883, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2702] Ref(MaxvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1883, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2703] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1885, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2704] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1887, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2705] Ref(MinvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1887, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2706] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1887, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2707] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1889, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2708] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1889, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2709] Ref(MinvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1889, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2710] StringParser("SEQUENCES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1889, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2711] StringParser("SERIALIZABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1889, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2712] StringParser("SERVER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1889, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2713] StringParser("SERVER_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1889, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2714] StringParser("SESSION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1889, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2715] StringParser("SESSION_USER")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1889, child_count: 4, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2716] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1893, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2717] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1893, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2718] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1895, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2719] Ref(SetClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1895, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2720] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1898, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2721] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1898, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2722] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1898, child_count: 7, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2723] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1905, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2724] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1905, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2725] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1905, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2726] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1905, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2727] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1905, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2728] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1905, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2729] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1905, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2730] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1905, child_count: 5, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2731] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1910, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2732] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1910, child_count: 1, min_times: 1, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2733] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1911, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2734] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1913, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2735] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1913, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2736] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1913, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2737] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1913, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2738] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1913, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2739] Ref(NamedWindowSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1913, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2740] StringParser("SET")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1913, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2741] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1916, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2742] Ref(UnionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1916, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2743] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1918, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2744] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1920, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2745] Ref(IntersectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1920, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2746] Ref(ExceptKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1920, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2747] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1920, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2748] Ref(MinusKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1920, child_count: 7, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2749] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1927, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2750] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1927, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2751] Ref(Query_bandKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1927, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2752] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1927, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2753] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1929, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2754] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1929, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2755] Ref(NoneKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1929, child_count: 1, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2756] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1930, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2757] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1930, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2758] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1930, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2759] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1932, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2760] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1934, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2761] Ref(SessionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1934, child_count: 1, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2762] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1935, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2763] Ref(VolatileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1935, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2764] Ref(TransactionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1935, child_count: 4, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2765] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1939, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2766] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1939, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2767] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1939, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2768] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1939, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2769] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1939, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2770] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1941, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2771] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1943, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2772] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1945, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2773] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1945, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2774] Ref(SessionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1945, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2775] Ref(SsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1945, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2776] Ref(DatabaseStatementSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1945, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2777] StringParser("SETOF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1945, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2778] StringParser("SETS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1945, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2779] StringParser("SETUSER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1945, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2780] StringParser("SHARE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1945, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2781] StringParser("SHARES")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1945, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2782] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1947, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2783] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1949, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2784] Ref(Expression_D_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1949, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2785] Ref(CaseExpressionSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1949, child_count: 1, min_times: 1, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2786] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1950, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2787] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1953, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2788] Ref(CastOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1953, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2789] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1953, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2790] Ref(TimeZoneGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1953, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2791] StringParser("SHOW")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1953, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2792] StringParser("SHUTDOWN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1953, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2793] StringParser("SIGNAL")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1953, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2794] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1955, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2795] Ref(PositiveSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1955, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2796] Ref(NegativeSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1955, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2797] StringParser("SIMILAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1955, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2798] StringParser("SIMPLE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1955, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 1, _padding: 0 }, // [2799] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1957, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2800] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1957, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2801] Ref(QuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1957, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [2802] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1957, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2803] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1959, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2804] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1959, child_count: 3, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2805] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1962, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2806] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1962, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2807] Ref(ObjectReferenceDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1962, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2808] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1962, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2809] TypedParser("single_quote")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1962, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2810] StringParser("SIZE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1962, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2811] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1964, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2812] Ref(ArrayTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1964, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2813] Ref(ArrayAccessorSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1964, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2814] StringParser("/")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1964, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2815] StringParser(":")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1964, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2816] StringParser("SMALLINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1964, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2817] StringParser("SOME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1964, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2818] StringParser("SONAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1964, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2819] StringParser("SOURCE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1964, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2820] StringParser("SPACE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1964, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2821] StringParser("SPATIAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1964, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2822] StringParser("SPECIFIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1964, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2823] StringParser("SPECIFIC_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1964, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2824] StringParser("SPECIFICTYPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1964, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2825] StringParser("SQL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1964, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2826] StringParser("SQL_BIG_RESULT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1964, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2827] StringParser("SQL_BIG_SELECTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1964, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2828] StringParser("SQL_BIG_TABLES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1964, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2829] StringParser("SQL_CALC_FOUND_ROWS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1964, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2830] StringParser("SQL_LOG_OFF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1964, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2831] StringParser("SQL_LOG_UPDATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1964, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2832] StringParser("SQL_LOW_PRIORITY_UPDATES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1964, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2833] StringParser("SQL_SELECT_LIMIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1964, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2834] StringParser("SQL_SMALL_RESULT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1964, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2835] StringParser("SQL_WARNINGS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1964, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2836] StringParser("SQLCA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1964, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2837] StringParser("SQLCODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1964, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2838] StringParser("SQLERROR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1964, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2839] StringParser("SQLEXCEPTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1964, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2840] StringParser("SQLSTATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1964, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2841] StringParser("SQLWARNING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1964, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2842] StringParser("SQRT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1964, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2843] StringParser("SS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1964, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2844] StringParser("SSL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1964, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2845] StringParser("STABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1964, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2846] StringParser("STAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1964, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2847] StringParser("STAGES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1964, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2848] StringParser("*")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1964, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2849] StringParser("START")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1964, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2850] StringParser("STARTING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1964, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2851] StringParser("STARTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1964, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2852] StringParser("STAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1964, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2853] StringParser("STATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1964, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2854] StringParser("STATEMENT")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1964, child_count: 47, min_times: 0, first_terminator_idx: 40, terminator_count: 1, _padding: 0 }, // [2855] OneOf(47 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2856] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2857] Ref(MergeStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2858] Ref(InsertStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2859] Ref(TransactionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2860] Ref(DropTableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2861] Ref(DropViewStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2862] Ref(CreateUserStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2863] Ref(DropUserStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2864] Ref(TruncateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2865] Ref(AccessStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2866] Ref(CreateTableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2867] Ref(CreateRoleStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2868] Ref(DropRoleStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2869] Ref(AlterTableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2870] Ref(CreateSchemaStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2871] Ref(SetSchemaStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2872] Ref(DropSchemaStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2873] Ref(DropTypeStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2874] Ref(CreateDatabaseStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2875] Ref(DropDatabaseStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2876] Ref(CreateIndexStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2877] Ref(DropIndexStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2878] Ref(CreateViewStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2879] Ref(DeleteStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2880] Ref(UpdateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2881] Ref(CreateCastStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2882] Ref(DropCastStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2883] Ref(CreateFunctionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2884] Ref(DropFunctionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2885] Ref(CreateModelStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2886] Ref(DropModelStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2887] Ref(DescribeStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2888] Ref(UseStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2889] Ref(ExplainStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2890] Ref(CreateSequenceStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2891] Ref(AlterSequenceStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2892] Ref(DropSequenceStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2893] Ref(CreateTriggerStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2894] Ref(DropTriggerStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2895] Ref(TdCollectStatisticsStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2896] Ref(BteqStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2897] Ref(TdRenameStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2898] Ref(QualifyClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2899] Ref(TdCommentStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2900] Ref(DatabaseStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2901] Ref(SetSessionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2902] Ref(SetQueryBandStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [2903] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [2904] StringParser("STATIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [2905] StringParser("STATISTICS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [2906] StringParser("STATS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [2907] StringParser("STDDEV_POP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [2908] StringParser("STDDEV_SAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [2909] StringParser("STDIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [2910] StringParser("STDOUT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [2911] StringParser("STORAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [2912] StringParser("STRAIGHT_JOIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [2913] StringParser("STREAM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [2914] StringParser("STREAMS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2011, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [2915] StringParser("STRICT")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2011, child_count: 1, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [2916] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2012, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [2917] Ref(ConcatSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2012, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [2918] StringParser("STRING")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2012, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [2919] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2015, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [2920] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2017, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [2921] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2019, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [2922] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2019, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [2923] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2019, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [2924] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2019, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [2925] StringParser("STRUCTURE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2019, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [2926] StringParser("STYLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2019, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [2927] StringParser("SUBCLASS_ORIGIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2019, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [2928] StringParser("SUBLIST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2019, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [2929] StringParser("SUBMULTISET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2019, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [2930] StringParser("SUBSTRING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2019, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [2931] StringParser("SUCCESSFUL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2019, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [2932] StringParser("SUM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2019, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [2933] StringParser("SUMMARY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2019, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [2934] StringParser("SUPERUSER")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2019, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [2935] Token("symbol")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2019, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [2936] StringParser("SYMMETRIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2019, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [2937] StringParser("SYNONYM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2019, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [2938] StringParser("SYSDATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2019, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [2939] StringParser("SYSID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2019, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [2940] StringParser("SYSTEM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2019, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [2941] StringParser("SYSTEM_USER")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2019, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [2942] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2021, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [2943] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2023, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [2944] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2023, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [2945] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2023, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [2946] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2026, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [2947] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2028, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [2948] Ref(UniqueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2028, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [2949] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2028, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [2950] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2030, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [2951] Ref(PrimaryKeyGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2030, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [2952] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2030, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [2953] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2033, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [2954] Ref(ForeignKeyGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2033, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [2955] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2033, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [2956] Ref(ReferenceDefinitionGrammar)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2033, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [2957] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2033, child_count: 6, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [2958] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2039, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [2959] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2039, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [2960] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2039, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [2961] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2039, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [2962] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2039, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [2963] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2042, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [2964] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2042, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [2965] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2045, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [2966] Ref(MergeStatementSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2045, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [2967] StringParser("TABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2045, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [2968] StringParser("TABLE_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2045, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [2969] StringParser("TABLES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2045, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [2970] StringParser("TABLESAMPLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2045, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [2971] StringParser("TABLESPACE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2045, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2972] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2047, child_count: 1, min_times: 0, first_terminator_idx: 41, terminator_count: 1, _padding: 0 }, // [2973] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2048, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [2974] Ref(Expression_A_Unary_Operator_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2048, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [2975] Ref(BinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2048, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2976] Ref(Expression_C_Grammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2048, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2977] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2050, child_count: 1, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2978] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2051, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2979] Ref(Expression_B_Unary_Operator_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2051, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2980] Ref(Expression_C_Grammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2051, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2981] StringParser("TASK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2051, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2982] StringParser("TASKS")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2051, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2983] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2053, child_count: 11, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2984] OneOf(11 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2064, child_count: 3, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2985] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2067, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2986] Ref(SampleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2067, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2987] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2067, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2988] Ref(PercentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2067, child_count: 3, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2989] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2070, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2990] Ref(SystemKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2070, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2991] Ref(ThresholdKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2070, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2992] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2072, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2993] Ref(PercentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2072, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2994] Ref(DaysKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2072, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2995] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2074, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2996] Ref(SystemKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2074, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2997] Ref(SampleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2074, child_count: 3, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2998] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2077, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [2999] Ref(ThresholdKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2077, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3000] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2077, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3001] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2079, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3002] Ref(PercentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2079, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3003] Ref(DaysKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2079, child_count: 3, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3004] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2082, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3005] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2082, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3006] Ref(ThresholdKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2082, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3007] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2084, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3008] Ref(PercentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2084, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3009] Ref(DaysKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2084, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3010] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2086, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3011] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2086, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3012] Ref(SampleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2086, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3013] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2088, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3014] Ref(MaxintervalsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2088, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3015] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2088, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3016] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2090, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3017] Ref(SystemKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2090, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3018] Ref(MaxintervalsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2090, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3019] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2092, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3020] Ref(MaxvaluelengthKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2092, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3021] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2092, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3022] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2094, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3023] Ref(SystemKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2094, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3024] Ref(MaxvaluelengthKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2094, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3025] Ref(SampleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2094, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3026] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2096, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3027] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2096, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3028] Ref(CurrentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2096, child_count: 9, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3029] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2105, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3030] Ref(CollectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2105, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3031] Ref(SummaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2105, child_count: 3, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3032] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2108, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3033] Ref(StatKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2108, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3034] Ref(StatsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2108, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3035] Ref(StatisticsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2108, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3036] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2110, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3037] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2110, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3038] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2112, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3039] Ref(TdCollectStatUsingOptionClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2112, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3040] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2112, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3041] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2114, child_count: 3, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3042] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2117, child_count: 6, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3043] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2123, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3044] Ref(UniqueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2123, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3045] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2123, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3046] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2123, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3047] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2123, child_count: 3, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3048] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2126, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3049] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2128, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3050] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2128, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3051] Ref(TdOrderByStatClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2128, child_count: 3, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3052] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2131, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3053] Ref(UniqueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2131, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3054] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2131, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3055] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2131, child_count: 3, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3056] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2134, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3057] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2134, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3058] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2136, child_count: 3, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3059] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2139, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3060] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2141, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3061] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2143, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3062] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2143, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3063] Ref(PartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2143, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3064] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2145, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3065] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2145, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3066] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2145, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3067] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2145, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3068] Ref(TemporaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2145, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3069] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2145, child_count: 3, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3070] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2148, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3071] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2148, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3072] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2150, child_count: 3, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3073] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2153, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3074] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2155, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3075] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2157, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3076] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2157, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3077] Ref(PartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2157, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3078] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2159, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3079] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2159, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3080] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2159, child_count: 1, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3081] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2160, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3082] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2162, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3083] Ref(CompressKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2162, child_count: 3, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3084] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2165, child_count: 3, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3085] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2168, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3086] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2170, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3087] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2170, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3088] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2170, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3089] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2170, child_count: 4, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3090] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2174, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3091] Ref(CommentKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2174, child_count: 1, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3092] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2175, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3093] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2175, child_count: 15, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3094] OneOf(15 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2190, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3095] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2192, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3096] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2192, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3097] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2192, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3098] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2194, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3099] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2194, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3100] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2194, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3101] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2196, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3102] Ref(MacroKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2196, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3103] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2196, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3104] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2198, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3105] Ref(MapKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2198, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3106] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2198, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3107] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2200, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3108] Ref(MethodKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2200, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3109] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2200, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3110] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2202, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3111] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2202, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3112] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2202, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3113] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2204, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3114] Ref(ProfileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2204, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3115] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2204, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3116] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2206, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3117] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2206, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3118] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2206, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3119] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2208, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3120] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2208, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3121] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2208, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3122] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2210, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3123] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2210, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3124] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2210, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3125] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2212, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3126] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2212, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3127] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2212, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3128] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2214, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3129] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2214, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3130] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2214, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3131] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2216, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3132] Ref(FileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2216, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3133] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2216, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3134] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2218, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3135] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2218, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3136] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2218, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3137] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2220, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3138] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2220, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3139] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2220, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3140] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2222, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3141] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2224, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3142] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2224, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3143] Ref(IsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2224, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3144] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2224, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3145] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2226, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3146] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2226, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3147] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2228, child_count: 6, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3148] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2234, child_count: 3, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3149] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2237, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3150] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2237, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3151] Ref(FallbackKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2237, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3152] Ref(ProtectionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2237, child_count: 3, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3153] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2240, child_count: 4, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3154] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2244, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3155] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2244, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3156] Ref(DualKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2244, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3157] Ref(LocalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2244, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3158] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2246, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3159] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2246, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3160] Ref(LocalKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2246, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3161] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2248, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3162] Ref(BeforeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2248, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3163] Ref(AfterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2248, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3164] Ref(JournalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2248, child_count: 3, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3165] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2251, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3166] Ref(ChecksumKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2251, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3167] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2251, child_count: 3, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3168] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2254, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3169] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2254, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3170] Ref(OffKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2254, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3171] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2254, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3172] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2256, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3173] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2258, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3174] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2258, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3175] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2258, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3176] Ref(MergeblockratioKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2258, child_count: 4, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3177] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2262, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3178] Ref(MergeblockratioKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2262, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3179] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2262, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3180] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2262, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3181] Ref(PercentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2262, child_count: 3, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3182] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2265, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3183] Ref(MapKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2265, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3184] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2265, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3185] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2265, child_count: 4, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3186] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2269, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3187] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2269, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3188] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2269, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3189] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2271, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3190] Ref(ValuesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2271, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3191] Ref(HashKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2271, child_count: 3, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3192] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2274, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3193] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2274, child_count: 5, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3194] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2279, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3195] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2279, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3196] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2279, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3197] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2279, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3198] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2281, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3199] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2281, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3200] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2281, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3201] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2281, child_count: 6, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3202] AnyNumberOf(6 elements, min=0)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2287, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3203] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2289, child_count: 5, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3204] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2294, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3205] Ref(UniqueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2294, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3206] Ref(PrimaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2294, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3207] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2294, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3208] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2294, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3209] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2296, child_count: 3, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3210] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2299, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3211] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2301, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3212] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2301, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3213] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2301, child_count: 3, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3214] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2304, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3215] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2304, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3216] Ref(PrimaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2304, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3217] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2304, child_count: 3, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3218] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2307, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3219] Ref(PartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2307, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3220] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2307, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3221] Ref(TdTablePartitioningLevel)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2307, child_count: 5, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3222] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2312, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3223] Ref(UniqueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2312, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3224] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2312, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3225] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2312, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3226] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2312, child_count: 3, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3227] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2315, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3228] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2317, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3229] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2317, child_count: 3, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3230] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2320, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3231] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2320, child_count: 1, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3232] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2321, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3233] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2321, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3234] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2321, child_count: 3, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3235] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2324, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3236] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2324, child_count: 1, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3237] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2325, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3238] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2325, child_count: 3, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3239] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2328, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3240] Ref(StatKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2328, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3241] Ref(StatsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2328, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3242] Ref(StatisticsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2328, child_count: 4, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3243] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2332, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3244] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2332, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3245] Ref(CommitKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2332, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3246] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2334, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3247] Ref(PreserveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2334, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3248] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2334, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3249] Ref(RowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2334, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3250] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2336, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3251] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2338, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3252] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2338, child_count: 3, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3253] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2341, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3254] Anything
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2341, child_count: 3, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3255] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2344, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3256] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2346, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3257] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2348, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3258] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2348, child_count: 3, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3259] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2351, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3260] Anything
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2351, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3261] StringParser("TEMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2351, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3262] StringParser("TEMPLATE")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2351, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3263] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2351, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3264] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2353, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3265] Ref(TempKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2353, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3266] Ref(TemporaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2353, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3267] StringParser("TEMPORARY")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2353, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3268] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2355, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3269] Ref(TransientKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2355, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3270] Ref(TemporaryGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2355, child_count: 4, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3271] Bracketed(2 elements -> 2 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2359, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3272] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2359, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3273] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2361, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3274] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2361, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3275] Ref(CharCharacterSetGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2361, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3276] StringParser("TERMINATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2361, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3277] StringParser("TERMINATED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2361, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3278] StringParser("TEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2361, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3279] StringParser("TEXTSIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2361, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3280] StringParser("THAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2361, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3281] StringParser("THEN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2361, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3282] StringParser("THRESHOLD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2361, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3283] StringParser("TIES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2361, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3284] StringParser("~")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2361, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3285] StringParser("TIME")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2361, child_count: 3, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3286] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2364, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3287] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2366, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3288] Ref(TimeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2366, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3289] Ref(TimestampKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2366, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3290] Ref(BracketedArguments)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2366, child_count: 3, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3291] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2369, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3292] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2371, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3293] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2371, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3294] Ref(WithoutKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2371, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3295] Ref(TimeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2371, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3296] Ref(ZoneKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2371, child_count: 1, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3297] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2372, child_count: 4, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3298] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2376, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3299] Ref(AtKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2376, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3300] Ref(TimeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2376, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3301] Ref(ZoneKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2376, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3302] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2376, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3303] StringParser("TIMESTAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2376, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3304] StringParser("TIMEZONE_HOUR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2376, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3305] StringParser("TIMEZONE_MINUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2376, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3306] StringParser("TINYBLOB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2376, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3307] StringParser("TINYINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2376, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3308] StringParser("TINYTEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2376, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3309] StringParser("TO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2376, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3310] StringParser("TOAST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2376, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3311] StringParser("TOP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2376, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3312] StringParser("TOP_LEVEL_COUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2376, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3313] StringParser("TRAILING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2376, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3314] StringParser("TRAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2376, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3315] StringParser("TRANSACTION")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2376, child_count: 4, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3316] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2380, child_count: 5, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3317] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2385, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3318] Ref(StartKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2385, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3319] Ref(BeginKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2385, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3320] Ref(CommitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2385, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3321] Ref(RollbackKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2385, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3322] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2385, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3323] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2387, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3324] Ref(TransactionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2387, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3325] Ref(WorkKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2387, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3326] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2389, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3327] Ref(NameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2389, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3328] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2389, child_count: 3, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3329] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2392, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3330] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2392, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3331] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2392, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3332] Ref(ChainKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2392, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3333] StringParser("TRANSACTION_ACTIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2392, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3334] StringParser("TRANSACTIONS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2392, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3335] StringParser("TRANSACTIONS_COMMITTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2392, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3336] StringParser("TRANSACTIONS_ROLLED_BACK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2392, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3337] StringParser("TRANSFORM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2392, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3338] StringParser("TRANSFORMS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2392, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3339] StringParser("TRANSIENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2392, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3340] StringParser("TRANSLATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2392, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3341] StringParser("TRANSLATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2392, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3342] StringParser("TREAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2392, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3343] StringParser("TRIGGER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2392, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3344] StringParser("TRIGGER_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2392, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3345] StringParser("TRIGGER_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2392, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3346] StringParser("TRIGGER_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2392, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3347] StringParser("TRIM")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2392, child_count: 3, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3348] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2395, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3349] Ref(BothKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2395, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3350] Ref(LeadingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2395, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3351] Ref(TrailingKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2395, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3352] StringParser("TRUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2395, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3353] StringParser("TRUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2395, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3354] StringParser("TRUNCATE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2395, child_count: 3, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3355] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2398, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3356] Ref(TruncateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2398, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3357] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2398, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3358] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2398, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3359] StringParser("TRUSTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2398, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3360] StringParser("TSEQUAL")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2398, child_count: 3, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3361] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2401, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3362] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2403, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3363] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2403, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3364] StringParser("TYPE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2403, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3365] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2405, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3366] Ref(ArrayTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2405, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3367] Ref(ArrayLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2405, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3368] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2407, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3369] Ref(StructTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2407, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3370] Ref(StructLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2407, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3371] StringParser("UC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2407, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3372] StringParser("UESCAPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2407, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3373] StringParser("UID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2407, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3374] StringParser("UNBOUNDED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2407, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3375] StringParser("UNCOMMITTED")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2407, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3376] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2407, child_count: 3, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3377] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2410, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3378] Ref(NaturalJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2410, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3379] Ref(UnconditionalCrossJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2410, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3380] Ref(HorizontalJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2410, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3381] StringParser("UNDER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2410, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3382] StringParser("UNDO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2410, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3383] StringParser("UNENCRYPTED")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2410, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3384] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2412, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3385] Ref(UnionKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2412, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3386] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2414, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3387] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2414, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3388] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2414, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3389] StringParser("UNION")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2414, child_count: 1, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3390] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2415, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3391] Ref(UniqueKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2415, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3392] StringParser("UNIQUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2415, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3393] StringParser("UNKNOWN")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2415, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3394] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2415, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3395] StringParser("UNLISTEN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2415, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3396] StringParser("UNLOCK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2415, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3397] StringParser("UNNAMED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2415, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3398] StringParser("UNNEST")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::GreedyOnceStarted, flags: GrammarFlags::from_bits(4), first_child_idx: 2415, child_count: 8, min_times: 0, first_terminator_idx: 42, terminator_count: 5, _padding: 0 }, // [3399] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2423, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3400] Ref(SelectClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2423, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3401] Ref(FromClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2423, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3402] Ref(WhereClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2423, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3403] Ref(GroupByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2423, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3404] Ref(HavingClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2423, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3405] Ref(QualifyClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2423, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3406] Ref(OverlapsClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2423, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3407] Ref(NamedWindowSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2423, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3408] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2423, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3409] Ref(WithNoSchemaBindingClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2423, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3410] Ref(WithDataClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2423, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3411] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2423, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3412] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2423, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3413] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2425, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3414] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2425, child_count: 1, min_times: 1, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3415] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2426, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3416] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2428, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3417] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2428, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3418] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2428, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3419] StringParser("UNSIGNED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2428, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3420] StringParser("UNTIL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2428, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3421] StringParser("UPDATE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2428, child_count: 6, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3422] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2434, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3423] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2434, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3424] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2437, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3425] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2437, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3426] Ref(FromUpdateClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2437, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3427] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2439, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3428] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2439, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3429] Ref(FromUpdateClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2439, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3430] Ref(SetClauseListSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2439, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3431] Ref(WhereClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2439, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3432] StringParser("UPDATETEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2439, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3433] StringParser("UPPER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2439, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3434] StringParser("UPPERCASE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2439, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3435] StringParser("USAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2439, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3436] StringParser("USE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2439, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3437] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2441, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3438] Ref(UseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2441, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3439] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2441, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3440] StringParser("USE_ANY_ROLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2441, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3441] StringParser("USER")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2441, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3442] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2441, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3443] StringParser("USER_DEFINED_TYPE_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2441, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3444] StringParser("USER_DEFINED_TYPE_CODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2441, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3445] StringParser("USER_DEFINED_TYPE_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2441, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3446] StringParser("USER_DEFINED_TYPE_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2441, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3447] StringParser("USERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2441, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3448] StringParser("USING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2441, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3449] StringParser("UTC_DATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2441, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3450] StringParser("UTC_TIME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2441, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3451] StringParser("UTC_TIMESTAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2441, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3452] StringParser("VACUUM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2441, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3453] StringParser("VALID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2441, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3454] StringParser("VALIDATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2441, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3455] StringParser("VALIDATOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2441, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3456] StringParser("VALUE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2441, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3457] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2443, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3458] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2445, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3459] Ref(ValueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2445, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3460] Ref(ValuesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2445, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3461] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2447, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3462] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2449, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3463] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 2449, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3464] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2452, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3465] Delimited(3 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2454, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3466] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2457, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3467] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2457, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3468] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2457, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3469] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2457, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3470] StringParser("VALUES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2457, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3471] StringParser("VAR_POP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2457, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3472] StringParser("VAR_SAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2457, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3473] StringParser("VARBINARY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2457, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3474] StringParser("VARCHAR2")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2457, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3475] StringParser("VARCHAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2457, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3476] StringParser("VARCHARACTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2457, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3477] StringParser("VARIABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2457, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3478] StringParser("VARIABLES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2457, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3479] StringParser("VARYING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2457, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3480] StringParser("VERBOSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2457, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3481] StringParser("VERSION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2457, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3482] StringParser("VIEW")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2457, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3483] StringParser("VIEWS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2457, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3484] StringParser("VOLATILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2457, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3485] StringParser("WAITFOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2457, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3486] StringParser("WAREHOUSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2457, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3487] StringParser("WAREHOUSES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2457, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3488] StringParser("WEEK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2457, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3489] StringParser("WEEKDAY")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2457, child_count: 8, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3490] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2465, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3491] Ref(WhenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2465, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3492] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2468, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3493] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 2468, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3494] Meta(conditional: indented_then=True -> indent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2468, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3495] Ref(ThenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 2468, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3496] Meta(conditional: indented_then_contents=True -> implicit_indent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2468, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3497] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 2468, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3498] Meta(conditional: indented_then_contents=True -> dedent)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 2468, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3499] Meta(conditional: indented_then=True -> dedent)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2468, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3500] StringParser("WHEN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2468, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3501] StringParser("WHENEVER")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2468, child_count: 4, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3502] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2472, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3503] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2472, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3504] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2474, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3505] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2477, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3506] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2477, child_count: 8, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3507] OneOf(8 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2485, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3508] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2485, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3509] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2487, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3510] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2487, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3511] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2487, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3512] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2489, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3513] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2489, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3514] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2489, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3515] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2489, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3516] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2489, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3517] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2489, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3518] Ref(OverlapsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2489, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3519] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2489, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3520] StringParser("WHERE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2489, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3521] StringParser("WHILE")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2489, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3522] Token("whitespace")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2489, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3523] StringParser("WIDTH_BUCKET")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2489, child_count: 1, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3524] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2490, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3525] Ref(WildcardIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2490, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3526] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2492, child_count: 1, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3527] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2493, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3528] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2495, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3529] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2497, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3530] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2497, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3531] Ref(ObjectReferenceDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2497, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3532] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2499, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3533] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2499, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3534] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2499, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3535] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2499, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3536] StringParser("WINDOW")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2499, child_count: 4, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3537] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(69), first_child_idx: 2503, child_count: 1, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3538] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2504, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3539] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2506, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3540] Ref(PartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2506, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3541] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2506, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3542] Ref(PartitionClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2506, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3543] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2506, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3544] Ref(FrameClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2506, child_count: 6, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3545] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2512, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3546] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2512, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3547] Ref(RecursiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 2512, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3548] Meta(conditional: indented_ctes=True -> indent)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2512, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 1, _padding: 0 }, // [3549] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2514, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3550] Ref(CTEDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2514, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3551] Ref(SelectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 2514, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3552] Meta(conditional: indented_ctes=True -> dedent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2514, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3553] Ref(NonWithNonSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2514, child_count: 7, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3554] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2521, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3555] Ref(LockingClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2521, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3556] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2521, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3557] Ref(RecursiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 2521, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3558] Meta(conditional: indented_ctes=True -> indent)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2521, child_count: 2, min_times: 0, first_terminator_idx: 48, terminator_count: 1, _padding: 0 }, // [3559] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2523, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3560] Ref(CTEDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2523, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3561] Ref(SelectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 2523, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3562] Meta(conditional: indented_ctes=True -> dedent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2523, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3563] Ref(NonWithSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2523, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3564] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2526, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3565] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2526, child_count: 1, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3566] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2527, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3567] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2527, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3568] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2527, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3569] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2527, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3570] StringParser("WITH")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2527, child_count: 4, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3571] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2531, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3572] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2531, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3573] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2531, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3574] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2531, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3575] Ref(BindingKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2531, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3576] StringParser("WITHIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2531, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3577] StringParser("WITHOUT")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2531, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3578] Token("word")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2531, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3579] StringParser("WORK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2531, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3580] StringParser("WRAPPER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2531, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3581] StringParser("WRITE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2531, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3582] StringParser("WRITETEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2531, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3583] StringParser("X509")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2531, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3584] StringParser("XML")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2531, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3585] StringParser("XOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2531, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3586] StringParser("YAML")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2531, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3587] StringParser("YEAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2531, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3588] StringParser("YEAR_MONTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2531, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3589] StringParser("ZEROFILL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2531, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3590] StringParser("ZONE")
];

pub static CHILD_IDS: &[u32] = &[
    5, 6, 61, 7, 51, 8, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
    21, 28, 33, 38, 39, 44, 9, 10, 22, 23, 24, 25, 26, 27, 29, 30,
    31, 32, 34, 35, 36, 37, 40, 41, 42, 43, 45, 46, 47, 48, 49, 50,
    52, 58, 53, 54, 57, 55, 56, 62, 63, 64, 66, 74, 78, 81, 84, 89,
    67, 68, 69, 70, 71, 72, 73, 75, 76, 77, 79, 80, 82, 83, 85, 86,
    87, 88, 90, 133, 91, 100, 105, 108, 109, 110, 111, 112, 113, 114, 115, 116,
    117, 118, 119, 120, 121, 122, 123, 124, 125, 126, 127, 128, 129, 130, 92, 93,
    94, 95, 98, 99, 96, 97, 101, 102, 103, 104, 106, 107, 131, 132, 135, 58,
    138, 139, 140, 141, 142, 143, 144, 145, 146, 147, 148, 151, 154, 157, 149, 150,
    152, 153, 155, 156, 158, 159, 161, 162, 163, 164, 165, 166, 167, 168, 169, 170,
    171, 174, 177, 180, 172, 173, 175, 176, 178, 179, 181, 182, 184, 185, 187, 191,
    195, 199, 203, 205, 207, 188, 189, 190, 58, 192, 193, 194, 58, 196, 197, 198,
    58, 200, 201, 202, 58, 204, 58, 206, 58, 209, 221, 222, 223, 231, 224, 230,
    225, 226, 227, 228, 229, 234, 235, 241, 245, 246, 247, 252, 255, 242, 243, 244,
    248, 251, 249, 250, 253, 254, 257, 258, 259, 260, 261, 263, 264, 265, 266, 268,
    274, 287, 288, 269, 270, 271, 272, 273, 275, 278, 279, 280, 276, 277, 281, 286,
    282, 285, 283, 284, 289, 290, 293, 291, 292, 295, 296, 297, 298, 299, 58, 310,
    311, 312, 313, 314, 315, 316, 317, 318, 319, 321, 326, 327, 322, 325, 323, 324,
    331, 326, 327, 332, 58, 335, 349, 358, 359, 360, 361, 362, 363, 364, 365, 366,
    378, 379, 380, 381, 389, 390, 393, 394, 399, 400, 403, 404, 407, 228, 229, 408,
    58, 410, 228, 229, 411, 58, 414, 228, 229, 419, 420, 421, 423, 424, 439, 425,
    426, 427, 428, 429, 430, 431, 432, 433, 434, 435, 436, 437, 438, 441, 442, 443,
    444, 445, 451, 446, 447, 448, 449, 450, 452, 453, 457, 228, 229, 459, 460, 461,
    462, 463, 228, 229, 471, 481, 472, 473, 474, 478, 231, 480, 475, 482, 483, 473,
    484, 488, 231, 490, 485, 505, 509, 514, 506, 507, 508, 510, 511, 512, 513, 515,
    516, 540, 228, 229, 544, 545, 546, 547, 557, 558, 559, 560, 562, 565, 563, 564,
    566, 569, 573, 576, 579, 580, 581, 584, 585, 588, 567, 568, 570, 571, 572, 228,
    229, 574, 575, 577, 578, 582, 583, 586, 587, 590, 591, 592, 594, 593, 228, 229,
    595, 596, 600, 601, 613, 614, 620, 621, 622, 623, 624, 625, 626, 627, 628, 629,
    630, 631, 632, 633, 634, 635, 636, 640, 637, 638, 639, 641, 642, 643, 644, 652,
    653, 658, 659, 660, 682, 683, 684, 688, 689, 690, 700, 701, 702, 705, 685, 686,
    687, 228, 229, 691, 692, 693, 694, 695, 699, 696, 697, 698, 703, 704, 706, 707,
    709, 710, 711, 712, 714, 715, 716, 717, 718, 719, 720, 721, 724, 722, 723, 726,
    727, 728, 729, 730, 731, 732, 733, 734, 735, 736, 228, 229, 737, 58, 740, 741,
    742, 743, 744, 745, 757, 758, 746, 747, 748, 228, 229, 749, 58, 750, 751, 752,
    753, 754, 755, 326, 327, 756, 58, 760, 761, 762, 763, 765, 766, 767, 768, 770,
    774, 778, 779, 780, 785, 788, 771, 772, 773, 775, 776, 777, 781, 784, 782, 783,
    786, 787, 790, 791, 792, 793, 794, 796, 797, 798, 807, 808, 809, 810, 811, 825,
    799, 802, 800, 801, 803, 806, 804, 805, 812, 819, 822, 813, 818, 814, 228, 229,
    815, 58, 816, 817, 820, 821, 823, 824, 826, 828, 829, 830, 831, 837, 850, 851,
    852, 889, 832, 833, 834, 835, 836, 838, 848, 839, 840, 841, 842, 843, 844, 845,
    58, 853, 863, 866, 879, 885, 854, 855, 856, 857, 858, 859, 860, 861, 862, 864,
    865, 867, 870, 868, 869, 871, 872, 873, 876, 874, 875, 877, 878, 880, 881, 882,
    883, 884, 886, 887, 888, 228, 229, 890, 891, 892, 893, 895, 896, 897, 899, 902,
    903, 904, 905, 906, 907, 910, 900, 901, 908, 909, 909, 228, 229, 920, 923, 921,
    922, 924, 228, 229, 941, 942, 945, 946, 947, 949, 950, 951, 955, 958, 952, 228,
    229, 953, 954, 58, 956, 957, 959, 963, 968, 960, 961, 962, 964, 965, 966, 967,
    969, 970, 975, 976, 228, 229, 977, 58, 978, 979, 981, 986, 982, 983, 984, 985,
    1007, 1008, 1020, 1023, 1024, 1021, 1022, 1036, 1037, 1038, 1059, 1060, 1062, 1063, 1064, 1068,
    1065, 1066, 1067, 228, 229, 1070, 1071, 1072, 1073, 1074, 1076, 1077, 1078, 1079, 1081, 1082,
    1083, 1084, 1085, 1088, 1089, 1090, 1091, 1093, 1094, 1095, 1096, 1098, 1099, 1100, 1101, 1102,
    1104, 1105, 1106, 1108, 1109, 1110, 1111, 1112, 1114, 1113, 58, 1116, 1117, 1118, 1119, 1121,
    1122, 1123, 1124, 1125, 1127, 1128, 1129, 1130, 1132, 1133, 1134, 1135, 1136, 1146, 473, 1147,
    231, 228, 229, 1152, 1153, 1185, 1186, 1187, 1188, 1189, 1190, 1193, 1194, 1196, 1197, 1198,
    1199, 1200, 1203, 1204, 1205, 1209, 1210, 1211, 1212, 1218, 1201, 1202, 1206, 1207, 1208, 1213,
    1214, 1215, 1216, 1217, 1219, 1220, 1222, 1225, 1226, 1227, 1223, 1224, 1229, 1230, 1231, 1232,
    1233, 1237, 1234, 1235, 1236, 1239, 1242, 1240, 1241, 1244, 1248, 1254, 1245, 1246, 1247, 228,
    229, 1249, 1252, 1250, 1251, 1253, 1257, 1292, 1258, 1259, 1260, 1272, 1273, 1274, 1281, 1289,
    1290, 1261, 228, 229, 1262, 1263, 1264, 1265, 1266, 58, 1267, 1268, 1269, 1270, 1271, 1275,
    1278, 1276, 1277, 1279, 228, 229, 1280, 58, 1282, 1283, 1284, 1285, 1286, 1287, 1288, 1294,
    1295, 1296, 1297, 1298, 1299, 1300, 1309, 1310, 1313, 1317, 1320, 1311, 1312, 1314, 1315, 1316,
    1318, 1319, 1321, 1322, 1323, 1324, 1329, 1331, 1335, 1330, 1332, 1333, 1334, 1336, 1339, 1340,
    1341, 228, 229, 1342, 1343, 1356, 1357, 1364, 1365, 1366, 1379, 1367, 1370, 1368, 1369, 1371,
    1376, 1372, 1373, 1374, 1375, 1377, 1378, 1380, 1366, 1381, 1366, 1383, 1384, 1390, 1391, 1392,
    58, 1394, 1395, 1396, 1399, 1402, 1403, 1404, 1405, 1406, 1407, 1408, 1409, 1397, 1398, 1400,
    1401, 1411, 1429, 1412, 1413, 1416, 1417, 1423, 1427, 1428, 1414, 1415, 1415, 228, 229, 1418,
    1419, 1420, 1421, 1422, 1424, 1425, 1426, 1430, 228, 229, 1411, 1431, 1432, 1434, 1435, 1435,
    228, 229, 473, 1436, 231, 1447, 1448, 1459, 1437, 1438, 1439, 1440, 228, 229, 1442, 1443,
    1445, 1446, 1449, 1450, 1451, 1452, 1454, 1455, 1457, 1458, 1462, 1463, 1464, 58, 1469, 1470,
    1474, 1480, 1486, 1492, 1493, 1496, 1506, 1507, 1508, 1509, 1471, 1472, 1473, 1475, 1476, 1478,
    1479, 1477, 1481, 1484, 1485, 1482, 1483, 1487, 1488, 1489, 1490, 1491, 58, 1494, 1495, 1497,
    1501, 1502, 1498, 1499, 1500, 1503, 1504, 1505, 1510, 1511, 1512, 1514, 1515, 228, 229, 1517,
    1518, 1519, 1520, 1521, 1525, 1530, 1526, 1527, 1528, 1531, 1532, 1535, 1542, 1536, 1537, 1538,
    1541, 1539, 1540, 1543, 1546, 1544, 1545, 1548, 228, 229, 1549, 58, 1551, 1554, 1555, 1552,
    1553, 1556, 1563, 1557, 1562, 1558, 1559, 1560, 1561, 1577, 1578, 1592, 1593, 1594, 1607, 1579,
    1583, 1586, 1591, 1580, 1581, 1582, 1584, 1585, 1587, 1588, 1589, 1590, 1595, 1599, 1603, 1596,
    1597, 1598, 1600, 1601, 1602, 1604, 1605, 1606, 1608, 1609, 1610, 1611, 1612, 1613, 1617, 1618,
    1624, 1625, 221, 1626, 231, 1627, 1628, 1629, 1630, 1631, 1632, 58, 1633, 1634, 1635, 1638,
    1641, 1642, 1643, 1644, 1645, 1639, 1640, 1648, 1649, 58, 1650, 1651, 1652, 1653, 228, 229,
    1657, 1658, 1659, 1660, 228, 229, 1661, 58, 1662, 1663, 1667, 473, 1668, 231, 1669, 1670,
    1670, 228, 229, 1672, 1675, 1676, 1677, 1678, 1673, 1674, 1699, 1700, 1703, 1704, 1705, 1708,
    1711, 1709, 1710, 1721, 1722, 1723, 1724, 1729, 1725, 228, 229, 1726, 1728, 1727, 58, 1734,
    1735, 1736, 1737, 1753, 1754, 1755, 1756, 1757, 1758, 1759, 1762, 1760, 1761, 1779, 1780, 1781,
    1786, 1787, 1782, 1783, 1784, 1785, 1788, 1789, 1790, 1791, 1796, 1797, 1798, 1799, 1800, 1802,
    1803, 1804, 1805, 1813, 1826, 1831, 1814, 1815, 221, 1816, 1817, 231, 1819, 1818, 1820, 1821,
    1822, 1825, 1823, 1824, 1827, 1828, 221, 1829, 1830, 231, 1832, 221, 1833, 231, 1836, 1839,
    1840, 1841, 1844, 1842, 1843, 1843, 228, 229, 1846, 1847, 1848, 1852, 1849, 1850, 1851, 1854,
    221, 1855, 231, 1856, 228, 229, 1857, 58, 1882, 1883, 1889, 1892, 1893, 1890, 1891, 1894,
    1895, 1897, 1898, 1899, 1903, 221, 1904, 1910, 231, 1905, 1906, 1906, 228, 229, 1907, 1908,
    1909, 1911, 1916, 1912, 1913, 1914, 1915, 1917, 1918, 1925, 1926, 1927, 1928, 1929, 1930, 1931,
    1932, 1933, 1946, 1949, 1960, 1963, 1977, 1979, 1947, 1948, 1950, 1951, 1954, 1957, 1952, 1953,
    1955, 1956, 1958, 1959, 1961, 1962, 1964, 1965, 1966, 1967, 1968, 1972, 1973, 1974, 1969, 1970,
    1971, 1975, 1976, 1978, 1980, 1995, 1996, 1997, 1998, 1999, 2002, 228, 229, 2000, 2001, 2003,
    2004, 2005, 228, 229, 2028, 221, 2029, 231, 2030, 2032, 2033, 2036, 2037, 2039, 2040, 2041,
    2044, 221, 2045, 231, 2042, 2043, 2046, 2047, 2049, 2050, 2051, 2052, 2055, 221, 2056, 231,
    2053, 2054, 2058, 221, 2059, 231, 2062, 221, 2063, 231, 2073, 2074, 2075, 2076, 2060, 2061,
    2064, 2065, 2066, 2070, 2067, 2069, 2068, 228, 229, 2071, 2072, 2078, 221, 2079, 231, 2116,
    2117, 2118, 2119, 2120, 2121, 228, 229, 2123, 221, 2124, 231, 2125, 58, 2131, 2132, 2155,
    2156, 2157, 2159, 2161, 2163, 2158, 228, 229, 2160, 228, 229, 2162, 228, 229, 2166, 2167,
    2168, 2169, 2171, 2172, 2175, 2173, 2174, 2174, 228, 229, 2185, 2188, 2186, 2187, 2189, 2190,
    2193, 2194, 2196, 2197, 2215, 2216, 2217, 2219, 2221, 1159, 2220, 58, 2223, 2224, 2225, 2226,
    2228, 2229, 2230, 2231, 2232, 2233, 2234, 2235, 2236, 2237, 2238, 412, 2246, 2247, 2251, 2248,
    2249, 2250, 2252, 2253, 2276, 2277, 2279, 2280, 221, 2281, 231, 2282, 58, 2283, 2287, 2290,
    2295, 2284, 2285, 2286, 2288, 2289, 2291, 2292, 2293, 2294, 2299, 2300, 2301, 2302, 2303, 2304,
    2305, 2308, 2309, 221, 2318, 2319, 2320, 231, 2321, 2322, 2323, 228, 229, 2326, 2327, 2328,
    2333, 2329, 2330, 2331, 2332, 228, 229, 2356, 2357, 221, 2358, 231, 2359, 2360, 2360, 228,
    229, 2361, 58, 2367, 2372, 2368, 2369, 2370, 2371, 2389, 2390, 2403, 2404, 2420, 2421, 2423,
    473, 2424, 231, 2425, 2426, 2426, 228, 229, 2429, 2430, 2434, 2431, 2432, 2433, 2435, 228,
    229, 2460, 2461, 2462, 2463, 2464, 2465, 2469, 2466, 2467, 2468, 2470, 2471, 2472, 2474, 2475,
    2476, 2477, 2478, 2483, 2484, 2485, 2488, 2491, 2486, 2487, 2489, 2490, 2492, 2493, 2530, 2531,
    2535, 2549, 2550, 2551, 2532, 2533, 2534, 2536, 2540, 2543, 2548, 2537, 2538, 2539, 2541, 2542,
    2544, 2545, 2546, 2547, 2577, 2578, 2581, 2583, 2579, 2580, 2582, 228, 229, 2584, 2586, 2585,
    2587, 228, 229, 2606, 2607, 2608, 2609, 2611, 2612, 2613, 2621, 2614, 2615, 2616, 2618, 2617,
    2619, 2620, 2622, 2623, 2624, 2629, 2632, 2625, 2626, 2627, 2628, 2630, 2631, 2633, 2634, 2635,
    2636, 2638, 2641, 221, 2642, 231, 2639, 2640, 2643, 58, 2647, 2648, 2652, 2654, 2655, 2658,
    2659, 2660, 2661, 2653, 2656, 2657, 2664, 2665, 2666, 2667, 2668, 2669, 2670, 2671, 2672, 2673,
    2674, 2675, 2676, 2677, 2682, 2685, 2688, 2689, 2683, 2684, 2684, 228, 229, 2686, 2687, 2687,
    228, 229, 2690, 228, 229, 2697, 2700, 2698, 2699, 2701, 2702, 2704, 2707, 2705, 2706, 2708,
    2709, 2717, 221, 2718, 231, 2719, 58, 2721, 2722, 2723, 2724, 2725, 2726, 2727, 2728, 2729,
    2730, 2732, 2733, 2737, 2738, 2739, 2734, 2735, 2736, 2742, 2743, 2748, 2744, 2747, 2745, 2746,
    2750, 2751, 2752, 2753, 2756, 2758, 2759, 2754, 2755, 2757, 2760, 2764, 2761, 2762, 2763, 2766,
    2767, 2768, 2769, 2771, 2776, 2772, 2775, 2773, 2774, 2783, 2786, 2784, 2785, 2787, 2788, 2789,
    2790, 2795, 2796, 2800, 2801, 2804, 58, 2806, 2807, 2808, 2812, 2813, 2856, 2857, 2858, 2859,
    2860, 2861, 2862, 2863, 2864, 2865, 2866, 2867, 2868, 2869, 2870, 2871, 2872, 2873, 2874, 2875,
    2876, 2877, 2878, 2879, 2880, 2881, 2882, 2883, 2884, 2885, 2886, 2887, 2888, 2889, 2890, 2891,
    2892, 2893, 2894, 2895, 2896, 2897, 2898, 2899, 2900, 2901, 2902, 2917, 2920, 228, 229, 2921,
    58, 2922, 2923, 2943, 2946, 2944, 2945, 2947, 2950, 2953, 2948, 2949, 2951, 2952, 2954, 2955,
    2956, 2959, 2960, 2961, 2962, 2963, 2965, 2964, 228, 229, 2966, 228, 229, 2973, 2976, 2974,
    2978, 2980, 2979, 2984, 3026, 2985, 2989, 2995, 2998, 3004, 3010, 3013, 3016, 3019, 3022, 3025,
    2986, 2987, 2988, 2990, 2991, 2992, 2993, 2994, 2996, 2997, 2999, 3000, 3001, 3002, 3003, 3005,
    3006, 3007, 3008, 3009, 3011, 3012, 3014, 3015, 3017, 3018, 3020, 3021, 3023, 3024, 3027, 3028,
    3030, 3031, 3032, 3036, 3041, 3067, 3068, 3069, 3070, 3033, 3034, 3035, 3037, 3038, 3039, 3040,
    3042, 58, 3043, 3052, 3056, 3044, 3045, 3046, 3047, 3048, 3051, 3049, 228, 229, 3050, 58,
    3053, 3054, 3055, 3057, 3058, 3064, 3059, 3060, 3060, 228, 229, 3061, 58, 3062, 3063, 3065,
    3066, 3071, 3072, 3078, 3073, 3074, 3074, 228, 229, 3075, 58, 3076, 3077, 3079, 3080, 3082,
    3083, 3084, 3085, 3088, 3089, 3086, 228, 229, 3087, 58, 3091, 3092, 3094, 3140, 3093, 3095,
    3098, 3101, 3104, 3107, 3110, 3113, 3116, 3119, 3122, 3125, 3128, 3131, 3134, 3137, 3096, 3097,
    3099, 3100, 3102, 3103, 3105, 3106, 3108, 3109, 3111, 3112, 3114, 3115, 3117, 3118, 3120, 3121,
    3123, 3124, 3126, 3127, 3129, 3130, 3132, 3133, 3135, 3136, 3138, 3139, 3141, 3144, 3142, 3143,
    3146, 3147, 3148, 58, 3149, 3153, 3165, 3172, 3177, 3182, 3150, 3151, 3152, 3154, 3161, 3164,
    3155, 3156, 3157, 3158, 3159, 3160, 3162, 3163, 3166, 3167, 3168, 3169, 3170, 3171, 3173, 3176,
    3174, 3175, 3178, 3179, 3180, 3181, 3183, 3184, 3185, 3187, 3188, 3189, 3192, 3190, 3191, 3193,
    228, 229, 3195, 3196, 3197, 3198, 3201, 3199, 3200, 3203, 3218, 3222, 3230, 3235, 3243, 3204,
    3214, 3205, 3206, 3207, 3208, 3209, 3210, 3213, 3211, 228, 229, 3212, 58, 3215, 3216, 3217,
    3219, 3220, 3221, 3223, 3224, 3225, 3226, 3227, 3228, 228, 229, 3229, 58, 3231, 3232, 3234,
    3233, 3236, 3237, 3239, 3238, 3240, 3241, 3242, 3244, 3245, 3246, 3249, 3247, 3248, 3251, 3255,
    3252, 3253, 3254, 228, 229, 3256, 228, 229, 3257, 58, 3258, 3259, 3260, 228, 229, 3265,
    3266, 3269, 3270, 3272, 3273, 228, 229, 3274, 3275, 3287, 3290, 3291, 3288, 3289, 3292, 3295,
    3296, 3293, 3294, 3298, 3299, 3300, 3301, 3302, 3317, 3323, 3326, 3329, 3318, 3319, 3320, 3321,
    3322, 3324, 3325, 3327, 3328, 3330, 3331, 3332, 3349, 3350, 3351, 3356, 3357, 3358, 3362, 228,
    229, 3363, 58, 3366, 3367, 3369, 3370, 3378, 3379, 3380, 3385, 3386, 3387, 3388, 3391, 3400,
    3401, 3402, 3403, 3404, 3405, 3406, 3407, 3414, 3415, 3416, 3417, 3418, 3423, 221, 3424, 231,
    3430, 3431, 3425, 3426, 3427, 3428, 3429, 3438, 3439, 3458, 3461, 3459, 3460, 3462, 58, 3463,
    3464, 3465, 228, 229, 3466, 58, 3467, 3468, 3469, 3491, 3492, 3494, 3495, 3496, 3497, 3498,
    3499, 473, 3493, 231, 3503, 473, 3504, 231, 3505, 3506, 3506, 228, 229, 3508, 3509, 3512,
    3515, 3516, 3517, 3518, 3519, 3510, 3511, 3513, 3514, 3525, 3527, 3535, 3528, 3529, 3532, 3530,
    3531, 3533, 3534, 3538, 3542, 3543, 3544, 3539, 3540, 3541, 3546, 3547, 3548, 3549, 3552, 3553,
    3550, 58, 3555, 3556, 3557, 3558, 3559, 3562, 3563, 3560, 58, 3565, 3566, 3568, 3567, 3572,
    3573, 3574, 3575,
];

pub static TERMINATORS: &[u32] = &[
    59, 60, 136, 367, 368, 476, 477, 479, 486, 487, 489, 491, 492, 493, 548, 602,
    846, 847, 849, 1255, 1291, 1441, 1444, 1453, 1456, 1529, 1533, 1636, 1654, 2296, 2297, 2644,
    2645, 2646, 2649, 2650, 2678, 2679, 2680, 2802, 2903, 2975, 3408, 3409, 3410, 3411, 3412, 3551,
    3561,
];

pub static STRINGS: &[&str] = &[
    "ABORT", // [0]
    "keyword", // [1]
    "KeywordSegment", // [2]
    "base", // [3]
    "raw", // [4]
    "word", // [5]
    "ABS", // [6]
    "ABSOLUTE", // [7]
    "ACCESS", // [8]
    "AccountKeywordSegment", // [9]
    "ACCOUNT", // [10]
    "ALL", // [11]
    "CATALOG", // [12]
    "DATABASE", // [13]
    "DOMAIN", // [14]
    "EXTERNAL", // [15]
    "FILE", // [16]
    "FOREIGN", // [17]
    "FUNCTION", // [18]
    "FUTURE", // [19]
    "INTEGRATION", // [20]
    "LANGUAGE", // [21]
    "MASKING", // [22]
    "MATERIALIZED", // [23]
    "PROCEDURE", // [24]
    "RESOURCE", // [25]
    "ROLE", // [26]
    "ROUTINE", // [27]
    "SCHEMA", // [28]
    "SEQUENCE", // [29]
    "STAGE", // [30]
    "STREAM", // [31]
    "TABLE", // [32]
    "TABLESPACE", // [33]
    "TASK", // [34]
    "TYPE", // [35]
    "VIEW", // [36]
    "WAREHOUSE", // [37]
    "ResourceKeywordSegment", // [38]
    "MonitorKeywordSegment", // [39]
    "MONITOR", // [40]
    "WarehouseKeywordSegment", // [41]
    "DatabaseKeywordSegment", // [42]
    "DomainKeywordSegment", // [43]
    "IntegrationKeywordSegment", // [44]
    "LanguageKeywordSegment", // [45]
    "SchemaKeywordSegment", // [46]
    "CatalogKeywordSegment", // [47]
    "RoleKeywordSegment", // [48]
    "TablespaceKeywordSegment", // [49]
    "TypeKeywordSegment", // [50]
    "ForeignKeywordSegment", // [51]
    "DATA", // [52]
    "SERVER", // [53]
    "ServerKeywordSegment", // [54]
    "DataKeywordSegment", // [55]
    "WrapperKeywordSegment", // [56]
    "WRAPPER", // [57]
    "AllKeywordSegment", // [58]
    "SchemasKeywordSegment", // [59]
    "SCHEMAS", // [60]
    "InKeywordSegment", // [61]
    "IN", // [62]
    "FutureKeywordSegment", // [63]
    "AccessSchemaObjectSegment", // [64]
    "access_schema_object", // [65]
    "AccessSchemaPluralObjectSegment", // [66]
    "FUNCTIONS", // [67]
    "PROCEDURES", // [68]
    "ROUTINES", // [69]
    "SEQUENCES", // [70]
    "STAGES", // [71]
    "STREAMS", // [72]
    "TABLES", // [73]
    "TASKS", // [74]
    "VIEWS", // [75]
    "access_schema_plural_object", // [76]
    "ObjectReferenceSegment", // [77]
    "object_reference", // [78]
    "FunctionNameSegment", // [79]
    "function_name", // [80]
    "FunctionParameterListGrammar", // [81]
    "(", // [82]
    "function_parameter_list", // [83]
    "WildcardIdentifierSegment", // [84]
    "wildcard_identifier", // [85]
    "CommaSegment", // [86]
    ",", // [87]
    "ToKeywordSegment", // [88]
    "TO", // [89]
    "FromKeywordSegment", // [90]
    "FROM", // [91]
    "LARGE", // [92]
    "LargeKeywordSegment", // [93]
    "ObjectKeywordSegment", // [94]
    "OBJECT", // [95]
    "NumericLiteralSegment", // [96]
    "numeric_literal", // [97]
    "APPLY", // [98]
    "CONNECT", // [99]
    "CREATE", // [100]
    "DELETE", // [101]
    "EXECUTE", // [102]
    "IMPORTED", // [103]
    "INSERT", // [104]
    "MANAGE", // [105]
    "MODIFY", // [106]
    "OPERATE", // [107]
    "OWNERSHIP", // [108]
    "READ", // [109]
    "REFERENCES", // [110]
    "REFERENCE_USAGE", // [111]
    "SELECT", // [112]
    "TEMP", // [113]
    "TEMPORARY", // [114]
    "TRIGGER", // [115]
    "TRUNCATE", // [116]
    "UPDATE", // [117]
    "USAGE", // [118]
    "USE", // [119]
    "USE_ANY_ROLE", // [120]
    "WRITE", // [121]
    "CreateKeywordSegment", // [122]
    "USER", // [123]
    "UserKeywordSegment", // [124]
    "ApplyKeywordSegment", // [125]
    "MaskingKeywordSegment", // [126]
    "PolicyKeywordSegment", // [127]
    "POLICY", // [128]
    "ExecuteKeywordSegment", // [129]
    "TaskKeywordSegment", // [130]
    "ManageKeywordSegment", // [131]
    "GrantsKeywordSegment", // [132]
    "GRANTS", // [133]
    "EXECUTION", // [134]
    "ExecutionKeywordSegment", // [135]
    "UsageKeywordSegment", // [136]
    "PIPE", // [137]
    "PipeKeywordSegment", // [138]
    "UseKeywordSegment", // [139]
    "ImportedKeywordSegment", // [140]
    "PrivilegesKeywordSegment", // [141]
    "PRIVILEGES", // [142]
    "ConnectKeywordSegment", // [143]
    "DeleteKeywordSegment", // [144]
    "InsertKeywordSegment", // [145]
    "ModifyKeywordSegment", // [146]
    "OperateKeywordSegment", // [147]
    "OwnershipKeywordSegment", // [148]
    "ReadKeywordSegment", // [149]
    "Reference_usageKeywordSegment", // [150]
    "ReferencesKeywordSegment", // [151]
    "SelectKeywordSegment", // [152]
    "TempKeywordSegment", // [153]
    "TemporaryKeywordSegment", // [154]
    "TriggerKeywordSegment", // [155]
    "TruncateKeywordSegment", // [156]
    "UpdateKeywordSegment", // [157]
    "Use_any_roleKeywordSegment", // [158]
    "WriteKeywordSegment", // [159]
    "BracketedColumnReferenceListGrammar", // [160]
    "AccessPermissionSegment", // [161]
    "access_permission", // [162]
    "OnKeywordSegment", // [163]
    "ON", // [164]
    "TableKeywordSegment", // [165]
    "ViewKeywordSegment", // [166]
    "StageKeywordSegment", // [167]
    "FunctionKeywordSegment", // [168]
    "ProcedureKeywordSegment", // [169]
    "RoutineKeywordSegment", // [170]
    "SequenceKeywordSegment", // [171]
    "StreamKeywordSegment", // [172]
    "MaterializedKeywordSegment", // [173]
    "ExternalKeywordSegment", // [174]
    "FileKeywordSegment", // [175]
    "FormatKeywordSegment", // [176]
    "FORMAT", // [177]
    "TablesKeywordSegment", // [178]
    "ViewsKeywordSegment", // [179]
    "StagesKeywordSegment", // [180]
    "FunctionsKeywordSegment", // [181]
    "ProceduresKeywordSegment", // [182]
    "RoutinesKeywordSegment", // [183]
    "SequencesKeywordSegment", // [184]
    "StreamsKeywordSegment", // [185]
    "TasksKeywordSegment", // [186]
    "PoliciesKeywordSegment", // [187]
    "FormatsKeywordSegment", // [188]
    "GRANT", // [189]
    "REVOKE", // [190]
    "GrantStatementSegment", // [191]
    "grant_statement", // [192]
    "RevokeStatementSegment", // [193]
    "revoke_statement", // [194]
    "GROUP", // [195]
    "GroupKeywordSegment", // [196]
    "UserReferenceSegment", // [197]
    "user_reference", // [198]
    "RoleReferenceSegment", // [199]
    "role_reference", // [200]
    "SHARE", // [201]
    "ShareKeywordSegment", // [202]
    "FunctionSegment", // [203]
    "function", // [204]
    "PublicKeywordSegment", // [205]
    "PUBLIC", // [206]
    "[", // [207]
    "ArrayAccessorSegment", // [208]
    "array_accessor", // [209]
    "ACCOUNTS", // [210]
    "ACTION", // [211]
    "ACTIVITYCOUNT", // [212]
    "ADA", // [213]
    "ADD", // [214]
    "ADMIN", // [215]
    "AFTER", // [216]
    "AGGREGATE", // [217]
    "OrderByClauseSegment", // [218]
    "ORDER", // [219]
    "orderby_clause", // [220]
    "indent", // [221]
    "Indent", // [222]
    "meta", // [223]
    "AsAliasOperatorSegment", // [224]
    "AS", // [225]
    "alias_operator", // [226]
    "SingleIdentifierGrammar", // [227]
    "SingleIdentifierListSegment", // [228]
    "identifier_list", // [229]
    "start_bracket", // [230]
    "SymbolSegment", // [231]
    "symbol", // [232]
    ")", // [233]
    "end_bracket", // [234]
    "SingleQuotedIdentifierSegment", // [235]
    "single_quote", // [236]
    "dedent", // [237]
    "Dedent", // [238]
    "ALIAS", // [239]
    "TableReferenceSegment", // [240]
    "table_reference", // [241]
    "AliasExpressionSegment", // [242]
    "alias_expression", // [243]
    "ALLOCATE", // [244]
    "ALSO", // [245]
    "ALTER", // [246]
    "CACHE", // [247]
    "CYCLE", // [248]
    "INCREMENT", // [249]
    "MAXVALUE", // [250]
    "MINVALUE", // [251]
    "NO", // [252]
    "NOCACHE", // [253]
    "NOCYCLE", // [254]
    "NOORDER", // [255]
    "IncrementKeywordSegment", // [256]
    "ByKeywordSegment", // [257]
    "BY", // [258]
    "SequenceMinValueGrammar", // [259]
    "SequenceMaxValueGrammar", // [260]
    "CacheKeywordSegment", // [261]
    "NocacheKeywordSegment", // [262]
    "CycleKeywordSegment", // [263]
    "NocycleKeywordSegment", // [264]
    "OrderNoOrderGrammar", // [265]
    "AlterKeywordSegment", // [266]
    "SequenceReferenceSegment", // [267]
    "sequence_reference", // [268]
    "AlterSequenceOptionsSegment", // [269]
    "alter_sequence_options_segment", // [270]
    "DROP", // [271]
    "DropKeywordSegment", // [272]
    "ColumnKeywordSegment", // [273]
    "COLUMN", // [274]
    "IfExistsGrammar", // [275]
    "IF", // [276]
    "ParameterNameSegment", // [277]
    "EqualsSegment", // [278]
    "=", // [279]
    "comparison_operator", // [280]
    "LiteralGrammar", // [281]
    "NakedIdentifierSegment", // [282]
    "AddKeywordSegment", // [283]
    "ColumnDefinitionSegment", // [284]
    "column_definition", // [285]
    "FIRST", // [286]
    "FirstKeywordSegment", // [287]
    "AfterKeywordSegment", // [288]
    "ColumnReferenceSegment", // [289]
    "column_reference", // [290]
    "AlterTableDropColumnGrammar", // [291]
    "RENAME", // [292]
    "RenameKeywordSegment", // [293]
    "AsKeywordSegment", // [294]
    "AlterTableOptionsGrammar", // [295]
    "ALWAYS", // [296]
    "&", // [297]
    "ampersand", // [298]
    "ANALYSE", // [299]
    "ANALYZE", // [300]
    "AND", // [301]
    "binary_operator", // [302]
    "BinaryOperatorSegment", // [303]
    "ANY", // [304]
    "ARE", // [305]
    "%", // [306]
    "*", // [307]
    "+", // [308]
    "-", // [309]
    "/", // [310]
    "<", // [311]
    ">", // [312]
    "^", // [313]
    "|", // [314]
    "PlusSegment", // [315]
    "MinusSegment", // [316]
    "DivideSegment", // [317]
    "MultiplySegment", // [318]
    "ModuloSegment", // [319]
    "BitwiseAndSegment", // [320]
    "BitwiseOrSegment", // [321]
    "BitwiseXorSegment", // [322]
    "BitwiseLShiftSegment", // [323]
    "BitwiseRShiftSegment", // [324]
    "ExpressionSegment", // [325]
    "expression", // [326]
    "SliceSegment", // [327]
    ":", // [328]
    "start_square_bracket", // [329]
    "]", // [330]
    "end_square_bracket", // [331]
    "ARRAY", // [332]
    "BaseExpressionElementGrammar", // [333]
    "ASC", // [334]
    "ASENSITIVE", // [335]
    "ASSERTION", // [336]
    "ASSIGNMENT", // [337]
    "ASYMMETRIC", // [338]
    "AT", // [339]
    "ATOMIC", // [340]
    "ATTRIBUTE", // [341]
    "ATTRIBUTES", // [342]
    "AUDIT", // [343]
    "AUTHORIZATION", // [344]
    "AUTO_INCREMENT", // [345]
    "Auto_incrementKeywordSegment", // [346]
    "AUTOINCREMENT", // [347]
    "AVG", // [348]
    "AVG_ROW_LENGTH", // [349]
    "BACKUP", // [350]
    "BACKWARD", // [351]
    "CURRENT_DATE", // [352]
    "CURRENT_TIME", // [353]
    "CURRENT_TIMESTAMP", // [354]
    "DATE", // [355]
    "bare_function", // [356]
    "CodeSegment", // [357]
    "BareFunctionSegment", // [358]
    "IntervalExpressionSegment", // [359]
    "INTERVAL", // [360]
    "interval_expression", // [361]
    "DatatypeSegment", // [362]
    "data_type", // [363]
    "file", // [364]
    "BaseFileSegment", // [365]
    "BaseSegment", // [366]
    "BEFORE", // [367]
    "BEGIN", // [368]
    "BERNOULLI", // [369]
    "BETWEEN", // [370]
    "BIGINT", // [371]
    "BINARY", // [372]
    "!", // [373]
    "EQ", // [374]
    "GE", // [375]
    "GT", // [376]
    "IS", // [377]
    "LE", // [378]
    "LT", // [379]
    "NE", // [380]
    "NOT", // [381]
    "OR", // [382]
    "OVERLAPS", // [383]
    "like_operator", // [384]
    "ArithmeticBinaryOperatorGrammar", // [385]
    "StringBinaryOperatorGrammar", // [386]
    "BooleanBinaryOperatorGrammar", // [387]
    "ComparisonOperatorGrammar", // [388]
    "BINDING", // [389]
    "BIT", // [390]
    "BIT_LENGTH", // [391]
    "BITVAR", // [392]
    "AmpersandSegment", // [393]
    "RawLessThanSegment", // [394]
    "PipeSegment", // [395]
    "RawGreaterThanSegment", // [396]
    "BLOB", // [397]
    "BOOL", // [398]
    "AndOperatorGrammar", // [399]
    "OrOperatorGrammar", // [400]
    "BOOLEAN", // [401]
    "FALSE", // [402]
    "TRUE", // [403]
    "TrueSegment", // [404]
    "FalseSegment", // [405]
    "BOTH", // [406]
    "bracketed", // [407]
    "BracketedSegment", // [408]
    "UnorderedSetExpressionSegment", // [409]
    "SEL", // [410]
    "VALUE", // [411]
    "VALUES", // [412]
    "set_expression", // [413]
    "BREADTH", // [414]
    "BREAK", // [415]
    "BROWSE", // [416]
    "DotSegment", // [417]
    ".", // [418]
    "SlashSegment", // [419]
    "ERRORCODE", // [420]
    "EXPORT", // [421]
    "GOTO", // [422]
    "IMPORT", // [423]
    "LABEL", // [424]
    "LOGOFF", // [425]
    "LOGON", // [426]
    "QUIT", // [427]
    "RUN", // [428]
    "THEN", // [429]
    "IfKeywordSegment", // [430]
    "ThenKeywordSegment", // [431]
    "LogonKeywordSegment", // [432]
    "ActivitycountKeywordSegment", // [433]
    "ErrorcodeKeywordSegment", // [434]
    "LabelKeywordSegment", // [435]
    "GotoKeywordSegment", // [436]
    "LogoffKeywordSegment", // [437]
    "ImportKeywordSegment", // [438]
    "ExportKeywordSegment", // [439]
    "RunKeywordSegment", // [440]
    "QuitKeywordSegment", // [441]
    "BteqKeyWordSegment", // [442]
    "bteq_key_word_segment", // [443]
    "QuotedLiteralSegment", // [444]
    "BteqFilePathSegment", // [445]
    "bteq_file_path", // [446]
    "BULK", // [447]
    "CTEColumnList", // [448]
    "cte_column_list", // [449]
    "SelectableGrammar", // [450]
    "LOCK", // [451]
    "LOCKING", // [452]
    "WITH", // [453]
    "CALL", // [454]
    "CALLED", // [455]
    "CARDINALITY", // [456]
    "CASCADE", // [457]
    "CASCADED", // [458]
    "CASE", // [459]
    "CaseKeywordSegment", // [460]
    "implicit_indent", // [461]
    "ImplicitIndent", // [462]
    "WHEN", // [463]
    "WhenClauseSegment", // [464]
    "when_clause", // [465]
    "ElseKeywordSegment", // [466]
    "ELSE", // [467]
    "EndKeywordSegment", // [468]
    "END", // [469]
    "ElseClauseSegment", // [470]
    "else_clause", // [471]
    "BinaryOperatorGrammar", // [472]
    "CASESPECIFIC", // [473]
    "CAST", // [474]
    "::", // [475]
    "casting_operator", // [476]
    "CATALOG_NAME", // [477]
    "CEIL", // [478]
    "CEILING", // [479]
    "CHAIN", // [480]
    "CHANGE", // [481]
    "CHARACTER", // [482]
    "CS", // [483]
    "UC", // [484]
    "UPPERCASE", // [485]
    "CharacterKeywordSegment", // [486]
    "SetKeywordSegment", // [487]
    "SET", // [488]
    "NotKeywordSegment", // [489]
    "CasespecificKeywordSegment", // [490]
    "CsKeywordSegment", // [491]
    "UppercaseKeywordSegment", // [492]
    "UcKeywordSegment", // [493]
    "CHAR", // [494]
    "CHAR_LENGTH", // [495]
    "CHARACTER_LENGTH", // [496]
    "CHARACTER_SET_CATALOG", // [497]
    "CHARACTER_SET_NAME", // [498]
    "CHARACTER_SET_SCHEMA", // [499]
    "CHARACTERISTICS", // [500]
    "CHARACTERS", // [501]
    "CHECK", // [502]
    "CHECKED", // [503]
    "CHECKPOINT", // [504]
    "CHECKSUM", // [505]
    "CLASS", // [506]
    "CLASS_ORIGIN", // [507]
    "CLOB", // [508]
    "CLOSE", // [509]
    "CLUSTER", // [510]
    "CLUSTERED", // [511]
    "COALESCE", // [512]
    "COBOL", // [513]
    "CharCharacterSetGrammar", // [514]
    "COLLATE", // [515]
    "COLLATION", // [516]
    "ObjectReferenceDelimiterGrammar", // [517]
    "ObjectReferenceTerminatorGrammar", // [518]
    "COLLATION_CATALOG", // [519]
    "COLLATION_NAME", // [520]
    "COLLATION_SCHEMA", // [521]
    "COLLECT", // [522]
    "colon_delimiter", // [523]
    "colon_prefix", // [524]
    "colon", // [525]
    "ShorthandCastSegment", // [526]
    "cast_expression", // [527]
    "CONSTRAINT", // [528]
    "ConstraintKeywordSegment", // [529]
    "NULL", // [530]
    "NullKeywordSegment", // [531]
    "CheckKeywordSegment", // [532]
    "DEFAULT", // [533]
    "DefaultKeywordSegment", // [534]
    "ColumnConstraintDefaultGrammar", // [535]
    "PRIMARY", // [536]
    "PrimaryKeyGrammar", // [537]
    "NotEnforcedGrammar", // [538]
    "UniqueKeyGrammar", // [539]
    "UNIQUE", // [540]
    "AutoIncrementGrammar", // [541]
    "ReferenceDefinitionGrammar", // [542]
    "CommentClauseSegment", // [543]
    "COMMENT", // [544]
    "comment_clause", // [545]
    "CollateKeywordSegment", // [546]
    "CollationReferenceSegment", // [547]
    "collation_reference", // [548]
    "ColumnGeneratedGrammar", // [549]
    "ColumnConstraintSegment", // [550]
    "column_constraint_segment", // [551]
    "TdColumnConstraintSegment", // [552]
    "COMPRESS", // [553]
    "td_column_attribute_constraint", // [554]
    "COLUMN_NAME", // [555]
    "ColumnsExpressionNameGrammar", // [556]
    "COLUMNS", // [557]
    "comma", // [558]
    "COMMAND_FUNCTION", // [559]
    "COMMAND_FUNCTION_CODE", // [560]
    "CommentKeywordSegment", // [561]
    "comment", // [562]
    "CommentSegment", // [563]
    "COMMIT", // [564]
    "COMMITTED", // [565]
    "EqualsSegment_a", // [566]
    "GreaterThanSegment", // [567]
    "GreaterThanSegment_a", // [568]
    "LessThanSegment", // [569]
    "LessThanSegment_a", // [570]
    "GreaterThanOrEqualToSegment", // [571]
    "GreaterThanOrEqualToSegment_a", // [572]
    "LessThanOrEqualToSegment", // [573]
    "LessThanOrEqualToSegment_a", // [574]
    "NotEqualToSegment", // [575]
    "NotEqualToSegment_a", // [576]
    "NotEqualToSegment_b", // [577]
    "NotEqualToSegment_c", // [578]
    "LikeOperatorSegment", // [579]
    "OverlapsOperatorSegment", // [580]
    "IsKeywordSegment", // [581]
    "DistinctKeywordSegment", // [582]
    "DISTINCT", // [583]
    "ComparisonOperatorSegment", // [584]
    "COMPLETION", // [585]
    "CompositeBinaryOperatorSegment", // [586]
    "CompositeComparisonOperatorSegment", // [587]
    "COMPUTE", // [588]
    "CONDITION", // [589]
    "CONDITION_NUMBER", // [590]
    "CrossKeywordSegment", // [591]
    "CROSS", // [592]
    "JoinTypeKeywordsGrammar", // [593]
    "FULL", // [594]
    "INNER", // [595]
    "LEFT", // [596]
    "RIGHT", // [597]
    "ConditionalCrossJoinKeywordsGrammar", // [598]
    "NonStandardJoinTypeKeywordsGrammar", // [599]
    "CONNECTION", // [600]
    "CONNECTION_NAME", // [601]
    "CONSTRAINT_CATALOG", // [602]
    "CONSTRAINT_NAME", // [603]
    "CONSTRAINT_SCHEMA", // [604]
    "CONSTRAINTS", // [605]
    "CONSTRUCTOR", // [606]
    "CONTAINS", // [607]
    "CONTAINSTABLE", // [608]
    "CONTINUE", // [609]
    "CONVERSION", // [610]
    "CONVERT", // [611]
    "COPY", // [612]
    "CORR", // [613]
    "CORRESPONDING", // [614]
    "COUNT", // [615]
    "COVAR_POP", // [616]
    "COVAR_SAMP", // [617]
    "CastKeywordSegment", // [618]
    "WithKeywordSegment", // [619]
    "SpecificKeywordSegment", // [620]
    "SPECIFIC", // [621]
    "INSTANCE", // [622]
    "METHOD", // [623]
    "STATIC", // [624]
    "InstanceKeywordSegment", // [625]
    "StaticKeywordSegment", // [626]
    "ConstructorKeywordSegment", // [627]
    "MethodKeywordSegment", // [628]
    "FOR", // [629]
    "ForKeywordSegment", // [630]
    "AssignmentKeywordSegment", // [631]
    "IfNotExistsGrammar", // [632]
    "DatabaseReferenceSegment", // [633]
    "database_reference", // [634]
    "OrReplaceGrammar", // [635]
    "TemporaryGrammar", // [636]
    "RETURNS", // [637]
    "ReturnsKeywordSegment", // [638]
    "FunctionDefinitionGrammar", // [639]
    "function_definition", // [640]
    "UniqueKeywordSegment", // [641]
    "IndexKeywordSegment", // [642]
    "INDEX", // [643]
    "IndexReferenceSegment", // [644]
    "index_reference", // [645]
    "IndexColumnDefinitionSegment", // [646]
    "index_column_definition", // [647]
    "ModelKeywordSegment", // [648]
    "MODEL", // [649]
    "OPTIONS", // [650]
    "OptionsKeywordSegment", // [651]
    "SchemaReferenceSegment", // [652]
    "schema_reference", // [653]
    "START", // [654]
    "StartKeywordSegment", // [655]
    "CreateSequenceOptionsSegment", // [656]
    "create_sequence_options_segment", // [657]
    "GLOBAL", // [658]
    "MULTISET", // [659]
    "VOLATILE", // [660]
    "MultisetKeywordSegment", // [661]
    "GlobalKeywordSegment", // [662]
    "VolatileKeywordSegment", // [663]
    "TdCreateTableOptions", // [664]
    "create_table_options_statement", // [665]
    "LIKE", // [666]
    "TableConstraintSegment", // [667]
    "table_constraint", // [668]
    "LikeKeywordSegment", // [669]
    "PARTITION", // [670]
    "TdTableConstraints", // [671]
    "td_table_constraint", // [672]
    "TriggerReferenceSegment", // [673]
    "trigger_reference", // [674]
    "INSTEAD", // [675]
    "BeforeKeywordSegment", // [676]
    "InsteadKeywordSegment", // [677]
    "OfKeywordSegment", // [678]
    "OF", // [679]
    "OrKeywordSegment", // [680]
    "DEFERRABLE", // [681]
    "INITIALLY", // [682]
    "REFERENCING", // [683]
    "ReferencingKeywordSegment", // [684]
    "OldKeywordSegment", // [685]
    "OLD", // [686]
    "RowKeywordSegment", // [687]
    "ROW", // [688]
    "NewKeywordSegment", // [689]
    "NEW", // [690]
    "DeferrableKeywordSegment", // [691]
    "InitiallyKeywordSegment", // [692]
    "ImmediateKeywordSegment", // [693]
    "IMMEDIATE", // [694]
    "DeferredKeywordSegment", // [695]
    "DEFERRED", // [696]
    "EachKeywordSegment", // [697]
    "EACH", // [698]
    "STATEMENT", // [699]
    "StatementKeywordSegment", // [700]
    "WhenKeywordSegment", // [701]
    "FunctionNameIdentifierSegment", // [702]
    "FunctionContentsSegment", // [703]
    "function_contents", // [704]
    "REPLACE", // [705]
    "ReplaceKeywordSegment", // [706]
    "WithNoSchemaBindingClauseSegment", // [707]
    "with_no_schema_binding_clause", // [708]
    "CREATEDB", // [709]
    "CREATEROLE", // [710]
    "CREATEUSER", // [711]
    "CSV", // [712]
    "CUBE", // [713]
    "function_name_identifier", // [714]
    "ROLLUP", // [715]
    "CubeFunctionNameSegment", // [716]
    "RollupFunctionNameSegment", // [717]
    "GroupingExpressionList", // [718]
    "grouping_expression_list", // [719]
    "CUME_DIST", // [720]
    "CURRENT", // [721]
    "CURRENT_DEFAULT_TRANSFORM_GROUP", // [722]
    "CURRENT_PATH", // [723]
    "CURRENT_ROLE", // [724]
    "CURRENT_TRANSFORM_GROUP_FOR_TYPE", // [725]
    "CURRENT_USER", // [726]
    "CURSOR", // [727]
    "CURSOR_NAME", // [728]
    "DATABASES", // [729]
    "data_type_identifier", // [730]
    "DatatypeIdentifierSegment", // [731]
    "BracketedArguments", // [732]
    "bracketed_arguments", // [733]
    "DATEADD", // [734]
    "DatePartFunctionName", // [735]
    "DatetimeUnitSegment", // [736]
    "DAY", // [737]
    "DAYOFYEAR", // [738]
    "HOUR", // [739]
    "MILLISECOND", // [740]
    "MINUTE", // [741]
    "MONTH", // [742]
    "QUARTER", // [743]
    "SECOND", // [744]
    "WEEK", // [745]
    "WEEKDAY", // [746]
    "YEAR", // [747]
    "FunctionContentsGrammar", // [748]
    "TIME", // [749]
    "TIMESTAMP", // [750]
    "DateKeywordSegment", // [751]
    "TimeKeywordSegment", // [752]
    "TimestampKeywordSegment", // [753]
    "IntervalKeywordSegment", // [754]
    "date_constructor_literal", // [755]
    "literal", // [756]
    "LiteralSegment", // [757]
    "DATETIME", // [758]
    "date_part", // [759]
    "DATETIME_INTERVAL_CODE", // [760]
    "DATETIME_INTERVAL_PRECISION", // [761]
    "DAY_HOUR", // [762]
    "DAY_MICROSECOND", // [763]
    "DAY_MINUTE", // [764]
    "DAY_SECOND", // [765]
    "DAYOFMONTH", // [766]
    "DAYOFWEEK", // [767]
    "DAYS", // [768]
    "DBCC", // [769]
    "DEALLOCATE", // [770]
    "DEC", // [771]
    "DECIMAL", // [772]
    "DECLARE", // [773]
    "ValuesKeywordSegment", // [774]
    "DEFAULTS", // [775]
    "DEFINED", // [776]
    "DEFINER", // [777]
    "DEGREE", // [778]
    "DEL", // [779]
    "DELAY_KEY_WRITE", // [780]
    "DELAYED", // [781]
    "DelKeywordSegment", // [782]
    "FromClauseSegment", // [783]
    "from_clause", // [784]
    "WhereClauseSegment", // [785]
    "WHERE", // [786]
    "where_clause", // [787]
    "SemicolonSegment", // [788]
    ";", // [789]
    "DELIMITER", // [790]
    "DELIMITERS", // [791]
    "DENSE_RANK", // [792]
    "DENY", // [793]
    "DEPTH", // [794]
    "DEREF", // [795]
    "DERIVED", // [796]
    "DESC", // [797]
    "DESCRIBE", // [798]
    "DescribeKeywordSegment", // [799]
    "DESCRIPTOR", // [800]
    "DESTROY", // [801]
    "DESTRUCTOR", // [802]
    "DETERMINISTIC", // [803]
    "DIAGNOSTICS", // [804]
    "DICTIONARY", // [805]
    "DISABLE", // [806]
    "DISCONNECT", // [807]
    "DISK", // [808]
    "DISPATCH", // [809]
    "DISTINCTROW", // [810]
    "DISTRIBUTED", // [811]
    "DIV", // [812]
    "DO", // [813]
    "dot", // [814]
    "DOUBLE", // [815]
    "RESTRICT", // [816]
    "RestrictKeywordSegment", // [817]
    "CascadeKeywordSegment", // [818]
    "DropBehaviorGrammar", // [819]
    "DUAL", // [820]
    "DUMMY", // [821]
    "DUMP", // [822]
    "DYNAMIC", // [823]
    "DYNAMIC_FUNCTION", // [824]
    "DYNAMIC_FUNCTION_CODE", // [825]
    "ELEMENT", // [826]
    "ELSEIF", // [827]
    "StructTypeSegment", // [828]
    "struct_type", // [829]
    "EmptyStructLiteralBracketsSegment", // [830]
    "struct_literal", // [831]
    "ENABLE", // [832]
    "ENCLOSED", // [833]
    "ENCODING", // [834]
    "ENCRYPTED", // [835]
    "END-EXEC", // [836]
    "}", // [837]
    "end_curly_bracket", // [838]
    "ENUM", // [839]
    "EQUALS", // [840]
    "RawEqualsSegment", // [841]
    "ERRLVL", // [842]
    "ESCAPE", // [843]
    "ESCAPED", // [844]
    "EVERY", // [845]
    "EXCEPT", // [846]
    "EXCEPTION", // [847]
    "EXCL", // [848]
    "EXCLUDE", // [849]
    "EXCLUDING", // [850]
    "EXCLUSIVE", // [851]
    "EXEC", // [852]
    "EXISTING", // [853]
    "EXISTS", // [854]
    "EXIT", // [855]
    "EXP", // [856]
    "EXPLAIN", // [857]
    "ExplainKeywordSegment", // [858]
    "InsertStatementSegment", // [859]
    "insert_statement", // [860]
    "UpdateStatementSegment", // [861]
    "update_statement", // [862]
    "DeleteStatementSegment", // [863]
    "delete_statement", // [864]
    "Expression_A_Grammar", // [865]
    "TeradataCastSegment", // [866]
    "Tail_Recurse_Expression_A_Grammar", // [867]
    "LikeExpressionGrammar", // [868]
    "ILIKE", // [869]
    "RLIKE", // [870]
    "InOperatorGrammar", // [871]
    "QuantifiedComparisonOperatorGrammar", // [872]
    "IsClauseGrammar", // [873]
    "IsNullGrammar", // [874]
    "NotNullGrammar", // [875]
    "CollateGrammar", // [876]
    "BetweenKeywordSegment", // [877]
    "Expression_B_Grammar", // [878]
    "AndKeywordSegment", // [879]
    "PatternMatchingGrammar", // [880]
    "PRIOR", // [881]
    "~", // [882]
    "SignedSegmentGrammar", // [883]
    "QualifiedNumericLiteralSegment", // [884]
    "TildeSegment", // [885]
    "NotOperatorGrammar", // [886]
    "PriorKeywordSegment", // [887]
    "Tail_Recurse_Expression_B_Grammar", // [888]
    "ExistsKeywordSegment", // [889]
    "Expression_D_Grammar", // [890]
    "CaseExpressionSegment", // [891]
    "case_expression", // [892]
    "TimeZoneGrammar", // [893]
    "time_zone_grammar", // [894]
    "LateralColumnAliasExpressionGrammar", // [895]
    "LocalAliasSegment", // [896]
    "local_alias_segment", // [897]
    "Expression_D_Potential_Select_Statement_Without_Brackets", // [898]
    "SingleIdentifierWildcardSegment", // [899]
    "MapTypeSegment", // [900]
    "map_type", // [901]
    "BooleanLiteralGrammar", // [902]
    "NullLiteralSegment", // [903]
    "DateTimeLiteralGrammar", // [904]
    "ListComprehensionGrammar", // [905]
    "AccessorGrammar", // [906]
    "SelectStatementSegment", // [907]
    "select_statement", // [908]
    "TypedStructLiteralSegment", // [909]
    "typed_struct_literal", // [910]
    "ArrayExpressionSegment", // [911]
    "array_expression", // [912]
    "OverlapsClauseSegment", // [913]
    "overlaps_clause", // [914]
    "EXTENSION", // [915]
    "EXTRACT", // [916]
    "FALLBACK", // [917]
    "boolean_literal", // [918]
    "LiteralKeywordSegment", // [919]
    "FETCH", // [920]
    "FetchKeywordSegment", // [921]
    "NEXT", // [922]
    "NextKeywordSegment", // [923]
    "ROWS", // [924]
    "RowsKeywordSegment", // [925]
    "ONLY", // [926]
    "OnlyKeywordSegment", // [927]
    "TiesKeywordSegment", // [928]
    "TIES", // [929]
    "FIELDS", // [930]
    "MERGE", // [931]
    "QUALIFY", // [932]
    "ROLLBACK", // [933]
    "SS", // [934]
    "DelimiterGrammar", // [935]
    "StatementSegment", // [936]
    "statement", // [937]
    "FILLFACTOR", // [938]
    "FILTER", // [939]
    "FilterKeywordSegment", // [940]
    "WhereKeywordSegment", // [941]
    "FINAL", // [942]
    "FLOAT4", // [943]
    "FLOAT8", // [944]
    "FLOAT", // [945]
    "FLOOR", // [946]
    "FLUSH", // [947]
    "FOLLOWING", // [948]
    "FORCE", // [949]
    "KeyKeywordSegment", // [950]
    "KEY", // [951]
    "FORTRAN", // [952]
    "FORWARD", // [953]
    "FOUND", // [954]
    "RANGE", // [955]
    "FrameClauseUnitGrammar", // [956]
    "CurrentKeywordSegment", // [957]
    "UnboundedKeywordSegment", // [958]
    "UNBOUNDED", // [959]
    "PRECEDING", // [960]
    "PrecedingKeywordSegment", // [961]
    "FollowingKeywordSegment", // [962]
    "RangeKeywordSegment", // [963]
    "FREE", // [964]
    "FREETEXT", // [965]
    "FREETEXTTABLE", // [966]
    "FREEZE", // [967]
    "FromExpressionSegment", // [968]
    "from_expression", // [969]
    "HAVING", // [970]
    "INTERSECT", // [971]
    "LIMIT", // [972]
    "MINUS", // [973]
    "OFFSET", // [974]
    "UNION", // [975]
    "WINDOW", // [976]
    "LimitClauseSegment", // [977]
    "limit_clause", // [978]
    "OrderKeywordSegment", // [979]
    "HavingKeywordSegment", // [980]
    "QualifyKeywordSegment", // [981]
    "WindowKeywordSegment", // [982]
    "SetOperatorSegment", // [983]
    "set_operator", // [984]
    "WithDataClauseSegment", // [985]
    "with_data_clause", // [986]
    "OffsetKeywordSegment", // [987]
    "PreTableFunctionKeywordsGrammar", // [988]
    "TableExpressionSegment", // [989]
    "table_expression", // [990]
    "TemporalQuerySegment", // [991]
    "temporal_query", // [992]
    "FromClauseTerminatorGrammar", // [993]
    "SamplingExpressionSegment", // [994]
    "TABLESAMPLE", // [995]
    "sample_expression", // [996]
    "JoinLikeClauseGrammar", // [997]
    "JoinClauseSegment", // [998]
    "join_clause", // [999]
    "PostTableExpressionGrammar", // [1000]
    "MLTableExpressionSegment", // [1001]
    "ML", // [1002]
    "ml_table_expression", // [1003]
    "FromExpressionElementSegment", // [1004]
    "from_expression_element", // [1005]
    "indented_joins", // [1006]
    "FULLTEXT", // [1007]
    "LEADING", // [1008]
    "TRAILING", // [1009]
    "TrimParametersGrammar", // [1010]
    "StarSegment", // [1011]
    "FunctionContentsExpressionGrammar", // [1012]
    "AggregateOrderByClause", // [1013]
    "aggregate_order_by", // [1014]
    "SEPARATOR", // [1015]
    "SeparatorKeywordSegment", // [1016]
    "IgnoreRespectNullsGrammar", // [1017]
    "IGNORE", // [1018]
    "RESPECT", // [1019]
    "EmptyStructLiteralSegment", // [1020]
    "WordSegment", // [1021]
    "double_quote", // [1022]
    "QuotedIdentifierSegment", // [1023]
    "AnyKeywordSegment", // [1024]
    "FunctionParameterGrammar", // [1025]
    "DatePartFunctionNameSegment", // [1026]
    "DateTimeFunctionContentsSegment", // [1027]
    "ColumnsExpressionGrammar", // [1028]
    "ColumnsExpressionFunctionNameSegment", // [1029]
    "ValuesClauseSegment", // [1030]
    "values_clause", // [1031]
    "PostFunctionGrammar", // [1032]
    "FUSION", // [1033]
    "G", // [1034]
    "GENERAL", // [1035]
    "GENERATED", // [1036]
    "GET", // [1037]
    "glob_operator", // [1038]
    "GO", // [1039]
    "GrantKeywordSegment", // [1040]
    "AccessPermissionsSegment", // [1041]
    "access_permissions", // [1042]
    "AccessObjectSegment", // [1043]
    "access_object", // [1044]
    "AccessTargetSegment", // [1045]
    "access_target", // [1046]
    "OptionKeywordSegment", // [1047]
    "OPTION", // [1048]
    "AdminKeywordSegment", // [1049]
    "CopyKeywordSegment", // [1050]
    "GRANTED", // [1051]
    "GrantedKeywordSegment", // [1052]
    "Current_userKeywordSegment", // [1053]
    "Session_userKeywordSegment", // [1054]
    "SESSION_USER", // [1055]
    "GREATEST", // [1056]
    "GroupingSetsClauseSegment", // [1057]
    "GROUPING", // [1058]
    "grouping_sets_clause", // [1059]
    "CubeRollupClauseSegment", // [1060]
    "cube_rollup_clause", // [1061]
    "GroupByClauseTerminatorGrammar", // [1062]
    "GroupingKeywordSegment", // [1063]
    "SetsKeywordSegment", // [1064]
    "SETS", // [1065]
    "HANDLER", // [1066]
    "HASH", // [1067]
    "HEADER", // [1068]
    "HEAP", // [1069]
    "HIERARCHY", // [1070]
    "HIGH_PRIORITY", // [1071]
    "HOLD", // [1072]
    "HOLDLOCK", // [1073]
    "HOST", // [1074]
    "HOSTS", // [1075]
    "HOUR_MICROSECOND", // [1076]
    "HOUR_MINUTE", // [1077]
    "HOUR_SECOND", // [1078]
    "IDENTIFIED", // [1079]
    "identifier", // [1080]
    "IdentifierSegment", // [1081]
    "IDENTITY", // [1082]
    "IDENTITY_INSERT", // [1083]
    "IDENTITYCOL", // [1084]
    "IgnoreKeywordSegment", // [1085]
    "RespectKeywordSegment", // [1086]
    "NullsKeywordSegment", // [1087]
    "NULLS", // [1088]
    "IMMUTABLE", // [1089]
    "IMPLEMENTATION", // [1090]
    "IMPLICIT", // [1091]
    "INCLUDE", // [1092]
    "INCLUDING", // [1093]
    "AscKeywordSegment", // [1094]
    "DescKeywordSegment", // [1095]
    "INDICATOR", // [1096]
    "INFILE", // [1097]
    "INFIX", // [1098]
    "INHERIT", // [1099]
    "INHERITS", // [1100]
    "INITIAL", // [1101]
    "INITIALIZE", // [1102]
    "INOUT", // [1103]
    "INPUT", // [1104]
    "INSENSITIVE", // [1105]
    "OverwriteKeywordSegment", // [1106]
    "OVERWRITE", // [1107]
    "IntoKeywordSegment", // [1108]
    "INTO", // [1109]
    "DefaultValuesGrammar", // [1110]
    "INSERT_ID", // [1111]
    "INSTANTIABLE", // [1112]
    "INT1", // [1113]
    "INT2", // [1114]
    "INT3", // [1115]
    "INT4", // [1116]
    "INT8", // [1117]
    "INT", // [1118]
    "INTEGER", // [1119]
    "INTEGRATIONS", // [1120]
    "INTERSECTION", // [1121]
    "INVOKER", // [1122]
    "NanLiteralSegment", // [1123]
    "NAN", // [1124]
    "UnknownLiteralSegment", // [1125]
    "NormalizedGrammar", // [1126]
    "ISAM", // [1127]
    "ISNULL", // [1128]
    "ISOLATION", // [1129]
    "ITERATE", // [1130]
    "ConditionalJoinKeywordsGrammar", // [1131]
    "JoinKeywordsGrammar", // [1132]
    "JOIN", // [1133]
    "NestedJoinGrammar", // [1134]
    "indented_using_on", // [1135]
    "MatchConditionSegment", // [1136]
    "match_condition", // [1137]
    "USING", // [1138]
    "JoinOnConditionSegment", // [1139]
    "join_on_condition", // [1140]
    "JoinUsingConditionGrammar", // [1141]
    "UnconditionalJoinKeywordsGrammar", // [1142]
    "ExtendedNaturalJoinKeywordsGrammar", // [1143]
    "JoinKeywordSegment", // [1144]
    "indented_on_contents", // [1145]
    "InnerKeywordSegment", // [1146]
    "FullKeywordSegment", // [1147]
    "LeftKeywordSegment", // [1148]
    "RightKeywordSegment", // [1149]
    "OuterKeywordSegment", // [1150]
    "OUTER", // [1151]
    "UsingKeywordSegment", // [1152]
    "JOURNAL", // [1153]
    "JSON", // [1154]
    "K", // [1155]
    "KEY_MEMBER", // [1156]
    "KEY_TYPE", // [1157]
    "KEYS", // [1158]
    "KILL", // [1159]
    "LANCOMPILER", // [1160]
    "LAST", // [1161]
    "LAST_INSERT_ID", // [1162]
    "LATERAL", // [1163]
    "LEAST", // [1164]
    "LEAVE", // [1165]
    "LENGTH", // [1166]
    "LESS", // [1167]
    "LEVEL", // [1168]
    "LikeGrammar", // [1169]
    "EscapeKeywordSegment", // [1170]
    "RlikeKeywordSegment", // [1171]
    "IlikeKeywordSegment", // [1172]
    "LimitKeywordSegment", // [1173]
    "LINENO", // [1174]
    "LINES", // [1175]
    "LISTEN", // [1176]
    "ArrayLiteralSegment", // [1177]
    "array_literal", // [1178]
    "TypedArrayLiteralSegment", // [1179]
    "typed_array_literal", // [1180]
    "ObjectLiteralSegment", // [1181]
    "{", // [1182]
    "object_literal", // [1183]
    "LN", // [1184]
    "LOAD", // [1185]
    "LOCAL", // [1186]
    "LOCALTIME", // [1187]
    "LOCALTIMESTAMP", // [1188]
    "LOCATION", // [1189]
    "LOCATOR", // [1190]
    "LockingKeywordSegment", // [1191]
    "LockKeywordSegment", // [1192]
    "AccessKeywordSegment", // [1193]
    "ExclusiveKeywordSegment", // [1194]
    "ExclKeywordSegment", // [1195]
    "OVERRIDE", // [1196]
    "OverrideKeywordSegment", // [1197]
    "ChecksumKeywordSegment", // [1198]
    "LoadKeywordSegment", // [1199]
    "CommittedKeywordSegment", // [1200]
    "MODE", // [1201]
    "ModeKeywordSegment", // [1202]
    "NOWAIT", // [1203]
    "NowaitKeywordSegment", // [1204]
    "LOCKS", // [1205]
    "LOGIN", // [1206]
    "LOGS", // [1207]
    "LONG", // [1208]
    "LONGBLOB", // [1209]
    "LONGTEXT", // [1210]
    "LOOP", // [1211]
    "LOW_PRIORITY", // [1212]
    "LOWER", // [1213]
    "M", // [1214]
    "MlKeywordSegment", // [1215]
    "MACRO", // [1216]
    "MAP", // [1217]
    "MATCH", // [1218]
    "MATCHED", // [1219]
    "MAX", // [1220]
    "MAX_ROWS", // [1221]
    "MAXEXTENTS", // [1222]
    "MAXINTERVALS", // [1223]
    "MAXVALUELENGTH", // [1224]
    "MEDIUMBLOB", // [1225]
    "MEDIUMINT", // [1226]
    "MEDIUMTEXT", // [1227]
    "MEETS", // [1228]
    "MEMBER", // [1229]
    "MergeKeywordSegment", // [1230]
    "MergeMatchedClauseSegment", // [1231]
    "merge_when_matched_clause", // [1232]
    "MergeNotMatchedClauseSegment", // [1233]
    "merge_when_not_matched_clause", // [1234]
    "MatchedKeywordSegment", // [1235]
    "MergeUpdateClauseSegment", // [1236]
    "merge_update_clause", // [1237]
    "MergeDeleteClauseSegment", // [1238]
    "merge_delete_clause", // [1239]
    "MergeInsertClauseSegment", // [1240]
    "merge_insert_clause", // [1241]
    "MergeIntoLiteralGrammar", // [1242]
    "AliasedTableReferenceGrammar", // [1243]
    "MergeMatchSegment", // [1244]
    "merge_match", // [1245]
    "SetClauseListSegment", // [1246]
    "set_clause_list", // [1247]
    "MERGEBLOCKRATIO", // [1248]
    "MESSAGE_LENGTH", // [1249]
    "MESSAGE_OCTET_LENGTH", // [1250]
    "MESSAGE_TEXT", // [1251]
    "MIDDLEINT", // [1252]
    "MIN", // [1253]
    "MIN_ROWS", // [1254]
    "MINUTE_MICROSECOND", // [1255]
    "MINUTE_SECOND", // [1256]
    "MLSLABEL", // [1257]
    "MOD", // [1258]
    "MODIFIES", // [1259]
    "MODULE", // [1260]
    "MONTHNAME", // [1261]
    "MORE", // [1262]
    "MOVE", // [1263]
    "MUMPS", // [1264]
    "MYISAM", // [1265]
    "naked_identifier", // [1266]
    "NAME", // [1267]
    "WindowSpecificationSegment", // [1268]
    "window_specification", // [1269]
    "NamedWindowExpressionSegment", // [1270]
    "named_window_expression", // [1271]
    "NAMES", // [1272]
    "null_literal", // [1273]
    "NATIONAL", // [1274]
    "NATURAL", // [1275]
    "NaturalKeywordSegment", // [1276]
    "NCHAR", // [1277]
    "NCLOB", // [1278]
    "sign_indicator", // [1279]
    "NESTING", // [1280]
    "newline", // [1281]
    "NewlineSegment", // [1282]
    "NO_WRITE_TO_BINLOG", // [1283]
    "NOAUDIT", // [1284]
    "NOCHECK", // [1285]
    "NOCOMPRESS", // [1286]
    "NOCREATEDB", // [1287]
    "NOCREATEROLE", // [1288]
    "NOCREATEUSER", // [1289]
    "NOINHERIT", // [1290]
    "NOLOGIN", // [1291]
    "UnorderedSelectStatementSegment", // [1292]
    "WithCompoundStatementSegment", // [1293]
    "with_compound_statement", // [1294]
    "NonSetSelectableGrammar", // [1295]
    "BracketedSetExpressionGrammar", // [1296]
    "MergeStatementSegment", // [1297]
    "merge_statement", // [1298]
    "SetExpressionSegment", // [1299]
    "NONCLUSTERED", // [1300]
    "NONE", // [1301]
    "NORMALIZE", // [1302]
    "NORMALIZED", // [1303]
    "NOSUPERUSER", // [1304]
    "RawNotSegment", // [1305]
    "NOTHING", // [1306]
    "NOTIFY", // [1307]
    "NOTNULL", // [1308]
    "NULLABLE", // [1309]
    "NULLIF", // [1310]
    "NUMBER", // [1311]
    "NUMERIC", // [1312]
    "ColonSegment", // [1313]
    "ObjectLiteralElementSegment", // [1314]
    "object_literal_element", // [1315]
    "start_curly_bracket", // [1316]
    "CastOperatorSegment", // [1317]
    "StartSquareBracketSegment", // [1318]
    "StartBracketSegment", // [1319]
    "OBJECTS", // [1320]
    "OCTET_LENGTH", // [1321]
    "OCTETS", // [1322]
    "OFF", // [1323]
    "OFFLINE", // [1324]
    "OFFSETS", // [1325]
    "OIDS", // [1326]
    "ONLINE", // [1327]
    "OPEN", // [1328]
    "OPENDATASOURCE", // [1329]
    "OPENQUERY", // [1330]
    "OPENROWSET", // [1331]
    "OPENXML", // [1332]
    "OPERATION", // [1333]
    "OPERATOR", // [1334]
    "OPTIMIZE", // [1335]
    "OPTIONALLY", // [1336]
    "LastKeywordSegment", // [1337]
    "WithFillSegment", // [1338]
    "with_fill", // [1339]
    "NoorderKeywordSegment", // [1340]
    "ORDERING", // [1341]
    "ORDINALITY", // [1342]
    "OTHERS", // [1343]
    "OUT", // [1344]
    "OUTFILE", // [1345]
    "OUTPUT", // [1346]
    "OverKeywordSegment", // [1347]
    "OVER", // [1348]
    "OverlapsKeywordSegment", // [1349]
    "OVERLAY", // [1350]
    "OVERRIDING", // [1351]
    "OWNER", // [1352]
    "PACK_KEYS", // [1353]
    "PAD", // [1354]
    "PARAMETER", // [1355]
    "parameter", // [1356]
    "?", // [1357]
    "PARAMETER_MODE", // [1358]
    "PARAMETER_NAME", // [1359]
    "PARAMETER_ORDINAL_POSITION", // [1360]
    "PARAMETER_SPECIFIC_CATALOG", // [1361]
    "PARAMETER_SPECIFIC_NAME", // [1362]
    "PARAMETER_SPECIFIC_SCHEMA", // [1363]
    "PARAMETERS", // [1364]
    "PARTIAL", // [1365]
    "PartitionKeywordSegment", // [1366]
    "PASCAL", // [1367]
    "PASSWORD", // [1368]
    "PATH", // [1369]
    "path_segment", // [1370]
    "PCTFREE", // [1371]
    "PERCENT", // [1372]
    "PERCENT_RANK", // [1373]
    "PERCENTILE_CONT", // [1374]
    "PERCENTILE_DISC", // [1375]
    "pipe", // [1376]
    "PLACING", // [1377]
    "PLAN", // [1378]
    "PLI", // [1379]
    "POSITION", // [1380]
    "OverClauseSegment", // [1381]
    "over_clause", // [1382]
    "FilterClauseGrammar", // [1383]
    "POSTFIX", // [1384]
    "POWER", // [1385]
    "PRECISION", // [1386]
    "PREFIX", // [1387]
    "PREORDER", // [1388]
    "PREPARE", // [1389]
    "PREPARED", // [1390]
    "PRESERVE", // [1391]
    "PrimaryKeywordSegment", // [1392]
    "PRINT", // [1393]
    "PROC", // [1394]
    "PROCEDURAL", // [1395]
    "PROCESS", // [1396]
    "PROCESSLIST", // [1397]
    "PROFILE", // [1398]
    "PROTECTION", // [1399]
    "PURGE", // [1400]
    "SOME", // [1401]
    "SomeKeywordSegment", // [1402]
    "QUERY_BAND", // [1403]
    "QUOTE", // [1404]
    "quoted_identifier", // [1405]
    "quoted_literal", // [1406]
    "RAID0", // [1407]
    "RAISERROR", // [1408]
    "RANK", // [1409]
    "raw_comparison_operator", // [1410]
    "RAW", // [1411]
    "READS", // [1412]
    "READTEXT", // [1413]
    "REAL", // [1414]
    "RECHECK", // [1415]
    "RECONFIGURE", // [1416]
    "RECURSIVE", // [1417]
    "REF", // [1418]
    "ReferenceMatchGrammar", // [1419]
    "ReferentialActionGrammar", // [1420]
    "MatchKeywordSegment", // [1421]
    "SIMPLE", // [1422]
    "PartialKeywordSegment", // [1423]
    "SimpleKeywordSegment", // [1424]
    "NoKeywordSegment", // [1425]
    "ActionKeywordSegment", // [1426]
    "REGEXP", // [1427]
    "REGR_AVGX", // [1428]
    "REGR_AVGY", // [1429]
    "REGR_COUNT", // [1430]
    "REGR_INTERCEPT", // [1431]
    "REGR_R2", // [1432]
    "REGR_SLOPE", // [1433]
    "REGR_SXX", // [1434]
    "REGR_SXY", // [1435]
    "REGR_SYY", // [1436]
    "REINDEX", // [1437]
    "RELATIVE", // [1438]
    "RELEASE", // [1439]
    "RELOAD", // [1440]
    "REPEAT", // [1441]
    "REPEATABLE", // [1442]
    "REPLICATION", // [1443]
    "REQUIRE", // [1444]
    "RESET", // [1445]
    "RESIGNAL", // [1446]
    "RESTART", // [1447]
    "RESTORE", // [1448]
    "RESULT", // [1449]
    "RETURN", // [1450]
    "RETURNED_CARDINALITY", // [1451]
    "RETURNED_LENGTH", // [1452]
    "RETURNED_OCTET_LENGTH", // [1453]
    "RETURNED_SQLSTATE", // [1454]
    "RevokeKeywordSegment", // [1455]
    "ROLES", // [1456]
    "ROUTINE_CATALOG", // [1457]
    "ROUTINE_NAME", // [1458]
    "ROUTINE_SCHEMA", // [1459]
    "ROW_COUNT", // [1460]
    "ROW_NUMBER", // [1461]
    "ROWCOUNT", // [1462]
    "ROWGUIDCOL", // [1463]
    "ROWID", // [1464]
    "ROWNUM", // [1465]
    "RULE", // [1466]
    "SAMPLE", // [1467]
    "TablesampleKeywordSegment", // [1468]
    "SYSTEM", // [1469]
    "BernoulliKeywordSegment", // [1470]
    "SystemKeywordSegment", // [1471]
    "RepeatableKeywordSegment", // [1472]
    "SAVE", // [1473]
    "SAVEPOINT", // [1474]
    "SCALE", // [1475]
    "SCHEMA_NAME", // [1476]
    "SCOPE", // [1477]
    "SCOPE_CATALOG", // [1478]
    "SCOPE_NAME", // [1479]
    "SCOPE_SCHEMA", // [1480]
    "SCROLL", // [1481]
    "SEARCH", // [1482]
    "SECOND_MICROSECOND", // [1483]
    "SECTION", // [1484]
    "SECURITY", // [1485]
    "WildcardExpressionSegment", // [1486]
    "wildcard_expression", // [1487]
    "TOP", // [1488]
    "TopKeywordSegment", // [1489]
    "PercentKeywordSegment", // [1490]
    "NormalizeKeywordSegment", // [1491]
    "MeetsKeywordSegment", // [1492]
    "SelKeywordSegment", // [1493]
    "SelectClauseModifierSegment", // [1494]
    "select_clause_modifier", // [1495]
    "SelectClauseElementSegment", // [1496]
    "select_clause_element", // [1497]
    "LockingClauseSegment", // [1498]
    "locking_clause", // [1499]
    "SelectClauseSegment", // [1500]
    "select_clause", // [1501]
    "GroupByClauseSegment", // [1502]
    "groupby_clause", // [1503]
    "HavingClauseSegment", // [1504]
    "having_clause", // [1505]
    "NamedWindowSegment", // [1506]
    "named_window", // [1507]
    "QualifyClauseSegment", // [1508]
    "qualify_clause", // [1509]
    "OffsetClauseSegment", // [1510]
    "offset_clause", // [1511]
    "FetchClauseSegment", // [1512]
    "fetch_clause", // [1513]
    "WithCompoundNonSelectStatementSegment", // [1514]
    "NonWithSelectableGrammar", // [1515]
    "SELF", // [1516]
    "statement_terminator", // [1517]
    "SENSITIVE", // [1518]
    "MaxvalueKeywordSegment", // [1519]
    "MinvalueKeywordSegment", // [1520]
    "SERIALIZABLE", // [1521]
    "SERVER_NAME", // [1522]
    "SESSION", // [1523]
    "SetClauseSegment", // [1524]
    "set_clause", // [1525]
    "UnionGrammar", // [1526]
    "IntersectKeywordSegment", // [1527]
    "ExceptKeywordSegment", // [1528]
    "MinusKeywordSegment", // [1529]
    "Query_bandKeywordSegment", // [1530]
    "NoneKeywordSegment", // [1531]
    "TRANSACTION", // [1532]
    "SessionKeywordSegment", // [1533]
    "TransactionKeywordSegment", // [1534]
    "SsKeywordSegment", // [1535]
    "DatabaseStatementSegment", // [1536]
    "database_statement", // [1537]
    "SETOF", // [1538]
    "SETUSER", // [1539]
    "SHARES", // [1540]
    "SHOW", // [1541]
    "SHUTDOWN", // [1542]
    "SIGNAL", // [1543]
    "PositiveSegment", // [1544]
    "NegativeSegment", // [1545]
    "SIMILAR", // [1546]
    "SIZE", // [1547]
    "ArrayTypeSegment", // [1548]
    "array_type", // [1549]
    "slash", // [1550]
    "slice", // [1551]
    "SMALLINT", // [1552]
    "SONAME", // [1553]
    "SOURCE", // [1554]
    "SPACE", // [1555]
    "SPATIAL", // [1556]
    "SPECIFIC_NAME", // [1557]
    "SPECIFICTYPE", // [1558]
    "SQL", // [1559]
    "SQL_BIG_RESULT", // [1560]
    "SQL_BIG_SELECTS", // [1561]
    "SQL_BIG_TABLES", // [1562]
    "SQL_CALC_FOUND_ROWS", // [1563]
    "SQL_LOG_OFF", // [1564]
    "SQL_LOG_UPDATE", // [1565]
    "SQL_LOW_PRIORITY_UPDATES", // [1566]
    "SQL_SELECT_LIMIT", // [1567]
    "SQL_SMALL_RESULT", // [1568]
    "SQL_WARNINGS", // [1569]
    "SQLCA", // [1570]
    "SQLCODE", // [1571]
    "SQLERROR", // [1572]
    "SQLEXCEPTION", // [1573]
    "SQLSTATE", // [1574]
    "SQLWARNING", // [1575]
    "SQRT", // [1576]
    "SSL", // [1577]
    "STABLE", // [1578]
    "star", // [1579]
    "STARTING", // [1580]
    "STARTS", // [1581]
    "STAT", // [1582]
    "STATE", // [1583]
    "TransactionStatementSegment", // [1584]
    "transaction_statement", // [1585]
    "DropTableStatementSegment", // [1586]
    "drop_table_statement", // [1587]
    "DropViewStatementSegment", // [1588]
    "drop_view_statement", // [1589]
    "CreateUserStatementSegment", // [1590]
    "create_user_statement", // [1591]
    "DropUserStatementSegment", // [1592]
    "drop_user_statement", // [1593]
    "TruncateStatementSegment", // [1594]
    "truncate_table", // [1595]
    "AccessStatementSegment", // [1596]
    "access_statement", // [1597]
    "CreateTableStatementSegment", // [1598]
    "create_table_statement", // [1599]
    "CreateRoleStatementSegment", // [1600]
    "create_role_statement", // [1601]
    "DropRoleStatementSegment", // [1602]
    "drop_role_statement", // [1603]
    "AlterTableStatementSegment", // [1604]
    "alter_table_statement", // [1605]
    "CreateSchemaStatementSegment", // [1606]
    "create_schema_statement", // [1607]
    "SetSchemaStatementSegment", // [1608]
    "set_schema_statement", // [1609]
    "DropSchemaStatementSegment", // [1610]
    "drop_schema_statement", // [1611]
    "DropTypeStatementSegment", // [1612]
    "drop_type_statement", // [1613]
    "CreateDatabaseStatementSegment", // [1614]
    "create_database_statement", // [1615]
    "DropDatabaseStatementSegment", // [1616]
    "drop_database_statement", // [1617]
    "CreateIndexStatementSegment", // [1618]
    "create_index_statement", // [1619]
    "DropIndexStatementSegment", // [1620]
    "drop_index_statement", // [1621]
    "CreateViewStatementSegment", // [1622]
    "create_view_statement", // [1623]
    "CreateCastStatementSegment", // [1624]
    "create_cast_statement", // [1625]
    "DropCastStatementSegment", // [1626]
    "drop_cast_statement", // [1627]
    "CreateFunctionStatementSegment", // [1628]
    "create_function_statement", // [1629]
    "DropFunctionStatementSegment", // [1630]
    "drop_function_statement", // [1631]
    "CreateModelStatementSegment", // [1632]
    "create_model_statement", // [1633]
    "DropModelStatementSegment", // [1634]
    "drop_MODELstatement", // [1635]
    "DescribeStatementSegment", // [1636]
    "describe_statement", // [1637]
    "UseStatementSegment", // [1638]
    "use_statement", // [1639]
    "ExplainStatementSegment", // [1640]
    "explain_statement", // [1641]
    "CreateSequenceStatementSegment", // [1642]
    "create_sequence_statement", // [1643]
    "AlterSequenceStatementSegment", // [1644]
    "alter_sequence_statement", // [1645]
    "DropSequenceStatementSegment", // [1646]
    "drop_sequence_statement", // [1647]
    "CreateTriggerStatementSegment", // [1648]
    "create_trigger", // [1649]
    "DropTriggerStatementSegment", // [1650]
    "drop_trigger", // [1651]
    "TdCollectStatisticsStatementSegment", // [1652]
    "collect_statistics_statement", // [1653]
    "BteqStatementSegment", // [1654]
    "bteq_statement", // [1655]
    "TdRenameStatementSegment", // [1656]
    "rename_table_statement", // [1657]
    "TdCommentStatementSegment", // [1658]
    "SetSessionStatementSegment", // [1659]
    "set_session_statement", // [1660]
    "SetQueryBandStatementSegment", // [1661]
    "set_query_band_statement", // [1662]
    "STATISTICS", // [1663]
    "STATS", // [1664]
    "STDDEV_POP", // [1665]
    "STDDEV_SAMP", // [1666]
    "STDIN", // [1667]
    "STDOUT", // [1668]
    "STORAGE", // [1669]
    "STRAIGHT_JOIN", // [1670]
    "STRICT", // [1671]
    "ConcatSegment", // [1672]
    "STRING", // [1673]
    "STRUCTURE", // [1674]
    "STYLE", // [1675]
    "SUBCLASS_ORIGIN", // [1676]
    "SUBLIST", // [1677]
    "SUBMULTISET", // [1678]
    "SUBSTRING", // [1679]
    "SUCCESSFUL", // [1680]
    "SUM", // [1681]
    "SUMMARY", // [1682]
    "SUPERUSER", // [1683]
    "SYMMETRIC", // [1684]
    "SYNONYM", // [1685]
    "SYSDATE", // [1686]
    "SYSID", // [1687]
    "SYSTEM_USER", // [1688]
    "ForeignKeyGrammar", // [1689]
    "TABLE_NAME", // [1690]
    "Expression_A_Unary_Operator_Grammar", // [1691]
    "Expression_C_Grammar", // [1692]
    "Expression_B_Unary_Operator_Grammar", // [1693]
    "THRESHOLD", // [1694]
    "SampleKeywordSegment", // [1695]
    "ThresholdKeywordSegment", // [1696]
    "DaysKeywordSegment", // [1697]
    "MaxintervalsKeywordSegment", // [1698]
    "MaxvaluelengthKeywordSegment", // [1699]
    "CollectKeywordSegment", // [1700]
    "SummaryKeywordSegment", // [1701]
    "StatKeywordSegment", // [1702]
    "StatsKeywordSegment", // [1703]
    "StatisticsKeywordSegment", // [1704]
    "TdCollectStatUsingOptionClauseSegment", // [1705]
    "collect_stat_using_option_clause", // [1706]
    "TdOrderByStatClauseSegment", // [1707]
    "stat_orderby_clause", // [1708]
    "CompressKeywordSegment", // [1709]
    "MacroKeywordSegment", // [1710]
    "MapKeywordSegment", // [1711]
    "ProfileKeywordSegment", // [1712]
    "FallbackKeywordSegment", // [1713]
    "ProtectionKeywordSegment", // [1714]
    "DualKeywordSegment", // [1715]
    "LocalKeywordSegment", // [1716]
    "JournalKeywordSegment", // [1717]
    "OffKeywordSegment", // [1718]
    "MergeblockratioKeywordSegment", // [1719]
    "HashKeywordSegment", // [1720]
    "TdTablePartitioningLevel", // [1721]
    "td_partitioning_level", // [1722]
    "CommitKeywordSegment", // [1723]
    "PreserveKeywordSegment", // [1724]
    "TEMPLATE", // [1725]
    "TRANSIENT", // [1726]
    "TransientKeywordSegment", // [1727]
    "TERMINATE", // [1728]
    "TERMINATED", // [1729]
    "TEXT", // [1730]
    "TEXTSIZE", // [1731]
    "THAN", // [1732]
    "tilde", // [1733]
    "WITHOUT", // [1734]
    "WithoutKeywordSegment", // [1735]
    "ZoneKeywordSegment", // [1736]
    "ZONE", // [1737]
    "AtKeywordSegment", // [1738]
    "TIMEZONE_HOUR", // [1739]
    "TIMEZONE_MINUTE", // [1740]
    "TINYBLOB", // [1741]
    "TINYINT", // [1742]
    "TINYTEXT", // [1743]
    "TOAST", // [1744]
    "TOP_LEVEL_COUNT", // [1745]
    "TRAN", // [1746]
    "BeginKeywordSegment", // [1747]
    "RollbackKeywordSegment", // [1748]
    "WORK", // [1749]
    "WorkKeywordSegment", // [1750]
    "NameKeywordSegment", // [1751]
    "ChainKeywordSegment", // [1752]
    "TRANSACTION_ACTIVE", // [1753]
    "TRANSACTIONS", // [1754]
    "TRANSACTIONS_COMMITTED", // [1755]
    "TRANSACTIONS_ROLLED_BACK", // [1756]
    "TRANSFORM", // [1757]
    "TRANSFORMS", // [1758]
    "TRANSLATE", // [1759]
    "TRANSLATION", // [1760]
    "TREAT", // [1761]
    "TRIGGER_CATALOG", // [1762]
    "TRIGGER_NAME", // [1763]
    "TRIGGER_SCHEMA", // [1764]
    "TRIM", // [1765]
    "BothKeywordSegment", // [1766]
    "LeadingKeywordSegment", // [1767]
    "TrailingKeywordSegment", // [1768]
    "TRUSTED", // [1769]
    "TSEQUAL", // [1770]
    "StructLiteralSegment", // [1771]
    "UESCAPE", // [1772]
    "UID", // [1773]
    "UNCOMMITTED", // [1774]
    "NaturalJoinKeywordsGrammar", // [1775]
    "UnconditionalCrossJoinKeywordsGrammar", // [1776]
    "HorizontalJoinKeywordsGrammar", // [1777]
    "UNDER", // [1778]
    "UNDO", // [1779]
    "UNENCRYPTED", // [1780]
    "UnionKeywordSegment", // [1781]
    "UNKNOWN", // [1782]
    "UNLISTEN", // [1783]
    "UNLOCK", // [1784]
    "UNNAMED", // [1785]
    "UNNEST", // [1786]
    "UNSIGNED", // [1787]
    "UNTIL", // [1788]
    "FromUpdateClauseSegment", // [1789]
    "from_in_update_clause", // [1790]
    "UPDATETEXT", // [1791]
    "UPPER", // [1792]
    "USER_DEFINED_TYPE_CATALOG", // [1793]
    "USER_DEFINED_TYPE_CODE", // [1794]
    "USER_DEFINED_TYPE_NAME", // [1795]
    "USER_DEFINED_TYPE_SCHEMA", // [1796]
    "USERS", // [1797]
    "UTC_DATE", // [1798]
    "UTC_TIME", // [1799]
    "UTC_TIMESTAMP", // [1800]
    "VACUUM", // [1801]
    "VALID", // [1802]
    "VALIDATE", // [1803]
    "VALIDATOR", // [1804]
    "ValueKeywordSegment", // [1805]
    "VAR_POP", // [1806]
    "VAR_SAMP", // [1807]
    "VARBINARY", // [1808]
    "VARCHAR2", // [1809]
    "VARCHAR", // [1810]
    "VARCHARACTER", // [1811]
    "VARIABLE", // [1812]
    "VARIABLES", // [1813]
    "VARYING", // [1814]
    "VERBOSE", // [1815]
    "VERSION", // [1816]
    "WAITFOR", // [1817]
    "WAREHOUSES", // [1818]
    "indented_then", // [1819]
    "indented_then_contents", // [1820]
    "WHENEVER", // [1821]
    "WHILE", // [1822]
    "whitespace", // [1823]
    "WhitespaceSegment", // [1824]
    "WIDTH_BUCKET", // [1825]
    "PartitionClauseSegment", // [1826]
    "partitionby_clause", // [1827]
    "FrameClauseSegment", // [1828]
    "frame_clause", // [1829]
    "RecursiveKeywordSegment", // [1830]
    "indented_ctes", // [1831]
    "CTEDefinitionSegment", // [1832]
    "common_table_expression", // [1833]
    "NonWithNonSelectableGrammar", // [1834]
    "BindingKeywordSegment", // [1835]
    "WITHIN", // [1836]
    "WRITETEXT", // [1837]
    "X509", // [1838]
    "XML", // [1839]
    "XOR", // [1840]
    "YAML", // [1841]
    "YEAR_MONTH", // [1842]
    "ZEROFILL", // [1843]
];

pub static AUX_DATA: &[u32] = &[
    0, 1, 2, 1, 1, 4, 3, 1, 4, 5, 6, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 7, 1, 2, 1, 1, 4, 3, 1, 4, 5, 8, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 0, 0, 1, 0, 0, 0,
    0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 1, 0, 1, 0,
    1, 0, 1, 0, 1, 0, 0, 0, 4294967295, 4294967295, 0, 10, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 210, 1, 2, 1, 1, 4, 3, 1, 4, 5, 211,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 212, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 213, 1, 2, 1, 1, 4, 3, 1, 4, 5, 214, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 215, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 216, 1, 2, 1, 1, 4, 3, 1, 4, 5, 217, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 82, 230, 231, 1, 230, 3, 3, 4, 232, 233, 234,
    231, 1, 234, 3, 3, 4, 232, 1, 2, 1, 0, 239, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 11, 1, 2, 1, 1, 4, 3, 1, 4, 5, 244,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 245, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 246, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 0,
    0, 4294967295, 4294967295, 0, 0, 0, 0, 0, 0, 0, 1, 0, 296, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 297, 298, 231, 1, 298, 3, 3, 4, 232, 299,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 300, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 301, 1, 2, 1, 1, 4, 3, 1, 4, 5, 301, 302, 303,
    1, 302, 3, 3, 302, 4, 304, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    98, 1, 2, 1, 1, 4, 3, 1, 4, 5, 305, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 0, 0, 1, 0, 207, 329, 231, 1, 329, 3, 3, 4,
    232, 330, 331, 231, 1, 331, 3, 3, 4, 232, 1, 2, 0, 332, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1, 0, 1, 2, 0, 225, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 334, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    335, 1, 2, 1, 1, 4, 3, 1, 4, 5, 336, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 337, 1, 2, 1, 1, 4, 3, 1, 4, 5, 338, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 339, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 340, 1, 2, 1, 1, 4, 3, 1, 4, 5, 341, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 342, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    343, 1, 2, 1, 1, 4, 3, 1, 4, 5, 344, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 345, 1, 2, 1, 1, 4, 3, 1, 4, 5, 347, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 348, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 349, 1, 2, 1, 1, 4, 3, 1, 4, 5, 350, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 351, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    352, 353, 354, 355, 560, 4, 356, 357, 1, 356, 2, 3, 4, 0, 364, 365,
    0, 2, 3, 364, 3, 366, 0, 1, 3, 367, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 368, 1, 2, 1, 1, 4, 3, 1, 4, 5, 369, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 370, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 371, 1, 2, 1, 1, 4, 3, 1, 4, 5, 372, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 302, 303, 9, 3, 3, 302, 4, 389, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 390, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 391, 1, 2, 1, 1, 4, 3, 1, 4, 5, 392, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 313, 302, 231, 1, 302, 3, 3, 4, 232, 397, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 398, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 0, 401, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 406, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 1, 2, 1, 1, 0, 1,
    2, 1, 407, 408, 0, 2, 3, 407, 1, 2, 1, 414, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 415, 1, 2, 1, 1, 4, 3, 1, 4, 5, 416,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 4294967295, 4294967295, 0, 0, 0, 0,
    4294967295, 4294967295, 0, 447, 1, 2, 1, 1, 4, 3, 1, 4, 5, 258, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1, 2, 1, 1, 2, 1, 247, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 454, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 455, 1, 2, 1, 1, 4, 3, 1, 4, 5, 456, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 457, 1, 2, 1, 1, 4, 3, 1, 4, 5, 458,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 4294967295, 4294967295, 0, 0, 4294967295, 4294967295,
    0, 0, 459, 1, 2, 1, 1, 4, 3, 1, 4, 5, 473, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 474, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    475, 476, 231, 1, 476, 3, 3, 4, 232, 12, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 477, 1, 2, 1, 1, 4, 3, 1, 4, 5, 478, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 479, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 480, 1, 2, 1, 1, 4, 3, 1, 4, 5, 481, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 0, 0, 494, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 495, 1, 2, 1, 1, 4, 3, 1, 4, 5, 482, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 496, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    497, 1, 2, 1, 1, 4, 3, 1, 4, 5, 498, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 499, 1, 2, 1, 1, 4, 3, 1, 4, 5, 500, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 501, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 502, 1, 2, 1, 1, 4, 3, 1, 4, 5, 503, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 504, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    505, 1, 2, 1, 1, 4, 3, 1, 4, 5, 506, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 507, 1, 2, 1, 1, 4, 3, 1, 4, 5, 508, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 509, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 510, 1, 2, 1, 1, 4, 3, 1, 4, 5, 511, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 512, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    513, 1, 2, 1, 1, 4, 3, 1, 4, 5, 4, 357, 9, 2, 3, 4,
    1, 2, 1, 515, 1, 2, 1, 1, 4, 3, 1, 4, 5, 516, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1, 0, 0, 519, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 520, 1, 2, 1, 1, 4, 3, 1, 4, 5, 521, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 522, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 328, 523, 231, 1, 523, 3, 3, 4, 232, 328, 524, 231, 1, 524,
    3, 3, 4, 232, 328, 525, 231, 1, 525, 3, 3, 4, 232, 0, 1, 2,
    1, 0, 1, 2, 1, 0, 4294967295, 4294967295, 0, 274, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1, 0, 555, 1, 2, 1, 1, 4, 3, 1, 4, 5, 557,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 87, 558, 231, 1, 558, 3, 3,
    4, 232, 559, 1, 2, 1, 1, 4, 3, 1, 4, 5, 560, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 544, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    562, 563, 10, 3, 3, 562, 4, 564, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 565, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 280, 584, 9, 3,
    3, 280, 4, 585, 1, 2, 1, 1, 4, 3, 1, 4, 5, 302, 586, 0,
    2, 3, 302, 280, 587, 0, 2, 3, 280, 553, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 588, 1, 2, 1, 1, 4, 3, 1, 4, 5, 589, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 590, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 99, 1, 2, 1, 1, 4, 3, 1, 4, 5, 600, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 601, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    528, 1, 2, 1, 1, 4, 3, 1, 4, 5, 602, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 603, 1, 2, 1, 1, 4, 3, 1, 4, 5, 604, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 605, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 606, 1, 2, 1, 1, 4, 3, 1, 4, 5, 607, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 608, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    609, 1, 2, 1, 1, 4, 3, 1, 4, 5, 610, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 611, 1, 2, 1, 1, 4, 3, 1, 4, 5, 612, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 613, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 614, 1, 2, 1, 1, 4, 3, 1, 4, 5, 615, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 616, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    617, 1, 2, 1, 1, 4, 3, 1, 4, 5, 3, 4, 1, 0, 0, 1,
    0, 1, 2, 1, 100, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0,
    1, 2, 0, 0, 1, 0, 1, 2, 1, 0, 0, 0, 0, 4294967295, 4294967295, 0,
    0, 0, 0, 4294967295, 1, 0, 0, 1, 0, 1, 2, 1, 0, 0, 0, 1,
    0, 0, 1, 0, 0, 0, 0, 1, 2, 1, 0, 4294967295, 4294967295, 0, 0, 1,
    2, 1, 0, 709, 1, 2, 1, 1, 4, 3, 1, 4, 5, 710, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 711, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 592, 1, 2, 1, 1, 4, 3, 1, 4, 5, 483, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 712, 1, 2, 1, 1, 4, 3, 1, 4, 5, 713,
    714, 357, 1, 714, 2, 3, 4, 713, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 1, 2, 1, 720, 1, 2, 1, 1, 4, 3, 1, 4, 5, 721,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 352, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 722, 1, 2, 1, 1, 4, 3, 1, 4, 5, 723, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 724, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 353, 1, 2, 1, 1, 4, 3, 1, 4, 5, 354, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 725, 1, 2, 1, 1, 4, 3, 1, 4, 5, 726,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 727, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 728, 1, 2, 1, 1, 4, 3, 1, 4, 5, 248, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 52, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 13, 1, 2, 1, 1, 4, 3, 1, 4, 5, 729, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 1, 730, 357, 1, 730, 2, 3, 4, 0, 1,
    0, 0, 1, 2, 1, 0, 0, 0, 4294967295, 1, 0, 355, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 734, 2037, 1, 714, 357, 1, 714, 2, 3, 4, 0,
    1, 0, 1, 2, 1, 0, 236, 755, 757, 3, 755, 756, 236, 3, 3, 756,
    4, 758, 1, 2, 1, 1, 4, 3, 1, 4, 5, 737, 738, 739, 740, 741,
    742, 743, 744, 745, 746, 747, 2075, 11, 759, 357, 1, 759, 2, 3, 4, 760,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 761, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 737, 1, 2, 1, 1, 4, 3, 1, 4, 5, 762, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 763, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 764, 1, 2, 1, 1, 4, 3, 1, 4, 5, 765, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 766, 1, 2, 1, 1, 4, 3, 1, 4, 5, 767,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 738, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 768, 1, 2, 1, 1, 4, 3, 1, 4, 5, 769, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 770, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 771, 1, 2, 1, 1, 4, 3, 1, 4, 5, 772, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 773, 1, 2, 1, 1, 4, 3, 1, 4, 5, 533,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 775, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 681, 1, 2, 1, 1, 4, 3, 1, 4, 5, 696, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 776, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 777, 1, 2, 1, 1, 4, 3, 1, 4, 5, 778, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 779, 1, 2, 1, 1, 4, 3, 1, 4, 5, 780,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 781, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 101, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 790, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 791, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 792, 1, 2, 1, 1, 4, 3, 1, 4, 5, 793, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 794, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    795, 1, 2, 1, 1, 4, 3, 1, 4, 5, 796, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 797, 1, 2, 1, 1, 4, 3, 1, 4, 5, 798, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 800, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 801, 1, 2, 1, 1, 4, 3, 1, 4, 5, 802, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 803, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    804, 1, 2, 1, 1, 4, 3, 1, 4, 5, 805, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 806, 1, 2, 1, 1, 4, 3, 1, 4, 5, 807, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 808, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 809, 1, 2, 1, 1, 4, 3, 1, 4, 5, 583, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 810, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    811, 1, 2, 1, 1, 4, 3, 1, 4, 5, 812, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 310, 302, 231, 1, 302, 3, 3, 4, 232, 813, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 14, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 418, 814, 231, 1, 814, 3, 3, 4, 232, 815, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 0, 3, 4, 1, 271, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1, 0, 820, 1, 2, 1, 1, 4, 3, 1, 4, 5, 821, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 822, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 823, 1, 2, 1, 1, 4, 3, 1, 4, 5, 824, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 825, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    698, 1, 2, 1, 1, 4, 3, 1, 4, 5, 826, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 467, 1, 2, 1, 1, 4, 3, 1, 4, 5, 827, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 1, 832, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 833, 1, 2, 1, 1, 4, 3, 1, 4, 5, 834,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 835, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 836, 1, 2, 1, 1, 4, 3, 1, 4, 5, 837, 838, 231,
    1, 838, 3, 3, 4, 232, 469, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    839, 1, 2, 1, 1, 4, 3, 1, 4, 5, 840, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 374, 280, 584, 1, 280, 3, 3, 280, 4, 842, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 420, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 843, 1, 2, 1, 1, 4, 3, 1, 4, 5, 844, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 845, 1, 2, 1, 1, 4, 3, 1, 4, 5, 846,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 847, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 848, 1, 2, 1, 1, 4, 3, 1, 4, 5, 849, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 850, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 851, 1, 2, 1, 1, 4, 3, 1, 4, 5, 852, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 102, 1, 2, 1, 1, 4, 3, 1, 4, 5, 134,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 853, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 854, 1, 2, 1, 1, 4, 3, 1, 4, 5, 855, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 856, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 857, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 421, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 0, 4294967295, 4294967295, 0, 0, 0, 0, 0, 4294967295,
    4294967295, 0, 0, 1, 2, 1, 0, 0, 4294967295, 4294967295, 0, 0, 0, 1, 0, 0,
    1, 2, 1, 0, 1, 0, 1, 2, 1, 0, 0, 0, 915, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 15, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    916, 1, 2, 1, 1, 4, 3, 1, 4, 5, 917, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 402, 1, 2, 1, 1, 4, 3, 1, 4, 5, 402, 918,
    919, 1, 918, 5, 3, 1, 756, 4, 5, 0, 0, 0, 0, 920, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 930, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 16, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 4294967295, 4294967295, 0, 1,
    4294967295, 4294967295, 0, 1, 0, 0, 4294967295, 4294967295, 0, 938, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1, 2, 1, 939, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    942, 1, 2, 1, 1, 4, 3, 1, 4, 5, 286, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 943, 1, 2, 1, 1, 4, 3, 1, 4, 5, 944, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 945, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 946, 1, 2, 1, 1, 4, 3, 1, 4, 5, 947, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 948, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    629, 1, 2, 1, 1, 4, 3, 1, 4, 5, 949, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 17, 1, 2, 1, 1, 4, 3, 1, 4, 5, 177, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 952, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 953, 1, 2, 1, 1, 4, 3, 1, 4, 5, 954, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 0, 0, 0, 0, 964, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 965, 1, 2, 1, 1, 4, 3, 1, 4, 5, 966,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 967, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1, 0, 0, 1, 2, 1, 0, 0, 0, 4294967295, 4294967295, 0, 1,
    2, 1, 0, 1, 2, 1, 0, 221, 1006, 1, 0, 0, 4294967295, 4294967295, 0, 237,
    1006, 1, 1, 2, 1, 0, 91, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1, 0, 594, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1007, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 1, 0, 0, 0, 0, 0, 4294967295, 4294967295, 0,
    1, 2, 1, 18, 1, 2, 1, 1, 4, 3, 1, 4, 5, 5, 714, 1021,
    2, 714, 5, 3, 3, 4, 5, 0, 4294967295, 4294967295, 0, 0, 0, 0, 0, 1,
    0, 1, 2, 1, 0, 0, 67, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1033, 1, 2, 1, 1, 4, 3, 1, 4, 5, 19, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1034, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1035, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1036, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1037, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1038, 1038, 584, 3,
    1038, 280, 1038, 3, 3, 280, 4, 658, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1039, 1, 2, 1, 1, 4, 3, 1, 4, 5, 422, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 189, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0,
    0, 0, 1051, 1, 2, 1, 1, 4, 3, 1, 4, 5, 133, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 375, 280, 584, 1, 280, 3, 3, 280, 4, 376,
    280, 584, 1, 280, 3, 3, 280, 4, 1056, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 0, 1, 0, 0, 0, 195, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 1, 1, 0, 1, 0, 1058, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 1, 0, 1, 2, 1, 1066, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1067, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 1, 0, 0,
    970, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1068, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1069, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1070, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1071, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1072, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1073, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1074, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1075, 1, 2, 1, 1, 4, 3, 1, 4, 5, 739, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1076, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1077, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1078, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1079, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1080, 1081, 9, 3,
    3, 1080, 4, 1082, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1083, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1084, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 276, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1018, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 869, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    694, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1089, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1090, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1091, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 423, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 103, 1, 2, 1, 1, 4, 3, 1, 4, 5, 62, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1, 0, 0, 1, 2, 1, 0, 1092, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1093, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 249, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 643, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1096, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1097, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1098, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1099, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1100, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1101, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1102, 1, 2, 1, 1, 4, 3, 1, 4, 5, 682, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 595, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1103, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1104, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1105, 1, 2, 1, 1, 4, 3, 1, 4, 5, 104, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 1111, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 622, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1112, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 675, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1113, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1114, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1115, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1116,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1117, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1118, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1119, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 20, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1120, 1, 2, 1, 1, 4, 3, 1, 4, 5, 971, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1121, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0,
    0, 0, 360, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1109, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1122, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 377, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1127, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1128, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1129,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1130, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 4294967295, 4294967295, 0, 221, 1135, 1, 0, 237, 1135, 1, 0, 1133,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 461, 1145, 1, 1, 2, 1, 0,
    237, 1145, 1, 0, 0, 1, 0, 1, 2, 1, 1153, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1154, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1155, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 951, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1156, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1157, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1158, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1, 2, 9, 4, 3, 1, 4, 5, 1159, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 424, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1160, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 21, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    92, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1161, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1162, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1163, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1008, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1164, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1165, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 596, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1166, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1167, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 378, 280, 584, 1, 280, 3, 3, 280, 4, 379, 280, 584,
    1, 280, 3, 3, 280, 4, 1168, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 666, 1, 2, 1, 1, 4, 3, 1, 4, 5, 384, 384, 584, 3, 384,
    280, 384, 3, 3, 280, 4, 0, 1, 2, 1, 0, 0, 0, 972, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1174, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1175, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1176, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 756, 919, 9, 5, 3, 1, 756, 4, 5, 756,
    757, 9, 3, 3, 756, 4, 1184, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1185, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1186, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1187, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1188, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1189, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1190, 1, 2, 1, 1, 4, 3, 1, 4, 5, 451, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 0, 0, 0, 452, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1205, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1206, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 425, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 426, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1207, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1208, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1209, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1210, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1211, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1212, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1213, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1214, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 1, 2,
    3, 1, 1216, 1, 2, 1, 1, 4, 3, 1, 4, 5, 105, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1217, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    22, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1218, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1219, 1, 2, 1, 1, 4, 3, 1, 4, 5, 23, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1220, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1221, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1222, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1223, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    250, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1224, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1225, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1226, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1227, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1228, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1229, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 931, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1, 4294967295, 4294967295, 0, 0, 0, 1, 2, 1, 0, 221, 1135, 1, 237, 1135, 1,
    1248, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1249, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1250, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1251, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 623, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1252, 1, 2, 1, 1, 4, 3, 1, 4, 5, 740, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1253, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1254, 1, 2, 1, 1, 4, 3, 1, 4, 5, 973, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 309, 302, 231, 1, 302, 3, 3, 4, 232, 741, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1255, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1256, 1, 2, 1, 1, 4, 3, 1, 4, 5, 251, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1002, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1257,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1258, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1201, 1, 2, 1, 1, 4, 3, 1, 4, 5, 649, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1259, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 106, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1260, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 306, 302, 231, 1, 302, 3, 3, 4, 232, 40, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 742, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1261, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1262, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1263, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    307, 302, 231, 1, 302, 3, 3, 4, 232, 659, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1264, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1265, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2, 3, 1266, 1081, 1, 1266, 3, 3, 1080,
    4, 1267, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 1, 0, 1,
    0, 1272, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1124, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1124, 1273, 919, 1, 1273, 5, 3, 1, 756, 4, 5,
    1274, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1275, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1277, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1278, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 309, 1279, 231, 1, 1279, 3, 3, 4,
    232, 1280, 1, 2, 1, 1, 4, 3, 1, 4, 5, 690, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1281, 1282, 12, 3, 3, 1281, 4, 922, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 252, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1283, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1284, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 253, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1285, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1286, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1287, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1288, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1289, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    254, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1290, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1291, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2,
    1, 1, 2, 1, 1, 2, 1, 0, 0, 1, 2, 1, 0, 0, 1300, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1301, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 255, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1302, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1303, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1304, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 380, 280, 584, 1, 280,
    3, 3, 280, 4, 381, 1, 2, 1, 1, 4, 3, 1, 4, 5, 381, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1306, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1307, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1308, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1203, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    530, 1, 2, 1, 1, 4, 3, 1, 4, 5, 530, 1273, 919, 1, 1273, 5,
    3, 1, 756, 4, 5, 1309, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1310,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1088, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1311, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1312, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 97, 97, 757, 3, 97, 756, 97, 3, 3,
    756, 4, 95, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 1182, 1316,
    231, 1, 1316, 3, 3, 4, 232, 1, 2, 0, 0, 0, 1320, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1321, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1322, 1, 2, 1, 1, 4, 3, 1, 4, 5, 679, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1323, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1324, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 974, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1325, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1326, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 686, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 164, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1327, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 926, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1328, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1329, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1330, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1331, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1332, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 107, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1333, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1334, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1335, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1048, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1336, 1, 2, 1, 1, 4, 3, 1, 4, 5, 650, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 382, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 382, 302, 303, 1, 302, 3, 3, 302, 4, 0, 0, 0, 1, 0,
    0, 219, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1341, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1342, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1343, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1344, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1151, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1345, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1346, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1, 2, 1, 0, 1348, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    3, 4, 1, 0, 383, 1, 2, 1, 1, 4, 3, 1, 4, 5, 383, 280,
    584, 1, 280, 3, 3, 280, 4, 1350, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1196, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1351, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1107, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1352,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 108, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1353, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1354, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1355, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 4, 4294967295, 1356, 357, 1, 1356, 2, 3, 4, 1357, 1356, 231, 1, 1356, 3,
    3, 4, 232, 1358, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1359, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1360, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1361, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1362, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1363, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1364,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1365, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1, 0, 1, 2, 1, 0, 670, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1367, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1368, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1369, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 5, 1370, 1021, 2, 1370, 5, 3, 3, 4, 5, 1, 0, 0, 1371, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1372, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1373, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1374, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1375, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    137, 1, 2, 1, 1, 4, 3, 1, 4, 5, 314, 1376, 231, 1, 1376, 3,
    3, 4, 232, 1377, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1378, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1379, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 308, 302, 231, 1, 302, 3, 3, 4, 232, 128, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1380, 1, 2, 1, 1, 4, 3, 1, 4, 5, 308, 1279,
    231, 1, 1279, 3, 3, 4, 232, 0, 1384, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1385, 1, 2, 1, 1, 4, 3, 1, 4, 5, 960, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1386, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1387, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1388, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1389, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1390, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1391, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 536, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1393, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 881, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    142, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1394, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1395, 1, 2, 1, 1, 4, 3, 1, 4, 5, 24, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 68, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1396, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1397, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1398, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1399, 1, 2, 1, 1, 4, 3, 1, 4, 5, 206, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1400, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2,
    1, 0, 932, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 2, 1,
    743, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1403, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 427, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1404, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1022, 1405, 1081, 3, 1405, 1080, 1022, 3,
    3, 1080, 4, 236, 1406, 757, 3, 1406, 756, 236, 3, 3, 756, 4, 1407, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1408, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 955, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1409, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 279, 1410, 231, 1, 1410, 3, 3, 4, 232, 312,
    1410, 231, 1, 1410, 3, 3, 4, 232, 1411, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 311, 1410, 231, 1, 1410, 3, 3, 4, 232, 373, 1410, 231, 1, 1410,
    3, 3, 4, 232, 109, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1412, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1413, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1414, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1415, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1416, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1417, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1418, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 0, 4294967295, 1, 0, 0, 111, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 110, 1, 2, 1, 1, 4, 3, 1, 4, 5, 683, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 0, 1427, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1428, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1429, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1430, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1431, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1432, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1433, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1434, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1435, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1436, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1437, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1438, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1439, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1440, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 292, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1441, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1442, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 705, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1443, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1444, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1445, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1446, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 25, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1019, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1447, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1448, 1, 2, 1, 1, 4, 3, 1, 4, 5, 816, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1449, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1450, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1451, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1452, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1453, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1454, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 637, 1, 2, 1, 1, 4, 3, 1, 4, 5, 190, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 597, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 870, 1, 2, 1, 1, 4, 3, 1, 4, 5, 26, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1456, 1, 2, 1, 1, 4, 3, 1, 4, 5, 933,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 715, 714, 357, 1, 714, 2, 3,
    4, 715, 1, 2, 1, 1, 4, 3, 1, 4, 5, 27, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1457, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1458,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1459, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 69, 1, 2, 1, 1, 4, 3, 1, 4, 5, 688, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1460, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1461, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1462, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1463, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1464,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1465, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 924, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1466, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 428, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1467, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 2, 1, 0,
    1, 2, 1, 1473, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1474, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1475, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 28, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1476, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 60, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1477,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1478, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1479, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1480, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1481, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1482, 1, 2, 1, 1, 4, 3, 1, 4, 5, 744, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1483, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1484,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1485, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 410, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 0,
    0, 1, 0, 2, 8168, 2, 8170, 3, 377, 583, 377, 381, 583, 0, 112, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 1, 0, 1, 2, 1, 0,
    1, 2, 1, 0, 1516, 1, 2, 1, 1, 4, 3, 1, 4, 5, 789, 1517,
    231, 1, 1517, 3, 3, 4, 232, 1518, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1015, 1, 2, 1, 1, 4, 3, 1, 4, 5, 29, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 0, 70, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1521, 1, 2, 1, 1, 4, 3, 1, 4, 5, 53, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1522, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1523,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1055, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1, 0, 0, 1, 4294967295, 4294967295, 0, 488, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 0, 0, 0, 0, 0, 1538, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1065, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1539, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 201, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1540, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 4294967295, 4294967295, 0,
    1541, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1542, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1543, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1546,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1422, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 1, 0, 236, 1405, 1081, 3, 1405, 1080, 236, 3, 3, 1080,
    4, 1547, 1, 2, 1, 1, 4, 3, 1, 4, 5, 310, 1550, 231, 1, 1550,
    3, 3, 4, 232, 328, 1551, 231, 1, 1551, 3, 3, 4, 232, 1552, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1401, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1553, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1554, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1555, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1556,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 621, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1557, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1558, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1559, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1560, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1561, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1562, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1563,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1564, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1565, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1566, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1567, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1568, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1569, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1570, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1571,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1572, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1573, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1574, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1575, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1576, 1, 2, 1, 1, 4, 3, 1, 4, 5, 934, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1577, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1578,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 30, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 71, 1, 2, 1, 1, 4, 3, 1, 4, 5, 307, 1579, 231,
    1, 1579, 3, 3, 4, 232, 654, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1580, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1581, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1582, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1583, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 699, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 0, 624, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1663, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1664, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1665, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1666, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1667, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1668,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1669, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1670, 1, 2, 1, 1, 4, 3, 1, 4, 5, 31, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 72, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1671, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1673, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1, 0, 1, 2, 1, 1674, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1675, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1676,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1677, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1678, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1679, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1680, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1681, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1682, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1683, 1, 2, 1, 1, 4, 3, 1, 4, 5, 232,
    231, 9, 3, 3, 4, 232, 1684, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1685, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1686, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1687, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1469, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1688, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 0, 1, 2, 1, 1, 2, 1, 0, 32, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1690, 1, 2, 1, 1, 4, 3, 1, 4, 5, 73, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 995, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 33, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 4294967295, 4294967295, 0,
    0, 4294967295, 4294967295, 0, 34, 1, 2, 1, 1, 4, 3, 1, 4, 5, 74, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 0, 0, 0, 1, 0, 1,
    0, 1, 2, 1, 0, 1, 0, 1, 2, 1, 0, 0, 1, 0, 0, 1,
    0, 1, 2, 1, 0, 1, 0, 1, 2, 1, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 1, 0, 0, 1, 2, 1, 0, 1, 0, 1, 2, 1, 0,
    0, 1, 0, 1, 2, 1, 0, 0, 0, 4294967295, 4294967295, 0, 1, 2, 1, 1,
    2, 1, 1, 0, 1, 2, 1, 0, 113, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1725, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 114, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 0, 2, 3, 1, 1728, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1729, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1730,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1731, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1732, 1, 2, 1, 1, 4, 3, 1, 4, 5, 429, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1694, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 929, 1, 2, 1, 1, 4, 3, 1, 4, 5, 882, 1733, 231, 1, 1733,
    3, 3, 4, 232, 749, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0,
    0, 4294967295, 4294967295, 0, 750, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1739, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1740, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1741, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1742, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1743, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    89, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1744, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1488, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1745, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1009, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1746, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1532, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 0, 1753, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1754, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1755, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1756, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1757, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1758, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1726, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1759, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1760, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1761, 1, 2, 1, 1, 4, 3, 1, 4, 5, 115, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1762, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1763, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1764, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1765, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 403,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 403, 918, 919, 1, 918, 5, 3,
    1, 756, 4, 5, 116, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1769, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1770, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1, 0, 1, 2, 1, 35, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 484, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1772, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1773, 1, 2, 1, 1, 4, 3, 1, 4, 5, 959,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1774, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 1778, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1779, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1780, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 0, 975, 1, 2, 1, 1, 4, 3, 1, 4, 5, 540, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1782, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1783, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1784, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1785, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1786,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 4294967295, 4294967295, 0, 1787, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1788, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 117, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1791, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1792, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    485, 1, 2, 1, 1, 4, 3, 1, 4, 5, 118, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 119, 1, 2, 1, 1, 4, 3, 1, 4, 5, 120, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 123, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1793, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1794, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1795, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1796, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1797, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1138, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1798, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1799, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1800, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1801, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1802, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1803, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1804, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 411, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0,
    1, 0, 1, 2, 1, 1, 0, 412, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1806, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1807, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1808, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1809,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1810, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1811, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1812, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1813, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1814, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1815, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1816, 1, 2, 1, 1, 4, 3, 1, 4, 5, 36,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 75, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 660, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1817, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 37, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1818, 1, 2, 1, 1, 4, 3, 1, 4, 5, 745, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 746, 1, 2, 1, 1, 4, 3, 1, 4, 5, 221,
    1819, 1, 461, 1820, 1, 237, 1820, 1, 237, 1819, 1, 463, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1821, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1,
    2, 1, 0, 0, 786, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1822, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1823, 1824, 12, 3, 3, 4, 1823, 1825,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 4294967295, 4294967295, 0, 976, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 221, 1831, 1, 1, 0, 237, 1831,
    1, 221, 1831, 1, 1, 0, 237, 1831, 1, 453, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1836, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1734, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 5, 1021, 9, 3, 3, 4, 5, 1749, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 57, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 121, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1837, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1838, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1839, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1840, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1841, 1, 2, 1, 1, 4, 3, 1, 4, 5, 747, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1842, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1843, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1737, 1, 2, 1,
    1, 4, 3, 1, 4, 5,
];

pub static AUX_DATA_OFFSETS: &[u32] = &[
    0, 10, 20, 30, 46, 9, 0, 42, 0, 38, 39, 41, 42, 43, 44, 45,
    46, 47, 48, 49, 50, 0, 51, 40, 54, 0, 55, 56, 0, 58, 59, 61,
    42, 0, 63, 59, 61, 42, 64, 0, 58, 66, 61, 46, 0, 63, 66, 61,
    41, 42, 46, 44, 43, 77, 0, 79, 81, 84, 86, 88, 90, 0, 93, 94,
    96, 52, 0, 122, 47, 48, 124, 41, 42, 44, 0, 125, 126, 127, 0, 129,
    130, 0, 131, 132, 0, 39, 48, 135, 136, 0, 51, 0, 122, 49, 46, 0,
    126, 127, 138, 64, 0, 139, 50, 46, 47, 0, 140, 141, 125, 143, 122, 144,
    129, 145, 146, 39, 147, 148, 149, 150, 151, 152, 153, 154, 155, 156, 157, 136,
    158, 159, 0, 58, 141, 160, 53, 161, 163, 55, 165, 166, 167, 168, 169, 170,
    171, 172, 130, 46, 0, 126, 127, 0, 173, 166, 0, 174, 165, 0, 175, 176,
    56, 178, 179, 180, 181, 182, 183, 184, 185, 186, 59, 0, 126, 187, 0, 173,
    179, 0, 174, 178, 0, 175, 188, 57, 191, 193, 70, 0, 196, 58, 77, 0,
    124, 60, 197, 0, 48, 62, 199, 0, 202, 64, 77, 66, 199, 68, 203, 205,
    71, 208, 75, 85, 95, 105, 115, 125, 135, 145, 155, 218, 0, 221, 224, 186,
    0, 227, 183, 228, 165, 174, 235, 237, 187, 0, 240, 242, 197, 207, 217, 227,
    239, 0, 256, 257, 96, 259, 260, 237, 0, 261, 96, 262, 238, 263, 264, 265,
    0, 266, 171, 267, 240, 269, 0, 272, 273, 275, 227, 249, 0, 277, 278, 244,
    281, 282, 0, 245, 283, 146, 273, 284, 247, 0, 246, 287, 288, 289, 160, 291,
    0, 293, 248, 294, 88, 240, 0, 266, 165, 240, 250, 295, 252, 262, 271, 281,
    291, 301, 310, 320, 330, 340, 315, 316, 317, 318, 319, 320, 321, 322, 323, 324,
    362, 342, 341, 96, 325, 327, 344, 353, 0, 365, 377, 375, 333, 0, 0, 294,
    380, 390, 400, 410, 420, 430, 440, 450, 460, 470, 480, 490, 0, 346, 500, 510,
    520, 530, 540, 550, 564, 573, 281, 358, 359, 203, 289, 325, 0, 362, 281, 86,
    294, 574, 580, 585, 595, 605, 615, 625, 635, 645, 385, 386, 387, 388, 646, 653,
    663, 673, 683, 393, 0, 394, 394, 395, 0, 396, 396, 693, 702, 712, 722, 399,
    400, 723, 733, 404, 405, 734, 746, 744, 281, 751, 749, 289, 754, 760, 409, 763,
    773, 783, 793, 227, 417, 419, 0, 417, 797, 430, 431, 432, 433, 434, 42, 435,
    436, 437, 438, 439, 440, 441, 433, 281, 0, 417, 442, 799, 442, 0, 175, 278,
    798, 444, 445, 0, 388, 281, 803, 813, 823, 228, 0, 227, 448, 294, 826, 450,
    829, 839, 849, 859, 869, 879, 897, 0, 460, 461, 889, 464, 466, 468, 470, 468,
    468, 0, 460, 325, 893, 464, 466, 468, 470, 468, 468, 388, 86, 472, 898, 908,
    918, 928, 937, 947, 957, 967, 977, 987, 999, 0, 486, 487, 227, 0, 489, 997,
    490, 491, 998, 492, 493, 1000, 1010, 1020, 1030, 1040, 1050, 1060, 1070, 1080, 1090, 1100,
    1110, 1120, 1130, 1140, 1150, 1160, 1170, 1180, 1190, 1200, 1210, 1216, 514, 1219, 1229, 1241,
    444, 1239, 227, 517, 518, 1242, 1252, 1262, 1272, 1282, 1291, 1300, 1309, 526, 281, 203,
    358, 0, 0, 529, 77, 1313, 0, 489, 531, 0, 532, 1310, 325, 0, 534, 535,
    0, 537, 538, 539, 541, 0, 542, 538, 543, 0, 546, 547, 549, 0, 289, 362,
    1314, 0, 1317, 550, 552, 0, 1321, 1331, 227, 517, 518, 1333, 0, 556, 0, 0,
    1343, 1353, 1362, 1372, 0, 561, 444, 1382, 1392, 1399, 1409, 1419, 278, 566, 567, 568,
    569, 570, 571, 572, 573, 574, 575, 576, 577, 578, 579, 580, 0, 581, 582, 90,
    0, 581, 489, 582, 90, 1420, 1427, 1437, 1443, 1449, 1459, 0, 395, 395, 1469, 1479,
    591, 1489, 593, 598, 599, 1490, 1500, 1510, 1520, 1530, 1540, 1550, 1560, 1570, 1580, 1590,
    1600, 1610, 1620, 1630, 1640, 1650, 1660, 1670, 1680, 0, 122, 618, 1690, 362, 294, 362,
    619, 620, 1694, 170, 168, 169, 0, 1693, 625, 626, 627, 628, 79, 81, 0, 630,
    77, 0, 294, 631, 0, 122, 42, 632, 633, 0, 122, 635, 636, 168, 632, 79,
    81, 0, 638, 362, 639, 0, 122, 635, 641, 642, 632, 644, 163, 240, 0, 1697,
    1695, 646, 1700, 0, 122, 635, 648, 632, 77, 0, 651, 1718, 1716, 0, 277, 278,
    1715, 281, 1712, 1710, 444, 294, 450, 0, 122, 48, 632, 199, 0, 122, 46, 632,
    652, 1723, 0, 256, 257, 96, 0, 655, 619, 96, 259, 260, 1721, 0, 261, 96,
    262, 1722, 263, 264, 265, 0, 122, 171, 267, 1724, 656, 0, 122, 635, 1730, 1728,
    487, 661, 1729, 0, 662, 154, 663, 165, 632, 240, 664, 1740, 0, 1737, 1735, 1734,
    284, 667, 543, 0, 294, 450, 0, 669, 240, 1741, 671, 0, 122, 155, 673, 1742,
    676, 288, 0, 677, 678, 1746, 1745, 145, 144, 0, 157, 678, 1743, 289, 680, 163,
    680, 163, 163, 240, 1754, 0, 684, 685, 687, 294, 277, 689, 687, 294, 277, 0,
    90, 240, 1749, 0, 489, 691, 0, 691, 1748, 0, 692, 693, 0, 692, 695, 0,
    630, 697, 1750, 687, 700, 0, 701, 1751, 325, 0, 129, 169, 702, 703, 0, 122,
    124, 199, 0, 1758, 122, 706, 166, 632, 240, 160, 294, 1762, 1759, 450, 707, 1763,
    1773, 1783, 1793, 1803, 1813, 1823, 1831, 0, 1841, 716, 717, 1842, 718, 1845, 1855, 1865,
    1875, 1885, 1895, 1905, 1915, 1925, 1935, 1945, 1955, 1965, 1975, 1985, 0, 42, 633, 1995,
    2014, 2005, 227, 282, 0, 731, 732, 2018, 2017, 2015, 325, 0, 176, 444, 2023, 0,
    486, 487, 227, 0, 489, 2021, 490, 491, 2022, 492, 493, 2027, 2038, 735, 0, 2050,
    2048, 2047, 736, 748, 0, 2053, 751, 752, 753, 754, 2054, 2065, 2086, 2095, 2105, 2115,
    2125, 2135, 2145, 2155, 2165, 2175, 2185, 2195, 2205, 2215, 2225, 2235, 2245, 2255, 0, 534,
    774, 2265, 2275, 2285, 2295, 2305, 2315, 2325, 2335, 2345, 2355, 0, 2365, 144, 782, 783,
    785, 788, 2366, 2376, 2386, 2396, 2406, 2416, 2426, 2436, 2446, 0, 799, 282, 77, 2456,
    2466, 2476, 2486, 2496, 2506, 2516, 2526, 2536, 2546, 2556, 2566, 2576, 2586, 2596, 2605, 2615,
    2625, 2634, 2644, 817, 818, 0, 272, 618, 2645, 362, 294, 362, 819, 0, 272, 42,
    275, 633, 819, 0, 272, 168, 275, 79, 0, 272, 642, 275, 644, 819, 2648, 0,
    272, 648, 275, 77, 0, 272, 48, 275, 227, 0, 272, 46, 275, 652, 819, 0,
    272, 171, 267, 0, 272, 636, 165, 275, 2658, 240, 819, 0, 272, 155, 275, 673,
    0, 272, 50, 275, 77, 819, 0, 272, 124, 275, 199, 0, 272, 166, 275, 240,
    819, 2660, 2670, 2680, 2690, 2700, 2710, 2720, 2730, 0, 466, 325, 2740, 2750, 2760, 0,
    828, 830, 2763, 2773, 2783, 2793, 2803, 2813, 2822, 2832, 2842, 841, 2852, 2861, 2871, 2881,
    2891, 2901, 2911, 2921, 2931, 2941, 2951, 2961, 2971, 2981, 2991, 3001, 3011, 3021, 3031, 3041,
    0, 858, 3051, 450, 859, 861, 863, 3052, 0, 865, 866, 0, 867, 3063, 3062, 868,
    0, 472, 867, 871, 872, 0, 581, 489, 873, 874, 875, 876, 0, 489, 877, 878,
    879, 867, 0, 880, 865, 3067, 883, 0, 884, 885, 886, 887, 0, 888, 3070, 3069,
    0, 3068, 385, 386, 388, 888, 3074, 883, 0, 884, 885, 3083, 0, 889, 3075, 450,
    0, 3078, 890, 891, 3079, 893, 526, 86, 0, 3098, 358, 203, 3088, 3087, 895, 325,
    450, 3085, 3084, 281, 289, 203, 896, 325, 898, 899, 0, 3091, 828, 900, 3094, 3092,
    325, 0, 362, 3097, 444, 96, 902, 903, 904, 896, 905, 86, 906, 3099, 907, 281,
    359, 909, 911, 289, 913, 0, 3100, 3110, 3120, 3130, 3140, 3150, 0, 921, 3161, 287,
    923, 3162, 96, 325, 687, 3163, 687, 925, 3164, 927, 0, 619, 928, 3165, 3175, 3185,
    0, 3195, 935, 3203, 936, 3199, 935, 3205, 935, 3209, 0, 940, 3219, 0, 941, 325,
    3222, 3232, 3242, 3252, 3262, 3272, 3282, 3292, 3302, 3312, 3322, 0, 51, 950, 3332, 3342,
    3352, 3362, 3372, 0, 956, 3385, 3384, 0, 957, 687, 0, 3382, 96, 359, 958, 289,
    3383, 961, 962, 0, 877, 879, 3386, 925, 963, 3387, 3397, 3407, 3417, 0, 90, 3427,
    968, 3429, 941, 977, 0, 196, 257, 0, 979, 257, 980, 981, 982, 983, 707, 985,
    921, 987, 3442, 0, 988, 3433, 3430, 989, 991, 242, 3434, 993, 994, 997, 998, 0,
    619, 987, 242, 994, 1000, 3439, 0, 3435, 998, 3461, 3458, 0, 3446, 1001, 1004, 3443,
    968, 0, 979, 257, 0, 196, 257, 3447, 3451, 0, 3450, 998, 997, 0, 979, 257,
    0, 196, 257, 3455, 3462, 0, 90, 3472, 1004, 3474, 3484, 325, 3500, 325, 0, 325,
    294, 362, 0, 1010, 325, 90, 90, 325, 0, 3494, 736, 325, 90, 325, 0, 582,
    3497, 1011, 3495, 1012, 1013, 0, 1016, 281, 0, 3498, 444, 227, 289, 61, 3499, 444,
    227, 289, 1017, 646, 1020, 0, 325, 294, 362, 0, 3504, 748, 0, 294, 444, 0,
    45, 282, 3507, 3517, 0, 3527, 0, 227, 417, 408, 3531, 702, 1023, 408, 3534, 0,
    277, 3532, 0, 1024, 50, 362, 3533, 0, 1024, 50, 362, 3537, 3535, 1025, 3541, 0,
    1026, 1027, 1028, 0, 0, 79, 3540, 1026, 1029, 1030, 703, 1032, 3542, 3552, 3562, 3572,
    3582, 3592, 3602, 3612, 3623, 3633, 3643, 3653, 0, 1040, 3663, 0, 1041, 163, 1043, 0,
    48, 199, 0, 148, 163, 124, 197, 77, 88, 1045, 3664, 0, 619, 1040, 1047, 0,
    619, 1049, 1047, 0, 1050, 957, 132, 0, 1052, 257, 3665, 1053, 1054, 197, 3666, 3676,
    0, 396, 841, 3686, 396, 3695, 3704, 0, 196, 257, 3717, 58, 1057, 1060, 0, 3715,
    3714, 289, 96, 325, 1062, 3718, 0, 979, 257, 977, 980, 981, 982, 921, 3719, 0,
    3733, 3732, 289, 96, 325, 3729, 1062, 3735, 0, 1063, 1064, 3748, 3746, 3745, 1060, 718,
    3751, 3761, 0, 980, 3774, 3771, 325, 3775, 0, 979, 257, 977, 981, 982, 921, 3776,
    3786, 3796, 3806, 3816, 3826, 3836, 0, 3846, 3856, 3866, 3876, 3886, 3896, 3906, 3916, 3923,
    3933, 3943, 0, 430, 889, 3953, 0, 430, 489, 889, 3963, 0, 3973, 1085, 1086, 1087,
    3974, 3984, 3994, 4004, 4014, 4024, 4034, 4044, 0, 489, 61, 4060, 4057, 4056, 4054, 865,
    450, 203, 4061, 4071, 4081, 0, 227, 4091, 1094, 1095, 4092, 4102, 4112, 4122, 4132, 4142,
    4152, 4162, 4172, 4182, 4192, 4202, 4212, 4222, 0, 145, 1106, 1108, 240, 4232, 450, 0,
    160, 450, 1110, 4233, 4243, 4253, 4263, 4273, 4283, 4293, 4303, 4313, 4323, 4333, 4343, 4353,
    4363, 4373, 0, 754, 4385, 0, 96, 4383, 444, 736, 444, 0, 444, 4384, 444, 736,
    4386, 4396, 4406, 4416, 903, 1123, 1125, 902, 1126, 0, 581, 489, 582, 90, 4417, 0,
    4427, 4437, 4447, 4457, 4478, 0, 1131, 1132, 1004, 4467, 1134, 0, 4471, 1136, 4474, 1139,
    1141, 4475, 0, 1142, 1132, 1004, 1136, 0, 1143, 1004, 4479, 0, 1144, 0, 0, 163,
    4489, 4495, 4492, 325, 4496, 4500, 1146, 0, 4499, 1147, 1148, 1149, 1150, 0, 1152, 4503,
    4501, 227, 4506, 4516, 4526, 4536, 4546, 4556, 4566, 4576, 4584, 4594, 4604, 4614, 4624, 4634,
    4644, 0, 4654, 4664, 4674, 4684, 4694, 4704, 4714, 0, 394, 841, 4724, 394, 4733, 4742,
    0, 0, 489, 1169, 865, 0, 1170, 867, 4752, 669, 1171, 1172, 4753, 4763, 0, 1173,
    4778, 4775, 4774, 96, 325, 58, 4780, 0, 987, 4779, 96, 325, 0, 86, 96, 4781,
    4791, 4801, 0, 4811, 4821, 444, 96, 902, 884, 903, 904, 1177, 1179, 1181, 4822, 4831,
    4838, 4848, 0, 4858, 4868, 4878, 4888, 4898, 4908, 0, 4918, 1191, 1192, 4919, 687, 0,
    165, 77, 0, 166, 77, 0, 42, 77, 4920, 630, 61, 4921, 1193, 159, 1194, 1195,
    0, 149, 0, 1197, 202, 1198, 0, 1199, 1200, 0, 1202, 0, 1204, 4922, 4932, 4942,
    4952, 4962, 4972, 4982, 4992, 5002, 5012, 5022, 5032, 5042, 0, 1215, 417, 227, 5055, 0,
    648, 77, 0, 86, 5052, 450, 5058, 5068, 5078, 0, 5088, 0, 5098, 5108, 5118, 5128,
    5138, 5148, 5158, 5168, 5178, 5188, 5198, 5208, 5218, 5228, 144, 0, 145, 160, 1030, 0,
    1230, 1108, 5238, 5248, 1231, 1233, 0, 701, 1235, 0, 879, 325, 431, 5252, 1236, 1238,
    0, 701, 489, 1235, 0, 879, 325, 431, 1240, 0, 1242, 5253, 240, 1243, 1152, 5257,
    240, 1243, 0, 5254, 450, 242, 0, 203, 242, 5258, 1139, 5261, 1244, 0, 157, 1246,
    5264, 5274, 5284, 5294, 5304, 5314, 5324, 5334, 5344, 5354, 5364, 5373, 5383, 5393, 5403, 5413,
    5423, 5433, 5443, 5453, 5463, 5473, 5483, 5493, 5502, 5512, 5522, 5532, 5542, 5552, 5561, 5571,
    5581, 5591, 5601, 0, 227, 294, 5614, 227, 5611, 1268, 0, 982, 5615, 1270, 5617, 5627,
    5637, 5648, 0, 1276, 593, 5658, 5668, 5678, 5688, 0, 5697, 5707, 5717, 5724, 5734, 5744,
    5754, 5764, 5774, 5784, 5794, 5804, 5814, 5824, 5834, 5844, 5863, 1030, 1292, 5854, 907, 5857,
    1293, 5860, 1295, 1296, 0, 5864, 861, 859, 863, 1297, 5869, 1299, 5868, 5865, 907, 1295,
    5870, 5880, 5890, 5900, 0, 5910, 5920, 0, 5930, 0, 1305, 841, 0, 394, 396, 5931,
    0, 886, 841, 0, 322, 841, 5940, 0, 5950, 5960, 5970, 5980, 5990, 6000, 6010, 6021,
    6031, 6041, 6051, 6061, 6071, 6082, 0, 444, 1313, 333, 6103, 6092, 1314, 6094, 6106, 417,
    0, 417, 417, 6107, 163, 294, 1152, 86, 1317, 1318, 1319, 472, 1313, 935, 997, 6108,
    6118, 6128, 6138, 6148, 6158, 0, 987, 6168, 96, 325, 687, 6169, 687, 925, 6170, 6180,
    6190, 6200, 6210, 6220, 6230, 6240, 6250, 6260, 6270, 6280, 6290, 6300, 6310, 6320, 6330, 6340,
    6350, 6360, 6370, 0, 680, 706, 0, 979, 257, 6382, 0, 6379, 289, 96, 325, 6380,
    1094, 1095, 0, 1087, 6381, 287, 1337, 1338, 977, 956, 6384, 977, 980, 981, 982, 956,
    1016, 921, 6385, 6395, 979, 1340, 6396, 6406, 6416, 6426, 6436, 6446, 6456, 0, 1017, 1347,
    6469, 227, 6466, 1268, 6470, 0, 1349, 6483, 0, 6480, 904, 86, 904, 289, 6484, 6494,
    6503, 6513, 6523, 6533, 6543, 6553, 6563, 6573, 6583, 6593, 6602, 6611, 6621, 6631, 6641, 6651,
    6661, 6671, 6681, 0, 1366, 257, 6696, 6693, 6691, 325, 6697, 6707, 6717, 6727, 6749, 0,
    419, 6747, 6737, 419, 444, 0, 6750, 6760, 6770, 6780, 6790, 6800, 6810, 6819, 6829, 6839,
    6849, 6858, 6868, 6878, 6887, 1381, 1383, 0, 6888, 6898, 0, 6908, 6918, 6928, 6938, 6948,
    6958, 6968, 0, 1392, 950, 6978, 6988, 6998, 7008, 7018, 7028, 7038, 7048, 7058, 7068, 7078,
    7088, 7098, 7108, 0, 883, 96, 0, 981, 7121, 7118, 325, 7122, 0, 388, 7132, 58,
    1024, 1402, 7133, 450, 7136, 7146, 7156, 7166, 7176, 7187, 7198, 7208, 7218, 7228, 7238, 7247,
    7256, 7266, 7275, 7284, 7294, 7304, 7314, 7324, 7334, 7344, 7354, 0, 151, 240, 160, 1419,
    7364, 0, 163, 144, 1420, 0, 163, 157, 1420, 0, 1421, 7368, 1147, 1423, 1424, 7369,
    7379, 7389, 7399, 817, 818, 0, 487, 531, 0, 1425, 1426, 0, 487, 534, 7400, 7410,
    7420, 7430, 7440, 7450, 7460, 7470, 7480, 7490, 7500, 7510, 7520, 7530, 7540, 7550, 7560, 7570,
    7580, 7590, 7600, 7610, 7620, 7630, 7640, 7650, 7660, 7670, 7680, 7690, 7700, 7710, 7720, 7730,
    7740, 0, 1455, 0, 1040, 1047, 630, 7750, 0, 1041, 163, 1043, 0, 48, 199, 0,
    148, 163, 124, 197, 197, 90, 1045, 819, 7751, 7761, 7771, 227, 7781, 7791, 7801, 7809,
    7819, 7829, 7839, 7849, 7859, 7869, 7879, 7889, 7899, 7909, 7919, 7929, 7939, 7949, 7959, 7969,
    0, 1468, 7979, 1470, 1471, 7980, 96, 0, 7983, 1472, 7984, 96, 7987, 7997, 8007, 8017,
    8027, 8037, 8047, 8057, 8067, 8077, 8087, 8097, 8107, 8117, 8127, 8137, 8147, 8157, 1486, 0,
    333, 242, 8159, 582, 58, 0, 1489, 325, 0, 1490, 0, 619, 928, 0, 1491, 8158,
    0, 163, 1492, 680, 1349, 0, 163, 1349, 0, 163, 1349, 680, 1492, 0, 8160, 152,
    1493, 1494, 8161, 1496, 90, 941, 0, 979, 257, 1173, 983, 8173, 90, 8163, 941, 0,
    979, 257, 977, 1349, 983, 921, 8174, 0, 1498, 1500, 783, 785, 1502, 1504, 913, 1506,
    1508, 218, 1510, 1512, 977, 1506, 983, 707, 985, 8195, 8187, 8184, 1293, 8191, 8188, 1514,
    1515, 8192, 450, 8196, 8206, 8215, 8225, 8235, 8245, 0, 1519, 96, 0, 1425, 1519, 8246,
    0, 1520, 96, 0, 1425, 1520, 8247, 8257, 8267, 8277, 8287, 8297, 0, 487, 8307, 1524,
    0, 289, 278, 8309, 281, 358, 203, 289, 325, 1030, 534, 0, 1295, 8310, 0, 983,
    1295, 218, 977, 1506, 8314, 8325, 1526, 0, 8324, 1527, 1528, 58, 1529, 0, 487, 1530,
    278, 8326, 444, 1531, 0, 157, 630, 8327, 0, 1533, 0, 663, 1534, 0, 487, 46,
    632, 652, 0, 8328, 0, 487, 1533, 1535, 1536, 8329, 8339, 8349, 8359, 8369, 0, 8379,
    890, 891, 8380, 0, 1317, 362, 893, 8384, 8394, 8404, 8414, 1544, 1545, 8415, 8425, 8435,
    282, 1023, 417, 8436, 227, 0, 227, 517, 1011, 8438, 8449, 0, 1548, 208, 8459, 8468,
    8477, 8487, 8497, 8507, 8517, 8527, 8537, 8547, 8557, 8567, 8577, 8587, 8597, 8607, 8617, 8627,
    8637, 8647, 8657, 8667, 8677, 8687, 8697, 8707, 8717, 8727, 8737, 8747, 8757, 8767, 8777, 8787,
    8797, 8806, 8816, 8826, 8836, 8846, 8856, 8866, 450, 1297, 859, 1584, 1586, 1588, 1590, 1592,
    1594, 1596, 1598, 1600, 1602, 1604, 1606, 1608, 1610, 1612, 1614, 1616, 1618, 1620, 1622, 863,
    861, 1624, 1626, 1628, 1630, 1632, 1634, 1636, 1638, 1640, 1642, 1644, 1646, 1648, 1650, 1652,
    1654, 1656, 1508, 1658, 1536, 1659, 1661, 935, 8867, 8877, 8887, 8897, 8907, 8917, 8927, 8937,
    8947, 8957, 8967, 8977, 8987, 1672, 8988, 9000, 8998, 0, 333, 242, 0, 9003, 9013, 9023,
    9033, 9043, 9053, 9063, 9073, 9083, 9093, 9103, 9110, 9120, 9130, 9140, 9150, 9160, 0, 0,
    529, 77, 9170, 0, 641, 160, 0, 537, 160, 0, 1689, 160, 542, 0, 9177, 1030,
    358, 203, 240, 9171, 450, 9174, 1297, 9178, 9188, 9198, 9208, 9218, 0, 9228, 1691, 472,
    1692, 0, 9232, 1693, 1692, 9236, 9246, 0, 9259, 0, 1695, 96, 1490, 0, 1471, 1696,
    9256, 1490, 1697, 0, 1471, 1695, 0, 1696, 96, 9257, 1490, 1697, 0, 1425, 1696, 9258,
    1490, 1697, 0, 1425, 1695, 0, 1698, 96, 0, 1471, 1698, 0, 1699, 96, 0, 1471,
    1699, 1695, 0, 630, 957, 0, 1700, 1701, 9260, 1702, 1703, 1704, 0, 1152, 9261, 1705,
    879, 9276, 9275, 0, 641, 642, 644, 58, 9265, 9263, 289, 1707, 0, 641, 642, 644,
    0, 273, 9274, 9271, 9269, 9268, 289, 1366, 0, 294, 77, 163, 154, 240, 0, 273,
    9284, 9281, 9279, 9278, 289, 1366, 0, 294, 77, 0, 0, 1709, 9290, 9287, 9285, 281,
    281, 531, 0, 561, 9291, 163, 9292, 0, 273, 289, 0, 168, 77, 0, 1710, 77,
    0, 1711, 77, 0, 628, 77, 0, 169, 77, 0, 1712, 77, 0, 48, 77, 0,
    155, 77, 0, 50, 77, 0, 166, 240, 0, 42, 633, 0, 175, 77, 0, 165,
    240, 0, 124, 77, 0, 9293, 294, 581, 444, 0, 86, 9299, 9298, 0, 1425, 1713,
    1714, 0, 9294, 1425, 1715, 1716, 0, 489, 1716, 9295, 676, 288, 1717, 0, 1198, 278,
    9296, 163, 1718, 534, 0, 9297, 534, 1425, 1719, 0, 1719, 278, 96, 1490, 0, 1711,
    278, 77, 0, 979, 257, 9301, 774, 1720, 9302, 289, 0, 293, 165, 240, 9305, 88,
    294, 240, 9320, 9312, 0, 641, 1392, 642, 77, 9311, 9308, 9306, 227, 227, 0, 1425,
    1392, 642, 0, 1366, 257, 1721, 0, 641, 642, 77, 58, 9315, 9313, 289, 0, 619,
    0, 1425, 55, 0, 879, 0, 1425, 9318, 1702, 1703, 1704, 0, 163, 1723, 9319, 1724,
    144, 925, 9335, 0, 79, 9324, 0, 9332, 9330, 0, 79, 9327, 0, 9336, 9346, 0,
    9356, 153, 154, 9357, 9367, 1727, 636, 9368, 362, 0, 86, 514, 9371, 9381, 9391, 9401,
    9411, 9421, 9431, 9441, 9451, 9460, 0, 9470, 752, 753, 732, 0, 9471, 619, 1735, 752,
    1736, 9472, 0, 1738, 752, 1736, 325, 9476, 9486, 9496, 9506, 9516, 9526, 9536, 9546, 9556,
    9566, 9576, 9586, 9596, 0, 9606, 655, 1747, 1723, 1748, 468, 9607, 1534, 1750, 0, 1751,
    227, 0, 879, 1425, 1752, 9608, 9618, 9628, 9638, 9648, 9658, 9668, 9678, 9688, 9698, 9708,
    9718, 9728, 9738, 9748, 9758, 1766, 1767, 1768, 9759, 9769, 9780, 0, 156, 165, 240, 9790,
    9800, 9812, 9810, 333, 9815, 0, 1548, 1177, 0, 828, 1771, 9825, 9835, 9845, 9855, 9865,
    0, 9875, 1775, 1776, 1777, 9876, 9886, 9896, 0, 1781, 9906, 582, 58, 9907, 0, 641,
    9917, 9927, 0, 9937, 9947, 9957, 9967, 0, 1500, 783, 785, 1502, 1504, 1508, 913, 1506,
    983, 707, 985, 218, 977, 0, 1295, 9977, 0, 983, 1295, 9981, 9991, 10001, 0, 157,
    10011, 240, 1789, 0, 240, 1789, 1246, 785, 10012, 10022, 10032, 10042, 10052, 0, 139, 633,
    10062, 10072, 227, 10082, 10092, 10102, 10112, 10122, 10132, 10142, 10152, 10162, 10172, 10182, 10192, 10202,
    10212, 0, 10222, 1805, 774, 10229, 0, 687, 10226, 10224, 10223, 534, 281, 325, 10231, 10241,
    10251, 10261, 10271, 10281, 10291, 10301, 10311, 10321, 10331, 10341, 10351, 10361, 10371, 10381, 10391, 10401,
    10411, 10421, 0, 701, 0, 325, 10431, 431, 10434, 325, 10437, 10440, 10443, 10453, 0, 941,
    10466, 10463, 325, 10467, 977, 0, 196, 257, 0, 979, 257, 980, 981, 982, 1349, 921,
    10468, 10478, 10488, 10495, 0, 84, 0, 10506, 10505, 0, 227, 517, 0, 1011, 417, 1011,
    10510, 0, 227, 10520, 1366, 979, 1826, 218, 1828, 0, 619, 1830, 10521, 10524, 1832, 152,
    10526, 1834, 0, 1498, 619, 1830, 10529, 10532, 1832, 152, 10534, 1515, 0, 619, 0, 1425,
    55, 0, 10537, 0, 619, 1425, 46, 1835, 10547, 10557, 10567, 10574, 10584, 10594, 10604, 10614,
    10624, 10634, 10644, 10654, 10664, 10674, 10684,
];

pub static SEGMENT_TYPE_OFFSETS: &[u32] = &[
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 65, 4294967295, 4294967295, 76, 4294967295, 4294967295, 4294967295, 4294967295, 76, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 78, 4294967295, 80, 83, 85, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 65, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 162, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 192, 194, 4294967295, 4294967295, 4294967295, 4294967295, 78, 4294967295,
    4294967295, 4294967295, 198, 4294967295, 4294967295, 4294967295, 200, 4294967295, 4294967295, 4294967295, 78, 4294967295, 200, 4294967295, 204, 4294967295,
    4294967295, 209, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 220, 4294967295, 221, 226, 4294967295,
    4294967295, 4294967295, 4294967295, 229, 4294967295, 4294967295, 4294967295, 237, 4294967295, 4294967295, 241, 243, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 268, 4294967295, 270, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 280, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 285, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 290, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 241, 4294967295, 4294967295, 4294967295, 241, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 302, 302, 4294967295, 302, 302,
    4294967295, 4294967295, 4294967295, 4294967295, 326, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 361, 204, 290, 326, 4294967295, 363, 4294967295, 4294967295,
    4294967295, 364, 3, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 302, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 290, 407, 4294967295, 413, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 443, 4294967295, 443, 4294967295, 4294967295, 280,
    4294967295, 4294967295, 446, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 229, 4294967295, 4294967295, 449, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 221, 4294967295, 465, 4294967295, 4294967295, 471, 4294967295,
    4294967295, 4294967295, 4294967295, 326, 4294967295, 465, 4294967295, 4294967295, 471, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 527, 4294967295, 204,
    4294967295, 4294967295, 4294967295, 4294967295, 78, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 326, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 545, 4294967295, 4294967295, 548, 4294967295, 4294967295, 290, 363,
    4294967295, 4294967295, 4294967295, 551, 554, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 562, 4294967295, 4294967295, 4294967295, 280, 4294967295, 280, 4294967295,
    280, 4294967295, 280, 4294967295, 280, 4294967295, 280, 4294967295, 280, 280, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 280, 4294967295, 302, 280, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 363, 4294967295, 363,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 80, 83, 4294967295, 4294967295,
    78, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 634, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 80,
    83, 4294967295, 4294967295, 363, 640, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 645, 4294967295, 241, 4294967295, 4294967295,
    4294967295, 647, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 78, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 280,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 200, 4294967295, 4294967295, 4294967295, 4294967295,
    653, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 268, 4294967295, 657, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 241, 665, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    285, 668, 545, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 241, 4294967295, 672, 4294967295, 4294967295, 4294967295, 674, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 290, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 241, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 241, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 326, 4294967295, 4294967295, 4294967295, 4294967295, 704, 4294967295, 4294967295,
    4294967295, 200, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 241, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 708, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 80, 80, 4294967295, 719, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 634, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 733, 4294967295, 4294967295, 4294967295, 326, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 784,
    787, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 78, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 363, 4294967295, 363, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 634, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 80, 4294967295, 4294967295, 4294967295, 4294967295, 645, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 78, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 653, 4294967295, 4294967295,
    4294967295, 4294967295, 268, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 241, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 674,
    4294967295, 4294967295, 4294967295, 4294967295, 78, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 200, 4294967295, 4294967295, 4294967295, 4294967295, 241,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 326, 4294967295, 4294967295, 4294967295, 4294967295,
    829, 831, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 860, 862, 864, 4294967295, 4294967295, 4294967295, 527, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 97, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 97, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 892, 4294967295, 894, 527, 4294967295, 4294967295, 4294967295, 4294967295, 204, 4294967295, 4294967295, 4294967295, 326,
    4294967295, 4294967295, 4294967295, 4294967295, 290, 204, 897, 326, 4294967295, 85, 4294967295, 4294967295, 829, 901, 4294967295, 4294967295,
    326, 4294967295, 363, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 897, 4294967295, 4294967295, 4294967295, 4294967295, 908, 4294967295,
    361, 910, 912, 290, 914, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 326, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 937, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 326,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 361, 4294967295, 290,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    969, 4294967295, 4294967295, 978, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 984, 708, 986,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 990, 992, 243, 4294967295, 4294967295, 996, 4294967295, 999, 4294967295,
    4294967295, 4294967295, 243, 996, 4294967295, 4294967295, 4294967295, 4294967295, 999, 4294967295, 4294967295, 4294967295, 4294967295, 1003, 1005, 4294967295,
    969, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 221, 4294967295, 4294967295, 4294967295, 999, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 237, 4294967295, 4294967295, 4294967295, 4294967295, 1005, 4294967295, 4294967295, 326, 4294967295, 326, 4294967295, 326,
    4294967295, 363, 4294967295, 4294967295, 326, 4294967295, 4294967295, 326, 4294967295, 4294967295, 4294967295, 326, 4294967295, 326, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 1014, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 290, 4294967295, 4294967295, 4294967295,
    4294967295, 290, 4294967295, 647, 910, 4294967295, 326, 4294967295, 363, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 407, 4294967295, 4294967295, 4294967295, 407, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 363, 4294967295, 4294967295, 4294967295, 4294967295, 363, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    80, 704, 4294967295, 4294967295, 4294967295, 80, 4294967295, 80, 80, 1031, 704, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1042, 4294967295, 1044, 4294967295,
    4294967295, 200, 4294967295, 4294967295, 4294967295, 4294967295, 198, 78, 4294967295, 1046, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 198, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1059, 1061, 4294967295, 4294967295,
    4294967295, 290, 4294967295, 326, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 978, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 290, 4294967295, 326, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1061, 719,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 326, 4294967295, 4294967295, 4294967295, 4294967295, 978, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1080, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 204, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 241, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1005, 4294967295, 4294967295, 4294967295, 221, 1137, 4294967295, 1140,
    4294967295, 237, 4294967295, 4294967295, 4294967295, 1005, 1137, 4294967295, 4294967295, 1005, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    221, 4294967295, 4294967295, 326, 237, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 326, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 326, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 97, 4294967295, 4294967295, 1178, 1180, 1183, 756, 756,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 78, 4294967295, 4294967295, 78, 4294967295, 4294967295, 78, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 78, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1031, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 1232, 1234, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 326, 4294967295, 4294967295, 1237, 1239,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 326, 4294967295, 1241, 4294967295, 4294967295, 4294967295, 241, 4294967295, 4294967295, 4294967295,
    241, 4294967295, 4294967295, 4294967295, 4294967295, 243, 4294967295, 204, 243, 221, 1140, 237, 1245, 4294967295, 4294967295, 1247,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1269, 4294967295, 4294967295, 4294967295, 1271, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1281, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1031, 908, 4294967295, 908, 4294967295,
    1294, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 862, 860, 864, 1298, 4294967295, 413, 4294967295, 4294967295, 908, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1315, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 326, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 290, 4294967295, 326, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1339, 978, 4294967295, 4294967295, 978, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1269, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 290, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 326, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1382, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 326, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 241, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1042, 4294967295, 1044, 4294967295, 4294967295, 200, 4294967295,
    4294967295, 4294967295, 4294967295, 198, 198, 4294967295, 1046, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1487, 4294967295,
    4294967295, 243, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 326, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1495, 4294967295, 1497, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 984, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 978, 4294967295, 984, 4294967295, 4294967295, 4294967295, 1499, 1501, 784, 787, 1503, 1505, 914, 1507,
    1509, 220, 1511, 1513, 978, 1507, 984, 708, 986, 4294967295, 4294967295, 4294967295, 1294, 4294967295, 4294967295, 1294,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1525,
    4294967295, 290, 280, 4294967295, 4294967295, 4294967295, 204, 290, 326, 1031, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 984,
    4294967295, 220, 978, 1507, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    280, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 653, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1537, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 892, 4294967295, 4294967295, 4294967295, 363, 894, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1549, 209, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1298, 860, 1585, 1587, 1589, 1591, 1593,
    1595, 1597, 1599, 1601, 1603, 1605, 1607, 1609, 1611, 1613, 1615, 1617, 1619, 1621, 1623, 864,
    862, 1625, 1627, 1629, 1631, 1633, 1635, 1637, 1639, 1641, 1643, 1645, 1647, 1649, 1651, 1653,
    1655, 1657, 1509, 545, 1537, 1660, 1662, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 302, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 243, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 232, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 78, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1031,
    4294967295, 204, 241, 4294967295, 4294967295, 4294967295, 1298, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1706,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 645, 4294967295, 4294967295, 4294967295, 290, 1708, 4294967295, 4294967295, 4294967295, 645,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 290, 4294967295, 4294967295, 4294967295, 78, 4294967295, 4294967295, 241, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 290, 4294967295, 4294967295, 4294967295, 78, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 290, 4294967295, 4294967295, 78, 4294967295, 4294967295, 78,
    4294967295, 4294967295, 78, 4294967295, 4294967295, 78, 4294967295, 4294967295, 78, 4294967295, 4294967295, 78, 4294967295, 4294967295, 78, 4294967295,
    4294967295, 78, 4294967295, 4294967295, 78, 4294967295, 4294967295, 241, 4294967295, 4294967295, 634, 4294967295, 4294967295, 78, 4294967295, 4294967295,
    241, 4294967295, 4294967295, 78, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 280,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 280, 4294967295, 4294967295, 4294967295, 4294967295,
    280, 78, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 290, 4294967295, 4294967295, 4294967295, 241, 4294967295, 4294967295,
    4294967295, 241, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 78, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1722, 4294967295, 4294967295, 4294967295, 78, 4294967295, 4294967295, 4294967295, 290, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 80, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 80, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 363, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 733, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 326, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 241, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1549, 1178, 4294967295, 829, 831, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1501, 784, 787, 1503, 1505, 1509, 914, 1507,
    984, 708, 986, 220, 978, 4294967295, 4294967295, 4294967295, 4294967295, 984, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 241, 1790, 4294967295, 241, 1790, 1247, 787, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 634,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 326, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 326, 221, 4294967295, 221, 326, 237, 237, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 326, 4294967295, 978, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 1823, 4294967295, 4294967295, 85, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1827, 220, 1829, 4294967295, 4294967295, 4294967295, 221, 4294967295, 1833, 4294967295,
    237, 4294967295, 4294967295, 1499, 4294967295, 4294967295, 221, 4294967295, 1833, 4294967295, 237, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 5, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
];

pub static SEGMENT_CLASS_OFFSETS: &[u32] = &[
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 64, 4294967295, 4294967295, 66, 4294967295, 4294967295, 4294967295, 4294967295, 66, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 79, 81, 84, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 64, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 161, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 191, 193, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295,
    4294967295, 4294967295, 197, 4294967295, 4294967295, 4294967295, 199, 4294967295, 4294967295, 4294967295, 77, 4294967295, 199, 4294967295, 203, 4294967295,
    4294967295, 208, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 218, 4294967295, 222, 224, 4294967295,
    4294967295, 4294967295, 4294967295, 228, 4294967295, 4294967295, 4294967295, 238, 4294967295, 4294967295, 240, 242, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 267, 4294967295, 269, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 278, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 284, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 289, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 240, 4294967295, 4294967295, 4294967295, 240, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 320, 321, 4294967295, 323, 324,
    4294967295, 4294967295, 4294967295, 4294967295, 325, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 359, 203, 289, 325, 4294967295, 362, 4294967295, 4294967295,
    4294967295, 365, 366, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 303, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 289, 408, 4294967295, 409, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 442, 4294967295, 442, 4294967295, 4294967295, 278,
    4294967295, 4294967295, 445, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 228, 4294967295, 4294967295, 448, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 462, 4294967295, 464, 4294967295, 4294967295, 470, 4294967295,
    4294967295, 4294967295, 4294967295, 325, 4294967295, 464, 4294967295, 4294967295, 470, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 357, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 526, 4294967295, 203,
    4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 325, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 543, 4294967295, 4294967295, 547, 4294967295, 4294967295, 289, 362,
    4294967295, 4294967295, 4294967295, 550, 552, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 563, 4294967295, 4294967295, 4294967295, 278, 4294967295, 567, 4294967295,
    569, 4294967295, 571, 4294967295, 573, 4294967295, 575, 4294967295, 577, 578, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 584, 4294967295, 586, 587, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 362, 4294967295, 362,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 79, 81, 4294967295, 4294967295,
    77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 633, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 79,
    81, 4294967295, 4294967295, 362, 639, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 644, 4294967295, 240, 4294967295, 4294967295,
    4294967295, 646, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 278,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 199, 4294967295, 4294967295, 4294967295, 4294967295,
    652, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 267, 4294967295, 656, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 240, 664, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    284, 667, 543, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 240, 4294967295, 671, 4294967295, 4294967295, 4294967295, 673, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 289, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 240, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 240, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 325, 4294967295, 4294967295, 4294967295, 4294967295, 703, 4294967295, 4294967295,
    4294967295, 199, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 240, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 707, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 716, 717, 4294967295, 718, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 633, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 732, 4294967295, 4294967295, 4294967295, 325, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 783,
    785, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 362, 4294967295, 362, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 633, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 79, 4294967295, 4294967295, 4294967295, 4294967295, 644, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 652, 4294967295, 4294967295,
    4294967295, 4294967295, 267, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 240, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 673,
    4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 199, 4294967295, 4294967295, 4294967295, 4294967295, 240,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 325, 4294967295, 4294967295, 4294967295, 4294967295,
    828, 830, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 859, 861, 863, 4294967295, 4294967295, 4294967295, 866, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 884, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 884, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 891, 4294967295, 893, 526, 4294967295, 4294967295, 4294967295, 4294967295, 203, 4294967295, 4294967295, 4294967295, 325,
    4294967295, 4294967295, 4294967295, 4294967295, 289, 203, 896, 325, 4294967295, 899, 4294967295, 4294967295, 828, 900, 4294967295, 4294967295,
    325, 4294967295, 362, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 896, 4294967295, 4294967295, 4294967295, 4294967295, 907, 4294967295,
    359, 909, 911, 289, 913, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 325, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 936, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 325,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 359, 4294967295, 289,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    968, 4294967295, 4294967295, 977, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 983, 707, 985,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 989, 991, 242, 4294967295, 4294967295, 994, 4294967295, 998, 4294967295,
    4294967295, 4294967295, 242, 994, 4294967295, 4294967295, 4294967295, 4294967295, 998, 4294967295, 4294967295, 4294967295, 4294967295, 1001, 1004, 4294967295,
    968, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 222, 4294967295, 4294967295, 4294967295, 998, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 238, 4294967295, 4294967295, 4294967295, 4294967295, 1004, 4294967295, 4294967295, 325, 4294967295, 325, 4294967295, 325,
    4294967295, 362, 4294967295, 4294967295, 325, 4294967295, 4294967295, 325, 4294967295, 4294967295, 4294967295, 325, 4294967295, 325, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 1013, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 289, 4294967295, 4294967295, 4294967295,
    4294967295, 289, 4294967295, 646, 1020, 4294967295, 325, 4294967295, 362, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 408, 4294967295, 4294967295, 4294967295, 408, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 362, 4294967295, 4294967295, 4294967295, 4294967295, 362, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    1026, 1027, 4294967295, 4294967295, 4294967295, 79, 4294967295, 1026, 1029, 1030, 703, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1041, 4294967295, 1043, 4294967295,
    4294967295, 199, 4294967295, 4294967295, 4294967295, 4294967295, 197, 77, 4294967295, 1045, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 197, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1057, 1060, 4294967295, 4294967295,
    4294967295, 289, 4294967295, 325, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 977, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 289, 4294967295, 325, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1060, 718,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 325, 4294967295, 4294967295, 4294967295, 4294967295, 977, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1081, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 203, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 240, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1004, 4294967295, 4294967295, 4294967295, 222, 1136, 4294967295, 1139,
    4294967295, 238, 4294967295, 4294967295, 4294967295, 1004, 1136, 4294967295, 4294967295, 1004, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    462, 4294967295, 4294967295, 325, 238, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 325, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 325, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 884, 4294967295, 4294967295, 1177, 1179, 1181, 919, 757,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 77, 4294967295, 4294967295, 77, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1030, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 1231, 1233, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 325, 4294967295, 4294967295, 1236, 1238,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 325, 4294967295, 1240, 4294967295, 4294967295, 4294967295, 240, 4294967295, 4294967295, 4294967295,
    240, 4294967295, 4294967295, 4294967295, 4294967295, 242, 4294967295, 203, 242, 222, 1139, 238, 1244, 4294967295, 4294967295, 1246,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1268, 4294967295, 4294967295, 4294967295, 1270, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1282, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1030, 1292, 4294967295, 907, 4294967295,
    1293, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 861, 859, 863, 1297, 4294967295, 1299, 4294967295, 4294967295, 907, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1314, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 325, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 289, 4294967295, 325, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1338, 977, 4294967295, 4294967295, 977, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1268, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 289, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 325, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1381, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 325, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 240, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1041, 4294967295, 1043, 4294967295, 4294967295, 199, 4294967295,
    4294967295, 4294967295, 4294967295, 197, 197, 4294967295, 1045, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1486, 4294967295,
    4294967295, 242, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 325, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1494, 4294967295, 1496, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 983, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 977, 4294967295, 983, 4294967295, 4294967295, 4294967295, 1498, 1500, 783, 785, 1502, 1504, 913, 1506,
    1508, 218, 1510, 1512, 977, 1506, 983, 707, 985, 4294967295, 4294967295, 4294967295, 1293, 4294967295, 4294967295, 1514,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1524,
    4294967295, 289, 278, 4294967295, 4294967295, 4294967295, 203, 289, 325, 1030, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 983,
    4294967295, 218, 977, 1506, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    278, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 652, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1536, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 891, 4294967295, 4294967295, 4294967295, 362, 893, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1548, 208, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1297, 859, 1584, 1586, 1588, 1590, 1592,
    1594, 1596, 1598, 1600, 1602, 1604, 1606, 1608, 1610, 1612, 1614, 1616, 1618, 1620, 1622, 863,
    861, 1624, 1626, 1628, 1630, 1632, 1634, 1636, 1638, 1640, 1642, 1644, 1646, 1648, 1650, 1652,
    1654, 1656, 1508, 1658, 1536, 1659, 1661, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1672, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 242, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 231, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1030,
    4294967295, 203, 240, 4294967295, 4294967295, 4294967295, 1297, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1705,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 644, 4294967295, 4294967295, 4294967295, 289, 1707, 4294967295, 4294967295, 4294967295, 644,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 289, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 240, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 289, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 289, 4294967295, 4294967295, 77, 4294967295, 4294967295, 77,
    4294967295, 4294967295, 77, 4294967295, 4294967295, 77, 4294967295, 4294967295, 77, 4294967295, 4294967295, 77, 4294967295, 4294967295, 77, 4294967295,
    4294967295, 77, 4294967295, 4294967295, 77, 4294967295, 4294967295, 240, 4294967295, 4294967295, 633, 4294967295, 4294967295, 77, 4294967295, 4294967295,
    240, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 278,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 278, 4294967295, 4294967295, 4294967295, 4294967295,
    278, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 289, 4294967295, 4294967295, 4294967295, 240, 4294967295, 4294967295,
    4294967295, 240, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1721, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 289, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 79, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 79, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 362, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 732, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 325, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 240, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1548, 1177, 4294967295, 828, 1771, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1500, 783, 785, 1502, 1504, 1508, 913, 1506,
    983, 707, 985, 218, 977, 4294967295, 4294967295, 4294967295, 4294967295, 983, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 240, 1789, 4294967295, 240, 1789, 1246, 785, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 633,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 325, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 325, 222, 4294967295, 462, 325, 238, 238, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 325, 4294967295, 977, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 1824, 4294967295, 4294967295, 84, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1826, 218, 1828, 4294967295, 4294967295, 4294967295, 222, 4294967295, 1832, 4294967295,
    238, 4294967295, 4294967295, 1498, 4294967295, 4294967295, 222, 4294967295, 1832, 4294967295, 238, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1021, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
];

pub static CASEFOLD_SPARSE: &[(u32, u8)] = &[
    (2113, 1),
    (2440, 1),
];

pub static TRIM_CHARS_SPARSE: &[(u32, u32, u8)] = &[
];

pub static TRIM_CHARS_DATA: &[u32] = &[
];

pub static SEGMENT_CLASS_TYPES_SPARSE: &[(u32, u32, u8)] = &[
    (38, 0, 2),
    (41, 2, 2),
    (46, 4, 2),
    (53, 6, 2),
    (55, 8, 2),
    (56, 10, 2),
    (57, 12, 3),
    (99, 15, 2),
    (135, 17, 2),
    (184, 19, 2),
    (185, 21, 2),
    (190, 23, 2),
    (194, 25, 3),
    (198, 28, 3),
    (202, 31, 2),
    (204, 33, 3),
    (206, 36, 2),
    (209, 38, 2),
    (219, 40, 2),
    (221, 42, 4),
    (222, 46, 2),
    (227, 48, 2),
    (231, 50, 5),
    (234, 55, 3),
    (235, 58, 2),
    (259, 60, 3),
    (261, 63, 2),
    (270, 65, 2),
    (279, 67, 2),
    (285, 69, 3),
    (293, 72, 3),
    (297, 75, 3),
    (315, 78, 2),
    (316, 80, 2),
    (318, 82, 2),
    (319, 84, 2),
    (324, 86, 2),
    (360, 88, 2),
    (361, 90, 2),
    (362, 92, 3),
    (363, 95, 2),
    (365, 97, 2),
    (369, 99, 2),
    (370, 101, 1),
    (382, 102, 3),
    (411, 105, 3),
    (412, 108, 2),
    (414, 110, 2),
    (442, 112, 2),
    (444, 114, 2),
    (447, 116, 2),
    (450, 118, 2),
    (457, 120, 2),
    (460, 122, 2),
    (473, 124, 4),
    (475, 128, 2),
    (478, 130, 2),
    (483, 132, 2),
    (485, 134, 2),
    (488, 136, 2),
    (538, 138, 2),
    (557, 140, 2),
    (559, 142, 2),
    (564, 144, 2),
    (572, 146, 2),
    (584, 148, 2),
    (587, 150, 3),
    (590, 153, 3),
    (591, 156, 2),
    (595, 158, 2),
    (596, 160, 2),
    (616, 162, 3),
    (620, 165, 2),
    (622, 167, 2),
    (624, 169, 2),
    (626, 171, 2),
    (628, 173, 2),
    (630, 175, 2),
    (632, 177, 2),
    (633, 179, 2),
    (645, 181, 3),
    (647, 184, 2),
    (648, 186, 2),
    (685, 188, 2),
    (687, 190, 2),
    (700, 192, 2),
    (701, 194, 2),
    (704, 196, 2),
    (712, 198, 3),
    (719, 201, 2),
    (720, 203, 2),
    (723, 205, 2),
    (724, 207, 2),
    (731, 209, 3),
    (733, 212, 3),
    (737, 215, 2),
    (744, 217, 2),
    (751, 219, 2),
    (763, 221, 3),
    (768, 224, 3),
    (792, 227, 3),
    (794, 230, 2),
    (809, 232, 3),
    (810, 235, 2),
    (816, 237, 2),
    (817, 239, 2),
    (818, 241, 2),
    (824, 243, 3),
    (826, 246, 2),
    (830, 248, 3),
    (845, 251, 3),
    (851, 254, 3),
    (865, 257, 3),
    (888, 260, 2),
    (893, 262, 2),
    (897, 264, 3),
    (904, 267, 3),
    (910, 270, 2),
    (921, 272, 2),
    (922, 274, 2),
    (924, 276, 2),
    (942, 278, 3),
    (950, 281, 2),
    (954, 283, 2),
    (1023, 285, 2),
    (1024, 287, 2),
    (1038, 289, 2),
    (1065, 291, 2),
    (1067, 293, 2),
    (1073, 295, 3),
    (1079, 298, 2),
    (1084, 300, 3),
    (1091, 303, 2),
    (1101, 305, 3),
    (1106, 308, 3),
    (1113, 311, 3),
    (1119, 314, 3),
    (1124, 317, 2),
    (1130, 319, 3),
    (1135, 322, 3),
    (1147, 325, 2),
    (1152, 327, 2),
    (1153, 329, 2),
    (1188, 331, 2),
    (1189, 333, 2),
    (1190, 335, 2),
    (1194, 337, 2),
    (1224, 339, 2),
    (1241, 341, 2),
    (1251, 343, 2),
    (1253, 345, 2),
    (1254, 347, 2),
    (1259, 349, 2),
    (1263, 351, 2),
    (1268, 353, 3),
    (1269, 356, 2),
    (1270, 358, 2),
    (1271, 360, 2),
    (1273, 362, 3),
    (1276, 365, 2),
    (1277, 367, 2),
    (1280, 369, 2),
    (1282, 371, 2),
    (1289, 373, 2),
    (1294, 375, 2),
    (1296, 377, 2),
    (1297, 379, 2),
    (1298, 381, 2),
    (1299, 383, 3),
    (1300, 386, 2),
    (1315, 388, 2),
    (1332, 390, 2),
    (1343, 392, 2),
    (1373, 394, 2),
    (1375, 396, 3),
    (1392, 399, 2),
    (1395, 401, 2),
    (1405, 403, 2),
    (1406, 405, 2),
    (1407, 407, 2),
    (1415, 409, 2),
    (1416, 411, 2),
    (1417, 417, 2),
    (1420, 413, 2),
    (1422, 415, 2),
    (1426, 419, 2),
    (1427, 421, 2),
    (1432, 423, 2),
    (1437, 425, 2),
    (1438, 427, 2),
    (1440, 429, 2),
    (1451, 431, 2),
    (1464, 433, 2),
    (1467, 435, 2),
    (1469, 437, 2),
    (1471, 439, 2),
    (1473, 441, 2),
    (1476, 443, 2),
    (1479, 445, 2),
    (1483, 447, 2),
    (1485, 449, 2),
    (1492, 451, 2),
    (1500, 453, 3),
    (1505, 456, 3),
    (1507, 459, 2),
    (1508, 461, 2),
    (1510, 463, 2),
    (1512, 465, 2),
    (1529, 467, 2),
    (1533, 469, 2),
    (1541, 471, 2),
    (1546, 473, 2),
    (1552, 475, 2),
    (1553, 477, 2),
    (1557, 485, 2),
    (1559, 479, 2),
    (1560, 481, 2),
    (1561, 483, 2),
    (1562, 487, 2),
    (1580, 489, 2),
    (1582, 491, 2),
    (1585, 493, 3),
    (1590, 496, 3),
    (1591, 499, 2),
    (1593, 501, 2),
    (1613, 503, 3),
    (1628, 506, 2),
    (1629, 508, 2),
    (1633, 510, 3),
    (1635, 513, 2),
    (1641, 515, 2),
    (1650, 517, 3),
    (1652, 520, 2),
    (1662, 522, 2),
    (1663, 524, 2),
    (1670, 526, 2),
    (1675, 528, 2),
    (1694, 530, 3),
    (1729, 533, 2),
    (1756, 535, 3),
    (1816, 538, 2),
    (1821, 540, 2),
    (1823, 542, 2),
    (1829, 544, 2),
    (1830, 546, 2),
    (1833, 548, 2),
    (1843, 550, 2),
    (1865, 552, 4),
    (1908, 556, 2),
    (1915, 558, 2),
    (1928, 560, 2),
    (1931, 562, 2),
    (1932, 564, 2),
    (1933, 566, 2),
    (1934, 568, 5),
    (1935, 573, 3),
    (1953, 576, 2),
    (1956, 578, 2),
    (1959, 580, 2),
    (2001, 582, 2),
    (2030, 584, 2),
    (2036, 586, 2),
    (2037, 588, 2),
    (2043, 590, 2),
    (2046, 592, 2),
    (2047, 594, 2),
    (2054, 596, 2),
    (2056, 598, 2),
    (2060, 600, 3),
    (2064, 603, 3),
    (2069, 606, 2),
    (2071, 608, 2),
    (2072, 610, 2),
    (2074, 612, 2),
    (2076, 614, 2),
    (2079, 616, 2),
    (2121, 618, 2),
    (2125, 620, 2),
    (2140, 622, 3),
    (2155, 625, 2),
    (2156, 627, 2),
    (2158, 629, 2),
    (2160, 631, 2),
    (2166, 633, 2),
    (2167, 635, 2),
    (2168, 637, 2),
    (2169, 639, 2),
    (2171, 641, 2),
    (2174, 643, 2),
    (2220, 645, 2),
    (2249, 647, 2),
    (2284, 649, 3),
    (2286, 652, 2),
    (2295, 654, 2),
    (2296, 656, 2),
    (2299, 658, 2),
    (2323, 660, 2),
    (2333, 662, 3),
    (2361, 665, 2),
    (2389, 667, 2),
    (2426, 669, 2),
    (2461, 671, 3),
    (2537, 674, 2),
    (2539, 676, 2),
    (2542, 678, 3),
    (2547, 681, 3),
    (2548, 684, 3),
    (2550, 687, 2),
    (2606, 689, 2),
    (2609, 691, 2),
    (2615, 693, 2),
    (2641, 695, 2),
    (2643, 697, 2),
    (2650, 699, 2),
    (2658, 701, 2),
    (2660, 703, 2),
    (2664, 705, 2),
    (2665, 707, 2),
    (2666, 709, 2),
    (2667, 711, 2),
    (2668, 713, 2),
    (2669, 715, 2),
    (2670, 717, 2),
    (2671, 719, 2),
    (2672, 721, 2),
    (2673, 723, 2),
    (2674, 725, 2),
    (2675, 727, 2),
    (2676, 729, 2),
    (2677, 731, 2),
    (2678, 733, 2),
    (2679, 735, 2),
    (2680, 737, 2),
    (2684, 739, 2),
    (2687, 741, 2),
    (2719, 743, 2),
    (2721, 745, 3),
    (2722, 748, 2),
    (2726, 750, 2),
    (2727, 752, 3),
    (2728, 755, 2),
    (2729, 757, 2),
    (2735, 759, 2),
    (2737, 761, 2),
    (2738, 763, 2),
    (2739, 765, 2),
    (2752, 767, 2),
    (2769, 769, 3),
    (2776, 772, 2),
    (2785, 774, 2),
    (2789, 776, 2),
    (2790, 778, 2),
    (2812, 780, 2),
    (2813, 782, 2),
    (2857, 784, 2),
    (2858, 786, 2),
    (2859, 788, 2),
    (2860, 790, 2),
    (2861, 792, 2),
    (2862, 794, 2),
    (2863, 796, 2),
    (2864, 798, 2),
    (2865, 800, 2),
    (2866, 802, 2),
    (2867, 804, 2),
    (2868, 806, 2),
    (2869, 808, 2),
    (2870, 810, 2),
    (2871, 812, 2),
    (2872, 814, 2),
    (2873, 816, 2),
    (2874, 818, 2),
    (2875, 820, 2),
    (2876, 822, 2),
    (2877, 824, 2),
    (2878, 826, 2),
    (2879, 828, 2),
    (2880, 830, 2),
    (2881, 832, 2),
    (2882, 834, 2),
    (2883, 836, 2),
    (2884, 838, 2),
    (2885, 840, 2),
    (2886, 842, 2),
    (2887, 844, 2),
    (2888, 846, 2),
    (2889, 848, 2),
    (2890, 850, 2),
    (2891, 852, 2),
    (2892, 854, 2),
    (2893, 856, 2),
    (2894, 858, 2),
    (2895, 860, 2),
    (2896, 862, 2),
    (2897, 864, 2),
    (2898, 866, 2),
    (2899, 868, 2),
    (2900, 870, 2),
    (2901, 872, 2),
    (2902, 874, 2),
    (2917, 876, 2),
    (2923, 878, 2),
    (2935, 880, 3),
    (2945, 883, 2),
    (2959, 885, 2),
    (2961, 887, 2),
    (2962, 889, 3),
    (2966, 892, 2),
    (3039, 894, 2),
    (3046, 896, 3),
    (3050, 899, 3),
    (3051, 902, 2),
    (3055, 904, 3),
    (3062, 907, 3),
    (3066, 910, 2),
    (3069, 912, 3),
    (3076, 915, 3),
    (3080, 918, 2),
    (3097, 920, 3),
    (3100, 923, 2),
    (3103, 925, 2),
    (3106, 927, 2),
    (3109, 929, 2),
    (3112, 931, 2),
    (3115, 933, 2),
    (3118, 935, 2),
    (3121, 937, 2),
    (3124, 939, 2),
    (3127, 941, 3),
    (3130, 944, 3),
    (3133, 947, 2),
    (3136, 949, 3),
    (3139, 952, 2),
    (3167, 954, 2),
    (3179, 956, 2),
    (3184, 958, 2),
    (3185, 960, 2),
    (3193, 962, 3),
    (3197, 965, 3),
    (3201, 968, 3),
    (3208, 971, 2),
    (3221, 973, 2),
    (3225, 975, 2),
    (3229, 977, 3),
    (3252, 980, 2),
    (3258, 982, 2),
    (3272, 984, 2),
    (3290, 986, 2),
    (3302, 988, 2),
    (3358, 990, 3),
    (3366, 993, 2),
    (3367, 995, 2),
    (3369, 997, 2),
    (3370, 999, 2),
    (3400, 1001, 2),
    (3401, 1003, 2),
    (3402, 1005, 2),
    (3403, 1007, 2),
    (3404, 1009, 2),
    (3405, 1011, 2),
    (3406, 1013, 2),
    (3407, 1015, 2),
    (3408, 1017, 2),
    (3409, 1019, 2),
    (3410, 1021, 2),
    (3411, 1023, 2),
    (3412, 1025, 2),
    (3417, 1027, 2),
    (3425, 1029, 3),
    (3426, 1032, 2),
    (3428, 1034, 3),
    (3429, 1037, 2),
    (3430, 1039, 2),
    (3431, 1041, 2),
    (3439, 1043, 3),
    (3469, 1046, 2),
    (3493, 1048, 2),
    (3497, 1050, 2),
    (3506, 1052, 2),
    (3508, 1054, 2),
    (3522, 1056, 3),
    (3525, 1059, 3),
    (3542, 1062, 2),
    (3543, 1064, 2),
    (3544, 1066, 2),
    (3550, 1068, 2),
    (3555, 1070, 2),
    (3560, 1072, 2),
    (3578, 1074, 3),
];

pub static SEGMENT_CLASS_TYPES_DATA: &[u32] = &[
    65, 3, 76, 3, 76, 3, 3, 78, 3, 80, 3, 83, 3, 78, 85, 65,
    3, 162, 3, 3, 192, 3, 194, 3, 78, 3, 78, 198, 3, 78, 200, 3,
    78, 3, 78, 200, 3, 204, 209, 3, 3, 220, 3, 221, 223, 4, 226, 3,
    3, 229, 3, 237, 221, 223, 4, 3, 78, 241, 243, 3, 3, 78, 268, 270,
    3, 3, 280, 3, 285, 3, 290, 78, 3, 78, 241, 3, 78, 241, 3, 302,
    3, 302, 3, 302, 3, 302, 3, 326, 3, 361, 3, 204, 3, 290, 78, 3,
    326, 3, 363, 3, 364, 3, 3, 302, 4, 3, 290, 78, 3, 407, 3, 413,
    3, 443, 3, 443, 3, 280, 3, 446, 3, 229, 3, 449, 3, 221, 223, 4,
    3, 465, 3, 471, 3, 326, 3, 465, 3, 471, 3, 4, 3, 527, 3, 204,
    3, 78, 3, 326, 3, 545, 3, 548, 78, 3, 290, 78, 3, 363, 3, 551,
    3, 554, 3, 562, 4, 3, 280, 3, 280, 3, 280, 3, 280, 3, 280, 3,
    280, 3, 280, 3, 280, 3, 280, 4, 3, 302, 3, 280, 3, 363, 3, 363,
    3, 80, 3, 83, 3, 78, 3, 634, 78, 3, 80, 3, 83, 3, 363, 3,
    640, 3, 645, 78, 3, 78, 241, 3, 647, 3, 78, 3, 280, 3, 78, 200,
    3, 78, 653, 3, 78, 268, 3, 657, 3, 78, 241, 3, 665, 3, 285, 3,
    668, 3, 545, 3, 78, 241, 3, 672, 3, 78, 674, 3, 290, 78, 3, 78,
    241, 3, 78, 241, 3, 326, 3, 704, 3, 78, 200, 3, 78, 241, 3, 708,
    3, 80, 3, 80, 3, 719, 3, 634, 78, 3, 733, 3, 326, 3, 784, 3,
    787, 3, 78, 3, 363, 3, 363, 3, 634, 78, 3, 80, 3, 645, 78, 3,
    78, 3, 78, 653, 3, 78, 268, 3, 78, 241, 3, 78, 674, 3, 78, 3,
    78, 200, 3, 78, 241, 3, 326, 3, 829, 3, 831, 3, 860, 3, 862, 3,
    864, 3, 527, 3, 97, 3, 97, 3, 892, 3, 894, 3, 527, 3, 204, 3,
    326, 3, 290, 78, 3, 204, 3, 897, 3, 326, 3, 78, 85, 3, 829, 3,
    901, 3, 326, 3, 363, 3, 897, 3, 908, 3, 361, 3, 910, 912, 3, 3,
    290, 78, 3, 914, 3, 326, 3, 937, 3, 326, 3, 361, 3, 290, 78, 3,
    969, 3, 978, 3, 984, 3, 708, 3, 986, 3, 990, 3, 992, 3, 996, 3,
    999, 243, 3, 243, 3, 3, 996, 3, 999, 3, 1003, 3, 1005, 3, 969, 3,
    999, 3, 1005, 3, 326, 3, 326, 3, 326, 3, 363, 3, 326, 3, 326, 3,
    326, 3, 326, 1014, 3, 3, 290, 78, 3, 290, 78, 3, 647, 3, 910, 3,
    326, 3, 363, 3, 407, 3, 407, 3, 363, 3, 363, 3, 80, 3, 704, 3,
    80, 3, 80, 3, 1031, 3, 80, 3, 704, 1042, 3, 1044, 3, 3, 78, 200,
    3, 78, 198, 3, 78, 1046, 3, 3, 78, 198, 3, 1059, 3, 1061, 3, 290,
    78, 3, 326, 3, 978, 3, 290, 78, 3, 326, 3, 1061, 3, 719, 3, 326,
    3, 978, 3, 1080, 4, 3, 204, 3, 78, 241, 3, 1005, 3, 1137, 3, 1140,
    3, 1005, 3, 1137, 3, 1005, 3, 326, 3, 1, 4, 5, 3, 326, 3, 326,
    3, 97, 1178, 3, 3, 1180, 3, 1183, 3, 1, 756, 4, 5, 3, 756, 4,
    3, 78, 3, 78, 3, 78, 3, 78, 3, 1031, 3, 1232, 3, 1234, 3, 326,
    3, 1237, 3, 1239, 3, 326, 3, 1241, 3, 78, 241, 3, 78, 241, 243, 3,
    3, 204, 243, 3, 3, 1140, 3, 1245, 3, 1247, 3, 1269, 3, 1271, 3, 1281,
    4, 3, 1031, 3, 908, 3, 908, 3, 1294, 3, 862, 3, 860, 3, 864, 3,
    1298, 3, 413, 3, 908, 3, 1315, 3, 326, 3, 290, 78, 3, 326, 3, 1339,
    3, 978, 3, 978, 3, 1269, 3, 290, 78, 3, 326, 3, 1382, 3, 326, 3,
    78, 241, 1042, 3, 1044, 3, 3, 78, 200, 3, 78, 198, 3, 78, 198, 1046,
    3, 3, 1487, 243, 3, 3, 326, 3, 1495, 3, 1497, 3, 984, 3, 978, 3,
    984, 3, 1499, 3, 1501, 3, 784, 3, 787, 3, 1503, 3, 1505, 3, 914, 3,
    1507, 3, 1509, 3, 220, 3, 1511, 3, 1513, 3, 978, 3, 1507, 3, 984, 3,
    708, 3, 986, 3, 1294, 3, 1294, 3, 1525, 3, 290, 78, 3, 280, 3, 204,
    3, 290, 78, 3, 326, 3, 1031, 3, 984, 3, 220, 3, 978, 3, 1507, 3,
    280, 3, 78, 653, 3, 1537, 3, 892, 3, 363, 3, 894, 1549, 3, 209, 3,
    3, 1298, 3, 860, 3, 1585, 3, 1587, 3, 1589, 3, 1591, 3, 1593, 3, 1595,
    1597, 3, 3, 1599, 3, 1601, 3, 1603, 1605, 3, 3, 1607, 3, 1609, 3, 1611,
    3, 1613, 3, 1615, 3, 1617, 3, 1619, 3, 1621, 3, 1623, 3, 864, 3, 862,
    3, 1625, 3, 1627, 3, 1629, 3, 1631, 3, 1633, 3, 1635, 3, 1637, 3, 1639,
    3, 1641, 3, 1643, 1645, 3, 3, 1647, 3, 1649, 3, 1651, 3, 1653, 3, 1655,
    3, 1657, 3, 1509, 3, 545, 3, 1537, 3, 1660, 3, 1662, 3, 302, 243, 3,
    3, 4, 232, 3, 78, 3, 1031, 3, 204, 3, 78, 241, 3, 1298, 3, 1706,
    3, 645, 78, 3, 290, 78, 3, 1708, 3, 645, 78, 3, 290, 78, 3, 78,
    3, 78, 241, 3, 290, 78, 3, 78, 3, 290, 78, 3, 78, 3, 78, 3,
    78, 3, 78, 3, 78, 3, 78, 3, 78, 3, 78, 3, 78, 3, 78, 241,
    3, 634, 78, 3, 78, 3, 78, 241, 3, 78, 3, 280, 3, 280, 3, 280,
    3, 78, 3, 290, 78, 3, 78, 241, 3, 78, 241, 3, 78, 3, 1722, 3,
    78, 3, 290, 78, 3, 80, 3, 80, 3, 363, 3, 733, 3, 326, 3, 78,
    241, 1549, 3, 1178, 3, 3, 829, 3, 831, 3, 1501, 3, 784, 3, 787, 3,
    1503, 3, 1505, 3, 1509, 3, 914, 3, 1507, 3, 984, 3, 708, 3, 986, 3,
    220, 3, 978, 3, 984, 3, 78, 241, 3, 1790, 3, 78, 241, 3, 1790, 3,
    1247, 3, 787, 3, 634, 78, 3, 326, 3, 326, 3, 326, 3, 326, 3, 978,
    3, 4, 1823, 3, 78, 85, 3, 1827, 3, 220, 3, 1829, 3, 1833, 3, 1499,
    3, 1833, 3, 4, 5,
];

pub static REGEX_PATTERNS: &[&str] = &[
    r#"[A-Z_][A-Z0-9_]*"#, // [0]
    r#"^(NOT)$"#, // [1]
    r#"[A-Z0-9_]*[A-Z][A-Z0-9_]*"#, // [2]
    r#"^(CASE|CASESPECIFIC|CROSS|CS|FULL|IGNORE|INNER|INTERVAL|JOIN|LEFT|LOCKING|NATURAL|NOT|NULL|ON|ORDER|OUTER|PARTITION|REPLACE|RESPECT|RIGHT|ROWS|SELECT|SET|TIMESTAMP|UC|UNION|UPPERCASE|USING)$"#, // [3]
    r#"\"?[A-Z][A-Z0-9_]*\"?"#, // [4]
];

pub static HINT_STRING_INDICES: &[u32] = &[
    0, 6, 7, 8, 10, 11, 12, 13, 14, 15,
    16, 17, 18, 19, 20, 21, 22, 23, 24, 25,
    26, 27, 28, 29, 30, 31, 32, 33, 34, 35,
    36, 37, 25, 40, 37, 13, 14, 20, 21, 28,
    12, 26, 33, 35, 17, 52, 53, 53, 52, 57,
    11, 60, 62, 19, 15, 16, 18, 22, 23, 24,
    27, 28, 29, 30, 31, 32, 34, 36, 15, 16,
    67, 22, 23, 68, 69, 60, 70, 71, 72, 73,
    74, 75, 13, 28, 82, 87, 89, 91, 92, 95,
    97, 11, 98, 99, 100, 101, 102, 103, 104, 105,
    106, 40, 107, 108, 109, 110, 111, 112, 113, 114,
    115, 116, 117, 118, 119, 120, 121, 100, 13, 20,
    26, 123, 37, 123, 98, 22, 128, 102, 34, 105,
    133, 134, 118, 134, 118, 11, 98, 99, 100, 101,
    102, 103, 104, 106, 40, 107, 108, 109, 110, 111,
    112, 113, 114, 115, 116, 117, 118, 119, 120, 121,
    15, 16, 18, 22, 23, 137, 24, 27, 28, 29,
    30, 31, 32, 34, 36, 137, 119, 12, 28, 103,
    142, 99, 101, 104, 106, 107, 108, 109, 111, 110,
    112, 113, 114, 115, 116, 117, 120, 121, 164, 32,
    36, 30, 18, 24, 27, 29, 31, 23, 15, 16,
    177, 73, 75, 71, 67, 68, 69, 70, 72, 74,
    189, 190, 189, 190, 195, 201, 206, 207, 210, 211,
    212, 213, 214, 215, 216, 217, 219, 225, 233, 236,
    239, 244, 245, 246, 247, 248, 249, 250, 251, 252,
    253, 254, 255, 219, 249, 258, 251, 252, 250, 252,
    247, 253, 247, 253, 248, 254, 248, 254, 255, 219,
    271, 274, 276, 279, 214, 106, 82, 216, 286, 216,
    286, 286, 292, 225, 89, 296, 297, 299, 300, 301,
    304, 305, 306, 297, 307, 308, 309, 310, 311, 312,
    313, 314, 308, 309, 310, 307, 306, 314, 313, 311,
    312, 328, 330, 332, 334, 335, 336, 337, 338, 339,
    340, 341, 342, 343, 344, 345, 347, 348, 349, 350,
    351, 352, 353, 354, 355, 360, 367, 368, 369, 370,
    371, 372, 373, 306, 297, 307, 308, 309, 310, 311,
    279, 312, 301, 374, 375, 376, 377, 378, 379, 380,
    381, 382, 383, 313, 314, 384, 301, 382, 373, 311,
    279, 312, 374, 375, 376, 377, 378, 379, 380, 381,
    383, 313, 384, 389, 390, 391, 392, 397, 398, 382,
    401, 402, 403, 403, 402, 406, 82, 410, 112, 411,
    412, 414, 415, 416, 418, 418, 212, 13, 420, 421,
    422, 276, 423, 424, 425, 426, 427, 428, 429, 212,
    13, 420, 421, 422, 276, 423, 424, 425, 426, 427,
    428, 429, 429, 426, 420, 424, 422, 425, 423, 421,
    428, 427, 373, 418, 311, 279, 312, 212, 13, 374,
    420, 421, 16, 375, 422, 376, 276, 423, 377, 424,
    378, 425, 426, 379, 380, 381, 383, 427, 428, 429,
    313, 384, 447, 82, 451, 452, 410, 112, 411, 412,
    453, 454, 455, 456, 457, 458, 459, 463, 467, 469,
    473, 474, 475, 477, 478, 479, 480, 481, 473, 482,
    483, 381, 484, 485, 482, 488, 473, 483, 381, 381,
    473, 483, 483, 484, 485, 485, 484, 494, 495, 496,
    497, 498, 499, 500, 501, 502, 503, 504, 505, 506,
    507, 508, 509, 510, 511, 512, 513, 515, 516, 519,
    520, 521, 522, 528, 381, 530, 530, 533, 536, 540,
    544, 553, 555, 557, 559, 560, 564, 565, 374, 376,
    379, 375, 378, 373, 311, 380, 384, 383, 377, 583,
    585, 588, 589, 590, 592, 594, 595, 596, 597, 600,
    601, 602, 603, 604, 605, 606, 607, 608, 609, 610,
    611, 612, 613, 614, 615, 616, 617, 453, 621, 606,
    18, 622, 623, 24, 27, 624, 606, 622, 623, 624,
    606, 622, 624, 622, 624, 623, 629, 113, 114, 637,
    643, 649, 650, 247, 248, 249, 250, 251, 252, 253,
    254, 255, 219, 654, 654, 658, 659, 488, 660, 659,
    488, 659, 658, 660, 658, 660, 82, 225, 666, 528,
    17, 536, 540, 666, 301, 643, 252, 164, 670, 536,
    540, 453, 216, 367, 675, 675, 679, 101, 104, 117,
    681, 629, 91, 682, 381, 683, 463, 683, 686, 688,
    690, 681, 682, 381, 681, 681, 682, 682, 694, 696,
    698, 688, 699, 699, 5, 100, 705, 705, 709, 710,
    711, 712, 713, 713, 715, 715, 720, 721, 352, 722,
    723, 724, 353, 354, 725, 726, 727, 728, 729, 355,
    734, 737, 738, 739, 740, 741, 742, 743, 744, 745,
    746, 747, 355, 360, 749, 750, 749, 750, 758, 760,
    761, 737, 762, 763, 764, 765, 766, 767, 738, 768,
    769, 770, 771, 772, 773, 412, 775, 776, 777, 778,
    779, 780, 781, 779, 101, 786, 789, 790, 791, 792,
    793, 794, 795, 796, 797, 798, 800, 801, 802, 803,
    804, 805, 806, 807, 808, 809, 810, 811, 812, 813,
    815, 457, 816, 816, 820, 821, 822, 823, 824, 825,
    826, 827, 832, 833, 834, 835, 836, 837, 839, 840,
    842, 843, 844, 845, 846, 847, 848, 849, 850, 851,
    852, 853, 854, 855, 856, 857, 82, 779, 101, 104,
    451, 452, 410, 112, 117, 411, 412, 453, 869, 666,
    381, 870, 62, 381, 370, 381, 308, 309, 381, 881,
    882, 308, 309, 882, 881, 373, 306, 297, 307, 308,
    309, 310, 311, 279, 312, 374, 375, 376, 377, 378,
    379, 380, 381, 383, 313, 314, 384, 308, 309, 882,
    355, 402, 360, 530, 749, 750, 403, 97, 236, 451,
    452, 410, 112, 915, 916, 917, 920, 286, 922, 922,
    688, 924, 924, 926, 453, 926, 929, 930, 82, 418,
    789, 246, 368, 522, 544, 564, 100, 13, 779, 101,
    798, 271, 469, 857, 189, 104, 451, 452, 931, 932,
    292, 705, 190, 933, 410, 112, 488, 934, 654, 116,
    117, 119, 411, 412, 453, 82, 418, 246, 368, 522,
    544, 564, 100, 13, 779, 101, 798, 271, 469, 857,
    189, 104, 451, 452, 931, 932, 292, 705, 190, 933,
    410, 112, 488, 934, 654, 116, 117, 119, 411, 412,
    453, 938, 939, 942, 943, 944, 945, 946, 947, 948,
    949, 951, 952, 953, 954, 955, 924, 959, 948, 960,
    960, 955, 964, 965, 966, 967, 846, 920, 195, 970,
    971, 972, 973, 974, 219, 932, 975, 786, 976, 453,
    972, 970, 932, 976, 846, 971, 973, 975, 974, 995,
    1002, 594, 1007, 406, 1008, 1009, 1015, 1018, 1019, 1022,
    5, 1022, 411, 412, 1033, 1034, 1035, 1036, 1037, 1038,
    1039, 612, 453, 1048, 1051, 1055, 1056, 1058, 920, 970,
    972, 219, 932, 976, 1065, 1066, 1067, 920, 972, 219,
    932, 976, 1068, 1069, 1070, 1071, 1072, 1073, 1074, 1075,
    739, 1076, 1077, 1078, 1079, 1082, 1083, 1084, 1018, 1019,
    1088, 869, 1089, 1090, 1091, 1092, 1093, 334, 797, 1096,
    1097, 1098, 1099, 1100, 1101, 1102, 595, 1103, 1104, 1105,
    1107, 1109, 82, 533, 451, 452, 410, 112, 411, 412,
    453, 1111, 1112, 1113, 1114, 1115, 1116, 1117, 1118, 1119,
    1120, 971, 1121, 97, 236, 737, 738, 739, 740, 741,
    742, 743, 744, 745, 746, 747, 236, 1122, 1124, 1127,
    1128, 1129, 1130, 1133, 164, 1138, 1138, 594, 596, 597,
    596, 597, 1151, 1153, 1154, 1155, 1156, 1157, 1158, 1159,
    1160, 1161, 1162, 1163, 1008, 1164, 1165, 1166, 1167, 1168,
    869, 666, 870, 870, 87, 974, 1174, 1175, 1176, 1182,
    1184, 1185, 1186, 1187, 1188, 1189, 1190, 451, 451, 452,
    452, 13, 688, 32, 36, 629, 62, 8, 505, 848,
    851, 1185, 109, 201, 121, 1196, 1201, 1203, 1205, 1206,
    1207, 1208, 1209, 1210, 1211, 1212, 1213, 1214, 1216, 1217,
    1218, 1219, 1220, 1221, 1222, 1223, 250, 1224, 1225, 1226,
    1227, 1228, 1229, 931, 101, 117, 1248, 1249, 1250, 1251,
    1252, 740, 1253, 1254, 973, 741, 1255, 1256, 251, 1257,
    1258, 1259, 1260, 742, 1261, 1262, 1263, 1264, 1265, 1267,
    1272, 1274, 1275, 1277, 1278, 1280, 252, 1283, 1284, 1285,
    1286, 1287, 1288, 1289, 1290, 1291, 410, 112, 451, 452,
    453, 779, 101, 104, 931, 117, 82, 451, 452, 410,
    112, 411, 412, 82, 451, 452, 410, 112, 1300, 1301,
    255, 1302, 1303, 1304, 373, 1306, 1307, 1308, 1309, 1310,
    1311, 1312, 1320, 1321, 1322, 1323, 1324, 1325, 1326, 1327,
    1328, 1329, 1330, 1331, 1332, 1333, 1334, 1335, 1336, 286,
    1161, 920, 970, 972, 932, 955, 924, 1015, 976, 1341,
    1342, 1343, 1344, 1345, 1346, 1348, 1350, 1351, 1352, 1353,
    1354, 1355, 1357, 1358, 1359, 1360, 1361, 1362, 1363, 1364,
    1365, 670, 1367, 1368, 1369, 310, 236, 1371, 1372, 1373,
    1374, 1375, 1377, 1378, 1379, 1380, 1384, 1385, 1386, 1387,
    1388, 1389, 1390, 1391, 1393, 1394, 1395, 1396, 1397, 1398,
    1399, 1400, 11, 304, 1401, 1401, 743, 1403, 1404, 1407,
    1408, 1409, 1411, 1412, 1413, 1414, 1415, 1416, 1417, 1418,
    457, 252, 816, 488, 594, 1365, 1422, 1422, 1427, 1428,
    1429, 1430, 1431, 1432, 1433, 1434, 1435, 1436, 1437, 1438,
    1439, 1440, 1441, 1442, 1443, 1444, 1445, 1446, 1447, 1448,
    1449, 1450, 1451, 1452, 1453, 1454, 1456, 933, 1457, 1458,
    1459, 1460, 1461, 1462, 1463, 1464, 1465, 1466, 1467, 369,
    1469, 1469, 1473, 1474, 1475, 1476, 1477, 1478, 1479, 1480,
    1481, 1482, 744, 1483, 1484, 1485, 410, 11, 583, 1302,
    1488, 1488, 846, 920, 91, 971, 972, 973, 219, 383,
    975, 786, 82, 451, 452, 453, 82, 453, 1516, 1518,
    1521, 1522, 1523, 975, 846, 971, 1301, 236, 1523, 1532,
    1532, 488, 934, 934, 1538, 1539, 1540, 1541, 1542, 1543,
    1546, 1547, 1552, 1553, 1554, 1555, 1556, 1557, 1558, 1559,
    1560, 1561, 1562, 1563, 1564, 1565, 1566, 1567, 1568, 1569,
    1570, 1571, 1572, 1573, 1574, 1575, 1576, 1577, 1578, 1580,
    1581, 1582, 1583, 368, 564, 469, 933, 654, 1663, 1664,
    1665, 1666, 1667, 1668, 1669, 1670, 1671, 1673, 1674, 1675,
    1676, 1677, 1678, 1679, 1680, 1681, 1682, 1683, 1684, 1685,
    1686, 1687, 1688, 17, 536, 540, 1690, 1223, 1224, 252,
    1467, 1469, 1694, 1694, 768, 1372, 1582, 1663, 1664, 274,
    643, 540, 643, 540, 274, 13, 16, 18, 1216, 1217,
    623, 24, 1398, 26, 32, 115, 35, 123, 36, 225,
    377, 236, 225, 377, 216, 367, 505, 533, 820, 917,
    1153, 1186, 1217, 1248, 252, 381, 917, 252, 216, 367,
    820, 1153, 1186, 252, 381, 820, 1186, 252, 381, 216,
    367, 533, 1323, 164, 533, 252, 1067, 412, 252, 536,
    540, 536, 540, 101, 1391, 1725, 113, 114, 1726, 1726,
    1728, 1729, 1730, 1731, 1732, 749, 750, 453, 1734, 1734,
    1737, 1739, 1740, 1741, 1742, 1743, 1744, 1745, 1009, 1746,
    1532, 1749, 1749, 1753, 1754, 1755, 1756, 1757, 1758, 1759,
    1760, 1761, 1762, 1763, 1764, 1765, 1769, 1770, 1772, 1773,
    1774, 1778, 1779, 1780, 11, 583, 1782, 1783, 1784, 1785,
    1786, 1787, 1788, 1791, 1792, 1793, 1794, 1795, 1796, 1797,
    1798, 1799, 1800, 1801, 1802, 1803, 1804, 411, 82, 688,
    1806, 1807, 1808, 1809, 1810, 1811, 1812, 1813, 1814, 1815,
    1816, 1817, 1818, 745, 746, 1821, 920, 195, 970, 972,
    219, 383, 932, 976, 1822, 1825, 219, 670, 1836, 1837,
    1838, 1839, 1840, 1841, 747, 1842, 1843,
];

pub static SIMPLE_HINTS: &[SimpleHintData] = &[
    SimpleHintData::empty(), // [0] reserved for no hint
    SimpleHintData { raw_values_start: 0, raw_values_count: 1, token_types_start: 1, token_types_count: 0 }, // [1]
    SimpleHintData { raw_values_start: 1, raw_values_count: 1, token_types_start: 2, token_types_count: 0 }, // [2]
    SimpleHintData { raw_values_start: 2, raw_values_count: 1, token_types_start: 3, token_types_count: 0 }, // [3]
    SimpleHintData { raw_values_start: 3, raw_values_count: 1, token_types_start: 4, token_types_count: 0 }, // [4]
    SimpleHintData { raw_values_start: 4, raw_values_count: 1, token_types_start: 5, token_types_count: 0 }, // [5]
    SimpleHintData { raw_values_start: 5, raw_values_count: 27, token_types_start: 32, token_types_count: 0 }, // [6]
    SimpleHintData { raw_values_start: 32, raw_values_count: 1, token_types_start: 33, token_types_count: 0 }, // [7]
    SimpleHintData { raw_values_start: 33, raw_values_count: 1, token_types_start: 34, token_types_count: 0 }, // [8]
    SimpleHintData { raw_values_start: 34, raw_values_count: 1, token_types_start: 35, token_types_count: 0 }, // [9]
    SimpleHintData { raw_values_start: 35, raw_values_count: 1, token_types_start: 36, token_types_count: 0 }, // [10]
    SimpleHintData { raw_values_start: 36, raw_values_count: 1, token_types_start: 37, token_types_count: 0 }, // [11]
    SimpleHintData { raw_values_start: 37, raw_values_count: 1, token_types_start: 38, token_types_count: 0 }, // [12]
    SimpleHintData { raw_values_start: 38, raw_values_count: 1, token_types_start: 39, token_types_count: 0 }, // [13]
    SimpleHintData { raw_values_start: 39, raw_values_count: 1, token_types_start: 40, token_types_count: 0 }, // [14]
    SimpleHintData { raw_values_start: 40, raw_values_count: 1, token_types_start: 41, token_types_count: 0 }, // [15]
    SimpleHintData { raw_values_start: 41, raw_values_count: 1, token_types_start: 42, token_types_count: 0 }, // [16]
    SimpleHintData { raw_values_start: 42, raw_values_count: 1, token_types_start: 43, token_types_count: 0 }, // [17]
    SimpleHintData { raw_values_start: 43, raw_values_count: 1, token_types_start: 44, token_types_count: 0 }, // [18]
    SimpleHintData { raw_values_start: 44, raw_values_count: 1, token_types_start: 45, token_types_count: 0 }, // [19]
    SimpleHintData { raw_values_start: 45, raw_values_count: 2, token_types_start: 47, token_types_count: 0 }, // [20]
    SimpleHintData { raw_values_start: 47, raw_values_count: 1, token_types_start: 48, token_types_count: 0 }, // [21]
    SimpleHintData { raw_values_start: 48, raw_values_count: 1, token_types_start: 49, token_types_count: 0 }, // [22]
    SimpleHintData { raw_values_start: 49, raw_values_count: 1, token_types_start: 50, token_types_count: 0 }, // [23]
    SimpleHintData { raw_values_start: 50, raw_values_count: 1, token_types_start: 51, token_types_count: 0 }, // [24]
    SimpleHintData { raw_values_start: 51, raw_values_count: 1, token_types_start: 52, token_types_count: 0 }, // [25]
    SimpleHintData { raw_values_start: 52, raw_values_count: 1, token_types_start: 53, token_types_count: 0 }, // [26]
    SimpleHintData { raw_values_start: 53, raw_values_count: 1, token_types_start: 54, token_types_count: 0 }, // [27]
    SimpleHintData { raw_values_start: 54, raw_values_count: 14, token_types_start: 68, token_types_count: 0 }, // [28]
    SimpleHintData { raw_values_start: 68, raw_values_count: 14, token_types_start: 82, token_types_count: 0 }, // [29]
    SimpleHintData { raw_values_start: 82, raw_values_count: 2, token_types_start: 84, token_types_count: 0 }, // [30]
    SimpleHintData { raw_values_start: 84, raw_values_count: 1, token_types_start: 85, token_types_count: 0 }, // [31]
    SimpleHintData { raw_values_start: 85, raw_values_count: 1, token_types_start: 86, token_types_count: 0 }, // [32]
    SimpleHintData { raw_values_start: 86, raw_values_count: 1, token_types_start: 87, token_types_count: 0 }, // [33]
    SimpleHintData { raw_values_start: 87, raw_values_count: 1, token_types_start: 88, token_types_count: 0 }, // [34]
    SimpleHintData { raw_values_start: 88, raw_values_count: 1, token_types_start: 89, token_types_count: 0 }, // [35]
    SimpleHintData { raw_values_start: 89, raw_values_count: 1, token_types_start: 90, token_types_count: 0 }, // [36]
    SimpleHintData { raw_values_start: 90, raw_values_count: 0, token_types_start: 90, token_types_count: 1 }, // [37]
    SimpleHintData { raw_values_start: 91, raw_values_count: 26, token_types_start: 117, token_types_count: 0 }, // [38]
    SimpleHintData { raw_values_start: 117, raw_values_count: 1, token_types_start: 118, token_types_count: 0 }, // [39]
    SimpleHintData { raw_values_start: 118, raw_values_count: 5, token_types_start: 123, token_types_count: 0 }, // [40]
    SimpleHintData { raw_values_start: 123, raw_values_count: 1, token_types_start: 124, token_types_count: 0 }, // [41]
    SimpleHintData { raw_values_start: 124, raw_values_count: 1, token_types_start: 125, token_types_count: 0 }, // [42]
    SimpleHintData { raw_values_start: 125, raw_values_count: 1, token_types_start: 126, token_types_count: 0 }, // [43]
    SimpleHintData { raw_values_start: 126, raw_values_count: 1, token_types_start: 127, token_types_count: 0 }, // [44]
    SimpleHintData { raw_values_start: 127, raw_values_count: 1, token_types_start: 128, token_types_count: 0 }, // [45]
    SimpleHintData { raw_values_start: 128, raw_values_count: 1, token_types_start: 129, token_types_count: 0 }, // [46]
    SimpleHintData { raw_values_start: 129, raw_values_count: 1, token_types_start: 130, token_types_count: 0 }, // [47]
    SimpleHintData { raw_values_start: 130, raw_values_count: 1, token_types_start: 131, token_types_count: 0 }, // [48]
    SimpleHintData { raw_values_start: 131, raw_values_count: 2, token_types_start: 133, token_types_count: 0 }, // [49]
    SimpleHintData { raw_values_start: 133, raw_values_count: 1, token_types_start: 134, token_types_count: 0 }, // [50]
    SimpleHintData { raw_values_start: 134, raw_values_count: 1, token_types_start: 135, token_types_count: 0 }, // [51]
    SimpleHintData { raw_values_start: 135, raw_values_count: 25, token_types_start: 160, token_types_count: 0 }, // [52]
    SimpleHintData { raw_values_start: 160, raw_values_count: 15, token_types_start: 175, token_types_count: 0 }, // [53]
    SimpleHintData { raw_values_start: 175, raw_values_count: 1, token_types_start: 176, token_types_count: 0 }, // [54]
    SimpleHintData { raw_values_start: 176, raw_values_count: 1, token_types_start: 177, token_types_count: 0 }, // [55]
    SimpleHintData { raw_values_start: 177, raw_values_count: 2, token_types_start: 179, token_types_count: 0 }, // [56]
    SimpleHintData { raw_values_start: 179, raw_values_count: 1, token_types_start: 180, token_types_count: 0 }, // [57]
    SimpleHintData { raw_values_start: 180, raw_values_count: 1, token_types_start: 181, token_types_count: 0 }, // [58]
    SimpleHintData { raw_values_start: 181, raw_values_count: 1, token_types_start: 182, token_types_count: 0 }, // [59]
    SimpleHintData { raw_values_start: 182, raw_values_count: 1, token_types_start: 183, token_types_count: 0 }, // [60]
    SimpleHintData { raw_values_start: 183, raw_values_count: 1, token_types_start: 184, token_types_count: 0 }, // [61]
    SimpleHintData { raw_values_start: 184, raw_values_count: 1, token_types_start: 185, token_types_count: 0 }, // [62]
    SimpleHintData { raw_values_start: 185, raw_values_count: 1, token_types_start: 186, token_types_count: 0 }, // [63]
    SimpleHintData { raw_values_start: 186, raw_values_count: 1, token_types_start: 187, token_types_count: 0 }, // [64]
    SimpleHintData { raw_values_start: 187, raw_values_count: 1, token_types_start: 188, token_types_count: 0 }, // [65]
    SimpleHintData { raw_values_start: 188, raw_values_count: 1, token_types_start: 189, token_types_count: 0 }, // [66]
    SimpleHintData { raw_values_start: 189, raw_values_count: 1, token_types_start: 190, token_types_count: 0 }, // [67]
    SimpleHintData { raw_values_start: 190, raw_values_count: 1, token_types_start: 191, token_types_count: 0 }, // [68]
    SimpleHintData { raw_values_start: 191, raw_values_count: 1, token_types_start: 192, token_types_count: 0 }, // [69]
    SimpleHintData { raw_values_start: 192, raw_values_count: 1, token_types_start: 193, token_types_count: 0 }, // [70]
    SimpleHintData { raw_values_start: 193, raw_values_count: 1, token_types_start: 194, token_types_count: 0 }, // [71]
    SimpleHintData { raw_values_start: 194, raw_values_count: 1, token_types_start: 195, token_types_count: 0 }, // [72]
    SimpleHintData { raw_values_start: 195, raw_values_count: 1, token_types_start: 196, token_types_count: 0 }, // [73]
    SimpleHintData { raw_values_start: 196, raw_values_count: 1, token_types_start: 197, token_types_count: 0 }, // [74]
    SimpleHintData { raw_values_start: 197, raw_values_count: 1, token_types_start: 198, token_types_count: 0 }, // [75]
    SimpleHintData { raw_values_start: 198, raw_values_count: 1, token_types_start: 199, token_types_count: 0 }, // [76]
    SimpleHintData { raw_values_start: 199, raw_values_count: 1, token_types_start: 200, token_types_count: 0 }, // [77]
    SimpleHintData { raw_values_start: 200, raw_values_count: 1, token_types_start: 201, token_types_count: 0 }, // [78]
    SimpleHintData { raw_values_start: 201, raw_values_count: 1, token_types_start: 202, token_types_count: 0 }, // [79]
    SimpleHintData { raw_values_start: 202, raw_values_count: 1, token_types_start: 203, token_types_count: 0 }, // [80]
    SimpleHintData { raw_values_start: 203, raw_values_count: 1, token_types_start: 204, token_types_count: 0 }, // [81]
    SimpleHintData { raw_values_start: 204, raw_values_count: 1, token_types_start: 205, token_types_count: 0 }, // [82]
    SimpleHintData { raw_values_start: 205, raw_values_count: 1, token_types_start: 206, token_types_count: 0 }, // [83]
    SimpleHintData { raw_values_start: 206, raw_values_count: 1, token_types_start: 207, token_types_count: 0 }, // [84]
    SimpleHintData { raw_values_start: 207, raw_values_count: 1, token_types_start: 208, token_types_count: 0 }, // [85]
    SimpleHintData { raw_values_start: 208, raw_values_count: 1, token_types_start: 209, token_types_count: 0 }, // [86]
    SimpleHintData { raw_values_start: 209, raw_values_count: 1, token_types_start: 210, token_types_count: 0 }, // [87]
    SimpleHintData { raw_values_start: 210, raw_values_count: 1, token_types_start: 211, token_types_count: 0 }, // [88]
    SimpleHintData { raw_values_start: 211, raw_values_count: 1, token_types_start: 212, token_types_count: 0 }, // [89]
    SimpleHintData { raw_values_start: 212, raw_values_count: 1, token_types_start: 213, token_types_count: 0 }, // [90]
    SimpleHintData { raw_values_start: 213, raw_values_count: 1, token_types_start: 214, token_types_count: 0 }, // [91]
    SimpleHintData { raw_values_start: 214, raw_values_count: 1, token_types_start: 215, token_types_count: 0 }, // [92]
    SimpleHintData { raw_values_start: 215, raw_values_count: 1, token_types_start: 216, token_types_count: 0 }, // [93]
    SimpleHintData { raw_values_start: 216, raw_values_count: 1, token_types_start: 217, token_types_count: 0 }, // [94]
    SimpleHintData { raw_values_start: 217, raw_values_count: 1, token_types_start: 218, token_types_count: 0 }, // [95]
    SimpleHintData { raw_values_start: 218, raw_values_count: 1, token_types_start: 219, token_types_count: 0 }, // [96]
    SimpleHintData { raw_values_start: 219, raw_values_count: 1, token_types_start: 220, token_types_count: 0 }, // [97]
    SimpleHintData { raw_values_start: 220, raw_values_count: 2, token_types_start: 222, token_types_count: 0 }, // [98]
    SimpleHintData { raw_values_start: 222, raw_values_count: 1, token_types_start: 223, token_types_count: 0 }, // [99]
    SimpleHintData { raw_values_start: 223, raw_values_count: 1, token_types_start: 224, token_types_count: 0 }, // [100]
    SimpleHintData { raw_values_start: 224, raw_values_count: 1, token_types_start: 225, token_types_count: 0 }, // [101]
    SimpleHintData { raw_values_start: 225, raw_values_count: 1, token_types_start: 226, token_types_count: 0 }, // [102]
    SimpleHintData { raw_values_start: 226, raw_values_count: 1, token_types_start: 227, token_types_count: 0 }, // [103]
    SimpleHintData { raw_values_start: 227, raw_values_count: 1, token_types_start: 228, token_types_count: 0 }, // [104]
    SimpleHintData { raw_values_start: 228, raw_values_count: 1, token_types_start: 229, token_types_count: 0 }, // [105]
    SimpleHintData { raw_values_start: 229, raw_values_count: 1, token_types_start: 230, token_types_count: 0 }, // [106]
    SimpleHintData { raw_values_start: 230, raw_values_count: 1, token_types_start: 231, token_types_count: 0 }, // [107]
    SimpleHintData { raw_values_start: 231, raw_values_count: 1, token_types_start: 232, token_types_count: 0 }, // [108]
    SimpleHintData { raw_values_start: 232, raw_values_count: 1, token_types_start: 233, token_types_count: 0 }, // [109]
    SimpleHintData { raw_values_start: 233, raw_values_count: 1, token_types_start: 234, token_types_count: 0 }, // [110]
    SimpleHintData { raw_values_start: 234, raw_values_count: 1, token_types_start: 235, token_types_count: 0 }, // [111]
    SimpleHintData { raw_values_start: 235, raw_values_count: 1, token_types_start: 236, token_types_count: 0 }, // [112]
    SimpleHintData { raw_values_start: 236, raw_values_count: 1, token_types_start: 237, token_types_count: 0 }, // [113]
    SimpleHintData { raw_values_start: 237, raw_values_count: 1, token_types_start: 238, token_types_count: 0 }, // [114]
    SimpleHintData { raw_values_start: 238, raw_values_count: 1, token_types_start: 239, token_types_count: 0 }, // [115]
    SimpleHintData { raw_values_start: 239, raw_values_count: 0, token_types_start: 239, token_types_count: 1 }, // [116]
    SimpleHintData { raw_values_start: 240, raw_values_count: 1, token_types_start: 241, token_types_count: 0 }, // [117]
    SimpleHintData { raw_values_start: 241, raw_values_count: 1, token_types_start: 242, token_types_count: 0 }, // [118]
    SimpleHintData { raw_values_start: 242, raw_values_count: 1, token_types_start: 243, token_types_count: 0 }, // [119]
    SimpleHintData { raw_values_start: 243, raw_values_count: 1, token_types_start: 244, token_types_count: 0 }, // [120]
    SimpleHintData { raw_values_start: 244, raw_values_count: 10, token_types_start: 254, token_types_count: 0 }, // [121]
    SimpleHintData { raw_values_start: 254, raw_values_count: 1, token_types_start: 255, token_types_count: 0 }, // [122]
    SimpleHintData { raw_values_start: 255, raw_values_count: 1, token_types_start: 256, token_types_count: 0 }, // [123]
    SimpleHintData { raw_values_start: 256, raw_values_count: 2, token_types_start: 258, token_types_count: 0 }, // [124]
    SimpleHintData { raw_values_start: 258, raw_values_count: 2, token_types_start: 260, token_types_count: 0 }, // [125]
    SimpleHintData { raw_values_start: 260, raw_values_count: 2, token_types_start: 262, token_types_count: 0 }, // [126]
    SimpleHintData { raw_values_start: 262, raw_values_count: 1, token_types_start: 263, token_types_count: 0 }, // [127]
    SimpleHintData { raw_values_start: 263, raw_values_count: 1, token_types_start: 264, token_types_count: 0 }, // [128]
    SimpleHintData { raw_values_start: 264, raw_values_count: 2, token_types_start: 266, token_types_count: 0 }, // [129]
    SimpleHintData { raw_values_start: 266, raw_values_count: 1, token_types_start: 267, token_types_count: 0 }, // [130]
    SimpleHintData { raw_values_start: 267, raw_values_count: 1, token_types_start: 268, token_types_count: 0 }, // [131]
    SimpleHintData { raw_values_start: 268, raw_values_count: 2, token_types_start: 270, token_types_count: 0 }, // [132]
    SimpleHintData { raw_values_start: 270, raw_values_count: 1, token_types_start: 271, token_types_count: 0 }, // [133]
    SimpleHintData { raw_values_start: 271, raw_values_count: 1, token_types_start: 272, token_types_count: 0 }, // [134]
    SimpleHintData { raw_values_start: 272, raw_values_count: 1, token_types_start: 273, token_types_count: 0 }, // [135]
    SimpleHintData { raw_values_start: 273, raw_values_count: 1, token_types_start: 274, token_types_count: 0 }, // [136]
    SimpleHintData { raw_values_start: 274, raw_values_count: 2, token_types_start: 276, token_types_count: 0 }, // [137]
    SimpleHintData { raw_values_start: 276, raw_values_count: 3, token_types_start: 279, token_types_count: 0 }, // [138]
    SimpleHintData { raw_values_start: 279, raw_values_count: 2, token_types_start: 281, token_types_count: 0 }, // [139]
    SimpleHintData { raw_values_start: 281, raw_values_count: 1, token_types_start: 282, token_types_count: 0 }, // [140]
    SimpleHintData { raw_values_start: 282, raw_values_count: 1, token_types_start: 283, token_types_count: 0 }, // [141]
    SimpleHintData { raw_values_start: 283, raw_values_count: 2, token_types_start: 285, token_types_count: 0 }, // [142]
    SimpleHintData { raw_values_start: 285, raw_values_count: 1, token_types_start: 286, token_types_count: 0 }, // [143]
    SimpleHintData { raw_values_start: 286, raw_values_count: 1, token_types_start: 287, token_types_count: 0 }, // [144]
    SimpleHintData { raw_values_start: 287, raw_values_count: 1, token_types_start: 288, token_types_count: 0 }, // [145]
    SimpleHintData { raw_values_start: 288, raw_values_count: 1, token_types_start: 289, token_types_count: 0 }, // [146]
    SimpleHintData { raw_values_start: 289, raw_values_count: 1, token_types_start: 290, token_types_count: 0 }, // [147]
    SimpleHintData { raw_values_start: 290, raw_values_count: 1, token_types_start: 291, token_types_count: 0 }, // [148]
    SimpleHintData { raw_values_start: 291, raw_values_count: 1, token_types_start: 292, token_types_count: 0 }, // [149]
    SimpleHintData { raw_values_start: 292, raw_values_count: 10, token_types_start: 302, token_types_count: 0 }, // [150]
    SimpleHintData { raw_values_start: 302, raw_values_count: 1, token_types_start: 303, token_types_count: 0 }, // [151]
    SimpleHintData { raw_values_start: 303, raw_values_count: 1, token_types_start: 304, token_types_count: 0 }, // [152]
    SimpleHintData { raw_values_start: 304, raw_values_count: 1, token_types_start: 305, token_types_count: 0 }, // [153]
    SimpleHintData { raw_values_start: 305, raw_values_count: 1, token_types_start: 306, token_types_count: 0 }, // [154]
    SimpleHintData { raw_values_start: 306, raw_values_count: 1, token_types_start: 307, token_types_count: 0 }, // [155]
    SimpleHintData { raw_values_start: 307, raw_values_count: 1, token_types_start: 308, token_types_count: 0 }, // [156]
    SimpleHintData { raw_values_start: 308, raw_values_count: 1, token_types_start: 309, token_types_count: 0 }, // [157]
    SimpleHintData { raw_values_start: 309, raw_values_count: 1, token_types_start: 310, token_types_count: 0 }, // [158]
    SimpleHintData { raw_values_start: 310, raw_values_count: 1, token_types_start: 311, token_types_count: 0 }, // [159]
    SimpleHintData { raw_values_start: 311, raw_values_count: 1, token_types_start: 312, token_types_count: 0 }, // [160]
    SimpleHintData { raw_values_start: 312, raw_values_count: 1, token_types_start: 313, token_types_count: 0 }, // [161]
    SimpleHintData { raw_values_start: 313, raw_values_count: 1, token_types_start: 314, token_types_count: 0 }, // [162]
    SimpleHintData { raw_values_start: 314, raw_values_count: 1, token_types_start: 315, token_types_count: 0 }, // [163]
    SimpleHintData { raw_values_start: 315, raw_values_count: 1, token_types_start: 316, token_types_count: 0 }, // [164]
    SimpleHintData { raw_values_start: 316, raw_values_count: 1, token_types_start: 317, token_types_count: 0 }, // [165]
    SimpleHintData { raw_values_start: 317, raw_values_count: 1, token_types_start: 318, token_types_count: 0 }, // [166]
    SimpleHintData { raw_values_start: 318, raw_values_count: 1, token_types_start: 319, token_types_count: 0 }, // [167]
    SimpleHintData { raw_values_start: 319, raw_values_count: 1, token_types_start: 320, token_types_count: 0 }, // [168]
    SimpleHintData { raw_values_start: 320, raw_values_count: 1, token_types_start: 321, token_types_count: 0 }, // [169]
    SimpleHintData { raw_values_start: 321, raw_values_count: 1, token_types_start: 322, token_types_count: 0 }, // [170]
    SimpleHintData { raw_values_start: 322, raw_values_count: 1, token_types_start: 323, token_types_count: 0 }, // [171]
    SimpleHintData { raw_values_start: 323, raw_values_count: 1, token_types_start: 324, token_types_count: 0 }, // [172]
    SimpleHintData { raw_values_start: 324, raw_values_count: 1, token_types_start: 325, token_types_count: 0 }, // [173]
    SimpleHintData { raw_values_start: 325, raw_values_count: 1, token_types_start: 326, token_types_count: 0 }, // [174]
    SimpleHintData { raw_values_start: 326, raw_values_count: 1, token_types_start: 327, token_types_count: 0 }, // [175]
    SimpleHintData { raw_values_start: 327, raw_values_count: 1, token_types_start: 328, token_types_count: 0 }, // [176]
    SimpleHintData { raw_values_start: 328, raw_values_count: 1, token_types_start: 329, token_types_count: 0 }, // [177]
    SimpleHintData { raw_values_start: 329, raw_values_count: 1, token_types_start: 330, token_types_count: 0 }, // [178]
    SimpleHintData { raw_values_start: 330, raw_values_count: 1, token_types_start: 331, token_types_count: 0 }, // [179]
    SimpleHintData { raw_values_start: 331, raw_values_count: 4, token_types_start: 335, token_types_count: 0 }, // [180]
    SimpleHintData { raw_values_start: 335, raw_values_count: 1, token_types_start: 336, token_types_count: 0 }, // [181]
    SimpleHintData { raw_values_start: 336, raw_values_count: 1, token_types_start: 337, token_types_count: 0 }, // [182]
    SimpleHintData { raw_values_start: 337, raw_values_count: 1, token_types_start: 338, token_types_count: 0 }, // [183]
    SimpleHintData { raw_values_start: 338, raw_values_count: 1, token_types_start: 339, token_types_count: 0 }, // [184]
    SimpleHintData { raw_values_start: 339, raw_values_count: 1, token_types_start: 340, token_types_count: 0 }, // [185]
    SimpleHintData { raw_values_start: 340, raw_values_count: 1, token_types_start: 341, token_types_count: 0 }, // [186]
    SimpleHintData { raw_values_start: 341, raw_values_count: 1, token_types_start: 342, token_types_count: 0 }, // [187]
    SimpleHintData { raw_values_start: 342, raw_values_count: 23, token_types_start: 365, token_types_count: 1 }, // [188]
    SimpleHintData { raw_values_start: 366, raw_values_count: 2, token_types_start: 368, token_types_count: 0 }, // [189]
    SimpleHintData { raw_values_start: 368, raw_values_count: 14, token_types_start: 382, token_types_count: 1 }, // [190]
    SimpleHintData { raw_values_start: 383, raw_values_count: 1, token_types_start: 384, token_types_count: 0 }, // [191]
    SimpleHintData { raw_values_start: 384, raw_values_count: 1, token_types_start: 385, token_types_count: 0 }, // [192]
    SimpleHintData { raw_values_start: 385, raw_values_count: 1, token_types_start: 386, token_types_count: 0 }, // [193]
    SimpleHintData { raw_values_start: 386, raw_values_count: 1, token_types_start: 387, token_types_count: 0 }, // [194]
    SimpleHintData { raw_values_start: 387, raw_values_count: 1, token_types_start: 388, token_types_count: 0 }, // [195]
    SimpleHintData { raw_values_start: 388, raw_values_count: 1, token_types_start: 389, token_types_count: 0 }, // [196]
    SimpleHintData { raw_values_start: 389, raw_values_count: 1, token_types_start: 390, token_types_count: 0 }, // [197]
    SimpleHintData { raw_values_start: 390, raw_values_count: 1, token_types_start: 391, token_types_count: 0 }, // [198]
    SimpleHintData { raw_values_start: 391, raw_values_count: 2, token_types_start: 393, token_types_count: 0 }, // [199]
    SimpleHintData { raw_values_start: 393, raw_values_count: 1, token_types_start: 394, token_types_count: 0 }, // [200]
    SimpleHintData { raw_values_start: 394, raw_values_count: 1, token_types_start: 395, token_types_count: 0 }, // [201]
    SimpleHintData { raw_values_start: 395, raw_values_count: 1, token_types_start: 396, token_types_count: 0 }, // [202]
    SimpleHintData { raw_values_start: 396, raw_values_count: 5, token_types_start: 401, token_types_count: 0 }, // [203]
    SimpleHintData { raw_values_start: 401, raw_values_count: 1, token_types_start: 402, token_types_count: 0 }, // [204]
    SimpleHintData { raw_values_start: 402, raw_values_count: 1, token_types_start: 403, token_types_count: 0 }, // [205]
    SimpleHintData { raw_values_start: 403, raw_values_count: 1, token_types_start: 404, token_types_count: 0 }, // [206]
    SimpleHintData { raw_values_start: 404, raw_values_count: 1, token_types_start: 405, token_types_count: 0 }, // [207]
    SimpleHintData { raw_values_start: 405, raw_values_count: 14, token_types_start: 419, token_types_count: 0 }, // [208]
    SimpleHintData { raw_values_start: 419, raw_values_count: 13, token_types_start: 432, token_types_count: 0 }, // [209]
    SimpleHintData { raw_values_start: 432, raw_values_count: 1, token_types_start: 433, token_types_count: 0 }, // [210]
    SimpleHintData { raw_values_start: 433, raw_values_count: 1, token_types_start: 434, token_types_count: 0 }, // [211]
    SimpleHintData { raw_values_start: 434, raw_values_count: 1, token_types_start: 435, token_types_count: 0 }, // [212]
    SimpleHintData { raw_values_start: 435, raw_values_count: 1, token_types_start: 436, token_types_count: 0 }, // [213]
    SimpleHintData { raw_values_start: 436, raw_values_count: 1, token_types_start: 437, token_types_count: 0 }, // [214]
    SimpleHintData { raw_values_start: 437, raw_values_count: 1, token_types_start: 438, token_types_count: 0 }, // [215]
    SimpleHintData { raw_values_start: 438, raw_values_count: 1, token_types_start: 439, token_types_count: 0 }, // [216]
    SimpleHintData { raw_values_start: 439, raw_values_count: 1, token_types_start: 440, token_types_count: 0 }, // [217]
    SimpleHintData { raw_values_start: 440, raw_values_count: 1, token_types_start: 441, token_types_count: 0 }, // [218]
    SimpleHintData { raw_values_start: 441, raw_values_count: 1, token_types_start: 442, token_types_count: 0 }, // [219]
    SimpleHintData { raw_values_start: 442, raw_values_count: 29, token_types_start: 471, token_types_count: 1 }, // [220]
    SimpleHintData { raw_values_start: 472, raw_values_count: 1, token_types_start: 473, token_types_count: 0 }, // [221]
    SimpleHintData { raw_values_start: 473, raw_values_count: 8, token_types_start: 481, token_types_count: 0 }, // [222]
    SimpleHintData { raw_values_start: 481, raw_values_count: 1, token_types_start: 482, token_types_count: 0 }, // [223]
    SimpleHintData { raw_values_start: 482, raw_values_count: 1, token_types_start: 483, token_types_count: 0 }, // [224]
    SimpleHintData { raw_values_start: 483, raw_values_count: 1, token_types_start: 484, token_types_count: 0 }, // [225]
    SimpleHintData { raw_values_start: 484, raw_values_count: 1, token_types_start: 485, token_types_count: 0 }, // [226]
    SimpleHintData { raw_values_start: 485, raw_values_count: 1, token_types_start: 486, token_types_count: 0 }, // [227]
    SimpleHintData { raw_values_start: 486, raw_values_count: 1, token_types_start: 487, token_types_count: 0 }, // [228]
    SimpleHintData { raw_values_start: 487, raw_values_count: 1, token_types_start: 488, token_types_count: 0 }, // [229]
    SimpleHintData { raw_values_start: 488, raw_values_count: 1, token_types_start: 489, token_types_count: 0 }, // [230]
    SimpleHintData { raw_values_start: 489, raw_values_count: 1, token_types_start: 490, token_types_count: 0 }, // [231]
    SimpleHintData { raw_values_start: 490, raw_values_count: 1, token_types_start: 491, token_types_count: 0 }, // [232]
    SimpleHintData { raw_values_start: 491, raw_values_count: 1, token_types_start: 492, token_types_count: 0 }, // [233]
    SimpleHintData { raw_values_start: 492, raw_values_count: 1, token_types_start: 493, token_types_count: 0 }, // [234]
    SimpleHintData { raw_values_start: 493, raw_values_count: 1, token_types_start: 494, token_types_count: 0 }, // [235]
    SimpleHintData { raw_values_start: 494, raw_values_count: 1, token_types_start: 495, token_types_count: 0 }, // [236]
    SimpleHintData { raw_values_start: 495, raw_values_count: 1, token_types_start: 496, token_types_count: 0 }, // [237]
    SimpleHintData { raw_values_start: 496, raw_values_count: 1, token_types_start: 497, token_types_count: 0 }, // [238]
    SimpleHintData { raw_values_start: 497, raw_values_count: 1, token_types_start: 498, token_types_count: 0 }, // [239]
    SimpleHintData { raw_values_start: 498, raw_values_count: 6, token_types_start: 504, token_types_count: 0 }, // [240]
    SimpleHintData { raw_values_start: 504, raw_values_count: 1, token_types_start: 505, token_types_count: 0 }, // [241]
    SimpleHintData { raw_values_start: 505, raw_values_count: 1, token_types_start: 506, token_types_count: 0 }, // [242]
    SimpleHintData { raw_values_start: 506, raw_values_count: 3, token_types_start: 509, token_types_count: 0 }, // [243]
    SimpleHintData { raw_values_start: 509, raw_values_count: 1, token_types_start: 510, token_types_count: 0 }, // [244]
    SimpleHintData { raw_values_start: 510, raw_values_count: 2, token_types_start: 512, token_types_count: 0 }, // [245]
    SimpleHintData { raw_values_start: 512, raw_values_count: 1, token_types_start: 513, token_types_count: 0 }, // [246]
    SimpleHintData { raw_values_start: 513, raw_values_count: 2, token_types_start: 515, token_types_count: 0 }, // [247]
    SimpleHintData { raw_values_start: 515, raw_values_count: 1, token_types_start: 516, token_types_count: 0 }, // [248]
    SimpleHintData { raw_values_start: 516, raw_values_count: 1, token_types_start: 517, token_types_count: 0 }, // [249]
    SimpleHintData { raw_values_start: 517, raw_values_count: 1, token_types_start: 518, token_types_count: 0 }, // [250]
    SimpleHintData { raw_values_start: 518, raw_values_count: 1, token_types_start: 519, token_types_count: 0 }, // [251]
    SimpleHintData { raw_values_start: 519, raw_values_count: 1, token_types_start: 520, token_types_count: 0 }, // [252]
    SimpleHintData { raw_values_start: 520, raw_values_count: 1, token_types_start: 521, token_types_count: 0 }, // [253]
    SimpleHintData { raw_values_start: 521, raw_values_count: 1, token_types_start: 522, token_types_count: 0 }, // [254]
    SimpleHintData { raw_values_start: 522, raw_values_count: 1, token_types_start: 523, token_types_count: 0 }, // [255]
    SimpleHintData { raw_values_start: 523, raw_values_count: 1, token_types_start: 524, token_types_count: 0 }, // [256]
    SimpleHintData { raw_values_start: 524, raw_values_count: 1, token_types_start: 525, token_types_count: 0 }, // [257]
    SimpleHintData { raw_values_start: 525, raw_values_count: 1, token_types_start: 526, token_types_count: 0 }, // [258]
    SimpleHintData { raw_values_start: 526, raw_values_count: 1, token_types_start: 527, token_types_count: 0 }, // [259]
    SimpleHintData { raw_values_start: 527, raw_values_count: 1, token_types_start: 528, token_types_count: 0 }, // [260]
    SimpleHintData { raw_values_start: 528, raw_values_count: 1, token_types_start: 529, token_types_count: 0 }, // [261]
    SimpleHintData { raw_values_start: 529, raw_values_count: 1, token_types_start: 530, token_types_count: 0 }, // [262]
    SimpleHintData { raw_values_start: 530, raw_values_count: 1, token_types_start: 531, token_types_count: 0 }, // [263]
    SimpleHintData { raw_values_start: 531, raw_values_count: 1, token_types_start: 532, token_types_count: 0 }, // [264]
    SimpleHintData { raw_values_start: 532, raw_values_count: 1, token_types_start: 533, token_types_count: 0 }, // [265]
    SimpleHintData { raw_values_start: 533, raw_values_count: 1, token_types_start: 534, token_types_count: 0 }, // [266]
    SimpleHintData { raw_values_start: 534, raw_values_count: 1, token_types_start: 535, token_types_count: 0 }, // [267]
    SimpleHintData { raw_values_start: 535, raw_values_count: 1, token_types_start: 536, token_types_count: 0 }, // [268]
    SimpleHintData { raw_values_start: 536, raw_values_count: 1, token_types_start: 537, token_types_count: 0 }, // [269]
    SimpleHintData { raw_values_start: 537, raw_values_count: 1, token_types_start: 538, token_types_count: 0 }, // [270]
    SimpleHintData { raw_values_start: 538, raw_values_count: 1, token_types_start: 539, token_types_count: 0 }, // [271]
    SimpleHintData { raw_values_start: 539, raw_values_count: 1, token_types_start: 540, token_types_count: 0 }, // [272]
    SimpleHintData { raw_values_start: 540, raw_values_count: 1, token_types_start: 541, token_types_count: 0 }, // [273]
    SimpleHintData { raw_values_start: 541, raw_values_count: 1, token_types_start: 542, token_types_count: 0 }, // [274]
    SimpleHintData { raw_values_start: 542, raw_values_count: 1, token_types_start: 543, token_types_count: 0 }, // [275]
    SimpleHintData { raw_values_start: 543, raw_values_count: 1, token_types_start: 544, token_types_count: 0 }, // [276]
    SimpleHintData { raw_values_start: 544, raw_values_count: 2, token_types_start: 546, token_types_count: 0 }, // [277]
    SimpleHintData { raw_values_start: 546, raw_values_count: 1, token_types_start: 547, token_types_count: 0 }, // [278]
    SimpleHintData { raw_values_start: 547, raw_values_count: 1, token_types_start: 548, token_types_count: 0 }, // [279]
    SimpleHintData { raw_values_start: 548, raw_values_count: 1, token_types_start: 549, token_types_count: 0 }, // [280]
    SimpleHintData { raw_values_start: 549, raw_values_count: 1, token_types_start: 550, token_types_count: 0 }, // [281]
    SimpleHintData { raw_values_start: 550, raw_values_count: 1, token_types_start: 551, token_types_count: 0 }, // [282]
    SimpleHintData { raw_values_start: 551, raw_values_count: 1, token_types_start: 552, token_types_count: 0 }, // [283]
    SimpleHintData { raw_values_start: 552, raw_values_count: 1, token_types_start: 553, token_types_count: 0 }, // [284]
    SimpleHintData { raw_values_start: 553, raw_values_count: 1, token_types_start: 554, token_types_count: 0 }, // [285]
    SimpleHintData { raw_values_start: 554, raw_values_count: 1, token_types_start: 555, token_types_count: 0 }, // [286]
    SimpleHintData { raw_values_start: 555, raw_values_count: 1, token_types_start: 556, token_types_count: 0 }, // [287]
    SimpleHintData { raw_values_start: 556, raw_values_count: 1, token_types_start: 557, token_types_count: 0 }, // [288]
    SimpleHintData { raw_values_start: 557, raw_values_count: 1, token_types_start: 558, token_types_count: 0 }, // [289]
    SimpleHintData { raw_values_start: 558, raw_values_count: 1, token_types_start: 559, token_types_count: 0 }, // [290]
    SimpleHintData { raw_values_start: 559, raw_values_count: 1, token_types_start: 560, token_types_count: 0 }, // [291]
    SimpleHintData { raw_values_start: 560, raw_values_count: 1, token_types_start: 561, token_types_count: 0 }, // [292]
    SimpleHintData { raw_values_start: 561, raw_values_count: 1, token_types_start: 562, token_types_count: 0 }, // [293]
    SimpleHintData { raw_values_start: 562, raw_values_count: 1, token_types_start: 563, token_types_count: 0 }, // [294]
    SimpleHintData { raw_values_start: 563, raw_values_count: 2, token_types_start: 565, token_types_count: 0 }, // [295]
    SimpleHintData { raw_values_start: 565, raw_values_count: 1, token_types_start: 566, token_types_count: 0 }, // [296]
    SimpleHintData { raw_values_start: 566, raw_values_count: 0, token_types_start: 566, token_types_count: 1 }, // [297]
    SimpleHintData { raw_values_start: 567, raw_values_count: 1, token_types_start: 568, token_types_count: 0 }, // [298]
    SimpleHintData { raw_values_start: 568, raw_values_count: 1, token_types_start: 569, token_types_count: 0 }, // [299]
    SimpleHintData { raw_values_start: 569, raw_values_count: 1, token_types_start: 570, token_types_count: 0 }, // [300]
    SimpleHintData { raw_values_start: 570, raw_values_count: 1, token_types_start: 571, token_types_count: 0 }, // [301]
    SimpleHintData { raw_values_start: 571, raw_values_count: 1, token_types_start: 572, token_types_count: 0 }, // [302]
    SimpleHintData { raw_values_start: 572, raw_values_count: 1, token_types_start: 573, token_types_count: 0 }, // [303]
    SimpleHintData { raw_values_start: 573, raw_values_count: 1, token_types_start: 574, token_types_count: 0 }, // [304]
    SimpleHintData { raw_values_start: 574, raw_values_count: 1, token_types_start: 575, token_types_count: 0 }, // [305]
    SimpleHintData { raw_values_start: 575, raw_values_count: 4, token_types_start: 579, token_types_count: 0 }, // [306]
    SimpleHintData { raw_values_start: 579, raw_values_count: 1, token_types_start: 580, token_types_count: 0 }, // [307]
    SimpleHintData { raw_values_start: 580, raw_values_count: 1, token_types_start: 581, token_types_count: 0 }, // [308]
    SimpleHintData { raw_values_start: 581, raw_values_count: 1, token_types_start: 582, token_types_count: 0 }, // [309]
    SimpleHintData { raw_values_start: 582, raw_values_count: 1, token_types_start: 583, token_types_count: 0 }, // [310]
    SimpleHintData { raw_values_start: 583, raw_values_count: 1, token_types_start: 584, token_types_count: 0 }, // [311]
    SimpleHintData { raw_values_start: 584, raw_values_count: 1, token_types_start: 585, token_types_count: 0 }, // [312]
    SimpleHintData { raw_values_start: 585, raw_values_count: 1, token_types_start: 586, token_types_count: 0 }, // [313]
    SimpleHintData { raw_values_start: 586, raw_values_count: 1, token_types_start: 587, token_types_count: 0 }, // [314]
    SimpleHintData { raw_values_start: 587, raw_values_count: 1, token_types_start: 588, token_types_count: 0 }, // [315]
    SimpleHintData { raw_values_start: 588, raw_values_count: 1, token_types_start: 589, token_types_count: 0 }, // [316]
    SimpleHintData { raw_values_start: 589, raw_values_count: 1, token_types_start: 590, token_types_count: 0 }, // [317]
    SimpleHintData { raw_values_start: 590, raw_values_count: 1, token_types_start: 591, token_types_count: 0 }, // [318]
    SimpleHintData { raw_values_start: 591, raw_values_count: 1, token_types_start: 592, token_types_count: 0 }, // [319]
    SimpleHintData { raw_values_start: 592, raw_values_count: 1, token_types_start: 593, token_types_count: 0 }, // [320]
    SimpleHintData { raw_values_start: 593, raw_values_count: 1, token_types_start: 594, token_types_count: 0 }, // [321]
    SimpleHintData { raw_values_start: 594, raw_values_count: 1, token_types_start: 595, token_types_count: 0 }, // [322]
    SimpleHintData { raw_values_start: 595, raw_values_count: 1, token_types_start: 596, token_types_count: 0 }, // [323]
    SimpleHintData { raw_values_start: 596, raw_values_count: 1, token_types_start: 597, token_types_count: 0 }, // [324]
    SimpleHintData { raw_values_start: 597, raw_values_count: 1, token_types_start: 598, token_types_count: 0 }, // [325]
    SimpleHintData { raw_values_start: 598, raw_values_count: 1, token_types_start: 599, token_types_count: 0 }, // [326]
    SimpleHintData { raw_values_start: 599, raw_values_count: 7, token_types_start: 606, token_types_count: 0 }, // [327]
    SimpleHintData { raw_values_start: 606, raw_values_count: 4, token_types_start: 610, token_types_count: 0 }, // [328]
    SimpleHintData { raw_values_start: 610, raw_values_count: 3, token_types_start: 613, token_types_count: 0 }, // [329]
    SimpleHintData { raw_values_start: 613, raw_values_count: 1, token_types_start: 614, token_types_count: 0 }, // [330]
    SimpleHintData { raw_values_start: 614, raw_values_count: 1, token_types_start: 615, token_types_count: 0 }, // [331]
    SimpleHintData { raw_values_start: 615, raw_values_count: 1, token_types_start: 616, token_types_count: 0 }, // [332]
    SimpleHintData { raw_values_start: 616, raw_values_count: 1, token_types_start: 617, token_types_count: 0 }, // [333]
    SimpleHintData { raw_values_start: 617, raw_values_count: 2, token_types_start: 619, token_types_count: 0 }, // [334]
    SimpleHintData { raw_values_start: 619, raw_values_count: 1, token_types_start: 620, token_types_count: 0 }, // [335]
    SimpleHintData { raw_values_start: 620, raw_values_count: 1, token_types_start: 621, token_types_count: 0 }, // [336]
    SimpleHintData { raw_values_start: 621, raw_values_count: 1, token_types_start: 622, token_types_count: 0 }, // [337]
    SimpleHintData { raw_values_start: 622, raw_values_count: 1, token_types_start: 623, token_types_count: 0 }, // [338]
    SimpleHintData { raw_values_start: 623, raw_values_count: 11, token_types_start: 634, token_types_count: 0 }, // [339]
    SimpleHintData { raw_values_start: 634, raw_values_count: 1, token_types_start: 635, token_types_count: 0 }, // [340]
    SimpleHintData { raw_values_start: 635, raw_values_count: 4, token_types_start: 639, token_types_count: 0 }, // [341]
    SimpleHintData { raw_values_start: 639, raw_values_count: 2, token_types_start: 641, token_types_count: 0 }, // [342]
    SimpleHintData { raw_values_start: 641, raw_values_count: 1, token_types_start: 642, token_types_count: 0 }, // [343]
    SimpleHintData { raw_values_start: 642, raw_values_count: 2, token_types_start: 644, token_types_count: 0 }, // [344]
    SimpleHintData { raw_values_start: 644, raw_values_count: 1, token_types_start: 645, token_types_count: 0 }, // [345]
    SimpleHintData { raw_values_start: 645, raw_values_count: 1, token_types_start: 646, token_types_count: 0 }, // [346]
    SimpleHintData { raw_values_start: 646, raw_values_count: 3, token_types_start: 649, token_types_count: 0 }, // [347]
    SimpleHintData { raw_values_start: 649, raw_values_count: 4, token_types_start: 653, token_types_count: 0 }, // [348]
    SimpleHintData { raw_values_start: 653, raw_values_count: 1, token_types_start: 654, token_types_count: 0 }, // [349]
    SimpleHintData { raw_values_start: 654, raw_values_count: 8, token_types_start: 662, token_types_count: 0 }, // [350]
    SimpleHintData { raw_values_start: 662, raw_values_count: 3, token_types_start: 665, token_types_count: 0 }, // [351]
    SimpleHintData { raw_values_start: 665, raw_values_count: 1, token_types_start: 666, token_types_count: 0 }, // [352]
    SimpleHintData { raw_values_start: 666, raw_values_count: 1, token_types_start: 667, token_types_count: 0 }, // [353]
    SimpleHintData { raw_values_start: 667, raw_values_count: 3, token_types_start: 670, token_types_count: 0 }, // [354]
    SimpleHintData { raw_values_start: 670, raw_values_count: 7, token_types_start: 677, token_types_count: 0 }, // [355]
    SimpleHintData { raw_values_start: 677, raw_values_count: 1, token_types_start: 678, token_types_count: 0 }, // [356]
    SimpleHintData { raw_values_start: 678, raw_values_count: 1, token_types_start: 679, token_types_count: 0 }, // [357]
    SimpleHintData { raw_values_start: 679, raw_values_count: 1, token_types_start: 680, token_types_count: 0 }, // [358]
    SimpleHintData { raw_values_start: 680, raw_values_count: 1, token_types_start: 681, token_types_count: 0 }, // [359]
    SimpleHintData { raw_values_start: 681, raw_values_count: 3, token_types_start: 684, token_types_count: 0 }, // [360]
    SimpleHintData { raw_values_start: 684, raw_values_count: 1, token_types_start: 685, token_types_count: 0 }, // [361]
    SimpleHintData { raw_values_start: 685, raw_values_count: 2, token_types_start: 687, token_types_count: 0 }, // [362]
    SimpleHintData { raw_values_start: 687, raw_values_count: 1, token_types_start: 688, token_types_count: 0 }, // [363]
    SimpleHintData { raw_values_start: 688, raw_values_count: 1, token_types_start: 689, token_types_count: 0 }, // [364]
    SimpleHintData { raw_values_start: 689, raw_values_count: 1, token_types_start: 690, token_types_count: 0 }, // [365]
    SimpleHintData { raw_values_start: 690, raw_values_count: 1, token_types_start: 691, token_types_count: 0 }, // [366]
    SimpleHintData { raw_values_start: 691, raw_values_count: 2, token_types_start: 693, token_types_count: 0 }, // [367]
    SimpleHintData { raw_values_start: 693, raw_values_count: 1, token_types_start: 694, token_types_count: 0 }, // [368]
    SimpleHintData { raw_values_start: 694, raw_values_count: 0, token_types_start: 694, token_types_count: 1 }, // [369]
    SimpleHintData { raw_values_start: 695, raw_values_count: 2, token_types_start: 697, token_types_count: 0 }, // [370]
    SimpleHintData { raw_values_start: 697, raw_values_count: 1, token_types_start: 698, token_types_count: 0 }, // [371]
    SimpleHintData { raw_values_start: 698, raw_values_count: 1, token_types_start: 699, token_types_count: 0 }, // [372]
    SimpleHintData { raw_values_start: 699, raw_values_count: 1, token_types_start: 700, token_types_count: 0 }, // [373]
    SimpleHintData { raw_values_start: 700, raw_values_count: 1, token_types_start: 701, token_types_count: 0 }, // [374]
    SimpleHintData { raw_values_start: 701, raw_values_count: 1, token_types_start: 702, token_types_count: 0 }, // [375]
    SimpleHintData { raw_values_start: 702, raw_values_count: 1, token_types_start: 703, token_types_count: 0 }, // [376]
    SimpleHintData { raw_values_start: 703, raw_values_count: 2, token_types_start: 705, token_types_count: 0 }, // [377]
    SimpleHintData { raw_values_start: 705, raw_values_count: 1, token_types_start: 706, token_types_count: 0 }, // [378]
    SimpleHintData { raw_values_start: 706, raw_values_count: 1, token_types_start: 707, token_types_count: 0 }, // [379]
    SimpleHintData { raw_values_start: 707, raw_values_count: 1, token_types_start: 708, token_types_count: 0 }, // [380]
    SimpleHintData { raw_values_start: 708, raw_values_count: 1, token_types_start: 709, token_types_count: 0 }, // [381]
    SimpleHintData { raw_values_start: 709, raw_values_count: 1, token_types_start: 710, token_types_count: 0 }, // [382]
    SimpleHintData { raw_values_start: 710, raw_values_count: 1, token_types_start: 711, token_types_count: 0 }, // [383]
    SimpleHintData { raw_values_start: 711, raw_values_count: 1, token_types_start: 712, token_types_count: 0 }, // [384]
    SimpleHintData { raw_values_start: 712, raw_values_count: 1, token_types_start: 713, token_types_count: 0 }, // [385]
    SimpleHintData { raw_values_start: 713, raw_values_count: 1, token_types_start: 714, token_types_count: 0 }, // [386]
    SimpleHintData { raw_values_start: 714, raw_values_count: 1, token_types_start: 715, token_types_count: 0 }, // [387]
    SimpleHintData { raw_values_start: 715, raw_values_count: 1, token_types_start: 716, token_types_count: 0 }, // [388]
    SimpleHintData { raw_values_start: 716, raw_values_count: 1, token_types_start: 717, token_types_count: 0 }, // [389]
    SimpleHintData { raw_values_start: 717, raw_values_count: 1, token_types_start: 718, token_types_count: 0 }, // [390]
    SimpleHintData { raw_values_start: 718, raw_values_count: 1, token_types_start: 719, token_types_count: 0 }, // [391]
    SimpleHintData { raw_values_start: 719, raw_values_count: 1, token_types_start: 720, token_types_count: 0 }, // [392]
    SimpleHintData { raw_values_start: 720, raw_values_count: 1, token_types_start: 721, token_types_count: 0 }, // [393]
    SimpleHintData { raw_values_start: 721, raw_values_count: 11, token_types_start: 732, token_types_count: 0 }, // [394]
    SimpleHintData { raw_values_start: 732, raw_values_count: 4, token_types_start: 736, token_types_count: 0 }, // [395]
    SimpleHintData { raw_values_start: 736, raw_values_count: 1, token_types_start: 737, token_types_count: 0 }, // [396]
    SimpleHintData { raw_values_start: 737, raw_values_count: 1, token_types_start: 738, token_types_count: 0 }, // [397]
    SimpleHintData { raw_values_start: 738, raw_values_count: 1, token_types_start: 739, token_types_count: 0 }, // [398]
    SimpleHintData { raw_values_start: 739, raw_values_count: 1, token_types_start: 740, token_types_count: 0 }, // [399]
    SimpleHintData { raw_values_start: 740, raw_values_count: 1, token_types_start: 741, token_types_count: 0 }, // [400]
    SimpleHintData { raw_values_start: 741, raw_values_count: 1, token_types_start: 742, token_types_count: 0 }, // [401]
    SimpleHintData { raw_values_start: 742, raw_values_count: 1, token_types_start: 743, token_types_count: 0 }, // [402]
    SimpleHintData { raw_values_start: 743, raw_values_count: 1, token_types_start: 744, token_types_count: 0 }, // [403]
    SimpleHintData { raw_values_start: 744, raw_values_count: 1, token_types_start: 745, token_types_count: 0 }, // [404]
    SimpleHintData { raw_values_start: 745, raw_values_count: 1, token_types_start: 746, token_types_count: 0 }, // [405]
    SimpleHintData { raw_values_start: 746, raw_values_count: 1, token_types_start: 747, token_types_count: 0 }, // [406]
    SimpleHintData { raw_values_start: 747, raw_values_count: 1, token_types_start: 748, token_types_count: 0 }, // [407]
    SimpleHintData { raw_values_start: 748, raw_values_count: 1, token_types_start: 749, token_types_count: 0 }, // [408]
    SimpleHintData { raw_values_start: 749, raw_values_count: 1, token_types_start: 750, token_types_count: 0 }, // [409]
    SimpleHintData { raw_values_start: 750, raw_values_count: 1, token_types_start: 751, token_types_count: 0 }, // [410]
    SimpleHintData { raw_values_start: 751, raw_values_count: 1, token_types_start: 752, token_types_count: 0 }, // [411]
    SimpleHintData { raw_values_start: 752, raw_values_count: 1, token_types_start: 753, token_types_count: 0 }, // [412]
    SimpleHintData { raw_values_start: 753, raw_values_count: 1, token_types_start: 754, token_types_count: 0 }, // [413]
    SimpleHintData { raw_values_start: 754, raw_values_count: 1, token_types_start: 755, token_types_count: 0 }, // [414]
    SimpleHintData { raw_values_start: 755, raw_values_count: 1, token_types_start: 756, token_types_count: 0 }, // [415]
    SimpleHintData { raw_values_start: 756, raw_values_count: 1, token_types_start: 757, token_types_count: 0 }, // [416]
    SimpleHintData { raw_values_start: 757, raw_values_count: 1, token_types_start: 758, token_types_count: 0 }, // [417]
    SimpleHintData { raw_values_start: 758, raw_values_count: 1, token_types_start: 759, token_types_count: 0 }, // [418]
    SimpleHintData { raw_values_start: 759, raw_values_count: 1, token_types_start: 760, token_types_count: 0 }, // [419]
    SimpleHintData { raw_values_start: 760, raw_values_count: 1, token_types_start: 761, token_types_count: 0 }, // [420]
    SimpleHintData { raw_values_start: 761, raw_values_count: 1, token_types_start: 762, token_types_count: 0 }, // [421]
    SimpleHintData { raw_values_start: 762, raw_values_count: 1, token_types_start: 763, token_types_count: 0 }, // [422]
    SimpleHintData { raw_values_start: 763, raw_values_count: 2, token_types_start: 765, token_types_count: 0 }, // [423]
    SimpleHintData { raw_values_start: 765, raw_values_count: 1, token_types_start: 766, token_types_count: 0 }, // [424]
    SimpleHintData { raw_values_start: 766, raw_values_count: 1, token_types_start: 767, token_types_count: 0 }, // [425]
    SimpleHintData { raw_values_start: 767, raw_values_count: 1, token_types_start: 768, token_types_count: 0 }, // [426]
    SimpleHintData { raw_values_start: 768, raw_values_count: 1, token_types_start: 769, token_types_count: 0 }, // [427]
    SimpleHintData { raw_values_start: 769, raw_values_count: 1, token_types_start: 770, token_types_count: 0 }, // [428]
    SimpleHintData { raw_values_start: 770, raw_values_count: 1, token_types_start: 771, token_types_count: 0 }, // [429]
    SimpleHintData { raw_values_start: 771, raw_values_count: 1, token_types_start: 772, token_types_count: 0 }, // [430]
    SimpleHintData { raw_values_start: 772, raw_values_count: 1, token_types_start: 773, token_types_count: 0 }, // [431]
    SimpleHintData { raw_values_start: 773, raw_values_count: 1, token_types_start: 774, token_types_count: 0 }, // [432]
    SimpleHintData { raw_values_start: 774, raw_values_count: 1, token_types_start: 775, token_types_count: 0 }, // [433]
    SimpleHintData { raw_values_start: 775, raw_values_count: 1, token_types_start: 776, token_types_count: 0 }, // [434]
    SimpleHintData { raw_values_start: 776, raw_values_count: 1, token_types_start: 777, token_types_count: 0 }, // [435]
    SimpleHintData { raw_values_start: 777, raw_values_count: 1, token_types_start: 778, token_types_count: 0 }, // [436]
    SimpleHintData { raw_values_start: 778, raw_values_count: 1, token_types_start: 779, token_types_count: 0 }, // [437]
    SimpleHintData { raw_values_start: 779, raw_values_count: 1, token_types_start: 780, token_types_count: 0 }, // [438]
    SimpleHintData { raw_values_start: 780, raw_values_count: 1, token_types_start: 781, token_types_count: 0 }, // [439]
    SimpleHintData { raw_values_start: 781, raw_values_count: 1, token_types_start: 782, token_types_count: 0 }, // [440]
    SimpleHintData { raw_values_start: 782, raw_values_count: 1, token_types_start: 783, token_types_count: 0 }, // [441]
    SimpleHintData { raw_values_start: 783, raw_values_count: 1, token_types_start: 784, token_types_count: 0 }, // [442]
    SimpleHintData { raw_values_start: 784, raw_values_count: 1, token_types_start: 785, token_types_count: 0 }, // [443]
    SimpleHintData { raw_values_start: 785, raw_values_count: 1, token_types_start: 786, token_types_count: 0 }, // [444]
    SimpleHintData { raw_values_start: 786, raw_values_count: 1, token_types_start: 787, token_types_count: 0 }, // [445]
    SimpleHintData { raw_values_start: 787, raw_values_count: 1, token_types_start: 788, token_types_count: 0 }, // [446]
    SimpleHintData { raw_values_start: 788, raw_values_count: 1, token_types_start: 789, token_types_count: 0 }, // [447]
    SimpleHintData { raw_values_start: 789, raw_values_count: 1, token_types_start: 790, token_types_count: 0 }, // [448]
    SimpleHintData { raw_values_start: 790, raw_values_count: 1, token_types_start: 791, token_types_count: 0 }, // [449]
    SimpleHintData { raw_values_start: 791, raw_values_count: 2, token_types_start: 793, token_types_count: 0 }, // [450]
    SimpleHintData { raw_values_start: 793, raw_values_count: 1, token_types_start: 794, token_types_count: 0 }, // [451]
    SimpleHintData { raw_values_start: 794, raw_values_count: 1, token_types_start: 795, token_types_count: 0 }, // [452]
    SimpleHintData { raw_values_start: 795, raw_values_count: 1, token_types_start: 796, token_types_count: 0 }, // [453]
    SimpleHintData { raw_values_start: 796, raw_values_count: 1, token_types_start: 797, token_types_count: 0 }, // [454]
    SimpleHintData { raw_values_start: 797, raw_values_count: 1, token_types_start: 798, token_types_count: 0 }, // [455]
    SimpleHintData { raw_values_start: 798, raw_values_count: 1, token_types_start: 799, token_types_count: 0 }, // [456]
    SimpleHintData { raw_values_start: 799, raw_values_count: 1, token_types_start: 800, token_types_count: 0 }, // [457]
    SimpleHintData { raw_values_start: 800, raw_values_count: 1, token_types_start: 801, token_types_count: 0 }, // [458]
    SimpleHintData { raw_values_start: 801, raw_values_count: 1, token_types_start: 802, token_types_count: 0 }, // [459]
    SimpleHintData { raw_values_start: 802, raw_values_count: 1, token_types_start: 803, token_types_count: 0 }, // [460]
    SimpleHintData { raw_values_start: 803, raw_values_count: 1, token_types_start: 804, token_types_count: 0 }, // [461]
    SimpleHintData { raw_values_start: 804, raw_values_count: 1, token_types_start: 805, token_types_count: 0 }, // [462]
    SimpleHintData { raw_values_start: 805, raw_values_count: 1, token_types_start: 806, token_types_count: 0 }, // [463]
    SimpleHintData { raw_values_start: 806, raw_values_count: 1, token_types_start: 807, token_types_count: 0 }, // [464]
    SimpleHintData { raw_values_start: 807, raw_values_count: 1, token_types_start: 808, token_types_count: 0 }, // [465]
    SimpleHintData { raw_values_start: 808, raw_values_count: 1, token_types_start: 809, token_types_count: 0 }, // [466]
    SimpleHintData { raw_values_start: 809, raw_values_count: 1, token_types_start: 810, token_types_count: 0 }, // [467]
    SimpleHintData { raw_values_start: 810, raw_values_count: 1, token_types_start: 811, token_types_count: 0 }, // [468]
    SimpleHintData { raw_values_start: 811, raw_values_count: 1, token_types_start: 812, token_types_count: 0 }, // [469]
    SimpleHintData { raw_values_start: 812, raw_values_count: 1, token_types_start: 813, token_types_count: 0 }, // [470]
    SimpleHintData { raw_values_start: 813, raw_values_count: 1, token_types_start: 814, token_types_count: 0 }, // [471]
    SimpleHintData { raw_values_start: 814, raw_values_count: 1, token_types_start: 815, token_types_count: 0 }, // [472]
    SimpleHintData { raw_values_start: 815, raw_values_count: 1, token_types_start: 816, token_types_count: 0 }, // [473]
    SimpleHintData { raw_values_start: 816, raw_values_count: 1, token_types_start: 817, token_types_count: 0 }, // [474]
    SimpleHintData { raw_values_start: 817, raw_values_count: 1, token_types_start: 818, token_types_count: 0 }, // [475]
    SimpleHintData { raw_values_start: 818, raw_values_count: 1, token_types_start: 819, token_types_count: 0 }, // [476]
    SimpleHintData { raw_values_start: 819, raw_values_count: 1, token_types_start: 820, token_types_count: 0 }, // [477]
    SimpleHintData { raw_values_start: 820, raw_values_count: 1, token_types_start: 821, token_types_count: 0 }, // [478]
    SimpleHintData { raw_values_start: 821, raw_values_count: 1, token_types_start: 822, token_types_count: 0 }, // [479]
    SimpleHintData { raw_values_start: 822, raw_values_count: 1, token_types_start: 823, token_types_count: 0 }, // [480]
    SimpleHintData { raw_values_start: 823, raw_values_count: 1, token_types_start: 824, token_types_count: 0 }, // [481]
    SimpleHintData { raw_values_start: 824, raw_values_count: 1, token_types_start: 825, token_types_count: 0 }, // [482]
    SimpleHintData { raw_values_start: 825, raw_values_count: 1, token_types_start: 826, token_types_count: 0 }, // [483]
    SimpleHintData { raw_values_start: 826, raw_values_count: 12, token_types_start: 838, token_types_count: 0 }, // [484]
    SimpleHintData { raw_values_start: 838, raw_values_count: 4, token_types_start: 842, token_types_count: 0 }, // [485]
    SimpleHintData { raw_values_start: 842, raw_values_count: 2, token_types_start: 844, token_types_count: 0 }, // [486]
    SimpleHintData { raw_values_start: 844, raw_values_count: 2, token_types_start: 846, token_types_count: 0 }, // [487]
    SimpleHintData { raw_values_start: 846, raw_values_count: 5, token_types_start: 851, token_types_count: 0 }, // [488]
    SimpleHintData { raw_values_start: 851, raw_values_count: 2, token_types_start: 853, token_types_count: 0 }, // [489]
    SimpleHintData { raw_values_start: 853, raw_values_count: 1, token_types_start: 854, token_types_count: 0 }, // [490]
    SimpleHintData { raw_values_start: 854, raw_values_count: 1, token_types_start: 855, token_types_count: 0 }, // [491]
    SimpleHintData { raw_values_start: 855, raw_values_count: 21, token_types_start: 876, token_types_count: 1 }, // [492]
    SimpleHintData { raw_values_start: 877, raw_values_count: 3, token_types_start: 880, token_types_count: 0 }, // [493]
    SimpleHintData { raw_values_start: 880, raw_values_count: 7, token_types_start: 887, token_types_count: 2 }, // [494]
    SimpleHintData { raw_values_start: 889, raw_values_count: 4, token_types_start: 893, token_types_count: 0 }, // [495]
    SimpleHintData { raw_values_start: 893, raw_values_count: 1, token_types_start: 894, token_types_count: 0 }, // [496]
    SimpleHintData { raw_values_start: 894, raw_values_count: 1, token_types_start: 895, token_types_count: 0 }, // [497]
    SimpleHintData { raw_values_start: 895, raw_values_count: 1, token_types_start: 896, token_types_count: 0 }, // [498]
    SimpleHintData { raw_values_start: 896, raw_values_count: 1, token_types_start: 897, token_types_count: 0 }, // [499]
    SimpleHintData { raw_values_start: 897, raw_values_count: 2, token_types_start: 899, token_types_count: 0 }, // [500]
    SimpleHintData { raw_values_start: 899, raw_values_count: 1, token_types_start: 900, token_types_count: 0 }, // [501]
    SimpleHintData { raw_values_start: 900, raw_values_count: 2, token_types_start: 902, token_types_count: 0 }, // [502]
    SimpleHintData { raw_values_start: 902, raw_values_count: 1, token_types_start: 903, token_types_count: 0 }, // [503]
    SimpleHintData { raw_values_start: 903, raw_values_count: 2, token_types_start: 905, token_types_count: 0 }, // [504]
    SimpleHintData { raw_values_start: 905, raw_values_count: 1, token_types_start: 906, token_types_count: 0 }, // [505]
    SimpleHintData { raw_values_start: 906, raw_values_count: 1, token_types_start: 907, token_types_count: 0 }, // [506]
    SimpleHintData { raw_values_start: 907, raw_values_count: 1, token_types_start: 908, token_types_count: 0 }, // [507]
    SimpleHintData { raw_values_start: 908, raw_values_count: 37, token_types_start: 945, token_types_count: 0 }, // [508]
    SimpleHintData { raw_values_start: 945, raw_values_count: 36, token_types_start: 981, token_types_count: 0 }, // [509]
    SimpleHintData { raw_values_start: 981, raw_values_count: 1, token_types_start: 982, token_types_count: 0 }, // [510]
    SimpleHintData { raw_values_start: 982, raw_values_count: 1, token_types_start: 983, token_types_count: 0 }, // [511]
    SimpleHintData { raw_values_start: 983, raw_values_count: 1, token_types_start: 984, token_types_count: 0 }, // [512]
    SimpleHintData { raw_values_start: 984, raw_values_count: 1, token_types_start: 985, token_types_count: 0 }, // [513]
    SimpleHintData { raw_values_start: 985, raw_values_count: 1, token_types_start: 986, token_types_count: 0 }, // [514]
    SimpleHintData { raw_values_start: 986, raw_values_count: 1, token_types_start: 987, token_types_count: 0 }, // [515]
    SimpleHintData { raw_values_start: 987, raw_values_count: 1, token_types_start: 988, token_types_count: 0 }, // [516]
    SimpleHintData { raw_values_start: 988, raw_values_count: 1, token_types_start: 989, token_types_count: 0 }, // [517]
    SimpleHintData { raw_values_start: 989, raw_values_count: 1, token_types_start: 990, token_types_count: 0 }, // [518]
    SimpleHintData { raw_values_start: 990, raw_values_count: 1, token_types_start: 991, token_types_count: 0 }, // [519]
    SimpleHintData { raw_values_start: 991, raw_values_count: 1, token_types_start: 992, token_types_count: 0 }, // [520]
    SimpleHintData { raw_values_start: 992, raw_values_count: 1, token_types_start: 993, token_types_count: 0 }, // [521]
    SimpleHintData { raw_values_start: 993, raw_values_count: 1, token_types_start: 994, token_types_count: 0 }, // [522]
    SimpleHintData { raw_values_start: 994, raw_values_count: 1, token_types_start: 995, token_types_count: 0 }, // [523]
    SimpleHintData { raw_values_start: 995, raw_values_count: 2, token_types_start: 997, token_types_count: 0 }, // [524]
    SimpleHintData { raw_values_start: 997, raw_values_count: 1, token_types_start: 998, token_types_count: 0 }, // [525]
    SimpleHintData { raw_values_start: 998, raw_values_count: 2, token_types_start: 1000, token_types_count: 0 }, // [526]
    SimpleHintData { raw_values_start: 1000, raw_values_count: 1, token_types_start: 1001, token_types_count: 0 }, // [527]
    SimpleHintData { raw_values_start: 1001, raw_values_count: 1, token_types_start: 1002, token_types_count: 0 }, // [528]
    SimpleHintData { raw_values_start: 1002, raw_values_count: 1, token_types_start: 1003, token_types_count: 0 }, // [529]
    SimpleHintData { raw_values_start: 1003, raw_values_count: 1, token_types_start: 1004, token_types_count: 0 }, // [530]
    SimpleHintData { raw_values_start: 1004, raw_values_count: 1, token_types_start: 1005, token_types_count: 0 }, // [531]
    SimpleHintData { raw_values_start: 1005, raw_values_count: 1, token_types_start: 1006, token_types_count: 0 }, // [532]
    SimpleHintData { raw_values_start: 1006, raw_values_count: 14, token_types_start: 1020, token_types_count: 0 }, // [533]
    SimpleHintData { raw_values_start: 1020, raw_values_count: 1, token_types_start: 1021, token_types_count: 0 }, // [534]
    SimpleHintData { raw_values_start: 1021, raw_values_count: 1, token_types_start: 1022, token_types_count: 0 }, // [535]
    SimpleHintData { raw_values_start: 1022, raw_values_count: 1, token_types_start: 1023, token_types_count: 0 }, // [536]
    SimpleHintData { raw_values_start: 1023, raw_values_count: 1, token_types_start: 1024, token_types_count: 0 }, // [537]
    SimpleHintData { raw_values_start: 1024, raw_values_count: 4, token_types_start: 1028, token_types_count: 0 }, // [538]
    SimpleHintData { raw_values_start: 1028, raw_values_count: 1, token_types_start: 1029, token_types_count: 0 }, // [539]
    SimpleHintData { raw_values_start: 1029, raw_values_count: 1, token_types_start: 1030, token_types_count: 0 }, // [540]
    SimpleHintData { raw_values_start: 1030, raw_values_count: 1, token_types_start: 1031, token_types_count: 0 }, // [541]
    SimpleHintData { raw_values_start: 1031, raw_values_count: 1, token_types_start: 1032, token_types_count: 0 }, // [542]
    SimpleHintData { raw_values_start: 1032, raw_values_count: 1, token_types_start: 1033, token_types_count: 0 }, // [543]
    SimpleHintData { raw_values_start: 1033, raw_values_count: 3, token_types_start: 1036, token_types_count: 0 }, // [544]
    SimpleHintData { raw_values_start: 1036, raw_values_count: 1, token_types_start: 1037, token_types_count: 0 }, // [545]
    SimpleHintData { raw_values_start: 1037, raw_values_count: 2, token_types_start: 1039, token_types_count: 0 }, // [546]
    SimpleHintData { raw_values_start: 1039, raw_values_count: 0, token_types_start: 1039, token_types_count: 2 }, // [547]
    SimpleHintData { raw_values_start: 1041, raw_values_count: 0, token_types_start: 1041, token_types_count: 1 }, // [548]
    SimpleHintData { raw_values_start: 1042, raw_values_count: 2, token_types_start: 1044, token_types_count: 0 }, // [549]
    SimpleHintData { raw_values_start: 1044, raw_values_count: 1, token_types_start: 1045, token_types_count: 0 }, // [550]
    SimpleHintData { raw_values_start: 1045, raw_values_count: 1, token_types_start: 1046, token_types_count: 0 }, // [551]
    SimpleHintData { raw_values_start: 1046, raw_values_count: 1, token_types_start: 1047, token_types_count: 0 }, // [552]
    SimpleHintData { raw_values_start: 1047, raw_values_count: 1, token_types_start: 1048, token_types_count: 0 }, // [553]
    SimpleHintData { raw_values_start: 1048, raw_values_count: 1, token_types_start: 1049, token_types_count: 0 }, // [554]
    SimpleHintData { raw_values_start: 1049, raw_values_count: 0, token_types_start: 1049, token_types_count: 1 }, // [555]
    SimpleHintData { raw_values_start: 1050, raw_values_count: 1, token_types_start: 1051, token_types_count: 0 }, // [556]
    SimpleHintData { raw_values_start: 1051, raw_values_count: 2, token_types_start: 1053, token_types_count: 0 }, // [557]
    SimpleHintData { raw_values_start: 1053, raw_values_count: 1, token_types_start: 1054, token_types_count: 0 }, // [558]
    SimpleHintData { raw_values_start: 1054, raw_values_count: 1, token_types_start: 1055, token_types_count: 0 }, // [559]
    SimpleHintData { raw_values_start: 1055, raw_values_count: 1, token_types_start: 1056, token_types_count: 0 }, // [560]
    SimpleHintData { raw_values_start: 1056, raw_values_count: 1, token_types_start: 1057, token_types_count: 0 }, // [561]
    SimpleHintData { raw_values_start: 1057, raw_values_count: 1, token_types_start: 1058, token_types_count: 0 }, // [562]
    SimpleHintData { raw_values_start: 1058, raw_values_count: 6, token_types_start: 1064, token_types_count: 0 }, // [563]
    SimpleHintData { raw_values_start: 1064, raw_values_count: 1, token_types_start: 1065, token_types_count: 0 }, // [564]
    SimpleHintData { raw_values_start: 1065, raw_values_count: 1, token_types_start: 1066, token_types_count: 0 }, // [565]
    SimpleHintData { raw_values_start: 1066, raw_values_count: 1, token_types_start: 1067, token_types_count: 0 }, // [566]
    SimpleHintData { raw_values_start: 1067, raw_values_count: 5, token_types_start: 1072, token_types_count: 0 }, // [567]
    SimpleHintData { raw_values_start: 1072, raw_values_count: 1, token_types_start: 1073, token_types_count: 0 }, // [568]
    SimpleHintData { raw_values_start: 1073, raw_values_count: 1, token_types_start: 1074, token_types_count: 0 }, // [569]
    SimpleHintData { raw_values_start: 1074, raw_values_count: 1, token_types_start: 1075, token_types_count: 0 }, // [570]
    SimpleHintData { raw_values_start: 1075, raw_values_count: 1, token_types_start: 1076, token_types_count: 0 }, // [571]
    SimpleHintData { raw_values_start: 1076, raw_values_count: 1, token_types_start: 1077, token_types_count: 0 }, // [572]
    SimpleHintData { raw_values_start: 1077, raw_values_count: 1, token_types_start: 1078, token_types_count: 0 }, // [573]
    SimpleHintData { raw_values_start: 1078, raw_values_count: 1, token_types_start: 1079, token_types_count: 0 }, // [574]
    SimpleHintData { raw_values_start: 1079, raw_values_count: 1, token_types_start: 1080, token_types_count: 0 }, // [575]
    SimpleHintData { raw_values_start: 1080, raw_values_count: 1, token_types_start: 1081, token_types_count: 0 }, // [576]
    SimpleHintData { raw_values_start: 1081, raw_values_count: 1, token_types_start: 1082, token_types_count: 0 }, // [577]
    SimpleHintData { raw_values_start: 1082, raw_values_count: 1, token_types_start: 1083, token_types_count: 0 }, // [578]
    SimpleHintData { raw_values_start: 1083, raw_values_count: 1, token_types_start: 1084, token_types_count: 0 }, // [579]
    SimpleHintData { raw_values_start: 1084, raw_values_count: 1, token_types_start: 1085, token_types_count: 0 }, // [580]
    SimpleHintData { raw_values_start: 1085, raw_values_count: 1, token_types_start: 1086, token_types_count: 0 }, // [581]
    SimpleHintData { raw_values_start: 1086, raw_values_count: 1, token_types_start: 1087, token_types_count: 0 }, // [582]
    SimpleHintData { raw_values_start: 1087, raw_values_count: 1, token_types_start: 1088, token_types_count: 0 }, // [583]
    SimpleHintData { raw_values_start: 1088, raw_values_count: 1, token_types_start: 1089, token_types_count: 0 }, // [584]
    SimpleHintData { raw_values_start: 1089, raw_values_count: 1, token_types_start: 1090, token_types_count: 0 }, // [585]
    SimpleHintData { raw_values_start: 1090, raw_values_count: 1, token_types_start: 1091, token_types_count: 0 }, // [586]
    SimpleHintData { raw_values_start: 1091, raw_values_count: 1, token_types_start: 1092, token_types_count: 0 }, // [587]
    SimpleHintData { raw_values_start: 1092, raw_values_count: 1, token_types_start: 1093, token_types_count: 0 }, // [588]
    SimpleHintData { raw_values_start: 1093, raw_values_count: 1, token_types_start: 1094, token_types_count: 0 }, // [589]
    SimpleHintData { raw_values_start: 1094, raw_values_count: 1, token_types_start: 1095, token_types_count: 0 }, // [590]
    SimpleHintData { raw_values_start: 1095, raw_values_count: 1, token_types_start: 1096, token_types_count: 0 }, // [591]
    SimpleHintData { raw_values_start: 1096, raw_values_count: 1, token_types_start: 1097, token_types_count: 0 }, // [592]
    SimpleHintData { raw_values_start: 1097, raw_values_count: 2, token_types_start: 1099, token_types_count: 0 }, // [593]
    SimpleHintData { raw_values_start: 1099, raw_values_count: 1, token_types_start: 1100, token_types_count: 0 }, // [594]
    SimpleHintData { raw_values_start: 1100, raw_values_count: 1, token_types_start: 1101, token_types_count: 0 }, // [595]
    SimpleHintData { raw_values_start: 1101, raw_values_count: 1, token_types_start: 1102, token_types_count: 0 }, // [596]
    SimpleHintData { raw_values_start: 1102, raw_values_count: 1, token_types_start: 1103, token_types_count: 0 }, // [597]
    SimpleHintData { raw_values_start: 1103, raw_values_count: 1, token_types_start: 1104, token_types_count: 0 }, // [598]
    SimpleHintData { raw_values_start: 1104, raw_values_count: 1, token_types_start: 1105, token_types_count: 0 }, // [599]
    SimpleHintData { raw_values_start: 1105, raw_values_count: 1, token_types_start: 1106, token_types_count: 0 }, // [600]
    SimpleHintData { raw_values_start: 1106, raw_values_count: 1, token_types_start: 1107, token_types_count: 0 }, // [601]
    SimpleHintData { raw_values_start: 1107, raw_values_count: 1, token_types_start: 1108, token_types_count: 0 }, // [602]
    SimpleHintData { raw_values_start: 1108, raw_values_count: 1, token_types_start: 1109, token_types_count: 0 }, // [603]
    SimpleHintData { raw_values_start: 1109, raw_values_count: 1, token_types_start: 1110, token_types_count: 0 }, // [604]
    SimpleHintData { raw_values_start: 1110, raw_values_count: 1, token_types_start: 1111, token_types_count: 0 }, // [605]
    SimpleHintData { raw_values_start: 1111, raw_values_count: 1, token_types_start: 1112, token_types_count: 0 }, // [606]
    SimpleHintData { raw_values_start: 1112, raw_values_count: 9, token_types_start: 1121, token_types_count: 0 }, // [607]
    SimpleHintData { raw_values_start: 1121, raw_values_count: 1, token_types_start: 1122, token_types_count: 0 }, // [608]
    SimpleHintData { raw_values_start: 1122, raw_values_count: 1, token_types_start: 1123, token_types_count: 0 }, // [609]
    SimpleHintData { raw_values_start: 1123, raw_values_count: 1, token_types_start: 1124, token_types_count: 0 }, // [610]
    SimpleHintData { raw_values_start: 1124, raw_values_count: 1, token_types_start: 1125, token_types_count: 0 }, // [611]
    SimpleHintData { raw_values_start: 1125, raw_values_count: 1, token_types_start: 1126, token_types_count: 0 }, // [612]
    SimpleHintData { raw_values_start: 1126, raw_values_count: 1, token_types_start: 1127, token_types_count: 0 }, // [613]
    SimpleHintData { raw_values_start: 1127, raw_values_count: 1, token_types_start: 1128, token_types_count: 0 }, // [614]
    SimpleHintData { raw_values_start: 1128, raw_values_count: 1, token_types_start: 1129, token_types_count: 0 }, // [615]
    SimpleHintData { raw_values_start: 1129, raw_values_count: 1, token_types_start: 1130, token_types_count: 0 }, // [616]
    SimpleHintData { raw_values_start: 1130, raw_values_count: 1, token_types_start: 1131, token_types_count: 0 }, // [617]
    SimpleHintData { raw_values_start: 1131, raw_values_count: 1, token_types_start: 1132, token_types_count: 0 }, // [618]
    SimpleHintData { raw_values_start: 1132, raw_values_count: 1, token_types_start: 1133, token_types_count: 0 }, // [619]
    SimpleHintData { raw_values_start: 1133, raw_values_count: 0, token_types_start: 1133, token_types_count: 2 }, // [620]
    SimpleHintData { raw_values_start: 1135, raw_values_count: 11, token_types_start: 1146, token_types_count: 1 }, // [621]
    SimpleHintData { raw_values_start: 1147, raw_values_count: 1, token_types_start: 1148, token_types_count: 0 }, // [622]
    SimpleHintData { raw_values_start: 1148, raw_values_count: 1, token_types_start: 1149, token_types_count: 0 }, // [623]
    SimpleHintData { raw_values_start: 1149, raw_values_count: 1, token_types_start: 1150, token_types_count: 0 }, // [624]
    SimpleHintData { raw_values_start: 1150, raw_values_count: 1, token_types_start: 1151, token_types_count: 0 }, // [625]
    SimpleHintData { raw_values_start: 1151, raw_values_count: 1, token_types_start: 1152, token_types_count: 0 }, // [626]
    SimpleHintData { raw_values_start: 1152, raw_values_count: 1, token_types_start: 1153, token_types_count: 0 }, // [627]
    SimpleHintData { raw_values_start: 1153, raw_values_count: 1, token_types_start: 1154, token_types_count: 0 }, // [628]
    SimpleHintData { raw_values_start: 1154, raw_values_count: 2, token_types_start: 1156, token_types_count: 0 }, // [629]
    SimpleHintData { raw_values_start: 1156, raw_values_count: 1, token_types_start: 1157, token_types_count: 0 }, // [630]
    SimpleHintData { raw_values_start: 1157, raw_values_count: 3, token_types_start: 1160, token_types_count: 0 }, // [631]
    SimpleHintData { raw_values_start: 1160, raw_values_count: 1, token_types_start: 1161, token_types_count: 0 }, // [632]
    SimpleHintData { raw_values_start: 1161, raw_values_count: 1, token_types_start: 1162, token_types_count: 0 }, // [633]
    SimpleHintData { raw_values_start: 1162, raw_values_count: 1, token_types_start: 1163, token_types_count: 0 }, // [634]
    SimpleHintData { raw_values_start: 1163, raw_values_count: 1, token_types_start: 1164, token_types_count: 0 }, // [635]
    SimpleHintData { raw_values_start: 1164, raw_values_count: 1, token_types_start: 1165, token_types_count: 0 }, // [636]
    SimpleHintData { raw_values_start: 1165, raw_values_count: 1, token_types_start: 1166, token_types_count: 0 }, // [637]
    SimpleHintData { raw_values_start: 1166, raw_values_count: 1, token_types_start: 1167, token_types_count: 0 }, // [638]
    SimpleHintData { raw_values_start: 1167, raw_values_count: 1, token_types_start: 1168, token_types_count: 0 }, // [639]
    SimpleHintData { raw_values_start: 1168, raw_values_count: 1, token_types_start: 1169, token_types_count: 0 }, // [640]
    SimpleHintData { raw_values_start: 1169, raw_values_count: 1, token_types_start: 1170, token_types_count: 0 }, // [641]
    SimpleHintData { raw_values_start: 1170, raw_values_count: 1, token_types_start: 1171, token_types_count: 0 }, // [642]
    SimpleHintData { raw_values_start: 1171, raw_values_count: 1, token_types_start: 1172, token_types_count: 0 }, // [643]
    SimpleHintData { raw_values_start: 1172, raw_values_count: 1, token_types_start: 1173, token_types_count: 0 }, // [644]
    SimpleHintData { raw_values_start: 1173, raw_values_count: 1, token_types_start: 1174, token_types_count: 0 }, // [645]
    SimpleHintData { raw_values_start: 1174, raw_values_count: 1, token_types_start: 1175, token_types_count: 0 }, // [646]
    SimpleHintData { raw_values_start: 1175, raw_values_count: 1, token_types_start: 1176, token_types_count: 0 }, // [647]
    SimpleHintData { raw_values_start: 1176, raw_values_count: 1, token_types_start: 1177, token_types_count: 0 }, // [648]
    SimpleHintData { raw_values_start: 1177, raw_values_count: 1, token_types_start: 1178, token_types_count: 0 }, // [649]
    SimpleHintData { raw_values_start: 1178, raw_values_count: 1, token_types_start: 1179, token_types_count: 0 }, // [650]
    SimpleHintData { raw_values_start: 1179, raw_values_count: 1, token_types_start: 1180, token_types_count: 0 }, // [651]
    SimpleHintData { raw_values_start: 1180, raw_values_count: 3, token_types_start: 1183, token_types_count: 0 }, // [652]
    SimpleHintData { raw_values_start: 1183, raw_values_count: 1, token_types_start: 1184, token_types_count: 0 }, // [653]
    SimpleHintData { raw_values_start: 1184, raw_values_count: 2, token_types_start: 1186, token_types_count: 0 }, // [654]
    SimpleHintData { raw_values_start: 1186, raw_values_count: 1, token_types_start: 1187, token_types_count: 0 }, // [655]
    SimpleHintData { raw_values_start: 1187, raw_values_count: 1, token_types_start: 1188, token_types_count: 0 }, // [656]
    SimpleHintData { raw_values_start: 1188, raw_values_count: 1, token_types_start: 1189, token_types_count: 0 }, // [657]
    SimpleHintData { raw_values_start: 1189, raw_values_count: 1, token_types_start: 1190, token_types_count: 0 }, // [658]
    SimpleHintData { raw_values_start: 1190, raw_values_count: 1, token_types_start: 1191, token_types_count: 0 }, // [659]
    SimpleHintData { raw_values_start: 1191, raw_values_count: 1, token_types_start: 1192, token_types_count: 0 }, // [660]
    SimpleHintData { raw_values_start: 1192, raw_values_count: 1, token_types_start: 1193, token_types_count: 0 }, // [661]
    SimpleHintData { raw_values_start: 1193, raw_values_count: 1, token_types_start: 1194, token_types_count: 0 }, // [662]
    SimpleHintData { raw_values_start: 1194, raw_values_count: 1, token_types_start: 1195, token_types_count: 0 }, // [663]
    SimpleHintData { raw_values_start: 1195, raw_values_count: 1, token_types_start: 1196, token_types_count: 0 }, // [664]
    SimpleHintData { raw_values_start: 1196, raw_values_count: 1, token_types_start: 1197, token_types_count: 0 }, // [665]
    SimpleHintData { raw_values_start: 1197, raw_values_count: 1, token_types_start: 1198, token_types_count: 0 }, // [666]
    SimpleHintData { raw_values_start: 1198, raw_values_count: 2, token_types_start: 1200, token_types_count: 0 }, // [667]
    SimpleHintData { raw_values_start: 1200, raw_values_count: 1, token_types_start: 1201, token_types_count: 0 }, // [668]
    SimpleHintData { raw_values_start: 1201, raw_values_count: 4, token_types_start: 1205, token_types_count: 0 }, // [669]
    SimpleHintData { raw_values_start: 1205, raw_values_count: 2, token_types_start: 1207, token_types_count: 0 }, // [670]
    SimpleHintData { raw_values_start: 1207, raw_values_count: 8, token_types_start: 1215, token_types_count: 0 }, // [671]
    SimpleHintData { raw_values_start: 1215, raw_values_count: 1, token_types_start: 1216, token_types_count: 0 }, // [672]
    SimpleHintData { raw_values_start: 1216, raw_values_count: 1, token_types_start: 1217, token_types_count: 0 }, // [673]
    SimpleHintData { raw_values_start: 1217, raw_values_count: 1, token_types_start: 1218, token_types_count: 0 }, // [674]
    SimpleHintData { raw_values_start: 1218, raw_values_count: 1, token_types_start: 1219, token_types_count: 0 }, // [675]
    SimpleHintData { raw_values_start: 1219, raw_values_count: 1, token_types_start: 1220, token_types_count: 0 }, // [676]
    SimpleHintData { raw_values_start: 1220, raw_values_count: 1, token_types_start: 1221, token_types_count: 0 }, // [677]
    SimpleHintData { raw_values_start: 1221, raw_values_count: 1, token_types_start: 1222, token_types_count: 0 }, // [678]
    SimpleHintData { raw_values_start: 1222, raw_values_count: 1, token_types_start: 1223, token_types_count: 0 }, // [679]
    SimpleHintData { raw_values_start: 1223, raw_values_count: 1, token_types_start: 1224, token_types_count: 0 }, // [680]
    SimpleHintData { raw_values_start: 1224, raw_values_count: 1, token_types_start: 1225, token_types_count: 0 }, // [681]
    SimpleHintData { raw_values_start: 1225, raw_values_count: 1, token_types_start: 1226, token_types_count: 0 }, // [682]
    SimpleHintData { raw_values_start: 1226, raw_values_count: 1, token_types_start: 1227, token_types_count: 0 }, // [683]
    SimpleHintData { raw_values_start: 1227, raw_values_count: 1, token_types_start: 1228, token_types_count: 0 }, // [684]
    SimpleHintData { raw_values_start: 1228, raw_values_count: 1, token_types_start: 1229, token_types_count: 0 }, // [685]
    SimpleHintData { raw_values_start: 1229, raw_values_count: 1, token_types_start: 1230, token_types_count: 0 }, // [686]
    SimpleHintData { raw_values_start: 1230, raw_values_count: 1, token_types_start: 1231, token_types_count: 0 }, // [687]
    SimpleHintData { raw_values_start: 1231, raw_values_count: 1, token_types_start: 1232, token_types_count: 0 }, // [688]
    SimpleHintData { raw_values_start: 1232, raw_values_count: 1, token_types_start: 1233, token_types_count: 0 }, // [689]
    SimpleHintData { raw_values_start: 1233, raw_values_count: 1, token_types_start: 1234, token_types_count: 0 }, // [690]
    SimpleHintData { raw_values_start: 1234, raw_values_count: 1, token_types_start: 1235, token_types_count: 0 }, // [691]
    SimpleHintData { raw_values_start: 1235, raw_values_count: 1, token_types_start: 1236, token_types_count: 0 }, // [692]
    SimpleHintData { raw_values_start: 1236, raw_values_count: 1, token_types_start: 1237, token_types_count: 0 }, // [693]
    SimpleHintData { raw_values_start: 1237, raw_values_count: 1, token_types_start: 1238, token_types_count: 0 }, // [694]
    SimpleHintData { raw_values_start: 1238, raw_values_count: 1, token_types_start: 1239, token_types_count: 0 }, // [695]
    SimpleHintData { raw_values_start: 1239, raw_values_count: 1, token_types_start: 1240, token_types_count: 0 }, // [696]
    SimpleHintData { raw_values_start: 1240, raw_values_count: 1, token_types_start: 1241, token_types_count: 0 }, // [697]
    SimpleHintData { raw_values_start: 1241, raw_values_count: 1, token_types_start: 1242, token_types_count: 0 }, // [698]
    SimpleHintData { raw_values_start: 1242, raw_values_count: 1, token_types_start: 1243, token_types_count: 0 }, // [699]
    SimpleHintData { raw_values_start: 1243, raw_values_count: 1, token_types_start: 1244, token_types_count: 0 }, // [700]
    SimpleHintData { raw_values_start: 1244, raw_values_count: 2, token_types_start: 1246, token_types_count: 0 }, // [701]
    SimpleHintData { raw_values_start: 1246, raw_values_count: 1, token_types_start: 1247, token_types_count: 0 }, // [702]
    SimpleHintData { raw_values_start: 1247, raw_values_count: 1, token_types_start: 1248, token_types_count: 0 }, // [703]
    SimpleHintData { raw_values_start: 1248, raw_values_count: 1, token_types_start: 1249, token_types_count: 0 }, // [704]
    SimpleHintData { raw_values_start: 1249, raw_values_count: 1, token_types_start: 1250, token_types_count: 0 }, // [705]
    SimpleHintData { raw_values_start: 1250, raw_values_count: 1, token_types_start: 1251, token_types_count: 0 }, // [706]
    SimpleHintData { raw_values_start: 1251, raw_values_count: 1, token_types_start: 1252, token_types_count: 0 }, // [707]
    SimpleHintData { raw_values_start: 1252, raw_values_count: 1, token_types_start: 1253, token_types_count: 0 }, // [708]
    SimpleHintData { raw_values_start: 1253, raw_values_count: 1, token_types_start: 1254, token_types_count: 0 }, // [709]
    SimpleHintData { raw_values_start: 1254, raw_values_count: 1, token_types_start: 1255, token_types_count: 0 }, // [710]
    SimpleHintData { raw_values_start: 1255, raw_values_count: 1, token_types_start: 1256, token_types_count: 0 }, // [711]
    SimpleHintData { raw_values_start: 1256, raw_values_count: 1, token_types_start: 1257, token_types_count: 0 }, // [712]
    SimpleHintData { raw_values_start: 1257, raw_values_count: 1, token_types_start: 1258, token_types_count: 0 }, // [713]
    SimpleHintData { raw_values_start: 1258, raw_values_count: 1, token_types_start: 1259, token_types_count: 0 }, // [714]
    SimpleHintData { raw_values_start: 1259, raw_values_count: 1, token_types_start: 1260, token_types_count: 0 }, // [715]
    SimpleHintData { raw_values_start: 1260, raw_values_count: 1, token_types_start: 1261, token_types_count: 0 }, // [716]
    SimpleHintData { raw_values_start: 1261, raw_values_count: 1, token_types_start: 1262, token_types_count: 0 }, // [717]
    SimpleHintData { raw_values_start: 1262, raw_values_count: 1, token_types_start: 1263, token_types_count: 0 }, // [718]
    SimpleHintData { raw_values_start: 1263, raw_values_count: 1, token_types_start: 1264, token_types_count: 0 }, // [719]
    SimpleHintData { raw_values_start: 1264, raw_values_count: 1, token_types_start: 1265, token_types_count: 0 }, // [720]
    SimpleHintData { raw_values_start: 1265, raw_values_count: 1, token_types_start: 1266, token_types_count: 0 }, // [721]
    SimpleHintData { raw_values_start: 1266, raw_values_count: 1, token_types_start: 1267, token_types_count: 0 }, // [722]
    SimpleHintData { raw_values_start: 1267, raw_values_count: 1, token_types_start: 1268, token_types_count: 0 }, // [723]
    SimpleHintData { raw_values_start: 1268, raw_values_count: 1, token_types_start: 1269, token_types_count: 0 }, // [724]
    SimpleHintData { raw_values_start: 1269, raw_values_count: 1, token_types_start: 1270, token_types_count: 0 }, // [725]
    SimpleHintData { raw_values_start: 1270, raw_values_count: 1, token_types_start: 1271, token_types_count: 0 }, // [726]
    SimpleHintData { raw_values_start: 1271, raw_values_count: 1, token_types_start: 1272, token_types_count: 0 }, // [727]
    SimpleHintData { raw_values_start: 1272, raw_values_count: 1, token_types_start: 1273, token_types_count: 0 }, // [728]
    SimpleHintData { raw_values_start: 1273, raw_values_count: 1, token_types_start: 1274, token_types_count: 0 }, // [729]
    SimpleHintData { raw_values_start: 1274, raw_values_count: 1, token_types_start: 1275, token_types_count: 0 }, // [730]
    SimpleHintData { raw_values_start: 1275, raw_values_count: 1, token_types_start: 1276, token_types_count: 0 }, // [731]
    SimpleHintData { raw_values_start: 1276, raw_values_count: 1, token_types_start: 1277, token_types_count: 0 }, // [732]
    SimpleHintData { raw_values_start: 1277, raw_values_count: 1, token_types_start: 1278, token_types_count: 0 }, // [733]
    SimpleHintData { raw_values_start: 1278, raw_values_count: 1, token_types_start: 1279, token_types_count: 0 }, // [734]
    SimpleHintData { raw_values_start: 1279, raw_values_count: 1, token_types_start: 1280, token_types_count: 0 }, // [735]
    SimpleHintData { raw_values_start: 1280, raw_values_count: 1, token_types_start: 1281, token_types_count: 0 }, // [736]
    SimpleHintData { raw_values_start: 1281, raw_values_count: 1, token_types_start: 1282, token_types_count: 0 }, // [737]
    SimpleHintData { raw_values_start: 1282, raw_values_count: 1, token_types_start: 1283, token_types_count: 0 }, // [738]
    SimpleHintData { raw_values_start: 1283, raw_values_count: 1, token_types_start: 1284, token_types_count: 0 }, // [739]
    SimpleHintData { raw_values_start: 1284, raw_values_count: 1, token_types_start: 1285, token_types_count: 0 }, // [740]
    SimpleHintData { raw_values_start: 1285, raw_values_count: 1, token_types_start: 1286, token_types_count: 0 }, // [741]
    SimpleHintData { raw_values_start: 1286, raw_values_count: 2, token_types_start: 1288, token_types_count: 0 }, // [742]
    SimpleHintData { raw_values_start: 1288, raw_values_count: 3, token_types_start: 1291, token_types_count: 0 }, // [743]
    SimpleHintData { raw_values_start: 1291, raw_values_count: 5, token_types_start: 1296, token_types_count: 0 }, // [744]
    SimpleHintData { raw_values_start: 1296, raw_values_count: 7, token_types_start: 1303, token_types_count: 0 }, // [745]
    SimpleHintData { raw_values_start: 1303, raw_values_count: 5, token_types_start: 1308, token_types_count: 0 }, // [746]
    SimpleHintData { raw_values_start: 1308, raw_values_count: 1, token_types_start: 1309, token_types_count: 0 }, // [747]
    SimpleHintData { raw_values_start: 1309, raw_values_count: 1, token_types_start: 1310, token_types_count: 0 }, // [748]
    SimpleHintData { raw_values_start: 1310, raw_values_count: 1, token_types_start: 1311, token_types_count: 0 }, // [749]
    SimpleHintData { raw_values_start: 1311, raw_values_count: 1, token_types_start: 1312, token_types_count: 0 }, // [750]
    SimpleHintData { raw_values_start: 1312, raw_values_count: 1, token_types_start: 1313, token_types_count: 0 }, // [751]
    SimpleHintData { raw_values_start: 1313, raw_values_count: 1, token_types_start: 1314, token_types_count: 0 }, // [752]
    SimpleHintData { raw_values_start: 1314, raw_values_count: 1, token_types_start: 1315, token_types_count: 0 }, // [753]
    SimpleHintData { raw_values_start: 1315, raw_values_count: 1, token_types_start: 1316, token_types_count: 0 }, // [754]
    SimpleHintData { raw_values_start: 1316, raw_values_count: 1, token_types_start: 1317, token_types_count: 0 }, // [755]
    SimpleHintData { raw_values_start: 1317, raw_values_count: 1, token_types_start: 1318, token_types_count: 0 }, // [756]
    SimpleHintData { raw_values_start: 1318, raw_values_count: 1, token_types_start: 1319, token_types_count: 0 }, // [757]
    SimpleHintData { raw_values_start: 1319, raw_values_count: 1, token_types_start: 1320, token_types_count: 0 }, // [758]
    SimpleHintData { raw_values_start: 1320, raw_values_count: 1, token_types_start: 1321, token_types_count: 0 }, // [759]
    SimpleHintData { raw_values_start: 1321, raw_values_count: 1, token_types_start: 1322, token_types_count: 0 }, // [760]
    SimpleHintData { raw_values_start: 1322, raw_values_count: 1, token_types_start: 1323, token_types_count: 0 }, // [761]
    SimpleHintData { raw_values_start: 1323, raw_values_count: 1, token_types_start: 1324, token_types_count: 0 }, // [762]
    SimpleHintData { raw_values_start: 1324, raw_values_count: 1, token_types_start: 1325, token_types_count: 0 }, // [763]
    SimpleHintData { raw_values_start: 1325, raw_values_count: 1, token_types_start: 1326, token_types_count: 0 }, // [764]
    SimpleHintData { raw_values_start: 1326, raw_values_count: 1, token_types_start: 1327, token_types_count: 0 }, // [765]
    SimpleHintData { raw_values_start: 1327, raw_values_count: 1, token_types_start: 1328, token_types_count: 0 }, // [766]
    SimpleHintData { raw_values_start: 1328, raw_values_count: 1, token_types_start: 1329, token_types_count: 0 }, // [767]
    SimpleHintData { raw_values_start: 1329, raw_values_count: 1, token_types_start: 1330, token_types_count: 0 }, // [768]
    SimpleHintData { raw_values_start: 1330, raw_values_count: 1, token_types_start: 1331, token_types_count: 0 }, // [769]
    SimpleHintData { raw_values_start: 1331, raw_values_count: 1, token_types_start: 1332, token_types_count: 0 }, // [770]
    SimpleHintData { raw_values_start: 1332, raw_values_count: 1, token_types_start: 1333, token_types_count: 0 }, // [771]
    SimpleHintData { raw_values_start: 1333, raw_values_count: 1, token_types_start: 1334, token_types_count: 0 }, // [772]
    SimpleHintData { raw_values_start: 1334, raw_values_count: 1, token_types_start: 1335, token_types_count: 0 }, // [773]
    SimpleHintData { raw_values_start: 1335, raw_values_count: 1, token_types_start: 1336, token_types_count: 0 }, // [774]
    SimpleHintData { raw_values_start: 1336, raw_values_count: 1, token_types_start: 1337, token_types_count: 0 }, // [775]
    SimpleHintData { raw_values_start: 1337, raw_values_count: 1, token_types_start: 1338, token_types_count: 0 }, // [776]
    SimpleHintData { raw_values_start: 1338, raw_values_count: 1, token_types_start: 1339, token_types_count: 0 }, // [777]
    SimpleHintData { raw_values_start: 1339, raw_values_count: 2, token_types_start: 1341, token_types_count: 0 }, // [778]
    SimpleHintData { raw_values_start: 1341, raw_values_count: 8, token_types_start: 1349, token_types_count: 0 }, // [779]
    SimpleHintData { raw_values_start: 1349, raw_values_count: 1, token_types_start: 1350, token_types_count: 0 }, // [780]
    SimpleHintData { raw_values_start: 1350, raw_values_count: 1, token_types_start: 1351, token_types_count: 0 }, // [781]
    SimpleHintData { raw_values_start: 1351, raw_values_count: 1, token_types_start: 1352, token_types_count: 0 }, // [782]
    SimpleHintData { raw_values_start: 1352, raw_values_count: 1, token_types_start: 1353, token_types_count: 0 }, // [783]
    SimpleHintData { raw_values_start: 1353, raw_values_count: 1, token_types_start: 1354, token_types_count: 0 }, // [784]
    SimpleHintData { raw_values_start: 1354, raw_values_count: 1, token_types_start: 1355, token_types_count: 0 }, // [785]
    SimpleHintData { raw_values_start: 1355, raw_values_count: 1, token_types_start: 1356, token_types_count: 0 }, // [786]
    SimpleHintData { raw_values_start: 1356, raw_values_count: 1, token_types_start: 1357, token_types_count: 0 }, // [787]
    SimpleHintData { raw_values_start: 1357, raw_values_count: 1, token_types_start: 1358, token_types_count: 0 }, // [788]
    SimpleHintData { raw_values_start: 1358, raw_values_count: 1, token_types_start: 1359, token_types_count: 0 }, // [789]
    SimpleHintData { raw_values_start: 1359, raw_values_count: 1, token_types_start: 1360, token_types_count: 0 }, // [790]
    SimpleHintData { raw_values_start: 1360, raw_values_count: 1, token_types_start: 1361, token_types_count: 0 }, // [791]
    SimpleHintData { raw_values_start: 1361, raw_values_count: 1, token_types_start: 1362, token_types_count: 0 }, // [792]
    SimpleHintData { raw_values_start: 1362, raw_values_count: 1, token_types_start: 1363, token_types_count: 0 }, // [793]
    SimpleHintData { raw_values_start: 1363, raw_values_count: 1, token_types_start: 1364, token_types_count: 0 }, // [794]
    SimpleHintData { raw_values_start: 1364, raw_values_count: 1, token_types_start: 1365, token_types_count: 0 }, // [795]
    SimpleHintData { raw_values_start: 1365, raw_values_count: 1, token_types_start: 1366, token_types_count: 0 }, // [796]
    SimpleHintData { raw_values_start: 1366, raw_values_count: 1, token_types_start: 1367, token_types_count: 0 }, // [797]
    SimpleHintData { raw_values_start: 1367, raw_values_count: 1, token_types_start: 1368, token_types_count: 0 }, // [798]
    SimpleHintData { raw_values_start: 1368, raw_values_count: 1, token_types_start: 1369, token_types_count: 0 }, // [799]
    SimpleHintData { raw_values_start: 1369, raw_values_count: 1, token_types_start: 1370, token_types_count: 0 }, // [800]
    SimpleHintData { raw_values_start: 1370, raw_values_count: 1, token_types_start: 1371, token_types_count: 0 }, // [801]
    SimpleHintData { raw_values_start: 1371, raw_values_count: 1, token_types_start: 1372, token_types_count: 0 }, // [802]
    SimpleHintData { raw_values_start: 1372, raw_values_count: 1, token_types_start: 1373, token_types_count: 0 }, // [803]
    SimpleHintData { raw_values_start: 1373, raw_values_count: 1, token_types_start: 1374, token_types_count: 0 }, // [804]
    SimpleHintData { raw_values_start: 1374, raw_values_count: 1, token_types_start: 1375, token_types_count: 0 }, // [805]
    SimpleHintData { raw_values_start: 1375, raw_values_count: 1, token_types_start: 1376, token_types_count: 1 }, // [806]
    SimpleHintData { raw_values_start: 1377, raw_values_count: 1, token_types_start: 1378, token_types_count: 0 }, // [807]
    SimpleHintData { raw_values_start: 1378, raw_values_count: 1, token_types_start: 1379, token_types_count: 0 }, // [808]
    SimpleHintData { raw_values_start: 1379, raw_values_count: 1, token_types_start: 1380, token_types_count: 0 }, // [809]
    SimpleHintData { raw_values_start: 1380, raw_values_count: 1, token_types_start: 1381, token_types_count: 0 }, // [810]
    SimpleHintData { raw_values_start: 1381, raw_values_count: 1, token_types_start: 1382, token_types_count: 0 }, // [811]
    SimpleHintData { raw_values_start: 1382, raw_values_count: 1, token_types_start: 1383, token_types_count: 0 }, // [812]
    SimpleHintData { raw_values_start: 1383, raw_values_count: 1, token_types_start: 1384, token_types_count: 0 }, // [813]
    SimpleHintData { raw_values_start: 1384, raw_values_count: 1, token_types_start: 1385, token_types_count: 0 }, // [814]
    SimpleHintData { raw_values_start: 1385, raw_values_count: 1, token_types_start: 1386, token_types_count: 0 }, // [815]
    SimpleHintData { raw_values_start: 1386, raw_values_count: 1, token_types_start: 1387, token_types_count: 0 }, // [816]
    SimpleHintData { raw_values_start: 1387, raw_values_count: 1, token_types_start: 1388, token_types_count: 0 }, // [817]
    SimpleHintData { raw_values_start: 1388, raw_values_count: 1, token_types_start: 1389, token_types_count: 0 }, // [818]
    SimpleHintData { raw_values_start: 1389, raw_values_count: 1, token_types_start: 1390, token_types_count: 0 }, // [819]
    SimpleHintData { raw_values_start: 1390, raw_values_count: 1, token_types_start: 1391, token_types_count: 0 }, // [820]
    SimpleHintData { raw_values_start: 1391, raw_values_count: 1, token_types_start: 1392, token_types_count: 0 }, // [821]
    SimpleHintData { raw_values_start: 1392, raw_values_count: 1, token_types_start: 1393, token_types_count: 0 }, // [822]
    SimpleHintData { raw_values_start: 1393, raw_values_count: 1, token_types_start: 1394, token_types_count: 0 }, // [823]
    SimpleHintData { raw_values_start: 1394, raw_values_count: 1, token_types_start: 1395, token_types_count: 0 }, // [824]
    SimpleHintData { raw_values_start: 1395, raw_values_count: 1, token_types_start: 1396, token_types_count: 0 }, // [825]
    SimpleHintData { raw_values_start: 1396, raw_values_count: 1, token_types_start: 1397, token_types_count: 0 }, // [826]
    SimpleHintData { raw_values_start: 1397, raw_values_count: 1, token_types_start: 1398, token_types_count: 0 }, // [827]
    SimpleHintData { raw_values_start: 1398, raw_values_count: 1, token_types_start: 1399, token_types_count: 0 }, // [828]
    SimpleHintData { raw_values_start: 1399, raw_values_count: 1, token_types_start: 1400, token_types_count: 0 }, // [829]
    SimpleHintData { raw_values_start: 1400, raw_values_count: 1, token_types_start: 1401, token_types_count: 0 }, // [830]
    SimpleHintData { raw_values_start: 1401, raw_values_count: 1, token_types_start: 1402, token_types_count: 0 }, // [831]
    SimpleHintData { raw_values_start: 1402, raw_values_count: 3, token_types_start: 1405, token_types_count: 0 }, // [832]
    SimpleHintData { raw_values_start: 1405, raw_values_count: 1, token_types_start: 1406, token_types_count: 0 }, // [833]
    SimpleHintData { raw_values_start: 1406, raw_values_count: 1, token_types_start: 1407, token_types_count: 0 }, // [834]
    SimpleHintData { raw_values_start: 1407, raw_values_count: 1, token_types_start: 1408, token_types_count: 0 }, // [835]
    SimpleHintData { raw_values_start: 1408, raw_values_count: 1, token_types_start: 1409, token_types_count: 0 }, // [836]
    SimpleHintData { raw_values_start: 1409, raw_values_count: 1, token_types_start: 1410, token_types_count: 0 }, // [837]
    SimpleHintData { raw_values_start: 1410, raw_values_count: 1, token_types_start: 1411, token_types_count: 0 }, // [838]
    SimpleHintData { raw_values_start: 1411, raw_values_count: 1, token_types_start: 1412, token_types_count: 0 }, // [839]
    SimpleHintData { raw_values_start: 1412, raw_values_count: 1, token_types_start: 1413, token_types_count: 0 }, // [840]
    SimpleHintData { raw_values_start: 1413, raw_values_count: 1, token_types_start: 1414, token_types_count: 0 }, // [841]
    SimpleHintData { raw_values_start: 1414, raw_values_count: 1, token_types_start: 1415, token_types_count: 0 }, // [842]
    SimpleHintData { raw_values_start: 1415, raw_values_count: 1, token_types_start: 1416, token_types_count: 0 }, // [843]
    SimpleHintData { raw_values_start: 1416, raw_values_count: 1, token_types_start: 1417, token_types_count: 0 }, // [844]
    SimpleHintData { raw_values_start: 1417, raw_values_count: 1, token_types_start: 1418, token_types_count: 0 }, // [845]
    SimpleHintData { raw_values_start: 1418, raw_values_count: 1, token_types_start: 1419, token_types_count: 0 }, // [846]
    SimpleHintData { raw_values_start: 1419, raw_values_count: 1, token_types_start: 1420, token_types_count: 0 }, // [847]
    SimpleHintData { raw_values_start: 1420, raw_values_count: 4, token_types_start: 1424, token_types_count: 0 }, // [848]
    SimpleHintData { raw_values_start: 1424, raw_values_count: 3, token_types_start: 1427, token_types_count: 0 }, // [849]
    SimpleHintData { raw_values_start: 1427, raw_values_count: 1, token_types_start: 1428, token_types_count: 0 }, // [850]
    SimpleHintData { raw_values_start: 1428, raw_values_count: 1, token_types_start: 1429, token_types_count: 0 }, // [851]
    SimpleHintData { raw_values_start: 1429, raw_values_count: 1, token_types_start: 1430, token_types_count: 0 }, // [852]
    SimpleHintData { raw_values_start: 1430, raw_values_count: 1, token_types_start: 1431, token_types_count: 0 }, // [853]
    SimpleHintData { raw_values_start: 1431, raw_values_count: 1, token_types_start: 1432, token_types_count: 0 }, // [854]
    SimpleHintData { raw_values_start: 1432, raw_values_count: 1, token_types_start: 1433, token_types_count: 0 }, // [855]
    SimpleHintData { raw_values_start: 1433, raw_values_count: 1, token_types_start: 1434, token_types_count: 0 }, // [856]
    SimpleHintData { raw_values_start: 1434, raw_values_count: 1, token_types_start: 1435, token_types_count: 0 }, // [857]
    SimpleHintData { raw_values_start: 1435, raw_values_count: 1, token_types_start: 1436, token_types_count: 0 }, // [858]
    SimpleHintData { raw_values_start: 1436, raw_values_count: 1, token_types_start: 1437, token_types_count: 0 }, // [859]
    SimpleHintData { raw_values_start: 1437, raw_values_count: 1, token_types_start: 1438, token_types_count: 0 }, // [860]
    SimpleHintData { raw_values_start: 1438, raw_values_count: 1, token_types_start: 1439, token_types_count: 0 }, // [861]
    SimpleHintData { raw_values_start: 1439, raw_values_count: 1, token_types_start: 1440, token_types_count: 0 }, // [862]
    SimpleHintData { raw_values_start: 1440, raw_values_count: 1, token_types_start: 1441, token_types_count: 0 }, // [863]
    SimpleHintData { raw_values_start: 1441, raw_values_count: 1, token_types_start: 1442, token_types_count: 0 }, // [864]
    SimpleHintData { raw_values_start: 1442, raw_values_count: 1, token_types_start: 1443, token_types_count: 0 }, // [865]
    SimpleHintData { raw_values_start: 1443, raw_values_count: 1, token_types_start: 1444, token_types_count: 0 }, // [866]
    SimpleHintData { raw_values_start: 1444, raw_values_count: 1, token_types_start: 1445, token_types_count: 0 }, // [867]
    SimpleHintData { raw_values_start: 1445, raw_values_count: 1, token_types_start: 1446, token_types_count: 0 }, // [868]
    SimpleHintData { raw_values_start: 1446, raw_values_count: 1, token_types_start: 1447, token_types_count: 0 }, // [869]
    SimpleHintData { raw_values_start: 1447, raw_values_count: 1, token_types_start: 1448, token_types_count: 0 }, // [870]
    SimpleHintData { raw_values_start: 1448, raw_values_count: 1, token_types_start: 1449, token_types_count: 0 }, // [871]
    SimpleHintData { raw_values_start: 1449, raw_values_count: 1, token_types_start: 1450, token_types_count: 0 }, // [872]
    SimpleHintData { raw_values_start: 1450, raw_values_count: 1, token_types_start: 1451, token_types_count: 0 }, // [873]
    SimpleHintData { raw_values_start: 1451, raw_values_count: 1, token_types_start: 1452, token_types_count: 0 }, // [874]
    SimpleHintData { raw_values_start: 1452, raw_values_count: 1, token_types_start: 1453, token_types_count: 0 }, // [875]
    SimpleHintData { raw_values_start: 1453, raw_values_count: 1, token_types_start: 1454, token_types_count: 0 }, // [876]
    SimpleHintData { raw_values_start: 1454, raw_values_count: 1, token_types_start: 1455, token_types_count: 0 }, // [877]
    SimpleHintData { raw_values_start: 1455, raw_values_count: 1, token_types_start: 1456, token_types_count: 0 }, // [878]
    SimpleHintData { raw_values_start: 1456, raw_values_count: 1, token_types_start: 1457, token_types_count: 0 }, // [879]
    SimpleHintData { raw_values_start: 1457, raw_values_count: 1, token_types_start: 1458, token_types_count: 0 }, // [880]
    SimpleHintData { raw_values_start: 1458, raw_values_count: 1, token_types_start: 1459, token_types_count: 0 }, // [881]
    SimpleHintData { raw_values_start: 1459, raw_values_count: 1, token_types_start: 1460, token_types_count: 0 }, // [882]
    SimpleHintData { raw_values_start: 1460, raw_values_count: 1, token_types_start: 1461, token_types_count: 0 }, // [883]
    SimpleHintData { raw_values_start: 1461, raw_values_count: 1, token_types_start: 1462, token_types_count: 0 }, // [884]
    SimpleHintData { raw_values_start: 1462, raw_values_count: 1, token_types_start: 1463, token_types_count: 0 }, // [885]
    SimpleHintData { raw_values_start: 1463, raw_values_count: 1, token_types_start: 1464, token_types_count: 0 }, // [886]
    SimpleHintData { raw_values_start: 1464, raw_values_count: 1, token_types_start: 1465, token_types_count: 0 }, // [887]
    SimpleHintData { raw_values_start: 1465, raw_values_count: 1, token_types_start: 1466, token_types_count: 0 }, // [888]
    SimpleHintData { raw_values_start: 1466, raw_values_count: 1, token_types_start: 1467, token_types_count: 0 }, // [889]
    SimpleHintData { raw_values_start: 1467, raw_values_count: 1, token_types_start: 1468, token_types_count: 0 }, // [890]
    SimpleHintData { raw_values_start: 1468, raw_values_count: 1, token_types_start: 1469, token_types_count: 0 }, // [891]
    SimpleHintData { raw_values_start: 1469, raw_values_count: 2, token_types_start: 1471, token_types_count: 0 }, // [892]
    SimpleHintData { raw_values_start: 1471, raw_values_count: 1, token_types_start: 1472, token_types_count: 0 }, // [893]
    SimpleHintData { raw_values_start: 1472, raw_values_count: 1, token_types_start: 1473, token_types_count: 0 }, // [894]
    SimpleHintData { raw_values_start: 1473, raw_values_count: 1, token_types_start: 1474, token_types_count: 0 }, // [895]
    SimpleHintData { raw_values_start: 1474, raw_values_count: 1, token_types_start: 1475, token_types_count: 0 }, // [896]
    SimpleHintData { raw_values_start: 1475, raw_values_count: 1, token_types_start: 1476, token_types_count: 0 }, // [897]
    SimpleHintData { raw_values_start: 1476, raw_values_count: 1, token_types_start: 1477, token_types_count: 0 }, // [898]
    SimpleHintData { raw_values_start: 1477, raw_values_count: 1, token_types_start: 1478, token_types_count: 0 }, // [899]
    SimpleHintData { raw_values_start: 1478, raw_values_count: 1, token_types_start: 1479, token_types_count: 0 }, // [900]
    SimpleHintData { raw_values_start: 1479, raw_values_count: 1, token_types_start: 1480, token_types_count: 0 }, // [901]
    SimpleHintData { raw_values_start: 1480, raw_values_count: 1, token_types_start: 1481, token_types_count: 0 }, // [902]
    SimpleHintData { raw_values_start: 1481, raw_values_count: 1, token_types_start: 1482, token_types_count: 0 }, // [903]
    SimpleHintData { raw_values_start: 1482, raw_values_count: 1, token_types_start: 1483, token_types_count: 0 }, // [904]
    SimpleHintData { raw_values_start: 1483, raw_values_count: 1, token_types_start: 1484, token_types_count: 0 }, // [905]
    SimpleHintData { raw_values_start: 1484, raw_values_count: 1, token_types_start: 1485, token_types_count: 0 }, // [906]
    SimpleHintData { raw_values_start: 1485, raw_values_count: 1, token_types_start: 1486, token_types_count: 0 }, // [907]
    SimpleHintData { raw_values_start: 1486, raw_values_count: 1, token_types_start: 1487, token_types_count: 0 }, // [908]
    SimpleHintData { raw_values_start: 1487, raw_values_count: 4, token_types_start: 1491, token_types_count: 0 }, // [909]
    SimpleHintData { raw_values_start: 1491, raw_values_count: 1, token_types_start: 1492, token_types_count: 0 }, // [910]
    SimpleHintData { raw_values_start: 1492, raw_values_count: 10, token_types_start: 1502, token_types_count: 0 }, // [911]
    SimpleHintData { raw_values_start: 1502, raw_values_count: 4, token_types_start: 1506, token_types_count: 0 }, // [912]
    SimpleHintData { raw_values_start: 1506, raw_values_count: 2, token_types_start: 1508, token_types_count: 0 }, // [913]
    SimpleHintData { raw_values_start: 1508, raw_values_count: 1, token_types_start: 1509, token_types_count: 0 }, // [914]
    SimpleHintData { raw_values_start: 1509, raw_values_count: 1, token_types_start: 1510, token_types_count: 0 }, // [915]
    SimpleHintData { raw_values_start: 1510, raw_values_count: 1, token_types_start: 1511, token_types_count: 0 }, // [916]
    SimpleHintData { raw_values_start: 1511, raw_values_count: 1, token_types_start: 1512, token_types_count: 0 }, // [917]
    SimpleHintData { raw_values_start: 1512, raw_values_count: 1, token_types_start: 1513, token_types_count: 0 }, // [918]
    SimpleHintData { raw_values_start: 1513, raw_values_count: 1, token_types_start: 1514, token_types_count: 0 }, // [919]
    SimpleHintData { raw_values_start: 1514, raw_values_count: 2, token_types_start: 1516, token_types_count: 0 }, // [920]
    SimpleHintData { raw_values_start: 1516, raw_values_count: 1, token_types_start: 1517, token_types_count: 1 }, // [921]
    SimpleHintData { raw_values_start: 1518, raw_values_count: 2, token_types_start: 1520, token_types_count: 0 }, // [922]
    SimpleHintData { raw_values_start: 1520, raw_values_count: 1, token_types_start: 1521, token_types_count: 0 }, // [923]
    SimpleHintData { raw_values_start: 1521, raw_values_count: 2, token_types_start: 1523, token_types_count: 0 }, // [924]
    SimpleHintData { raw_values_start: 1523, raw_values_count: 1, token_types_start: 1524, token_types_count: 0 }, // [925]
    SimpleHintData { raw_values_start: 1524, raw_values_count: 1, token_types_start: 1525, token_types_count: 0 }, // [926]
    SimpleHintData { raw_values_start: 1525, raw_values_count: 1, token_types_start: 1526, token_types_count: 0 }, // [927]
    SimpleHintData { raw_values_start: 1526, raw_values_count: 1, token_types_start: 1527, token_types_count: 0 }, // [928]
    SimpleHintData { raw_values_start: 1527, raw_values_count: 1, token_types_start: 1528, token_types_count: 0 }, // [929]
    SimpleHintData { raw_values_start: 1528, raw_values_count: 1, token_types_start: 1529, token_types_count: 0 }, // [930]
    SimpleHintData { raw_values_start: 1529, raw_values_count: 1, token_types_start: 1530, token_types_count: 0 }, // [931]
    SimpleHintData { raw_values_start: 1530, raw_values_count: 1, token_types_start: 1531, token_types_count: 0 }, // [932]
    SimpleHintData { raw_values_start: 1531, raw_values_count: 1, token_types_start: 1532, token_types_count: 0 }, // [933]
    SimpleHintData { raw_values_start: 1532, raw_values_count: 1, token_types_start: 1533, token_types_count: 0 }, // [934]
    SimpleHintData { raw_values_start: 1533, raw_values_count: 1, token_types_start: 1534, token_types_count: 0 }, // [935]
    SimpleHintData { raw_values_start: 1534, raw_values_count: 1, token_types_start: 1535, token_types_count: 0 }, // [936]
    SimpleHintData { raw_values_start: 1535, raw_values_count: 1, token_types_start: 1536, token_types_count: 0 }, // [937]
    SimpleHintData { raw_values_start: 1536, raw_values_count: 1, token_types_start: 1537, token_types_count: 0 }, // [938]
    SimpleHintData { raw_values_start: 1537, raw_values_count: 1, token_types_start: 1538, token_types_count: 0 }, // [939]
    SimpleHintData { raw_values_start: 1538, raw_values_count: 1, token_types_start: 1539, token_types_count: 0 }, // [940]
    SimpleHintData { raw_values_start: 1539, raw_values_count: 1, token_types_start: 1540, token_types_count: 0 }, // [941]
    SimpleHintData { raw_values_start: 1540, raw_values_count: 1, token_types_start: 1541, token_types_count: 0 }, // [942]
    SimpleHintData { raw_values_start: 1541, raw_values_count: 1, token_types_start: 1542, token_types_count: 0 }, // [943]
    SimpleHintData { raw_values_start: 1542, raw_values_count: 1, token_types_start: 1543, token_types_count: 0 }, // [944]
    SimpleHintData { raw_values_start: 1543, raw_values_count: 1, token_types_start: 1544, token_types_count: 0 }, // [945]
    SimpleHintData { raw_values_start: 1544, raw_values_count: 1, token_types_start: 1545, token_types_count: 0 }, // [946]
    SimpleHintData { raw_values_start: 1545, raw_values_count: 1, token_types_start: 1546, token_types_count: 0 }, // [947]
    SimpleHintData { raw_values_start: 1546, raw_values_count: 1, token_types_start: 1547, token_types_count: 0 }, // [948]
    SimpleHintData { raw_values_start: 1547, raw_values_count: 1, token_types_start: 1548, token_types_count: 0 }, // [949]
    SimpleHintData { raw_values_start: 1548, raw_values_count: 1, token_types_start: 1549, token_types_count: 0 }, // [950]
    SimpleHintData { raw_values_start: 1549, raw_values_count: 1, token_types_start: 1550, token_types_count: 0 }, // [951]
    SimpleHintData { raw_values_start: 1550, raw_values_count: 1, token_types_start: 1551, token_types_count: 0 }, // [952]
    SimpleHintData { raw_values_start: 1551, raw_values_count: 1, token_types_start: 1552, token_types_count: 0 }, // [953]
    SimpleHintData { raw_values_start: 1552, raw_values_count: 1, token_types_start: 1553, token_types_count: 0 }, // [954]
    SimpleHintData { raw_values_start: 1553, raw_values_count: 1, token_types_start: 1554, token_types_count: 0 }, // [955]
    SimpleHintData { raw_values_start: 1554, raw_values_count: 1, token_types_start: 1555, token_types_count: 0 }, // [956]
    SimpleHintData { raw_values_start: 1555, raw_values_count: 1, token_types_start: 1556, token_types_count: 0 }, // [957]
    SimpleHintData { raw_values_start: 1556, raw_values_count: 1, token_types_start: 1557, token_types_count: 0 }, // [958]
    SimpleHintData { raw_values_start: 1557, raw_values_count: 1, token_types_start: 1558, token_types_count: 0 }, // [959]
    SimpleHintData { raw_values_start: 1558, raw_values_count: 1, token_types_start: 1559, token_types_count: 0 }, // [960]
    SimpleHintData { raw_values_start: 1559, raw_values_count: 1, token_types_start: 1560, token_types_count: 0 }, // [961]
    SimpleHintData { raw_values_start: 1560, raw_values_count: 1, token_types_start: 1561, token_types_count: 0 }, // [962]
    SimpleHintData { raw_values_start: 1561, raw_values_count: 1, token_types_start: 1562, token_types_count: 0 }, // [963]
    SimpleHintData { raw_values_start: 1562, raw_values_count: 1, token_types_start: 1563, token_types_count: 0 }, // [964]
    SimpleHintData { raw_values_start: 1563, raw_values_count: 5, token_types_start: 1568, token_types_count: 0 }, // [965]
    SimpleHintData { raw_values_start: 1568, raw_values_count: 1, token_types_start: 1569, token_types_count: 0 }, // [966]
    SimpleHintData { raw_values_start: 1569, raw_values_count: 1, token_types_start: 1570, token_types_count: 0 }, // [967]
    SimpleHintData { raw_values_start: 1570, raw_values_count: 1, token_types_start: 1571, token_types_count: 0 }, // [968]
    SimpleHintData { raw_values_start: 1571, raw_values_count: 1, token_types_start: 1572, token_types_count: 0 }, // [969]
    SimpleHintData { raw_values_start: 1572, raw_values_count: 1, token_types_start: 1573, token_types_count: 0 }, // [970]
    SimpleHintData { raw_values_start: 1573, raw_values_count: 1, token_types_start: 1574, token_types_count: 0 }, // [971]
    SimpleHintData { raw_values_start: 1574, raw_values_count: 1, token_types_start: 1575, token_types_count: 0 }, // [972]
    SimpleHintData { raw_values_start: 1575, raw_values_count: 1, token_types_start: 1576, token_types_count: 0 }, // [973]
    SimpleHintData { raw_values_start: 1576, raw_values_count: 1, token_types_start: 1577, token_types_count: 0 }, // [974]
    SimpleHintData { raw_values_start: 1577, raw_values_count: 1, token_types_start: 1578, token_types_count: 0 }, // [975]
    SimpleHintData { raw_values_start: 1578, raw_values_count: 1, token_types_start: 1579, token_types_count: 0 }, // [976]
    SimpleHintData { raw_values_start: 1579, raw_values_count: 1, token_types_start: 1580, token_types_count: 0 }, // [977]
    SimpleHintData { raw_values_start: 1580, raw_values_count: 1, token_types_start: 1581, token_types_count: 0 }, // [978]
    SimpleHintData { raw_values_start: 1581, raw_values_count: 1, token_types_start: 1582, token_types_count: 0 }, // [979]
    SimpleHintData { raw_values_start: 1582, raw_values_count: 1, token_types_start: 1583, token_types_count: 0 }, // [980]
    SimpleHintData { raw_values_start: 1583, raw_values_count: 1, token_types_start: 1584, token_types_count: 0 }, // [981]
    SimpleHintData { raw_values_start: 1584, raw_values_count: 1, token_types_start: 1585, token_types_count: 0 }, // [982]
    SimpleHintData { raw_values_start: 1585, raw_values_count: 1, token_types_start: 1586, token_types_count: 0 }, // [983]
    SimpleHintData { raw_values_start: 1586, raw_values_count: 1, token_types_start: 1587, token_types_count: 0 }, // [984]
    SimpleHintData { raw_values_start: 1587, raw_values_count: 1, token_types_start: 1588, token_types_count: 0 }, // [985]
    SimpleHintData { raw_values_start: 1588, raw_values_count: 1, token_types_start: 1589, token_types_count: 0 }, // [986]
    SimpleHintData { raw_values_start: 1589, raw_values_count: 1, token_types_start: 1590, token_types_count: 0 }, // [987]
    SimpleHintData { raw_values_start: 1590, raw_values_count: 1, token_types_start: 1591, token_types_count: 0 }, // [988]
    SimpleHintData { raw_values_start: 1591, raw_values_count: 1, token_types_start: 1592, token_types_count: 0 }, // [989]
    SimpleHintData { raw_values_start: 1592, raw_values_count: 1, token_types_start: 1593, token_types_count: 0 }, // [990]
    SimpleHintData { raw_values_start: 1593, raw_values_count: 3, token_types_start: 1596, token_types_count: 0 }, // [991]
    SimpleHintData { raw_values_start: 1596, raw_values_count: 1, token_types_start: 1597, token_types_count: 0 }, // [992]
    SimpleHintData { raw_values_start: 1597, raw_values_count: 6, token_types_start: 1603, token_types_count: 0 }, // [993]
    SimpleHintData { raw_values_start: 1603, raw_values_count: 1, token_types_start: 1604, token_types_count: 0 }, // [994]
    SimpleHintData { raw_values_start: 1604, raw_values_count: 2, token_types_start: 1606, token_types_count: 0 }, // [995]
    SimpleHintData { raw_values_start: 1606, raw_values_count: 3, token_types_start: 1609, token_types_count: 0 }, // [996]
    SimpleHintData { raw_values_start: 1609, raw_values_count: 3, token_types_start: 1612, token_types_count: 0 }, // [997]
    SimpleHintData { raw_values_start: 1612, raw_values_count: 2, token_types_start: 1614, token_types_count: 0 }, // [998]
    SimpleHintData { raw_values_start: 1614, raw_values_count: 15, token_types_start: 1629, token_types_count: 0 }, // [999]
    SimpleHintData { raw_values_start: 1629, raw_values_count: 2, token_types_start: 1631, token_types_count: 1 }, // [1000]
    SimpleHintData { raw_values_start: 1632, raw_values_count: 2, token_types_start: 1634, token_types_count: 0 }, // [1001]
    SimpleHintData { raw_values_start: 1634, raw_values_count: 12, token_types_start: 1646, token_types_count: 0 }, // [1002]
    SimpleHintData { raw_values_start: 1646, raw_values_count: 2, token_types_start: 1648, token_types_count: 0 }, // [1003]
    SimpleHintData { raw_values_start: 1648, raw_values_count: 7, token_types_start: 1655, token_types_count: 0 }, // [1004]
    SimpleHintData { raw_values_start: 1655, raw_values_count: 4, token_types_start: 1659, token_types_count: 0 }, // [1005]
    SimpleHintData { raw_values_start: 1659, raw_values_count: 2, token_types_start: 1661, token_types_count: 0 }, // [1006]
    SimpleHintData { raw_values_start: 1661, raw_values_count: 3, token_types_start: 1664, token_types_count: 0 }, // [1007]
    SimpleHintData { raw_values_start: 1664, raw_values_count: 2, token_types_start: 1666, token_types_count: 0 }, // [1008]
    SimpleHintData { raw_values_start: 1666, raw_values_count: 2, token_types_start: 1668, token_types_count: 0 }, // [1009]
    SimpleHintData { raw_values_start: 1668, raw_values_count: 3, token_types_start: 1671, token_types_count: 0 }, // [1010]
    SimpleHintData { raw_values_start: 1671, raw_values_count: 2, token_types_start: 1673, token_types_count: 0 }, // [1011]
    SimpleHintData { raw_values_start: 1673, raw_values_count: 2, token_types_start: 1675, token_types_count: 0 }, // [1012]
    SimpleHintData { raw_values_start: 1675, raw_values_count: 1, token_types_start: 1676, token_types_count: 0 }, // [1013]
    SimpleHintData { raw_values_start: 1676, raw_values_count: 3, token_types_start: 1679, token_types_count: 0 }, // [1014]
    SimpleHintData { raw_values_start: 1679, raw_values_count: 1, token_types_start: 1680, token_types_count: 0 }, // [1015]
    SimpleHintData { raw_values_start: 1680, raw_values_count: 1, token_types_start: 1681, token_types_count: 0 }, // [1016]
    SimpleHintData { raw_values_start: 1681, raw_values_count: 1, token_types_start: 1682, token_types_count: 0 }, // [1017]
    SimpleHintData { raw_values_start: 1682, raw_values_count: 1, token_types_start: 1683, token_types_count: 0 }, // [1018]
    SimpleHintData { raw_values_start: 1683, raw_values_count: 1, token_types_start: 1684, token_types_count: 0 }, // [1019]
    SimpleHintData { raw_values_start: 1684, raw_values_count: 1, token_types_start: 1685, token_types_count: 0 }, // [1020]
    SimpleHintData { raw_values_start: 1685, raw_values_count: 2, token_types_start: 1687, token_types_count: 0 }, // [1021]
    SimpleHintData { raw_values_start: 1687, raw_values_count: 2, token_types_start: 1689, token_types_count: 0 }, // [1022]
    SimpleHintData { raw_values_start: 1689, raw_values_count: 1, token_types_start: 1690, token_types_count: 0 }, // [1023]
    SimpleHintData { raw_values_start: 1690, raw_values_count: 1, token_types_start: 1691, token_types_count: 0 }, // [1024]
    SimpleHintData { raw_values_start: 1691, raw_values_count: 1, token_types_start: 1692, token_types_count: 0 }, // [1025]
    SimpleHintData { raw_values_start: 1692, raw_values_count: 1, token_types_start: 1693, token_types_count: 0 }, // [1026]
    SimpleHintData { raw_values_start: 1693, raw_values_count: 1, token_types_start: 1694, token_types_count: 0 }, // [1027]
    SimpleHintData { raw_values_start: 1694, raw_values_count: 1, token_types_start: 1695, token_types_count: 0 }, // [1028]
    SimpleHintData { raw_values_start: 1695, raw_values_count: 1, token_types_start: 1696, token_types_count: 0 }, // [1029]
    SimpleHintData { raw_values_start: 1696, raw_values_count: 1, token_types_start: 1697, token_types_count: 0 }, // [1030]
    SimpleHintData { raw_values_start: 1697, raw_values_count: 1, token_types_start: 1698, token_types_count: 0 }, // [1031]
    SimpleHintData { raw_values_start: 1698, raw_values_count: 1, token_types_start: 1699, token_types_count: 0 }, // [1032]
    SimpleHintData { raw_values_start: 1699, raw_values_count: 1, token_types_start: 1700, token_types_count: 0 }, // [1033]
    SimpleHintData { raw_values_start: 1700, raw_values_count: 2, token_types_start: 1702, token_types_count: 0 }, // [1034]
    SimpleHintData { raw_values_start: 1702, raw_values_count: 1, token_types_start: 1703, token_types_count: 0 }, // [1035]
    SimpleHintData { raw_values_start: 1703, raw_values_count: 1, token_types_start: 1704, token_types_count: 0 }, // [1036]
    SimpleHintData { raw_values_start: 1704, raw_values_count: 1, token_types_start: 1705, token_types_count: 0 }, // [1037]
    SimpleHintData { raw_values_start: 1705, raw_values_count: 1, token_types_start: 1706, token_types_count: 0 }, // [1038]
    SimpleHintData { raw_values_start: 1706, raw_values_count: 1, token_types_start: 1707, token_types_count: 0 }, // [1039]
    SimpleHintData { raw_values_start: 1707, raw_values_count: 1, token_types_start: 1708, token_types_count: 0 }, // [1040]
    SimpleHintData { raw_values_start: 1708, raw_values_count: 1, token_types_start: 1709, token_types_count: 0 }, // [1041]
    SimpleHintData { raw_values_start: 1709, raw_values_count: 1, token_types_start: 1710, token_types_count: 0 }, // [1042]
    SimpleHintData { raw_values_start: 1710, raw_values_count: 1, token_types_start: 1711, token_types_count: 0 }, // [1043]
    SimpleHintData { raw_values_start: 1711, raw_values_count: 1, token_types_start: 1712, token_types_count: 0 }, // [1044]
    SimpleHintData { raw_values_start: 1712, raw_values_count: 1, token_types_start: 1713, token_types_count: 0 }, // [1045]
    SimpleHintData { raw_values_start: 1713, raw_values_count: 1, token_types_start: 1714, token_types_count: 0 }, // [1046]
    SimpleHintData { raw_values_start: 1714, raw_values_count: 1, token_types_start: 1715, token_types_count: 0 }, // [1047]
    SimpleHintData { raw_values_start: 1715, raw_values_count: 1, token_types_start: 1716, token_types_count: 0 }, // [1048]
    SimpleHintData { raw_values_start: 1716, raw_values_count: 1, token_types_start: 1717, token_types_count: 0 }, // [1049]
    SimpleHintData { raw_values_start: 1717, raw_values_count: 1, token_types_start: 1718, token_types_count: 0 }, // [1050]
    SimpleHintData { raw_values_start: 1718, raw_values_count: 1, token_types_start: 1719, token_types_count: 0 }, // [1051]
    SimpleHintData { raw_values_start: 1719, raw_values_count: 1, token_types_start: 1720, token_types_count: 0 }, // [1052]
    SimpleHintData { raw_values_start: 1720, raw_values_count: 1, token_types_start: 1721, token_types_count: 0 }, // [1053]
    SimpleHintData { raw_values_start: 1721, raw_values_count: 1, token_types_start: 1722, token_types_count: 0 }, // [1054]
    SimpleHintData { raw_values_start: 1722, raw_values_count: 1, token_types_start: 1723, token_types_count: 0 }, // [1055]
    SimpleHintData { raw_values_start: 1723, raw_values_count: 1, token_types_start: 1724, token_types_count: 0 }, // [1056]
    SimpleHintData { raw_values_start: 1724, raw_values_count: 2, token_types_start: 1726, token_types_count: 0 }, // [1057]
    SimpleHintData { raw_values_start: 1726, raw_values_count: 1, token_types_start: 1727, token_types_count: 0 }, // [1058]
    SimpleHintData { raw_values_start: 1727, raw_values_count: 1, token_types_start: 1728, token_types_count: 0 }, // [1059]
    SimpleHintData { raw_values_start: 1728, raw_values_count: 1, token_types_start: 1729, token_types_count: 0 }, // [1060]
    SimpleHintData { raw_values_start: 1729, raw_values_count: 1, token_types_start: 1730, token_types_count: 0 }, // [1061]
    SimpleHintData { raw_values_start: 1730, raw_values_count: 1, token_types_start: 1731, token_types_count: 0 }, // [1062]
    SimpleHintData { raw_values_start: 1731, raw_values_count: 1, token_types_start: 1732, token_types_count: 0 }, // [1063]
    SimpleHintData { raw_values_start: 1732, raw_values_count: 1, token_types_start: 1733, token_types_count: 0 }, // [1064]
    SimpleHintData { raw_values_start: 1733, raw_values_count: 1, token_types_start: 1734, token_types_count: 0 }, // [1065]
    SimpleHintData { raw_values_start: 1734, raw_values_count: 1, token_types_start: 1735, token_types_count: 0 }, // [1066]
    SimpleHintData { raw_values_start: 1735, raw_values_count: 1, token_types_start: 1736, token_types_count: 0 }, // [1067]
    SimpleHintData { raw_values_start: 1736, raw_values_count: 1, token_types_start: 1737, token_types_count: 0 }, // [1068]
    SimpleHintData { raw_values_start: 1737, raw_values_count: 1, token_types_start: 1738, token_types_count: 0 }, // [1069]
    SimpleHintData { raw_values_start: 1738, raw_values_count: 1, token_types_start: 1739, token_types_count: 0 }, // [1070]
    SimpleHintData { raw_values_start: 1739, raw_values_count: 1, token_types_start: 1740, token_types_count: 0 }, // [1071]
    SimpleHintData { raw_values_start: 1740, raw_values_count: 1, token_types_start: 1741, token_types_count: 0 }, // [1072]
    SimpleHintData { raw_values_start: 1741, raw_values_count: 1, token_types_start: 1742, token_types_count: 0 }, // [1073]
    SimpleHintData { raw_values_start: 1742, raw_values_count: 1, token_types_start: 1743, token_types_count: 0 }, // [1074]
    SimpleHintData { raw_values_start: 1743, raw_values_count: 1, token_types_start: 1744, token_types_count: 0 }, // [1075]
    SimpleHintData { raw_values_start: 1744, raw_values_count: 1, token_types_start: 1745, token_types_count: 0 }, // [1076]
    SimpleHintData { raw_values_start: 1745, raw_values_count: 1, token_types_start: 1746, token_types_count: 0 }, // [1077]
    SimpleHintData { raw_values_start: 1746, raw_values_count: 1, token_types_start: 1747, token_types_count: 0 }, // [1078]
    SimpleHintData { raw_values_start: 1747, raw_values_count: 1, token_types_start: 1748, token_types_count: 0 }, // [1079]
    SimpleHintData { raw_values_start: 1748, raw_values_count: 2, token_types_start: 1750, token_types_count: 0 }, // [1080]
    SimpleHintData { raw_values_start: 1750, raw_values_count: 1, token_types_start: 1751, token_types_count: 0 }, // [1081]
    SimpleHintData { raw_values_start: 1751, raw_values_count: 1, token_types_start: 1752, token_types_count: 0 }, // [1082]
    SimpleHintData { raw_values_start: 1752, raw_values_count: 1, token_types_start: 1753, token_types_count: 0 }, // [1083]
    SimpleHintData { raw_values_start: 1753, raw_values_count: 1, token_types_start: 1754, token_types_count: 0 }, // [1084]
    SimpleHintData { raw_values_start: 1754, raw_values_count: 1, token_types_start: 1755, token_types_count: 0 }, // [1085]
    SimpleHintData { raw_values_start: 1755, raw_values_count: 1, token_types_start: 1756, token_types_count: 0 }, // [1086]
    SimpleHintData { raw_values_start: 1756, raw_values_count: 1, token_types_start: 1757, token_types_count: 0 }, // [1087]
    SimpleHintData { raw_values_start: 1757, raw_values_count: 1, token_types_start: 1758, token_types_count: 0 }, // [1088]
    SimpleHintData { raw_values_start: 1758, raw_values_count: 1, token_types_start: 1759, token_types_count: 0 }, // [1089]
    SimpleHintData { raw_values_start: 1759, raw_values_count: 1, token_types_start: 1760, token_types_count: 0 }, // [1090]
    SimpleHintData { raw_values_start: 1760, raw_values_count: 1, token_types_start: 1761, token_types_count: 0 }, // [1091]
    SimpleHintData { raw_values_start: 1761, raw_values_count: 1, token_types_start: 1762, token_types_count: 0 }, // [1092]
    SimpleHintData { raw_values_start: 1762, raw_values_count: 1, token_types_start: 1763, token_types_count: 0 }, // [1093]
    SimpleHintData { raw_values_start: 1763, raw_values_count: 1, token_types_start: 1764, token_types_count: 0 }, // [1094]
    SimpleHintData { raw_values_start: 1764, raw_values_count: 1, token_types_start: 1765, token_types_count: 0 }, // [1095]
    SimpleHintData { raw_values_start: 1765, raw_values_count: 1, token_types_start: 1766, token_types_count: 0 }, // [1096]
    SimpleHintData { raw_values_start: 1766, raw_values_count: 8, token_types_start: 1774, token_types_count: 0 }, // [1097]
    SimpleHintData { raw_values_start: 1774, raw_values_count: 1, token_types_start: 1775, token_types_count: 0 }, // [1098]
    SimpleHintData { raw_values_start: 1775, raw_values_count: 1, token_types_start: 1776, token_types_count: 0 }, // [1099]
    SimpleHintData { raw_values_start: 1776, raw_values_count: 2, token_types_start: 1778, token_types_count: 0 }, // [1100]
    SimpleHintData { raw_values_start: 1778, raw_values_count: 1, token_types_start: 1779, token_types_count: 0 }, // [1101]
    SimpleHintData { raw_values_start: 1779, raw_values_count: 1, token_types_start: 1780, token_types_count: 0 }, // [1102]
    SimpleHintData { raw_values_start: 1780, raw_values_count: 1, token_types_start: 1781, token_types_count: 0 }, // [1103]
    SimpleHintData { raw_values_start: 1781, raw_values_count: 1, token_types_start: 1782, token_types_count: 0 }, // [1104]
    SimpleHintData { raw_values_start: 1782, raw_values_count: 1, token_types_start: 1783, token_types_count: 0 }, // [1105]
    SimpleHintData { raw_values_start: 1783, raw_values_count: 1, token_types_start: 1784, token_types_count: 0 }, // [1106]
    SimpleHintData { raw_values_start: 1784, raw_values_count: 1, token_types_start: 1785, token_types_count: 0 }, // [1107]
    SimpleHintData { raw_values_start: 1785, raw_values_count: 1, token_types_start: 1786, token_types_count: 0 }, // [1108]
    SimpleHintData { raw_values_start: 1786, raw_values_count: 1, token_types_start: 1787, token_types_count: 0 }, // [1109]
];

pub static SIMPLE_HINT_INDICES: &[u32] = &[
    1, 2, 3, 4, 0, 5, 0, 6, 7, 7, 8, 9, 10, 11, 12, 13,
    14, 15, 16, 17, 18, 19, 19, 20, 21, 22, 22, 23, 24, 24, 25, 26,
    10, 27, 27, 25, 26, 10, 28, 24, 24, 29, 26, 14, 27, 27, 29, 26,
    30, 10, 14, 0, 0, 0, 0, 0, 31, 0, 32, 33, 34, 35, 35, 36,
    37, 38, 39, 39, 40, 16, 41, 9, 10, 12, 42, 42, 43, 44, 45, 45,
    46, 47, 47, 48, 8, 8, 49, 50, 51, 52, 52, 39, 39, 53, 14, 43,
    43, 44, 54, 28, 55, 55, 56, 14, 15, 57, 57, 58, 42, 59, 39, 60,
    45, 61, 62, 8, 63, 64, 65, 66, 67, 68, 69, 70, 71, 72, 73, 51,
    74, 75, 24, 24, 58, 31, 38, 38, 76, 28, 77, 78, 79, 80, 81, 82,
    83, 84, 46, 14, 43, 43, 44, 85, 85, 78, 86, 86, 77, 87, 87, 88,
    29, 89, 90, 91, 92, 93, 94, 95, 96, 97, 25, 43, 43, 0, 85, 85,
    90, 86, 86, 89, 87, 87, 0, 98, 99, 100, 0, 101, 101, 0, 0, 41,
    41, 0, 0, 16, 16, 0, 0, 102, 102, 0, 0, 0, 0, 0, 0, 103,
    104, 104, 5, 105, 106, 107, 108, 109, 110, 111, 112, 113, 0, 0, 114, 0,
    0, 0, 31, 0, 31, 115, 116, 0, 117, 0, 0, 0, 24, 118, 119, 120,
    121, 122, 122, 123, 37, 124, 125, 126, 127, 127, 37, 128, 129, 130, 131, 132,
    120, 120, 83, 0, 121, 121, 133, 133, 134, 135, 0, 0, 0, 0, 136, 0,
    0, 0, 137, 137, 109, 62, 134, 0, 138, 139, 139, 140, 111, 0, 31, 133,
    141, 141, 142, 114, 33, 0, 120, 120, 77, 0, 0, 0, 143, 144, 145, 146,
    147, 147, 148, 42, 149, 150, 151, 152, 153, 154, 155, 144, 156, 157, 158, 159,
    104, 0, 0, 37, 0, 160, 104, 161, 0, 162, 104, 0, 0, 0, 114, 114,
    114, 163, 164, 165, 166, 167, 168, 169, 170, 171, 172, 173, 174, 174, 174, 175,
    176, 177, 178, 179, 180, 0, 0, 180, 181, 0, 0, 0, 0, 0, 0, 32,
    114, 0, 0, 182, 183, 184, 185, 186, 187, 188, 150, 156, 189, 190, 0, 191,
    192, 193, 194, 144, 158, 158, 158, 156, 159, 159, 159, 157, 195, 196, 189, 147,
    197, 198, 199, 200, 201, 202, 31, 0, 0, 31, 0, 0, 0, 31, 203, 204,
    205, 206, 0, 0, 207, 153, 208, 207, 209, 135, 210, 211, 107, 212, 10, 213,
    214, 215, 216, 217, 218, 219, 107, 0, 207, 207, 208, 220, 208, 87, 87, 136,
    0, 116, 0, 190, 190, 0, 221, 123, 31, 0, 0, 0, 31, 114, 31, 222,
    127, 223, 224, 225, 226, 227, 228, 228, 228, 0, 229, 229, 230, 231, 230, 231,
    231, 228, 228, 0, 229, 229, 230, 231, 230, 231, 231, 190, 32, 188, 228, 232,
    233, 234, 15, 235, 236, 237, 238, 239, 240, 241, 241, 242, 0, 243, 244, 245,
    232, 246, 247, 248, 249, 250, 251, 241, 252, 253, 254, 255, 256, 257, 258, 259,
    260, 261, 262, 263, 264, 265, 266, 267, 268, 269, 0, 31, 240, 270, 271, 0,
    116, 0, 0, 207, 0, 272, 273, 274, 275, 160, 160, 160, 0, 0, 0, 0,
    180, 0, 276, 276, 0, 0, 277, 244, 278, 258, 258, 31, 0, 279, 279, 0,
    280, 280, 0, 281, 174, 67, 67, 0, 282, 270, 270, 0, 0, 0, 0, 0,
    31, 0, 0, 0, 283, 0, 134, 0, 0, 207, 0, 284, 0, 0, 0, 0,
    285, 32, 286, 287, 282, 282, 116, 282, 0, 288, 289, 190, 136, 290, 159, 291,
    158, 292, 159, 293, 158, 294, 295, 296, 244, 157, 297, 298, 299, 299, 300, 34,
    299, 299, 244, 300, 34, 0, 301, 0, 0, 283, 302, 156, 156, 156, 303, 304,
    305, 0, 306, 305, 0, 59, 307, 308, 276, 309, 310, 311, 312, 313, 314, 315,
    316, 317, 318, 319, 320, 321, 322, 323, 324, 39, 39, 233, 31, 0, 114, 0,
    325, 326, 327, 82, 80, 81, 328, 329, 330, 331, 313, 332, 0, 31, 333, 333,
    0, 114, 114, 166, 39, 39, 10, 135, 0, 39, 39, 197, 334, 80, 135, 0,
    31, 335, 335, 0, 114, 39, 39, 197, 281, 336, 135, 0, 76, 0, 31, 31,
    0, 0, 39, 39, 39, 197, 337, 135, 0, 338, 338, 31, 0, 0, 0, 136,
    0, 0, 104, 116, 116, 114, 222, 39, 39, 16, 135, 0, 39, 39, 14, 135,
    0, 339, 122, 122, 123, 37, 340, 340, 325, 37, 124, 125, 126, 127, 127, 37,
    128, 129, 130, 131, 132, 39, 39, 83, 0, 339, 339, 39, 39, 197, 341, 342,
    242, 343, 344, 345, 345, 70, 346, 77, 135, 0, 32, 347, 31, 31, 0, 0,
    0, 348, 282, 114, 114, 222, 349, 349, 0, 350, 350, 39, 39, 71, 0, 351,
    182, 111, 352, 352, 353, 354, 354, 61, 60, 73, 73, 353, 0, 0, 197, 76,
    197, 76, 76, 0, 355, 356, 356, 357, 358, 114, 0, 359, 358, 114, 0, 34,
    34, 0, 360, 244, 244, 361, 362, 361, 363, 363, 363, 364, 363, 363, 365, 333,
    333, 366, 367, 358, 368, 229, 229, 31, 0, 45, 45, 81, 369, 31, 39, 39,
    41, 0, 370, 370, 39, 371, 78, 135, 0, 31, 114, 222, 31, 222, 325, 372,
    373, 374, 305, 246, 375, 376, 376, 377, 377, 376, 378, 31, 0, 379, 380, 381,
    382, 383, 384, 385, 386, 387, 388, 389, 390, 130, 22, 10, 10, 10, 0, 391,
    0, 0, 0, 0, 0, 0, 31, 31, 0, 0, 0, 88, 88, 116, 240, 241,
    241, 242, 0, 243, 244, 245, 232, 246, 247, 248, 249, 392, 393, 393, 31, 31,
    0, 0, 394, 0, 395, 395, 392, 396, 397, 181, 116, 398, 394, 399, 400, 401,
    402, 403, 404, 405, 406, 407, 408, 409, 410, 411, 412, 413, 414, 279, 279, 279,
    415, 416, 361, 365, 417, 418, 419, 420, 421, 422, 60, 423, 423, 60, 420, 34,
    424, 425, 426, 427, 428, 429, 430, 431, 432, 433, 434, 434, 434, 0, 0, 435,
    436, 437, 438, 439, 440, 441, 442, 443, 444, 300, 445, 446, 447, 153, 448, 11,
    207, 449, 450, 451, 226, 133, 133, 233, 31, 0, 114, 0, 450, 133, 133, 10,
    135, 0, 450, 133, 133, 80, 135, 0, 133, 133, 336, 135, 0, 450, 133, 133,
    133, 337, 135, 0, 133, 133, 16, 135, 0, 133, 133, 14, 135, 0, 450, 133,
    133, 83, 0, 133, 133, 334, 77, 135, 0, 0, 450, 133, 133, 71, 135, 0,
    133, 133, 18, 135, 0, 450, 133, 133, 41, 135, 0, 133, 133, 78, 135, 0,
    450, 452, 453, 454, 455, 456, 457, 366, 458, 230, 230, 0, 230, 459, 31, 0,
    0, 31, 460, 461, 462, 463, 464, 465, 231, 466, 467, 136, 290, 468, 212, 469,
    470, 471, 472, 473, 474, 475, 476, 477, 478, 45, 50, 479, 480, 481, 482, 483,
    483, 483, 484, 222, 61, 73, 423, 217, 0, 0, 31, 0, 0, 0, 0, 485,
    188, 188, 0, 486, 190, 299, 299, 244, 0, 0, 0, 31, 487, 244, 185, 0,
    147, 0, 0, 0, 0, 488, 489, 489, 489, 490, 244, 491, 0, 0, 492, 492,
    492, 492, 150, 156, 190, 0, 493, 489, 489, 489, 490, 0, 480, 480, 31, 222,
    0, 0, 0, 228, 168, 168, 0, 32, 0, 0, 180, 0, 31, 0, 0, 0,
    222, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 31, 0,
    0, 0, 0, 494, 116, 37, 199, 278, 395, 0, 0, 32, 104, 0, 495, 0,
    181, 0, 0, 0, 298, 0, 496, 86, 497, 498, 201, 201, 499, 499, 500, 140,
    501, 0, 37, 0, 358, 502, 358, 503, 504, 505, 325, 325, 506, 499, 507, 87,
    508, 425, 425, 509, 509, 425, 425, 425, 425, 510, 511, 511, 31, 424, 424, 0,
    511, 512, 140, 513, 514, 515, 516, 517, 518, 333, 519, 19, 19, 520, 19, 88,
    521, 522, 523, 524, 524, 0, 0, 380, 380, 358, 0, 0, 37, 181, 525, 0,
    526, 527, 518, 185, 185, 147, 524, 503, 528, 529, 530, 531, 532, 34, 34, 0,
    0, 533, 424, 534, 101, 101, 123, 113, 113, 123, 535, 536, 537, 538, 325, 325,
    499, 539, 0, 0, 0, 0, 31, 0, 0, 0, 0, 533, 540, 0, 0, 325,
    325, 539, 0, 540, 0, 31, 0, 0, 0, 0, 31, 0, 0, 541, 0, 31,
    0, 113, 113, 123, 101, 101, 123, 0, 0, 0, 0, 0, 0, 113, 113, 123,
    101, 101, 123, 0, 34, 34, 34, 0, 0, 542, 543, 0, 0, 0, 0, 0,
    114, 0, 544, 544, 0, 34, 34, 0, 0, 0, 394, 0, 34, 0, 0, 300,
    0, 154, 0, 0, 113, 545, 545, 0, 0, 0, 116, 0, 0, 26, 0, 116,
    0, 0, 546, 0, 0, 0, 0, 114, 0, 31, 31, 0, 114, 114, 116, 13,
    13, 0, 80, 369, 0, 0, 0, 0, 207, 31, 547, 369, 548, 31, 0, 0,
    0, 0, 148, 148, 18, 0, 0, 148, 148, 18, 0, 31, 0, 0, 0, 393,
    393, 31, 0, 0, 0, 0, 0, 393, 0, 549, 31, 0, 92, 550, 27, 551,
    552, 553, 554, 555, 345, 556, 214, 99, 99, 99, 0, 38, 38, 76, 0, 16,
    16, 0, 64, 64, 76, 41, 0, 0, 33, 0, 557, 325, 325, 99, 558, 325,
    325, 110, 558, 319, 319, 380, 48, 559, 559, 123, 0, 388, 560, 0, 559, 48,
    159, 159, 136, 293, 159, 291, 561, 101, 101, 123, 0, 24, 562, 377, 0, 0,
    0, 0, 37, 0, 563, 563, 113, 113, 123, 534, 535, 536, 537, 499, 101, 0,
    0, 0, 0, 37, 0, 31, 563, 562, 562, 562, 564, 31, 0, 0, 377, 0,
    565, 566, 535, 535, 0, 31, 0, 567, 113, 113, 123, 534, 536, 537, 499, 535,
    568, 569, 570, 571, 572, 573, 0, 574, 575, 576, 577, 578, 579, 580, 0, 581,
    582, 583, 135, 135, 480, 135, 135, 135, 244, 480, 584, 546, 546, 584, 585, 586,
    587, 364, 588, 589, 590, 216, 57, 26, 486, 244, 26, 0, 31, 0, 0, 0,
    222, 0, 591, 592, 122, 0, 0, 593, 163, 433, 336, 594, 595, 596, 597, 598,
    599, 600, 363, 601, 602, 603, 604, 61, 61, 61, 605, 606, 0, 607, 222, 31,
    31, 222, 279, 608, 330, 609, 352, 610, 611, 612, 613, 614, 615, 616, 12, 617,
    618, 619, 181, 181, 620, 37, 37, 621, 116, 394, 116, 116, 116, 621, 116, 394,
    181, 606, 622, 0, 278, 623, 0, 199, 0, 299, 299, 244, 300, 34, 299, 0,
    624, 625, 626, 627, 0, 0, 0, 628, 0, 0, 0, 0, 0, 0, 629, 76,
    630, 0, 0, 0, 628, 0, 0, 0, 0, 0, 628, 628, 628, 0, 76, 76,
    0, 0, 31, 0, 0, 306, 601, 631, 631, 542, 632, 633, 634, 630, 630, 31,
    0, 0, 635, 636, 637, 520, 638, 639, 640, 0, 641, 213, 642, 13, 35, 643,
    644, 0, 645, 646, 647, 648, 632, 649, 650, 158, 158, 136, 294, 158, 292, 651,
    485, 485, 244, 652, 0, 469, 469, 0, 652, 349, 653, 587, 349, 297, 534, 534,
    0, 31, 0, 37, 0, 24, 654, 539, 539, 0, 37, 0, 32, 32, 37, 534,
    655, 656, 0, 657, 0, 116, 37, 199, 489, 278, 395, 104, 0, 658, 0, 0,
    659, 660, 0, 661, 662, 663, 664, 665, 666, 667, 667, 668, 666, 669, 358, 77,
    77, 0, 78, 78, 0, 10, 10, 0, 670, 333, 26, 671, 4, 75, 477, 474,
    65, 65, 672, 672, 102, 261, 660, 660, 289, 673, 673, 674, 674, 668, 675, 676,
    215, 211, 677, 678, 679, 680, 681, 682, 683, 684, 541, 541, 207, 0, 31, 337,
    337, 0, 32, 32, 31, 222, 685, 47, 686, 0, 43, 0, 687, 688, 85, 689,
    690, 691, 692, 693, 694, 695, 696, 697, 698, 699, 60, 61, 61, 31, 549, 700,
    700, 606, 700, 229, 229, 229, 229, 229, 688, 147, 147, 0, 210, 701, 73, 60,
    229, 229, 244, 688, 147, 147, 0, 210, 61, 700, 700, 0, 0, 0, 630, 0,
    0, 0, 31, 31, 222, 0, 0, 0, 0, 0, 76, 0, 229, 73, 73, 242,
    702, 703, 704, 705, 332, 706, 707, 708, 709, 710, 152, 711, 712, 713, 714, 541,
    715, 716, 673, 337, 717, 62, 718, 155, 8, 719, 720, 721, 722, 154, 343, 723,
    724, 0, 725, 0, 0, 114, 0, 0, 31, 0, 537, 537, 0, 0, 726, 623,
    623, 727, 728, 728, 306, 728, 729, 730, 152, 0, 731, 359, 0, 501, 732, 733,
    734, 128, 735, 736, 737, 738, 739, 131, 740, 741, 203, 549, 742, 31, 495, 31,
    743, 31, 203, 31, 0, 744, 73, 61, 423, 700, 745, 203, 746, 31, 495, 203,
    747, 748, 749, 750, 0, 751, 752, 0, 295, 753, 753, 136, 158, 158, 159, 296,
    244, 244, 136, 157, 157, 136, 244, 0, 244, 754, 755, 756, 674, 278, 278, 757,
    758, 586, 759, 760, 37, 36, 116, 116, 160, 0, 658, 116, 116, 658, 207, 207,
    207, 207, 207, 0, 76, 114, 630, 32, 234, 104, 31, 188, 160, 425, 0, 761,
    762, 763, 353, 764, 765, 539, 539, 0, 37, 0, 358, 502, 358, 503, 539, 766,
    767, 357, 76, 768, 505, 769, 770, 771, 772, 773, 63, 774, 775, 776, 558, 777,
    338, 197, 197, 197, 197, 371, 113, 113, 123, 0, 0, 0, 0, 37, 0, 593,
    163, 433, 586, 586, 778, 140, 643, 0, 534, 524, 779, 534, 535, 536, 537, 524,
    545, 499, 113, 132, 113, 749, 780, 781, 782, 783, 634, 784, 785, 0, 546, 786,
    0, 0, 31, 0, 786, 298, 298, 0, 31, 31, 395, 32, 395, 0, 298, 298,
    787, 672, 788, 605, 789, 64, 790, 791, 792, 0, 793, 794, 795, 796, 797, 798,
    799, 800, 801, 802, 802, 123, 0, 31, 0, 0, 802, 803, 804, 805, 806, 153,
    153, 369, 369, 153, 116, 0, 807, 808, 809, 810, 811, 54, 156, 812, 813, 814,
    151, 44, 815, 151, 0, 0, 511, 0, 816, 817, 0, 527, 818, 819, 820, 821,
    822, 823, 280, 280, 520, 280, 824, 491, 58, 825, 826, 81, 93, 827, 828, 829,
    830, 103, 831, 489, 489, 37, 536, 536, 0, 31, 0, 536, 190, 190, 832, 24,
    148, 833, 31, 222, 834, 835, 219, 836, 548, 116, 837, 838, 528, 839, 136, 159,
    840, 158, 753, 65, 841, 842, 843, 844, 845, 846, 847, 67, 67, 0, 31, 687,
    76, 76, 76, 60, 848, 76, 76, 73, 848, 687, 687, 849, 542, 801, 850, 66,
    67, 356, 848, 451, 226, 242, 242, 278, 732, 732, 106, 242, 242, 279, 851, 852,
    853, 854, 855, 856, 857, 858, 859, 860, 861, 862, 863, 864, 141, 865, 866, 371,
    867, 868, 869, 870, 7, 585, 871, 872, 451, 873, 874, 875, 876, 877, 878, 335,
    100, 100, 100, 99, 99, 558, 333, 0, 38, 38, 76, 0, 16, 16, 0, 64,
    64, 76, 41, 0, 0, 34, 0, 450, 633, 653, 16, 0, 879, 880, 378, 378,
    82, 881, 882, 883, 94, 358, 884, 885, 886, 887, 888, 889, 503, 890, 218, 891,
    540, 540, 892, 184, 893, 31, 37, 866, 866, 866, 31, 37, 894, 895, 896, 14,
    897, 25, 898, 899, 900, 901, 902, 903, 904, 905, 906, 907, 908, 0, 0, 0,
    0, 0, 909, 300, 24, 910, 910, 0, 808, 808, 325, 325, 506, 750, 750, 76,
    76, 76, 698, 197, 298, 76, 76, 298, 76, 76, 298, 197, 698, 742, 742, 68,
    908, 909, 0, 0, 34, 424, 113, 113, 123, 534, 538, 911, 34, 0, 424, 113,
    113, 123, 534, 298, 538, 499, 68, 495, 667, 742, 34, 424, 101, 535, 298, 537,
    536, 113, 539, 499, 534, 537, 538, 325, 325, 222, 912, 31, 743, 913, 31, 325,
    745, 31, 222, 914, 425, 915, 545, 83, 125, 693, 693, 37, 732, 732, 693, 124,
    714, 714, 37, 732, 732, 714, 95, 916, 21, 917, 918, 560, 242, 242, 0, 0,
    0, 0, 136, 0, 0, 180, 0, 0, 0, 549, 279, 203, 203, 538, 538, 538,
    203, 113, 534, 537, 242, 538, 919, 920, 920, 618, 472, 24, 710, 242, 242, 835,
    136, 921, 116, 748, 73, 73, 333, 922, 918, 918, 346, 346, 923, 242, 242, 14,
    135, 0, 924, 924, 242, 242, 918, 925, 10, 926, 564, 927, 102, 928, 0, 0,
    0, 228, 234, 234, 234, 0, 168, 929, 930, 931, 489, 151, 152, 932, 850, 0,
    0, 548, 207, 0, 0, 0, 0, 207, 154, 116, 933, 0, 0, 104, 153, 160,
    934, 833, 935, 936, 937, 938, 326, 939, 940, 941, 942, 943, 944, 945, 946, 947,
    948, 949, 950, 951, 952, 953, 954, 955, 956, 957, 958, 925, 959, 960, 79, 91,
    154, 340, 961, 962, 963, 964, 368, 509, 222, 700, 61, 965, 133, 133, 39, 133,
    72, 98, 39, 39, 133, 120, 39, 242, 133, 133, 39, 133, 39, 133, 370, 423,
    73, 39, 133, 39, 133, 39, 133, 434, 55, 483, 39, 120, 133, 39, 133, 275,
    207, 141, 536, 282, 10, 924, 242, 425, 331, 966, 967, 968, 969, 970, 971, 972,
    973, 84, 96, 974, 156, 156, 975, 31, 0, 0, 0, 0, 0, 976, 977, 978,
    979, 980, 981, 982, 983, 984, 985, 0, 986, 987, 988, 989, 893, 990, 348, 276,
    276, 0, 991, 281, 281, 31, 280, 280, 31, 19, 19, 31, 67, 0, 0, 549,
    180, 0, 0, 31, 222, 31, 700, 77, 992, 89, 540, 17, 0, 488, 488, 188,
    0, 0, 493, 493, 0, 46, 97, 993, 993, 891, 891, 37, 808, 893, 893, 994,
    995, 808, 409, 893, 893, 891, 994, 994, 37, 995, 808, 409, 732, 732, 994, 995,
    808, 409, 732, 732, 891, 692, 692, 37, 893, 893, 692, 694, 694, 37, 893, 893,
    694, 891, 333, 333, 380, 275, 275, 984, 996, 963, 967, 966, 630, 630, 993, 993,
    147, 997, 997, 998, 281, 336, 0, 24, 31, 0, 0, 113, 998, 281, 336, 0,
    134, 134, 0, 31, 0, 0, 0, 802, 0, 114, 0, 76, 70, 0, 134, 134,
    0, 31, 0, 0, 0, 802, 0, 114, 0, 283, 283, 283, 0, 31, 0, 0,
    0, 278, 282, 282, 76, 76, 999, 134, 134, 0, 80, 80, 0, 685, 685, 0,
    686, 686, 0, 332, 332, 0, 81, 81, 0, 829, 829, 0, 16, 16, 0, 71,
    71, 0, 18, 18, 0, 78, 78, 0, 10, 10, 0, 87, 87, 0, 77, 77,
    0, 41, 41, 0, 1000, 1001, 114, 299, 116, 32, 32, 1002, 1002, 1003, 732, 498,
    830, 1004, 1005, 732, 452, 661, 244, 244, 661, 1006, 182, 111, 635, 261, 261, 136,
    1007, 76, 764, 279, 1008, 1008, 279, 732, 702, 702, 702, 136, 37, 808, 686, 686,
    136, 0, 113, 113, 123, 1009, 415, 566, 31, 0, 141, 141, 77, 0, 142, 33,
    114, 0, 350, 1010, 1011, 281, 280, 336, 0, 0, 31, 0, 0, 0, 732, 732,
    280, 336, 802, 802, 123, 0, 998, 281, 336, 0, 24, 31, 0, 0, 325, 325,
    732, 732, 22, 147, 147, 732, 732, 996, 963, 967, 966, 76, 76, 288, 1012, 823,
    60, 503, 0, 0, 0, 31, 0, 31, 0, 0, 0, 31, 0, 69, 1013, 0,
    334, 69, 70, 70, 1014, 1015, 334, 31, 0, 32, 32, 240, 1016, 1017, 1018, 1019,
    1020, 210, 994, 506, 490, 396, 1021, 1021, 396, 397, 31, 1022, 1022, 325, 1023, 396,
    1024, 168, 168, 168, 396, 1024, 0, 397, 1025, 1026, 1027, 1028, 1029, 33, 1030, 910,
    1031, 1032, 1033, 923, 965, 965, 340, 183, 288, 880, 231, 1034, 923, 1035, 725, 725,
    0, 147, 147, 732, 238, 1036, 1037, 1038, 1039, 1040, 1041, 1015, 1042, 1043, 1044, 71,
    1045, 1046, 1047, 1048, 544, 202, 646, 1032, 200, 200, 72, 72, 72, 77, 0, 1049,
    1050, 31, 0, 0, 18, 0, 0, 104, 0, 0, 31, 249, 1051, 1052, 525, 1053,
    0, 0, 728, 0, 0, 1054, 1055, 1056, 919, 919, 1057, 300, 24, 919, 281, 281,
    281, 1058, 0, 1059, 1060, 1061, 1062, 742, 742, 34, 424, 101, 535, 536, 298, 537,
    538, 325, 325, 113, 534, 203, 203, 538, 538, 538, 203, 1063, 1064, 73, 73, 73,
    0, 0, 34, 0, 0, 34, 242, 424, 1065, 1066, 248, 51, 55, 55, 55, 0,
    74, 41, 0, 1067, 1068, 1069, 1070, 1071, 630, 1072, 1073, 1074, 1075, 1076, 1077, 1078,
    1079, 549, 549, 1079, 415, 1080, 1080, 358, 31, 0, 0, 279, 0, 0, 415, 1081,
    1082, 1083, 1084, 1085, 1086, 1087, 1088, 1089, 1090, 1091, 78, 90, 346, 1092, 9, 1093,
    1094, 1095, 229, 229, 0, 0, 0, 210, 0, 0, 0, 0, 229, 1096, 424, 424,
    0, 31, 0, 1097, 534, 101, 101, 123, 113, 113, 123, 535, 536, 537, 298, 499,
    424, 1098, 0, 1099, 0, 0, 0, 0, 0, 0, 0, 207, 154, 154, 207, 154,
    537, 0, 0, 1100, 802, 113, 802, 113, 524, 325, 325, 846, 0, 0, 0, 68,
    0, 744, 743, 667, 325, 846, 0, 0, 0, 68, 0, 745, 325, 325, 732, 732,
    22, 0, 325, 325, 325, 732, 14, 191, 1101, 1023, 0, 1035, 23, 75, 1102, 1103,
    1104, 1105, 1106, 1107, 1108, 1109, 1024,
];

pub fn get_teradata_segment_grammar(name: &str) -> Option<RootGrammar> {
    match name {
        "AbortKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(0), tables: &TERADATA_TABLES }),
        "AbsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1), tables: &TERADATA_TABLES }),
        "AbsoluteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2), tables: &TERADATA_TABLES }),
        "AccessKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3), tables: &TERADATA_TABLES }),
        "AccessObjectSegment" => Some(RootGrammar { grammar_id: GrammarId(4), tables: &TERADATA_TABLES }),
        "AccessPermissionSegment" => Some(RootGrammar { grammar_id: GrammarId(65), tables: &TERADATA_TABLES }),
        "AccessPermissionsSegment" => Some(RootGrammar { grammar_id: GrammarId(134), tables: &TERADATA_TABLES }),
        "AccessSchemaObjectSegment" => Some(RootGrammar { grammar_id: GrammarId(137), tables: &TERADATA_TABLES }),
        "AccessSchemaPluralObjectSegment" => Some(RootGrammar { grammar_id: GrammarId(160), tables: &TERADATA_TABLES }),
        "AccessStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(183), tables: &TERADATA_TABLES }),
        "AccessTargetSegment" => Some(RootGrammar { grammar_id: GrammarId(186), tables: &TERADATA_TABLES }),
        "AccessorGrammar" => Some(RootGrammar { grammar_id: GrammarId(208), tables: &TERADATA_TABLES }),
        "AccountKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(210), tables: &TERADATA_TABLES }),
        "AccountsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(211), tables: &TERADATA_TABLES }),
        "ActionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(212), tables: &TERADATA_TABLES }),
        "ActivitycountKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(213), tables: &TERADATA_TABLES }),
        "AdaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(214), tables: &TERADATA_TABLES }),
        "AddKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(215), tables: &TERADATA_TABLES }),
        "AdminKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(216), tables: &TERADATA_TABLES }),
        "AfterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(217), tables: &TERADATA_TABLES }),
        "AggregateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(218), tables: &TERADATA_TABLES }),
        "AggregateOrderByClause" => Some(RootGrammar { grammar_id: GrammarId(219), tables: &TERADATA_TABLES }),
        "AliasExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(220), tables: &TERADATA_TABLES }),
        "AliasKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(232), tables: &TERADATA_TABLES }),
        "AliasedTableReferenceGrammar" => Some(RootGrammar { grammar_id: GrammarId(233), tables: &TERADATA_TABLES }),
        "AllKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(236), tables: &TERADATA_TABLES }),
        "AllocateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(237), tables: &TERADATA_TABLES }),
        "AlsoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(238), tables: &TERADATA_TABLES }),
        "AlterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(239), tables: &TERADATA_TABLES }),
        "AlterSequenceOptionsSegment" => Some(RootGrammar { grammar_id: GrammarId(240), tables: &TERADATA_TABLES }),
        "AlterSequenceStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(256), tables: &TERADATA_TABLES }),
        "AlterTableDropColumnGrammar" => Some(RootGrammar { grammar_id: GrammarId(262), tables: &TERADATA_TABLES }),
        "AlterTableOptionsGrammar" => Some(RootGrammar { grammar_id: GrammarId(267), tables: &TERADATA_TABLES }),
        "AlterTableStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(294), tables: &TERADATA_TABLES }),
        "AlwaysKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(300), tables: &TERADATA_TABLES }),
        "AmpersandSegment" => Some(RootGrammar { grammar_id: GrammarId(301), tables: &TERADATA_TABLES }),
        "AnalyseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(302), tables: &TERADATA_TABLES }),
        "AnalyzeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(303), tables: &TERADATA_TABLES }),
        "AndKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(304), tables: &TERADATA_TABLES }),
        "AndOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(305), tables: &TERADATA_TABLES }),
        "AnyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(306), tables: &TERADATA_TABLES }),
        "ApplyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(307), tables: &TERADATA_TABLES }),
        "AreKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(308), tables: &TERADATA_TABLES }),
        "ArithmeticBinaryOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(309), tables: &TERADATA_TABLES }),
        "ArrayAccessorSegment" => Some(RootGrammar { grammar_id: GrammarId(320), tables: &TERADATA_TABLES }),
        "ArrayExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(328), tables: &TERADATA_TABLES }),
        "ArrayKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(329), tables: &TERADATA_TABLES }),
        "ArrayLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(330), tables: &TERADATA_TABLES }),
        "ArrayTypeSegment" => Some(RootGrammar { grammar_id: GrammarId(333), tables: &TERADATA_TABLES }),
        "AsAliasOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(334), tables: &TERADATA_TABLES }),
        "AsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(336), tables: &TERADATA_TABLES }),
        "AscKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(337), tables: &TERADATA_TABLES }),
        "AsensitiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(338), tables: &TERADATA_TABLES }),
        "AssertionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(339), tables: &TERADATA_TABLES }),
        "AssignmentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(340), tables: &TERADATA_TABLES }),
        "AsymmetricKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(341), tables: &TERADATA_TABLES }),
        "AtKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(342), tables: &TERADATA_TABLES }),
        "AtomicKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(343), tables: &TERADATA_TABLES }),
        "AttributeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(344), tables: &TERADATA_TABLES }),
        "AttributesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(345), tables: &TERADATA_TABLES }),
        "AuditKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(346), tables: &TERADATA_TABLES }),
        "AuthorizationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(347), tables: &TERADATA_TABLES }),
        "AutoIncrementGrammar" => Some(RootGrammar { grammar_id: GrammarId(348), tables: &TERADATA_TABLES }),
        "Auto_incrementKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(350), tables: &TERADATA_TABLES }),
        "AutoincrementKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(351), tables: &TERADATA_TABLES }),
        "AvgKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(352), tables: &TERADATA_TABLES }),
        "Avg_row_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(353), tables: &TERADATA_TABLES }),
        "BackupKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(354), tables: &TERADATA_TABLES }),
        "BackwardKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(355), tables: &TERADATA_TABLES }),
        "BareFunctionSegment" => Some(RootGrammar { grammar_id: GrammarId(356), tables: &TERADATA_TABLES }),
        "BaseExpressionElementGrammar" => Some(RootGrammar { grammar_id: GrammarId(357), tables: &TERADATA_TABLES }),
        "BaseFileSegment" => Some(RootGrammar { grammar_id: GrammarId(369), tables: &TERADATA_TABLES }),
        "BaseSegment" => Some(RootGrammar { grammar_id: GrammarId(370), tables: &TERADATA_TABLES }),
        "BeforeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(371), tables: &TERADATA_TABLES }),
        "BeginKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(372), tables: &TERADATA_TABLES }),
        "BernoulliKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(373), tables: &TERADATA_TABLES }),
        "BetweenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(374), tables: &TERADATA_TABLES }),
        "BigintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(375), tables: &TERADATA_TABLES }),
        "BinaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(376), tables: &TERADATA_TABLES }),
        "BinaryOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(377), tables: &TERADATA_TABLES }),
        "BinaryOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(382), tables: &TERADATA_TABLES }),
        "BindingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(383), tables: &TERADATA_TABLES }),
        "BitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(384), tables: &TERADATA_TABLES }),
        "Bit_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(385), tables: &TERADATA_TABLES }),
        "BitvarKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(386), tables: &TERADATA_TABLES }),
        "BitwiseAndSegment" => Some(RootGrammar { grammar_id: GrammarId(387), tables: &TERADATA_TABLES }),
        "BitwiseLShiftSegment" => Some(RootGrammar { grammar_id: GrammarId(388), tables: &TERADATA_TABLES }),
        "BitwiseOrSegment" => Some(RootGrammar { grammar_id: GrammarId(391), tables: &TERADATA_TABLES }),
        "BitwiseRShiftSegment" => Some(RootGrammar { grammar_id: GrammarId(392), tables: &TERADATA_TABLES }),
        "BitwiseXorSegment" => Some(RootGrammar { grammar_id: GrammarId(395), tables: &TERADATA_TABLES }),
        "BlobKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(396), tables: &TERADATA_TABLES }),
        "BoolKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(397), tables: &TERADATA_TABLES }),
        "BooleanBinaryOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(398), tables: &TERADATA_TABLES }),
        "BooleanKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(401), tables: &TERADATA_TABLES }),
        "BooleanLiteralGrammar" => Some(RootGrammar { grammar_id: GrammarId(402), tables: &TERADATA_TABLES }),
        "BothKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(405), tables: &TERADATA_TABLES }),
        "BracketedArguments" => Some(RootGrammar { grammar_id: GrammarId(406), tables: &TERADATA_TABLES }),
        "BracketedColumnReferenceListGrammar" => Some(RootGrammar { grammar_id: GrammarId(409), tables: &TERADATA_TABLES }),
        "BracketedSegment" => Some(RootGrammar { grammar_id: GrammarId(412), tables: &TERADATA_TABLES }),
        "BracketedSetExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(413), tables: &TERADATA_TABLES }),
        "BreadthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(415), tables: &TERADATA_TABLES }),
        "BreakKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(416), tables: &TERADATA_TABLES }),
        "BrowseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(417), tables: &TERADATA_TABLES }),
        "BteqFilePathSegment" => Some(RootGrammar { grammar_id: GrammarId(418), tables: &TERADATA_TABLES }),
        "BteqKeyWordSegment" => Some(RootGrammar { grammar_id: GrammarId(422), tables: &TERADATA_TABLES }),
        "BteqStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(440), tables: &TERADATA_TABLES }),
        "BulkKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(454), tables: &TERADATA_TABLES }),
        "ByKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(455), tables: &TERADATA_TABLES }),
        "CTEColumnList" => Some(RootGrammar { grammar_id: GrammarId(456), tables: &TERADATA_TABLES }),
        "CTEDefinitionSegment" => Some(RootGrammar { grammar_id: GrammarId(458), tables: &TERADATA_TABLES }),
        "CacheKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(464), tables: &TERADATA_TABLES }),
        "CallKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(465), tables: &TERADATA_TABLES }),
        "CalledKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(466), tables: &TERADATA_TABLES }),
        "CardinalityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(467), tables: &TERADATA_TABLES }),
        "CascadeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(468), tables: &TERADATA_TABLES }),
        "CascadedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(469), tables: &TERADATA_TABLES }),
        "CaseExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(470), tables: &TERADATA_TABLES }),
        "CaseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(494), tables: &TERADATA_TABLES }),
        "CasespecificKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(495), tables: &TERADATA_TABLES }),
        "CastKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(496), tables: &TERADATA_TABLES }),
        "CastOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(497), tables: &TERADATA_TABLES }),
        "CatalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(498), tables: &TERADATA_TABLES }),
        "Catalog_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(499), tables: &TERADATA_TABLES }),
        "CeilKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(500), tables: &TERADATA_TABLES }),
        "CeilingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(501), tables: &TERADATA_TABLES }),
        "ChainKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(502), tables: &TERADATA_TABLES }),
        "ChangeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(503), tables: &TERADATA_TABLES }),
        "CharCharacterSetGrammar" => Some(RootGrammar { grammar_id: GrammarId(504), tables: &TERADATA_TABLES }),
        "CharKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(517), tables: &TERADATA_TABLES }),
        "Char_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(518), tables: &TERADATA_TABLES }),
        "CharacterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(519), tables: &TERADATA_TABLES }),
        "Character_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(520), tables: &TERADATA_TABLES }),
        "Character_set_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(521), tables: &TERADATA_TABLES }),
        "Character_set_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(522), tables: &TERADATA_TABLES }),
        "Character_set_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(523), tables: &TERADATA_TABLES }),
        "CharacteristicsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(524), tables: &TERADATA_TABLES }),
        "CharactersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(525), tables: &TERADATA_TABLES }),
        "CheckKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(526), tables: &TERADATA_TABLES }),
        "CheckedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(527), tables: &TERADATA_TABLES }),
        "CheckpointKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(528), tables: &TERADATA_TABLES }),
        "ChecksumKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(529), tables: &TERADATA_TABLES }),
        "ClassKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(530), tables: &TERADATA_TABLES }),
        "Class_originKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(531), tables: &TERADATA_TABLES }),
        "ClobKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(532), tables: &TERADATA_TABLES }),
        "CloseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(533), tables: &TERADATA_TABLES }),
        "ClusterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(534), tables: &TERADATA_TABLES }),
        "ClusteredKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(535), tables: &TERADATA_TABLES }),
        "CoalesceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(536), tables: &TERADATA_TABLES }),
        "CobolKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(537), tables: &TERADATA_TABLES }),
        "CodeSegment" => Some(RootGrammar { grammar_id: GrammarId(538), tables: &TERADATA_TABLES }),
        "CollateGrammar" => Some(RootGrammar { grammar_id: GrammarId(539), tables: &TERADATA_TABLES }),
        "CollateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(541), tables: &TERADATA_TABLES }),
        "CollationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(542), tables: &TERADATA_TABLES }),
        "CollationReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(543), tables: &TERADATA_TABLES }),
        "Collation_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(549), tables: &TERADATA_TABLES }),
        "Collation_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(550), tables: &TERADATA_TABLES }),
        "Collation_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(551), tables: &TERADATA_TABLES }),
        "CollectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(552), tables: &TERADATA_TABLES }),
        "ColonDelimiterSegment" => Some(RootGrammar { grammar_id: GrammarId(553), tables: &TERADATA_TABLES }),
        "ColonPrefixSegment" => Some(RootGrammar { grammar_id: GrammarId(554), tables: &TERADATA_TABLES }),
        "ColonSegment" => Some(RootGrammar { grammar_id: GrammarId(555), tables: &TERADATA_TABLES }),
        "ColumnConstraintDefaultGrammar" => Some(RootGrammar { grammar_id: GrammarId(556), tables: &TERADATA_TABLES }),
        "ColumnConstraintSegment" => Some(RootGrammar { grammar_id: GrammarId(561), tables: &TERADATA_TABLES }),
        "ColumnDefinitionSegment" => Some(RootGrammar { grammar_id: GrammarId(589), tables: &TERADATA_TABLES }),
        "ColumnGeneratedGrammar" => Some(RootGrammar { grammar_id: GrammarId(597), tables: &TERADATA_TABLES }),
        "ColumnKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(598), tables: &TERADATA_TABLES }),
        "ColumnReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(599), tables: &TERADATA_TABLES }),
        "Column_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(603), tables: &TERADATA_TABLES }),
        "ColumnsExpressionFunctionContentsSegment" => Some(RootGrammar { grammar_id: GrammarId(604), tables: &TERADATA_TABLES }),
        "ColumnsExpressionFunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(605), tables: &TERADATA_TABLES }),
        "ColumnsExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(606), tables: &TERADATA_TABLES }),
        "ColumnsExpressionNameGrammar" => Some(RootGrammar { grammar_id: GrammarId(607), tables: &TERADATA_TABLES }),
        "ColumnsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(608), tables: &TERADATA_TABLES }),
        "CommaSegment" => Some(RootGrammar { grammar_id: GrammarId(609), tables: &TERADATA_TABLES }),
        "Command_functionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(610), tables: &TERADATA_TABLES }),
        "Command_function_codeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(611), tables: &TERADATA_TABLES }),
        "CommentClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(612), tables: &TERADATA_TABLES }),
        "CommentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(615), tables: &TERADATA_TABLES }),
        "CommentSegment" => Some(RootGrammar { grammar_id: GrammarId(616), tables: &TERADATA_TABLES }),
        "CommitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(617), tables: &TERADATA_TABLES }),
        "CommittedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(618), tables: &TERADATA_TABLES }),
        "ComparisonOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(619), tables: &TERADATA_TABLES }),
        "ComparisonOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(645), tables: &TERADATA_TABLES }),
        "CompletionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(646), tables: &TERADATA_TABLES }),
        "CompositeBinaryOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(647), tables: &TERADATA_TABLES }),
        "CompositeComparisonOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(648), tables: &TERADATA_TABLES }),
        "CompressKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(649), tables: &TERADATA_TABLES }),
        "ComputeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(650), tables: &TERADATA_TABLES }),
        "ConcatSegment" => Some(RootGrammar { grammar_id: GrammarId(651), tables: &TERADATA_TABLES }),
        "ConditionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(654), tables: &TERADATA_TABLES }),
        "Condition_numberKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(655), tables: &TERADATA_TABLES }),
        "ConditionalCrossJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(656), tables: &TERADATA_TABLES }),
        "ConditionalJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(657), tables: &TERADATA_TABLES }),
        "ConnectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(661), tables: &TERADATA_TABLES }),
        "ConnectionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(662), tables: &TERADATA_TABLES }),
        "Connection_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(663), tables: &TERADATA_TABLES }),
        "ConstraintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(664), tables: &TERADATA_TABLES }),
        "Constraint_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(665), tables: &TERADATA_TABLES }),
        "Constraint_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(666), tables: &TERADATA_TABLES }),
        "Constraint_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(667), tables: &TERADATA_TABLES }),
        "ConstraintsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(668), tables: &TERADATA_TABLES }),
        "ConstructorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(669), tables: &TERADATA_TABLES }),
        "ContainsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(670), tables: &TERADATA_TABLES }),
        "ContainstableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(671), tables: &TERADATA_TABLES }),
        "ContinueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(672), tables: &TERADATA_TABLES }),
        "ConversionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(673), tables: &TERADATA_TABLES }),
        "ConvertKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(674), tables: &TERADATA_TABLES }),
        "CopyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(675), tables: &TERADATA_TABLES }),
        "CorrKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(676), tables: &TERADATA_TABLES }),
        "CorrespondingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(677), tables: &TERADATA_TABLES }),
        "CountKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(678), tables: &TERADATA_TABLES }),
        "Covar_popKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(679), tables: &TERADATA_TABLES }),
        "Covar_sampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(680), tables: &TERADATA_TABLES }),
        "CreateCastStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(681), tables: &TERADATA_TABLES }),
        "CreateDatabaseStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(708), tables: &TERADATA_TABLES }),
        "CreateFunctionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(713), tables: &TERADATA_TABLES }),
        "CreateIndexStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(725), tables: &TERADATA_TABLES }),
        "CreateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(738), tables: &TERADATA_TABLES }),
        "CreateModelStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(739), tables: &TERADATA_TABLES }),
        "CreateRoleStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(759), tables: &TERADATA_TABLES }),
        "CreateSchemaStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(764), tables: &TERADATA_TABLES }),
        "CreateSequenceOptionsSegment" => Some(RootGrammar { grammar_id: GrammarId(769), tables: &TERADATA_TABLES }),
        "CreateSequenceStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(789), tables: &TERADATA_TABLES }),
        "CreateTableStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(795), tables: &TERADATA_TABLES }),
        "CreateTriggerStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(827), tables: &TERADATA_TABLES }),
        "CreateUserStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(894), tables: &TERADATA_TABLES }),
        "CreateViewStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(898), tables: &TERADATA_TABLES }),
        "CreatedbKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(911), tables: &TERADATA_TABLES }),
        "CreateroleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(912), tables: &TERADATA_TABLES }),
        "CreateuserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(913), tables: &TERADATA_TABLES }),
        "CrossKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(914), tables: &TERADATA_TABLES }),
        "CsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(915), tables: &TERADATA_TABLES }),
        "CsvKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(916), tables: &TERADATA_TABLES }),
        "CubeFunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(917), tables: &TERADATA_TABLES }),
        "CubeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(918), tables: &TERADATA_TABLES }),
        "CubeRollupClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(919), tables: &TERADATA_TABLES }),
        "Cume_distKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(925), tables: &TERADATA_TABLES }),
        "CurrentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(926), tables: &TERADATA_TABLES }),
        "Current_dateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(927), tables: &TERADATA_TABLES }),
        "Current_default_transform_groupKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(928), tables: &TERADATA_TABLES }),
        "Current_pathKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(929), tables: &TERADATA_TABLES }),
        "Current_roleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(930), tables: &TERADATA_TABLES }),
        "Current_timeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(931), tables: &TERADATA_TABLES }),
        "Current_timestampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(932), tables: &TERADATA_TABLES }),
        "Current_transform_group_for_typeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(933), tables: &TERADATA_TABLES }),
        "Current_userKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(934), tables: &TERADATA_TABLES }),
        "CursorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(935), tables: &TERADATA_TABLES }),
        "Cursor_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(936), tables: &TERADATA_TABLES }),
        "CycleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(937), tables: &TERADATA_TABLES }),
        "DataKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(938), tables: &TERADATA_TABLES }),
        "DatabaseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(939), tables: &TERADATA_TABLES }),
        "DatabaseReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(599), tables: &TERADATA_TABLES }),
        "DatabaseStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(940), tables: &TERADATA_TABLES }),
        "DatabasesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(943), tables: &TERADATA_TABLES }),
        "DatatypeIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(944), tables: &TERADATA_TABLES }),
        "DatatypeSegment" => Some(RootGrammar { grammar_id: GrammarId(948), tables: &TERADATA_TABLES }),
        "DateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(971), tables: &TERADATA_TABLES }),
        "DatePartFunctionName" => Some(RootGrammar { grammar_id: GrammarId(972), tables: &TERADATA_TABLES }),
        "DatePartFunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(973), tables: &TERADATA_TABLES }),
        "DateTimeFunctionContentsSegment" => Some(RootGrammar { grammar_id: GrammarId(974), tables: &TERADATA_TABLES }),
        "DateTimeLiteralGrammar" => Some(RootGrammar { grammar_id: GrammarId(980), tables: &TERADATA_TABLES }),
        "DatetimeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(987), tables: &TERADATA_TABLES }),
        "DatetimeUnitSegment" => Some(RootGrammar { grammar_id: GrammarId(988), tables: &TERADATA_TABLES }),
        "Datetime_interval_codeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(989), tables: &TERADATA_TABLES }),
        "Datetime_interval_precisionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(990), tables: &TERADATA_TABLES }),
        "DayKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(991), tables: &TERADATA_TABLES }),
        "Day_hourKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(992), tables: &TERADATA_TABLES }),
        "Day_microsecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(993), tables: &TERADATA_TABLES }),
        "Day_minuteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(994), tables: &TERADATA_TABLES }),
        "Day_secondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(995), tables: &TERADATA_TABLES }),
        "DayofmonthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(996), tables: &TERADATA_TABLES }),
        "DayofweekKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(997), tables: &TERADATA_TABLES }),
        "DayofyearKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(998), tables: &TERADATA_TABLES }),
        "DaysKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(999), tables: &TERADATA_TABLES }),
        "DbccKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1000), tables: &TERADATA_TABLES }),
        "DeallocateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1001), tables: &TERADATA_TABLES }),
        "DecKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1002), tables: &TERADATA_TABLES }),
        "DecimalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1003), tables: &TERADATA_TABLES }),
        "DeclareKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1004), tables: &TERADATA_TABLES }),
        "Dedent" => Some(RootGrammar { grammar_id: GrammarId(231), tables: &TERADATA_TABLES }),
        "DefaultKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1005), tables: &TERADATA_TABLES }),
        "DefaultValuesGrammar" => Some(RootGrammar { grammar_id: GrammarId(1006), tables: &TERADATA_TABLES }),
        "DefaultsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1009), tables: &TERADATA_TABLES }),
        "DeferrableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1010), tables: &TERADATA_TABLES }),
        "DeferredKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1011), tables: &TERADATA_TABLES }),
        "DefinedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1012), tables: &TERADATA_TABLES }),
        "DefinerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1013), tables: &TERADATA_TABLES }),
        "DegreeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1014), tables: &TERADATA_TABLES }),
        "DelKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1015), tables: &TERADATA_TABLES }),
        "Delay_key_writeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1016), tables: &TERADATA_TABLES }),
        "DelayedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1017), tables: &TERADATA_TABLES }),
        "DeleteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1018), tables: &TERADATA_TABLES }),
        "DeleteStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1019), tables: &TERADATA_TABLES }),
        "DelimiterGrammar" => Some(RootGrammar { grammar_id: GrammarId(1025), tables: &TERADATA_TABLES }),
        "DelimiterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1026), tables: &TERADATA_TABLES }),
        "DelimitersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1027), tables: &TERADATA_TABLES }),
        "Dense_rankKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1028), tables: &TERADATA_TABLES }),
        "DenyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1029), tables: &TERADATA_TABLES }),
        "DepthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1030), tables: &TERADATA_TABLES }),
        "DerefKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1031), tables: &TERADATA_TABLES }),
        "DerivedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1032), tables: &TERADATA_TABLES }),
        "DescKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1033), tables: &TERADATA_TABLES }),
        "DescribeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1034), tables: &TERADATA_TABLES }),
        "DescribeStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1035), tables: &TERADATA_TABLES }),
        "DescriptorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1039), tables: &TERADATA_TABLES }),
        "DestroyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1040), tables: &TERADATA_TABLES }),
        "DestructorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1041), tables: &TERADATA_TABLES }),
        "DeterministicKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1042), tables: &TERADATA_TABLES }),
        "DiagnosticsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1043), tables: &TERADATA_TABLES }),
        "DictionaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1044), tables: &TERADATA_TABLES }),
        "DisableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1045), tables: &TERADATA_TABLES }),
        "DisconnectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1046), tables: &TERADATA_TABLES }),
        "DiskKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1047), tables: &TERADATA_TABLES }),
        "DispatchKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1048), tables: &TERADATA_TABLES }),
        "DistinctKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1049), tables: &TERADATA_TABLES }),
        "DistinctrowKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1050), tables: &TERADATA_TABLES }),
        "DistributedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1051), tables: &TERADATA_TABLES }),
        "DivKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1052), tables: &TERADATA_TABLES }),
        "DivideSegment" => Some(RootGrammar { grammar_id: GrammarId(1053), tables: &TERADATA_TABLES }),
        "DoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1054), tables: &TERADATA_TABLES }),
        "DomainKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1055), tables: &TERADATA_TABLES }),
        "DotSegment" => Some(RootGrammar { grammar_id: GrammarId(1056), tables: &TERADATA_TABLES }),
        "DoubleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1057), tables: &TERADATA_TABLES }),
        "DropBehaviorGrammar" => Some(RootGrammar { grammar_id: GrammarId(1058), tables: &TERADATA_TABLES }),
        "DropCastStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1061), tables: &TERADATA_TABLES }),
        "DropDatabaseStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1069), tables: &TERADATA_TABLES }),
        "DropFunctionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1075), tables: &TERADATA_TABLES }),
        "DropIndexStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1080), tables: &TERADATA_TABLES }),
        "DropKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1086), tables: &TERADATA_TABLES }),
        "DropModelStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1087), tables: &TERADATA_TABLES }),
        "DropRoleStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1092), tables: &TERADATA_TABLES }),
        "DropSchemaStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1097), tables: &TERADATA_TABLES }),
        "DropSequenceStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1103), tables: &TERADATA_TABLES }),
        "DropTableStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1107), tables: &TERADATA_TABLES }),
        "DropTriggerStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1115), tables: &TERADATA_TABLES }),
        "DropTypeStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1120), tables: &TERADATA_TABLES }),
        "DropUserStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1126), tables: &TERADATA_TABLES }),
        "DropViewStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1131), tables: &TERADATA_TABLES }),
        "DualKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1137), tables: &TERADATA_TABLES }),
        "DummyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1138), tables: &TERADATA_TABLES }),
        "DumpKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1139), tables: &TERADATA_TABLES }),
        "DynamicKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1140), tables: &TERADATA_TABLES }),
        "Dynamic_functionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1141), tables: &TERADATA_TABLES }),
        "Dynamic_function_codeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1142), tables: &TERADATA_TABLES }),
        "EachKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1143), tables: &TERADATA_TABLES }),
        "ElementKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1144), tables: &TERADATA_TABLES }),
        "ElseClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1145), tables: &TERADATA_TABLES }),
        "ElseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1148), tables: &TERADATA_TABLES }),
        "ElseifKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1149), tables: &TERADATA_TABLES }),
        "EmptyStructLiteralBracketsSegment" => Some(RootGrammar { grammar_id: GrammarId(1150), tables: &TERADATA_TABLES }),
        "EmptyStructLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(1151), tables: &TERADATA_TABLES }),
        "EnableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1154), tables: &TERADATA_TABLES }),
        "EnclosedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1155), tables: &TERADATA_TABLES }),
        "EncodingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1156), tables: &TERADATA_TABLES }),
        "EncryptedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1157), tables: &TERADATA_TABLES }),
        "End-execKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1158), tables: &TERADATA_TABLES }),
        "EndBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(229), tables: &TERADATA_TABLES }),
        "EndCurlyBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(1159), tables: &TERADATA_TABLES }),
        "EndKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1160), tables: &TERADATA_TABLES }),
        "EndSquareBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(327), tables: &TERADATA_TABLES }),
        "EnumKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1161), tables: &TERADATA_TABLES }),
        "EqualsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1162), tables: &TERADATA_TABLES }),
        "EqualsSegment" => Some(RootGrammar { grammar_id: GrammarId(1163), tables: &TERADATA_TABLES }),
        "EqualsSegment_a" => Some(RootGrammar { grammar_id: GrammarId(1164), tables: &TERADATA_TABLES }),
        "ErrlvlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1165), tables: &TERADATA_TABLES }),
        "ErrorcodeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1166), tables: &TERADATA_TABLES }),
        "EscapeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1167), tables: &TERADATA_TABLES }),
        "EscapedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1168), tables: &TERADATA_TABLES }),
        "EveryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1169), tables: &TERADATA_TABLES }),
        "ExceptKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1170), tables: &TERADATA_TABLES }),
        "ExceptionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1171), tables: &TERADATA_TABLES }),
        "ExclKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1172), tables: &TERADATA_TABLES }),
        "ExcludeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1173), tables: &TERADATA_TABLES }),
        "ExcludingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1174), tables: &TERADATA_TABLES }),
        "ExclusiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1175), tables: &TERADATA_TABLES }),
        "ExecKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1176), tables: &TERADATA_TABLES }),
        "ExecuteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1177), tables: &TERADATA_TABLES }),
        "ExecutionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1178), tables: &TERADATA_TABLES }),
        "ExistingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1179), tables: &TERADATA_TABLES }),
        "ExistsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1180), tables: &TERADATA_TABLES }),
        "ExitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1181), tables: &TERADATA_TABLES }),
        "ExpKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1182), tables: &TERADATA_TABLES }),
        "ExplainKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1183), tables: &TERADATA_TABLES }),
        "ExplainStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1184), tables: &TERADATA_TABLES }),
        "ExportKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1191), tables: &TERADATA_TABLES }),
        "ExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(1192), tables: &TERADATA_TABLES }),
        "Expression_A_Grammar" => Some(RootGrammar { grammar_id: GrammarId(1195), tables: &TERADATA_TABLES }),
        "Expression_A_Unary_Operator_Grammar" => Some(RootGrammar { grammar_id: GrammarId(1221), tables: &TERADATA_TABLES }),
        "Expression_B_Grammar" => Some(RootGrammar { grammar_id: GrammarId(1228), tables: &TERADATA_TABLES }),
        "Expression_B_Unary_Operator_Grammar" => Some(RootGrammar { grammar_id: GrammarId(1238), tables: &TERADATA_TABLES }),
        "Expression_C_Grammar" => Some(RootGrammar { grammar_id: GrammarId(1243), tables: &TERADATA_TABLES }),
        "Expression_D_Grammar" => Some(RootGrammar { grammar_id: GrammarId(1256), tables: &TERADATA_TABLES }),
        "Expression_D_Potential_Select_Statement_Without_Brackets" => Some(RootGrammar { grammar_id: GrammarId(1293), tables: &TERADATA_TABLES }),
        "ExtendedNaturalJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(1301), tables: &TERADATA_TABLES }),
        "ExtensionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1302), tables: &TERADATA_TABLES }),
        "ExtensionReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(599), tables: &TERADATA_TABLES }),
        "ExternalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1303), tables: &TERADATA_TABLES }),
        "ExtractKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1304), tables: &TERADATA_TABLES }),
        "FallbackKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1305), tables: &TERADATA_TABLES }),
        "FalseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1306), tables: &TERADATA_TABLES }),
        "FalseSegment" => Some(RootGrammar { grammar_id: GrammarId(1307), tables: &TERADATA_TABLES }),
        "FetchClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1308), tables: &TERADATA_TABLES }),
        "FetchKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1325), tables: &TERADATA_TABLES }),
        "FieldsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1326), tables: &TERADATA_TABLES }),
        "FileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1327), tables: &TERADATA_TABLES }),
        "FileSegment" => Some(RootGrammar { grammar_id: GrammarId(1328), tables: &TERADATA_TABLES }),
        "FillfactorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1337), tables: &TERADATA_TABLES }),
        "FilterClauseGrammar" => Some(RootGrammar { grammar_id: GrammarId(1338), tables: &TERADATA_TABLES }),
        "FilterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1344), tables: &TERADATA_TABLES }),
        "FinalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1345), tables: &TERADATA_TABLES }),
        "FirstKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1346), tables: &TERADATA_TABLES }),
        "Float4KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1347), tables: &TERADATA_TABLES }),
        "Float8KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1348), tables: &TERADATA_TABLES }),
        "FloatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1349), tables: &TERADATA_TABLES }),
        "FloorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1350), tables: &TERADATA_TABLES }),
        "FlushKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1351), tables: &TERADATA_TABLES }),
        "FollowingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1352), tables: &TERADATA_TABLES }),
        "ForKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1353), tables: &TERADATA_TABLES }),
        "ForceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1354), tables: &TERADATA_TABLES }),
        "ForeignKeyGrammar" => Some(RootGrammar { grammar_id: GrammarId(1355), tables: &TERADATA_TABLES }),
        "ForeignKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1358), tables: &TERADATA_TABLES }),
        "FormatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1359), tables: &TERADATA_TABLES }),
        "FortranKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1360), tables: &TERADATA_TABLES }),
        "ForwardKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1361), tables: &TERADATA_TABLES }),
        "FoundKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1362), tables: &TERADATA_TABLES }),
        "FrameClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1363), tables: &TERADATA_TABLES }),
        "FrameClauseUnitGrammar" => Some(RootGrammar { grammar_id: GrammarId(1382), tables: &TERADATA_TABLES }),
        "FreeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1385), tables: &TERADATA_TABLES }),
        "FreetextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1386), tables: &TERADATA_TABLES }),
        "FreetexttableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1387), tables: &TERADATA_TABLES }),
        "FreezeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1388), tables: &TERADATA_TABLES }),
        "FromClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1389), tables: &TERADATA_TABLES }),
        "FromClauseTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(1393), tables: &TERADATA_TABLES }),
        "FromExpressionElementSegment" => Some(RootGrammar { grammar_id: GrammarId(1410), tables: &TERADATA_TABLES }),
        "FromExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(1433), tables: &TERADATA_TABLES }),
        "FromKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1460), tables: &TERADATA_TABLES }),
        "FromUpdateClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1461), tables: &TERADATA_TABLES }),
        "FullKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1465), tables: &TERADATA_TABLES }),
        "FulltextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1466), tables: &TERADATA_TABLES }),
        "FunctionContentsExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(1467), tables: &TERADATA_TABLES }),
        "FunctionContentsGrammar" => Some(RootGrammar { grammar_id: GrammarId(1468), tables: &TERADATA_TABLES }),
        "FunctionContentsSegment" => Some(RootGrammar { grammar_id: GrammarId(1513), tables: &TERADATA_TABLES }),
        "FunctionDefinitionGrammar" => Some(RootGrammar { grammar_id: GrammarId(1516), tables: &TERADATA_TABLES }),
        "FunctionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1522), tables: &TERADATA_TABLES }),
        "FunctionNameIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(1523), tables: &TERADATA_TABLES }),
        "FunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(1524), tables: &TERADATA_TABLES }),
        "FunctionParameterGrammar" => Some(RootGrammar { grammar_id: GrammarId(1534), tables: &TERADATA_TABLES }),
        "FunctionParameterListGrammar" => Some(RootGrammar { grammar_id: GrammarId(1547), tables: &TERADATA_TABLES }),
        "FunctionSegment" => Some(RootGrammar { grammar_id: GrammarId(1550), tables: &TERADATA_TABLES }),
        "FunctionsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1564), tables: &TERADATA_TABLES }),
        "FusionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1565), tables: &TERADATA_TABLES }),
        "FutureKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1566), tables: &TERADATA_TABLES }),
        "GKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1567), tables: &TERADATA_TABLES }),
        "GeneralKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1568), tables: &TERADATA_TABLES }),
        "GeneratedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1569), tables: &TERADATA_TABLES }),
        "GetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1570), tables: &TERADATA_TABLES }),
        "GlobOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(1571), tables: &TERADATA_TABLES }),
        "GlobalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1572), tables: &TERADATA_TABLES }),
        "GoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1573), tables: &TERADATA_TABLES }),
        "GotoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1574), tables: &TERADATA_TABLES }),
        "GrantKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1575), tables: &TERADATA_TABLES }),
        "GrantStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1576), tables: &TERADATA_TABLES }),
        "GrantedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1614), tables: &TERADATA_TABLES }),
        "GrantsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1615), tables: &TERADATA_TABLES }),
        "GreaterThanOrEqualToSegment" => Some(RootGrammar { grammar_id: GrammarId(1616), tables: &TERADATA_TABLES }),
        "GreaterThanOrEqualToSegment_a" => Some(RootGrammar { grammar_id: GrammarId(1619), tables: &TERADATA_TABLES }),
        "GreaterThanSegment" => Some(RootGrammar { grammar_id: GrammarId(1620), tables: &TERADATA_TABLES }),
        "GreaterThanSegment_a" => Some(RootGrammar { grammar_id: GrammarId(1621), tables: &TERADATA_TABLES }),
        "GreatestKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1622), tables: &TERADATA_TABLES }),
        "GroupByClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1623), tables: &TERADATA_TABLES }),
        "GroupByClauseTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(1637), tables: &TERADATA_TABLES }),
        "GroupKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1646), tables: &TERADATA_TABLES }),
        "GroupingExpressionList" => Some(RootGrammar { grammar_id: GrammarId(1647), tables: &TERADATA_TABLES }),
        "GroupingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1655), tables: &TERADATA_TABLES }),
        "GroupingSetsClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1656), tables: &TERADATA_TABLES }),
        "HandlerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1664), tables: &TERADATA_TABLES }),
        "HashKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1665), tables: &TERADATA_TABLES }),
        "HavingClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1666), tables: &TERADATA_TABLES }),
        "HavingClauseTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(1671), tables: &TERADATA_TABLES }),
        "HavingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1679), tables: &TERADATA_TABLES }),
        "HeaderKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1680), tables: &TERADATA_TABLES }),
        "HeapKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1681), tables: &TERADATA_TABLES }),
        "HierarchyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1682), tables: &TERADATA_TABLES }),
        "High_priorityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1683), tables: &TERADATA_TABLES }),
        "HoldKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1684), tables: &TERADATA_TABLES }),
        "HoldlockKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1685), tables: &TERADATA_TABLES }),
        "HorizontalJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(1686), tables: &TERADATA_TABLES }),
        "HostKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1687), tables: &TERADATA_TABLES }),
        "HostsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1688), tables: &TERADATA_TABLES }),
        "HourKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1689), tables: &TERADATA_TABLES }),
        "Hour_microsecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1690), tables: &TERADATA_TABLES }),
        "Hour_minuteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1691), tables: &TERADATA_TABLES }),
        "Hour_secondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1692), tables: &TERADATA_TABLES }),
        "IdentifiedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1693), tables: &TERADATA_TABLES }),
        "IdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(1694), tables: &TERADATA_TABLES }),
        "IdentityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1695), tables: &TERADATA_TABLES }),
        "Identity_insertKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1696), tables: &TERADATA_TABLES }),
        "IdentitycolKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1697), tables: &TERADATA_TABLES }),
        "IfExistsGrammar" => Some(RootGrammar { grammar_id: GrammarId(1698), tables: &TERADATA_TABLES }),
        "IfKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1701), tables: &TERADATA_TABLES }),
        "IfNotExistsGrammar" => Some(RootGrammar { grammar_id: GrammarId(1702), tables: &TERADATA_TABLES }),
        "IgnoreKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1706), tables: &TERADATA_TABLES }),
        "IgnoreRespectNullsGrammar" => Some(RootGrammar { grammar_id: GrammarId(1707), tables: &TERADATA_TABLES }),
        "IlikeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1712), tables: &TERADATA_TABLES }),
        "ImmediateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1713), tables: &TERADATA_TABLES }),
        "ImmutableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1714), tables: &TERADATA_TABLES }),
        "ImplementationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1715), tables: &TERADATA_TABLES }),
        "ImplicitIndent" => Some(RootGrammar { grammar_id: GrammarId(473), tables: &TERADATA_TABLES }),
        "ImplicitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1716), tables: &TERADATA_TABLES }),
        "ImportKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1717), tables: &TERADATA_TABLES }),
        "ImportedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1718), tables: &TERADATA_TABLES }),
        "InKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1719), tables: &TERADATA_TABLES }),
        "InOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(1720), tables: &TERADATA_TABLES }),
        "IncludeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1730), tables: &TERADATA_TABLES }),
        "IncludingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1731), tables: &TERADATA_TABLES }),
        "IncrementKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1732), tables: &TERADATA_TABLES }),
        "Indent" => Some(RootGrammar { grammar_id: GrammarId(221), tables: &TERADATA_TABLES }),
        "IndexColumnDefinitionSegment" => Some(RootGrammar { grammar_id: GrammarId(1733), tables: &TERADATA_TABLES }),
        "IndexKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1738), tables: &TERADATA_TABLES }),
        "IndexReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(599), tables: &TERADATA_TABLES }),
        "IndicatorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1739), tables: &TERADATA_TABLES }),
        "InfileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1740), tables: &TERADATA_TABLES }),
        "InfixKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1741), tables: &TERADATA_TABLES }),
        "InheritKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1742), tables: &TERADATA_TABLES }),
        "InheritsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1743), tables: &TERADATA_TABLES }),
        "InitialKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1744), tables: &TERADATA_TABLES }),
        "InitializeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1745), tables: &TERADATA_TABLES }),
        "InitiallyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1746), tables: &TERADATA_TABLES }),
        "InnerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1747), tables: &TERADATA_TABLES }),
        "InoutKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1748), tables: &TERADATA_TABLES }),
        "InputKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1749), tables: &TERADATA_TABLES }),
        "InsensitiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1750), tables: &TERADATA_TABLES }),
        "InsertKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1751), tables: &TERADATA_TABLES }),
        "InsertStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1752), tables: &TERADATA_TABLES }),
        "Insert_idKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1763), tables: &TERADATA_TABLES }),
        "InstanceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1764), tables: &TERADATA_TABLES }),
        "InstantiableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1765), tables: &TERADATA_TABLES }),
        "InsteadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1766), tables: &TERADATA_TABLES }),
        "Int1KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1767), tables: &TERADATA_TABLES }),
        "Int2KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1768), tables: &TERADATA_TABLES }),
        "Int3KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1769), tables: &TERADATA_TABLES }),
        "Int4KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1770), tables: &TERADATA_TABLES }),
        "Int8KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1771), tables: &TERADATA_TABLES }),
        "IntKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1772), tables: &TERADATA_TABLES }),
        "IntegerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1773), tables: &TERADATA_TABLES }),
        "IntegrationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1774), tables: &TERADATA_TABLES }),
        "IntegrationsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1775), tables: &TERADATA_TABLES }),
        "IntersectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1776), tables: &TERADATA_TABLES }),
        "IntersectionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1777), tables: &TERADATA_TABLES }),
        "IntervalExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(1778), tables: &TERADATA_TABLES }),
        "IntervalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1792), tables: &TERADATA_TABLES }),
        "IntoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1793), tables: &TERADATA_TABLES }),
        "InvokerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1794), tables: &TERADATA_TABLES }),
        "IsClauseGrammar" => Some(RootGrammar { grammar_id: GrammarId(1795), tables: &TERADATA_TABLES }),
        "IsDistinctFromGrammar" => Some(RootGrammar { grammar_id: GrammarId(1801), tables: &TERADATA_TABLES }),
        "IsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1806), tables: &TERADATA_TABLES }),
        "IsNullGrammar" => Some(RootGrammar { grammar_id: GrammarId(1807), tables: &TERADATA_TABLES }),
        "IsamKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1808), tables: &TERADATA_TABLES }),
        "IsnullKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1809), tables: &TERADATA_TABLES }),
        "IsolationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1810), tables: &TERADATA_TABLES }),
        "IterateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1811), tables: &TERADATA_TABLES }),
        "JoinClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1812), tables: &TERADATA_TABLES }),
        "JoinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1834), tables: &TERADATA_TABLES }),
        "JoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(1835), tables: &TERADATA_TABLES }),
        "JoinLikeClauseGrammar" => Some(RootGrammar { grammar_id: GrammarId(1837), tables: &TERADATA_TABLES }),
        "JoinOnConditionSegment" => Some(RootGrammar { grammar_id: GrammarId(1838), tables: &TERADATA_TABLES }),
        "JoinTypeKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(1845), tables: &TERADATA_TABLES }),
        "JoinUsingConditionGrammar" => Some(RootGrammar { grammar_id: GrammarId(1853), tables: &TERADATA_TABLES }),
        "JournalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1858), tables: &TERADATA_TABLES }),
        "JsonKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1859), tables: &TERADATA_TABLES }),
        "KKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1860), tables: &TERADATA_TABLES }),
        "KeyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1861), tables: &TERADATA_TABLES }),
        "Key_memberKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1862), tables: &TERADATA_TABLES }),
        "Key_typeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1863), tables: &TERADATA_TABLES }),
        "KeysKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1864), tables: &TERADATA_TABLES }),
        "KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1865), tables: &TERADATA_TABLES }),
        "KillKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1866), tables: &TERADATA_TABLES }),
        "LabelKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1867), tables: &TERADATA_TABLES }),
        "LancompilerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1868), tables: &TERADATA_TABLES }),
        "LanguageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1869), tables: &TERADATA_TABLES }),
        "LargeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1870), tables: &TERADATA_TABLES }),
        "LastKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1871), tables: &TERADATA_TABLES }),
        "Last_insert_idKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1872), tables: &TERADATA_TABLES }),
        "LateralColumnAliasExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(1873), tables: &TERADATA_TABLES }),
        "LateralKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1874), tables: &TERADATA_TABLES }),
        "LeadingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1875), tables: &TERADATA_TABLES }),
        "LeastKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1876), tables: &TERADATA_TABLES }),
        "LeaveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1877), tables: &TERADATA_TABLES }),
        "LeftKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1878), tables: &TERADATA_TABLES }),
        "LengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1879), tables: &TERADATA_TABLES }),
        "LessKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1880), tables: &TERADATA_TABLES }),
        "LessThanOrEqualToSegment" => Some(RootGrammar { grammar_id: GrammarId(1881), tables: &TERADATA_TABLES }),
        "LessThanOrEqualToSegment_a" => Some(RootGrammar { grammar_id: GrammarId(1884), tables: &TERADATA_TABLES }),
        "LessThanSegment" => Some(RootGrammar { grammar_id: GrammarId(1885), tables: &TERADATA_TABLES }),
        "LessThanSegment_a" => Some(RootGrammar { grammar_id: GrammarId(1886), tables: &TERADATA_TABLES }),
        "LevelKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1887), tables: &TERADATA_TABLES }),
        "LikeExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(1888), tables: &TERADATA_TABLES }),
        "LikeGrammar" => Some(RootGrammar { grammar_id: GrammarId(1896), tables: &TERADATA_TABLES }),
        "LikeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1900), tables: &TERADATA_TABLES }),
        "LikeOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(1901), tables: &TERADATA_TABLES }),
        "LimitClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1902), tables: &TERADATA_TABLES }),
        "LimitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1919), tables: &TERADATA_TABLES }),
        "LinenoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1920), tables: &TERADATA_TABLES }),
        "LinesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1921), tables: &TERADATA_TABLES }),
        "ListComprehensionGrammar" => Some(RootGrammar { grammar_id: GrammarId(1922), tables: &TERADATA_TABLES }),
        "ListenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1923), tables: &TERADATA_TABLES }),
        "LiteralGrammar" => Some(RootGrammar { grammar_id: GrammarId(1924), tables: &TERADATA_TABLES }),
        "LiteralKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1934), tables: &TERADATA_TABLES }),
        "LiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(1935), tables: &TERADATA_TABLES }),
        "LnKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1936), tables: &TERADATA_TABLES }),
        "LoadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1937), tables: &TERADATA_TABLES }),
        "LocalAliasSegment" => Some(RootGrammar { grammar_id: GrammarId(1938), tables: &TERADATA_TABLES }),
        "LocalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1939), tables: &TERADATA_TABLES }),
        "LocaltimeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1940), tables: &TERADATA_TABLES }),
        "LocaltimestampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1941), tables: &TERADATA_TABLES }),
        "LocationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1942), tables: &TERADATA_TABLES }),
        "LocatorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1943), tables: &TERADATA_TABLES }),
        "LockKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1944), tables: &TERADATA_TABLES }),
        "LockingClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1945), tables: &TERADATA_TABLES }),
        "LockingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1981), tables: &TERADATA_TABLES }),
        "LocksKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1982), tables: &TERADATA_TABLES }),
        "LoginKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1983), tables: &TERADATA_TABLES }),
        "LogoffKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1984), tables: &TERADATA_TABLES }),
        "LogonKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1985), tables: &TERADATA_TABLES }),
        "LogsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1986), tables: &TERADATA_TABLES }),
        "LongKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1987), tables: &TERADATA_TABLES }),
        "LongblobKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1988), tables: &TERADATA_TABLES }),
        "LongtextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1989), tables: &TERADATA_TABLES }),
        "LoopKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1990), tables: &TERADATA_TABLES }),
        "Low_priorityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1991), tables: &TERADATA_TABLES }),
        "LowerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1992), tables: &TERADATA_TABLES }),
        "MKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1993), tables: &TERADATA_TABLES }),
        "MLTableExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(1994), tables: &TERADATA_TABLES }),
        "MacroKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2006), tables: &TERADATA_TABLES }),
        "ManageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2007), tables: &TERADATA_TABLES }),
        "MapKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2008), tables: &TERADATA_TABLES }),
        "MapTypeSegment" => Some(RootGrammar { grammar_id: GrammarId(2009), tables: &TERADATA_TABLES }),
        "MaskingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2010), tables: &TERADATA_TABLES }),
        "MatchConditionSegment" => Some(RootGrammar { grammar_id: GrammarId(2011), tables: &TERADATA_TABLES }),
        "MatchKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2012), tables: &TERADATA_TABLES }),
        "MatchedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2013), tables: &TERADATA_TABLES }),
        "MaterializedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2014), tables: &TERADATA_TABLES }),
        "MaxKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2015), tables: &TERADATA_TABLES }),
        "Max_rowsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2016), tables: &TERADATA_TABLES }),
        "MaxextentsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2017), tables: &TERADATA_TABLES }),
        "MaxintervalsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2018), tables: &TERADATA_TABLES }),
        "MaxvalueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2019), tables: &TERADATA_TABLES }),
        "MaxvaluelengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2020), tables: &TERADATA_TABLES }),
        "MediumblobKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2021), tables: &TERADATA_TABLES }),
        "MediumintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2022), tables: &TERADATA_TABLES }),
        "MediumtextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2023), tables: &TERADATA_TABLES }),
        "MeetsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2024), tables: &TERADATA_TABLES }),
        "MemberKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2025), tables: &TERADATA_TABLES }),
        "MergeDeleteClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2026), tables: &TERADATA_TABLES }),
        "MergeInsertClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2027), tables: &TERADATA_TABLES }),
        "MergeIntoLiteralGrammar" => Some(RootGrammar { grammar_id: GrammarId(2031), tables: &TERADATA_TABLES }),
        "MergeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2034), tables: &TERADATA_TABLES }),
        "MergeMatchSegment" => Some(RootGrammar { grammar_id: GrammarId(2035), tables: &TERADATA_TABLES }),
        "MergeMatchedClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2038), tables: &TERADATA_TABLES }),
        "MergeNotMatchedClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2048), tables: &TERADATA_TABLES }),
        "MergeStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2057), tables: &TERADATA_TABLES }),
        "MergeUpdateClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2077), tables: &TERADATA_TABLES }),
        "MergeblockratioKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2080), tables: &TERADATA_TABLES }),
        "Message_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2081), tables: &TERADATA_TABLES }),
        "Message_octet_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2082), tables: &TERADATA_TABLES }),
        "Message_textKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2083), tables: &TERADATA_TABLES }),
        "MethodKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2084), tables: &TERADATA_TABLES }),
        "MiddleintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2085), tables: &TERADATA_TABLES }),
        "MillisecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2086), tables: &TERADATA_TABLES }),
        "MinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2087), tables: &TERADATA_TABLES }),
        "Min_rowsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2088), tables: &TERADATA_TABLES }),
        "MinusKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2089), tables: &TERADATA_TABLES }),
        "MinusSegment" => Some(RootGrammar { grammar_id: GrammarId(2090), tables: &TERADATA_TABLES }),
        "MinuteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2091), tables: &TERADATA_TABLES }),
        "Minute_microsecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2092), tables: &TERADATA_TABLES }),
        "Minute_secondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2093), tables: &TERADATA_TABLES }),
        "MinvalueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2094), tables: &TERADATA_TABLES }),
        "MlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2095), tables: &TERADATA_TABLES }),
        "MlslabelKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2096), tables: &TERADATA_TABLES }),
        "ModKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2097), tables: &TERADATA_TABLES }),
        "ModeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2098), tables: &TERADATA_TABLES }),
        "ModelKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2099), tables: &TERADATA_TABLES }),
        "ModifiesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2100), tables: &TERADATA_TABLES }),
        "ModifyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2101), tables: &TERADATA_TABLES }),
        "ModuleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2102), tables: &TERADATA_TABLES }),
        "ModuloSegment" => Some(RootGrammar { grammar_id: GrammarId(2103), tables: &TERADATA_TABLES }),
        "MonitorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2104), tables: &TERADATA_TABLES }),
        "MonthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2105), tables: &TERADATA_TABLES }),
        "MonthnameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2106), tables: &TERADATA_TABLES }),
        "MoreKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2107), tables: &TERADATA_TABLES }),
        "MoveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2108), tables: &TERADATA_TABLES }),
        "MultiplySegment" => Some(RootGrammar { grammar_id: GrammarId(2109), tables: &TERADATA_TABLES }),
        "MultisetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2110), tables: &TERADATA_TABLES }),
        "MumpsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2111), tables: &TERADATA_TABLES }),
        "MyisamKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2112), tables: &TERADATA_TABLES }),
        "NakedIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(2113), tables: &TERADATA_TABLES }),
        "NameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2114), tables: &TERADATA_TABLES }),
        "NamedWindowExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(2115), tables: &TERADATA_TABLES }),
        "NamedWindowSegment" => Some(RootGrammar { grammar_id: GrammarId(2122), tables: &TERADATA_TABLES }),
        "NamesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2126), tables: &TERADATA_TABLES }),
        "NanKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2127), tables: &TERADATA_TABLES }),
        "NanLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(2128), tables: &TERADATA_TABLES }),
        "NationalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2129), tables: &TERADATA_TABLES }),
        "NaturalJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(2130), tables: &TERADATA_TABLES }),
        "NaturalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2133), tables: &TERADATA_TABLES }),
        "NcharKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2134), tables: &TERADATA_TABLES }),
        "NclobKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2135), tables: &TERADATA_TABLES }),
        "NegativeSegment" => Some(RootGrammar { grammar_id: GrammarId(2136), tables: &TERADATA_TABLES }),
        "NestedJoinGrammar" => Some(RootGrammar { grammar_id: GrammarId(2137), tables: &TERADATA_TABLES }),
        "NestingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2138), tables: &TERADATA_TABLES }),
        "NewKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2139), tables: &TERADATA_TABLES }),
        "NewlineSegment" => Some(RootGrammar { grammar_id: GrammarId(2140), tables: &TERADATA_TABLES }),
        "NextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2141), tables: &TERADATA_TABLES }),
        "NoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2142), tables: &TERADATA_TABLES }),
        "No_write_to_binlogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2143), tables: &TERADATA_TABLES }),
        "NoauditKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2144), tables: &TERADATA_TABLES }),
        "NocacheKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2145), tables: &TERADATA_TABLES }),
        "NocheckKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2146), tables: &TERADATA_TABLES }),
        "NocompressKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2147), tables: &TERADATA_TABLES }),
        "NocreatedbKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2148), tables: &TERADATA_TABLES }),
        "NocreateroleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2149), tables: &TERADATA_TABLES }),
        "NocreateuserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2150), tables: &TERADATA_TABLES }),
        "NocycleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2151), tables: &TERADATA_TABLES }),
        "NoinheritKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2152), tables: &TERADATA_TABLES }),
        "NologinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2153), tables: &TERADATA_TABLES }),
        "NonSetSelectableGrammar" => Some(RootGrammar { grammar_id: GrammarId(2154), tables: &TERADATA_TABLES }),
        "NonStandardJoinTypeKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(2164), tables: &TERADATA_TABLES }),
        "NonWithNonSelectableGrammar" => Some(RootGrammar { grammar_id: GrammarId(2165), tables: &TERADATA_TABLES }),
        "NonWithSelectableGrammar" => Some(RootGrammar { grammar_id: GrammarId(2170), tables: &TERADATA_TABLES }),
        "NonclusteredKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2176), tables: &TERADATA_TABLES }),
        "NoneKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2177), tables: &TERADATA_TABLES }),
        "NoorderKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2178), tables: &TERADATA_TABLES }),
        "NormalizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2179), tables: &TERADATA_TABLES }),
        "NormalizedGrammar" => Some(RootGrammar { grammar_id: GrammarId(2180), tables: &TERADATA_TABLES }),
        "NormalizedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2181), tables: &TERADATA_TABLES }),
        "NosuperuserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2182), tables: &TERADATA_TABLES }),
        "NotEnforcedGrammar" => Some(RootGrammar { grammar_id: GrammarId(2183), tables: &TERADATA_TABLES }),
        "NotEqualToSegment" => Some(RootGrammar { grammar_id: GrammarId(2184), tables: &TERADATA_TABLES }),
        "NotEqualToSegment_a" => Some(RootGrammar { grammar_id: GrammarId(2191), tables: &TERADATA_TABLES }),
        "NotEqualToSegment_b" => Some(RootGrammar { grammar_id: GrammarId(2192), tables: &TERADATA_TABLES }),
        "NotEqualToSegment_c" => Some(RootGrammar { grammar_id: GrammarId(2195), tables: &TERADATA_TABLES }),
        "NotKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2198), tables: &TERADATA_TABLES }),
        "NotNullGrammar" => Some(RootGrammar { grammar_id: GrammarId(2199), tables: &TERADATA_TABLES }),
        "NotOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(2200), tables: &TERADATA_TABLES }),
        "NothingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2201), tables: &TERADATA_TABLES }),
        "NotifyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2202), tables: &TERADATA_TABLES }),
        "NotnullKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2203), tables: &TERADATA_TABLES }),
        "NowaitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2204), tables: &TERADATA_TABLES }),
        "NullKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2205), tables: &TERADATA_TABLES }),
        "NullLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(2206), tables: &TERADATA_TABLES }),
        "NullableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2207), tables: &TERADATA_TABLES }),
        "NullifKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2208), tables: &TERADATA_TABLES }),
        "NullsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2209), tables: &TERADATA_TABLES }),
        "NumberKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2210), tables: &TERADATA_TABLES }),
        "NumericKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2211), tables: &TERADATA_TABLES }),
        "NumericLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(2212), tables: &TERADATA_TABLES }),
        "ObjectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2213), tables: &TERADATA_TABLES }),
        "ObjectLiteralElementSegment" => Some(RootGrammar { grammar_id: GrammarId(2214), tables: &TERADATA_TABLES }),
        "ObjectLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(2218), tables: &TERADATA_TABLES }),
        "ObjectReferenceDelimiterGrammar" => Some(RootGrammar { grammar_id: GrammarId(2222), tables: &TERADATA_TABLES }),
        "ObjectReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(599), tables: &TERADATA_TABLES }),
        "ObjectReferenceTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(2227), tables: &TERADATA_TABLES }),
        "ObjectsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2239), tables: &TERADATA_TABLES }),
        "Octet_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2240), tables: &TERADATA_TABLES }),
        "OctetsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2241), tables: &TERADATA_TABLES }),
        "OfKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2242), tables: &TERADATA_TABLES }),
        "OffKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2243), tables: &TERADATA_TABLES }),
        "OfflineKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2244), tables: &TERADATA_TABLES }),
        "OffsetClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2245), tables: &TERADATA_TABLES }),
        "OffsetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2254), tables: &TERADATA_TABLES }),
        "OffsetsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2255), tables: &TERADATA_TABLES }),
        "OidsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2256), tables: &TERADATA_TABLES }),
        "OldKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2257), tables: &TERADATA_TABLES }),
        "OnKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2258), tables: &TERADATA_TABLES }),
        "OnlineKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2259), tables: &TERADATA_TABLES }),
        "OnlyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2260), tables: &TERADATA_TABLES }),
        "OpenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2261), tables: &TERADATA_TABLES }),
        "OpendatasourceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2262), tables: &TERADATA_TABLES }),
        "OpenqueryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2263), tables: &TERADATA_TABLES }),
        "OpenrowsetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2264), tables: &TERADATA_TABLES }),
        "OpenxmlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2265), tables: &TERADATA_TABLES }),
        "OperateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2266), tables: &TERADATA_TABLES }),
        "OperationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2267), tables: &TERADATA_TABLES }),
        "OperatorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2268), tables: &TERADATA_TABLES }),
        "OptimizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2269), tables: &TERADATA_TABLES }),
        "OptionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2270), tables: &TERADATA_TABLES }),
        "OptionallyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2271), tables: &TERADATA_TABLES }),
        "OptionsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2272), tables: &TERADATA_TABLES }),
        "OrKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2273), tables: &TERADATA_TABLES }),
        "OrOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(2274), tables: &TERADATA_TABLES }),
        "OrReplaceGrammar" => Some(RootGrammar { grammar_id: GrammarId(2275), tables: &TERADATA_TABLES }),
        "OrderByClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2278), tables: &TERADATA_TABLES }),
        "OrderByClauseTerminators" => Some(RootGrammar { grammar_id: GrammarId(2298), tables: &TERADATA_TABLES }),
        "OrderKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2306), tables: &TERADATA_TABLES }),
        "OrderNoOrderGrammar" => Some(RootGrammar { grammar_id: GrammarId(2307), tables: &TERADATA_TABLES }),
        "OrderingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2310), tables: &TERADATA_TABLES }),
        "OrdinalityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2311), tables: &TERADATA_TABLES }),
        "OthersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2312), tables: &TERADATA_TABLES }),
        "OutKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2313), tables: &TERADATA_TABLES }),
        "OuterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2314), tables: &TERADATA_TABLES }),
        "OutfileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2315), tables: &TERADATA_TABLES }),
        "OutputKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2316), tables: &TERADATA_TABLES }),
        "OverClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2317), tables: &TERADATA_TABLES }),
        "OverKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2324), tables: &TERADATA_TABLES }),
        "OverlapsClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2325), tables: &TERADATA_TABLES }),
        "OverlapsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2334), tables: &TERADATA_TABLES }),
        "OverlapsOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(2335), tables: &TERADATA_TABLES }),
        "OverlayKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2336), tables: &TERADATA_TABLES }),
        "OverrideKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2337), tables: &TERADATA_TABLES }),
        "OverridingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2338), tables: &TERADATA_TABLES }),
        "OverwriteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2339), tables: &TERADATA_TABLES }),
        "OwnerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2340), tables: &TERADATA_TABLES }),
        "OwnershipKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2341), tables: &TERADATA_TABLES }),
        "Pack_keysKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2342), tables: &TERADATA_TABLES }),
        "PadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2343), tables: &TERADATA_TABLES }),
        "ParameterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2344), tables: &TERADATA_TABLES }),
        "ParameterNameSegment" => Some(RootGrammar { grammar_id: GrammarId(2345), tables: &TERADATA_TABLES }),
        "ParameterSegment" => Some(RootGrammar { grammar_id: GrammarId(2346), tables: &TERADATA_TABLES }),
        "Parameter_modeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2347), tables: &TERADATA_TABLES }),
        "Parameter_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2348), tables: &TERADATA_TABLES }),
        "Parameter_ordinal_positionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2349), tables: &TERADATA_TABLES }),
        "Parameter_specific_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2350), tables: &TERADATA_TABLES }),
        "Parameter_specific_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2351), tables: &TERADATA_TABLES }),
        "Parameter_specific_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2352), tables: &TERADATA_TABLES }),
        "ParametersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2353), tables: &TERADATA_TABLES }),
        "PartialKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2354), tables: &TERADATA_TABLES }),
        "PartitionClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2355), tables: &TERADATA_TABLES }),
        "PartitionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2362), tables: &TERADATA_TABLES }),
        "PascalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2363), tables: &TERADATA_TABLES }),
        "PasswordKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2364), tables: &TERADATA_TABLES }),
        "PathKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2365), tables: &TERADATA_TABLES }),
        "PathSegment" => Some(RootGrammar { grammar_id: GrammarId(2366), tables: &TERADATA_TABLES }),
        "PatternMatchingGrammar" => Some(RootGrammar { grammar_id: GrammarId(2373), tables: &TERADATA_TABLES }),
        "PctfreeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2374), tables: &TERADATA_TABLES }),
        "PercentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2375), tables: &TERADATA_TABLES }),
        "Percent_rankKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2376), tables: &TERADATA_TABLES }),
        "Percentile_contKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2377), tables: &TERADATA_TABLES }),
        "Percentile_discKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2378), tables: &TERADATA_TABLES }),
        "PipeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2379), tables: &TERADATA_TABLES }),
        "PipeSegment" => Some(RootGrammar { grammar_id: GrammarId(2380), tables: &TERADATA_TABLES }),
        "PlacingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2381), tables: &TERADATA_TABLES }),
        "PlanKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2382), tables: &TERADATA_TABLES }),
        "PliKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2383), tables: &TERADATA_TABLES }),
        "PlusSegment" => Some(RootGrammar { grammar_id: GrammarId(2384), tables: &TERADATA_TABLES }),
        "PolicyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2385), tables: &TERADATA_TABLES }),
        "PositionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2386), tables: &TERADATA_TABLES }),
        "PositiveSegment" => Some(RootGrammar { grammar_id: GrammarId(2387), tables: &TERADATA_TABLES }),
        "PostFunctionGrammar" => Some(RootGrammar { grammar_id: GrammarId(2388), tables: &TERADATA_TABLES }),
        "PostTableExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(2391), tables: &TERADATA_TABLES }),
        "PostfixKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2392), tables: &TERADATA_TABLES }),
        "PowerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2393), tables: &TERADATA_TABLES }),
        "PreTableFunctionKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(2394), tables: &TERADATA_TABLES }),
        "PrecedingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2395), tables: &TERADATA_TABLES }),
        "PrecisionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2396), tables: &TERADATA_TABLES }),
        "PrefixKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2397), tables: &TERADATA_TABLES }),
        "PreorderKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2398), tables: &TERADATA_TABLES }),
        "PrepareKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2399), tables: &TERADATA_TABLES }),
        "PreparedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2400), tables: &TERADATA_TABLES }),
        "PreserveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2401), tables: &TERADATA_TABLES }),
        "PrimaryKeyGrammar" => Some(RootGrammar { grammar_id: GrammarId(2402), tables: &TERADATA_TABLES }),
        "PrimaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2405), tables: &TERADATA_TABLES }),
        "PrintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2406), tables: &TERADATA_TABLES }),
        "PriorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2407), tables: &TERADATA_TABLES }),
        "PrivilegesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2408), tables: &TERADATA_TABLES }),
        "ProcKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2409), tables: &TERADATA_TABLES }),
        "ProceduralKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2410), tables: &TERADATA_TABLES }),
        "ProcedureKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2411), tables: &TERADATA_TABLES }),
        "ProceduresKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2412), tables: &TERADATA_TABLES }),
        "ProcessKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2413), tables: &TERADATA_TABLES }),
        "ProcesslistKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2414), tables: &TERADATA_TABLES }),
        "ProfileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2415), tables: &TERADATA_TABLES }),
        "ProtectionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2416), tables: &TERADATA_TABLES }),
        "PublicKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2417), tables: &TERADATA_TABLES }),
        "PurgeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2418), tables: &TERADATA_TABLES }),
        "QualifiedNumericLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(2419), tables: &TERADATA_TABLES }),
        "QualifyClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2422), tables: &TERADATA_TABLES }),
        "QualifyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2427), tables: &TERADATA_TABLES }),
        "QuantifiedComparisonOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(2428), tables: &TERADATA_TABLES }),
        "QuarterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2436), tables: &TERADATA_TABLES }),
        "Query_bandKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2437), tables: &TERADATA_TABLES }),
        "QuitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2438), tables: &TERADATA_TABLES }),
        "QuoteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2439), tables: &TERADATA_TABLES }),
        "QuotedIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(2440), tables: &TERADATA_TABLES }),
        "QuotedLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(2441), tables: &TERADATA_TABLES }),
        "Raid0KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2442), tables: &TERADATA_TABLES }),
        "RaiserrorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2443), tables: &TERADATA_TABLES }),
        "RangeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2444), tables: &TERADATA_TABLES }),
        "RankKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2445), tables: &TERADATA_TABLES }),
        "RawEqualsSegment" => Some(RootGrammar { grammar_id: GrammarId(2446), tables: &TERADATA_TABLES }),
        "RawGreaterThanSegment" => Some(RootGrammar { grammar_id: GrammarId(2447), tables: &TERADATA_TABLES }),
        "RawKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2448), tables: &TERADATA_TABLES }),
        "RawLessThanSegment" => Some(RootGrammar { grammar_id: GrammarId(2449), tables: &TERADATA_TABLES }),
        "RawNotSegment" => Some(RootGrammar { grammar_id: GrammarId(2450), tables: &TERADATA_TABLES }),
        "ReadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2451), tables: &TERADATA_TABLES }),
        "ReadsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2452), tables: &TERADATA_TABLES }),
        "ReadtextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2453), tables: &TERADATA_TABLES }),
        "RealKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2454), tables: &TERADATA_TABLES }),
        "RecheckKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2455), tables: &TERADATA_TABLES }),
        "ReconfigureKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2456), tables: &TERADATA_TABLES }),
        "RecursiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2457), tables: &TERADATA_TABLES }),
        "RefKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2458), tables: &TERADATA_TABLES }),
        "ReferenceDefinitionGrammar" => Some(RootGrammar { grammar_id: GrammarId(2459), tables: &TERADATA_TABLES }),
        "ReferenceMatchGrammar" => Some(RootGrammar { grammar_id: GrammarId(2473), tables: &TERADATA_TABLES }),
        "Reference_usageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2479), tables: &TERADATA_TABLES }),
        "ReferencesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2480), tables: &TERADATA_TABLES }),
        "ReferencingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2481), tables: &TERADATA_TABLES }),
        "ReferentialActionGrammar" => Some(RootGrammar { grammar_id: GrammarId(2482), tables: &TERADATA_TABLES }),
        "RegexpKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2494), tables: &TERADATA_TABLES }),
        "Regr_avgxKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2495), tables: &TERADATA_TABLES }),
        "Regr_avgyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2496), tables: &TERADATA_TABLES }),
        "Regr_countKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2497), tables: &TERADATA_TABLES }),
        "Regr_interceptKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2498), tables: &TERADATA_TABLES }),
        "Regr_r2KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2499), tables: &TERADATA_TABLES }),
        "Regr_slopeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2500), tables: &TERADATA_TABLES }),
        "Regr_sxxKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2501), tables: &TERADATA_TABLES }),
        "Regr_sxyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2502), tables: &TERADATA_TABLES }),
        "Regr_syyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2503), tables: &TERADATA_TABLES }),
        "ReindexKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2504), tables: &TERADATA_TABLES }),
        "RelativeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2505), tables: &TERADATA_TABLES }),
        "ReleaseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2506), tables: &TERADATA_TABLES }),
        "ReloadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2507), tables: &TERADATA_TABLES }),
        "RenameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2508), tables: &TERADATA_TABLES }),
        "RepeatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2509), tables: &TERADATA_TABLES }),
        "RepeatableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2510), tables: &TERADATA_TABLES }),
        "ReplaceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2511), tables: &TERADATA_TABLES }),
        "ReplicationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2512), tables: &TERADATA_TABLES }),
        "RequireKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2513), tables: &TERADATA_TABLES }),
        "ResetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2514), tables: &TERADATA_TABLES }),
        "ResignalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2515), tables: &TERADATA_TABLES }),
        "ResourceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2516), tables: &TERADATA_TABLES }),
        "RespectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2517), tables: &TERADATA_TABLES }),
        "RestartKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2518), tables: &TERADATA_TABLES }),
        "RestoreKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2519), tables: &TERADATA_TABLES }),
        "RestrictKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2520), tables: &TERADATA_TABLES }),
        "ResultKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2521), tables: &TERADATA_TABLES }),
        "ReturnKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2522), tables: &TERADATA_TABLES }),
        "Returned_cardinalityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2523), tables: &TERADATA_TABLES }),
        "Returned_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2524), tables: &TERADATA_TABLES }),
        "Returned_octet_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2525), tables: &TERADATA_TABLES }),
        "Returned_sqlstateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2526), tables: &TERADATA_TABLES }),
        "ReturnsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2527), tables: &TERADATA_TABLES }),
        "RevokeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2528), tables: &TERADATA_TABLES }),
        "RevokeStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2529), tables: &TERADATA_TABLES }),
        "RightKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2552), tables: &TERADATA_TABLES }),
        "RlikeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2553), tables: &TERADATA_TABLES }),
        "RoleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2554), tables: &TERADATA_TABLES }),
        "RoleReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(2555), tables: &TERADATA_TABLES }),
        "RolesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2556), tables: &TERADATA_TABLES }),
        "RollbackKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2557), tables: &TERADATA_TABLES }),
        "RollupFunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(2558), tables: &TERADATA_TABLES }),
        "RollupKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2559), tables: &TERADATA_TABLES }),
        "RoutineKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2560), tables: &TERADATA_TABLES }),
        "Routine_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2561), tables: &TERADATA_TABLES }),
        "Routine_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2562), tables: &TERADATA_TABLES }),
        "Routine_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2563), tables: &TERADATA_TABLES }),
        "RoutinesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2564), tables: &TERADATA_TABLES }),
        "RowKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2565), tables: &TERADATA_TABLES }),
        "Row_countKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2566), tables: &TERADATA_TABLES }),
        "Row_numberKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2567), tables: &TERADATA_TABLES }),
        "RowcountKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2568), tables: &TERADATA_TABLES }),
        "RowguidcolKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2569), tables: &TERADATA_TABLES }),
        "RowidKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2570), tables: &TERADATA_TABLES }),
        "RownumKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2571), tables: &TERADATA_TABLES }),
        "RowsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2572), tables: &TERADATA_TABLES }),
        "RuleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2573), tables: &TERADATA_TABLES }),
        "RunKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2574), tables: &TERADATA_TABLES }),
        "SampleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2575), tables: &TERADATA_TABLES }),
        "SamplingExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(2576), tables: &TERADATA_TABLES }),
        "SaveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2588), tables: &TERADATA_TABLES }),
        "SavepointKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2589), tables: &TERADATA_TABLES }),
        "ScaleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2590), tables: &TERADATA_TABLES }),
        "SchemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2591), tables: &TERADATA_TABLES }),
        "SchemaReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(599), tables: &TERADATA_TABLES }),
        "Schema_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2592), tables: &TERADATA_TABLES }),
        "SchemasKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2593), tables: &TERADATA_TABLES }),
        "ScopeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2594), tables: &TERADATA_TABLES }),
        "Scope_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2595), tables: &TERADATA_TABLES }),
        "Scope_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2596), tables: &TERADATA_TABLES }),
        "Scope_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2597), tables: &TERADATA_TABLES }),
        "ScrollKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2598), tables: &TERADATA_TABLES }),
        "SearchKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2599), tables: &TERADATA_TABLES }),
        "SecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2600), tables: &TERADATA_TABLES }),
        "Second_microsecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2601), tables: &TERADATA_TABLES }),
        "SectionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2602), tables: &TERADATA_TABLES }),
        "SecurityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2603), tables: &TERADATA_TABLES }),
        "SelKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2604), tables: &TERADATA_TABLES }),
        "SelectClauseElementSegment" => Some(RootGrammar { grammar_id: GrammarId(2605), tables: &TERADATA_TABLES }),
        "SelectClauseModifierSegment" => Some(RootGrammar { grammar_id: GrammarId(2610), tables: &TERADATA_TABLES }),
        "SelectClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2637), tables: &TERADATA_TABLES }),
        "SelectClauseTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(2651), tables: &TERADATA_TABLES }),
        "SelectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2662), tables: &TERADATA_TABLES }),
        "SelectStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2663), tables: &TERADATA_TABLES }),
        "SelectableGrammar" => Some(RootGrammar { grammar_id: GrammarId(2681), tables: &TERADATA_TABLES }),
        "SelfKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2691), tables: &TERADATA_TABLES }),
        "SemicolonSegment" => Some(RootGrammar { grammar_id: GrammarId(2692), tables: &TERADATA_TABLES }),
        "SensitiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2693), tables: &TERADATA_TABLES }),
        "SeparatorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2694), tables: &TERADATA_TABLES }),
        "SequenceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2695), tables: &TERADATA_TABLES }),
        "SequenceMaxValueGrammar" => Some(RootGrammar { grammar_id: GrammarId(2696), tables: &TERADATA_TABLES }),
        "SequenceMinValueGrammar" => Some(RootGrammar { grammar_id: GrammarId(2703), tables: &TERADATA_TABLES }),
        "SequenceReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(599), tables: &TERADATA_TABLES }),
        "SequencesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2710), tables: &TERADATA_TABLES }),
        "SerializableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2711), tables: &TERADATA_TABLES }),
        "ServerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2712), tables: &TERADATA_TABLES }),
        "Server_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2713), tables: &TERADATA_TABLES }),
        "SessionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2714), tables: &TERADATA_TABLES }),
        "Session_userKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2715), tables: &TERADATA_TABLES }),
        "SetClauseListSegment" => Some(RootGrammar { grammar_id: GrammarId(2716), tables: &TERADATA_TABLES }),
        "SetClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2720), tables: &TERADATA_TABLES }),
        "SetExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(2731), tables: &TERADATA_TABLES }),
        "SetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2740), tables: &TERADATA_TABLES }),
        "SetOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(2741), tables: &TERADATA_TABLES }),
        "SetQueryBandStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2749), tables: &TERADATA_TABLES }),
        "SetSchemaStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2765), tables: &TERADATA_TABLES }),
        "SetSessionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2770), tables: &TERADATA_TABLES }),
        "SetofKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2777), tables: &TERADATA_TABLES }),
        "SetsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2778), tables: &TERADATA_TABLES }),
        "SetuserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2779), tables: &TERADATA_TABLES }),
        "ShareKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2780), tables: &TERADATA_TABLES }),
        "SharesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2781), tables: &TERADATA_TABLES }),
        "ShorthandCastSegment" => Some(RootGrammar { grammar_id: GrammarId(2782), tables: &TERADATA_TABLES }),
        "ShowKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2791), tables: &TERADATA_TABLES }),
        "ShutdownKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2792), tables: &TERADATA_TABLES }),
        "SignalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2793), tables: &TERADATA_TABLES }),
        "SignedSegmentGrammar" => Some(RootGrammar { grammar_id: GrammarId(2794), tables: &TERADATA_TABLES }),
        "SimilarKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2797), tables: &TERADATA_TABLES }),
        "SimpleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2798), tables: &TERADATA_TABLES }),
        "SingleIdentifierGrammar" => Some(RootGrammar { grammar_id: GrammarId(2799), tables: &TERADATA_TABLES }),
        "SingleIdentifierListSegment" => Some(RootGrammar { grammar_id: GrammarId(2803), tables: &TERADATA_TABLES }),
        "SingleIdentifierWildcardSegment" => Some(RootGrammar { grammar_id: GrammarId(2805), tables: &TERADATA_TABLES }),
        "SingleQuotedIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(2809), tables: &TERADATA_TABLES }),
        "SizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2810), tables: &TERADATA_TABLES }),
        "SizedArrayTypeSegment" => Some(RootGrammar { grammar_id: GrammarId(2811), tables: &TERADATA_TABLES }),
        "SlashSegment" => Some(RootGrammar { grammar_id: GrammarId(2814), tables: &TERADATA_TABLES }),
        "SliceSegment" => Some(RootGrammar { grammar_id: GrammarId(2815), tables: &TERADATA_TABLES }),
        "SmallintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2816), tables: &TERADATA_TABLES }),
        "SomeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2817), tables: &TERADATA_TABLES }),
        "SonameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2818), tables: &TERADATA_TABLES }),
        "SourceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2819), tables: &TERADATA_TABLES }),
        "SpaceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2820), tables: &TERADATA_TABLES }),
        "SpatialKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2821), tables: &TERADATA_TABLES }),
        "SpecificKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2822), tables: &TERADATA_TABLES }),
        "Specific_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2823), tables: &TERADATA_TABLES }),
        "SpecifictypeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2824), tables: &TERADATA_TABLES }),
        "SqlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2825), tables: &TERADATA_TABLES }),
        "Sql_big_resultKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2826), tables: &TERADATA_TABLES }),
        "Sql_big_selectsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2827), tables: &TERADATA_TABLES }),
        "Sql_big_tablesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2828), tables: &TERADATA_TABLES }),
        "Sql_calc_found_rowsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2829), tables: &TERADATA_TABLES }),
        "Sql_log_offKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2830), tables: &TERADATA_TABLES }),
        "Sql_log_updateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2831), tables: &TERADATA_TABLES }),
        "Sql_low_priority_updatesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2832), tables: &TERADATA_TABLES }),
        "Sql_select_limitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2833), tables: &TERADATA_TABLES }),
        "Sql_small_resultKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2834), tables: &TERADATA_TABLES }),
        "Sql_warningsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2835), tables: &TERADATA_TABLES }),
        "SqlcaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2836), tables: &TERADATA_TABLES }),
        "SqlcodeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2837), tables: &TERADATA_TABLES }),
        "SqlerrorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2838), tables: &TERADATA_TABLES }),
        "SqlexceptionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2839), tables: &TERADATA_TABLES }),
        "SqlstateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2840), tables: &TERADATA_TABLES }),
        "SqlwarningKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2841), tables: &TERADATA_TABLES }),
        "SqrtKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2842), tables: &TERADATA_TABLES }),
        "SsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2843), tables: &TERADATA_TABLES }),
        "SslKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2844), tables: &TERADATA_TABLES }),
        "StableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2845), tables: &TERADATA_TABLES }),
        "StageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2846), tables: &TERADATA_TABLES }),
        "StagesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2847), tables: &TERADATA_TABLES }),
        "StarSegment" => Some(RootGrammar { grammar_id: GrammarId(2848), tables: &TERADATA_TABLES }),
        "StartBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(228), tables: &TERADATA_TABLES }),
        "StartCurlyBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(2221), tables: &TERADATA_TABLES }),
        "StartKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2849), tables: &TERADATA_TABLES }),
        "StartSquareBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(326), tables: &TERADATA_TABLES }),
        "StartingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2850), tables: &TERADATA_TABLES }),
        "StartsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2851), tables: &TERADATA_TABLES }),
        "StatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2852), tables: &TERADATA_TABLES }),
        "StateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2853), tables: &TERADATA_TABLES }),
        "StatementKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2854), tables: &TERADATA_TABLES }),
        "StatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2855), tables: &TERADATA_TABLES }),
        "StaticKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2904), tables: &TERADATA_TABLES }),
        "StatisticsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2905), tables: &TERADATA_TABLES }),
        "StatsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2906), tables: &TERADATA_TABLES }),
        "Stddev_popKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2907), tables: &TERADATA_TABLES }),
        "Stddev_sampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2908), tables: &TERADATA_TABLES }),
        "StdinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2909), tables: &TERADATA_TABLES }),
        "StdoutKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2910), tables: &TERADATA_TABLES }),
        "StorageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2911), tables: &TERADATA_TABLES }),
        "Straight_joinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2912), tables: &TERADATA_TABLES }),
        "StreamKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2913), tables: &TERADATA_TABLES }),
        "StreamsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2914), tables: &TERADATA_TABLES }),
        "StrictKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2915), tables: &TERADATA_TABLES }),
        "StringBinaryOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(2916), tables: &TERADATA_TABLES }),
        "StringKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2918), tables: &TERADATA_TABLES }),
        "StructLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(2919), tables: &TERADATA_TABLES }),
        "StructTypeSegment" => Some(RootGrammar { grammar_id: GrammarId(2924), tables: &TERADATA_TABLES }),
        "StructureKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2925), tables: &TERADATA_TABLES }),
        "StyleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2926), tables: &TERADATA_TABLES }),
        "Subclass_originKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2927), tables: &TERADATA_TABLES }),
        "SublistKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2928), tables: &TERADATA_TABLES }),
        "SubmultisetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2929), tables: &TERADATA_TABLES }),
        "SubstringKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2930), tables: &TERADATA_TABLES }),
        "SuccessfulKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2931), tables: &TERADATA_TABLES }),
        "SumKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2932), tables: &TERADATA_TABLES }),
        "SummaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2933), tables: &TERADATA_TABLES }),
        "SuperuserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2934), tables: &TERADATA_TABLES }),
        "SymbolSegment" => Some(RootGrammar { grammar_id: GrammarId(2935), tables: &TERADATA_TABLES }),
        "SymmetricKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2936), tables: &TERADATA_TABLES }),
        "SynonymKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2937), tables: &TERADATA_TABLES }),
        "SysdateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2938), tables: &TERADATA_TABLES }),
        "SysidKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2939), tables: &TERADATA_TABLES }),
        "SystemKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2940), tables: &TERADATA_TABLES }),
        "System_userKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2941), tables: &TERADATA_TABLES }),
        "TableConstraintSegment" => Some(RootGrammar { grammar_id: GrammarId(2942), tables: &TERADATA_TABLES }),
        "TableEndClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2957), tables: &TERADATA_TABLES }),
        "TableExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(2958), tables: &TERADATA_TABLES }),
        "TableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2967), tables: &TERADATA_TABLES }),
        "TableReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(599), tables: &TERADATA_TABLES }),
        "Table_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2968), tables: &TERADATA_TABLES }),
        "TablesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2969), tables: &TERADATA_TABLES }),
        "TablesampleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2970), tables: &TERADATA_TABLES }),
        "TablespaceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2971), tables: &TERADATA_TABLES }),
        "TablespaceReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(599), tables: &TERADATA_TABLES }),
        "TagReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(599), tables: &TERADATA_TABLES }),
        "Tail_Recurse_Expression_A_Grammar" => Some(RootGrammar { grammar_id: GrammarId(2972), tables: &TERADATA_TABLES }),
        "Tail_Recurse_Expression_B_Grammar" => Some(RootGrammar { grammar_id: GrammarId(2977), tables: &TERADATA_TABLES }),
        "TaskKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2981), tables: &TERADATA_TABLES }),
        "TasksKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2982), tables: &TERADATA_TABLES }),
        "TdCollectStatUsingOptionClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2983), tables: &TERADATA_TABLES }),
        "TdCollectStatisticsStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3029), tables: &TERADATA_TABLES }),
        "TdColumnConstraintSegment" => Some(RootGrammar { grammar_id: GrammarId(3081), tables: &TERADATA_TABLES }),
        "TdCommentStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3090), tables: &TERADATA_TABLES }),
        "TdCreateTableOptions" => Some(RootGrammar { grammar_id: GrammarId(3145), tables: &TERADATA_TABLES }),
        "TdOrderByStatClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3186), tables: &TERADATA_TABLES }),
        "TdRenameStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3194), tables: &TERADATA_TABLES }),
        "TdTableConstraints" => Some(RootGrammar { grammar_id: GrammarId(3202), tables: &TERADATA_TABLES }),
        "TdTablePartitioningLevel" => Some(RootGrammar { grammar_id: GrammarId(3250), tables: &TERADATA_TABLES }),
        "TempKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3261), tables: &TERADATA_TABLES }),
        "TemplateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3262), tables: &TERADATA_TABLES }),
        "TemporalQuerySegment" => Some(RootGrammar { grammar_id: GrammarId(3263), tables: &TERADATA_TABLES }),
        "TemporaryGrammar" => Some(RootGrammar { grammar_id: GrammarId(3264), tables: &TERADATA_TABLES }),
        "TemporaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3267), tables: &TERADATA_TABLES }),
        "TemporaryTransientGrammar" => Some(RootGrammar { grammar_id: GrammarId(3268), tables: &TERADATA_TABLES }),
        "TeradataCastSegment" => Some(RootGrammar { grammar_id: GrammarId(3271), tables: &TERADATA_TABLES }),
        "TerminateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3276), tables: &TERADATA_TABLES }),
        "TerminatedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3277), tables: &TERADATA_TABLES }),
        "TextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3278), tables: &TERADATA_TABLES }),
        "TextsizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3279), tables: &TERADATA_TABLES }),
        "ThanKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3280), tables: &TERADATA_TABLES }),
        "ThenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3281), tables: &TERADATA_TABLES }),
        "ThresholdKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3282), tables: &TERADATA_TABLES }),
        "TiesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3283), tables: &TERADATA_TABLES }),
        "TildeSegment" => Some(RootGrammar { grammar_id: GrammarId(3284), tables: &TERADATA_TABLES }),
        "TimeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3285), tables: &TERADATA_TABLES }),
        "TimeWithTZGrammar" => Some(RootGrammar { grammar_id: GrammarId(3286), tables: &TERADATA_TABLES }),
        "TimeZoneGrammar" => Some(RootGrammar { grammar_id: GrammarId(3297), tables: &TERADATA_TABLES }),
        "TimestampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3303), tables: &TERADATA_TABLES }),
        "Timezone_hourKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3304), tables: &TERADATA_TABLES }),
        "Timezone_minuteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3305), tables: &TERADATA_TABLES }),
        "TinyblobKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3306), tables: &TERADATA_TABLES }),
        "TinyintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3307), tables: &TERADATA_TABLES }),
        "TinytextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3308), tables: &TERADATA_TABLES }),
        "ToKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3309), tables: &TERADATA_TABLES }),
        "ToastKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3310), tables: &TERADATA_TABLES }),
        "TopKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3311), tables: &TERADATA_TABLES }),
        "Top_level_countKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3312), tables: &TERADATA_TABLES }),
        "TrailingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3313), tables: &TERADATA_TABLES }),
        "TranKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3314), tables: &TERADATA_TABLES }),
        "TransactionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3315), tables: &TERADATA_TABLES }),
        "TransactionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3316), tables: &TERADATA_TABLES }),
        "Transaction_activeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3333), tables: &TERADATA_TABLES }),
        "TransactionsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3334), tables: &TERADATA_TABLES }),
        "Transactions_committedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3335), tables: &TERADATA_TABLES }),
        "Transactions_rolled_backKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3336), tables: &TERADATA_TABLES }),
        "TransformKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3337), tables: &TERADATA_TABLES }),
        "TransformsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3338), tables: &TERADATA_TABLES }),
        "TransientKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3339), tables: &TERADATA_TABLES }),
        "TranslateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3340), tables: &TERADATA_TABLES }),
        "TranslationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3341), tables: &TERADATA_TABLES }),
        "TreatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3342), tables: &TERADATA_TABLES }),
        "TriggerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3343), tables: &TERADATA_TABLES }),
        "TriggerReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(599), tables: &TERADATA_TABLES }),
        "Trigger_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3344), tables: &TERADATA_TABLES }),
        "Trigger_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3345), tables: &TERADATA_TABLES }),
        "Trigger_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3346), tables: &TERADATA_TABLES }),
        "TrimKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3347), tables: &TERADATA_TABLES }),
        "TrimParametersGrammar" => Some(RootGrammar { grammar_id: GrammarId(3348), tables: &TERADATA_TABLES }),
        "TrueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3352), tables: &TERADATA_TABLES }),
        "TrueSegment" => Some(RootGrammar { grammar_id: GrammarId(3353), tables: &TERADATA_TABLES }),
        "TruncateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3354), tables: &TERADATA_TABLES }),
        "TruncateStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3355), tables: &TERADATA_TABLES }),
        "TrustedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3359), tables: &TERADATA_TABLES }),
        "TsequalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3360), tables: &TERADATA_TABLES }),
        "TupleSegment" => Some(RootGrammar { grammar_id: GrammarId(3361), tables: &TERADATA_TABLES }),
        "TypeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3364), tables: &TERADATA_TABLES }),
        "TypedArrayLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(3365), tables: &TERADATA_TABLES }),
        "TypedStructLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(3368), tables: &TERADATA_TABLES }),
        "UcKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3371), tables: &TERADATA_TABLES }),
        "UescapeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3372), tables: &TERADATA_TABLES }),
        "UidKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3373), tables: &TERADATA_TABLES }),
        "UnboundedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3374), tables: &TERADATA_TABLES }),
        "UncommittedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3375), tables: &TERADATA_TABLES }),
        "UnconditionalCrossJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(3376), tables: &TERADATA_TABLES }),
        "UnconditionalJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(3377), tables: &TERADATA_TABLES }),
        "UnderKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3381), tables: &TERADATA_TABLES }),
        "UndoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3382), tables: &TERADATA_TABLES }),
        "UnencryptedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3383), tables: &TERADATA_TABLES }),
        "UnionGrammar" => Some(RootGrammar { grammar_id: GrammarId(3384), tables: &TERADATA_TABLES }),
        "UnionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3389), tables: &TERADATA_TABLES }),
        "UniqueKeyGrammar" => Some(RootGrammar { grammar_id: GrammarId(3390), tables: &TERADATA_TABLES }),
        "UniqueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3392), tables: &TERADATA_TABLES }),
        "UnknownKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3393), tables: &TERADATA_TABLES }),
        "UnknownLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(3394), tables: &TERADATA_TABLES }),
        "UnlistenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3395), tables: &TERADATA_TABLES }),
        "UnlockKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3396), tables: &TERADATA_TABLES }),
        "UnnamedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3397), tables: &TERADATA_TABLES }),
        "UnnestKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3398), tables: &TERADATA_TABLES }),
        "UnorderedSelectStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3399), tables: &TERADATA_TABLES }),
        "UnorderedSetExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(3413), tables: &TERADATA_TABLES }),
        "UnsignedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3419), tables: &TERADATA_TABLES }),
        "UntilKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3420), tables: &TERADATA_TABLES }),
        "UpdateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3421), tables: &TERADATA_TABLES }),
        "UpdateStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3422), tables: &TERADATA_TABLES }),
        "UpdatetextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3432), tables: &TERADATA_TABLES }),
        "UpperKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3433), tables: &TERADATA_TABLES }),
        "UppercaseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3434), tables: &TERADATA_TABLES }),
        "UsageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3435), tables: &TERADATA_TABLES }),
        "UseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3436), tables: &TERADATA_TABLES }),
        "UseStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3437), tables: &TERADATA_TABLES }),
        "Use_any_roleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3440), tables: &TERADATA_TABLES }),
        "UserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3441), tables: &TERADATA_TABLES }),
        "UserReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(3442), tables: &TERADATA_TABLES }),
        "User_defined_type_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3443), tables: &TERADATA_TABLES }),
        "User_defined_type_codeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3444), tables: &TERADATA_TABLES }),
        "User_defined_type_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3445), tables: &TERADATA_TABLES }),
        "User_defined_type_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3446), tables: &TERADATA_TABLES }),
        "UsersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3447), tables: &TERADATA_TABLES }),
        "UsingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3448), tables: &TERADATA_TABLES }),
        "Utc_dateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3449), tables: &TERADATA_TABLES }),
        "Utc_timeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3450), tables: &TERADATA_TABLES }),
        "Utc_timestampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3451), tables: &TERADATA_TABLES }),
        "VacuumKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3452), tables: &TERADATA_TABLES }),
        "ValidKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3453), tables: &TERADATA_TABLES }),
        "ValidateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3454), tables: &TERADATA_TABLES }),
        "ValidatorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3455), tables: &TERADATA_TABLES }),
        "ValueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3456), tables: &TERADATA_TABLES }),
        "ValuesClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3457), tables: &TERADATA_TABLES }),
        "ValuesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3470), tables: &TERADATA_TABLES }),
        "Var_popKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3471), tables: &TERADATA_TABLES }),
        "Var_sampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3472), tables: &TERADATA_TABLES }),
        "VarbinaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3473), tables: &TERADATA_TABLES }),
        "Varchar2KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3474), tables: &TERADATA_TABLES }),
        "VarcharKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3475), tables: &TERADATA_TABLES }),
        "VarcharacterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3476), tables: &TERADATA_TABLES }),
        "VariableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3477), tables: &TERADATA_TABLES }),
        "VariablesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3478), tables: &TERADATA_TABLES }),
        "VaryingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3479), tables: &TERADATA_TABLES }),
        "VerboseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3480), tables: &TERADATA_TABLES }),
        "VersionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3481), tables: &TERADATA_TABLES }),
        "ViewKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3482), tables: &TERADATA_TABLES }),
        "ViewsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3483), tables: &TERADATA_TABLES }),
        "VolatileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3484), tables: &TERADATA_TABLES }),
        "WaitforKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3485), tables: &TERADATA_TABLES }),
        "WarehouseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3486), tables: &TERADATA_TABLES }),
        "WarehousesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3487), tables: &TERADATA_TABLES }),
        "WeekKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3488), tables: &TERADATA_TABLES }),
        "WeekdayKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3489), tables: &TERADATA_TABLES }),
        "WhenClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3490), tables: &TERADATA_TABLES }),
        "WhenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3500), tables: &TERADATA_TABLES }),
        "WheneverKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3501), tables: &TERADATA_TABLES }),
        "WhereClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3502), tables: &TERADATA_TABLES }),
        "WhereClauseTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(3507), tables: &TERADATA_TABLES }),
        "WhereKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3520), tables: &TERADATA_TABLES }),
        "WhileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3521), tables: &TERADATA_TABLES }),
        "WhitespaceSegment" => Some(RootGrammar { grammar_id: GrammarId(3522), tables: &TERADATA_TABLES }),
        "Width_bucketKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3523), tables: &TERADATA_TABLES }),
        "WildcardExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(3524), tables: &TERADATA_TABLES }),
        "WildcardIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(3526), tables: &TERADATA_TABLES }),
        "WindowKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3536), tables: &TERADATA_TABLES }),
        "WindowSpecificationSegment" => Some(RootGrammar { grammar_id: GrammarId(3537), tables: &TERADATA_TABLES }),
        "WithCompoundNonSelectStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3545), tables: &TERADATA_TABLES }),
        "WithCompoundStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3554), tables: &TERADATA_TABLES }),
        "WithDataClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3564), tables: &TERADATA_TABLES }),
        "WithFillSegment" => Some(RootGrammar { grammar_id: GrammarId(3569), tables: &TERADATA_TABLES }),
        "WithKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3570), tables: &TERADATA_TABLES }),
        "WithNoSchemaBindingClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3571), tables: &TERADATA_TABLES }),
        "WithinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3576), tables: &TERADATA_TABLES }),
        "WithoutKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3577), tables: &TERADATA_TABLES }),
        "WordSegment" => Some(RootGrammar { grammar_id: GrammarId(3578), tables: &TERADATA_TABLES }),
        "WorkKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3579), tables: &TERADATA_TABLES }),
        "WrapperKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3580), tables: &TERADATA_TABLES }),
        "WriteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3581), tables: &TERADATA_TABLES }),
        "WritetextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3582), tables: &TERADATA_TABLES }),
        "X509KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3583), tables: &TERADATA_TABLES }),
        "XmlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3584), tables: &TERADATA_TABLES }),
        "XorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3585), tables: &TERADATA_TABLES }),
        "YamlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3586), tables: &TERADATA_TABLES }),
        "YearKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3587), tables: &TERADATA_TABLES }),
        "Year_monthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3588), tables: &TERADATA_TABLES }),
        "ZerofillKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3589), tables: &TERADATA_TABLES }),
        "ZoneKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3590), tables: &TERADATA_TABLES }),
        _ => None,
    }
}

pub fn get_teradata_segment_type(name: &str) -> Option<&'static str> {
    match name {
        "AccessObjectSegment" => Some("access_object"),
        "AccessPermissionSegment" => Some("access_permission"),
        "AccessPermissionsSegment" => Some("access_permissions"),
        "AccessSchemaObjectSegment" => Some("access_schema_object"),
        "AccessSchemaPluralObjectSegment" => Some("access_schema_plural_object"),
        "AccessStatementSegment" => Some("access_statement"),
        "AccessTargetSegment" => Some("access_target"),
        "AggregateOrderByClause" => Some("aggregate_order_by"),
        "AliasExpressionSegment" => Some("alias_expression"),
        "AlterSequenceOptionsSegment" => Some("alter_sequence_options_segment"),
        "AlterSequenceStatementSegment" => Some("alter_sequence_statement"),
        "AlterTableStatementSegment" => Some("alter_table_statement"),
        "ArrayAccessorSegment" => Some("array_accessor"),
        "ArrayExpressionSegment" => Some("array_expression"),
        "ArrayLiteralSegment" => Some("array_literal"),
        "ArrayTypeSegment" => Some("array_type"),
        "AsAliasOperatorSegment" => Some("alias_operator"),
        "BaseFileSegment" => Some("file"),
        "BaseSegment" => Some("base"),
        "BinaryOperatorSegment" => Some("binary_operator"),
        "BitwiseAndSegment" => Some("binary_operator"),
        "BitwiseLShiftSegment" => Some("binary_operator"),
        "BitwiseOrSegment" => Some("binary_operator"),
        "BitwiseRShiftSegment" => Some("binary_operator"),
        "BracketedArguments" => Some("bracketed_arguments"),
        "BracketedSegment" => Some("bracketed"),
        "BteqFilePathSegment" => Some("bteq_file_path"),
        "BteqKeyWordSegment" => Some("bteq_key_word_segment"),
        "BteqStatementSegment" => Some("bteq_statement"),
        "CTEColumnList" => Some("cte_column_list"),
        "CTEDefinitionSegment" => Some("common_table_expression"),
        "CaseExpressionSegment" => Some("case_expression"),
        "CodeSegment" => Some("raw"),
        "CollationReferenceSegment" => Some("collation_reference"),
        "ColumnConstraintSegment" => Some("column_constraint_segment"),
        "ColumnDefinitionSegment" => Some("column_definition"),
        "ColumnReferenceSegment" => Some("column_reference"),
        "ColumnsExpressionFunctionContentsSegment" => Some("columns_expression"),
        "ColumnsExpressionFunctionNameSegment" => Some("function_name"),
        "CommentClauseSegment" => Some("comment_clause"),
        "CommentSegment" => Some("comment"),
        "ComparisonOperatorSegment" => Some("comparison_operator"),
        "CompositeBinaryOperatorSegment" => Some("binary_operator"),
        "CompositeComparisonOperatorSegment" => Some("comparison_operator"),
        "ConcatSegment" => Some("binary_operator"),
        "CreateCastStatementSegment" => Some("create_cast_statement"),
        "CreateDatabaseStatementSegment" => Some("create_database_statement"),
        "CreateFunctionStatementSegment" => Some("create_function_statement"),
        "CreateIndexStatementSegment" => Some("create_index_statement"),
        "CreateModelStatementSegment" => Some("create_model_statement"),
        "CreateRoleStatementSegment" => Some("create_role_statement"),
        "CreateSchemaStatementSegment" => Some("create_schema_statement"),
        "CreateSequenceOptionsSegment" => Some("create_sequence_options_segment"),
        "CreateSequenceStatementSegment" => Some("create_sequence_statement"),
        "CreateTableStatementSegment" => Some("create_table_statement"),
        "CreateTriggerStatementSegment" => Some("create_trigger"),
        "CreateUserStatementSegment" => Some("create_user_statement"),
        "CreateViewStatementSegment" => Some("create_view_statement"),
        "CubeFunctionNameSegment" => Some("function_name"),
        "CubeRollupClauseSegment" => Some("cube_rollup_clause"),
        "DatabaseReferenceSegment" => Some("database_reference"),
        "DatabaseStatementSegment" => Some("database_statement"),
        "DatatypeSegment" => Some("data_type"),
        "DatePartFunctionNameSegment" => Some("function_name"),
        "DateTimeFunctionContentsSegment" => Some("function_contents"),
        "Dedent" => Some("dedent"),
        "DeleteStatementSegment" => Some("delete_statement"),
        "DescribeStatementSegment" => Some("describe_statement"),
        "DropCastStatementSegment" => Some("drop_cast_statement"),
        "DropDatabaseStatementSegment" => Some("drop_database_statement"),
        "DropFunctionStatementSegment" => Some("drop_function_statement"),
        "DropIndexStatementSegment" => Some("drop_index_statement"),
        "DropModelStatementSegment" => Some("drop_MODELstatement"),
        "DropRoleStatementSegment" => Some("drop_role_statement"),
        "DropSchemaStatementSegment" => Some("drop_schema_statement"),
        "DropSequenceStatementSegment" => Some("drop_sequence_statement"),
        "DropTableStatementSegment" => Some("drop_table_statement"),
        "DropTriggerStatementSegment" => Some("drop_trigger"),
        "DropTypeStatementSegment" => Some("drop_type_statement"),
        "DropUserStatementSegment" => Some("drop_user_statement"),
        "DropViewStatementSegment" => Some("drop_view_statement"),
        "ElseClauseSegment" => Some("else_clause"),
        "EmptyStructLiteralBracketsSegment" => Some("struct_literal"),
        "EmptyStructLiteralSegment" => Some("typed_struct_literal"),
        "EqualsSegment" => Some("comparison_operator"),
        "ExplainStatementSegment" => Some("explain_statement"),
        "ExpressionSegment" => Some("expression"),
        "ExtensionReferenceSegment" => Some("extension_reference"),
        "FetchClauseSegment" => Some("fetch_clause"),
        "FileSegment" => Some("file"),
        "FrameClauseSegment" => Some("frame_clause"),
        "FromClauseSegment" => Some("from_clause"),
        "FromExpressionElementSegment" => Some("from_expression_element"),
        "FromExpressionSegment" => Some("from_expression"),
        "FromUpdateClauseSegment" => Some("from_in_update_clause"),
        "FunctionContentsSegment" => Some("function_contents"),
        "FunctionDefinitionGrammar" => Some("function_definition"),
        "FunctionNameSegment" => Some("function_name"),
        "FunctionParameterListGrammar" => Some("function_parameter_list"),
        "FunctionSegment" => Some("function"),
        "GrantStatementSegment" => Some("grant_statement"),
        "GreaterThanOrEqualToSegment" => Some("comparison_operator"),
        "GreaterThanSegment" => Some("comparison_operator"),
        "GroupByClauseSegment" => Some("groupby_clause"),
        "GroupingExpressionList" => Some("grouping_expression_list"),
        "GroupingSetsClauseSegment" => Some("grouping_sets_clause"),
        "HavingClauseSegment" => Some("having_clause"),
        "IdentifierSegment" => Some("identifier"),
        "ImplicitIndent" => Some("indent"),
        "Indent" => Some("indent"),
        "IndexColumnDefinitionSegment" => Some("index_column_definition"),
        "IndexReferenceSegment" => Some("index_reference"),
        "InsertStatementSegment" => Some("insert_statement"),
        "IntervalExpressionSegment" => Some("interval_expression"),
        "JoinClauseSegment" => Some("join_clause"),
        "JoinOnConditionSegment" => Some("join_on_condition"),
        "KeywordSegment" => Some("keyword"),
        "LessThanOrEqualToSegment" => Some("comparison_operator"),
        "LessThanSegment" => Some("comparison_operator"),
        "LimitClauseSegment" => Some("limit_clause"),
        "LiteralKeywordSegment" => Some("literal"),
        "LiteralSegment" => Some("literal"),
        "LocalAliasSegment" => Some("local_alias_segment"),
        "LockingClauseSegment" => Some("locking_clause"),
        "MLTableExpressionSegment" => Some("ml_table_expression"),
        "MapTypeSegment" => Some("map_type"),
        "MatchConditionSegment" => Some("match_condition"),
        "MergeDeleteClauseSegment" => Some("merge_delete_clause"),
        "MergeInsertClauseSegment" => Some("merge_insert_clause"),
        "MergeMatchSegment" => Some("merge_match"),
        "MergeMatchedClauseSegment" => Some("merge_when_matched_clause"),
        "MergeNotMatchedClauseSegment" => Some("merge_when_not_matched_clause"),
        "MergeStatementSegment" => Some("merge_statement"),
        "MergeUpdateClauseSegment" => Some("merge_update_clause"),
        "NamedWindowExpressionSegment" => Some("named_window_expression"),
        "NamedWindowSegment" => Some("named_window"),
        "NewlineSegment" => Some("newline"),
        "NotEqualToSegment" => Some("comparison_operator"),
        "NotEqualToSegment_b" => Some("comparison_operator"),
        "NotEqualToSegment_c" => Some("comparison_operator"),
        "ObjectLiteralElementSegment" => Some("object_literal_element"),
        "ObjectLiteralSegment" => Some("object_literal"),
        "ObjectReferenceSegment" => Some("object_reference"),
        "OffsetClauseSegment" => Some("offset_clause"),
        "OrderByClauseSegment" => Some("orderby_clause"),
        "OverClauseSegment" => Some("over_clause"),
        "OverlapsClauseSegment" => Some("overlaps_clause"),
        "PartitionClauseSegment" => Some("partitionby_clause"),
        "PathSegment" => Some("path_segment"),
        "QualifiedNumericLiteralSegment" => Some("numeric_literal"),
        "QualifyClauseSegment" => Some("qualify_clause"),
        "RevokeStatementSegment" => Some("revoke_statement"),
        "RoleReferenceSegment" => Some("role_reference"),
        "RollupFunctionNameSegment" => Some("function_name"),
        "SamplingExpressionSegment" => Some("sample_expression"),
        "SchemaReferenceSegment" => Some("schema_reference"),
        "SelectClauseElementSegment" => Some("select_clause_element"),
        "SelectClauseModifierSegment" => Some("select_clause_modifier"),
        "SelectClauseSegment" => Some("select_clause"),
        "SelectStatementSegment" => Some("select_statement"),
        "SequenceReferenceSegment" => Some("sequence_reference"),
        "SetClauseListSegment" => Some("set_clause_list"),
        "SetClauseSegment" => Some("set_clause"),
        "SetExpressionSegment" => Some("set_expression"),
        "SetOperatorSegment" => Some("set_operator"),
        "SetQueryBandStatementSegment" => Some("set_query_band_statement"),
        "SetSchemaStatementSegment" => Some("set_schema_statement"),
        "SetSessionStatementSegment" => Some("set_session_statement"),
        "ShorthandCastSegment" => Some("cast_expression"),
        "SingleIdentifierListSegment" => Some("identifier_list"),
        "SingleIdentifierWildcardSegment" => Some("wildcard_identifier"),
        "SizedArrayTypeSegment" => Some("sized_array_type"),
        "StatementSegment" => Some("statement"),
        "StructLiteralSegment" => Some("struct_literal"),
        "StructTypeSegment" => Some("struct_type"),
        "SymbolSegment" => Some("symbol"),
        "TableConstraintSegment" => Some("table_constraint"),
        "TableEndClauseSegment" => Some("table_end_clause_segment"),
        "TableExpressionSegment" => Some("table_expression"),
        "TableReferenceSegment" => Some("table_reference"),
        "TablespaceReferenceSegment" => Some("tablespace_reference"),
        "TagReferenceSegment" => Some("tag_reference"),
        "TdCollectStatUsingOptionClauseSegment" => Some("collect_stat_using_option_clause"),
        "TdCollectStatisticsStatementSegment" => Some("collect_statistics_statement"),
        "TdColumnConstraintSegment" => Some("td_column_attribute_constraint"),
        "TdCommentStatementSegment" => Some("comment_clause"),
        "TdCreateTableOptions" => Some("create_table_options_statement"),
        "TdOrderByStatClauseSegment" => Some("stat_orderby_clause"),
        "TdRenameStatementSegment" => Some("rename_table_statement"),
        "TdTableConstraints" => Some("td_table_constraint"),
        "TdTablePartitioningLevel" => Some("td_partitioning_level"),
        "TemporalQuerySegment" => Some("temporal_query"),
        "TeradataCastSegment" => Some("cast_expression"),
        "TimeZoneGrammar" => Some("time_zone_grammar"),
        "TransactionStatementSegment" => Some("transaction_statement"),
        "TriggerReferenceSegment" => Some("trigger_reference"),
        "TruncateStatementSegment" => Some("truncate_table"),
        "TupleSegment" => Some("tuple"),
        "TypedArrayLiteralSegment" => Some("typed_array_literal"),
        "TypedStructLiteralSegment" => Some("typed_struct_literal"),
        "UnorderedSelectStatementSegment" => Some("select_statement"),
        "UnorderedSetExpressionSegment" => Some("set_expression"),
        "UpdateStatementSegment" => Some("update_statement"),
        "UseStatementSegment" => Some("use_statement"),
        "UserReferenceSegment" => Some("user_reference"),
        "ValuesClauseSegment" => Some("values_clause"),
        "WhenClauseSegment" => Some("when_clause"),
        "WhereClauseSegment" => Some("where_clause"),
        "WhitespaceSegment" => Some("whitespace"),
        "WildcardExpressionSegment" => Some("wildcard_expression"),
        "WildcardIdentifierSegment" => Some("wildcard_identifier"),
        "WindowSpecificationSegment" => Some("window_specification"),
        "WithCompoundNonSelectStatementSegment" => Some("with_compound_statement"),
        "WithCompoundStatementSegment" => Some("with_compound_statement"),
        "WithDataClauseSegment" => Some("with_data_clause"),
        "WithFillSegment" => Some("with_fill"),
        "WithNoSchemaBindingClauseSegment" => Some("with_no_schema_binding_clause"),
        "WordSegment" => Some("word"),
        _ => None,
    }
}

pub static TERADATA_TABLES: GrammarTables = GrammarTables {
    instructions: INSTRUCTIONS,
    child_ids: CHILD_IDS,
    terminators: TERMINATORS,
    strings: STRINGS,
    aux_data: AUX_DATA,
    aux_data_offsets: AUX_DATA_OFFSETS,
    regex_patterns: REGEX_PATTERNS,
    simple_hints: SIMPLE_HINTS,
    hint_string_indices: HINT_STRING_INDICES,
    simple_hint_indices: SIMPLE_HINT_INDICES,
    segment_type_offsets: SEGMENT_TYPE_OFFSETS,
    segment_class_offsets: SEGMENT_CLASS_OFFSETS,
    casefold_sparse: CASEFOLD_SPARSE,
    trim_chars_sparse: TRIM_CHARS_SPARSE,
    trim_chars_data: TRIM_CHARS_DATA,
    segment_class_types_sparse: SEGMENT_CLASS_TYPES_SPARSE,
    segment_class_types_data: SEGMENT_CLASS_TYPES_DATA,
};

pub fn get_teradata_root_grammar_id() -> GrammarId {
    GrammarId(1328)
}
pub fn get_teradata_root_grammar_table() -> RootGrammar {
    RootGrammar {
        grammar_id: get_teradata_root_grammar_id(),
        tables: &TERADATA_TABLES,
    }
}
pub fn get_teradata_root_grammar() -> RootGrammar {
    get_teradata_root_grammar_table()
}
